# LLM Platform Core Concepts & Terminology

## Shared Terminology

This document defines core concepts used across all LLM Platform projects to ensure consistent understanding and reduce documentation duplication.

## Architecture Concepts

### Service Tiers

**Tier 1: Foundation Services (NPM Packages)**
- Self-contained TypeScript/Node.js services
- Provide specialized AI functionality
- Can be deployed independently
- Communicate via REST APIs or direct imports

**Tier 2: Integration Layer (Drupal Modules)**
- Extend Drupal with AI capabilities
- Bridge between Drupal and NPM services
- Provide configuration and administration interfaces
- Handle user authentication and permissions

**Tier 3: Deployment Layer (Recipes & Themes)**
- Complete deployment configurations
- Pre-configured workflows and content types
- Administrative themes and interfaces
- Installation automation

### Provider Abstraction

**AI Provider Interface**
```typescript
interface AIProviderInterface {
  generateText(prompt: string, options: GenerationOptions): Promise<AIResponse>;
  generateImage(prompt: string, options: ImageOptions): Promise<AIResponse>;
  processAudio(audio: string, options: AudioOptions): Promise<AIResponse>;
}
```

**Supported Providers**
- **OpenAI**: GPT-3.5, GPT-4, GPT-4 Turbo, DALL-E
- **Anthropic**: Claude 3, Claude 3.5 Sonnet
- **Apple Intelligence**: On-device and Private Cloud Compute
- **Google**: Gemini Pro, Gemini Ultra, PaLM 2
- **Meta**: Llama 2, Llama 3, Code Llama
- **Local/Self-Hosted**: Ollama, LocalAI, Text Generation WebUI

## Data Flow Patterns

### Request Processing Pipeline

```
Input → Validation → Provider Selection → AI Processing → Response Processing → Output
  ↓         ↓             ↓                 ↓               ↓               ↓
Security  Auth      Load Balancing    API Calls     Content Safety    Caching
Check     Check     & Rate Limiting   & Retries     & Filtering       & Delivery
```

### Caching Strategy

**Multi-Layer Caching**
1. **Application Cache**: In-memory caching for frequently accessed data
2. **Redis Cache**: Distributed caching for AI responses and configurations
3. **Vector Cache**: Specialized caching for embeddings and similarity searches
4. **CDN Cache**: Edge caching for static AI-generated content

**Cache Keys Pattern**
```
ai_response:{provider}:{model}:{content_hash}:{options_hash}
vector_embedding:{text_hash}:{model}
config:{module}:{setting}:{version}
```

## Security Patterns

### API Key Management

**Key Storage Hierarchy**
1. **Environment Variables**: Development and simple deployments
2. **Drupal Key Module**: Encrypted storage with rotation support
3. **External Key Management**: AWS KMS, Azure Key Vault, HashiCorp Vault
4. **Hardware Security Modules**: Enterprise-grade key protection

**Key Rotation Policy**
- Automatic rotation every 90 days
- Manual rotation on security events
- Graceful transition with dual-key periods
- Audit logging for all key operations

### Circuit Breaker Pattern

```typescript
class CircuitBreaker {
  constructor(
    private failureThreshold: number = 5,
    private timeout: number = 60000,
    private monitoringPeriod: number = 60000
  ) {}
  
  async execute<T>(operation: () => Promise<T>): Promise<T> {
    if (this.state === CircuitState.OPEN) {
      throw new Error('Circuit breaker is OPEN');
    }
    // Implementation details...
  }
}
```

## Testing Patterns

### Test-Driven Development (TDD)

**Red-Green-Refactor Cycle**
1. **Red**: Write failing test that describes desired behavior
2. **Green**: Write minimal code to make test pass
3. **Refactor**: Improve code quality while keeping tests green

**Coverage Requirements**
- **NPM Packages**: 95% minimum code coverage
- **Drupal Modules**: 85% minimum code coverage
- **Integration Tests**: All critical user paths covered
- **Performance Tests**: Response time benchmarks established

### Testing Strategy by Component Type

**Unit Tests**
```typescript
describe('AIProvider', () => {
  beforeEach(() => {
    provider = new OpenAIProvider(mockConfig);
  });

  it('should generate text with proper options', async () => {
    const result = await provider.generateText('test prompt', {
      maxTokens: 100,
      temperature: 0.7
    });
    
    expect(result.content).toBeDefined();
    expect(result.tokens_used).toBeGreaterThan(0);
  });
});
```

**Integration Tests**
```php
// Drupal integration test
class AIProviderIntegrationTest extends BrowserTestBase {
  public function testAppleIntelligenceIntegration() {
    $service = $this->container->get('llm.apple_intelligence_provider');
    $result = $service->generateContent('Test prompt');
    
    $this->assertArrayHasKey('content', $result);
    $this->assertNotEmpty($result['content']);
  }
}
```

## Performance Patterns

### Async Processing

**Queue-Based Processing**
```php
// Drupal queue integration
$queue = \Drupal::queue('ai_processing');
$queue->createItem([
  'operation' => 'generate_content',
  'prompt' => $prompt,
  'options' => $options,
  'callback' => $callback_url
]);
```

**WebSocket Real-Time Updates**
```typescript
// Real-time progress updates
const ws = new WebSocket('ws://localhost:3000/progress');
ws.onmessage = (event) => {
  const update = JSON.parse(event.data);
  updateProgress(update.percentage, update.status);
};
```

### Response Time Optimization

**Target Response Times**
- **Simple Text Generation**: < 2 seconds
- **Complex Document Processing**: < 30 seconds
- **Image Generation**: < 10 seconds
- **API Health Checks**: < 500ms

**Performance Monitoring**
```typescript
const performanceMonitor = {
  trackOperation: (operation: string, duration: number) => {
    metrics.timing(`ai.operation.${operation}`, duration);
    if (duration > thresholds[operation]) {
      logger.warn(`Slow operation: ${operation} took ${duration}ms`);
    }
  }
};
```

## Configuration Patterns

### Environment-Based Configuration

**Configuration Hierarchy**
1. **Default Values**: Hardcoded reasonable defaults
2. **Environment Variables**: Override defaults for deployment
3. **Configuration Files**: Complex structured configuration
4. **Database Configuration**: Runtime-changeable settings
5. **User Preferences**: Per-user customizations

**Configuration Schema Example**
```yaml
ai_providers:
  openai:
    enabled: true
    endpoint: 'https://api.openai.com/v1'
    timeout: 30000
    rate_limit:
      requests_per_minute: 60
      burst_allowance: 10
    fallback_providers: ['anthropic', 'local']
```

### Feature Flags

**Flag Categories**
- **Provider Switches**: Enable/disable specific AI providers
- **Feature Rollouts**: Gradual feature enablement
- **Emergency Shutoffs**: Quick disable of problematic features
- **A/B Testing**: Compare different implementations

## Error Handling Patterns

### Standardized Error Types

```typescript
export enum ErrorType {
  VALIDATION_ERROR = 'validation_error',
  PROVIDER_ERROR = 'provider_error',
  RATE_LIMIT_ERROR = 'rate_limit_error',
  AUTHENTICATION_ERROR = 'authentication_error',
  TIMEOUT_ERROR = 'timeout_error',
  CIRCUIT_BREAKER_ERROR = 'circuit_breaker_error'
}

export class LLMPlatformError extends Error {
  constructor(
    public type: ErrorType,
    message: string,
    public details?: any,
    public retryable: boolean = false
  ) {
    super(message);
  }
}
```

### Error Recovery Strategies

**Retry Logic**
```typescript
async function withRetry<T>(
  operation: () => Promise<T>,
  maxRetries: number = 3,
  backoffMs: number = 1000
): Promise<T> {
  for (let attempt = 1; attempt <= maxRetries; attempt++) {
    try {
      return await operation();
    } catch (error) {
      if (attempt === maxRetries || !isRetryable(error)) {
        throw error;
      }
      await sleep(backoffMs * Math.pow(2, attempt - 1));
    }
  }
}
```

## Monitoring & Observability

### Metrics Collection

**System Metrics**
- **Response Times**: Per-endpoint and per-provider timing
- **Error Rates**: Categorized by error type and provider
- **Throughput**: Requests per second and tokens per minute
- **Resource Usage**: CPU, memory, and storage utilization

**Business Metrics**
- **AI Costs**: Per-provider cost tracking and optimization
- **User Engagement**: Feature usage and adoption rates
- **Content Quality**: Generated content scoring and feedback
- **Conversion Rates**: Feature effectiveness measurements

### Logging Standards

**Structured Logging Format**
```json
{
  "timestamp": "2024-01-15T10:30:00Z",
  "level": "info",
  "service": "llm-gateway",
  "operation": "text_generation",
  "duration_ms": 1250,
  "tokens_used": 150,
  "provider": "openai",
  "model": "gpt-4",
  "user_id": "user_123",
  "request_id": "req_abc123",
  "metadata": {
    "content_type": "technical_documentation",
    "prompt_length": 500
  }
}
```

## Integration Patterns

### NPM Package → Drupal Module

**Service Container Registration**
```php
// Register NPM service in Drupal
services:
  llm.npm_bridge:
    class: Drupal\llm\Service\NpmBridgeService
    arguments:
      - '@config.factory'
      - '@http_client'
      - '@cache.default'
```

**Configuration Mapping**
```php
// Map Drupal config to NPM service
public function callNpmService(string $endpoint, array $data): array {
  $config = $this->configFactory->get('llm.settings');
  $service_url = $config->get('npm_services.endpoint');
  
  return $this->httpClient->post($service_url . $endpoint, [
    'json' => $data,
    'headers' => $this->getAuthHeaders()
  ]);
}
```

### Event-Driven Communication

**Event Types**
- **ai.content.generated**: AI content generation completed
- **ai.provider.failed**: AI provider encountered error
- **ai.cost.threshold**: Cost threshold exceeded
- **ai.quality.degraded**: Content quality below threshold

**Event Handler Example**
```php
public function onContentGenerated(AIContentGeneratedEvent $event) {
  $content = $event->getContent();
  $metadata = $event->getMetadata();
  
  // Update usage statistics
  $this->updateUsageStats($metadata['tokens_used'], $metadata['cost']);
  
  // Quality assessment
  if ($metadata['quality_score'] < 0.8) {
    $this->flagForReview($content);
  }
}
```

## Deployment Patterns

### Container Orchestration

**Docker Compose Development**
```yaml
version: '3.8'
services:
  drupal:
    image: drupal:11-apache
    volumes:
      - ./web:/var/www/html
    environment:
      - DATABASE_URL=mysql://user:pass@db/drupal
  
  apple-fm:
    build: ./_CommonNPM/apple-fm
    ports:
      - "3001:3001"
    environment:
      - OLLAMA_ENDPOINT=http://ollama:11434
  
  redis:
    image: redis:alpine
    ports:
      - "6379:6379"
```

**Kubernetes Production**
```yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: llm-gateway
spec:
  replicas: 3
  selector:
    matchLabels:
      app: llm-gateway
  template:
    spec:
      containers:
      - name: llm-gateway
        image: llm-gateway:latest
        ports:
        - containerPort: 3000
        env:
        - name: REDIS_URL
          valueFrom:
            secretKeyRef:
              name: llm-secrets
              key: redis-url
```

## Version Compatibility

### Semantic Versioning

**Version Format**: `MAJOR.MINOR.PATCH`
- **MAJOR**: Breaking changes to APIs or contracts
- **MINOR**: New features, backward compatible
- **PATCH**: Bug fixes, no API changes

**Compatibility Matrix**
```
NPM Package Version | Compatible Drupal Module Version
1.0.x              | 1.0.x
1.1.x              | 1.0.x, 1.1.x
2.0.x              | 2.0.x (breaking changes)
```

### Migration Strategies

**Configuration Migration**
```php
// Update hook for configuration changes
function llm_update_8201() {
  $config = \Drupal::configFactory()->getEditable('llm.settings');
  
  // Migrate old provider config to new format
  $old_provider = $config->get('provider');
  if ($old_provider) {
    $config->set('providers.default', $old_provider);
    $config->clear('provider');
    $config->save();
  }
}
```

This concepts document serves as the single source of truth for architectural patterns, terminology, and implementation strategies across all LLM Platform projects. Reference this document instead of duplicating explanations in individual project documentation.