---
id: ai_coding_assistant-usage
---
# Usage Guide

This guide provides examples and best practices for using the LLM PlatformAI Coding Assistant in your development workflow.

## Basic Usage

### Importing the AI Coding Assistant

To use the AI Coding Assistant in your code, import the appropriate implementation:

```typescript
// Using the llm-mcp implementation
import { AICodingAssistant } from '@bluefly/llm-mcp/services/ai-coding-assistant';

// Using the native_aijs implementation
import { CodingAssistant } from '@bluefly/native_aijs/core/neurosymbolic/CodingAssistant';
```

### Initializing the Assistant

```typescript
// For the llm-mcp implementation
const toolRegistry = new McpToolRegistry();
const vectorBridge = new VectorBridge();
const toolManager = new ToolManager();
const intelligenceManager = new IntelligenceManager();
const connectionPool = new ConnectionPool();

const codingAssistant = new AICodingAssistant(
  toolRegistry,
  vectorBridge,
  toolManager,
  intelligenceManager,
  connectionPool
);

// For the native_aijs implementation
const neuralFabric = new NeuralFabric();
const patternAnalyzer = new BasicPatternAnalyzer();
const entityLearner = new BasicEntityLearner();
const emergentConsciousness = new EmergentConsciousness();
const neuroSymbolicHypergraph = new NeuroSymbolicHypergraph();
const llmHypergraphBridge = new LLMHypergraphBridge();
const embeddingProvider = new EmbeddingProvider();

const codingAssistant = new CodingAssistant(
  neuralFabric,
  patternAnalyzer,
  entityLearner,
  emergentConsciousness,
  neuroSymbolicHypergraph,
  llmHypergraphBridge,
  embeddingProvider,
  {
    numberOfSuggestions: 3,
    maxSuggestionLength: 100,
    enableSemanticPreservation: true
  }
);
```

## Common Tasks

### Searching Code

Use the AI Coding Assistant to search for code and understand patterns:

```typescript
// Define search query
const query = {
  text: "How does the authentication system work?",
  maxResults: 10,
  includeComments: true
};

// Define context
const context = {
  currentFile: {
    path: '/path/to/file.ts',
    content: '/* file content */',
    language: 'typescript'
  },
  language: 'typescript',
  framework: 'react'
};

// Search for code
const results = await codingAssistant.searchCode(query, context);

// Process results
for (const result of results) {
  console.log(`File: ${result.path}`);
  console.log(`Snippet: ${result.snippet}`);
  console.log(`Relevance: ${result.relevance}`);
  console.log(`Concepts: ${result.concepts.join(', ')}`);
  console.log('---');
}
```

### Generating Code

Generate code based on natural language descriptions:

```typescript
// Define natural language request
const request = "Create a function that validates email addresses using regex";

// Generate code
const generated = await codingAssistant.processRequest(request, context);

// Use the generated code
if (generated.success) {
  console.log(generated.code);
  console.log(generated.explanation);
}
```

### Editing Code

Make intelligent edits to existing code:

```typescript
// Define edit operations
const operations = [
  {
    type: 'replace',
    position: { start: 120, end: 150 },
    text: 'const validateEmail = (email) => /^[^\\s@]+@[^\\s@]+\\.[^\\s@]+$/.test(email);',
    preserveSemantics: true
  }
];

// Apply edits
const editResult = await codingAssistant.editCode(operations, context);

// Handle the result
if (editResult.success) {
  console.log(editResult.updatedContent);
} else {
  console.error(editResult.failedOperations);
}
```

### Visualizing Code

Generate visualizations of code architecture and dependencies:

```typescript
// Define visualization config
const config = {
  type: 'dependency',
  format: 'svg',
  includePatterns: true,
  includeRelationships: true
};

// Generate visualization
const visualization = await codingAssistant.visualizeCode(config, context);

// Use the visualization
if (visualization) {
  // Save SVG to file
  fs.writeFileSync('dependency-graph.svg', visualization.data);
  
  // Log metadata
  console.log(`Nodes: ${visualization.metadata.nodes.length}`);
  console.log(`Edges: ${visualization.metadata.edges.length}`);
  console.log(`Concepts: ${visualization.metadata.concepts.join(', ')}`);
}
```

## Advanced Usage

### Multi-step Task Planning

Break down complex coding tasks into manageable steps:

```typescript
// Define a complex task
const task = {
  description: "Implement user authentication with JWT",
  context: {
    currentProject: {
      framework: "express",
      database: "mongodb",
      authStrategy: "jwt"
    }
  }
};

// Process the task
const result = await codingAssistant.processCodingTask(task, context);

// Execute the plan
for (const step of result.plan.steps) {
  console.log(`Executing step: ${step.description}`);
  await executeStep(step);
}
```

### Git Operations

Perform git operations using natural language:

```typescript
// Define the git operation
const gitOperation = {
  operation: "commit",
  message: "Fix authentication token validation",
  options: {
    files: ["auth.js", "middleware/auth.js"],
    push: true
  }
};

// Execute the git operation
const gitResult = await codingAssistant.handleGitOperations(gitOperation, context);

console.log(`Git operation ${gitResult.success ? 'succeeded' : 'failed'}`);
console.log(gitResult.details);
```

## Best Practices

### Providing Context

For best results, provide as much context as possible:

```typescript
const context = {
  currentFile: {
    path: '/path/to/file.ts',
    content: '/* full file content */',
    language: 'typescript'
  },
  language: 'typescript',
  framework: 'react',
  projectStructure: {
    rootDir: '/path/to/project',
    srcDir: '/path/to/project/src',
    testDir: '/path/to/project/tests'
  },
  codebase: {
    size: 50000,
    fileCount: 120,
    languages: ['typescript', 'javascript', 'css'],
    frameworks: ['react', 'express']
  },
  preferences: {
    codingStyle: 'functional',
    testFramework: 'jest',
    linter: 'eslint'
  }
};
```

### Error Handling

Always handle errors and provide fallbacks:

```typescript
try {
  const result = await codingAssistant.processRequest(request, context);
  if (result.success) {
    // Process successful result
  } else {
    // Handle failure case
    console.error(`Failed to process request: ${result.error}`);
    // Implement fallback behavior
  }
} catch (error) {
  console.error(`Error: ${error.message}`);
  // Implement error recovery
}
```

### Performance Optimization

For large codebases, optimize performance:

1. **Limit Context**: Only include relevant files and context
2. **Batch Operations**: Group related operations together
3. **Cache Results**: Cache search results and visualizations
4. **Optimize Queries**: Use specific search queries instead of broad ones

## Integration with Other Tools

### VSCode Extension

Use the AI Coding Assistant with the VSCode extension:

1. Install the extension from the marketplace
2. Configure the extension with your API keys
3. Use the command palette to access coding assistant features

### Command Line Interface

Use the AI Coding Assistant from the command line:

```bash
# Search code
bf-code search "authentication system"

# Generate code
bf-code generate "function to validate email"

# Visualize code
bf-code visualize --type dependency --file src/auth.js
```

## Next Steps

- [API Reference](./api-reference)
- [Examples](./examples)
- [Integration Guide](./integration)
