---
id: api-rest
---
# REST API Guide

<!-- title: REST API Guide -->
<!-- description: Guide for the LLM Platform Platform REST API -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the REST API implementation for the LLM Platform Platform. It covers endpoints, authentication, request/response formats, and best practices.

## Architecture

### Core Components

1. **API Gateway**
   - Request routing
   - Authentication
   - Rate limiting
   - Caching

2. **Controllers**
   - Request handling
   - Response formatting
   - Error handling
   - Validation

3. **Services**
   - Business logic
   - Data access
   - Integration
   - Caching

## Implementation Guide

### Base Controller

```typescript
// src/controllers/base.controller.ts
export class BaseController {
  constructor(
    private normalizationService: NormalizationService,
    private errorNormalizer: ErrorNormalizerService
  ) {}
  
  protected async handleRequest<T>(
    handler: () => Promise<T>,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    try {
      const data = await handler();
      return this.normalizeResponse(data, schema);
    } catch (error) {
      throw this.errorNormalizer.normalize(error);
    }
  }
  
  protected async normalizeResponse<T>(
    data: T,
    schema: Schema
  ): Promise<NormalizedResponse<T>> {
    return this.normalizationService.normalize(data, schema);
  }
}
```

### Agent Controller

```typescript
// src/controllers/agent.controller.ts
@Controller('agents')
export class AgentController extends BaseController {
  constructor(
    private agentService: AgentService,
    normalizationService: NormalizationService,
    errorNormalizer: ErrorNormalizerService
  ) {
    super(normalizationService, errorNormalizer);
  }
  
  @Get()
  async list(
    @Query() query: ListQuery
  ): Promise<NormalizedResponse<Agent[]>> {
    return this.handleRequest(
      () => this.agentService.list(query),
      agentListSchema
    );
  }
  
  @Get(':id')
  async get(
    @Param('id') id: string
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.get(id),
      agentSchema
    );
  }
  
  @Post()
  async create(
    @Body() input: CreateAgentInput
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.create(input),
      agentSchema
    );
  }
  
  @Put(':id')
  async update(
    @Param('id') id: string,
    @Body() input: UpdateAgentInput
  ): Promise<NormalizedResponse<Agent>> {
    return this.handleRequest(
      () => this.agentService.update(id, input),
      agentSchema
    );
  }
  
  @Delete(':id')
  async delete(
    @Param('id') id: string
  ): Promise<NormalizedResponse<void>> {
    return this.handleRequest(
      () => this.agentService.delete(id),
      voidSchema
    );
  }
}
```

### Model Controller

```typescript
// src/controllers/model.controller.ts
@Controller('models')
export class ModelController extends BaseController {
  constructor(
    private modelService: ModelService,
    normalizationService: NormalizationService,
    errorNormalizer: ErrorNormalizerService
  ) {
    super(normalizationService, errorNormalizer);
  }
  
  @Get()
  async list(
    @Query() query: ListQuery
  ): Promise<NormalizedResponse<Model[]>> {
    return this.handleRequest(
      () => this.modelService.list(query),
      modelListSchema
    );
  }
  
  @Get(':id')
  async get(
    @Param('id') id: string
  ): Promise<NormalizedResponse<Model>> {
    return this.handleRequest(
      () => this.modelService.get(id),
      modelSchema
    );
  }
}
```

## Integration Guide

### Authentication

1. **Bearer Token**
   ```typescript
   // src/guards/auth.guard.ts
   @Injectable()
   export class AuthGuard implements CanActivate {
     constructor(private authService: AuthService) {}
     
     async canActivate(context: ExecutionContext): Promise<boolean> {
       const request = context.switchToHttp().getRequest();
       const token = this.extractToken(request);
       
       if (!token) {
         throw new UnauthorizedException();
       }
       
       try {
         const user = await this.authService.validateToken(token);
         request.user = user;
         return true;
       } catch (error) {
         throw new UnauthorizedException();
       }
     }
     
     private extractToken(request: Request): string {
       const auth = request.headers.authorization;
       if (!auth || !auth.startsWith('Bearer ')) {
         return null;
       }
       return auth.substring(7);
     }
   }
   ```

2. **API Key**
   ```typescript
   // src/guards/api-key.guard.ts
   @Injectable()
   export class ApiKeyGuard implements CanActivate {
     constructor(private apiKeyService: ApiKeyService) {}
     
     async canActivate(context: ExecutionContext): Promise<boolean> {
       const request = context.switchToHttp().getRequest();
       const key = request.headers['x-api-key'];
       
       if (!key) {
         throw new UnauthorizedException();
       }
       
       try {
         const apiKey = await this.apiKeyService.validate(key);
         request.apiKey = apiKey;
         return true;
       } catch (error) {
         throw new UnauthorizedException();
       }
     }
   }
   ```

### Rate Limiting

```typescript
// src/middleware/rate-limit.middleware.ts
@Injectable()
export class RateLimitMiddleware implements NestMiddleware {
  constructor(private rateLimitService: RateLimitService) {}
  
  async use(req: Request, res: Response, next: Function) {
    const key = this.getKey(req);
    const limit = await this.rateLimitService.check(key);
    
    res.setHeader('X-RateLimit-Limit', limit.limit);
    res.setHeader('X-RateLimit-Remaining', limit.remaining);
    res.setHeader('X-RateLimit-Reset', limit.reset);
    
    if (limit.remaining <= 0) {
      throw new TooManyRequestsException();
    }
    
    next();
  }
  
  private getKey(req: Request): string {
    return `${req.ip}:${req.path}`;
  }
}
```

### Caching

```typescript
// src/interceptors/cache.interceptor.ts
@Injectable()
export class CacheInterceptor implements NestInterceptor {
  constructor(private cacheService: CacheService) {}
  
  async intercept(
    context: ExecutionContext,
    next: CallHandler
  ): Promise<Observable<any>> {
    const request = context.switchToHttp().getRequest();
    const key = this.getCacheKey(request);
    
    const cached = await this.cacheService.get(key);
    if (cached) {
      return of(cached);
    }
    
    return next.handle().pipe(
      tap(async (data) => {
        await this.cacheService.set(key, data);
      })
    );
  }
  
  private getCacheKey(request: Request): string {
    return `${request.method}:${request.url}`;
  }
}
```

## Best Practices

1. **Request Handling**
   - Validate input
   - Handle errors
   - Use proper methods
   - Support filtering

2. **Response Format**
   - Use consistent structure
   - Include metadata
   - Handle pagination
   - Support caching

3. **Security**
   - Authenticate requests
   - Authorize access
   - Rate limit
   - Validate input

4. **Performance**
   - Cache responses
   - Optimize queries
   - Use compression
   - Monitor usage

## Related Documentation

- [OpenAPI Validation](openapi-validation)
- [API Reference](reference)
- [API Normalization](normalization)
- [GraphQL API](graphql)
- [Security Guide](../operations/security)
- [Development Guide](../development/README) 
