---
id: architecture-deployment
---
# Deployment Architecture Guide

<!-- title: Deployment Architecture Guide -->
<!-- description: Guide for deployment architecture in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide details the deployment architecture of the LLM Platform Platform. It covers infrastructure, containerization, orchestration, and deployment strategies.

## Infrastructure

### Kubernetes Configuration

```yaml
# kubernetes/base/deployment.yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: bfapi
  namespace: bluefly
spec:
  replicas: 3
  selector:
    matchLabels:
      app: bfapi
  template:
    metadata:
      labels:
        app: bfapi
    spec:
      containers:
      - name: bfapi
        image: bluefly/bfapi:latest
        ports:
        - containerPort: 3000
        env:
        - name: NODE_ENV
          value: production
        - name: DATABASE_URL
          valueFrom:
            secretKeyRef:
              name: bfapi-secrets
              key: database-url
        resources:
          requests:
            cpu: 100m
            memory: 256Mi
          limits:
            cpu: 500m
            memory: 512Mi
        livenessProbe:
          httpGet:
            path: /health
            port: 3000
          initialDelaySeconds: 30
          periodSeconds: 10
        readinessProbe:
          httpGet:
            path: /ready
            port: 3000
          initialDelaySeconds: 5
          periodSeconds: 5
```

### Service Configuration

```yaml
# kubernetes/base/service.yaml
apiVersion: v1
kind: Service
metadata:
  name: bfapi
  namespace: bluefly
spec:
  type: ClusterIP
  ports:
  - port: 80
    targetPort: 3000
    protocol: TCP
    name: http
  selector:
    app: bfapi
```

### Ingress Configuration

```yaml
# kubernetes/base/ingress.yaml
apiVersion: networking.k8s.io/v1
kind: Ingress
metadata:
  name: bfapi
  namespace: bluefly
  annotations:
    kubernetes.io/ingress.class: nginx
    cert-manager.io/cluster-issuer: letsencrypt-prod
spec:
  tls:
  - hosts:
    - api.bluefly.io
    secretName: bfapi-tls
  rules:
  - host: api.bluefly.io
    http:
      paths:
      - path: /
        pathType: Prefix
        backend:
          service:
            name: bfapi
            port:
              number: 80
```

## Containerization

### Dockerfile

```dockerfile
# Dockerfile
FROM node:18-alpine AS builder

WORKDIR /app

COPY package*.json ./
RUN npm ci

COPY . .
RUN npm run build

FROM node:18-alpine

WORKDIR /app

COPY --from=builder /app/dist ./dist
COPY --from=builder /app/node_modules ./node_modules
COPY package*.json ./

ENV NODE_ENV=production

EXPOSE 3000

CMD ["npm", "run", "start:prod"]
```

### Docker Compose

```yaml
# docker-compose.yml
version: '3.8'

services:
  bfapi:
    build:
      context: .
      dockerfile: Dockerfile
    ports:
      - "3000:3000"
    environment:
      - NODE_ENV=development
      - DATABASE_URL=postgresql://postgres:postgres@db:5432/bluefly
    depends_on:
      - db
      - redis
    volumes:
      - .:/app
      - /app/node_modules
  
  db:
    image: postgres:14-alpine
    environment:
      - POSTGRES_USER=postgres
      - POSTGRES_PASSWORD=postgres
      - POSTGRES_DB=bluefly
    volumes:
      - postgres_data:/var/lib/postgresql/data
  
  redis:
    image: redis:7-alpine
    volumes:
      - redis_data:/data

volumes:
  postgres_data:
  redis_data:
```

## Orchestration

### Helm Chart

```yaml
# helm/bfapi/values.yaml
replicaCount: 3

image:
  repository: bluefly/bfapi
  tag: latest
  pullPolicy: IfNotPresent

service:
  type: ClusterIP
  port: 80

ingress:
  enabled: true
  className: nginx
  hosts:
    - host: api.bluefly.io
      paths:
        - path: /
          pathType: Prefix
  tls:
    - secretName: bfapi-tls
      hosts:
        - api.bluefly.io

resources:
  requests:
    cpu: 100m
    memory: 256Mi
  limits:
    cpu: 500m
    memory: 512Mi

env:
  - name: NODE_ENV
    value: production
  - name: DATABASE_URL
    valueFrom:
      secretKeyRef:
        name: bfapi-secrets
        key: database-url
```

### CI/CD Pipeline

```yaml
# .gitlab-ci.yml
stages:
  - test
  - build
  - deploy

variables:
  DOCKER_REGISTRY: registry.bluefly.io
  DOCKER_IMAGE: $DOCKER_REGISTRY/bfapi

test:
  stage: test
  image: node:18-alpine
  script:
    - npm ci
    - npm run test
    - npm run lint

build:
  stage: build
  image: docker:20.10
  services:
    - docker:20.10-dind
  script:
    - docker build -t $DOCKER_IMAGE:$CI_COMMIT_SHA .
    - docker push $DOCKER_IMAGE:$CI_COMMIT_SHA
    - docker tag $DOCKER_IMAGE:$CI_COMMIT_SHA $DOCKER_IMAGE:latest
    - docker push $DOCKER_IMAGE:latest

deploy:
  stage: deploy
  image: dtzar/helm-kubectl:3.8.0
  script:
    - helm upgrade --install bfapi ./helm/bfapi
      --set image.tag=$CI_COMMIT_SHA
      --namespace bluefly
  environment:
    name: production
  only:
    - main
```

## Deployment Strategies

### Rolling Update

```yaml
# kubernetes/base/deployment.yaml
spec:
  strategy:
    type: RollingUpdate
    rollingUpdate:
      maxSurge: 1
      maxUnavailable: 0
```

### Blue-Green Deployment

```yaml
# kubernetes/blue-green/deployment.yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: bfapi-blue
spec:
  replicas: 3
  selector:
    matchLabels:
      app: bfapi
      version: blue
  template:
    metadata:
      labels:
        app: bfapi
        version: blue
    spec:
      containers:
      - name: bfapi
        image: bluefly/bfapi:blue
---
apiVersion: apps/v1
kind: Deployment
metadata:
  name: bfapi-green
spec:
  replicas: 3
  selector:
    matchLabels:
      app: bfapi
      version: green
  template:
    metadata:
      labels:
        app: bfapi
        version: green
    spec:
      containers:
      - name: bfapi
        image: bluefly/bfapi:green
```

### Canary Deployment

```yaml
# kubernetes/canary/deployment.yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: bfapi-canary
spec:
  replicas: 1
  selector:
    matchLabels:
      app: bfapi
      version: canary
  template:
    metadata:
      labels:
        app: bfapi
        version: canary
    spec:
      containers:
      - name: bfapi
        image: bluefly/bfapi:canary
```

## Best Practices

1. **Infrastructure**
   - Use infrastructure as code
   - Implement auto-scaling
   - Configure monitoring
   - Set up logging

2. **Containerization**
   - Use multi-stage builds
   - Optimize image size
   - Implement health checks
   - Configure resource limits

3. **Orchestration**
   - Use Helm for deployment
   - Implement CI/CD
   - Configure secrets
   - Set up networking

4. **Deployment**
   - Use rolling updates
   - Implement blue-green
   - Configure canary
   - Monitor deployments

## Related Documentation

- [Architecture Overview](overview)
- [Component Architecture](components)
- [Security Architecture](security)
- [Integration Architecture](integration)
- [API Guide](../api/README)
- [Development Guide](../development/README)
- [Operations Guide](../operations/README) 
