---
id: architecture-emergent-consciousness
---
# Component Relationships

This document outlines how the various components of the LLM Platform AI Platform interact with each other to form a cohesive enterprise AI solution.

## System Architecture Overview

The LLM Platform AI Platform uses a microservices architecture with the following key components:

```
                          ┌────────────┐
                          │   Client   │
                          └─────┬──────┘
                                │
                                ▼
┌─────────────────────────────────────────────────┐
│                   bfapi (API Gateway)           │
└─┬─────────────────────┬───────────────────────┬─┘
  │                     │                       │
  ▼                     ▼                       ▼
┌────────┐       ┌─────────────┐       ┌───────────────┐
│ bfllm  │◄─────►│    llm-mcp    │◄─────►│     bfrfp     │
└────────┘       └─────────────┘       └───────────────┘
    ▲                   ▲                      ▲
    │                   │                      │
    └───────────────────┼──────────────────────┘
                        │
                        ▼
                  ┌─────────────┐
                  │ Drupal LLM  │
                  │  Platform   │
                  └─────────────┘
```

## Detailed Architecture

### Multi-Layered Architecture

The platform implements a multi-layered architecture for separation of concerns:

```
┌───────────────────────────────────────────────────────────────────┐
│                       Presentation Layer                          │
│  ┌──────────────┐  ┌──────────────┐  ┌──────────────────────┐    │
│  │  Web Client  │  │ Mobile Client │  │ Command Line Client  │    │
│  └──────────────┘  └──────────────┘  └──────────────────────┘    │
└───────────────────────────────────────────────────────────────────┘
                              │
                              ▼
┌───────────────────────────────────────────────────────────────────┐
│                          API Layer                                │
│  ┌──────────────────────────────────────────────────────────┐    │
│  │                  bfapi (API Gateway)                      │    │
│  │  ┌────────────┐  ┌────────────┐  ┌────────────────────┐  │    │
│  │  │ Auth & JWT │  │Rate Limiter│  │Request Validation  │  │    │
│  │  └────────────┘  └────────────┘  └────────────────────┘  │    │
│  └──────────────────────────────────────────────────────────┘    │
└───────────────────────────────────────────────────────────────────┘
                              │
                              ▼
┌───────────────────────────────────────────────────────────────────┐
│                        Service Layer                              │
│  ┌──────────────┐  ┌──────────────┐  ┌──────────────────────┐    │
│  │    bfllm     │  │    llm-mcp     │  │        bfrfp         │    │
│  └──────────────┘  └──────────────┘  └──────────────────────┘    │
└───────────────────────────────────────────────────────────────────┘
                              │
                              ▼
┌───────────────────────────────────────────────────────────────────┐
│                        Data Layer                                 │
│  ┌──────────────┐  ┌──────────────┐  ┌──────────────────────┐    │
│  │ Models & Vecs│  │ Configuration│  │  Content Templates   │    │
│  └──────────────┘  └──────────────┘  └──────────────────────┘    │
└───────────────────────────────────────────────────────────────────┘
                              │
                              ▼
┌───────────────────────────────────────────────────────────────────┐
│                     External Services Layer                        │
│  ┌──────────────┐  ┌──────────────┐  ┌──────────────────────┐    │
│  │ AI Providers │  │ Cloud Storage│  │ External APIs        │    │
│  └──────────────┘  └──────────────┘  └──────────────────────┘    │
└───────────────────────────────────────────────────────────────────┘
```

### Container Orchestration Architecture

The platform is deployed using Kubernetes for container orchestration:

```
                           ┌─────────────────┐
                           │  Ingress/Load   │
                           │    Balancer     │
                           └────────┬────────┘
                                    │
                                    ▼
┌───────────────────────────────────────────────────────────────────┐
│                        Kubernetes Cluster                         │
│                                                                   │
│  ┌─────────────────────┐  ┌─────────────────────┐                │
│  │     bfapi Pods      │  │    bfllm Pods       │                │
│  │  ┌───────┐ ┌───────┐│  │  ┌───────┐ ┌───────┐│                │
│  │  │Pod 1  │ │Pod 2  ││  │  │Pod 1  │ │Pod 2  ││                │
│  │  └───────┘ └───────┘│  │  └───────┘ └───────┘│                │
│  │  ┌───────┐ ┌───────┐│  │  ┌───────┐ ┌───────┐│                │
│  │  │Pod 3  │ │Pod 4  ││  │  │Pod 3  │ │Pod 4  ││                │
│  │  └───────┘ └───────┘│  │  └───────┘ └───────┘│                │
│  └─────────────────────┘  └─────────────────────┘                │
│                                                                   │
│  ┌─────────────────────┐  ┌─────────────────────┐                │
│  │     llm-mcp Pods      │  │    bfrfp Pods       │                │
│  │  ┌───────┐ ┌───────┐│  │  ┌───────┐ ┌───────┐│                │
│  │  │Pod 1  │ │Pod 2  ││  │  │Pod 1  │ │Pod 2  ││                │
│  │  └───────┘ └───────┘│  │  └───────┘ └───────┘│                │
│  └─────────────────────┘  └─────────────────────┘                │
│                                                                   │
│  ┌─────────────────────┐  ┌─────────────────────┐                │
│  │   Drupal Pods       │  │ Supporting Services │                │
│  │  ┌───────┐ ┌───────┐│  │  ┌───────┐ ┌───────┐│                │
│  │  │Pod 1  │ │Pod 2  ││  │  │Redis  │ │Postgres││               │
│  │  └───────┘ └───────┘│  │  └───────┘ └───────┘│                │
│  └─────────────────────┘  │  ┌───────┐ ┌───────┐│                │
│                           │  │MLflow │ │Jaeger ││                │
│                           │  └───────┘ └───────┘│                │
│                           └─────────────────────┘                │
└───────────────────────────────────────────────────────────────────┘
```

## Component Interactions

### API Gateway (bfapi) Interactions

The API Gateway serves as the central entry point for all external requests.

- **Client → bfapi**: All client requests are routed through the API Gateway
- **bfapi → bfllm**: Forwards LLM inference requests
- **bfapi → llm-mcp**: Forwards MCP protocol requests for tool orchestration
- **bfapi → bfrfp**: Forwards content generation and processing requests
- **bfapi → Drupal**: Interfaces with Drupal for control plane operations

Key responsibilities:
- Authentication and authorization
- Request validation
- Rate limiting
- Request routing
- Response caching
- Load balancing
- Circuit breaking
- Observability

```javascript
// Example of bfapi routing to bfllm
router.post('/api/v1/inference', authenticate(), validate(inferenceSchema), async (req, res) => {
  try {
    const result = await llmServiceClient.infer({
      model: req.body.model,
      prompt: req.body.prompt,
      parameters: req.body.parameters
    });
    
    res.json(result);
  } catch (error) {
    handleError(error, res);
  }
});
```

### bfapi Component Architecture

The API Gateway has a modular internal architecture:

```
┌──────────────────────────────────────────────────────────────┐
│                          bfapi                               │
│                                                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │  HTTP Server   │──►│ Middleware Chain│─►│Route Handlers│  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                                        │          │
│          ▼                                        ▼          │
│  ┌────────────────┐                      ┌───────────────┐   │
│  │  TLS Manager   │                      │Service Clients│   │
│  └────────────────┘                      └───────────────┘   │
│                                                 │            │
│  ┌────────────────┐   ┌────────────────┐        │            │
│  │  Rate Limiter  │◄──┤ Auth Manager   │◄───────┘            │
│  └────────────────┘   └────────────────┘                     │
│          │                    │                              │
│          ▼                    ▼                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │ Circuit Breaker│   │  Load Balancer │  │  API Versioner│  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                    │                  │           │
│          └──────────┬─────────┘──────────┬──────┘           │
│                     │                    │                   │
│                     ▼                    ▼                   │
│               ┌────────────┐      ┌────────────┐            │
│               │  Metrics   │      │   Logger   │            │
│               └────────────┘      └────────────┘            │
└──────────────────────────────────────────────────────────────┘
```

### LLM Inference Engine (bfllm) Interactions

The LLM Inference Engine manages AI model execution.

- **bfapi → bfllm**: Receives inference requests
- **bfllm → llm-mcp**: Requests tool execution during inference
- **bfllm → External Models**: Connects to hosted models (OpenAI, Azure, etc.)
- **bfllm → MLflow**: Tracks model performance and metrics

Key responsibilities:
- Model loading and management
- Inference execution
- Prompt engineering
- Context management
- Token counting and optimization
- Model performance tracking

```javascript
// Example of bfllm handling an inference request
async function handleInference(req) {
  const { model, prompt, parameters } = req;
  
  // Get model provider
  const provider = modelRegistry.getProvider(model);
  
  // Perform inference
  const result = await provider.generateCompletion(prompt, parameters);
  
  // Track metrics
  await metricsService.recordInference({
    model,
    promptTokens: result.usage.prompt_tokens,
    completionTokens: result.usage.completion_tokens,
    latencyMs: result.latency
  });
  
  return result;
}
```

### bfllm Component Architecture

The LLM Inference Engine has a modular architecture:

```
┌──────────────────────────────────────────────────────────────┐
│                          bfllm                               │
│                                                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │ Model Registry │──►│Provider Factory│─►│Model Providers│  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                                        │          │
│          ▼                                        ▼          │
│  ┌────────────────┐                      ┌───────────────┐   │
│  │ Inference API  │                      │   Adapter     │   │
│  └────────────────┘                      │   System      │   │
│          │                               └───────────────┘   │
│          │                                      │            │
│          │                ┌───────────────────┐ │            │
│          └───────────────►│  Context Manager  │◄┘            │
│                           └───────────────────┘              │
│                                    │                         │
│  ┌────────────────┐   ┌────────────▼───────┐                 │
│  │ Token Counter  │◄──┤  Prompt Manager   │                 │
│  └────────────────┘   └────────────────────┘                 │
│                                    │                         │
│  ┌────────────────┐   ┌────────────▼───────┐                 │
│  │  MLflow Client │◄──┤ Metrics Service   │                 │
│  └────────────────┘   └────────────────────┘                 │
└──────────────────────────────────────────────────────────────┘
```

### MCP Protocol Server (llm-mcp) Interactions

The MCP Protocol Server orchestrates tools and manages tool registration.

- **bfapi → llm-mcp**: Receives tool registration and execution requests
- **bfllm → llm-mcp**: Requests tool execution during inference
- **llm-mcp → External Tools**: Communicates with registered tools
- **llm-mcp → bfrfp**: Requests content processing as a tool

Key responsibilities:
- Tool registration and discovery
- Tool execution and orchestration
- Tool capability matching
- Protocol standardization
- Tool authentication and authorization

```javascript
// Example of llm-mcp handling a tool execution request
async function executeTool(toolId, params) {
  // Get tool from registry
  const tool = toolRegistry.getTool(toolId);
  
  if (!tool) {
    throw new Error(`Tool not found: ${toolId}`);
  }
  
  // Check tool capabilities
  if (!tool.supportsParameters(params)) {
    throw new Error(`Tool ${toolId} does not support the provided parameters`);
  }
  
  // Execute tool
  const result = await tool.execute(params);
  
  // Log execution
  await logService.logToolExecution({
    toolId,
    params,
    status: 'success',
    latencyMs: result.latency
  });
  
  return result;
}
```

### llm-mcp Component Architecture

The MCP Protocol Server has a modular architecture:

```
┌──────────────────────────────────────────────────────────────┐
│                          llm-mcp                               │
│                                                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │ Tool Registry  │──►│ Tool Factory   │─►│Tool Providers │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                                        │          │
│          ▼                                        ▼          │
│  ┌────────────────┐                      ┌───────────────┐   │
│  │ MCP API Server │                      │  Protocol     │   │
│  └────────────────┘                      │  Handler      │   │
│          │                               └───────────────┘   │
│          │                                      │            │
│          │                ┌───────────────────┐ │            │
│          └───────────────►│Capability Matcher │◄┘            │
│                           └───────────────────┘              │
│                                    │                         │
│  ┌────────────────┐   ┌────────────▼───────┐                 │
│  │ Rate Limiter   │◄──┤  Tool Executor    │                 │
│  └────────────────┘   └────────────────────┘                 │
│                                    │                         │
│  ┌────────────────┐   ┌────────────▼───────┐                 │
│  │ Security Module│◄──┤ Execution Logger  │                 │
│  └────────────────┘   └────────────────────┘                 │
└──────────────────────────────────────────────────────────────┘
```

### Content Engine (bfrfp) Interactions

The Content Engine handles document generation and processing.

- **bfapi → bfrfp**: Receives content processing requests
- **llm-mcp → bfrfp**: Requests content processing as a tool
- **bfrfp → bfllm**: Requests LLM inference for content generation
- **bfrfp → External Services**: Connects to external content services

Key responsibilities:
- Document generation and formatting
- Multi-modal content processing
- Document parsing and analysis
- Template management
- Brand consistency enforcement

```javascript
// Example of bfrfp generating a document
async function generateDocument(template, data) {
  // Load template
  const templateContent = await templateService.getTemplate(template);
  
  // Process template with data
  const processedTemplate = await templateProcessor.process(templateContent, data);
  
  // Generate content using LLM
  const generatedContent = await llmClient.generateContent({
    model: 'gpt-4',
    prompt: processedTemplate,
    parameters: {
      temperature: 0.7,
      max_tokens: 2000
    }
  });
  
  // Format final document
  const document = documentFormatter.format(generatedContent, data.format || 'pdf');
  
  return document;
}
```

### bfrfp Component Architecture

The Content Engine has a modular architecture:

```
┌──────────────────────────────────────────────────────────────┐
│                          bfrfp                               │
│                                                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │Template Storage│──►│Template Loader │─►│Template Engine│  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                                        │          │
│          ▼                                        ▼          │
│  ┌────────────────┐                      ┌───────────────┐   │
│  │  Content API   │                      │  Content      │   │
│  └────────────────┘                      │  Processor    │   │
│          │                               └───────────────┘   │
│          │                                      │            │
│          │                ┌───────────────────┐ │            │
│          └───────────────►│Document Generator │◄┘            │
│                           └───────────────────┘              │
│                                    │                         │
│  ┌────────────────┐   ┌────────────▼───────┐                 │
│  │Format Converter│◄──┤  Brand Manager    │                 │
│  └────────────────┘   └────────────────────┘                 │
│                                    │                         │
│  ┌────────────────┐   ┌────────────▼───────┐                 │
│  │Storage Service │◄──┤  Export Service   │                 │
│  └────────────────┘   └────────────────────┘                 │
└──────────────────────────────────────────────────────────────┘
```

### Drupal LLM Platform Recipe Interactions

The Drupal LLM Platform Recipe provides the control plane for the AI Platform.

- **Drupal → bfapi**: Configures and manages API Gateway
- **Drupal → bfllm**: Manages models and inference settings
- **Drupal → llm-mcp**: Configures tool providers and orchestration
- **Drupal → bfrfp**: Manages content templates and generation settings

Key responsibilities:
- System configuration and management
- Model registry and management
- User interface for AI operations
- Workflow automation
- Integration with content management

```php
// Example of Drupal service configuring bfllm
public function configureModel($modelId, array $configuration) {
  // Get API client
  $apiClient = $this->apiClientFactory->getClient('bfllm');
  
  // Send configuration request
  $response = $apiClient->post('/api/v1/models/' . $modelId . '/configuration', [
    'json' => $configuration,
    'headers' => [
      'Authorization' => 'Bearer ' . $this->tokenManager->getSystemToken(),
    ],
  ]);
  
  // Log configuration change
  $this->logger->info('Model {model} configuration updated', [
    'model' => $modelId,
    'configuration' => $configuration,
  ]);
  
  return $response;
}
```

### Drupal LLM Platform Component Architecture

The Drupal control plane has a modular architecture:

```
┌───────────────────────────────────────────────────────────────┐
│               Drupal LLM Platform Recipe                      │
│                                                               │
│  ┌───────────────────────────────────────────────────────┐    │
│  │                   Drupal Core                          │    │
│  └───────────────────────────────────────────────────────┘    │
│                             │                                  │
│                             ▼                                  │
│  ┌──────────────┬───────────┴──────────┬──────────────┐       │
│  │              │                      │              │       │
│  ▼              ▼                      ▼              ▼       │
│┌─────────┐ ┌──────────┐ ┌────────────────┐ ┌───────────────┐ │
││  Field  │ │  Event   │ │Configuration   │ │  Entity       │ │
││  API    │ │Condition │ │Management      │ │  Types        │ │
│└─────────┘ │ Action   │ └────────────────┘ └───────────────┘ │
│            └──────────┘                                      │
│                │                  ┌───────────────┐          │
│                │                  │ Service       │          │
│                └─────────────────►│ Integration   │          │
│                                   └───────┬───────┘          │
│  ┌────────────────┐   ┌────────────────┐        │            │
│  │ Model Registry │◄──┤ LLM Services   │◄───────┘            │
│  └────────────────┘   └────────────────┘                     │
│          │                    │                              │
│          ▼                    ▼                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │ Tool Registry  │   │  Template Mgr  │  │  User         │  │
│  └────────────────┘   └────────────────┘  │  Interface    │  │
│          │                    │           └───────────────┘  │
│          └──────────┬─────────┘──────────┬──────┘           │
│                     │                    │                   │
│                     ▼                    ▼                   │
│               ┌────────────┐      ┌────────────┐            │
│               │Workflow    │      │ Content    │            │
│               │Automation  │      │ Management │            │
│               └────────────┘      └────────────┘            │
└───────────────────────────────────────────────────────────────┘
```

## Data Flow Diagrams

### 1. Simple Inference Request

```
┌────────┐       ┌────────┐       ┌────────┐       ┌──────────────┐
│        │  1    │        │   2   │        │   3   │              │
│ Client ├──────►│ bfapi  ├──────►│ bfllm  ├──────►│External Model│
│        │       │        │       │        │       │              │
└────────┘       └────────┘       └────────┘       └──────┬───────┘
    ▲                                                     │
    │                                                     │
    │               ┌────────┐       ┌────────┐       ┌──▼───┐
    │               │        │       │        │       │      │
    └───────────────┤ bfapi  │◄──────┤ bfllm  │◄──────┤Model │
                    │        │   5   │        │   4   │      │
                    └────────┘       └────────┘       └──────┘

1. Client sends inference request to API Gateway
2. API Gateway authenticates, validates, and routes request to LLM Inference Engine
3. LLM Inference Engine processes request and sends to appropriate model
4. Model generates response and returns to LLM Inference Engine
5. LLM Inference Engine processes response and returns to API Gateway
6. API Gateway returns response to client
```

### 2. Inference with Tool Use

```
┌────────┐      ┌────────┐      ┌────────┐      ┌────────┐      ┌─────────────┐
│        │  1   │        │  2   │        │  3   │        │  4   │             │
│ Client ├─────►│ bfapi  ├─────►│ bfllm  ├─────►│ llm-mcp  ├─────►│External Tool│
│        │      │        │      │        │      │        │      │             │
└────────┘      └────────┘      └────────┘      └────────┘      └──────┬──────┘
    ▲                                                                  │
    │                                                                  │
    │                                                                  │
    │                                                                  │
    │               ┌────────┐      ┌────────┐      ┌────────┐      ┌─▼───┐
    │               │        │  7   │        │  6   │        │  5   │     │
    └───────────────┤ bfapi  │◄─────┤ bfllm  │◄─────┤ llm-mcp  │◄─────┤Tool │
                    │        │      │        │      │        │      │     │
                    └────────┘      └────────┘      └────────┘      └─────┘

1. Client sends inference request to API Gateway
2. API Gateway routes request to LLM Inference Engine
3. LLM Inference Engine determines tool is needed and requests from MCP Protocol Server
4. MCP Protocol Server routes tool request to appropriate tool
5. Tool executes and returns result to MCP Protocol Server
6. MCP Protocol Server returns tool result to LLM Inference Engine
7. LLM Inference Engine incorporates tool result and completes inference
8. Result is returned through API Gateway to client
```

### 3. Document Generation with LLM

```
┌────────┐       ┌────────┐       ┌────────┐       ┌────────┐       ┌──────────────┐
│        │  1    │        │   2   │        │   3   │        │   4   │              │
│ Client ├──────►│ bfapi  ├──────►│ bfrfp  ├──────►│ bfllm  ├──────►│External Model│
│        │       │        │       │        │       │        │       │              │
└────────┘       └────────┘       └────────┘       └────────┘       └──────┬───────┘
    ▲                                                                      │
    │                                                                      │
    │                                                                      │
    │               ┌────────┐       ┌────────┐       ┌────────┐       ┌──▼───┐
    │               │        │       │        │       │        │       │      │
    └───────────────┤ bfapi  │◄──────┤ bfrfp  │◄──────┤ bfllm  │◄──────┤Model │
                    │        │   7   │        │   6   │        │   5   │      │
                    └────────┘       └────────┘       └────────┘       └──────┘

1. Client sends document generation request to API Gateway
2. API Gateway routes request to Content Engine
3. Content Engine processes template and data
4. Content Engine requests content generation from LLM Inference Engine
5. LLM Inference Engine generates content and returns to Content Engine
6. Content Engine formats final document
7. Formatted document is returned through API Gateway to client
```

### 4. Configuration Change via Drupal

```
┌────────┐       ┌────────────┐       ┌────────┐       ┌─────────────┐
│        │  1    │            │   2   │        │   3   │             │
│ Admin  ├──────►│ Drupal LLM ├──────►│ bfapi  ├──────►│ bfllm/llm-mcp │
│        │       │ Platform   │       │        │       │ bfrfp       │
└────────┘       └────────────┘       └────────┘       └──────┬──────┘
    ▲                  ▲                                      │
    │                  │                                      │
    │                  │                                      │
    │                  │              ┌────────┐          ┌──▼───┐
    │                  │              │        │   4      │      │
    │                  └──────────────┤ bfapi  │◄─────────┤Service│
    │                                 │        │          │      │
    │                                 └────────┘          └──────┘
    │                                      │
    │                                      │
    │                  ┌────────────┐      │
    │                  │            │      │
    └──────────────────┤ Drupal LLM │◄─────┘
                       │ Platform   │   5
                       └────────────┘

1. Admin makes configuration change in Drupal interface
2. Drupal processes change and sends to API Gateway
3. API Gateway routes configuration to appropriate service(s)
4. Services apply configuration changes
5. Configuration result is returned to Drupal
6. Drupal displays confirmation to admin
```

## Integration Points

### Service Integration

Services integrate with each other through HTTP/HTTPS APIs and gRPC:

```javascript
// bfllm client in bfapi
const llmClient = new LLMClient({
  baseUrl: process.env.BFLLM_URL || 'http://bfllm:3002',
  timeout: 30000,
  retries: 3
});

// llm-mcp client in bfllm
const mcpClient = new MCPClient({
  serverUrl: process.env.llm-mcp_URL || 'http://llm-mcp:3001',
  apiKey: process.env.llm-mcp_API_KEY
});
```

### External System Integration

The platform integrates with external systems through:

1. **API Integration**: RESTful APIs for most services
2. **Webhook Integration**: Event-based integration
3. **Database Integration**: Direct database connections where appropriate
4. **Message Queue Integration**: For asynchronous processing

## Deployment Relationships

Services can be deployed independently but have these deployment dependencies:

- **bfapi**: Can operate independently but with limited functionality
- **bfllm**: Depends on external AI models or local model files
- **llm-mcp**: Depends on registered tools to be useful
- **bfrfp**: Depends on bfllm for content generation
- **Drupal**: Depends on all services for full functionality

## Configuration Management

Configuration relationships:

1. **Environment Variables**: Primary configuration method for services
2. **Config Files**: Used for complex configuration
3. **Database Config**: Used for dynamic configuration
4. **Drupal Config**: Used for user-managed configuration

## Scaling Relationships

Services can scale independently with these considerations:

- **bfapi**: Scales horizontally for increased request handling
- **bfllm**: Scales vertically for larger models, horizontally for more concurrent requests
- **llm-mcp**: Scales horizontally for more tool orchestration
- **bfrfp**: Scales horizontally for more concurrent content processing
- **Drupal**: Scales with traditional Drupal scaling methods

## Monitoring Relationships

All services expose metrics endpoints for centralized monitoring:

```
                 ┌────────────┐
                 │ Prometheus │
                 └─────┬──────┘
                       │
        ┌──────────────┼──────────────┐
        │              │              │
        ▼              ▼              ▼
┌───────────────┐ ┌─────────┐ ┌─────────────┐
│ bfapi metrics │ │  bfllm  │ │    llm-mcp    │
└───────────────┘ │ metrics │ │   metrics   │
                  └─────────┘ └─────────────┘
                       ▲              ▲
                       │              │
                       └──────┬───────┘
                              │
                              ▼
                     ┌─────────────────┐
                     │ bfrfp metrics   │
                     └─────────────────┘
```

## Database Schema Relationships

The platform uses multiple databases with the following relationships:

```
┌──────────────────────────────────────────────────────────────┐
│                   Drupal Database                            │
│                                                              │
│  ┌────────────┐   ┌────────────┐   ┌────────────┐            │
│  │ llm_model  │───┤llm_provider│───┤llm_settings│            │
│  └────────────┘   └────────────┘   └────────────┘            │
│        │                                  │                   │
│        │                                  │                   │
│        ▼                                  ▼                   │
│  ┌────────────┐                    ┌────────────┐            │
│  │ llm_agent  │────────────────────┤ llm_tool   │            │
│  └────────────┘                    └────────────┘            │
└──────────────────────────────────────────────────────────────┘
                       │                    │
                       │                    │
                       ▼                    ▼
┌──────────────────────────────────────────────────────────────┐
│                   Service Databases                          │
│                                                              │
│  ┌────────────┐   ┌────────────┐   ┌────────────┐            │
│  │model_cache │───┤model_usage │───┤model_metrics│           │
│  └────────────┘   └────────────┘   └────────────┘            │
│                                                              │
│  ┌────────────┐   ┌────────────┐   ┌────────────┐            │
│  │tool_registry│───┤tool_executions│─┤tool_metrics│           │
│  └────────────┘   └────────────┘   └────────────┘            │
│                                                              │
│  ┌────────────┐   ┌────────────┐   ┌────────────┐            │
│  │templates   │───┤documents   │───┤content_cache│           │
│  └────────────┘   └────────────┘   └────────────┘            │
└──────────────────────────────────────────────────────────────┘
```

## Security Architecture

The platform implements a comprehensive security architecture across all components:

```
┌──────────────────────────────────────────────────────────────┐
│                  Security Architecture                       │
│                                                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │Identity & Access│──►│Authentication  │─►│Authorization  │  │
│  │  Management    │   │  Service       │  │ Service       │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                    │                  │           │
│          ▼                    ▼                  ▼           │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │ API Security   │   │ Secret Manager │  │ HTTPS/TLS     │  │
│  │ Gateway        │   │                │  │ Enforcement   │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                    │                  │           │
│          ▼                    ▼                  ▼           │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │Vulnerability   │   │Security Audit  │  │Intrusion      │  │
│  │Scanner         │   │Logger          │  │Detection      │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                    │                  │           │
│          ▼                    ▼                  ▼           │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │Data Encryption │   │Input Validation│  │Security       │  │
│  │Service         │   │& Sanitization  │  │Compliance     │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
└──────────────────────────────────────────────────────────────┘
```

### JWT Authentication Flow

```
┌─────────┐          ┌────────┐          ┌──────────────┐
│         │  1. Login │        │ 2. Auth  │              │
│  User   ├──────────►│ bfapi  ├─────────►│ Auth Service │
│         │          │        │          │              │
└─────────┘          └────────┘          └──────┬───────┘
    ▲                     ▲                     │
    │                     │                     │
    │                     │ 3. JWT Token        │
    │                     │◄────────────────────┘
    │                     │
    │  4. JWT Token       │
    │◄────────────────────┘
    │
    │  5. API Request + JWT
    │ ┌───────────────────►┌────────┐
    │ │                    │        │
    │ │                    │ bfapi  │───┐
    │ │                    │        │   │ 6. Validate
    └─┘                    └────────┘   │    Token
                               ▲        │
                               │        ▼
                               │    ┌──────────┐
                               │    │JWT       │
                               └────┤Validator │
                        7. Response │          │
                                    └──────────┘
```

### Data Encryption System

```
┌────────────────────────────────────────────────────────────┐
│                 Data Encryption System                     │
│                                                            │
│  ┌──────────────┐   ┌────────────────┐  ┌─────────────┐    │
│  │ Key Vault    │──►│ Key Rotation   │─►│Encryption   │    │
│  │              │   │ Service        │  │Service      │    │
│  └──────────────┘   └────────────────┘  └─────────────┘    │
│          │                 │                  │            │
│          │                 │                  │            │
│          ▼                 ▼                  ▼            │
│  ┌─────────────────────────────────────────────────────┐   │
│  │              Encrypted Data Storage                  │   │
│  │                                                      │   │
│  │  ┌───────────┐  ┌───────────┐  ┌────────────────┐   │   │
│  │  │ Encrypted │  │ Encrypted │  │ Encrypted      │   │   │
│  │  │ Config    │  │ Secrets   │  │ Personal Data  │   │   │
│  │  └───────────┘  └───────────┘  └────────────────┘   │   │
│  └─────────────────────────────────────────────────────┘   │
│                             │                              │
│                             │                              │
│                             ▼                              │
│  ┌─────────────────┐  ┌───────────────┐  ┌──────────────┐  │
│  │Transport Layer  │  │Application    │  │Database      │  │
│  │Encryption (TLS) │  │Layer Encryption│ │Encryption    │  │
│  └─────────────────┘  └───────────────┘  └──────────────┘  │
└────────────────────────────────────────────────────────────┘
```

## Error Handling Architecture

The platform implements robust error handling patterns across services:

```
┌──────────────────────────────────────────────────────────────┐
│                  Error Handling Architecture                 │
│                                                              │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │ Error Detection│──►│Error Logging   │─►│Error          │  │
│  │                │   │                │  │Classification  │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                    │                  │           │
│          ▼                    ▼                  ▼           │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │ Circuit Breaker│   │ Retry Strategy │  │ Fallback      │  │
│  │ Pattern        │   │                │  │ Mechanisms    │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
│          │                    │                  │           │
│          ▼                    ▼                  ▼           │
│  ┌────────────────┐   ┌────────────────┐  ┌───────────────┐  │
│  │Error Reporting │   │Alert Generation│  │Error Recovery │  │
│  │                │   │                │  │               │  │
│  └────────────────┘   └────────────────┘  └───────────────┘  │
└──────────────────────────────────────────────────────────────┘
```

### Circuit Breaker Pattern Implementation

```
┌──────────┐     ┌────────────────────────────────────────┐
│          │     │           Circuit Breaker              │
│ Service A│     │                                        │
│  Client  │     │  ┌─────────┐    ┌───────┐   ┌───────┐  │
│          │─────┼─►│ Closed  │───►│ Open  │◄──┤Half-Open│ │
└──────────┘     │  │(Normal) │    │(Failing)│  │(Testing)│ │
     │           │  └─────────┘    └───────┘   └───────┘  │
     │           │       │             │           ▲      │
     │           └───────┼─────────────┼───────────┼──────┘
     │                   │             │           │
     │                   │             │           │
     ▼                   ▼             │           │
┌──────────┐      ┌────────────┐       │           │
│          │      │            │       │           │
│ Service B│◄─────┤ Success    │       │           │
│          │      │            │       │           │
└──────────┘      └────────────┘       │           │
                                       ▼           │
                                  ┌────────────┐   │
                                  │            │   │
                                  │ Fallback   │   │
                                  │            │   │
                                  └────────────┘   │
                                       │           │
                                       │           │
                                       ▼           │
                                  ┌────────────┐   │
                                  │            │   │
                                  │ Wait/Reset │───┘
                                  │            │
                                  └────────────┘
```

### Global Exception Handling

```typescript
// Example of global exception handling
app.use((err, req, res, next) => {
  // Log error details
  logger.error('Unhandled exception', {
    error: err.message,
    stack: err.stack,
    requestId: req.id,
    path: req.path,
    method: req.method
  });
  
  // Classify error
  const errorResponse = errorClassifier.classify(err);
  
  // Alert if critical
  if (errorResponse.severity === 'critical') {
    alertService.sendAlert({
      title: 'Critical API Error',
      message: err.message,
      service: 'bfapi',
      requestId: req.id
    });
  }
  
  // Return appropriate response
  res.status(errorResponse.statusCode).json({
    error: errorResponse.publicMessage,
    requestId: req.id,
    code: errorResponse.code
  });
});
```

## Advanced Data Processing Flows

### ETL Pipeline for Analytics

```
┌────────┐     ┌─────────┐     ┌──────────┐     ┌─────────────┐
│        │     │         │     │          │     │             │
│ Source │────►│ Extract │────►│ Transform│────►│ Load        │
│ Data   │     │         │     │          │     │             │
└────────┘     └─────────┘     └──────────┘     └─────────────┘
                    │                │                 │
                    ▼                ▼                 ▼
               ┌──────────┐    ┌─────────────┐   ┌─────────────┐
               │          │    │             │   │             │
               │ Validation│    │ Enrichment │   │ Data        │
               │          │    │             │   │ Warehouse   │
               └──────────┘    └─────────────┘   └─────────────┘
                                                       │
                                                       │
                                                       ▼
                                                 ┌──────────────┐
                                                 │              │
                                                 │ Analytics    │
                                                 │ Dashboard    │
                                                 │              │
                                                 └──────────────┘
```

### Streaming Data Architecture

```
┌────────────┐   ┌────────────┐   ┌────────────┐   ┌────────────┐
│            │   │            │   │            │   │            │
│ Event      │──►│ Kafka      │──►│ Stream     │──►│ Real-time  │
│ Sources    │   │ Broker     │   │ Processor  │   │ Analytics  │
│            │   │            │   │            │   │            │
└────────────┘   └────────────┘   └────────────┘   └────────────┘
      │                                 │                │
      │                                 │                │
      ▼                                 ▼                ▼
┌────────────┐                    ┌────────────┐   ┌────────────┐
│            │                    │            │   │            │
│ Monitoring │                    │ Data Lake  │   │ Alerts &   │
│ Dashboard  │                    │            │   │ Actions    │
│            │                    │            │   │            │
└────────────┘                    └────────────┘   └────────────┘
```

## High Availability Architecture

The platform is designed for high availability across all components:

```
┌─────────────────────────────────────────────────────────────────┐
│                  High Availability Architecture                 │
│                                                                 │
│  ┌─────────────────────┐         ┌─────────────────────┐        │
│  │   Region A          │         │   Region B          │        │
│  │                     │         │                     │        │
│  │  ┌───────────────┐  │         │  ┌───────────────┐  │        │
│  │  │ Load Balancer │  │         │  │ Load Balancer │  │        │
│  │  └───────┬───────┘  │         │  └───────┬───────┘  │        │
│  │          │          │         │          │          │        │
│  │  ┌───────▼───────┐  │         │  ┌───────▼───────┐  │        │
│  │  │   Services    │  │◄────────┼─►│   Services    │  │        │
│  │  │ Cluster A     │  │         │  │ Cluster B     │  │        │
│  │  └───────┬───────┘  │         │  └───────┬───────┘  │        │
│  │          │          │         │          │          │        │
│  │  ┌───────▼───────┐  │         │  ┌───────▼───────┐  │        │
│  │  │  Database     │  │◄────────┼─►│  Database     │  │        │
│  │  │  Cluster A    │  │         │  │  Cluster B    │  │        │
│  │  └───────────────┘  │         │  └───────────────┘  │        │
│  │                     │         │                     │        │
│  └─────────────────────┘         └─────────────────────┘        │
│                                                                 │
│  ┌─────────────────────────────────────────────────────────┐    │
│  │                Global Traffic Manager                    │    │
│  └─────────────────────────────────────────────────────────┘    │
│                                                                 │
│  ┌─────────────────────────────────────────────────────────┐    │
│  │                Disaster Recovery System                  │    │
│  └─────────────────────────────────────────────────────────┘    │
└─────────────────────────────────────────────────────────────────┘
```

### Database Replication Architecture

```
┌───────────────────────────────────────────────────────────┐
│              Database Replication Architecture            │
│                                                           │
│  ┌──────────────┐      ┌────────────────────────────┐     │
│  │              │      │                            │     │
│  │ Primary DB   │─────►│ Replica 1                  │     │
│  │              │      │                            │     │
│  └──────┬───────┘      └────────────────────────────┘     │
│         │                                                 │
│         │              ┌────────────────────────────┐     │
│         │              │                            │     │
│         └─────────────►│ Replica 2                  │     │
│                        │                            │     │
│                        └────────────────────────────┘     │
│                                                           │
│  ┌──────────────┐      ┌────────────────────────────┐     │
│  │              │      │                            │     │
│  │ Failover     │◄────►│ Automatic Failover         │     │
│  │ Manager      │      │ Detection                  │     │
│  │              │      │                            │     │
│  └──────────────┘      └────────────────────────────┘     │
└───────────────────────────────────────────────────────────┘
```

## Advanced API Management

```
┌─────────────────────────────────────────────────────────────────┐
│                  API Management Architecture                    │
│                                                                 │
│  ┌─────────────────┐    ┌─────────────────┐   ┌─────────────┐   │
│  │                 │    │                 │   │             │   │
│  │ API Developer   │───►│ API Design &    │──►│API Gateway  │   │
│  │ Portal          │    │ Documentation   │   │             │   │
│  │                 │    │                 │   │             │   │
│  └─────────────────┘    └─────────────────┘   └─────┬───────┘   │
│                                                     │           │
│  ┌─────────────────┐    ┌─────────────────┐   ┌─────▼───────┐   │
│  │                 │    │                 │   │             │   │
│  │ API Analytics   │◄───┤ API Monitoring  │◄──┤API Security │   │
│  │                 │    │                 │   │             │   │
│  └─────────────────┘    └─────────────────┘   └─────────────┘   │
│                                                                 │
│  ┌─────────────────┐    ┌─────────────────┐   ┌─────────────┐   │
│  │                 │    │                 │   │             │   │
│  │ API Versioning  │───►│ API Testing     │──►│API Lifecycle│   │
│  │ & Deprecation   │    │                 │   │Management   │   │
│  │                 │    │                 │   │             │   │
│  └─────────────────┘    └─────────────────┘   └─────────────┘   │
└─────────────────────────────────────────────────────────────────┘
```

## Conclusion

The component relationships in the LLM Platform AI Platform create a flexible, scalable architecture where:

1. Each component has a clear, focused responsibility
2. Components communicate through well-defined interfaces
3. Components can be deployed, scaled, and updated independently
4. The system remains resilient through circuit breaking and fallbacks
5. The control plane (Drupal) provides centralized management
6. Security is implemented at multiple layers with encryption, authentication, and authorization
7. Error handling follows robust patterns for resilience and recovery
8. High availability is achieved through redundancy and automatic failover
9. Advanced data processing supports analytics and real-time insights

This architecture enables the platform to handle enterprise-scale AI workloads while maintaining flexibility for different deployment scenarios and use cases.
