# CLAUDE.md for BFCIComponents

This document provides essential guidance for Claude Code when working with the BFCIComponents repository.

## Module Overview

BFCIComponents is an enterprise-grade GitLab CI/CD component catalog providing production-ready pipeline components for modern software development. It implements GitLab 17.0+ catalog standards with OpenAPI 3.1 compliance, offering standardized CI/CD components for Drupal, Node.js, Kubernetes, and platform services with comprehensive testing, security validation, and observability features.

## Ecosystem Context

BFCIComponents serves as the central CI/CD automation hub for the entire Bluefly AI Platform ecosystem. It provides specialized pipeline components and "Golden Components" that orchestrate complete CI/CD workflows for:

- **llm-gateway**: API Gateway deployment pipelines with security scanning and performance testing
- **bfllm**: LLM service CI/CD with model validation and inference testing
- **llm-mcp**: Model Control Plane deployment with tool orchestration validation
- **bfrfp**: Content generation workflow testing and deployment automation
- **bfui**: Frontend component testing, building, and deployment pipelines
- **llm-gatewayclient**: Client library testing, publishing, and distribution automation
- **llmcli**: CLI tool testing, building, and NPM publishing workflows
- **bfdocs**: Documentation generation, validation, and GitLab Pages deployment
- **Drupal Modules**: Comprehensive Drupal module, recipe, and site CI/CD pipelines
- **Kubernetes Services**: Container orchestration and deployment automation

The component catalog ensures consistent, secure, and reliable deployment practices across all platform services while providing reusable, composable pipeline components that reduce configuration complexity and maintenance overhead.

## Project Structure

The repository follows GitLab CI/CD component catalog standards with comprehensive organization:

```
bfcicomponents/
├── components/                    # Component catalog
│   ├── api/                      # API-related pipeline components
│   ├── bfc/                      # BlueFly Components framework
│   ├── ci/                       # CI automation components
│   ├── docs/                     # Documentation pipeline components
│   ├── drupal/                   # Drupal-specific components
│   │   ├── drupal_golden_ci_module/    # Golden component for modules
│   │   ├── drupal_golden_ci_recipe/    # Golden component for recipes
│   │   ├── drupal_golden_ci_site/      # Golden component for sites
│   │   └── shared/               # Shared Drupal utilities
│   ├── k8s/                      # Kubernetes deployment components
│   ├── monitoring/               # Monitoring & observability
│   ├── nodejs/                   # Node.js application components
│   │   ├── npm_golden_ci_package/      # Golden component for NPM packages
│   │   └── ml_node_package/      # ML-specific Node.js components
│   └── shared/                   # Shared/common components
├── tools/                        # Development and validation tools
│   ├── scripts/                  # Component validation and build scripts
│   ├── templates/                # Component generation templates
│   └── validators/               # Schema and structure validators
├── tests/                        # Comprehensive test suite
│   ├── unit/                     # Unit tests for components
│   ├── integration/              # Integration testing
│   ├── e2e/                      # End-to-end pipeline testing
│   └── fixtures/                 # Test fixtures and mock data
├── docs/                         # Documentation and guides
├── examples/                     # Usage examples and demos
└── catalog-manifest.yml          # GitLab catalog manifest
```

### Component Structure Standard

Each component follows this consistent structure:

```
/components/[category]/[component-name]/
├── README.md                     # Component documentation
├── LICENSE.md                    # License information
├── template.yml                  # GitLab CI component template
├── config/                       # Configuration files
│   ├── defaults.yml             # Default configuration
│   └── validation.yml           # Input validation schema
├── resources/                    # Supporting resources
│   ├── k8s/                     # Kubernetes manifests
│   ├── docker/                  # Dockerfile templates
│   └── scripts/                 # Supporting scripts
└── tests/                        # Component-specific tests
    ├── unit.test.js             # Unit tests
    └── fixtures/                # Test fixtures
```

## Key Features

### Golden Components System

The cornerstone of BFCIComponents is the "Golden Components" concept - comprehensive, battle-tested pipeline templates that provide complete CI/CD workflows with minimal configuration:

#### Drupal Golden Components
- **drupal_golden_ci_module**: Complete Drupal module CI/CD (lint, test, security, Drupal.org release)
- **drupal_golden_ci_recipe**: Drupal recipe validation, testing, and distribution
- **drupal_golden_ci_site**: Full-site deployment with multi-environment support

#### Node.js Golden Components  
- **npm_golden_ci_package**: NPM package CI/CD with TypeScript, testing, and publishing
- **ml_node_package**: ML-specific Node.js packages with model validation and testing

#### Platform Components
- **bfc/core**: BlueFly Components framework for creating new components
- **api/gateway**: API Gateway deployment with OpenAPI validation
- **k8s/platform**: Kubernetes platform deployment and orchestration
- **monitoring/observability**: Comprehensive monitoring and alerting setup

### Component Categories

```typescript
// Component categorization system
interface ComponentCategory {
  api: 'API Gateway and service components';
  bfc: 'BlueFly Components framework';
  ci: 'General CI automation components';
  docs: 'Documentation generation and deployment';
  drupal: 'Drupal-specific pipeline components';
  k8s: 'Kubernetes deployment and orchestration';
  monitoring: 'Observability and monitoring setup';
  nodejs: 'Node.js application components';
  shared: 'Common utilities and shared components';
}
```

## Development Commands

```bash
# Installation and Setup
npm install
npm run setup:dev               # Setup development environment

# Component Development
npm run create:component        # Interactive component generator
npm run validate                # Validate all components
npm run validate:structure      # Validate component structure
npm run validate:schema         # Validate component schemas
npm run validate:docs           # Validate component documentation
npm run validate:catalog        # Validate GitLab catalog compliance
npm run validate:security       # Security validation of components

# Code Quality
npm run lint                    # ESLint checks
npm run lint:fix                # Fix ESLint issues
npm run lint:yaml               # YAML file linting
npm run format                  # Format code with Prettier
npm run typecheck               # TypeScript type checking

# Testing
npm test                        # Run all tests
npm run test:unit               # Unit tests only
npm run test:integration        # Integration tests only
npm run test:e2e                # End-to-end tests
npm run test:components         # Component-specific tests
npm run test:performance        # Performance benchmarks
npm run test:coverage           # Tests with coverage report
npm run test:shared             # Test shared utilities
npm run test:watch              # Tests in watch mode

# Building and Publishing
npm run build                   # Build components
npm run build:docs              # Build documentation
npm run build:catalog           # Build catalog manifest
npm run semantic-release        # Create semantic release
npm run publish:catalog         # Publish to GitLab catalog
npm run publish:npm             # Publish to NPM registry

# Development Workflow
npm run dev                     # Development mode with validation
npm run ci                      # Full CI pipeline locally
npm run clean                   # Clean build artifacts
npm run migrate:shared          # Migrate to shared utilities

# Documentation
npm run docs:generate           # Generate component documentation
npm run docs:validate           # Validate documentation
npm run docs:serve              # Serve documentation locally
```

## Important Conventions

### Component Development Workflow

Follow this strict workflow when creating or modifying components:

```bash
# 1. Create component structure
npm run create:component
# Interactive prompts will guide you through:
# - Component category selection
# - Component naming (kebab-case)
# - Template type (golden, specialized, shared)
# - Input parameter definition
# - Documentation generation

# 2. Validate component structure
npm run validate:structure

# 3. Test component functionality
npm run test:components -- --component=your-component

# 4. Validate GitLab CI compatibility
npm run validate:schema

# 5. Generate and validate documentation
npm run build:docs
npm run validate:docs

# 6. Full validation suite
npm run validate
```

### Component Naming Conventions

```yaml
# Component naming pattern
component_naming:
  format: kebab-case
  pattern: "[category]_[purpose]_[type]"
  examples:
    - "drupal_golden_ci_module"     # Golden component
    - "nodejs_test_runner"          # Specialized component
    - "shared_security_scanner"     # Shared utility
    - "k8s_deployment_manager"      # Infrastructure component

# Input parameter naming
input_naming:
  format: snake_case
  examples:
    - "php_version"
    - "node_version"
    - "min_test_coverage"
    - "deployment_environment"
```

### Component Template Structure

```yaml
# GitLab CI Component Template Standard
spec:
  inputs:
    # Always include these standard inputs
    component_version:
      type: string
      description: "Component version for tracking"
      default: "1.0.0"
    
    debug_mode:
      type: boolean
      description: "Enable debug output"
      default: false
    
    # Component-specific inputs
    custom_input:
      type: string
      description: "Description with examples"
      default: "sensible_default"
      required: false

variables:
  # Derive variables from inputs
  COMPONENT_VERSION: "$[[ inputs.component_version ]]"
  DEBUG_ENABLED: "$[[ inputs.debug_mode ]]"
  
stages:
  - validate
  - prepare  
  - build
  - test
  - security
  - deploy
  - cleanup

# Include shared components for common functionality
include:
  - component: $CI_PROJECT_PATH/shared/base-setup@$CI_COMMIT_SHA
  - component: $CI_PROJECT_PATH/shared/security-scanner@$CI_COMMIT_SHA
    inputs:
      scan_type: "comprehensive"
```

## Testing Approach

BFCIComponents follows comprehensive testing standards with Test-Driven Development methodology:

### Test Categories

- **Unit Tests**: Individual component validation and functionality testing
- **Integration Tests**: Cross-component interaction and pipeline integration testing
- **E2E Tests**: Complete pipeline execution testing in GitLab CI environment
- **Performance Tests**: Component execution time and resource usage benchmarks
- **Security Tests**: Vulnerability scanning and security validation
- **Compliance Tests**: GitLab catalog standard compliance verification

### Test Requirements

- **Coverage**: Minimum 89% test coverage across all components
- **TDD**: All components developed using Test-Driven Development
- **Validation**: Automated component structure and schema validation
- **Performance**: Component execution under 5 minutes for standard workflows
- **Security**: Automated security scanning for all components

### Component Testing Implementation

```typescript
// Component testing pattern
describe('Drupal Golden CI Module Component', () => {
  let component: ComponentValidator;
  let mockGitLabCI: jest.Mocked<GitLabCIRunner>;
  
  beforeEach(() => {
    component = new ComponentValidator({
      path: './components/drupal/drupal_golden_ci_module',
      schema: 'gitlab-ci-component.schema.json'
    });
    mockGitLabCI = createMockGitLabCI();
  });
  
  it('should validate component structure', async () => {
    // Arrange
    const validation = await component.validateStructure();
    
    // Assert
    expect(validation.isValid).toBe(true);
    expect(validation.errors).toHaveLength(0);
    expect(validation.hasRequiredFiles).toBe(true);
  });
  
  it('should execute complete Drupal module pipeline', async () => {
    // Arrange
    const inputs = {
      php_version: '8.2',
      drupal_version_constraint: '^10',
      module_name: 'test_module'
    };
    
    // Act
    const result = await mockGitLabCI.executePipeline(
      component.getTemplate(),
      inputs
    );
    
    // Assert
    expect(result.status).toBe('success');
    expect(result.stages).toEqual([
      'validate', 'prepare', 'build', 'test', 'security', 'deploy'
    ]);
    expect(result.artifacts).toContain('module_package.tar.gz');
  });
  
  it('should handle invalid inputs gracefully', async () => {
    // Arrange
    const invalidInputs = {
      php_version: '7.4', // Unsupported version
      module_name: 'Invalid Module Name' // Invalid naming
    };
    
    // Act & Assert
    await expect(
      component.validateInputs(invalidInputs)
    ).rejects.toThrow('Invalid component inputs');
  });
});

// Pipeline integration testing
describe('Component Pipeline Integration', () => {
  it('should integrate multiple components successfully', async () => {
    const pipeline = new PipelineBuilder()
      .addComponent('shared/base-setup')
      .addComponent('drupal/lint')
      .addComponent('drupal/test')
      .addComponent('shared/security-scanner')
      .addComponent('drupal/deploy');
    
    const result = await pipeline.execute({
      php_version: '8.2',
      environment: 'staging'
    });
    
    expect(result.success).toBe(true);
    expect(result.executionTime).toBeLessThan(300); // 5 minutes
  });
});
```

## Integration With Other Projects

BFCIComponents provides specialized CI/CD automation for the entire Bluefly ecosystem:

### Core Platform Services

- **llm-gateway**: API Gateway deployment with OpenAPI validation, security scanning, and multi-environment deployment
- **bfllm**: LLM service CI/CD with model validation, performance testing, and inference benchmarking
- **llm-mcp**: Model Control Plane deployment with tool orchestration testing and MCP protocol validation
- **bfrfp**: Content generation workflow testing, validation, and automated deployment
- **bfui**: Frontend component testing, Storybook generation, accessibility testing, and CDN deployment
- **llm-gatewayclient**: Client library testing, documentation generation, NPM publishing, and SDK distribution
- **llmcli**: CLI tool testing, cross-platform building, NPM publishing, and global installation testing
- **bfdocs**: Documentation generation, link validation, search indexing, and GitLab Pages deployment

### Drupal Platform Integration

- **Drupal Modules**: Located in `./DEMO/web/modules/custom/` with comprehensive module CI/CD
- **drupal_native_ai**: Core AI integration module with Drupal.org release automation
- **llm**: LLM platform integration with dependency management and compatibility testing
- **secure_drupal**: Security framework with compliance testing and audit automation
- **field_eca_bridge**: Field automation with ECA integration testing

### Component Usage Patterns

```yaml
# Platform service deployment pattern
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
    inputs:
      service_name: "llm-gateway"
      node_version: "20"
      openapi_spec: "openapi.yaml"
      security_scan: true
      performance_test: true
      deployment_environments: ["dev", "staging", "prod"]

# Drupal module pattern
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      module_name: "drupal_native_ai"
      php_version: "8.3"
      drupal_version_constraint: "^11"
      drupalorg_release: true
      security_scan: true
      accessibility_test: true
```

## Shared Utilities (@bluefly/shared)

BFCIComponents extensively uses the `@bluefly/shared` package for common functionality:

### Import Patterns

```typescript
// Correct import patterns for BFCIComponents
import [logger] from '@bluefly/shared/utils';
import [GitLabCIValidator] from '@bluefly/shared/ci';
import [ComponentBuilder] from '@bluefly/shared/components';
import [SecurityScanner] from '@bluefly/shared/security';

// Component-specific utilities
import [PipelineOrchestrator] from '@bluefly/shared/pipelines';
import [ArtifactManager] from '@bluefly/shared/artifacts';
import [DeploymentManager] from '@bluefly/shared/deployment';
```

### Development Setup

```bash
# Link shared package for development
cd ../llmcli/shared
npm install && npm link

cd ../bfcicomponents
npm link @bluefly/shared
```

## Common Pitfalls

1. **Component Complexity**: Keep components focused on single responsibility; use composition for complex workflows
2. **Input Validation**: Always validate component inputs with comprehensive schema validation
3. **GitLab CI Compatibility**: Test components against actual GitLab CI environment before release
4. **Resource Management**: Monitor component resource usage to prevent pipeline timeouts
5. **Version Management**: Use semantic versioning and maintain backward compatibility
6. **Security**: Never expose secrets in component templates or logs
7. **Documentation Drift**: Keep component documentation synchronized with implementation changes

## Performance Considerations

- **Execution Time**: Components should complete standard workflows within 5 minutes
- **Resource Usage**: Optimize memory and CPU usage for GitLab shared runners
- **Parallel Execution**: Design components to support parallel job execution
- **Caching**: Implement intelligent caching for dependencies and build artifacts
- **Artifact Management**: Optimize artifact size and retention policies

## Security Features

- **Secret Management**: Secure handling of CI/CD secrets and tokens
- **Vulnerability Scanning**: Automated security scanning for all deployments
- **Access Control**: Role-based access control for component usage
- **Audit Logging**: Comprehensive logging of all pipeline activities
- **Compliance**: Built-in compliance checks for security standards

## Troubleshooting

### Common Issues

1. **Component Validation Failures**: Check component structure and schema compliance
2. **Pipeline Execution Errors**: Verify GitLab CI syntax and component inputs
3. **Resource Timeouts**: Optimize component resource usage and execution time
4. **Integration Failures**: Validate component dependencies and includes
5. **Security Scan Issues**: Review security scan results and remediate findings

### Debug Mode

```bash
# Enable debug mode for components
DEBUG=bfci:* npm run dev

# Validate specific component
npm run validate -- --component=drupal/drupal_golden_ci_module

# Test component in isolation
npm run test:components -- --component=nodejs/npm_golden_ci_package
```

## File Preservation Policy

**IMPORTANT**: Files in this codebase should NEVER be deleted. Instead:

1. Move files that need to be "removed" to the `__DELETE_LATER` directory
2. Preserve the original directory structure
3. Add a date suffix to the filename (e.g., `filename_YYYY-MM-DD.ext`)

## Component Roadmap

### Current Development Focus

1. **Enhanced Golden Components**: More comprehensive and opinionated pipeline templates
2. **AI-Powered Testing**: Integration with LLM services for intelligent testing
3. **Multi-Cloud Support**: Components for AWS, Azure, and GCP deployments
4. **Advanced Security**: Enhanced security scanning and compliance automation
5. **Performance Optimization**: Faster component execution and resource efficiency

### Future Enhancements

- **Visual Pipeline Builder**: GUI for creating and configuring components
- **Component Marketplace**: Community-driven component sharing and discovery
- **Smart Recommendations**: AI-powered component suggestions based on project analysis
- **Real-time Monitoring**: Live pipeline monitoring and alerting
- **Cost Optimization**: Automated cost analysis and optimization recommendations