# GitLab CI/CD Component Catalog

## Overview

This catalog contains 50+ production-ready GitLab CI/CD components for modern software development.

## Component Categories

### 🏗️ Drupal Components

| Component | Description | Key Features |
|-----------|-------------|--------------|
| `drupal/build` | Build Drupal projects | PHP 8.1-8.3, Composer 2, Node.js support |
| `drupal/test` | Run Drupal tests | PHPUnit, Nightwatch, coverage reports |
| `drupal/deploy` | Deploy Drupal sites | Drush commands, database backups, config sync |
| `drupal/security` | Security scanning | PHPCS, security advisories, dependency audit |
| `drupal/quality` | Code quality checks | PHPMD, PHPCPD, Drupal coding standards |
| `drupal/module` | Module CI/CD | Drupal.org packaging, version constraints |
| `drupal/recipe` | Recipe testing | Recipe validation, dependency resolution |

### 📦 Node.js/TypeScript Components  

| Component | Description | Key Features |
|-----------|-------------|--------------|
| `nodejs/build` | Build Node.js projects | TypeScript, Webpack, Rollup support |
| `nodejs/test` | Test Node.js apps | Jest, Vitest, coverage thresholds |
| `nodejs/publish` | Publish to npm | npm/GitHub/GitLab registries |
| `nodejs/lint` | Code quality | ESLint, Prettier, TypeScript checks |
| `nodejs/security` | Security audit | npm audit, Snyk, dependency scanning |
| `nodejs/bundle` | Bundle optimization | Tree shaking, code splitting |

### 🐍 Python Components

| Component | Description | Key Features |
|-----------|-------------|--------------|
| `python/build` | Build Python packages | setuptools, poetry, pip |
| `python/test` | Run Python tests | pytest, coverage, tox |
| `python/lint` | Code quality | flake8, black, mypy |
| `python/publish` | Publish packages | PyPI, private registries |
| `python/security` | Security scanning | bandit, safety, pip-audit |

### 🤖 AI/ML Components

| Component | Description | Key Features |
|-----------|-------------|--------------|
| `ai/ollama` | Ollama integration | Local LLM deployment |
| `ai/langchain` | LangChain pipelines | Document processing, RAG |
| `ai/model-training` | Train ML models | PyTorch, TensorFlow |
| `ai/documentation` | AI doc generation | Auto-generate from code |
| `ai/evaluation` | Model evaluation | Benchmarks, metrics |

### 🐳 Docker Components

| Component | Description | Key Features |
|-----------|-------------|--------------|
| `docker/build` | Build containers | Multi-platform, caching |
| `docker/scan` | Security scanning | Trivy, Clair, Snyk |
| `docker/publish` | Push to registry | DockerHub, GitLab, ECR |
| `docker/compose` | Docker Compose | Multi-service testing |

### ☸️ Kubernetes Components

| Component | Description | Key Features |
|-----------|-------------|--------------|
| `kubernetes/deploy` | Deploy to K8s | Rolling updates, health checks |
| `kubernetes/helm` | Helm charts | Chart testing, publishing |
| `kubernetes/validate` | Manifest validation | kubeval, conftest |
| `kubernetes/rollout` | Progressive delivery | Canary, blue-green |

### 🔧 Shared Components

| Component | Description | Key Features |
|-----------|-------------|--------------|
| `shared/setup` | Common setup tasks | Tool installation, caching |
| `shared/cache` | Caching strategies | Dependency, build caching |
| `shared/artifacts` | Artifact handling | Upload, download, retention |
| `shared/notifications` | Send notifications | Slack, email, webhooks |

## Usage Examples

### Complete Drupal Pipeline

```yaml
include:
  # Build
  - component: gitlab.com/bluefly/cicd-components/drupal/build@main
    inputs:
      php_version: '8.3'
      drupal_version: '11'
      node_version: '20'
  
  # Test
  - component: gitlab.com/bluefly/cicd-components/drupal/test@main
    inputs:
      test_suite: 'all'
      coverage: true
      browser: 'chrome'
  
  # Security
  - component: gitlab.com/bluefly/cicd-components/drupal/security@main
    inputs:
      phpcs_standard: 'Drupal,DrupalPractice'
      check_advisories: true
  
  # Deploy
  - component: gitlab.com/bluefly/cicd-components/drupal/deploy@main
    inputs:
      environment: 'production'
      backup_database: true
      drush_commands: ['updb', 'cim', 'cr']

stages:
  - build
  - test
  - security
  - deploy

# The components handle everything!
```

### Node.js Package Pipeline

```yaml
include:
  # Setup
  - component: gitlab.com/bluefly/cicd-components/shared/setup@main
    inputs:
      tools: ['node', 'docker']
  
  # Build
  - component: gitlab.com/bluefly/cicd-components/nodejs/build@main
    inputs:
      node_version: '20'
      package_manager: 'pnpm'
      typescript: true
  
  # Test
  - component: gitlab.com/bluefly/cicd-components/nodejs/test@main
    inputs:
      coverage_threshold:
        statements: 90
        branches: 85
        functions: 90
        lines: 90
  
  # Publish
  - component: gitlab.com/bluefly/cicd-components/nodejs/publish@main
    inputs:
      registry: 'npmjs'
      access: 'public'
      tag: 'latest'
```

### AI-Enhanced Documentation

```yaml
include:
  # Generate docs with AI
  - component: gitlab.com/bluefly/cicd-components/ai/documentation@main
    inputs:
      provider: 'ollama'
      model: 'mixtral:latest'
      mode: 'create'
      file_patterns: ['src/**/*.ts', 'lib/**/*.js']
      languages: ['en', 'es', 'fr']
  
  # Evaluate documentation quality
  - component: gitlab.com/bluefly/cicd-components/ai/evaluation@main
    inputs:
      metrics: ['readability', 'completeness', 'accuracy']
```

### Multi-Platform Docker Build

```yaml
include:
  # Build for multiple platforms
  - component: gitlab.com/bluefly/cicd-components/docker/build@main
    inputs:
      platforms: ['linux/amd64', 'linux/arm64', 'linux/arm/v7']
      dockerfile: 'Dockerfile.production'
      build_args:
        NODE_VERSION: '20'
        APP_VERSION: $CI_COMMIT_TAG
  
  # Security scan
  - component: gitlab.com/bluefly/cicd-components/docker/scan@main
    inputs:
      scanner: 'trivy'
      severity: 'CRITICAL,HIGH'
      fail_on_vulnerabilities: true
  
  # Publish to multiple registries
  - component: gitlab.com/bluefly/cicd-components/docker/publish@main
    inputs:
      registries:
        - registry: 'docker.io'
          image: 'myorg/myapp'
        - registry: 'ghcr.io'
          image: 'myorg/myapp'
        - registry: '$CI_REGISTRY'
          image: '$CI_PROJECT_PATH'
```

### Kubernetes Progressive Deployment

```yaml
include:
  # Validate manifests
  - component: gitlab.com/bluefly/cicd-components/kubernetes/validate@main
    inputs:
      manifests: 'k8s/**/*.yaml'
      policies: 'policies/**/*.rego'
  
  # Deploy with Helm
  - component: gitlab.com/bluefly/cicd-components/kubernetes/helm@main
    inputs:
      chart: './helm/myapp'
      release: 'myapp-$CI_ENVIRONMENT_SLUG'
      values_files: ['values.yaml', 'values.$CI_ENVIRONMENT_SLUG.yaml']
  
  # Progressive rollout
  - component: gitlab.com/bluefly/cicd-components/kubernetes/rollout@main
    inputs:
      strategy: 'canary'
      steps: [10, 25, 50, 100]
      analysis:
        metrics: ['error_rate', 'latency_p99']
        thresholds:
          error_rate: 0.01
          latency_p99: 500
```

## Input Types Reference

### String Inputs
```yaml
spec:
  inputs:
    version:
      type: string
      default: '1.0.0'
      description: 'Version number'
      pattern: '^\d+\.\d+\.\d+$'  # Regex validation
```

### Boolean Inputs
```yaml
spec:
  inputs:
    enable_cache:
      type: boolean
      default: true
      description: 'Enable caching'
```

### Array Inputs
```yaml
spec:
  inputs:
    commands:
      type: array
      default: ['build', 'test']
      description: 'Commands to run'
```

### Object Inputs
```yaml
spec:
  inputs:
    thresholds:
      type: object
      default:
        cpu: 80
        memory: 90
      description: 'Resource thresholds'
```

### Enum Inputs
```yaml
spec:
  inputs:
    environment:
      type: string
      default: 'production'
      options: ['development', 'staging', 'production']
      description: 'Deployment environment'
```

### Secret Inputs
```yaml
spec:
  inputs:
    api_key:
      type: string
      secret: true
      required: true
      description: 'API key (stored securely)'
```

## Advanced Features

### Component Inheritance
```yaml
include:
  - component: gitlab.com/bluefly/cicd-components/nodejs/build@main
    inputs:
      node_version: '20'

# Extend the build job
build:prod:
  extends: .nodejs:build
  variables:
    NODE_ENV: production
  only:
    - tags
```

### Conditional Components
```yaml
include:
  - component: gitlab.com/bluefly/cicd-components/drupal/deploy@main
    inputs:
      environment: $CI_COMMIT_BRANCH
    rules:
      - if: '$CI_COMMIT_BRANCH == "main"'
        inputs:
          backup_database: true
      - if: '$CI_COMMIT_BRANCH =~ /^feature/'
        inputs:
          backup_database: false
```

### Component Composition
```yaml
include:
  # Base setup
  - component: gitlab.com/bluefly/cicd-components/shared/setup@main
  
  # Language-specific components
  - component: gitlab.com/bluefly/cicd-components/nodejs/build@main
  - component: gitlab.com/bluefly/cicd-components/python/build@main
  
  # Shared deployment
  - component: gitlab.com/bluefly/cicd-components/kubernetes/deploy@main

# Components work together seamlessly
```

## Best Practices

1. **Use Semantic Versioning**: Tag components with versions
   ```yaml
   - component: gitlab.com/bluefly/cicd-components/drupal/build@v1.2.3
   ```

2. **Override Defaults**: Customize for your needs
   ```yaml
   inputs:
     php_version: '8.2'  # Override default 8.3
   ```

3. **Combine Components**: Build complex pipelines
   ```yaml
   include:
     - component: .../build@main
     - component: .../test@main
     - component: .../deploy@main
   ```

4. **Use Caching**: Enable for faster builds
   ```yaml
   inputs:
     enable_cache: true
   ```

5. **Set Thresholds**: Enforce quality standards
   ```yaml
   inputs:
     coverage_threshold:
       statements: 90
   ```

## Contributing

See [CONTRIBUTING.md](./CONTRIBUTING.md) for guidelines on:
- Adding new components
- Testing components
- Documentation standards
- Release process

## Support

- 📚 [Documentation](https://bluefly.gitlab.io/cicd-components)
- 🐛 [Issue Tracker](https://gitlab.com/bluefly/cicd-components/issues)
- 💬 [Discussions](https://gitlab.com/bluefly/cicd-components/discussions)
- 📧 [Email Support](mailto:cicd-support@bluefly.io)