# Golden CI/CD Components

The "golden" components pattern is a key strategy for standardizing CI/CD across different project types. These comprehensive components serve as the entry point for CI/CD configuration, abstracting away complexity and ensuring consistency across the platform.

## What Are Golden Components?

Golden components are top-level, comprehensive CI/CD templates that:

1. **Aggregate multiple components** into a cohesive pipeline
2. **Standardize configuration** across similar project types
3. **Enforce best practices** for testing, security, and deployment
4. **Reduce boilerplate** in project-level CI configuration
5. **Centralize updates** for multiple projects

Rather than having each project include multiple individual components and configure them separately, projects can include a single golden component that handles the orchestration of all necessary CI/CD processes.

## Available Golden Components

### `drupal_golden_ci_module`

Comprehensive CI/CD for Drupal modules with:
- Complete testing suite (unit, kernel, functional)
- Code quality and security scanning
- Documentation generation
- Performance and accessibility testing
- Packagist publishing support

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      php_version: "8.2"
      drupal_version_constraint: "^10"
      module_name: "my_module"
```

### `drupal_golden_ci_recipe`

Comprehensive CI/CD for Drupal recipes with:
- Recipe validation and testing
- Dependency resolution verification
- Installation testing
- Integration testing
- Documentation generation

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_recipe@main
    inputs:
      php_version: "8.2"
      drupal_version_constraint: "^10"
      recipe_name: "my_recipe"
```

### `drupal_golden_ci_site`

Comprehensive CI/CD for complete Drupal sites with:
- Multi-environment deployment
- Comprehensive testing
- Performance monitoring
- Database migrations
- Content validation

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_site@main
    inputs:
      php_version: "8.2"
      drupal_version_constraint: "^10"
      site_name: "my_site"
      environments: ["dev", "stage", "prod"]
```

### `npm_golden_ci_package`

Comprehensive CI/CD for Node.js/NPM packages with:
- Complete testing suite
- Type checking
- Bundle analysis
- Security scanning
- Performance benchmarking
- NPM publishing

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
    inputs:
      node_version: "20"
      npm_token: "$NPM_TOKEN"
      min_test_coverage: 90
```

## Benefits of Using Golden Components

### 1. Standardization

All projects of a certain type (Drupal module, NPM package, etc.) use the exact same pipeline configuration, ensuring consistency in:
- Testing standards
- Code quality requirements
- Security practices
- Release processes

### 2. Reduced Configuration

Projects only need to include a single component with minimal configuration rather than multiple components with redundant settings.

**Before (without golden components):**
```yaml
# .gitlab-ci.yml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal-prepare@main
    inputs:
      php_version: "8.2"
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal-lint@main
    inputs:
      php_version: "8.2"
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal-test-unit@main
    inputs:
      php_version: "8.2"
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal-test-kernel@main
    inputs:
      php_version: "8.2"
  # ... many more components
```

**After (with golden components):**
```yaml
# .gitlab-ci.yml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      php_version: "8.2"
      module_name: "my_module"
```

### 3. Central Updates

When best practices or tooling changes, updating the golden component updates all projects that use it. This means:
- Security patches are applied consistently
- New testing standards are adopted automatically
- Performance improvements are distributed to all projects

### 4. Customization with Sensible Defaults

Golden components provide sensible defaults while still allowing customization through their input parameters. Projects can:
- Enable/disable specific features
- Configure thresholds and standards
- Adapt to project-specific requirements

### 5. Enforced Best Practices

Golden components enforce best practices by default, such as:
- Minimum test coverage requirements
- Security scanning
- Code quality checks
- Documentation generation

## Implementation Details

Golden components achieve their functionality by including multiple specialized components and configuring them appropriately. For example, `drupal_golden_ci_module` includes:

```yaml
include:
  - component: $CI_PROJECT_PATH/drupal/drupal-prepare@$CI_COMMIT_SHA
    inputs:
      php_version: $[[ inputs.php_version ]]
      # ...
  - component: $CI_PROJECT_PATH/drupal/drupal-lint@$CI_COMMIT_SHA
    inputs:
      php_version: $[[ inputs.php_version ]]
      # ...
  # ... other components
```

This composition allows golden components to:
1. Pass configuration from their inputs to the included components
2. Add additional jobs beyond what the included components provide
3. Coordinate between components to ensure a cohesive pipeline

## Adopting Golden Components

To adopt golden components in your project:

1. Identify the appropriate golden component for your project type
2. Include it in your `.gitlab-ci.yml` with the minimal required configuration
3. Remove any redundant component includes or custom job configurations

Example for a Drupal module:
```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      php_version: "8.2"
      drupal_version_constraint: "^10"
      module_name: "my_module"
      min_test_coverage: 90
```

## Extending Golden Components

While golden components provide comprehensive CI/CD pipelines, you can still extend them for project-specific needs:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/drupal/drupal_golden_ci_module@main
    inputs:
      php_version: "8.2"
      module_name: "my_module"

# Custom job for project-specific needs
custom_deployment:
  stage: deploy
  script:
    - echo "Deploying to custom environment"
  only:
    - main
```

## Conclusion

Golden components represent the recommended way to implement CI/CD across your projects. They encapsulate best practices, reduce configuration complexity, and ensure consistency. By standardizing on these components, you can focus on building features rather than maintaining CI/CD configurations.