# BFCIComponents Maintenance Guide

This document provides guidance for maintaining and evolving the BFCIComponents project and its relationship with BFCommon.

## Project Responsibilities

### BFCIComponents

- **Responsibility**: CI / CD standardization and configuration
- **Key Components**:
  - GitLab CI / CD component templates
  - CI pipeline scripts and utilities
  - Docker image configuration
  - Migration and validation tools

### BFCommon

- **Responsibility**: Shared utility code and libraries
- **Key Components**:
  - Logging infrastructure
  - Error handling framework
  - API client utilities
  - Configuration management
  - File utilities
  - TypeScript type definitions

## Maintaining Clear Separation

To maintain a clean separation of concerns between these projects:

1. **Code vs. Configuration**:

   - Utility code belongs in BFCommon
   - CI / CD configuration belongs in BFCIComponents
   - Never mix the two responsibilities

2. **Dependency Management**:

   - BFCIComponents should not depend on BFCommon code
   - Keep CI components self - contained and focused on CI / CD concerns
   - The migration scripts should handle the transition from one to the other

3. **Documentation**:

   - Clearly document which project should be used for which purpose
   - Keep CI / CD documentation in BFCIComponents
   - Keep utility code documentation in BFCommon

## Ongoing Maintenance

### BFCIComponents

1. **Adding New Components**:

   - Follow the established component structure
   - Always include proper input specifications
   - Support the custom Docker image
   - Add comprehensive documentation

2. **Updating Existing Components**:

   - Maintain backward compatibility when possible
   - Increment the component version
   - Update documentation

3. **Handling Custom Docker Image**:

   - Always test with the custom Docker image
   - Keep the image path configurable
   - Ensure all components support the custom image

### Relationship Management

1. **Project Coordination**:

   - When making changes to either project, consider the impact on the other
   - Coordinate releases when changes affect both projects
   - Keep migration scripts updated as projects evolve

2. **Version Compatibility**:

   - Document which versions of BFCommon and BFCIComponents are compatible
   - Tag releases in a coordinated manner

## Future Considerations

1. **Further Modularization**:

   - Consider breaking BFCommon into smaller, more focused packages
   - Potential modules: logging, error - handling, api - client, config

2. **GitLab Component Registry**:

   - Move toward publishing components to GitLab's Component Registry
   - This would simplify version management and distribution

3. **Docker Image Management**:

   - Consider moving Docker image configuration to its own project
   - This would allow for more focused management of container configurations

## Maintenance Schedule

1. **Regular Reviews**:

   - Quarterly review of components to ensure they meet current needs
   - Check for deprecated practices or outdated configurations

2. **Dependency Updates**:

   - Regular updates of dependencies in both projects
   - Security scanning and updates

3. **Documentation Updates**:

   - Update documentation as new best practices emerge
   - Keep migration guides current

## Component Governance (Updated January 2025)

### Component Lifecycle Management

Following the comprehensive audit completed in January 2025, new governance guidelines are established:

#### 1. Component Creation Standards
- **Catalog Registration**: All new components MUST be registered in `catalog-manifest.yml`
- **Input Specification**: Components MUST include complete `spec:` sections with inputs/outputs
- **Documentation**: Components MUST include usage examples and documentation
- **Testing**: Components MUST include validation tests before acceptance

#### 2. Architecture Compliance
- **Single Pattern**: Follow the standardized architecture pattern (components-first approach)
- **No Duplicates**: Components cannot exist in multiple locations simultaneously
- **Version Management**: Use semantic versioning for all component updates
- **Deprecation Process**: 3-month notice period before removing deprecated components

#### 3. Quality Gates
Before any component is accepted:
- [ ] OpenAPI-style input specification complete
- [ ] Security audit passed (no hardcoded secrets)
- [ ] Documentation includes usage examples
- [ ] Component registered in catalog manifest
- [ ] Integration tests created and passing

#### 4. Maintenance Responsibilities

**Component Owners Must**:
- Maintain backward compatibility during minor/patch updates
- Provide migration documentation for breaking changes
- Respond to security issues within 48 hours
- Update documentation with any behavioral changes

**Platform Team Must**:
- Review all component submissions within 5 business days
- Maintain catalog manifest accuracy
- Enforce architecture standards
- Provide component validation tools

### Component Review Process

1. **Submission**: Component submitted via merge request
2. **Automated Validation**: CI pipeline validates component structure
3. **Security Review**: Automated security scanning for hardcoded secrets
4. **Architecture Review**: Manual review for architecture compliance
5. **Documentation Review**: Ensure complete documentation
6. **Integration Testing**: Test component with sample projects
7. **Approval**: Platform team approval required for merge

### Emergency Procedures

#### Security Issues
1. Immediately create security issue in GitLab
2. Patch component within 24 hours
3. Notify all known users of the component
4. Update security documentation

#### Breaking Changes
1. Create deprecation notice 3 months in advance
2. Provide migration documentation and tools
3. Support both old and new versions during transition
4. Remove deprecated version only after transition period

## Guidelines for Project Owners

Project owners looking to use these repositories should:

1. Use BFCommon for shared utility code
2. Use BFCIComponents for CI / CD configuration
3. Always reference components by specific version
4. Follow the migration guides when transitioning between versions
5. Monitor component updates and security notices
6. Participate in component standardization process

### Compliance with Platform Standards
All components must align with the Bluefly AI Platform architectural principles:
- Drupal 11 as the central control plane where applicable
- OpenAPI standardization for all interfaces
- Event-driven automation with ECA integration
- GitLab-native CI/CD lifecycle management
- Comprehensive observability and monitoring
