# Reusable GitLab CI Components

This directory contains reusable GitLab CI/CD components that can be included in any project to standardize and simplify pipeline configurations.

##  Quick Start

### Using the Update Script (Recommended)

```bash
# For API services (llm-gateway, bfllm, llm-mcp)
./scripts/update-gitlab-ci.sh --type api --path /path/to/your/api/project

# For libraries (bfui, llm-gatewayclient, llmcli)
./scripts/update-gitlab-ci.sh --type library --path /path/to/your/library/project

# For documentation projects (bfdocs)
./scripts/update-gitlab-ci.sh --type docs --path /path/to/your/docs/project
```

### Manual Integration

Add this to your project's `.gitlab-ci.yml`:

```yaml
include:
  - project: 'llm/bfcicomponents'
    file: 'components/api-service.yml'  # or library.yml, docs.yml
```

## 📦 Available Components

### 1. `common-config.yml`
**Base configuration included by all other components**

Features:
- Docker rate limit mitigation with retry logic
- Optimized npm configuration
- Standard cache templates
- Common rule sets for branches and tags
- Global variables and environment setup

### 2. `api-service.yml`
**For API services like llm-gateway, bfllm, llm-mcp**

Pipeline stages:
- `setup` - Install dependencies with enhanced error handling
- `quality` - Linting and TypeScript type checking
- `build` - Compile TypeScript with robust error handling
- `test` - Run tests with coverage reporting
- `security` - Security audit scanning
- `package` - Create deployment packages

Features:
- OpenAPI validation
- Test coverage reporting
- Security scanning
- Artifact management
- Graceful failure handling

### 3. `library.yml`
**For libraries like bfui, llm-gatewayclient, llmcli**

Pipeline stages:
- `setup` - Install dependencies
- `quality` - Code quality checks
- `build` - Build library with type declarations
- `test` - Unit and integration tests
- `publish` - Semantic release and npm publishing

Features:
- Type declaration generation
- Integration test support
- Semantic versioning
- npm publishing
- Package distribution

### 4. `nodejs-enhanced.yml`
**Flexible Node.js component with configurable options**

Configurable inputs:
- `node_version` - Node.js version (default: "20")
- `package_manager` - npm, yarn, or pnpm (default: "npm")
- `test_coverage` - Enable coverage reporting (default: true)
- `enable_linting` - Run ESLint checks (default: true)
- `enable_typecheck` - Run TypeScript checking (default: true)
- `allow_build_failure` - Allow build failures (default: false)

##  Features

### Docker Rate Limit Mitigation
All components include:
- Retry logic for Docker image pulls
- Optimized Docker driver configuration
- Fallback strategies for infrastructure failures

### Enhanced Error Handling
- Graceful failure modes
- Detailed error reporting
- Environment debugging information
- Continuation on non-critical failures

### Performance Optimizations
- Intelligent caching strategies
- Parallel job execution
- Artifact optimization
- Build time reduction

### Security
- npm audit integration
- Dependency vulnerability scanning
- Secret detection prevention
- Access control enforcement

##  Project-Specific Customization

### Adding Custom Variables

```yaml
include:
  - project: 'llm/bfcicomponents'
    file: 'components/api-service.yml'

variables:
  CUSTOM_API_URL: "https://api.example.com"
  SPECIAL_BUILD_FLAG: "true"
```

### Adding Custom Jobs

```yaml
include:
  - project: 'llm/bfcicomponents'
    file: 'components/library.yml'

# Custom integration test
e2e_test:
  stage: test
  image: node:20-alpine
  script:
    - npm run test:e2e
  rules:
    - if: $CI_COMMIT_BRANCH == "main"
```

### Overriding Default Behavior

```yaml
include:
  - project: 'llm/bfcicomponents'
    file: 'components/api-service.yml'

# Override the default build job
build:
  extends: .node_job_template
  stage: build
  script:
    - echo "Custom build process"
    - npm run custom-build
```

## 📋 Migration Guide

### From Custom CI to Reusable Components

1. **Backup** your current `.gitlab-ci.yml`
2. **Identify** your project type (api, library, docs)
3. **Run** the update script or manually include the component
4. **Test** the pipeline in a feature branch
5. **Customize** any project-specific requirements
6. **Deploy** to main branch

### Common Migration Patterns

**Before (Custom CI):**
```yaml
stages:
  - install
  - test
  - build

install:
  image: node:20-alpine
  script:
    - npm ci
  # ... lots of custom configuration
```

**After (Reusable Components):**
```yaml
include:
  - project: 'llm/bfcicomponents'
    file: 'components/api-service.yml'

# Only project-specific customizations needed
```

## 🐛 Troubleshooting

### Common Issues

1. **Docker Rate Limits**
   - Components include automatic retry logic
   - Contact DevOps for runner authentication setup

2. **Build Failures**
   - Check the enhanced error logs in build jobs
   - Verify package.json scripts are properly defined

3. **Test Failures**
   - Components allow test failures by default
   - Check coverage reports in artifacts

4. **Missing Dependencies**
   - Ensure package.json includes all required dependencies
   - Check for peer dependency warnings

### Getting Help

1. Check the [troubleshooting docs](../docs/TROUBLESHOOTING.md)
2. Review pipeline logs for detailed error information
3. Check component validation reports in artifacts
4. Create an issue in the bfcicomponents project

##  Monitoring and Metrics

### Pipeline Performance
- Build time tracking
- Test execution time
- Cache hit rates
- Artifact sizes

### Quality Metrics
- Test coverage trends
- Linting issue counts
- Security vulnerability reports
- Type checking results

### Usage Analytics
- Component adoption rates
- Pipeline success rates
- Most common customizations
- Performance improvements

## 🔄 Updates and Maintenance

### Component Updates
Components are automatically updated when bfcicomponents is updated. Projects using the components will inherit improvements without manual intervention.

### Versioning
- Components follow semantic versioning
- Breaking changes are communicated in advance
- Migration guides provided for major updates

### Contributing
See [CONTRIBUTING.md](../CONTRIBUTING.md) for guidelines on:
- Adding new components
- Improving existing components
- Testing component changes
- Documentation requirements

---

**Version**: 2.0.0  
**Last Updated**: January 2025  
**Maintained by**: Bluefly Platform Engineering Team