# Drupal Core API Replacements Guide

## Overview
This document maps custom implementations to their Drupal Core API equivalents, providing specific examples for the refactoring process.

## 1. Cache API Replacements

### Custom Cache Services → Core Cache API

**Current Custom Implementations**:
- `VectorCacheService`
- `OptimizedStatsService` 
- `ApiNormalizationStatsService`
- Custom Redis handlers

**Core Replacement**:
```php
// Define cache bins in *.services.yml
services:
  cache.ai_vectors:
    class: Drupal\Core\Cache\CacheBackendInterface
    tags:
      - { name: cache.bin }
    factory: cache_factory:get
    arguments: [ai_vectors]

// Usage
$cache = \Drupal::cache('ai_vectors');
$cache->set('vector:' . $id, $data, Cache::PERMANENT, ['ai_vectors', 'node:' . $nid]);

// Invalidate by tag
Cache::invalidateTags(['ai_vectors']);
```

### Cache Tags Strategy
```php
// Tag patterns for AI platform
'ai_model:' . $model_id
'ai_provider:' . $provider
'ai_query:' . $hash
'ai_vector:' . $vector_id
```

## 2. Entity API Replacements

### Custom Entity Services → Entity Type Manager

**Current**: `NativeEntityService`, custom entity handlers
**Replacement**:
```php
// OLD: Custom entity service
$entity = $this->nativeEntityService->loadEntity($type, $id);

// NEW: Entity Type Manager
$entity = \Drupal::entityTypeManager()
  ->getStorage($type)
  ->load($id);

// Batch operations
$entities = \Drupal::entityTypeManager()
  ->getStorage('node')
  ->loadByProperties(['type' => 'ai_content', 'status' => 1]);
```

### Entity Query API
```php
// Complex queries
$query = \Drupal::entityQuery('node')
  ->condition('type', 'ai_content')
  ->condition('field_ai_provider', 'openai')
  ->condition('created', strtotime('-1 week'), '>')
  ->sort('created', 'DESC')
  ->pager(10);

$nids = $query->execute();
```

## 3. Field API Replacements

### Custom Field Types → Core Field Types

**Current Custom Fields**:
- `LlmPromptItem`
- `SimplifiedLlmPromptItem`
- `AiModelReferenceItem`

**Core Replacements**:
```php
// Replace custom prompt field with text_long + field settings
public function fieldDefinition() {
  return BaseFieldDefinition::create('text_long')
    ->setLabel(t('AI Prompt'))
    ->setSettings([
      'ai_enhanced' => TRUE,
      'ai_provider' => 'openai',
      'max_tokens' => 1000,
    ])
    ->setDisplayOptions('form', [
      'type' => 'text_textarea',
      'settings' => ['rows' => 5],
    ]);
}
```

### Field Storage
```php
// Use core's field storage with third-party settings
$field_storage = FieldStorageConfig::create([
  'field_name' => 'field_ai_embeddings',
  'entity_type' => 'node',
  'type' => 'string_long',
  'settings' => [],
  'third_party_settings' => [
    'ai_platform' => [
      'vector_dimensions' => 1536,
      'index_type' => 'hnsw',
    ],
  ],
]);
```

## 4. REST API Replacements

### Custom REST Controllers → JSON:API

**Current**: 66 custom REST endpoints
**Replacement**: JSON:API with custom resources

```php
// Custom JSON:API resource
namespace Drupal\ai_platform\Resource;

use Drupal\jsonapi\Resource\JsonApiResource;

class AiQueryResource extends JsonApiResource {
  
  public function process(Request $request) {
    $prompt = $request->get('prompt');
    $model = $request->get('model', 'gpt-3.5-turbo');
    
    // Process with AI service
    $result = $this->aiService->query($prompt, ['model' => $model]);
    
    // Return JSON:API compliant response
    return new ResourceResponse([
      'data' => [
        'type' => 'ai_response',
        'id' => $result['id'],
        'attributes' => [
          'response' => $result['text'],
          'tokens' => $result['usage'],
          'model' => $model,
        ],
      ],
    ]);
  }
}
```

### REST Resource Plugin
```php
/**
 * @RestResource(
 *   id = "ai_embeddings",
 *   label = @Translation("AI Embeddings"),
 *   uri_paths = [*     "create" = "/api/ai/embeddings"
 *]
 * )
 */
class AiEmbeddingsResource extends ResourceBase {
  public function post(array $data) {
    // Validate
    if (empty($data['text'])) {
      throw new BadRequestHttpException('Text is required');
    }
    
    // Generate embeddings
    $embeddings = $this->embeddingService->generate($data['text']);
    
    return new ModifiedResourceResponse(['embeddings' => $embeddings], 201);
  }
}
```

## 5. Queue API Replacements

### Custom Queue Workers → Core Queue API

**Current**: Custom task queues, workers
**Replacement**:
```php
// Define queue worker
/**
 * @QueueWorker(
 *   id = "ai_content_generation",
 *   title = @Translation("AI Content Generation"),
 *   cron = ["time" = 60]
 * )
 */
class AiContentGenerationWorker extends QueueWorkerBase {
  
  public function processItem($data) {
    $prompt = $data['prompt'];
    $node = Node::load($data['node_id']);
    
    // Generate content
    $content = $this->aiService->generateContent($prompt);
    
    // Update node
    $node->set('body', $content);
    $node->save();
  }
}

// Queue items
$queue = \Drupal::queue('ai_content_generation');
$queue->createItem([
  'prompt' => 'Write about Drupal development',
  'node_id' => $node->id(),
]);
```

## 6. Plugin System Replacements

### Custom Provider System → Plugin API

**Current**: Hard-coded provider classes
**Replacement**: Plugin-based providers

```php
// Plugin manager
namespace Drupal\ai_platform\Plugin;

/**
 * Plugin manager for AI providers.
 *
 * @see \Drupal\ai_platform\Annotation\AiProvider
 * @see \Drupal\ai_platform\Plugin\AiProviderInterface
 */
class AiProviderManager extends DefaultPluginManager {
  
  public function __construct() {
    parent::__construct(
      'Plugin/AiProvider',
      $namespaces,
      $module_handler,
      'Drupal\ai_platform\Plugin\AiProviderInterface',
      'Drupal\ai_platform\Annotation\AiProvider'
    );
    
    $this->alterInfo('ai_provider_info');
    $this->setCacheBackend($cache_backend, 'ai_provider_plugins');
  }
}

// Plugin implementation
/**
 * @AiProvider(
 *   id = "openai",
 *   label = @Translation("OpenAI"),
 *   models = {
 *     "gpt-4" = @Translation("GPT-4"),
 *     "gpt-3.5-turbo" = @Translation("GPT-3.5 Turbo"),
 *   }
 * )
 */
class OpenAiProvider extends AiProviderBase {
  
  public function query($prompt, array $options = []) {
    $response = $this->httpClient->post($this->configuration['endpoint'], [
      'json' => [
        'model' => $options['model'] ?? 'gpt-3.5-turbo',
        'messages' => [['role' => 'user', 'content' => $prompt]],
      ],
      'headers' => [
        'Authorization' => 'Bearer ' . $this->configuration['api_key'],
      ],
    ]);
    
    return json_decode($response->getBody(), TRUE);
  }
}
```

## 7. Service Container Replacements

### Custom Service Discovery → Tagged Services

**Current**: `EnvironmentServiceDiscovery`, `ServiceRegistry`
**Replacement**:
```php
// Tagged service definition
services:
  ai_platform.provider.openai:
    class: Drupal\ai_platform\Provider\OpenAiProvider
    arguments: ['@http_client', '@config.factory']
    tags:
      - { name: ai_provider, priority: 100 }

  ai_platform.provider.anthropic:
    class: Drupal\ai_platform\Provider\AnthropicProvider
    arguments: ['@http_client', '@config.factory']
    tags:
      - { name: ai_provider, priority: 90 }

// Compiler pass to collect tagged services
class AiProviderCompilerPass implements CompilerPassInterface {
  
  public function process(ContainerBuilder $container) {
    if (!$container->has('ai_platform.provider_manager')) {
      return;
    }
    
    $definition = $container->findDefinition('ai_platform.provider_manager');
    $taggedServices = $container->findTaggedServiceIds('ai_provider');
    
    foreach ($taggedServices as $id => $tags) {
      $definition->addMethodCall('addProvider', [new Reference($id), $tags[0]['priority'] ?? 0]);
    }
  }
}
```

## 8. Event System Replacements

### Custom Hooks → Event Subscribers

**Current**: Custom integration services
**Replacement**:
```php
// Event subscriber
class AiContentSubscriber implements EventSubscriberInterface {
  
  public static function getSubscribedEvents() {
    return [
      EntityEvents::ENTITY_INSERT => 'onEntityInsert',
      EntityEvents::ENTITY_UPDATE => 'onEntityUpdate',
      'ai_platform.query_complete' => 'onQueryComplete',
    ];
  }
  
  public function onEntityInsert(EntityEvent $event) {
    $entity = $event->getEntity();
    if ($entity->getEntityTypeId() === 'node' && $entity->bundle() === 'ai_content') {
      // Generate embeddings
      $this->generateEmbeddings($entity);
    }
  }
  
  public function onQueryComplete(AiQueryEvent $event) {
    // Log query metrics
    $this->logger->info('AI query completed', [
      'model' => $event->getModel(),
      'tokens' => $event->getTokenCount(),
      'duration' => $event->getDuration(),
    ]);
  }
}
```

## 9. Configuration Management

### Custom Config → Configuration API

**Current**: Hard-coded settings, custom config tables
**Replacement**:
```php
// Config schema (config/schema/ai_platform.schema.yml)
ai_platform.settings:
  type: config_object
  label: 'AI Platform settings'
  mapping:
    providers:
      type: sequence
      sequence:
        type: mapping
        mapping:
          id:
            type: string
          api_key:
            type: string
          endpoint:
            type: uri
          models:
            type: sequence

// Config form
class AiPlatformSettingsForm extends ConfigFormBase {
  
  protected function getEditableConfigNames() {
    return ['ai_platform.settings'];
  }
  
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ai_platform.settings');
    
    $form['providers']['openai']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('OpenAI API Key'),
      '#default_value' => $config->get('providers.openai.api_key'),
    ];
    
    return parent::buildForm($form, $form_state);
  }
}
```

## 10. Logging & Monitoring

### Custom Logging → Core Logging

**Current**: Custom log handlers
**Replacement**:
```php
// Use core logger
\Drupal::logger('ai_platform')->notice('AI query processed', [
  'prompt' => $prompt,
  'model' => $model,
  'response_time' => $duration,
]);

// Custom log channel
services:
  logger.channel.ai_platform:
    parent: logger.channel_base
    arguments: ['ai_platform']

// Performance logging with watchdog
$start = microtime(TRUE);
$result = $this->aiService->query($prompt);
$duration = microtime(TRUE) - $start;

watchdog('ai_performance', 'Query took @time seconds', [
  '@time' => round($duration, 3),
], RfcLogLevel::INFO);
```

## Summary of Core APIs to Use

1. **Cache API** - Replace all custom caching
2. **Entity API** - Replace custom entity handling
3. **Field API** - Use core field types with settings
4. **JSON:API** - Replace custom REST endpoints
5. **Queue API** - Replace custom workers
6. **Plugin API** - Replace hard-coded providers
7. **Service Container** - Replace custom discovery
8. **Event API** - Replace custom hooks
9. **Configuration API** - Replace custom settings
10. **Logging API** - Replace custom loggers

## Migration Priority

1. **High Priority**: Cache API, Entity API (immediate performance gains)
2. **Medium Priority**: JSON:API, Plugin API (architectural improvements)
3. **Low Priority**: Logging, Configuration (nice-to-have standardization)