# Drupal Production Refactor Plan

## Overview
This plan outlines the systematic refactoring of the LLM platform from experimental code to production-ready Drupal standard code. The focus is on simplification, leveraging Drupal core APIs, and reducing custom code.

## Phase 1: Immediate Actions (Week 1-2)

### 1.1 Remove Custom Caching Implementations
**Modules Affected**: All

**Replace**:
- `VectorCacheService` → `cache.custom` bin
- `OptimizedStatsService` → `cache.data` with cache tags
- Custom Redis implementations → Redis module

**Implementation**:
```php
// OLD: Custom cache service
$this->vectorCache->set($key, $data);

// NEW: Core cache API
\Drupal::cache('custom')->set($key, $data, Cache::PERMANENT, ['ai_vector']);
```

### 1.2 Consolidate Service Files
**Module**: api_normalization

**Current**: 6 service files
- api_normalization.services.yml
- api_normalization.services.complex.yml
- api_normalization.services.native.yml
- api_normalization.services.revolutionary.yml
- api_normalization.neural.services.yml
- api_normalization.quantum.services.yml

**New**: Single api_normalization.services.yml

### 1.3 Install Required Contrib Modules
```bash
cd DEMO
composer require drupal/eca drupal/redis drupal/simple_oauth drupal/consumers
ddev drush en eca redis simple_oauth consumers -y
```

## Phase 2: Core API Migration (Week 3-4)

### 2.1 Replace Custom REST with JSON:API
**Module**: api_normalization

**Current Structure**:
- 66 custom REST endpoints
- Custom authentication
- Custom response formatting

**New Structure**:
- JSON:API resources
- OAuth2 authentication (simple_oauth)
- Standard JSON:API responses

**Example Migration**:
```php
// OLD: Custom REST controller
class ApiEndpointController extends ControllerBase {
  public function getContent($type, $id) {
    // Custom logic
    return new JsonResponse($data);
  }
}

// NEW: JSON:API resource
class AiContentResource extends JsonApiResource {
  // Automatic CRUD, filtering, includes
}
```

### 2.2 Convert Custom Entities to Config Entities
**Module**: llm, native_ai

**Current**: Custom entity handling
**New**: Config entities for AI models, providers

```yaml
# config/schema/llm.schema.yml
llm.ai_model.*:
  type: config_entity
  label: 'AI Model'
  mapping:
    id:
      type: string
    label:
      type: string
    provider:
      type: string
    api_key:
      type: string
    endpoint:
      type: string
```

### 2.3 Implement Plugin System for AI Providers
**Module**: native_ai

```php
// src/Plugin/AiProvider/OpenAiProvider.php
namespace Drupal\native_ai\Plugin\AiProvider;

/**
 * @AiProvider(
 *   id = "openai",
 *   label = @Translation("OpenAI"),
 *   description = @Translation("OpenAI GPT models")
 * )
 */
class OpenAiProvider extends AiProviderBase {
  public function query($prompt, array $options = []) {
    // Implementation
  }
}
```

## Phase 3: Module Consolidation (Week 5-6)

### 3.1 Merge LLM and Native AI Modules
**New Module**: ai_platform

**Features**:
- Unified AI provider management
- Single field type for AI-enhanced fields
- Consolidated routing (20 routes total)
- Plugin-based architecture

### 3.2 Simplify Field Types
**Current**: 5 custom field types
**New**: 1 configurable field type

```php
// NEW: Single AI field type with settings
class AiEnhancedTextItem extends TextLongItem {
  public static function defaultFieldSettings() {
    return [
      'ai_provider' => '',
      'ai_model' => '',
      'enhancement_type' => 'summary', // summary, expand, translate
    ] + parent::defaultFieldSettings();
  }
}
```

### 3.3 Simplify Alternative Services
**Current**: Custom service discovery
**New**: Tagged services

```yaml
# alternative_services.services.yml
services:
  alternative_services.langfuse:
    class: Drupal\alternative_services\Service\LangfuseService
    tags:
      - { name: ai_service, priority: 100 }
```

## Phase 4: NPM Package Consolidation (Week 7-8)

### 4.1 Consolidate to 3 Core Services

**1. ai-gateway** (merge llm-gateway + llm-mcp)
- Unified API gateway
- Service orchestration
- Authentication/authorization
- Rate limiting

**2. ai-client** (merge bfllm + bfrfp + llm-gatewayclient)
- Multi-provider LLM client
- Content generation
- Embeddings/vector operations
- Retry/resilience logic

**3. ai-ui** (simplify bfui)
- React components for AI features
- Drupal-specific integrations
- Theme-aware styling

### 4.2 Remove Redundant Packages
- **llmcli**: Move essential commands to Drush
- **bfdocs**: Use standard docs tools
- **secure_project**: Integrate security into main packages

## Phase 5: Testing & Documentation (Week 9-10)

### 5.1 PHPUnit Test Structure
```
tests/
├── src/
│   ├── Unit/           # Unit tests
│   ├── Kernel/         # Kernel tests
│   └── Functional/     # Functional tests
└── modules/
    └── ai_platform/
        └── tests/
```

### 5.2 Testing Standards
- Minimum 80% code coverage
- All public APIs tested
- Integration tests for AI providers
- Performance benchmarks

### 5.3 Documentation
- README.md per module
- UPGRADE.md for migration
- API documentation
- User guides

## Phase 6: Production Configuration (Week 11-12)

### 6.1 Environment Configuration
```yaml
# config/prod/system.performance.yml
cache:
  page:
    max_age: 86400
preprocess:
  css: true
  js: true
```

### 6.2 Security Hardening
- Implement CSP headers
- API rate limiting
- Input validation
- SQL injection prevention

### 6.3 Performance Optimization
- Redis caching
- CDN configuration
- Image optimization
- Lazy loading

## Migration Checklist

### Per Module
- [ ] Analyze current implementation
- [ ] Identify core/contrib replacements
- [ ] Write migration plan
- [ ] Implement changes
- [ ] Write tests
- [ ] Update documentation
- [ ] Performance testing
- [ ] Security review

### Overall Platform
- [ ] Consolidate modules
- [ ] Reduce NPM packages
- [ ] Implement caching strategy
- [ ] Add monitoring/logging
- [ ] Create deployment scripts
- [ ] Load testing
- [ ] Security audit
- [ ] Documentation review

## Success Metrics

### Code Quality
- **Cyclomatic Complexity**: < 10 per method
- **Code Coverage**: > 80%
- **Drupal Coding Standards**: 100% compliance
- **Dependencies**: Reduce by 60%

### Performance
- **Page Load**: < 2s
- **API Response**: < 200ms
- **Memory Usage**: < 128MB
- **Cache Hit Rate**: > 90%

### Maintenance
- **Module Count**: From 6 to 3
- **NPM Packages**: From 11 to 3
- **Custom Code**: Reduce by 60%
- **Documentation**: 100% coverage

## Timeline Summary

**Weeks 1-2**: Immediate fixes (caching, service consolidation)
**Weeks 3-4**: Core API migration
**Weeks 5-6**: Module consolidation
**Weeks 7-8**: NPM package consolidation
**Weeks 9-10**: Testing and documentation
**Weeks 11-12**: Production configuration

**Total Duration**: 12 weeks
**Team Size**: 2-3 developers
**Effort**: ~360 hours

## Risk Mitigation

1. **Data Migration**: Create migration scripts for existing content
2. **Breaking Changes**: Maintain compatibility layer during transition
3. **Performance**: Benchmark before/after each phase
4. **Testing**: Automated tests for all critical paths
5. **Rollback**: Version control and backup strategies

## Next Steps

1. Get stakeholder approval
2. Set up development environment
3. Create feature branches
4. Begin Phase 1 implementation
5. Weekly progress reviews