# GitLab CI Documentation Components

This directory contains reusable GitLab CI components for documentation deployment and aggregation across the Bluefly AI Platform.

## Available Components

### 1. `drupal-tdd.yml` - Test-Driven Development for Drupal Modules

Enhanced Drupal module testing component with comprehensive TDD support and flexible enforcement levels.

**Features:**
- Progressive TDD adoption (strict, progressive, optional, disabled modes)
- Comprehensive test coverage (unit, kernel, functional, security)
- Quality gates and coverage analysis
- Mutation testing for test quality validation
- Automated test scaffolding and recommendations
- Security vulnerability testing
- Performance testing capabilities

**Usage:**
```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-tdd@main
    inputs:
      tdd_mode: "progressive"
      min_test_coverage: "70"
      test_frameworks: "phpunit,behat"
      enable_mutation_testing: "false"
      modules_path: "web/modules/custom"
```

**TDD Modes:**
- **strict**: Full TDD enforcement, all tests required
- **progressive**: Encourages TDD without blocking deployments
- **optional**: All TDD jobs are manual
- **disabled**: Falls back to basic drupal-modules functionality

**Input Parameters:**
- `tdd_mode`: TDD enforcement level (strict, progressive, optional, disabled)
- `min_test_coverage`: Minimum required test coverage percentage
- `test_frameworks`: Comma-separated list of test frameworks
- `enable_mutation_testing`: Enable mutation testing for test quality
- `test_data_fixtures`: Generate test data fixtures automatically
- `performance_testing`: Enable performance testing
- `php_version`: PHP version (default: "8.3")
- `drupal_version`: Target Drupal version (default: "11.x")

### 2. `docs-pages.yml` - GitLab Pages Documentation Deployment

Universal documentation deployment component supporting multiple documentation engines.

**Supported Engines:**
- MkDocs (Material theme)
- Sphinx (with RTD theme)
- Hugo (static site generator)
- Static HTML files

**Features:**
- Automatic GitLab Pages deployment
- Custom domain support
- Documentation quality checks
- Performance optimization
- SEO enhancements (sitemap, robots.txt)

**Usage:**
```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/docs-pages@main
    inputs:
      docs_engine: "mkdocs"
      docs_source: "docs"
      custom_domain: "docs.example.com"
      aggregate_to_bfdocs: true
      bfdocs_project_path: "projects/my-project"
```

**Input Parameters:**
- `docs_engine`: Documentation engine (mkdocs, sphinx, hugo, static)
- `docs_source`: Source directory for documentation (default: "docs")
- `aggregate_to_bfdocs`: Whether to aggregate to central bfdocs
- `bfdocs_project_path`: Path in bfdocs for aggregation
- `custom_domain`: Custom domain for GitLab Pages
- `node_version`: Node.js version (default: "20")
- `python_version`: Python version (default: "3.11")

### 2. `drupal-docs.yml` - Drupal Module Documentation

Specialized component for Drupal custom modules with automatic API documentation generation.

**Features:**
- Drupal module structure analysis
- OpenAPI documentation generation
- Module metadata extraction
- Automatic MkDocs configuration
- Integration with bfdocs aggregation

**Usage:**
```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-docs@main
    inputs:
      module_name: "api_normalization"
      module_type: "custom"
      drupal_version: "11"
      generate_api_docs: true
      bfdocs_aggregation: true
```

**Input Parameters:**
- `module_name`: Name of the Drupal module (required)
- `module_type`: Type of module (custom, contrib, core)
- `drupal_version`: Target Drupal version (default: "11")
- `generate_api_docs`: Generate API docs from OpenAPI specs
- `bfdocs_aggregation`: Aggregate to central bfdocs

### 3. `docs-aggregator.yml` - Central Documentation Aggregation

Aggregates documentation from multiple GitLab projects into a central repository.

**Features:**
- Scheduled documentation aggregation
- Multi-project source support
- Automatic navigation generation
- GitLab API integration
- MkDocs configuration updates

**Usage:**
```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/docs-aggregator@main
    inputs:
      aggregation_sources:
        - "llm/llm-gateway"
        - "llm/bfllm"
        - "llm/demo_llm_drupal"
      target_branch: "main"
      cleanup_old_docs: true
```

**Input Parameters:**
- `aggregation_sources`: List of GitLab projects to aggregate from
- `aggregation_schedule`: Cron schedule for automatic aggregation
- `target_branch`: Target branch for aggregated docs
- `cleanup_old_docs`: Remove outdated documentation

## Implementation Guide

### For Individual Projects

Add GitLab CI configuration to enable documentation deployment:

```yaml
# .gitlab-ci.yml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/docs-pages@main
    inputs:
      docs_engine: "mkdocs"
      aggregate_to_bfdocs: true
      bfdocs_project_path: "projects/my-project"

variables:
  PAGES_DOMAIN: "docs.bluefly.io"
```

### For Drupal Modules

Each custom module gets its own GitLab CI configuration:

```yaml
# web/modules/custom/my_module/.gitlab-ci.yml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-docs@main
    inputs:
      module_name: "my_module"
      drupal_version: "11"
      generate_api_docs: true

variables:
  MODULE_NAME: "my_module"
  BFDOCS_PROJECT_ID: "llm/bfdocs"
```

### For Drupal Projects with TDD

Enable comprehensive Test-Driven Development for Drupal projects:

```yaml
# .gitlab-ci.yml (project root)
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-tdd@main
    inputs:
      tdd_mode: "progressive"        # Start with progressive mode
      min_test_coverage: "70"
      test_frameworks: "phpunit,behat"
      modules_path: "web/modules/custom"

variables:
  # Optional: Customize TDD behavior
  TDD_REPORT_SLACK_WEBHOOK: $SLACK_WEBHOOK_URL
```

**Migration Strategy for Existing Projects:**

1. **Start with Optional Mode** (no impact on existing pipeline):
```yaml
inputs:
  tdd_mode: "optional"              # All TDD jobs are manual
  min_test_coverage: "30"
```

2. **Progress to Progressive Mode** (warnings, no failures):
```yaml
inputs:
  tdd_mode: "progressive"           # Encourages TDD adoption
  min_test_coverage: "60"
```

3. **Adopt Strict Mode** (full TDD enforcement):
```yaml
inputs:
  tdd_mode: "strict"                # Requires all tests to pass
  min_test_coverage: "80"
  enable_mutation_testing: "true"
```

### For Central Documentation (bfdocs)

The central repository aggregates all documentation:

```yaml
# bfdocs/.gitlab-ci.yml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/docs-aggregator@main
    inputs:
      aggregation_sources:
        - "llm/llm-gateway"
        - "llm/bfllm"
        - "llm/llm-mcp"
        - "llm/demo_llm_drupal"
      
  - component: gitlab.bluefly.io/llm/bfcicomponents/docs-pages@main
    inputs:
      docs_engine: "mkdocs"
      custom_domain: "docs.bluefly.io"
```

## Required GitLab Variables

### Project-Level Variables

Set these in your GitLab project settings:

```bash
# Optional: For Slack notifications
SLACK_WEBHOOK_URL=https://hooks.slack.com/...

# Optional: For custom analytics
GOOGLE_ANALYTICS_KEY=GA-123456789

# For bfdocs aggregation (set in aggregating projects)
BFDOCS_TRIGGER_TOKEN=glptt-xxxxx...
BFDOCS_PROJECT_ID=123456
```

### Group-Level Variables

Set these at the group level for all projects:

```bash
# GitLab Pages domain
PAGES_DOMAIN=docs.bluefly.io

# Default documentation settings
DOCS_ENGINE=mkdocs
SITE_URL=https://docs.bluefly.io
```

## Directory Structure

Each project should organize documentation as follows:

```
project/
├── .gitlab-ci.yml           # Uses docs-pages component
├── docs/                    # Documentation source
│   ├── index.md            # Main documentation
│   ├── api.md              # API documentation
│   └── guides/             # User guides
├── mkdocs.yml              # MkDocs configuration (auto-generated)
└── openapi.yml             # OpenAPI specification (optional)
```

For Drupal modules:

```
web/modules/custom/my_module/
├── .gitlab-ci.yml          # Uses drupal-docs component
├── README.md               # Main module documentation
├── ARCHITECTURE.md         # Architecture documentation
├── docs/                   # Additional documentation
├── openapi.yml             # API specification
└── my_module.info.yml      # Drupal module info
```

## Deployment Workflow

1. **Individual Project Deployment:**
   - Developer commits documentation changes
   - GitLab CI builds and deploys to project-specific Pages
   - Documentation is automatically aggregated to bfdocs

2. **Central Documentation Deployment:**
   - bfdocs aggregates documentation from all projects
   - Builds comprehensive documentation site
   - Deploys to main documentation domain

3. **Scheduled Synchronization:**
   - Automated sync every 6 hours
   - Pulls latest documentation from all projects
   - Updates navigation and search index

## Benefits

- **Decentralized Authoring**: Each team maintains their own documentation
- **Centralized Access**: All documentation accessible from single domain
- **Automatic Synchronization**: No manual aggregation required
- **Consistent Structure**: Standardized documentation format
- **Quality Assurance**: Automated link checking and validation
- **Performance Optimized**: Fast loading with caching and compression

## Troubleshooting

### Common Issues

1. **Component Not Found**
   ```
   Error: Component gitlab.bluefly.io/llm/bfcicomponents/docs-pages@main not found
   ```
   - Ensure the component repository is accessible
   - Check GitLab permissions and authentication

2. **MkDocs Build Fails**
   ```
   Error: Config file 'mkdocs.yml' does not exist
   ```
   - Verify mkdocs.yml exists or allow auto-generation
   - Check documentation source directory path

3. **Aggregation Fails**
   ```
   Error: Could not fetch project info
   ```
   - Verify BFDOCS_TRIGGER_TOKEN is configured
   - Check project permissions and API access

### Debug Mode

Enable verbose logging by adding to your `.gitlab-ci.yml`:

```yaml
variables:
  CI_DEBUG_TRACE: "true"
  DOCS_DEBUG: "true"
```

## Support

- **Component Issues**: Create issue in `llm/bfcicomponents`
- **Documentation**: See [bfdocs](https://docs.bluefly.io)
- **GitLab CI**: [GitLab CI Documentation](https://docs.gitlab.bluefly..io/ee/ci/)

---

*These components are part of the Bluefly AI Platform documentation infrastructure, enabling distributed documentation authoring with centralized aggregation and deployment.*