# GitLab CI Components Guide for bluefly.io Platform

This guide provides comprehensive documentation on using the standardized GitLab CI / CD components developed for the bluefly.io LLM Platform. These components are designed to ensure consistency, reusability, and maintainability across all projects in the ecosystem.

## Table of Contents

1. [Introduction](#introduction)
2. [Component Architecture](#component - architecture)
3. [Base Configuration](#base - configuration)
4. [Available Components](#available - components)
    - [Caching](#caching)
    - [Testing](#testing)
    - [Linting](#linting)
    - [Security Scanning](#security - scanning)
5. [Project Type Templates](#project - type - templates)
6. [Usage Examples](#usage - examples)
7. [Best Practices](#best - practices)
8. [Troubleshooting](#troubleshooting)

## Introduction

The bluefly.io CI / CD framework provides a set of standardized, reusable components that enable consistent validation, testing, and deployment across all projects. These components follow GitLab CI Components best practices and are designed to work together seamlessly.

Key benefits include:

- **Consistency**: Ensure all projects follow the same patterns and standards
- **Reusability**: Avoid duplicate code across CI configurations
- **Maintainability**: Centralized updates to CI logic
- **Performance**: Optimized for speed and resource efficiency
- **Comprehensive**: Cover all aspects of the development lifecycle

## Component Architecture

The CI components follow a layered approach:

1. **Base Configuration** (`templates / enhanced - base.yml`): Provides foundational CI / CD configuration used by all projects
2. **Component Modules** (`components/ * .yml`): Specific functional modules for common CI tasks
3. **Project Type Templates** (`templates / enhanced - *.yml`): Type - specific configurations for different project types

Components use standard GitLab CI inputs and specs for easy integration and documentation.

## Base Configuration

The enhanced base configuration (`templates / enhanced - base.yml`) provides:

- Stage definitions
- Performance optimizations
- Caching strategies
- Notification systems
- Pipeline metrics
- Security configurations

To include the base configuration:

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / templates / enhanced - base.yml'
```text

## Available Components

### Caching
The caching component (`components / cache.yml`) provides optimized caching strategies for different project types.

**Inputs:**
- `project_type`: Type of project (nodejs, python, php, drupal)
- `cache_key`: Key used for cache identification
- `custom_paths`: Additional custom paths to cache
- `cache_fallback_key`: Fallback key used when main key is not found
- `pull_policy`: When to pull the cache (pull, pull - push)
- `when`: When to save cache (on_success, on_failure, always)
- `max_age`: Cache expiration time

**Example Usage:**

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / components / cache.yml'

variables:
  project_type: nodejs
  custom_paths:
    - .next / cache
```text

### Testing
The testing component (`components / testing.yml`) provides standardized test execution with coverage reporting.

**Inputs:**
- `project_type`: Type of project (nodejs, python, php, drupal)
- `coverage_enabled`: Whether to collect code coverage
- `coverage_threshold`: Minimum code coverage percentage required
- `parallel_tests`: Whether to run tests in parallel
- `parallel_jobs`: Number of parallel jobs if enabled
- `test_command`: Custom test command to override default
- `reports_path`: Directory for test reports
- `artifacts_expire_in`: Expiration time for artifacts
- `test_stage`: CI stage for testing

**Example Usage:**

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / components / testing.yml'

variables:
  project_type: nodejs
  coverage_threshold: 80
  parallel_tests: true
  parallel_jobs: 4
```text

### Linting
The linting component (`components / linting.yml`) provides code quality checks and linting.

**Inputs:**
- `project_type`: Type of project (nodejs, python, php, drupal)
- `fix_issues`: Whether to automatically fix linting issues
- `fail_on_error`: Whether to fail pipeline on linting errors
- `custom_config_path`: Path to custom linting configuration
- `lint_command`: Custom lint command to override default
- `reports_path`: Directory for lint reports
- `artifacts_expire_in`: Expiration time for artifacts
- `lint_stage`: CI stage for linting

**Example Usage:**

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / components / linting.yml'

variables:
  project_type: nodejs
  fix_issues: false
  fail_on_error: true
  custom_config_path: .eslintrc.json
```text

### Security Scanning
The security component (`components / security.yml`) provides vulnerability scanning and security checks.

**Inputs:**
- `project_type`: Type of project (nodejs, python, php, drupal)
- `scan_dependencies`: Whether to scan dependencies for vulnerabilities
- `scan_sast`: Whether to perform static application security testing
- `scan_secrets`: Whether to scan for secrets in codebase
- `scan_container`: Whether to scan container images
- `fail_on_vulnerabilities`: Whether to fail pipeline on vulnerabilities
- `security_level`: Security level (high, medium, low)
- `custom_security_command`: Custom security command to override default
- `reports_path`: Directory for security reports
- `artifacts_expire_in`: Expiration time for artifacts
- `security_stage`: CI stage for security scanning

**Example Usage:**

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / components / security.yml'

variables:
  project_type: nodejs
  scan_dependencies: true
  scan_sast: true
  scan_secrets: true
  security_level: medium
```text

## Project Type Templates
The following project - specific templates are available:

### Node.js Template
For JavaScript / TypeScript projects using Node.js:

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / templates / enhanced - nodejs.yml'
```text

### Python Template
For Python - based projects:

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / templates / enhanced - python.yml'
```text

### Drupal Template
For Drupal - based projects:

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / templates / enhanced - drupal.yml'
```text

### MCP Template
For Model Context Protocol servers:

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / templates / enhanced - mcp.yml'
```text

## Usage Examples

### Complete Pipeline Example
This example shows a complete CI / CD pipeline for a Node.js project:

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / templates / enhanced - nodejs.yml'

variables:

# Cache configuration

  custom_paths:
    - .next / cache

# Test configuration

  coverage_threshold: 80
  parallel_tests: true
  parallel_jobs: 4

# Lint configuration

  fix_issues: false
  fail_on_error: true

# Security configuration

  scan_dependencies: true
  scan_sast: true
  scan_secrets: true
  security_level: high

# Custom job definitions can be added here

deploy:
  stage: deploy
  script:
    - npm run deploy
  only:
    - main
```text

### Using Individual Components
You can use individual components if needed:

```yaml
include:
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / templates / enhanced - base.yml'
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / components / cache.yml'
  - project: 'Bluefly / BFCIComponents'
    ref: main
    file: ' / components / testing.yml'

variables:
  project_type: nodejs
  custom_paths:
    - .next / cache
  coverage_threshold: 80
```text

## Best Practices
1. **Use Project - Type Templates**: Start with the appropriate project - type template for your project
2. **Override Only What's Necessary**: Use variables to customize behavior rather than redefining jobs
3. **Maintain Consistent Stage Names**: Keep the standard stage names to ensure proper ordering
4. **Regular Updates**: Regularly update to the latest version of the components
5. **Validate Changes**: Use `llmcli ci validate`to validate CI configuration changes

## Troubleshooting

### Common Issues
1. **Cache Not Working**: Ensure cache keys and paths are correctly defined
2. **Test Failures**: Check test command and environment variables
3. **Linting Errors**: Verify linting configuration matches project requirements
4. **Security Scan Failures**: Review security reports and address vulnerabilities

### Getting Help
Use the following commands for support:

```bash

# Validate CI configuration

llmcli ci validate <project>

# Check CI status

llmcli ci status <project>

# Get component documentation

llmcli ci docs <component>
```text

For further assistance, contact the DevOps team or create an issue in the BFCIComponents repository.
