# BFCIComponents Architecture Guide

This guide explains the new component-based architecture for BFCIComponents, designed to provide a more maintainable, flexible, and powerful approach to CI/CD pipeline configuration across the Bluefly LLM ecosystem.

## Component Architecture Overview

The new architecture follows GitLab's component-based approach, which offers several advantages:

1. **Explicit Input/Output Contracts**: Components define explicit inputs with types, defaults, and validation
2. **Composability**: Components can be composed together to create complex pipelines
3. **Reusability**: Components can be reused across projects with consistent behavior
4. **Versioning**: Components are explicitly versioned for stability and evolution
5. **Dynamic Configuration**: Components can generate dynamic configuration based on inputs
6. **Validation**: Inputs can be validated against schemas

## Key Concepts

### Component Inclusion

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/component-name@version
    inputs:
      parameter_name: value
```

### Component Structure

Components consist of:

1. **Metadata**: Name, description, version
2. **Inputs**: Parameters that control component behavior
3. **Outputs**: Results and artifacts produced by the component
4. **Examples**: Sample usage patterns
5. **Documentation**: Usage guides and best practices

### Expression Syntax

Components use GitLab's expression syntax for dynamic configuration:

```yaml
$[[ inputs.parameter_name || 'default_value' ]]
```

### Conditional Sections

Components can conditionally include sections based on inputs:

```yaml
$[[ inputs.enable_feature == true ]]:
  job_name:
    script:
      - echo "Feature enabled"
```

## Migrating from Templates to Components

### Before (Template-based)

```yaml
include:
  - project: 'bluefly/bfcicomponents'
    file: 'templates/nodejs.yml'
    ref: main

variables:
  NODE_VERSION: '20.0.0'
  ENABLE_LINT: 'true'
```

### After (Component-based)

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
    inputs:
      node_version: '20.0.0'
      enable_lint: true
```

## Core Components

### Enhanced NodeJS

A comprehensive CI/CD pipeline for Node.js projects:

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
    inputs:
      node_version: "20.0.0"
      enable_lint: true
      enable_test: true
      enable_build: true
```

Key features:
- Configurable linting, testing, and building
- Security scanning integration
- Deployment to staging and production
- Release management with semantic versioning

### TypeScript Build

Specialized component for TypeScript projects:

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/typescript-build@1.0.0
    inputs:
      typescript_version: "5.0.0"
      node_version: "20.0.0"
      enable_bundling: true
```

Key features:
- Type checking
- Transpilation
- Bundling
- Type definition generation

### Drupal Validation

Comprehensive validation for Drupal projects:

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal-validate@1.1.0
    inputs:
      modules_path: "web/modules/custom"
      validation_types: ["syntax", "standards", "config"]
```

### Model Registry

Advanced model tracking for ML projects:

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/model-registry@1.1.0
    inputs:
      model_name: "sentiment-analyzer"
      model_version: "1.0.0"
      model_framework: "PyTorch"
```

## Composition Patterns

Components can be composed to create comprehensive pipelines:

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/base-setup@1.0.0
    inputs:
      cache_enabled: true
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/nodejs-install@1.0.0
    inputs:
      node_version: "20.0.0"
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/eslint@1.0.0
    inputs:
      config_path: ".eslintrc.json"
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/jest@1.0.0
    inputs:
      coverage_threshold: 80
```

## Advanced Usage

### Dynamic Job Names

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
    inputs:
      lint_job_name: "eslint-check"
      test_job_name: "jest-tests"
      build_job_name: "typescript-build"
```

### Conditional Jobs

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
    inputs:
      enable_lint: true
      enable_test: $CI_COMMIT_BRANCH == 'main'
      enable_build: $CI_COMMIT_TAG != null
```

### Customized Behavior

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
    inputs:
      test_command: "npm run test:e2e"
      coverage_format: "lcov"
      lint_allow_failure: false
```

## Best Practices

1. **Use Specific Versions**: Always specify the component version to ensure stability

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
```

2. **Configure Only What You Need**: Default values are sensible, override only when necessary

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
    inputs:
      node_version: "20.0.0"  # Only specify what differs from defaults
```

3. **Compose Specialized Components**: Use composable components for maximum flexibility

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/nodejs-base@1.0.0
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/typescript-lint@1.0.0
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/jest-test@1.0.0
```

4. **Define Custom Jobs**: Extend component behavior with your own jobs

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0

# Custom job
custom_e2e_tests:
  stage: test
  script:
    - npm run test:e2e
  needs:
    - $[[ inputs.build_job_name || "build" ]]
```

5. **Document Component Usage**: Add comments explaining component configuration

```yaml
include:
  # Main CI pipeline for NodeJS applications
  # Configured for TypeScript with Jest testing
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/enhanced-nodejs@2.0.0
    inputs:
      node_version: "20.0.0"
      lint_command: "npx eslint --ext .ts,.tsx ."
      test_command: "npm run test:coverage"
```

## Troubleshooting

### Component Version Issues

If you encounter issues with a component:

1. Check the component version
2. Verify input parameters
3. Check the component documentation
4. Review GitLab CI logs for errors

### Migrating from Old Templates

When migrating from old templates:

1. Identify equivalent components
2. Map variables to inputs
3. Test thoroughly before committing
4. Update CI/CD variables in GitLab project settings

### Input Validation Errors

If you encounter input validation errors:

1. Check the input types and values
2. Verify enum values for restricted inputs
3. Check regex patterns for string inputs
4. Ensure boolean inputs are actual booleans, not strings

##  Current Architecture Status (January 2025)

### Known Issues Identified in Recent Audit

1. **Catalog Manifest Incomplete**: Only 6 of 26 existing components are documented in `catalog-manifest.yml`
2. **Architecture Fragmentation**: Mixed usage of `components/` and `templates/` directories
3. **Duplicate Components**: Same components exist in multiple locations
4. **Version Management**: Multiple versions of the same component without clear deprecation

### Immediate Actions Required

Before using components from this repository:

1. **Verify Component Availability**: Check both `components/` and `templates/` directories
2. **Use Specific Paths**: Reference components by their actual filesystem location
3. **Check Version Compatibility**: Multiple versions may exist for the same component
4. **Monitor Updates**: Architecture standardization is in progress

### Recommended Usage Pattern (Current)

Until standardization is complete, use this pattern:

```yaml
# For components in the components/ directory
include:
  - project: 'bluefly/bfcicomponents'
    file: 'components/component-name.yml'
    ref: main

# For template-based components
include:
  - project: 'bluefly/bfcicomponents'
    file: 'templates/component-name/template.yml'
    ref: main
```

### Standardization Roadmap

1. **Phase 1**: Complete catalog manifest with all existing components
2. **Phase 2**: Consolidate architecture (choose components/ OR templates/)
3. **Phase 3**: Remove duplicates and deprecated versions
4. **Phase 4**: Implement proper GitLab Component Catalog publishing

## Resources

- [GitLab CI/CD Components Documentation](https://docs.gitlab.bluefly..io/ci/components/)
- [GitLab CI/CD YAML Reference](https://docs.gitlab.bluefly..io/ci/yaml/)
- [GitLab CI/CD Inputs](https://docs.gitlab.bluefly..io/ci/inputs/)
- [GitLab CI/CD Steps](https://docs.gitlab.bluefly..io/ci/steps/)
- [Component Audit Report](CHANGELOG.md#130-2025-01-23---component-architecture-audit--standardization)

## Conclusion

The component architecture shows promise but requires immediate standardization to meet enterprise requirements. The audit identified significant organizational issues that must be addressed before production deployment at scale.

**Current Status**:  Architecture under standardization  
**Production Ready**: ❌ Not recommended until standardization complete  
**Development Use**:  Acceptable with awareness of current limitations

By completing the standardization roadmap, this will become a powerful, flexible, and maintainable approach to CI/CD pipelines in the Bluefly LLM ecosystem.