# CI Component Guidelines

This document outlines the standards and best practices for creating and maintaining CI / CD components in the BFCIComponents library.

## Component Structure

### Directory Structure

Components should be organized in a logical directory structure:

```text
templates/
├── enhanced - base.yml                 # Base templates at the root level
├── enhanced - typescript.yml           # Enhanced framework templates
├── category/                         # Group components by category
│   └── component.yml                 # Standard component file naming
│── another - category/
│   └── component.yml
```text

### Component File Format
All component files should follow this format:

```yaml

# Title and description (Required)

# Brief explanation of what the component does
name: component - name                  # Component name (Required)
version: "1.0.0"                      # Component version (Required)
description: "Description text"       # Component description (Required)
author: "Author name"                 # Component author (Required)
last_updated: "2025 - 04 - 19T00:00:00Z"  # ISO date (Auto - updated)

variables:                            # Variables section (Optional)
  VARIABLE_NAME: "default_value"      # Provide sensible defaults

stages:                               # Stages section (Optional)
  - stage - name                        # Define stages this component uses

# Component - specific YAML below

```text

## Design Principles

### 1. Reusability
Components should be designed for reuse across multiple projects:

- Use variables for customization
- Avoid hardcoding project - specific configurations
- Define sensible defaults that work in most cases

### 2. Modularity
Components should follow the Single Responsibility Principle:

- Each component should focus on one aspect of the CI / CD pipeline
- Complex workflows should be composed of multiple components
- Avoid creating monolithic components that try to do too much

### 3. Documentation
Components should be self - documenting:

- Include clear comments for non - obvious configurations
- Provide usage examples
- Document available variables and their purpose
- Include links to references when using complex features

### 4. Compatibility
Components should maintain compatibility:

- Follow semantic versioning for breaking changes
- Add new features without breaking existing usage
- Test components across multiple project types
- Document compatibility requirements

## Best Practices

### Variables
- Use SCREAMING_SNAKE_CASE for variable names
- Provide sensible defaults for all variables
- Use descriptive names that clearly indicate purpose
- Document acceptable values and constraints
- Use GitLab predefined variables when appropriate

Example:

```yaml
variables:
  NODE_VERSION: "18"                    # Node.js version to use
  CACHE_KEY: "$CI_COMMIT_REF_SLUG - npm"  # Cache key for npm dependencies
  COVERAGE_THRESHOLD: "80"              # Minimum test coverage percentage
```text

### Scripts
- Keep scripts concise and focused
- Use multiline scripts for complex operations
- Add comments for script blocks
- Use exit codes to indicate success / failure
- Include error handling for critical operations

Example:

```yaml
script:
  - echo "Installing dependencies..."
  - npm ci
  - |

# Run tests with coverage reporting

    echo "Running tests..."
    npm test

# Check coverage threshold

    COVERAGE = $(cat coverage / coverage - summary.json | jq -r '.total.lines.pct')
    if (( $(echo "$COVERAGE < [COVERAGE_THRESHOLD]" | bc -l) )); then
      echo "Coverage below threshold: $COVERAGE% < [COVERAGE_THRESHOLD]%"
      exit 1
    fi
```text

### Rules and Conditionals
- Use rules to control when jobs run
- Provide sensible default rules
- Document the conditions under which jobs will run
- Use variables to allow customization of rules

Example:

```yaml
rules:
  - if: $CI_COMMIT_BRANCH == "main"
    when: always
  - if: $CI_PIPELINE_SOURCE == "merge_request_event"
    when: always
  - when: manual
    allow_failure: true
```text

### Artifacts and Caching
- Use cache for dependencies and intermediary files
- Use artifacts for sharing build outputs between jobs
- Set reasonable expiry times
- Be careful with cache size and artifact retention

Example:

```yaml
cache:
  key: [CACHE_KEY]
  paths:
    - node_modules/

artifacts:
  paths:
    - dist/
  expire_in: 1 week
```text

## Testing Components
Before submitting a new component:

1. Test the component in a real project
2. Verify the component works with different configurations
3. Ensure the component follows the guidelines above
4. Validate the component with `npm run validate`## Component Maintenance

### Versioning
Components follow semantic versioning:

- MAJOR: Breaking changes
- MINOR: New features (backward compatible)
- PATCH: Bug fixes (backward compatible)

### Deprecation Process
When a component needs to be deprecated:

1. Mark it as deprecated in the component metadata
2. Document the replacement or alternative
3. Maintain backward compatibility for at least one major version
4. Remove only after adequate notice to users

Example:```yaml
name: deprecated - component
version: "1.0.0"
description: "This component is deprecated. Use new - component instead."
deprecated: true
replacement: "new - component"

```text

### Review Process
All components must go through code review:

1. Submit a merge request
2. Assign reviewers from the CI / CD team
3. Address feedback and update as needed
4. Obtain approval before merging
