# Claude Autotest Advanced Features Guide

## Overview

Claude Autotest provides automated, AI-powered test generation and self-healing for Node.js, Drupal, and other supported platforms. It is designed for developer productivity, maintainability, and robust CI/CD integration.

---

## Core Capabilities

- **AI Requirements Analysis**: Context-aware test planning and risk assessment.
- **Automated TDD Workflow**: Red-green-refactor cycles, minimal implementation, and refactoring.
- **Multi-Modal Test Generation**: Unit, integration, E2E, and contract tests.
- **Self-Healing**: Tests adapt to code changes automatically.
- **OpenAPI-Driven Testing**: API contract, schema, and security validation from OpenAPI specs.
- **CI/CD Integration**: Works with GitLab pipelines and BFCIComponents.

---

## Developer Usage Example

```javascript
import AIFeedbackOrchestrator from './scripts/ai-feedback-orchestrator.js';

const orchestrator = new AIFeedbackOrchestrator({
    enableContinuousLearning: true,
    enablePredictiveGeneration: true,
    enableMultiModelEnsemble: true,
    enableAutoDeployment: true
});

const session = await orchestrator.orchestrateTestGeneration({
    feature: 'Payment Processing System',
    requirements: [
        'Process credit card payments securely',
        'Handle payment failures gracefully',
        'Support multiple payment providers',
        'Implement fraud detection'
    ],
    priority: 'high',
    deadline: '2024-02-15'
}, {
    existingCode: codeContext,
    hasOpenAPISpec: true,
    teamExperience: 'senior',
    infrastructureConstraints: ['kubernetes', 'postgresql']
});

console.log(`Quality Score: [session.quality]`);
console.log(`Generated [session.testCount] tests in [session.duration]ms`);
```

---

## Drupal-Specific Best Practices

- **Custom Modules**: Place all custom code in `web/modules/custom/` and ensure each new class or service has a corresponding test.
- **TDD Enforcement**: Use the TDD workflow to require tests before code merges. Fail the pipeline if coverage drops or tests are missing.
- **OpenAPI Integration**: Store OpenAPI specs in `web/modules/custom/*/openapi/` and enable API contract/security testing jobs.
- **Recipe Validation**: Validate install profiles and recipes for config schema coverage and test install/uninstall hooks.
- **Security**: Integrate security review jobs for permissions, access checks, and privacy compliance.

---

## CI/CD Integration

- Use BFCIComponents for standardized pipeline stages.
- Add jobs for linting, security, and documentation validation.
- Publish coverage and test reports as pipeline artifacts and GitLab Pages.

---

*Developer-focused. Built for reliability, maintainability, and automation.*