# Code Quality Component

> Comprehensive code quality analysis with integrated TODO management, static analysis, and quality reporting for GitLab CI pipelines.

The Code Quality component provides a unified approach to code quality in your CI/CD pipeline, combining multiple quality tools including TODO analysis, ESLint, PHPCS, and more into a cohesive reporting system.

## Overview

Maintaining code quality is essential for sustainable development, but setting up and managing multiple quality tools can be time-consuming and complex. The Code Quality component solves this by:

1. Running multiple code quality tools in parallel
2. Aggregating results into a unified quality report
3. Providing a consolidated quality dashboard
4. Enforcing configurable quality thresholds
5. Tracking quality metrics over time
6. Integrating with GitLab Merge Requests

## Key Features

- **Unified Quality Checks**: Combines multiple quality tools into a single pipeline stage
- **TODO Management**: Integrates todo-aggregator for TODO tracking and policy enforcement
- **Static Analysis**: Supports ESLint, PHPCS, and language-specific linters
- **Quality Dashboard**: Interactive visualization of quality metrics
- **GitLab MR Integration**: Automatic comments on merge requests
- **Quality Trends**: Tracking quality metrics over time
- **Configurable Thresholds**: Customize severity levels and failure conditions

## Use Cases

- **Continuous Quality Monitoring**: Keep track of code quality metrics during development
- **Merge Request Validation**: Ensure quality standards are met before merging
- **Technical Debt Management**: Track and prioritize technical debt items
- **Onboarding**: Help new team members understand quality expectations
- **Release Readiness**: Ensure quality standards are met before releases
- **Cross-Language Projects**: Apply consistent quality standards across different languages

## Getting Started

### Basic GitLab CI Usage

Include the Code Quality component in your GitLab CI pipeline:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'
```

### Advanced Configuration

For projects with specific needs, customize the component with additional variables:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  # General settings
  CODE_QUALITY_FAIL_ON_ERROR: "true"
  CODE_QUALITY_REPORT_PATH: "reports/quality-report.md"
  
  # Tool enablement
  CODE_QUALITY_ESLINT_ENABLED: "true"
  CODE_QUALITY_TODO_ENABLED: "true"
  CODE_QUALITY_PHPCS_ENABLED: "true"
  CODE_QUALITY_STYLELINT_ENABLED: "true"
  CODE_QUALITY_MARKDOWNLINT_ENABLED: "true"
  
  # Tool-specific settings
  ESLINT_EXTENSIONS: "js,jsx,ts,tsx"
  PHPCS_STANDARD: "PSR12"
  TODO_MAX_CRITICAL: 3
  TODO_MAX_HIGH: 15
```

### Dashboard Integration

To generate an interactive dashboard as part of your pipeline:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

# This will create the dashboard in the GitLab Pages site
pages:
  extends: code-quality-dashboard
  stage: deploy
```

### Merge Request Comments

The component can automatically add quality reports as comments on merge requests:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  # Ensure you have a GitLab API token with appropriate permissions
  GITLAB_API_TOKEN: $GITLAB_API_TOKEN
```

## Examples

### Example 1: Basic Code Quality Check

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'
```

### Example 2: JavaScript/TypeScript Focus

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  CODE_QUALITY_ESLINT_ENABLED: "true"
  CODE_QUALITY_STYLELINT_ENABLED: "true"
  CODE_QUALITY_TODO_ENABLED: "true"
  CODE_QUALITY_PHPCS_ENABLED: "false"
  
  # Configure ESLint
  ESLINT_EXTENSIONS: "js,jsx,ts,tsx"
  ESLINT_CONFIG: ".eslintrc.js"
  
  # Configure TODOs
  TODO_INCLUDE_PATTERNS: "src/**/*.{js,jsx,ts,tsx}"
  TODO_MAX_CRITICAL: 2
```

### Example 3: PHP Focus

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  CODE_QUALITY_ESLINT_ENABLED: "false"
  CODE_QUALITY_PHPCS_ENABLED: "true"
  CODE_QUALITY_TODO_ENABLED: "true"
  
  # Configure PHPCS
  PHPCS_STANDARD: "PSR12"
  PHPCS_PATHS: "src,modules,tests"
  
  # Configure TODOs
  TODO_INCLUDE_PATTERNS: "src/**/*.php,modules/**/*.php"
  TODO_MAX_CRITICAL: 3
```

### Example 4: Full-Stack Project with Dashboard

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  # Enable all tools
  CODE_QUALITY_ESLINT_ENABLED: "true"
  CODE_QUALITY_PHPCS_ENABLED: "true"
  CODE_QUALITY_TODO_ENABLED: "true"
  CODE_QUALITY_STYLELINT_ENABLED: "true"
  CODE_QUALITY_MARKDOWNLINT_ENABLED: "true"
  
  # Initially don't fail the build
  CODE_QUALITY_FAIL_ON_ERROR: "false"

# Publish the dashboard to GitLab Pages
pages:
  extends: code-quality-dashboard
  stage: deploy
```

## Best Practices

1. **Start with Non-Failing Checks**: Begin with `CODE_QUALITY_FAIL_ON_ERROR: "false"` to establish baselines.
2. **Gradually Tighten Thresholds**: Reduce quality thresholds over time to drive improvement.
3. **Enable Only Relevant Tools**: Only enable the tools that are relevant to your project.
4. **Custom Configurations**: Use project-specific configurations for each tool.
5. **Regular Dashboard Reviews**: Schedule regular reviews of the quality dashboard.
6. **MR Integration**: Use quality checks as part of your code review process.

## Configuration Reference

### General Configuration

| Variable | Description | Default |
|----------|-------------|---------|
| `CODE_QUALITY_FAIL_ON_ERROR` | Whether to fail the build if quality thresholds are exceeded | "true" |
| `CODE_QUALITY_REPORT_PATH` | Path to write the quality report | "code-quality-report.md" |
| `CODE_QUALITY_DASHBOARD_PATH` | Path to generate the dashboard files | "public/code-quality" |

### Tool Enablement

| Variable | Description | Default |
|----------|-------------|---------|
| `CODE_QUALITY_ESLINT_ENABLED` | Enable ESLint checks | "true" |
| `CODE_QUALITY_TODO_ENABLED` | Enable TODO analysis | "true" |
| `CODE_QUALITY_PHPCS_ENABLED` | Enable PHP CodeSniffer | "false" |
| `CODE_QUALITY_STYLELINT_ENABLED` | Enable Stylelint | "false" |
| `CODE_QUALITY_MARKDOWNLINT_ENABLED` | Enable Markdownlint | "false" |

### ESLint Configuration

| Variable | Description | Default |
|----------|-------------|---------|
| `ESLINT_EXTENSIONS` | File extensions to lint | "js,jsx,ts,tsx" |
| `ESLINT_CONFIG` | ESLint configuration file | ".eslintrc.js" |
| `ESLINT_IGNORE` | ESLint ignore file | ".eslintignore" |

### PHP CodeSniffer Configuration

| Variable | Description | Default |
|----------|-------------|---------|
| `PHPCS_STANDARD` | Coding standard to use | "PSR12" |
| `PHPCS_PATHS` | Paths to scan | "src" |

### Stylelint Configuration

| Variable | Description | Default |
|----------|-------------|---------|
| `STYLELINT_CONFIG` | Stylelint configuration file | ".stylelintrc.json" |
| `STYLELINT_IGNORE` | Stylelint ignore file | ".stylelintignore" |

### Markdownlint Configuration

| Variable | Description | Default |
|----------|-------------|---------|
| `MARKDOWNLINT_CONFIG` | Markdownlint configuration file | ".markdownlint.json" |
| `MARKDOWNLINT_PATHS` | Paths to scan | "**/*.md" |

### TODO Configuration

The Code Quality component integrates with the todo-aggregator component. All todo-aggregator configuration variables are supported. See the [todo-aggregator documentation](./todo-aggregator.md) for more details.

## Related Components

- **[Todo Aggregator](./todo-aggregator.md)**: Management and analysis of TODOs in your codebase
- **[ESLint Component](./eslint.md)**: JavaScript/TypeScript linting and code style
- **[Drupal Lint Component](./drupal-tdd.md)**: Drupal-specific linting and standards enforcement

## Further Reading

- [Component Repository](https://gitlab.com/blueflyio/bfcicomponents/-/tree/main/components/code-quality)
- [GitLab Code Quality Documentation](https://docs.gitlab.com/ee/ci/testing/code_quality.html)
- [ESLint Documentation](https://eslint.org/docs/user-guide/getting-started)
- [PHP_CodeSniffer Documentation](https://github.com/squizlabs/PHP_CodeSniffer/wiki)