# Code Quality Component

[![pipeline status](https://gitlab.com/blueflyio/bfcicomponents/badges/main/pipeline.svg)](https://gitlab.com/blueflyio/bfcicomponents/-/commits/main)

> Comprehensive code quality analysis with integrated TODO management, static analysis, and quality reporting for GitLab CI pipelines.

This component provides a unified approach to code quality in your CI/CD pipeline, combining multiple quality tools including TODO analysis, ESLint, PHPCS, and more into a cohesive reporting system.

## What it does

The Code Quality component enhances your development workflow by:

- Running multiple code quality tools in parallel
- Aggregating results into a unified quality report
- Providing a consolidated quality dashboard
- Enforcing configurable quality thresholds
- Tracking quality metrics over time
- Integrating with GitLab Merge Requests

## Key Features

- **Unified Quality Checks**: Combines multiple quality tools into a single pipeline stage
- **TODO Management**: Integrates todo-aggregator for TODO tracking and policy enforcement
- **Static Analysis**: Supports ESLint, PHPCS, and language-specific linters
- **Quality Dashboard**: Interactive visualization of quality metrics
- **GitLab MR Integration**: Automatic comments on merge requests
- **Quality Trends**: Tracking quality metrics over time
- **Configurable Thresholds**: Customize severity levels and failure conditions

## Usage

### 1. Basic Usage in GitLab CI

Include the Code Quality template in your `.gitlab-ci.yml`:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  # Optional: Configure specific tools
  CODE_QUALITY_ESLINT_ENABLED: "true"
  CODE_QUALITY_TODO_ENABLED: "true"
  CODE_QUALITY_PHPCS_ENABLED: "false"
```

### 2. Advanced Configuration

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  # General settings
  CODE_QUALITY_FAIL_ON_ERROR: "true"
  CODE_QUALITY_REPORT_PATH: "reports/quality-report.md"
  
  # Tool-specific settings
  CODE_QUALITY_ESLINT_CONFIG: ".eslintrc.js"
  
  # TODO settings
  TODO_MAX_CRITICAL: 3
  TODO_MAX_HIGH: 15
  TODO_FAIL_ON_THRESHOLD: "true"
  
  # Linting settings
  ESLINT_EXTENSIONS: "js,jsx,ts,tsx"
  PHPCS_STANDARD: "PSR12"
```

### 3. Using the Quality Dashboard

The Code Quality component generates an interactive dashboard that can be published using GitLab Pages:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

# Publish quality dashboard to GitLab Pages
pages:
  extends: code-quality-dashboard
  stage: deploy
```

## Configuration

### General Configuration Options

| Variable | Description | Default |
|----------|-------------|---------|
| `CODE_QUALITY_FAIL_ON_ERROR` | Whether to fail the build if quality thresholds are exceeded | "true" |
| `CODE_QUALITY_REPORT_PATH` | Path to write the quality report | "code-quality-report.md" |
| `CODE_QUALITY_DASHBOARD_PATH` | Path to generate the dashboard files | "public/code-quality" |

### Tool Enablement

| Variable | Description | Default |
|----------|-------------|---------|
| `CODE_QUALITY_ESLINT_ENABLED` | Enable ESLint checks | "true" |
| `CODE_QUALITY_TODO_ENABLED` | Enable TODO analysis | "true" |
| `CODE_QUALITY_PHPCS_ENABLED` | Enable PHP CodeSniffer | "false" |
| `CODE_QUALITY_STYLELINT_ENABLED` | Enable Stylelint | "false" |
| `CODE_QUALITY_MARKDOWNLINT_ENABLED` | Enable Markdownlint | "false" |

### TODO Configuration

The Code Quality component integrates with the todo-aggregator component. All todo-aggregator configuration variables are supported:

| Variable | Description | Default |
|----------|-------------|---------|
| `TODO_MAX_CRITICAL` | Maximum allowed critical TODOs | 5 |
| `TODO_MAX_HIGH` | Maximum allowed high priority TODOs | 20 |
| `TODO_INCLUDE_PATTERNS` | Glob patterns to include | "**/*.{js,ts,php,css,scss}" |

See the [todo-aggregator documentation](../todo-aggregator/README.md) for more configuration options.

## Examples

### Example 1: Basic Code Quality Check

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'
```

### Example 2: JavaScript/TypeScript Focus

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  CODE_QUALITY_ESLINT_ENABLED: "true"
  CODE_QUALITY_STYLELINT_ENABLED: "true"
  CODE_QUALITY_TODO_ENABLED: "true"
  CODE_QUALITY_PHPCS_ENABLED: "false"
  
  # Configure ESLint
  ESLINT_EXTENSIONS: "js,jsx,ts,tsx"
  ESLINT_CONFIG: ".eslintrc.js"
  
  # Configure TODOs
  TODO_INCLUDE_PATTERNS: "src/**/*.{js,jsx,ts,tsx}"
  TODO_MAX_CRITICAL: 2
```

### Example 3: PHP Focus

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/code-quality/template.yml'

variables:
  CODE_QUALITY_ESLINT_ENABLED: "false"
  CODE_QUALITY_PHPCS_ENABLED: "true"
  CODE_QUALITY_TODO_ENABLED: "true"
  
  # Configure PHPCS
  PHPCS_STANDARD: "PSR12"
  PHPCS_PATHS: "src,modules,tests"
  
  # Configure TODOs
  TODO_INCLUDE_PATTERNS: "src/**/*.php,modules/**/*.php"
  TODO_MAX_CRITICAL: 3
```

## Best Practices

1. **Start with Non-Failing Checks**: Begin with `CODE_QUALITY_FAIL_ON_ERROR: "false"` to establish baselines.
2. **Gradually Tighten Thresholds**: Reduce quality thresholds over time to drive improvement.
3. **Different Configurations for Different Projects**: Customize the enabled tools based on project needs.
4. **Regular Dashboard Reviews**: Schedule regular reviews of the quality dashboard.
5. **Integrate with Code Review**: Use the quality reports as part of your code review process.

## Related Components

- **[Todo Aggregator](../todo-aggregator)**: Management and analysis of TODOs in your codebase
- **[ESLint Component](../eslint)**: JavaScript/TypeScript linting and code style
- **[Drupal Lint Component](../drupal/drupal-lint)**: Drupal-specific linting and standards enforcement

## License

MIT