# Code Quality MCP Connector

This module integrates the code-quality component with the Bluefly Model Context Protocol (MCP) server, allowing it to be used as a tool within the broader LLM ecosystem.

## Overview

The MCP Connector enables AI-powered code quality analysis to be invoked through the llm-mcp server, making it available to any component that uses the MCP protocol, including bfllm agents, Drupal workflows, and other ecosystem services.

## Architecture

```
┌───────────────────┐         ┌───────────────────┐
│                   │         │                   │
│     llm-mcp         │◄────────┤  AI-Powered       │
│     Server        │         │  Code Quality     │
│                   │         │  Component        │
└───────┬───────────┘         └───────────────────┘
        │                               ▲
        │                               │
        ▼                               │
┌───────────────────┐         ┌────────┴──────────┐
│                   │         │                   │
│     bfllm         │         │  llm_platform     │
│     Agents        │         │  recipe           │
│                   │         │                   │
└───────────────────┘         └───────────────────┘
```

## Features

### 1. Tool Registration

Registers the AI Quality Analyzer as a tool with the llm-mcp server, exposing the following capabilities:

- `analyzeCode`: Run code analysis on specified files
- `classifyTodos`: Enhance TODOs with AI-powered classification
- `generateQualityReport`: Create a comprehensive quality report
- `predictQualityTrends`: Forecast future quality metrics
- `suggestRefactoring`: Generate intelligent refactoring suggestions

### 2. LLM Integration

Enables natural language interactions with the code quality tools:

- Ask questions about code quality in natural language
- Request specific analyses through conversational interfaces
- Integrate code quality insights into agent workflows

### 3. Ecosystem Interoperability

Works seamlessly with other components in the LLM ecosystem:

- **native_aijs**: Uses the unified model interface for AI analysis
- **bfllm**: Leverages the Agent Execution Engine for complex quality workflows
- **llm-mcp**: Registers and executes as a standard MCP tool
- **llm_platform_recipe**: Integrates with Drupal's Progressive Assessment workflows
- **mcp_client_extras**: Uses additional MCP capabilities for enhanced functionality

## Usage

### 1. Register with MCP Server

```typescript
import [registerCodeQualityTools] from '@bfcicomponents/code-quality/mcp-connector';

// Register tools with MCP server
registerCodeQualityTools({
  mcpServerUrl: 'http://localhost:3001',
  aiModelConfig: {
    modelPath: './models/code-analyzer',
    confidenceThreshold: 0.75
  }
});
```

### 2. Invoke Through MCP Protocol

```typescript
// Example of invoking the code quality tools through MCP
const result = await mcpClient.execute({
  tool: 'code-quality.analyzeCode',
  parameters: {
    files: ['src/app.ts', 'src/utils.ts'],
    options: {
      detectAntiPatterns: true,
      securityVulnerabilities: true
    }
  }
});
```

### 3. Use in Drupal Workflows

```php
// Example Drupal integration with ECA
$event = new CodeQualityEvent();
$event->setRepository($repository_url);
$event->setBranch($branch);

// This will trigger the code quality analysis through MCP
$event_dispatcher->dispatch($event, CodeQualityEvent::ANALYZE);

// Get results
$quality_score = $event->getQualityScore();
$critical_issues = $event->getCriticalIssues();
```

## Prerequisites

- **llm-mcp Server**: Running instance of the llm-mcp server
- **native_aijs**: Core library for AI model interfaces
- **Node.js**: v16+ for running the connector
- **AI Models**: Access to code analysis models (local or remote)

## Configuration

The connector can be configured through environment variables or a configuration file:

- `MCP_SERVER_URL`: URL of the llm-mcp server
- `AI_MODEL_PATH`: Path to the AI models for code analysis
- `CONFIDENCE_THRESHOLD`: Minimum confidence threshold for AI findings
- `MAX_FILES_PER_ANALYSIS`: Maximum number of files to analyze in a single request
- `TIMEOUT_SECONDS`: Timeout for analysis operations

## Integration with GitLab CI

The MCP connector can be used in GitLab CI pipelines to provide code quality insights as part of the CI/CD process:

```yaml
code-quality-mcp:
  image: blueflyio/code-quality-mcp:latest
  script:
    - code-quality-mcp-connector --register
    - code-quality-mcp-connector --analyze
    - code-quality-mcp-connector --report
  variables:
    MCP_SERVER_URL: [MCP_SERVER_URL]
    AI_MODEL_PATH: models/code-analyzer
  artifacts:
    paths:
      - code-quality-report.md
      - quality-dashboard/
```

## Extensibility

The connector is designed to be extensible, allowing additional code quality tools to be registered with the MCP server:

- Custom analyzers can be added as new tools
- Language-specific analysis can be implemented
- Project-specific quality rules can be incorporated

## Security

The connector implements security best practices:

- Authentication with the MCP server using JWT
- Secure handling of code and analysis results
- Permission-based access to tools and capabilities
- Audit logging of all tool invocations