# Drupal AI Site Testing Component

GitLab CI/CD component for AI-powered Drupal site testing and analysis.

## Overview

This component leverages artificial intelligence to perform comprehensive site testing including content analysis, visual regression testing, accessibility validation, and SEO optimization recommendations.

## Usage

### Basic Usage

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal-ai-site-testing@main

ai_site_test:
  extends: .drupal_ai_site_testing
```

### Advanced Usage with Custom Configuration

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal-ai-site-testing@main

comprehensive_ai_test:
  extends: .drupal_ai_site_testing
  variables:
    php_version: "8.3"
    ai_model: "claude-3-haiku"
    test_coverage: "comprehensive"
    enable_visual_testing: "true"
```

## Inputs

| Input | Type | Default | Description | Options |
|-------|------|---------|-------------|---------|
| `php_version` | string | "8.3" | PHP version to use for AI site testing | "8.1", "8.2", "8.3", "8.4" |
| `ai_model` | string | "claude-3-haiku" | AI model to use for testing | "claude-3-haiku", "gpt-4", "gemini-pro" |
| `test_coverage` | string | "standard" | AI testing coverage level | "basic", "standard", "comprehensive" |
| `enable_visual_testing` | string | "true" | Enable AI-powered visual regression testing | "true", "false" |

## Features

###  AI-Powered Analysis
- **Content Quality Assessment**: AI evaluates content structure, readability, and SEO optimization
- **Accessibility Validation**: Automated accessibility testing with AI-enhanced reporting
- **Performance Analysis**: AI-driven performance recommendations

### 👁️ Visual Regression Testing
- **AI Similarity Detection**: Advanced visual comparison using machine learning
- **Cross-Browser Validation**: Automated testing across multiple browsers
- **Mobile Responsiveness**: AI-powered mobile layout analysis

###  Intelligent Reporting
- **Actionable Insights**: AI-generated recommendations for improvements
- **Trend Analysis**: Historical comparison and pattern recognition
- **Priority Scoring**: AI-ranked issues by impact and effort

## AI Models

### Claude 3 Haiku (Default)
- Fast and efficient for standard testing
- Excellent content analysis capabilities
- Cost-effective for regular testing

### GPT-4
- Advanced reasoning for complex scenarios
- Superior visual analysis capabilities
- Best for comprehensive testing

### Gemini Pro
- Multimodal analysis capabilities
- Strong performance optimization insights
- Good balance of speed and accuracy

## Coverage Levels

### Basic
- Content structure validation
- Basic accessibility checks
- Simple visual regression

### Standard (Default)
- Full content analysis
- Comprehensive accessibility testing
- Visual regression with AI similarity
- SEO optimization recommendations

### Comprehensive
- Advanced AI content analysis
- Multi-device visual testing
- Performance optimization analysis
- Security vulnerability assessment
- User experience evaluation

## Output Artifacts

### Test Results
- `ai-test-results/content-analysis.json` - AI content evaluation
- `ai-test-results/visual-analysis.json` - Visual regression results
- `ai-test-results/ai-test-summary.json` - Overall test summary

### Visual Assets
- `ai-screenshots/` - Screenshots and visual comparisons
- Baseline images for regression testing
- Visual diff reports

### Reports
- JUnit XML for GitLab integration
- AI-generated improvement recommendations
- Performance metrics and trends

## Requirements

- Drupal 9.5+ or Drupal 10+
- AI model API access (configured via CI/CD variables)
- Selenium WebDriver for visual testing
- Node.js for AI testing tools

## Configuration

### Environment Variables
Set these in your GitLab CI/CD variables:

```
AI_API_KEY=your_ai_api_key
VISUAL_TESTING_BASELINE_URL=https://your-site.com
```

### AI Model Configuration
Configure AI model access in your project settings or use provided defaults for basic functionality.

## Examples

### Daily AI Monitoring
```yaml
ai_daily_check:
  extends: .drupal_ai_site_testing
  variables:
    test_coverage: "standard"
    ai_model: "claude-3-haiku"
  rules:
    - if: $CI_PIPELINE_SOURCE == "schedule"
```

### Pre-Deployment Validation
```yaml
ai_pre_deploy:
  extends: .drupal_ai_site_testing
  variables:
    test_coverage: "comprehensive"
    ai_model: "gpt-4"
    enable_visual_testing: "true"
  rules:
    - if: $CI_COMMIT_BRANCH == "main"
```

## Contribute

This component is part of the BFCIComponents project. For issues, feature requests, or contributions, please visit our [GitLab repository](https://gitlab.bluefly..io/bluefly/bfcicomponents).

## License

MIT License - see LICENSE.md for details.