# Drupal Lint Component

This component performs comprehensive code quality and standards checking for Drupal projects, including PHP, JavaScript, and CSS linting.

## Features

-  **PHPCS Integration**: Drupal and DrupalPractice coding standards
-  **PHPStan Analysis**: Static analysis with configurable levels
-  **Security Checking**: Symfony security checker for vulnerabilities
-  **Frontend Linting**: ESLint for JavaScript, Stylelint for CSS
-  **Auto-Fix Support**: Automatic fixing of coding standards issues
-  **Flexible Configuration**: Custom rules and excluded paths
-  **Comprehensive Reports**: JUnit reports for GitLab integration

## Usage

### Basic Usage
```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-prepare@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
```

### Advanced Configuration
```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
    inputs:
      phpcs_standards: "Drupal,DrupalPractice,Custom"
      phpstan_level: 8
      fix_automatically: true
      fail_on_warnings: true
      excluded_paths: "vendor,tests/fixtures"
```

## Inputs

| Input | Default | Description |
|-------|---------|-------------|
| `phpcs_standards` | `Drupal,DrupalPractice` | PHPCS coding standards |
| `phpstan_enabled` | `true` | Enable PHPStan static analysis |
| `phpstan_level` | `7` | PHPStan analysis level (0-9) |
| `eslint_enabled` | `true` | Enable ESLint for JavaScript |
| `stylelint_enabled` | `true` | Enable Stylelint for CSS |
| `security_checker` | `true` | Run security vulnerability checks |
| `fix_automatically` | `false` | Auto-fix coding standards issues |
| `fail_on_warnings` | `false` | Fail pipeline on warnings |
| `custom_phpcs_config` | `""` | Path to custom PHPCS config |
| `excluded_paths` | `vendor,node_modules,...` | Paths to exclude |

## What Gets Checked

### PHP Files
- **Extensions**: `.php`, `.module`, `.inc`, `.install`, `.profile`, `.theme`
- **Standards**: Drupal coding standards, DrupalPractice
- **Static Analysis**: PHPStan for type checking and bug detection
- **Security**: Composer dependencies vulnerability scanning

### JavaScript Files
- **Extensions**: `.js`, `.jsx`, `.ts`, `.tsx`
- **Linting**: ESLint with configurable rules
- **Reports**: JUnit format for GitLab integration

### CSS/SCSS Files
- **Extensions**: `.css`, `.scss`, `.sass`
- **Linting**: Stylelint for consistent styling
- **Reports**: JSON format for detailed feedback

## Excluded Paths (Default)

The following paths are excluded by default:
- `vendor/` - Composer dependencies
- `node_modules/` - NPM dependencies
- `web/core/` - Drupal core
- `web/modules/contrib/` - Contributed modules
- `web/themes/contrib/` - Contributed themes

## Custom Configuration

### Custom PHPCS Configuration

Create a custom `phpcs.xml` or `phpcs.xml.dist` file:

```xml
<?xml version="1.0" encoding="UTF-8"?>
<ruleset name="custom">
  <description>Custom Drupal coding standards</description>
  
  <rule ref="Drupal"/>
  <rule ref="DrupalPractice"/>
  
  <!-- Custom exclusions -->
  <rule ref="Drupal.Commenting.FunctionComment.Missing">
    <exclude-pattern>*/tests/*</exclude-pattern>
  </rule>
</ruleset>
```

Then use it:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
    inputs:
      custom_phpcs_config: "phpcs.xml"
```

### Custom PHPStan Configuration

Create a `phpstan.neon` file:

```neon
parameters:
    level: 8
    paths:
        - src
        - modules
    excludePaths:
        - */vendor/*
        - */tests/fixtures/*
    ignoreErrors:
        - '#Call to an undefined method#'
```

## Auto-Fix Feature

Enable automatic fixing of coding standards issues:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
    inputs:
      fix_automatically: true
```

**Note**: Auto-fix only works for PHPCS issues that can be automatically corrected.

## Reports and Integration

The component generates several reports:

- **phpcs-report.xml**: JUnit format for GitLab test reports
- **phpstan-report.xml**: JUnit format for static analysis
- **security-report.json**: Security vulnerabilities report
- **eslint-report.xml**: JavaScript linting results
- **stylelint-report.json**: CSS linting results

These reports are automatically integrated with GitLab's test reports and security features.

## Troubleshooting

### Common Issues

1. **"No PHP files found"**: Check that your project has PHP files in the expected locations
2. **Memory errors**: PHPStan may need more memory for large projects
3. **ESLint not found**: Ensure ESLint is installed in your `package.json`
4. **Custom standards not found**: Verify custom PHPCS configuration paths

### Debug Mode

For debugging linting issues:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
    inputs:
      fail_on_warnings: false
      phpstan_level: 5  # Lower level for debugging
```

## Integration with Other Components

This component works best with the prepare component:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-prepare@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-lint@main
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-test-unit@main
```
