# Drupal TDD Legacy Component

GitLab CI/CD component for test-driven development in Drupal projects.

## Overview

This component provides a comprehensive test-driven development pipeline for Drupal projects, enabling developers to follow TDD best practices with PHPUnit tests, code coverage analysis, and automated reporting.

## Usage

### Basic Usage

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal/drupal-tdd-legacy@main

tdd:
  extends: .drupal_tdd
```

### Advanced Usage with Custom Configuration

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal/drupal-tdd-legacy@main

custom_tdd:
  extends: .drupal_tdd
  variables:
    php_version: "8.3"
    test_suite: "unit"
    coverage_enabled: "true"
    coverage_threshold: "80"
```

## Inputs

| Input | Type | Default | Description | Options |
|-------|------|---------|-------------|---------|
| `php_version` | string | "8.2" | PHP version to use for testing | "8.1", "8.2", "8.3" |
| `test_suite` | string | "all" | Test suite to run | "all", "unit", "kernel", "functional" |
| `coverage_enabled` | string | "true" | Enable code coverage reporting | "true", "false" |
| `coverage_threshold` | string | "70" | Minimum code coverage percentage | "50", "70", "80", "90" |
| `strict_mode` | string | "true" | Run tests in strict mode | "true", "false" |

## Features

### Test-Driven Development
- **PHPUnit Integration**: Full PHPUnit test suite execution
- **TDD Workflow**: Support for Red-Green-Refactor cycle
- **Test Isolation**: Each test runs in a clean environment

### Code Quality
- **Code Coverage**: Track test coverage with threshold enforcement
- **Test Results**: JUnit XML reports for GitLab integration
- **Coverage Reports**: HTML and Clover coverage reports

### Performance
- **Test Caching**: Smart caching of dependencies and test results
- **Parallel Testing**: Run tests in parallel for faster feedback
- **Optimized Execution**: Only run tests for changed files

## Example Pipelines

### Basic TDD Pipeline

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal/drupal-prepare@main
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal/drupal-tdd-legacy@main

stages:
  - prepare
  - test

prepare:
  extends: .drupal_prepare

test_driven_development:
  extends: .drupal_tdd
  needs:
    - prepare
```

### Multi-Stage TDD Pipeline

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal/drupal-prepare@main
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/drupal/drupal-tdd-legacy@main

stages:
  - prepare
  - unit
  - kernel
  - functional

prepare:
  extends: .drupal_prepare

unit_tests:
  extends: .drupal_tdd
  variables:
    test_suite: "unit"
  needs:
    - prepare

kernel_tests:
  extends: .drupal_tdd
  variables:
    test_suite: "kernel"
  needs:
    - unit_tests

functional_tests:
  extends: .drupal_tdd
  variables:
    test_suite: "functional"
  needs:
    - kernel_tests
```

## Requirements

- Drupal 9.5+ or Drupal 10+
- PHPUnit 9.5+
- GitLab 17.0+ for component support
- Properly configured `phpunit.xml` in your project

## Output Artifacts

- `junit.xml`: Test results in JUnit format
- `coverage/`: Code coverage reports in HTML and Clover format
- `phpunit.log`: Detailed test execution log

## Notes

This is a legacy component that has been standardized. For new projects, consider using the more advanced `drupal-tdd` or `drupal-tdd-enhanced` components.

## License

MIT License - see LICENSE.md for details