# Drupal UI/UX Testing Component

Comprehensive user interface and user experience testing for Drupal sites with a focus on LLM Platform integration. This component provides a complete toolkit for validating and improving the UI/UX of your Drupal sites.

## Features

- **Visual Regression Testing**: Ensure UI consistency across changes with automated screenshot comparison
- **Accessibility Testing**: Verify WCAG compliance for inclusive user experience
- **Usability Testing**: Evaluate user flows and interaction patterns
- **Responsive Design Testing**: Test across multiple device viewports
- **Performance Metrics**: Measure Core Web Vitals and user-centric performance metrics
- **AI-Powered UX Evaluation**: Leverage Claude AI for intelligent UX insights

## Usage

Include this component in your .gitlab-ci.yml file to enable comprehensive UI/UX testing:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-ui-ux-testing@1.0.0
    inputs:
      php_version: "8.3"
      drupal_version: "11.x"
      test_environment: "ci"
      test_coverage: "comprehensive"
      visual_regression_enabled: true
      accessibility_compliance: "WCAG2AA"
      usability_metrics_enabled: true
      responsive_testing: true
      performance_metrics_enabled: true
      ai_evaluation_enabled: true
      ai_model: "claude-3-haiku"
```

## Requirements

- PHP 8.1+ with Drupal-compatible extensions
- MySQL/MariaDB database
- Node.js 16+ for frontend testing tools
- Selenium WebDriver for browser automation
- GitLab Runner with Docker executor

## Inputs

### Core Configuration

| Input | Default | Description |
|-------|---------|-------------|
| `php_version` | `8.3` | PHP version to use for testing |
| `drupal_version` | `11.x` | Drupal version to test against |
| `test_environment` | `ci` | Testing environment (ci, staging, production) |
| `test_coverage` | `comprehensive` | UI/UX test coverage level |

### Visual Testing

| Input | Default | Description |
|-------|---------|-------------|
| `visual_regression_enabled` | `true` | Enable visual regression testing |
| `visual_baseline_branch` | `main` | Branch to use as visual baseline |
| `screenshot_quality` | `high` | Screenshot capture quality |
| `visual_diff_threshold` | `0.1` | Visual difference threshold (0-1) |

### Accessibility Testing

| Input | Default | Description |
|-------|---------|-------------|
| `accessibility_compliance` | `WCAG2AA` | Accessibility compliance level |
| `accessibility_scan_level` | `comprehensive` | Accessibility scan detail level |

### Usability Testing

| Input | Default | Description |
|-------|---------|-------------|
| `usability_metrics_enabled` | `true` | Enable usability metrics collection |
| `user_flow_testing` | `true` | Test complete user flows and journeys |
| `interaction_testing` | `true` | Test interactive elements |

### Responsive Testing

| Input | Default | Description |
|-------|---------|-------------|
| `responsive_testing` | `true` | Enable responsive design testing |
| `viewport_presets` | `mobile,tablet,desktop,large` | Viewport sizes to test |
| `custom_viewports` | `""` | Custom viewport sizes (format: WxH) |

### Performance Testing

| Input | Default | Description |
|-------|---------|-------------|
| `performance_metrics_enabled` | `true` | Enable performance metrics collection |
| `lighthouse_categories` | `performance,accessibility,best-practices,seo,pwa` | Lighthouse categories to test |

### AI Integration

| Input | Default | Description |
|-------|---------|-------------|
| `ai_evaluation_enabled` | `true` | Enable AI-powered UX evaluation |
| `ai_model` | `claude-3-haiku` | AI model for UX evaluation |

### Test Automation

| Input | Default | Description |
|-------|---------|-------------|
| `headless` | `true` | Run tests in headless mode |
| `parallel_instances` | `4` | Number of parallel test instances |
| `retry_flaky_tests` | `true` | Automatically retry flaky tests |

### Reporting Configuration

| Input | Default | Description |
|-------|---------|-------------|
| `detailed_reporting` | `true` | Generate detailed UI/UX reports |
| `report_format` | `html,json` | Report formats |
| `video_capture` | `false` | Capture video of test sessions |

## Implementation Details

This component orchestrates a comprehensive suite of UI/UX testing methodologies:

1. **Preparation**: Sets up the Drupal environment and installs necessary testing tools
2. **Visual Testing**: Captures and compares screenshots to detect unintended visual changes
3. **Accessibility**: Evaluates WCAG compliance using automated tools
4. **Usability**: Tests user flows and interaction patterns
5. **Responsive**: Verifies site appearance and functionality across device sizes
6. **Performance**: Measures load times, rendering performance, and Core Web Vitals
7. **AI Evaluation**: Uses Claude to provide intelligent UI/UX insights
8. **Reporting**: Generates comprehensive reports in multiple formats

## AI Integration

The AI evaluation uses Claude to analyze screenshots and page structures to provide:

- Overall UX impression and scoring
- Page-by-page layout assessment
- Identification of usability issues
- Suggestions for improving user experience
- Accessibility insights from visual analysis
- Consistent design language evaluation

## Output Artifacts

- **Screenshots**: Before/after comparisons for visual regression
- **HTML Reports**: Detailed HTML reports for each testing area
- **JSON Data**: Structured results data for integration with other tools
- **GitLab Pages**: Automatically published comprehensive report site

## Examples

### Basic Usage

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-ui-ux-testing@1.0.0
    inputs:
      php_version: "8.3"
      test_coverage: "basic"
```

### Comprehensive Enterprise Testing

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/drupal-ui-ux-testing@1.0.0
    inputs:
      php_version: "8.3"
      drupal_version: "11.x"
      test_coverage: "comprehensive"
      visual_regression_enabled: true
      visual_diff_threshold: 0.05
      accessibility_compliance: "WCAG2AAA"
      accessibility_scan_level: "comprehensive"
      usability_metrics_enabled: true
      responsive_testing: true
      viewport_presets: "mobile,tablet,desktop,large"
      custom_viewports: "414x896,1024x1366"
      performance_metrics_enabled: true
      lighthouse_categories: "performance,accessibility,best-practices,seo,pwa"
      ai_evaluation_enabled: true
      ai_model: "claude-3-sonnet"
      parallel_instances: 8
      detailed_reporting: true
      report_format: "html,json,pdf"
      video_capture: true
```

## Integration with Other Components

This component works well with:

- `drupal-tdd`: For test-driven development of UI components
- `drupal-performance`: For more in-depth performance analysis
- `drupal-security`: To include security aspects of the UI
- `claude-autotest`: For advanced AI-powered testing

## Maintainers

- Bluefly AI Platform Team