# GitLab CI Utilities Component

This component provides PHP utility classes for working with GitLab CI configurations programmatically.

## Usage

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/gitlab/ci-utilities@main
    inputs:
      mode: composer
      php_version: "8.1"
```

## Features

This component provides:

- PHP classes for programmatically working with GitLab CI YAML configurations
- Support for generating and validating GitLab CI/CD pipelines
- Utilities for managing jobs, stages, artifacts, caches, and rules
- Integration options via Composer or direct file inclusion

## Classes

The component includes these PHP utility classes:

- `CIComponent`: Base class for CI components
- `GitLabArtifact`: Class for managing GitLab CI artifacts
- `GitLabCache`: Class for managing GitLab CI caches
- `GitLabJob`: Class for managing GitLab CI jobs
- `GitLabPipeline`: Class for managing GitLab CI pipelines
- `GitLabRule`: Class for managing GitLab CI rules
- `GitLabStage`: Class for managing GitLab CI stages
- `GitLabTemplate`: Class for managing GitLab CI templates

## Scripts

This component provides the following CI/CD jobs:

- `gitlab-ci-utilities-install`: Installs the utility classes
- `gitlab-ci-utilities-test`: Tests the utility classes

## Inputs

| Input | Description | Type | Default | Required |
|-------|-------------|------|---------|----------|
| `mode` | How to include the utilities | string | `composer` | ❌ |
| `composer_namespace` | Composer namespace for utilities | string | `Bluefly\\CIComponents` | ❌ |
| `php_version` | PHP version to use | string | `8.1` | ❌ |

## Examples

### Install via Composer

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/gitlab/ci-utilities@main
    inputs:
      mode: composer
```

### Direct File Inclusion

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/gitlab/ci-utilities@main
    inputs:
      mode: include
```

### Custom Namespace

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/gitlab/ci-utilities@main
    inputs:
      mode: composer
      composer_namespace: "MyProject\\CI"
```

## Example Usage in PHP

```php
<?php

require_once 'vendor/autoload.php';

use Bluefly\CIComponents\GitLabPipeline;
use Bluefly\CIComponents\GitLabJob;
use Bluefly\CIComponents\GitLabStage;
use Bluefly\CIComponents\GitLabCache;

// Create a pipeline
$pipeline = new GitLabPipeline();

// Add stages
$pipeline->addStage('build');
$pipeline->addStage('test');
$pipeline->addStage('deploy');

// Create and configure a job
$job = new GitLabJob('build-job');
$job->setStage('build');
$job->addScript('npm ci');
$job->addScript('npm run build');

// Add cache
$cache = new GitLabCache(['node_modules/'], 'npm');
$job->setCache($cache);

// Add job to pipeline
$pipeline->addJob($job);

// Generate YAML
$yaml = $pipeline->toYaml();
file_put_contents('gitlab-ci.yml', $yaml);
```

## License

MIT License - see LICENSE.md for details