# GitLab ECA System

This component provides a powerful Event-Condition-Action (ECA) system for GitLab automation. It allows you to create rules that trigger actions based on GitLab events when specific conditions are met.

## Features

- Event-driven architecture for GitLab automation
- Condition evaluation with rich expressions
- Configurable actions with GitLab API integration
- Webhook integration for real-time event processing
- Rule management and serialization

## Usage

### Installation

Add this component to your project:

```yaml
include:
  - component: 'gitlab/eca-system'
    from: 'bfcicomponents'
```

### Configuration

Configure the component in your `.gitlab-ci.yml` file:

```yaml
variables:
  ECA_RULES_DIR: .gitlab/eca/rules
  ECA_WEBHOOK_URL: https://eca.example.com/webhook
  ECA_WEBHOOK_TOKEN: your-secret-token
```

### Creating ECA Rules

Create ECA rules in JSON format:

```json
// .gitlab/eca/rules/auto-approve-dependencies.json
{
  "id": "auto-approve-dependencies",
  "name": "Auto-approve dependency updates",
  "event": "merge_request",
  "conditions": [
    {
      "type": "branch_name_matches",
      "pattern": "^dependabot/"
    },
    {
      "type": "file_changes_limited_to",
      "paths": ["package.json", "package-lock.json"]
    }
  ],
  "actions": [
    {
      "type": "approve_mr",
      "message": "Automatically approved dependency update"
    },
    {
      "type": "add_label",
      "label": "dependencies"
    }
  ],
  "enabled": true
}
```

### Supported Events

- `merge_request` - Triggered on MR creation or update
- `push` - Triggered on push to repository
- `issue` - Triggered on issue creation or update
- `pipeline` - Triggered on pipeline status change
- `comment` - Triggered on comment creation
- `tag` - Triggered on tag creation

### Supported Conditions

- `branch_name_matches` - Check if branch name matches pattern
- `file_changes_limited_to` - Check if changes are limited to specific files
- `has_label` - Check if MR/issue has specific label
- `pipeline_status` - Check pipeline status
- `author_role` - Check if author has specific role
- `changed_lines_count` - Check number of changed lines
- `javascript_expression` - Evaluate custom JavaScript expression

### Supported Actions

- `approve_mr` - Approve merge request
- `add_label` - Add label to MR/issue
- `add_comment` - Add comment to MR/issue
- `assign_to` - Assign MR/issue to user
- `set_milestone` - Set milestone on MR/issue
- `merge_mr` - Merge a merge request
- `trigger_pipeline` - Trigger a pipeline
- `webhook_callback` - Call external webhook with event data

### Commands

#### Register Rules

```bash
gitlab-eca register-rules
```

#### Set Up Webhook

```bash
gitlab-eca setup-webhook --url=https://example.com/webhook --token=secret
```

#### Test Rules

```bash
gitlab-eca test-rules --event=merge_request --data-file=test-data.json
```

## Integration with llmcli

This component is fully integrated with llmcli. You can use it with the following commands:

```bash
llmcli gitlab eca register
llmcli gitlab eca setup-webhook
llmcli gitlab eca test
```

## Examples

### Auto-Label Based on Path

```json
{
  "id": "auto-label-frontend",
  "name": "Auto-label frontend changes",
  "event": "merge_request",
  "conditions": [
    {
      "type": "files_changed_in_path",
      "path": "src/frontend"
    }
  ],
  "actions": [
    {
      "type": "add_label",
      "label": "frontend"
    }
  ],
  "enabled": true
}
```

### Auto-Assign Based on Team

```json
{
  "id": "auto-assign-backend",
  "name": "Auto-assign backend MRs",
  "event": "merge_request",
  "conditions": [
    {
      "type": "has_label",
      "label": "backend"
    },
    {
      "type": "no_assignee"
    }
  ],
  "actions": [
    {
      "type": "assign_to_team",
      "team": "backend"
    }
  ],
  "enabled": true
}
```

### Notify on Pipeline Failure

```json
{
  "id": "notify-pipeline-failure",
  "name": "Notify on pipeline failure",
  "event": "pipeline",
  "conditions": [
    {
      "type": "pipeline_status",
      "status": "failed"
    },
    {
      "type": "branch_name_matches",
      "pattern": "^(main|release/.*)$"
    }
  ],
  "actions": [
    {
      "type": "send_slack_notification",
      "channel": "#pipeline-alerts",
      "message": "Pipeline failed on [branch_name]"
    }
  ],
  "enabled": true
}
```

## Advanced Usage

### Custom Condition Types

You can implement custom condition types:

```javascript
// .gitlab/eca/conditions/custom-condition.js
module.exports = class CustomCondition {
  constructor(options) {
    this.options = options;
  }
  
  evaluate(eventData) {
    // Custom evaluation logic here
    return true;
  }
  
  toJSON() {
    return {
      type: 'custom',
      ...this.options
    };
  }
  
  static fromJSON(json) {
    return new CustomCondition(json);
  }
};
```

### Custom Action Types

You can implement custom action types:

```javascript
// .gitlab/eca/actions/custom-action.js
module.exports = class CustomAction {
  constructor(options) {
    this.options = options;
  }
  
  async execute(eventData, apiClient) {
    // Custom action logic here
    return {
      success: true
    };
  }
  
  toJSON() {
    return {
      type: 'custom',
      ...this.options
    };
  }
  
  static fromJSON(json) {
    return new CustomAction(json);
  }
};
```

## Support

For issues or feature requests, please contact the LLM Platform team.