# GitLab AI Notes Component

A GitLab CI/CD component that automatically adds intelligent notes to merge requests, pipelines, and issues using AI analysis.

## Overview

This component uses the GitLab CLI (`glab`) to automatically add contextual, AI-enhanced notes to GitLab objects (MRs, pipelines, issues) based on build results, code analysis, and project history. It enables better communication, documentation, and traceability across your CI/CD processes.

## Usage

### Basic Usage

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/gitlab-ai-notes@main

pipeline_notes:
  extends: .gitlab_ai_notes
  variables:
    note_targets: "pipeline"
```

### Advanced Usage with Multiple Targets

```yaml
include:
  - component: gitlab.bluefly..io/bluefly/bfcicomponents/gitlab-ai-notes@main

comprehensive_notes:
  extends: .gitlab_ai_notes
  variables:
    ai_model: "claude-3-opus"
    note_targets: "pipeline,mr,issue"
    analysis_depth: "comprehensive"
    include_artifacts: "true"
    roadmap_integration: "true"
```

## Inputs

| Input | Type | Default | Description | Options |
|-------|------|---------|-------------|---------|
| `ai_model` | string | "claude-3-haiku" | AI model to use for generating notes | "claude-3-haiku", "claude-3-opus", "gpt-4" |
| `note_targets` | string | "pipeline" | Where to add AI-generated notes | "pipeline", "mr", "issue", "pipeline,mr", "pipeline,mr,issue" |
| `analysis_depth` | string | "standard" | Depth of AI analysis | "basic", "standard", "comprehensive" |
| `include_artifacts` | string | "true" | Include artifact analysis in notes | "true", "false" |
| `roadmap_integration` | string | "false" | Add insights to project roadmap | "true", "false" |

## Features

###  AI-Powered Analysis
- **Pipeline Analysis**: Intelligent analysis of build success/failure patterns
- **Code Quality Insights**: AI evaluation of changes and quality metrics
- **Test Coverage Trends**: Analysis of test coverage changes
- **Security Scanning**: Vulnerability detection and severity assessment

### 📝 Contextual Notes
- **Smart Summaries**: Concise summaries of complex pipeline results
- **Failure Analysis**: Root cause identification and suggested fixes
- **Improvement Suggestions**: AI-generated code improvement recommendations
- **Performance Insights**: Identify potential performance issues

### 🗺️ Roadmap Integration
- **Project Roadmap Updates**: Automatically post key insights to project roadmap
- **Progress Tracking**: Monitor development progress against roadmap
- **Dependency Insights**: Identify cross-project dependencies
- **Technical Debt Tracking**: Flag areas requiring refactoring

## AI Models

### Claude 3 Haiku (Default)
- Fast and efficient for basic notes
- Good pipeline analysis capabilities
- Cost-effective for frequent CI runs

### Claude 3 Opus
- In-depth analysis capabilities
- Superior code understanding
- Best for comprehensive insights

### GPT-4
- Advanced reasoning for complex codebases
- Strong technical recommendation capabilities
- Alternative option for diverse analysis

## Analysis Levels

### Basic
- Simple pass/fail analysis
- Basic error identification
- Pipeline status summaries

### Standard (Default)
- Detailed error analysis
- Test coverage assessment
- Code quality insights
- Security scan interpretation

### Comprehensive
- Deep build failure analysis
- Cross-pipeline trend identification
- Advanced security vulnerability assessment
- Code architecture recommendations
- Technical debt identification

## Requirements

- GitLab 17.0+
- GitLab CLI (`glab`) installed in CI environment
- CI token with API access
- AI model API access (configured via CI/CD variables)

## Configuration

### Environment Variables
Set these in your GitLab CI/CD variables:

```
GITLAB_TOKEN=your_gitlab_token
AI_API_KEY=your_ai_api_key
```

### Note Templates
The component includes default templates, but you can customize by providing your own in `.gitlab/note-templates/`.

## Examples

### MR Code Review Notes
```yaml
mr_review_notes:
  extends: .gitlab_ai_notes
  variables:
    note_targets: "mr"
    analysis_depth: "comprehensive"
  rules:
    - if: $CI_PIPELINE_SOURCE == "merge_request_event"
```

### Pipeline Status Notes with Roadmap Integration
```yaml
pipeline_status_notes:
  extends: .gitlab_ai_notes
  variables:
    note_targets: "pipeline"
    roadmap_integration: "true"
  rules:
    - if: $CI_COMMIT_BRANCH == "main"
```

### Issue Update Notes
```yaml
issue_update_notes:
  extends: .gitlab_ai_notes
  variables:
    note_targets: "issue"
    ai_model: "claude-3-opus"
  rules:
    - if: $CI_PIPELINE_SOURCE == "merge_request_event" && $CI_MERGE_REQUEST_TITLE =~ /^fix #\d+/
```

## Contribute

This component is part of the BFCIComponents project. For issues, feature requests, or contributions, please visit our [GitLab repository](https://gitlab.bluefly..io/bluefly/bfcicomponents).

## License

MIT License - see LICENSE.md for details.