# Todo Aggregator Component

> Aggregates and analyzes TODOs from your codebase with priority detection, visualization, and CI integration.

The Todo Aggregator component is a comprehensive solution for managing and tracking TODOs in your codebase. It automatically detects priorities and categories, provides interactive visualizations, and integrates with GitLab CI to enforce policies around critical TODOs.

## Overview

Development teams frequently use TODOs, FIXMEs, and similar comments to track pending work within the codebase. However, without proper management, these can accumulate and become difficult to track. The Todo Aggregator addresses this challenge by:

1. Scanning files for TODOs and similar comments
2. Automatically detecting priorities and categories
3. Generating organized roadmap documentation
4. Providing interactive visualizations for analysis
5. Enforcing policies around critical TODOs in CI pipelines
6. Tracking changes in TODOs over time

## Key Features

- **Priority Detection**: Automatically detects priorities based on text content (Critical, High, Medium, Low)
- **Category Classification**: Categorizes TODOs by type (bug, feature, security, etc.)
- **Interactive Dashboard**: Visualizes TODOs with charts and filtering capabilities
- **CI/CD Integration**: Enforces policies around critical TODOs in GitLab pipelines
- **Trend Analysis**: Tracks changes in TODOs over time
- **Project Management Integration**: Create issues from TODOs in systems like JIRA
- **Advanced EcosystemicIntelligence**: AI-powered deep analysis and insights

## Use Cases

- **Technical Debt Management**: Track and prioritize technical debt items
- **Security Vulnerability Tracking**: Ensure security-related TODOs are addressed
- **Sprint Planning**: Use categorized TODOs for sprint planning
- **Code Review**: Enforce policies around critical TODOs in merge requests
- **Release Readiness**: Ensure critical TODOs are addressed before releases
- **Project Onboarding**: Help new team members understand pending work

## Getting Started

### Basic GitLab CI Usage

Include the Todo Aggregator in your GitLab CI pipeline:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/template.yml'

variables:
  TODO_MAX_CRITICAL: 3
  TODO_FAIL_ON_THRESHOLD: "false"  # Start with non-failing checks
```

### Advanced Configuration

For teams with specific needs, customize the component with additional variables:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/template.yml'

variables:
  TODO_MAX_CRITICAL: 3
  TODO_MAX_HIGH: 15
  TODO_MAX_PER_FILE: 8
  TODO_FAIL_ON_THRESHOLD: "true"
  TODO_INCLUDE_PATTERNS: "src/**/*.ts,lib/**/*.js,app/**/*.php"
  TODO_EXCLUDE_PATTERNS: "**/node_modules/**,**/dist/**,**/vendor/**"
  TODO_CRITICAL_CATEGORIES: "security,crash,data-loss,compliance"
  TODO_REPORT_PATH: "reports/todo-analysis.md"
```

### Dashboard Generation

To generate an interactive dashboard as part of your pipeline:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/template.yml'

# This will create the dashboard in the GitLab Pages site
pages:
  extends: todo-dashboard
  stage: deploy
```

### Trend Analysis

For tracking changes in TODOs over time, use the trend analysis job on a schedule:

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/template.yml'

# Set up a scheduled pipeline in GitLab
# Go to CI/CD > Schedules > New schedule
# e.g., Weekly on Monday at 8:00 AM
```

## Best Practices

1. **Start with Non-Failing Checks**: Begin with `TODO_FAIL_ON_THRESHOLD: "false"` to establish baselines.
2. **Gradually Tighten Thresholds**: Reduce thresholds over time to drive improvement.
3. **Different Thresholds for Different Branches**: Use stricter rules for main branches.
4. **Regular Dashboard Reviews**: Schedule regular reviews of the todo dashboard.
5. **Track Trends Over Time**: Use trend analysis to track progress and identify patterns.

## Examples

### Example 1: Basic Todo Check

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/template.yml'
```

### Example 2: Security-Focused Check

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/template.yml'

variables:
  TODO_MAX_CRITICAL: 0  # Zero tolerance for critical TODOs
  TODO_CRITICAL_CATEGORIES: "security,vulnerability,data-leak,compliance,auth"
  TODO_FAIL_ON_THRESHOLD: "true"
```

### Example 3: Dashboard and Trend Analysis

```yaml
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/template.yml'

# Custom job to combine dashboard and trends
todo-analytics:
  extends: todo-dashboard
  needs:
    - todo-trends
  script:
    - !reference [todo-dashboard, script]
    - cp -r todo-trends/ public/todo-dashboard/trends/
  rules:
    - if: $CI_PIPELINE_SOURCE == "schedule"
```

## Configuration Reference

| Variable | Description | Default |
|----------|-------------|---------|
| `TODO_MAX_CRITICAL` | Maximum allowed critical TODOs | 5 |
| `TODO_MAX_HIGH` | Maximum allowed high priority TODOs | 20 |
| `TODO_MAX_PER_FILE` | Maximum allowed TODOs per file | 10 |
| `TODO_FAIL_ON_THRESHOLD` | Whether to fail the build if thresholds are exceeded | "true" |
| `TODO_INCLUDE_PATTERNS` | Comma-separated glob patterns to include | "**/*.ts,**/*.js,**/*.tsx,**/*.jsx" |
| `TODO_EXCLUDE_PATTERNS` | Comma-separated glob patterns to exclude | "**/node_modules/**,**/dist/**,**/build/**" |
| `TODO_CRITICAL_CATEGORIES` | Comma-separated list of categories always treated as critical | "security,crash,data-loss" |
| `TODO_REPORT_PATH` | Path to write the report | "todo-report.md" |

## Related Components

- **[ESLint Component](./eslint.md)**: Code linting and style enforcement
- **[Drupal TDD Component](./drupal-tdd.md)**: Test-driven development for Drupal

## Further Reading

- [Component Repository](https://gitlab.com/blueflyio/bfcicomponents/-/tree/main/components/todo-aggregator)
- [Priority Detection Feature](https://gitlab.com/blueflyio/bfcicomponents/-/blob/main/components/todo-aggregator/PRIORITY_DETECTION_FEATURE.md)
- [GitLab CI Integration Guide](https://gitlab.com/blueflyio/bfcicomponents/-/blob/main/components/todo-aggregator/docs/CI_INTEGRATION.md)