# Priority and Category Detection Feature Implementation

This document outlines the implementation of the priority detection and categorization feature added to the todo-aggregator component.

## Overview

The priority detection feature automatically analyzes todo text content and assigns priorities and categories to todo items. This makes the roadmap more useful by helping teams prioritize work more effectively.

## Files Modified

1. `src/priority.ts` (NEW):
   - Implemented priority and category detection logic
   - Defined Priority enum and EnhancedTodoItem interface
   - Created enhanceTodos utility function

2. `src/index.ts`:
   - Added support for enhancing todos with priority and category information
   - Added filtering functionality based on priority and category
   - Added priority and category statistics to the intelligence output

3. `src/formatter.ts`:
   - Updated to support both basic and enhanced todo items
   - Added priority-based organization for enhanced todos
   - Added category badges to todo items in the output

4. `src/cli.ts`:
   - Added command-line options for filtering by priority and category
   - Added validation for priority values

## Added Tests

1. `test/priority.test.ts` (NEW):
   - Tests for priority detection logic
   - Tests for category detection logic
   - Tests for the enhanceTodos utility function

2. `test/filtering.test.ts` (NEW):
   - Tests for filtering todos by priority and category

3. Updated existing tests to account for the new feature:
   - `test/formatter.test.ts`
   - `test/IntelligenceIntegration.test.ts`
   - `test/cli.test.ts`

## Examples

1. `examples/priority-demo.ts` (NEW):
   - Demonstrates priority and category detection functionality
   - Shows how to filter todos by priority and category
   - Generates sample roadmaps with different filters

## Usage

### CLI Usage

```sh
# Generate roadmap with all todos
npx bfcicomponents-todo-aggregator --patterns "src/**/*.ts" --out roadmap.md

# Filter by priority
npx bfcicomponents-todo-aggregator --priority HIGH --out high-priority-todos.md

# Filter by category
npx bfcicomponents-todo-aggregator --category bug --out bugs.md

# Filter by both priority and category
npx bfcicomponents-todo-aggregator --priority CRITICAL --category security --out critical-security-issues.md
```

### API Usage

```typescript
import { 
  aggregateTodosToRoadmap, 
  enhanceTodos, 
  Priority 
} from '@bfcicomponents/todo-aggregator';

// Generate roadmap with all todos
await aggregateTodosToRoadmap(['src/**/*.ts'], 'roadmap.md');

// Filter by priority
process.env.TODO_FILTER_PRIORITY = Priority.Critical;
await aggregateTodosToRoadmap(['src/**/*.ts'], 'critical-todos.md');

// Filter by category
process.env.TODO_FILTER_PRIORITY = '';  // Clear priority filter
process.env.TODO_FILTER_CATEGORY = 'security';
await aggregateTodosToRoadmap(['src/**/*.ts'], 'security-todos.md');

// Filter by both priority and category
process.env.TODO_FILTER_PRIORITY = Priority.High;
process.env.TODO_FILTER_CATEGORY = 'bug';
await aggregateTodosToRoadmap(['src/**/*.ts'], 'high-priority-bugs.md');
```

## Priority Detection

Priorities are automatically detected from todo text content:

- **CRITICAL**: Keywords like "critical", "ASAP", "blocking", "security", "vulnerability", "crash", etc.
- **HIGH**: Keywords like "high", "important", "necessary", "must", "bug", "error", "fix", etc.
- **MEDIUM**: Keywords like "medium", "enhancement", "feature", "update", "improve", etc.
- **LOW**: Keywords like "low", "minor", "cosmetic", "nice-to-have", "eventually", "refactor", etc.

You can also explicitly set priority with tags:
- `[p0]` or `[P0]` for CRITICAL
- `[p1]` or `[P1]` for HIGH
- `[p2]` or `[P2]` for MEDIUM
- `[p3]` or `[P3]` for LOW

## Category Detection

Categories are automatically detected from todo text content:

- **bug**: "bug", "issue", "fix", "broken", "error", etc.
- **feature**: "feature", "new", "add", "create", "implement", etc.
- **refactor**: "refactor", "clean", "improve", "optimize", etc.
- **docs**: "doc", "documentation", "comment", "explain", etc.
- **test**: "test", "spec", "unit", "integration", "coverage", etc.
- **security**: "security", "vulnerability", "XSS", "CSRF", etc.
- **performance**: "performance", "speed", "optimize", "slow", etc.
- **ui**: "UI", "interface", "visual", "layout", "style", etc.

You can also explicitly set a category with tags like `[api]`, `[db]`, etc.

## Next Steps

1. Fix remaining TypeScript errors in test files
2. Enhance priority and category detection with machine learning
3. Add support for custom priority and category patterns
4. Add more sophisticated filtering options
5. Improve visualization of priority and category data in the roadmap