# @bfcicomponents/todo-aggregator

[![pipeline status](https://gitlab.bluefly.io/llm/bfcicomponents/badges/main/pipeline.svg)](https://gitlab.bluefly.io/llm/bfcicomponents/-/commits/main)

Aggregates and analyzes TODOs from your codebase with priority detection, visualization, and CI integration. Now with advanced AI-powered EcosystemicIntelligence for deep task analysis and interactive dashboard capabilities!

## Features
- Scans all source files for TODOs and similar comments
- Deduplicates and categorizes by tag, file, or type
- Outputs a single markdown roadmap (e.g., `docs/roadmap.md`)
- Supports glob patterns and direct file lists
- CLI and Node API
- Generates summary reports across multiple projects
- **Priority Detection**: Automatically detect priorities based on text content
- **Category Classification**: Automatically categorize TODOs by type (bug, feature, etc.)
- **Interactive Dashboard**: Visualize TODOs with charts and filtering capabilities
- **CI/CD Integration**: Enforce policies around critical TODOs in CI pipelines
- **Trend Analysis**: Track changes in TODOs over time
- **Project Management Integration**: Create issues from TODOs in systems like JIRA
- **MCP Integration**: Connect with the Model Context Protocol for AI-powered workflows
- **Advanced EcosystemicIntelligence** system that provides:
  - Advanced task clustering and similarity detection
  - Dependency analysis and chain detection
  - Task prioritization based on context
  - Workload analysis and balancing recommendations
  - Intelligent insights generation

## CLI Usage

```sh
# Scan using glob patterns (default)
npx bfcicomponents-todo-aggregator --patterns "src/**/*.{ts,js,php}" --out docs/roadmap.md

# Scan using a file containing a list of files (one per line)
npx bfcicomponents-todo-aggregator --files file-list.txt --out docs/roadmap.md

# Generate intelligent roadmap with advanced insights
npx bfcicomponents-todo-aggregator --intelligence --out docs/intelligent-roadmap.md

# Filter by priority
npx bfcicomponents-todo-aggregator --priority HIGH --out docs/high-priority-todos.md

# Filter by category
npx bfcicomponents-todo-aggregator --category bug --out docs/bugs.md

# Generate HTML output with visualizations
npx bfcicomponents-todo-aggregator --format html --out docs/todos.html

# Generate JSON output for programmatic use
npx bfcicomponents-todo-aggregator --format json --out docs/todos.json

# Auto-detect format from file extension
npx bfcicomponents-todo-aggregator --out todos.html  # HTML format
npx bfcicomponents-todo-aggregator --out todos.json  # JSON format

# Include code snippets in the output
npx bfcicomponents-todo-aggregator --include-code-snippets --out docs/todos-with-snippets.md

# Include file links to repository
npx bfcicomponents-todo-aggregator --include-file-links --repo-url https://github.com/user/repo --out docs/todos.md

# Start the interactive dashboard
npx bfcicomponents-todo-aggregator --dashboard

# Run CI check for TODOs
npx bfcicomponents-todo-aggregator --ci-check

# Run CI check with custom configuration
npx bfcicomponents-todo-aggregator --ci-check --config todo-ci-config.json

# Use with GitLab CI (add to your .gitlab-ci.yml)
include:
  - project: 'blueflyio/bfcicomponents'
    ref: main
    file: 'components/todo-aggregator/examples/gitlab-ci-template.yml'

# Help
npx bfcicomponents-todo-aggregator --help
```

## API Usage

```js
// Using glob patterns
import [aggregateTodosToRoadmap] from '@bfcicomponents/todo-aggregator';

await aggregateTodosToRoadmap(['src/**/*.ts', 'lib/**/*.js'], 'docs/roadmap.md');

// Using specific file list
import [aggregateTodosFromFilesToRoadmap] from '@bfcicomponents/todo-aggregator';

const filePaths = [
  '/path/to/file1.ts',
  '/path/to/file2.js'
];

await aggregateTodosFromFilesToRoadmap(filePaths, 'docs/roadmap.md');

// Using EcosystemicIntelligence for advanced insights
import [aggregateTodosWithIntelligence] from '@bfcicomponents/todo-aggregator';

const result = await aggregateTodosWithIntelligence(
  ['src/**/*.ts', 'lib/**/*.js'], 
  'docs/intelligent-roadmap.md'
);

console.log(`Generated [result.insights.length] intelligent insights`);

// Working with enhanced todos and priorities
import { enhanceTodos, Priority } from '@bfcicomponents/todo-aggregator';

const todos = [/* your todos here */];
const enhancedTodos = enhanceTodos(todos);

// Filter todos by priority
const criticalTodos = enhancedTodos.filter(todo => todo.priority === Priority.Critical);

// Filter todos by category
const bugTodos = enhancedTodos.filter(todo => todo.categories.includes('bug'));

// Generate output in different formats
import { 
  formatTodosAsMarkdown, 
  formatTodosAsHtml, 
  formatTodosAsJson,
  formatTodos
} from '@bfcicomponents/todo-aggregator';

// Markdown format
const markdown = await formatTodosAsMarkdown(enhancedTodos, {
  includeCodeSnippets: true,
  includeFileLinks: true,
  repositoryUrl: 'https://github.com/user/repo'
});
await fs.writeFile('todos.md', markdown);

// HTML format with visualizations
const html = await formatTodosAsHtml(enhancedTodos, {
  includeCodeSnippets: true,
  includeCharts: true,
  includeSummary: true
});
await fs.writeFile('todos.html', html);

// JSON format for programmatic use
const json = await formatTodosAsJson(enhancedTodos, {
  prettyPrint: true,
  includeSummary: true
});
await fs.writeFile('todos.json', json);

// Using the unified formatter
const output = await formatTodos(enhancedTodos, {
  format: 'html',  // 'markdown', 'html', or 'json'
  html: {
    includeCodeSnippets: true,
    includeCharts: true
  }
});
await fs.writeFile('todos.html', output);

// MCP Integration
import { createTodoMcpConnector, startServer } from '@bfcicomponents/todo-aggregator';

// Create and initialize MCP connector
const connector = createTodoMcpConnector({
  mcpServerUrl: 'http://localhost:3001',
  apiKey: 'your-api-key',
  toolPrefix: 'todo-aggregator'
});
await connector.initialize();

// Start server with MCP integration
startServer({ 
  mcpConfig: {
    mcpServerUrl: 'http://localhost:3001',
    apiKey: 'your-api-key'
  }
});
```

## Batch Processing

This package also includes helper scripts for processing multiple projects:

```sh
# Generate roadmaps for multiple projects
./generate-all-roadmaps.sh

# Generate a summary of all roadmaps
./generate-all-roadmaps-summary.sh
```

## Supported Comment Styles

- Standard comment styles: `// TODO: message`, `/* TODO: message */`, `# TODO: message`, `<!-- TODO: message -->`
- Category tagging: `TODO [category]: message`, `TODO[category]: message`
- Priority tagging: `TODO [p0]: message` (p0-p3 for priority levels)
- Common tags: `TODO`, `FIXME`, `XXX`, `HACK`, `BUG`, `NOTE`, `OPTIMIZE`, `REVIEW`

## Priority Detection

The tool automatically detects priority levels based on the content of your TODOs:

- **CRITICAL**: Keywords like "critical", "ASAP", "blocking", "security", "vulnerability", "crash", etc.
- **HIGH**: Keywords like "high", "important", "necessary", "must", "bug", "error", "fix", etc.
- **MEDIUM**: Keywords like "medium", "enhancement", "feature", "update", "improve", etc.
- **LOW**: Keywords like "low", "minor", "cosmetic", "nice-to-have", "eventually", "refactor", etc.

You can also explicitly set priority with tags:
- `[p0]` or `[P0]` for CRITICAL
- `[p1]` or `[P1]` for HIGH
- `[p2]` or `[P2]` for MEDIUM
- `[p3]` or `[P3]` for LOW

## Category Detection

The tool automatically categorizes TODOs based on their content:

- **bug**: "bug", "issue", "fix", "broken", "error", etc.
- **feature**: "feature", "new", "add", "create", "implement", etc.
- **refactor**: "refactor", "clean", "improve", "optimize", etc.
- **docs**: "doc", "documentation", "comment", "explain", etc.
- **test**: "test", "spec", "unit", "integration", "coverage", etc.
- **security**: "security", "vulnerability", "XSS", "CSRF", etc.
- **performance**: "performance", "speed", "optimize", "slow", etc.
- **ui**: "UI", "interface", "visual", "layout", "style", etc.

You can also explicitly set a category with tags like `[api]`, `[db]`, etc.

## Roadmap
-  Configurable globs and tag filters
-  Support for file lists
-  Multi-project scanning and summary reports
-  EcosystemicIntelligence integration
-  Advanced dependency analysis and task clustering
-  Automatic priority and category detection
-  Filtering by priority and category
-  Interactive dashboard with visualizations
-  CI/CD integration with policy enforcement
-  Project management system integration (JIRA)
-  Trend analysis and time-based tracking
-  MCP integration for AI-powered workflows
-  HTML/JSON output formats
- Docs integration (Docusaurus, VuePress, Drupal)
- GitHub/GitLab issue sync
- Advanced ML-based task estimation

## EcosystemicIntelligence System

The new EcosystemicIntelligence system provides advanced capabilities for managing and analyzing tasks:

### Cognitive Architecture

```
EcosystemicIntelligence
├── EmergenceEngine       - Pattern detection and application
│   ├── CognitiveNodes    - Processing units with specialized capabilities  
│   │   └── ProcessingModules - Individual data transformation components
│   └── EmergencePatterns - Detects and applies emergent patterns in data
└── Intelligence API      - High-level interface for todo processing
```

### Key Components

- **ProcessingModule**: Atomic unit that performs a single transformation on data
- **CognitiveNode**: Container for processing modules with defined capabilities
- **EmergencePattern**: Rules for detecting and responding to data patterns
- **EmergenceEngine**: Orchestrates pattern detection and application
- **EcosystemicIntelligence**: High-level facade for the entire system

### Available Intelligence Features

- **Task Clustering**: Identifies groups of similar tasks that could be consolidated
- **Dependency Detection**: Discovers implicit dependencies between tasks
- **Priority Analysis**: Calculates task priorities based on context and keywords
- **Workload Balancing**: Analyzes task distribution and suggests rebalancing
- **Task Validation**: Validates todo structure and provides quality feedback

### Using the Intelligence API

```typescript
import [EcosystemicIntelligence] from '@bfcicomponents/todo-aggregator';

// Create and initialize the intelligence system
const intelligence = new EcosystemicIntelligence('project-intel', 'Project Intelligence');
await intelligence.initialize();

// Process todos with the intelligence system
const todos = [/* your todos here */];
const results = await intelligence.processTodos(todos);

// Access insights
const insights = results.insights;
console.log(`Generated [insights.length] insights`);

// Access processed todos
const processedTodos = results.todos;
```

You can also use the intelligence system directly in your own applications to analyze tasks and generate insights.

## MCP Integration

The component includes integration with the Model Context Protocol (MCP) to make todo-aggregator capabilities available as tools in the Bluefly LLM ecosystem:

```typescript
import [createTodoMcpConnector] from '@bfcicomponents/todo-aggregator';

// Create and initialize the MCP connector
const connector = createTodoMcpConnector({
  mcpServerUrl: 'http://localhost:3001',
  apiKey: 'your-api-key',
  toolPrefix: 'todo-aggregator'
});
await connector.initialize();
```

Available MCP tools:
- `todo-aggregator.scanFiles` - Scan files for TODOs based on glob patterns
- `todo-aggregator.analyzeTodos` - Analyze TODOs with priority and category detection
- `todo-aggregator.generateRoadmap` - Generate a roadmap markdown file from TODOs
- `todo-aggregator.generateJson` - Generate a JSON file from TODOs for visualization
- `todo-aggregator.formatTodosAsMarkdown` - Format TODOs as markdown

See the [MCP Integration Guide](./docs/MCP_INTEGRATION.md) for more details.

## Interactive Dashboard

The todo-aggregator includes an interactive dashboard for visualizing TODO distribution:

```bash
# Start the dashboard server
npx bfcicomponents-todo-aggregator --dashboard
```

Then open http://localhost:3000 in your browser.

The dashboard provides:
- Priority distribution charts
- Category analysis
- File-based TODO statistics
- Filtering capabilities
- Detailed table view

## CI/CD Integration

Integrate with CI/CD pipelines to enforce policies around TODOs:

```bash
# Run CI check with default thresholds
npx bfcicomponents-todo-aggregator --ci-check

# Run with custom configuration
npx bfcicomponents-todo-aggregator --ci-check --config todo-ci-config.json
```

Example configuration file:
```json
{
  "maxCriticalTodos": 5,
  "maxHighPriorityTodos": 20,
  "maxTodosPerFile": 10,
  "reportPath": "todo-report.md",
  "failOnThresholdExceeded": true,
  "criticalCategories": ["security", "crash", "data-loss"],
  "includePatterns": ["**/*.ts", "**/*.js"],
  "excludePatterns": ["**/node_modules/**", "**/dist/**"]
}
```

## Examples

The package includes multiple examples demonstrating various features:

1. **Priority Demo**: Basic priority and category detection
2. **Custom Patterns**: Customizing priority and category detection patterns
3. **JIRA Integration**: Integration with project management systems
4. **Visualize Priorities**: ASCII chart visualization
5. **Interactive Dashboard**: Web-based dashboard
6. **Trend Analysis**: Tracking changes over time
7. **CI Integration**: Integrating with CI/CD pipelines
8. **MCP Integration**: Integrating with the Model Context Protocol

Run examples:
```bash
npm run examples
```

## Documentation

Comprehensive documentation is available in the [docs](./docs) directory:

- [Priority Detection Feature](./PRIORITY_DETECTION_FEATURE.md)
- [GitLab CI Integration Guide](./docs/CI_INTEGRATION.md)
- [MCP Integration Guide](./docs/MCP_INTEGRATION.md)