# Todo Aggregator Examples

This directory contains example scripts that demonstrate the features of the todo-aggregator package, including priority and category detection, custom pattern matching, visualization, and integration with project management systems.

## Available Examples

### 1. Priority and Category Detection Demo

The `priority-demo.ts` script demonstrates the priority and category detection functionality. It shows how todo-aggregator automatically detects priorities and categories from todo text content, and how these can be used to organize and filter the roadmap.

### Running the Demo

To run the demo:

1. Make sure you have TypeScript installed:
   ```bash
   npm install -g typescript
   ```

2. Compile the demo script:
   ```bash
   tsc -t es2020 -m commonjs priority-demo.ts
   ```

3. Run the compiled script:
   ```bash
   node priority-demo.js
   ```

4. Check the generated roadmap files:
   - `all-todos.md`: Complete roadmap with all TODOs
   - `critical-todos.md`: Only critical priority TODOs
   - `security-todos.md`: Only security-related TODOs
   - `high-priority-bugs.md`: Only high priority bugs

## Demo Output

The demo creates sample TypeScript files with various TODOs of different priorities and categories, then generates several roadmap files to demonstrate the filtering capabilities of the package.

### Example TODOs

The sample files include:

1. Critical priority TODOs:
   - "Fix security vulnerability in authentication system"
   - "This is a blocking issue that needs immediate attention"
   - "[p0] Update password hashing with stronger algorithm"
   - "Fix critical data leak in user profile"
   - "ASAP: Add input validation to prevent XSS"

2. High priority TODOs:
   - "Fix bug in payment processing"
   - "Add error handling for failed transactions"
   - "Important: Transaction ID is not being recorded"
   - "[p1] Update pricing calculation"
   - "High priority - Fix incorrect tax calculation"

3. Medium priority TODOs:
   - "Implement new feature for user notifications"
   - "Add support for different notification channels"
   - "Improve notification templating system"
   - "[p2] Refactor database connection logic"
   - "Add connection pooling for better performance"

4. Low priority TODOs:
   - "Refactor utility functions when time permits"
   - "Nice-to-have - Add support for different date formats"
   - "Optimize date parsing algorithm later"
   - "[p3] Update documentation"
   - "Eventually add more test cases"

### 2. Custom Patterns (`custom-patterns.ts`)

Demonstrates how to customize priority and category detection patterns to match your project's specific conventions.

```bash
npx tsc examples/custom-patterns.ts --esModuleInterop --target ES2020 --module commonjs
node examples/custom-patterns.js
```

Key features:
- Custom priority patterns based on project-specific terminology
- Custom category patterns for specialized domains
- Configuration management for pattern persistence
- Integration with existing workflows

### 3. JIRA Integration (`jira-integration.ts`)

Shows how to integrate the todo-aggregator with project management systems like JIRA.

```bash
npx tsc examples/jira-integration.ts --esModuleInterop --target ES2020 --module commonjs
node examples/jira-integration.js
```

Key features:
- Filtering TODOs by priority and category before creating issues
- Mapping TODO priorities to JIRA issue priorities
- Mapping TODO categories to JIRA components or labels
- Maintaining traceability between code TODOs and JIRA issues

### 4. Visualize Priorities (`visualize-priorities.ts`)

Generates visual reports of TODO priorities and categories using ASCII charts.

```bash
npx tsc examples/visualize-priorities.ts --esModuleInterop --target ES2020 --module commonjs
node examples/visualize-priorities.js
```

Key features:
- Bar charts showing distribution of priorities and categories
- Pie charts for proportion visualization
- Stacked bar charts showing priority distribution within categories
- Ideas for enhanced visualization in real-world implementations

### 5. Interactive Dashboard (`visualization-dashboard/index.html`)

Provides an interactive web-based dashboard for visualizing and exploring TODOs.

```bash
# Generate JSON data from your codebase
npx tsc examples/generate-json.ts --esModuleInterop --target ES2020 --module commonjs
node examples/generate-json.js "src/**/*.ts"

# Open the dashboard in your browser
open examples/visualization-dashboard/index.html
```

Key features:
- Interactive charts showing TODO distribution by priority and category
- Filtering capabilities to focus on specific priorities or categories
- Detailed table view with sorting and highlighting
- File-based analysis showing which files have the most TODOs
- Sample data for demonstration purposes

### 6. Trend Analysis (`trend-analysis.ts`)

Demonstrates how to track and analyze TODO changes over time.

```bash
npx tsc examples/trend-analysis.ts --esModuleInterop --target ES2020 --module commonjs
node examples/trend-analysis.js
```

Key features:
- Generation of historical TODO data with timestamps
- Trend analysis showing how priorities and categories change over time
- Statistics for added and resolved TODOs
- Recommendations based on trend data
- Foundation for implementing real-time TODO monitoring

### 7. CI Integration (`ci-integration.ts`)

Demonstrates how to integrate todo-aggregator with CI/CD pipelines.

```bash
npx tsc examples/ci-integration.ts --esModuleInterop --target ES2020 --module commonjs
node examples/ci-integration.js
```

Key features:
- Automated TODO priority checking in CI pipelines
- Configurable thresholds for critical and high-priority TODOs
- Detailed Markdown reports for TODO status
- GitHub Actions and GitLab CI configuration generation
- Policy enforcement to prevent accumulation of critical TODOs

## Customizing the Examples

### Adding Your Own Examples

1. Create a new TypeScript file in the examples directory
2. Import necessary components from the todo-aggregator
3. Implement your example logic
4. Compile and run as described above

### Adapting for Your Project

To adapt these examples for your project:

1. Modify the pattern detection in `custom-patterns.ts` to match your team's conventions
2. Update the JIRA configuration in `jira-integration.ts` to match your instance
3. Enhance the visualization formats in `visualize-priorities.ts` to meet your reporting needs
4. Integrate with your project's specific workflow or build system

## Advanced Usage

### Environment Variables

Many examples support configuration via environment variables:

- `TODO_FILTER_PRIORITY`: Filter TODOs by a specific priority level
- `TODO_FILTER_CATEGORY`: Filter TODOs by a specific category
- `TODO_CUSTOM_PATTERNS`: Path to a JSON file containing custom patterns
- `TODO_JIRA_URL`: URL for your JIRA instance (in the integration example)

Example:
```bash
TODO_FILTER_PRIORITY=CRITICAL node examples/priority-demo.js
```

### Integration with Build Systems

You can integrate these examples with your build system by:

1. Adding npm scripts to your package.json
2. Creating custom workflows in your CI/CD pipeline
3. Generating reports as part of your build process
4. Setting up automated issue creation based on critical TODOs

## Further Development

Ideas for extending these examples:

1. Add real-time monitoring of priority changes
2. Implement a web dashboard for visualizing TODO statistics
3. Create notification systems for critical TODOs
4. Develop ML-based priority detection based on historical data
5. Integrate with team chat platforms for notifications