# BFCI Validation Component

This component provides validation utilities for Bluefly CI components. It includes validators for Node.js projects, GitLab components, libraries, Drupal modules, and API services.

## Features

- Node.js project validation
- GitLab component validation
- Library validation
- Drupal TDD validation
- API service validation
- TDD ecosystem validation

## Installation

```bash
npm install @bluefly/bfci-validation
```

## Usage

### Command Line

```bash
# Validate a Node.js project
llmcli validate nodejs ./path/to/project

# Validate GitLab components
llmcli validate gitlab ./path/to/components

# Validate a library
llmcli validate library ./path/to/library

# Validate Drupal TDD compliance
llmcli validate drupal-tdd ./path/to/module

# Validate API service
llmcli validate api ./path/to/service

# Validate TDD ecosystem
llmcli validate tdd ./path/to/project
```

### Programmatic Usage

```typescript
import { NodeJSValidator, GitLabValidator, LibraryValidator } from '@bluefly/bfci-validation';

// Validate a Node.js project
const nodeValidator = new NodeJSValidator();
const nodeResults = await nodeValidator.validate('./path/to/project');

// Validate GitLab components
const gitlabValidator = new GitLabValidator();
const gitlabResults = await gitlabValidator.validate('./path/to/components');

// Validate a library
const libraryValidator = new LibraryValidator();
const libraryResults = await libraryValidator.validate('./path/to/library');
```

## Validation Rules

### Node.js Projects
- package.json exists and is valid
- TypeScript configuration present
- Test configuration present
- CI configuration present
- Dependencies up to date
- No security vulnerabilities

### GitLab Components
- .gitlab-ci.yml present
- Component structure follows standards
- Documentation present
- Tests present
- Dependencies declared

### Libraries
- Package structure follows standards
- Documentation present
- Tests present
- Type definitions present
- CI configuration present

### Drupal TDD
- PHPUnit configuration present
- Test classes follow standards
- Coverage requirements met
- TDD workflow enforced
- CI integration present

### API Services
- OpenAPI specification present
- API documentation present
- Tests present
- Security measures implemented
- CI configuration present

## Development

### Setup

```bash
# Install dependencies
npm install

# Build the project
npm run build

# Run tests
npm test

# Run linting
npm run lint
```

### Adding New Validators

1. Create a new validator class in `src/validators/`
2. Implement the `Validator` interface
3. Add tests in `__tests__/validators/`
4. Update the CLI commands in `src/cli/`
5. Update documentation

## Contributing

1. Fork the repository
2. Create a feature branch
3. Make your changes
4. Add tests
5. Submit a pull request

## License

MIT 