---
id: core_documentation-deployment_system
---
# Deployment System

## Overview

The Deployment System is a core component of the LLM Platform AI Platform, providing a robust, scalable infrastructure for deploying AI models into production environments. It integrates with the Model Registry to manage the lifecycle of model deployments, from creation to scaling to monitoring.

This system enables users to deploy models from the registry to various environments (development, staging, production), with fine-grained control over scaling, resource allocation, and runtime configuration.

## Architecture

The Deployment System follows a layered architecture that integrates with multiple components of the LLM Platform AI Platform:

```
┌───────────────────────────────────────────────┐
│                 API Gateway                   │
│                   (bfapi)                     │
└───────────────────────┬───────────────────────┘
                        │
┌───────────────────────▼───────────────────────┐
│              Model Registry API               │
│           (Model & Deployment APIs)           │
└───────────────────────┬───────────────────────┘
                        │
┌───────────────────────▼───────────────────────┐
│             Deployment Controller             │
│       (Orchestrates deployment process)       │
└┬──────────────────────┬──────────────────────┬┘
 │                      │                      │
┌▼──────────────────┐  ┌▼──────────────────┐  ┌▼──────────────────┐
│ Resource Manager  │  │ Scaling Manager   │  │ Health Monitor    │
│ (K8s integration) │  │ (Auto-scaling)    │  │ (Status checks)   │
└───────────────────┘  └───────────────────┘  └───────────────────┘
          │                     │                      │
┌─────────▼─────────────────────▼──────────────────────▼─────────┐
│                    Kubernetes Cluster                           │
│                                                                 │
│  ┌─────────────┐   ┌─────────────┐   ┌─────────────┐           │
│  │ Model Pod 1 │   │ Model Pod 2 │   │ Model Pod n │   ...     │
│  └─────────────┘   └─────────────┘   └─────────────┘           │
└─────────────────────────────────────────────────────────────────┘
```

### Key Components

1. **API Gateway (bfapi)**: Entry point for all external requests, handling authentication, authorization, and routing.

2. **Model Registry API**: Manages model metadata, versions, and deployments.

3. **Deployment Controller**: Orchestrates the deployment process, managing the lifecycle of deployments.

4. **Resource Manager**: Handles resource allocation and Kubernetes integration.

5. **Scaling Manager**: Manages auto-scaling based on configured parameters and traffic.

6. **Health Monitor**: Continuously checks deployment health and status.

7. **Kubernetes Cluster**: Underlying infrastructure where model pods are deployed and managed.

## Deployment Workflow

```mermaid
sequenceDiagram
    participant User
    participant API as API Gateway
    participant Registry as Model Registry
    participant Controller as Deployment Controller
    participant K8s as Kubernetes
    
    User->>API: Create deployment request
    API->>Registry: Validate model & version
    Registry->>Controller: Initiate deployment
    Controller->>Registry: Update status (provisioning)
    Controller->>K8s: Create deployment resources
    K8s-->>Controller: Resources created
    Controller->>K8s: Deploy model containers
    K8s-->>Controller: Containers running
    Controller->>K8s: Configure networking & scaling
    K8s-->>Controller: Configuration applied
    Controller->>Registry: Update status (running)
    Registry-->>API: Return deployment details
    API-->>User: Deployment successful
```

### Workflow Steps

1. **Deployment Request**: User submits a deployment request via the API.
2. **Model Validation**: System validates the requested model and version exist.
3. **Provisioning**: Resources are allocated in the Kubernetes cluster.
4. **Deployment**: Model containers are deployed with specified configuration.
5. **Configuration**: Networking, scaling, and monitoring are configured.
6. **Activation**: The deployment is activated and made available for inference.
7. **Monitoring**: Continuous health checking and metric collection begins.

## Database Integration

The Deployment System uses the following database tables:

### Deployments Table

Stores deployment configurations and status information:

| Column | Type | Description |
|--------|------|-------------|
| id | INTEGER | Primary key for the deployment |
| name | VARCHAR(100) | Unique deployment name |
| model_id | INTEGER | Reference to the model being deployed |
| model_version_id | INTEGER | Reference to the specific model version |
| status | VARCHAR(20) | Current status: 'provisioning', 'running', 'error', 'stopped' |
| environment | VARCHAR(20) | Deployment environment: 'development', 'staging', 'production' |
| endpoint | VARCHAR(255) | Inference endpoint URL |
| scaling | JSON | Scaling configuration for the deployment |
| config | JSON | Model configuration parameters |
| created_at | TIMESTAMP | Creation timestamp |
| updated_at | TIMESTAMP | Last update timestamp |
| created_by | VARCHAR(100) | User who created the deployment |

### Relationships

- Each deployment is associated with a specific model (models.id)
- Each deployment uses a specific model version (model_versions.id)
- Deployments have a one-to-many relationship with model inference logs

## Configuration Options

The Deployment System supports a wide range of configuration options for fine-tuning model deployments:

### Basic Configuration

| Parameter | Type | Description | Default |
|-----------|------|-------------|---------|
| name | string | Unique deployment name | Required |
| model_id | integer | ID of the model to deploy | Required |
| model_version_id | integer | ID of the model version | Required |
| environment | string | Target environment | Required |

### Scaling Configuration

| Parameter | Type | Description | Default |
|-----------|------|-------------|---------|
| min_replicas | integer | Minimum number of running instances | 1 |
| max_replicas | integer | Maximum number of running instances | 5 |
| target_cpu_utilization | integer | CPU threshold for scaling (%) | 70 |
| target_memory_utilization | integer | Memory threshold for scaling (%) | 70 |
| startup_timeout_seconds | integer | Maximum time for pod startup | 300 |
| cooldown_period_seconds | integer | Time between scaling operations | 60 |

### Runtime Configuration

| Parameter | Type | Description | Default |
|-----------|------|-------------|---------|
| timeout_seconds | integer | Maximum request processing time | 60 |
| max_tokens | integer | Maximum tokens to generate | 1024 |
| temperature | number | Model temperature setting | 0.7 |
| top_p | number | Top-p sampling parameter | 1.0 |
| top_k | integer | Top-k sampling parameter | 40 |
| stop_sequences | array | Sequences that stop generation | [] |
| cache_ttl_seconds | integer | Cache TTL for identical requests | 3600 |

### Resource Configuration

| Parameter | Type | Description | Default |
|-----------|------|-------------|---------|
| cpu_request | string | Kubernetes CPU request | "1" |
| cpu_limit | string | Kubernetes CPU limit | "2" |
| memory_request | string | Kubernetes memory request | "2Gi" |
| memory_limit | string | Kubernetes memory limit | "4Gi" |
| gpu_request | string | Kubernetes GPU request | null |
| gpu_limit | string | Kubernetes GPU limit | null |
| disk_size_gb | integer | Storage allocation in GB | 20 |

### Network Configuration

| Parameter | Type | Description | Default |
|-----------|------|-------------|---------|
| enable_ssl | boolean | Enable HTTPS for endpoint | true |
| enable_compression | boolean | Enable response compression | true |
| connection_timeout_ms | integer | Connection timeout in ms | 5000 |
| keep_alive_timeout_ms | integer | Keep-alive timeout in ms | 120000 |
| rate_limit_per_minute | integer | Rate limit per minute | 100 |

## Kubernetes Integration

The Deployment System creates and manages the following Kubernetes resources:

1. **Deployments**: Manages the model container replicas
2. **Services**: Exposes the model endpoints
3. **HorizontalPodAutoscalers**: Manages automatic scaling
4. **ConfigMaps**: Stores model configuration
5. **Secrets**: Stores sensitive configuration (API keys, etc.)
6. **PersistentVolumes**: Stores model weights and cache data

### Example Kubernetes YAML

```yaml
# Deployment
apiVersion: apps/v1
kind: Deployment
metadata:
  name: model-gpt4-prod
  namespace: bluefly-ai
spec:
  replicas: 2
  selector:
    matchLabels:
      app: model-gpt4-prod
  template:
    metadata:
      labels:
        app: model-gpt4-prod
    spec:
      containers:
      - name: model-container
        image: bluefly/model-server:latest
        ports:
        - containerPort: 8080
        env:
        - name: MODEL_ID
          value: "1"
        - name: MODEL_VERSION_ID
          value: "3"
        - name: MAX_TOKENS
          value: "8192"
        resources:
          requests:
            cpu: "1"
            memory: "2Gi"
          limits:
            cpu: "2"
            memory: "4Gi"
        readinessProbe:
          httpGet:
            path: /health
            port: 8080
          initialDelaySeconds: 30
          periodSeconds: 10

# Service
apiVersion: v1
kind: Service
metadata:
  name: model-gpt4-prod-svc
  namespace: bluefly-ai
spec:
  selector:
    app: model-gpt4-prod
  ports:
  - port: 80
    targetPort: 8080
  type: ClusterIP

# HorizontalPodAutoscaler
apiVersion: autoscaling/v2
kind: HorizontalPodAutoscaler
metadata:
  name: model-gpt4-prod-hpa
  namespace: bluefly-ai
spec:
  scaleTargetRef:
    apiVersion: apps/v1
    kind: Deployment
    name: model-gpt4-prod
  minReplicas: 2
  maxReplicas: 10
  metrics:
  - type: Resource
    resource:
      name: cpu
      target:
        type: Utilization
        averageUtilization: 70
```

## Monitoring and Health Checks

The Deployment System includes comprehensive monitoring and health check capabilities:

### Health Check Endpoints

Each model deployment exposes the following health check endpoints:

- **/health**: Basic health check for readiness probes
- **/health/live**: Liveness check for Kubernetes probes
- **/health/ready**: Readiness check for Kubernetes probes
- **/health/detailed**: Detailed health status with metrics

### Monitoring Metrics

The system collects and exposes the following metrics:

| Metric | Type | Description |
|--------|------|-------------|
| request_count | Counter | Total number of inference requests |
| request_latency | Histogram | Request processing time distribution |
| token_count | Counter | Total tokens processed (input + output) |
| error_count | Counter | Total number of errors |
| model_load_time | Gauge | Time taken to load model in memory |
| memory_usage | Gauge | Memory usage of the model container |
| gpu_memory_usage | Gauge | GPU memory usage (if applicable) |
| cache_hit_ratio | Gauge | Ratio of cache hits to total requests |
| active_connections | Gauge | Number of active connections |
| queue_depth | Gauge | Number of requests in processing queue |

### Alerting

The system can be configured to generate alerts based on the following conditions:

- High error rate (>5% of requests)
- Excessive latency (p95 > 1000ms)
- Resource constraints (memory/CPU usage >90%)
- Deployment status changes to 'error'
- Repeated health check failures

## Error Handling and Recovery

The Deployment System implements robust error handling and recovery mechanisms:

### Error Types

| Error Type | Description | Recovery Strategy |
|------------|-------------|-------------------|
| ResourceAllocationError | Failed to allocate required resources | Retry with exponential backoff |
| ModelLoadError | Failed to load model weights | Attempt reload, verify model version |
| DeploymentTimeoutError | Deployment exceeded timeout period | Increase timeout, check resource constraints |
| ScalingError | Failed to scale deployment | Verify resource availability, check limits |
| NetworkError | Network connectivity issues | Retry with different networking config |
| ConfigurationError | Invalid configuration parameters | Fix configuration, retry deployment |
| HealthCheckError | Repeated health check failures | Restart pods, check underlying issues |

### Recovery Procedures

1. **Automatic Pod Restart**: Kubernetes automatically restarts failed pods
2. **Rolling Updates**: New deployments use rolling updates to maintain availability
3. **Fallback Versions**: Option to fallback to previous working version on failure
4. **Circuit Breaking**: Prevent cascading failures by circuit breaking
5. **Graceful Degradation**: Fall back to smaller models or reduced functionality
6. **Automatic Redeployment**: Auto-retry failed deployments with adjusted parameters

## Security Considerations

The Deployment System implements several security features:

### Authentication and Authorization

- JWT-based authentication for all API endpoints
- Role-based access control for deployment management
- Environment-specific permissions (e.g., separate production permissions)

### Network Security

- TLS encryption for all endpoint communication
- Internal network isolation for model containers
- Rate limiting to prevent abuse
- IP-based access controls for sensitive environments

### Data Security

- Secure storage of model weights and configuration
- No persistent storage of inference inputs/outputs
- Audit logging of all deployment operations
- Encryption of sensitive configuration values

### Compliance

- Automated security scanning of container images
- Regular vulnerability assessments
- Compliance with enterprise security standards
- Support for FedRAMP, HIPAA, and other regulatory frameworks

## Example Usage

### Creating a Deployment

```bash
# Example API request to create a deployment
curl -X POST https://api.bluefly.io/v1/deployments \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..." \
  -H "Content-Type: application/json" \
  -d '{
    "name": "production-gpt4",
    "model_id": 1,
    "model_version_id": 3,
    "environment": "production",
    "scaling": {
      "min_replicas": 2,
      "max_replicas": 10,
      "target_cpu_utilization": 70
    },
    "config": {
      "timeout_seconds": 60,
      "max_tokens": 8192,
      "temperature": 0.7
    }
  }'
```

### Checking Deployment Status

```bash
# Check the status of a deployment
curl -X GET https://api.bluefly.io/v1/deployments/42 \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."
```

### Running Inference

```bash
# Run inference on a deployed model
curl -X POST https://api.bluefly.io/v1/inference/deployments/production-gpt4 \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..." \
  -H "Content-Type: application/json" \
  -d '{
    "prompt": "Explain quantum computing in simple terms",
    "max_tokens": 500,
    "temperature": 0.7
  }'
```

### Updating Scaling Configuration

```bash
# Update the scaling configuration for a deployment
curl -X PATCH https://api.bluefly.io/v1/deployments/42/scaling \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..." \
  -H "Content-Type: application/json" \
  -d '{
    "min_replicas": 3,
    "max_replicas": 15,
    "target_cpu_utilization": 65
  }'
```

## Best Practices

### Deployment Strategy

1. **Start Small**: Begin with smaller replicas and scale up based on traffic
2. **Use Staging**: Test deployments in staging environment before production
3. **Canary Deployments**: Roll out changes to a small subset of users first
4. **Performance Testing**: Benchmark models with expected traffic patterns
5. **Monitoring Setup**: Establish monitoring before production deployment

### Resource Optimization

1. **Right-sizing**: Allocate resources based on model requirements
2. **Efficient Scaling**: Set appropriate scaling thresholds
3. **Caching Strategy**: Implement response caching for repeated queries
4. **Batch Processing**: Use batching for higher throughput
5. **GPU Sharing**: Configure GPU sharing for smaller models

### High Availability

1. **Multi-zone Deployment**: Deploy across multiple availability zones
2. **Redundancy**: Maintain minimum 2 replicas for critical models
3. **Graceful Degradation**: Plan fallback options for failure scenarios
4. **Connection Draining**: Properly handle connection termination
5. **Load Balancing**: Distribute traffic evenly across instances

## Troubleshooting

### Common Issues and Solutions

| Issue | Possible Causes | Solutions |
|-------|----------------|-----------|
| Deployment stuck in 'provisioning' | Resource constraints, network issues | Check cluster resources, verify network connectivity |
| High latency | Insufficient resources, large model size | Increase resources, optimize model, enable caching |
| Out of memory errors | Model too large for allocated memory | Increase memory limits, use model quantization |
| Failed health checks | Model initialization issues, networking problems | Check logs, verify model weights, check network policies |
| Scaling not working | Misconfigured autoscaler, metric issues | Verify HPA configuration, check metric collection |
| Connection timeouts | Network latency, overloaded instances | Increase timeout settings, scale up deployment |

### Diagnostic Commands

```bash
# Get detailed deployment status
kubectl describe deployment model-gpt4-prod -n bluefly-ai

# Check pod logs
kubectl logs -l app=model-gpt4-prod -n bluefly-ai

# Verify horizontal pod autoscaler
kubectl get hpa model-gpt4-prod-hpa -n bluefly-ai

# Check resource usage
kubectl top pods -l app=model-gpt4-prod -n bluefly-ai

# Exec into a pod for debugging
kubectl exec -it $(kubectl get pods -l app=model-gpt4-prod -n bluefly-ai -o name | head -1) -n bluefly-ai -- /bin/bash
```

## Integration with Other Systems

### Model Registry

The Deployment System integrates with the Model Registry to access model metadata, version information, and capabilities.

### Monitoring Stack

Deployments expose metrics that are collected by the monitoring stack (Prometheus, Grafana) for visualization and alerting.

### CI/CD Pipeline

Automated deployment can be triggered from CI/CD pipelines using the Deployment API.

### Logging System

All deployment operations and model logs are sent to the centralized logging system for analysis and troubleshooting.

## Future Enhancements

1. **Multi-cluster Deployment**: Support for deploying across multiple Kubernetes clusters
2. **GPU Optimization**: Advanced GPU sharing and optimization techniques
3. **Model Compression**: Automatic quantization and pruning for efficient deployment
4. **A/B Testing**: Built-in support for A/B testing between model versions
5. **Edge Deployment**: Support for deploying models to edge devices
6. **Serverless Inference**: Integration with serverless platforms for on-demand scaling
7. **Cost Optimization**: Advanced cost tracking and optimization recommendations

---

**Version**: 1.0.0  
**Last Updated**: May 31, 2025  
**Status**: Production-Ready
