---
id: database_schemas-llm_platform
---
# Llm Platform Database Schema

This document describes the database schema for llm platform in the LLM Platform AI Platform.

## Overview

The Llm Platform database stores information about LLM platform models, users, configurations, and interactions.

## Entity Relationship Diagram

```
┌───────────────────┐     ┌───────────────────┐
│                   │     │                   │
│  Models           │─┬──►│  ModelVersions    │
│                   │ │   │                   │
└───────────────────┘ │   └───────────────────┘
                      │      │
                      │      ▼
                      │   ┌───────────────────┐
                      └──►│                   │
                          │  Deployments      │
                          │                   │
                          └───────────────────┘
```

## Tables

### Models

Stores the core model information.

| Column | Type | Constraints | Description |
|--------|------|-------------|-------------|
| id | INTEGER | PRIMARY KEY, AUTO_INCREMENT | Unique identifier for the model |
| name | VARCHAR(100) | NOT NULL, UNIQUE | Model name |
| provider | VARCHAR(50) | NOT NULL | AI provider (e.g., OpenAI, Anthropic) |
| base_model | VARCHAR(100) | NOT NULL | Base model identifier |
| description | TEXT | NULL | Model description |
| capabilities | JSON | NULL | JSON object defining model capabilities |
| created_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP | Record creation timestamp |
| updated_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP | Last update timestamp |
| is_active | BOOLEAN | NOT NULL, DEFAULT TRUE | Whether the model is active |

#### Indexes

- PRIMARY KEY (id)
- UNIQUE INDEX idx_model_name (name)
- INDEX idx_provider (provider)
- INDEX idx_base_model (base_model)
- INDEX idx_is_active (is_active)

### ModelVersions

Stores different versions of each model.

| Column | Type | Constraints | Description |
|--------|------|-------------|-------------|
| id | INTEGER | PRIMARY KEY, AUTO_INCREMENT | Unique identifier for the model version |
| model_id | INTEGER | NOT NULL, FOREIGN KEY | Reference to models.id |
| version | VARCHAR(50) | NOT NULL | Version identifier |
| parameters | BIGINT | NULL | Number of parameters |
| context_window | INTEGER | NULL | Context window size in tokens |
| input_token_cost | DECIMAL(10,6) | NULL | Cost per input token |
| output_token_cost | DECIMAL(10,6) | NULL | Cost per output token |
| created_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP | Record creation timestamp |
| updated_at | TIMESTAMP | NOT NULL, DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP | Last update timestamp |
| is_default | BOOLEAN | NOT NULL, DEFAULT FALSE | Whether this is the default version |

#### Indexes

- PRIMARY KEY (id)
- FOREIGN KEY fk_model_version (model_id) REFERENCES models(id) ON DELETE CASCADE
- UNIQUE INDEX idx_model_version (model_id, version)
- INDEX idx_is_default (is_default)

## Relationships

- One-to-Many: Each Models record can have multiple ModelVersions records
- One-to-Many: Each Models record can have multiple Deployments records
- Many-to-One: Each Deployments record belongs to one Models record

## Data Flow

1. New models are added to the Models table
2. Model versions are tracked in the ModelVersions table with relationship to the parent model
3. Deployments are created referencing a specific model
4. User interactions query model capabilities and version information to determine optimal models for specific tasks

## Migration Considerations

- Add new columns with NULL constraint first, then populate data, then add NOT NULL constraints if needed
- Use transaction-safe operations for schema changes
- Schedule migrations during low-traffic periods
- Create backup before significant schema changes
- Consider using tools like Liquibase or Flyway for migration management

## Query Examples

```sql
-- Get all active models with their default versions
SELECT m.id, m.name, m.provider, m.base_model, mv.version, mv.parameters, mv.context_window
FROM models m
JOIN model_versions mv ON m.id = mv.model_id
WHERE m.is_active = TRUE AND mv.is_default = TRUE
ORDER BY m.provider, m.name;

-- Find models with specific capabilities
SELECT m.id, m.name, m.provider, m.base_model
FROM models m
WHERE JSON_EXTRACT(m.capabilities, '$.vision') = TRUE
  AND JSON_EXTRACT(m.capabilities, '$.streaming') = TRUE;
```

## Performance Considerations

- Keep indexed columns as small as possible
- Use appropriate data types (e.g., INT for IDs, not VARCHAR)
- Consider partitioning the model_versions table if it grows very large
- Use EXPLAIN to analyze query performance
- Set appropriate buffer pool size for InnoDB
- Consider using covering indexes for frequent queries
- Monitor slow queries and optimize as needed

## Security Considerations

- Implement proper access controls and least privilege
- Audit sensitive operations (model activation/deactivation)
- Use TLS for database connections
- Implement row-level security if needed for multi-tenant deployments
- Regularly update and patch database software
- Ensure proper backup and recovery procedures

## Related Documentation

- [Model Registry API](../api_documentation/model_registry_api)
- [Deployment System](../core_documentation/deployment_system)

## References

- [Database Design Best Practices](https://example.com/db-best-practices)
- [MySQL Performance Tuning](https://example.com/mysql-performance)
- [Security Guidelines](https://example.com/security-guidelines)

---

**Version**: 1.0.0  
**Last Updated**: May 31, 2025  
**Status**: Draft
