-- LLM Platform Database Query Examples
-- Version: 1.0.0
-- Example queries for working with the Models and ModelVersions tables

-- 1. Get all active models with their default versions
SELECT 
    m.id, 
    m.name, 
    m.provider, 
    m.base_model, 
    mv.version, 
    mv.parameters, 
    mv.context_window,
    mv.input_token_cost,
    mv.output_token_cost
FROM 
    models m
JOIN 
    model_versions mv ON m.id = mv.model_id
WHERE 
    m.is_active = TRUE 
    AND mv.is_default = TRUE
ORDER BY 
    m.provider, m.name;

-- 2. Find models with specific capabilities (vision and streaming)
SELECT 
    m.id, 
    m.name, 
    m.provider, 
    m.base_model
FROM 
    models m
WHERE 
    JSON_EXTRACT(m.capabilities, '$.vision') = TRUE
    AND JSON_EXTRACT(m.capabilities, '$.streaming') = TRUE;

-- 3. Find the most cost-effective model for a given provider
SELECT 
    m.id, 
    m.name, 
    m.provider, 
    mv.input_token_cost,
    mv.output_token_cost
FROM 
    models m
JOIN 
    model_versions mv ON m.id = mv.model_id
WHERE 
    m.is_active = TRUE 
    AND mv.is_default = TRUE
    AND m.provider = 'OpenAI'
ORDER BY 
    mv.output_token_cost ASC
LIMIT 1;

-- 4. Find models with the largest context window
SELECT 
    m.id, 
    m.name, 
    m.provider, 
    mv.context_window
FROM 
    models m
JOIN 
    model_versions mv ON m.id = mv.model_id
WHERE 
    m.is_active = TRUE 
    AND mv.is_default = TRUE
ORDER BY 
    mv.context_window DESC
LIMIT 5;

-- 5. Count models by provider
SELECT 
    provider, 
    COUNT(*) as model_count
FROM 
    models
WHERE 
    is_active = TRUE
GROUP BY 
    provider
ORDER BY 
    model_count DESC;

-- 6. Find all versions of a specific model
SELECT 
    mv.id,
    mv.version,
    mv.parameters,
    mv.context_window,
    mv.input_token_cost,
    mv.output_token_cost,
    mv.created_at,
    mv.is_default
FROM 
    model_versions mv
JOIN 
    models m ON mv.model_id = m.id
WHERE 
    m.name = 'claude-3-sonnet'
ORDER BY 
    mv.created_at DESC;

-- 7. Find models that support function calling but not vision
SELECT 
    m.id, 
    m.name, 
    m.provider, 
    m.base_model
FROM 
    models m
WHERE 
    JSON_EXTRACT(m.capabilities, '$.function_calling') = TRUE
    AND (JSON_EXTRACT(m.capabilities, '$.vision') = FALSE OR JSON_EXTRACT(m.capabilities, '$.vision') IS NULL);

-- 8. Get models sorted by parameter count (largest first)
SELECT 
    m.id, 
    m.name, 
    m.provider, 
    mv.parameters
FROM 
    models m
JOIN 
    model_versions mv ON m.id = mv.model_id
WHERE 
    m.is_active = TRUE 
    AND mv.is_default = TRUE
    AND mv.parameters IS NOT NULL
ORDER BY 
    mv.parameters DESC;

-- 9. Update a model's capabilities (example: adding a new capability)
UPDATE models
SET 
    capabilities = JSON_SET(capabilities, '$.new_feature', TRUE),
    updated_at = CURRENT_TIMESTAMP
WHERE 
    name = 'gpt-4o';

-- 10. Find the newest models by creation date
SELECT 
    m.id, 
    m.name, 
    m.provider, 
    m.created_at
FROM 
    models m
ORDER BY 
    m.created_at DESC
LIMIT 5;