---
id: deployment-kubernetes
---
# Kubernetes Deployment Guide

<!-- title: Kubernetes Deployment Guide -->
<!-- description: Practical guide for Kubernetes deployment in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical Kubernetes deployment for the LLM Platform Platform, including configuration, deployment, scaling, and monitoring.

## Base Configuration

### Namespace

```yaml
# kubernetes/base/namespace.yaml
apiVersion: v1
kind: Namespace
metadata:
  name: bluefly
  labels:
    name: bluefly
    environment: production
```

### ConfigMap

```yaml
# kubernetes/base/configmap.yaml
apiVersion: v1
kind: ConfigMap
metadata:
  name: bfapi-config
  namespace: bluefly
data:
  NODE_ENV: "production"
  API_URL: "https://api.bluefly.io"
  MCP_URL: "https://mcp.bluefly.io"
  LOG_LEVEL: "info"
  CORS_ORIGINS: "https://bluefly.io,https://admin.bluefly.io"
  RATE_LIMIT_WINDOW: "60"
  RATE_LIMIT_MAX: "100"
```

### Secrets

```yaml
# kubernetes/base/secrets.yaml
apiVersion: v1
kind: Secret
metadata:
  name: bfapi-secrets
  namespace: bluefly
type: Opaque
data:
  DATABASE_URL: base64-encoded-url
  REDIS_URL: base64-encoded-url
  JWT_SECRET: base64-encoded-secret
  OPENAI_API_KEY: base64-encoded-key
  ANTHROPIC_API_KEY: base64-encoded-key
  ENCRYPTION_KEY: base64-encoded-key
```

## Application Deployment

### Deployment

```yaml
# kubernetes/base/deployment.yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: bfapi
  namespace: bluefly
spec:
  replicas: 3
  selector:
    matchLabels:
      app: bfapi
  strategy:
    type: RollingUpdate
    rollingUpdate:
      maxSurge: 1
      maxUnavailable: 0
  template:
    metadata:
      labels:
        app: bfapi
      annotations:
        prometheus.io/scrape: "true"
        prometheus.io/port: "3000"
        prometheus.io/path: "/metrics"
    spec:
      containers:
      - name: bfapi
        image: registry.bluefly.io/bfapi:latest
        imagePullPolicy: Always
        ports:
        - containerPort: 3000
          name: http
        envFrom:
        - configMapRef:
            name: bfapi-config
        - secretRef:
            name: bfapi-secrets
        resources:
          requests:
            cpu: 100m
            memory: 256Mi
          limits:
            cpu: 500m
            memory: 512Mi
        livenessProbe:
          httpGet:
            path: /health
            port: http
          initialDelaySeconds: 30
          periodSeconds: 10
          timeoutSeconds: 5
          failureThreshold: 3
        readinessProbe:
          httpGet:
            path: /ready
            port: http
          initialDelaySeconds: 5
          periodSeconds: 5
          timeoutSeconds: 3
          failureThreshold: 3
        startupProbe:
          httpGet:
            path: /health
            port: http
          initialDelaySeconds: 5
          periodSeconds: 10
          timeoutSeconds: 5
          failureThreshold: 30
        volumeMounts:
        - name: tmp
          mountPath: /tmp
        - name: logs
          mountPath: /var/log/bfapi
      volumes:
      - name: tmp
        emptyDir: {}
      - name: logs
        persistentVolumeClaim:
          claimName: bfapi-logs
```

### Service

```yaml
# kubernetes/base/service.yaml
apiVersion: v1
kind: Service
metadata:
  name: bfapi
  namespace: bluefly
spec:
  type: ClusterIP
  ports:
  - port: 80
    targetPort: http
    protocol: TCP
    name: http
  selector:
    app: bfapi
```

### Ingress

```yaml
# kubernetes/base/ingress.yaml
apiVersion: networking.k8s.io/v1
kind: Ingress
metadata:
  name: bfapi
  namespace: bluefly
  annotations:
    kubernetes.io/ingress.class: nginx
    cert-manager.io/cluster-issuer: letsencrypt-prod
    nginx.ingress.kubernetes.io/ssl-redirect: "true"
    nginx.ingress.kubernetes.io/force-ssl-redirect: "true"
    nginx.ingress.kubernetes.io/proxy-body-size: "10m"
    nginx.ingress.kubernetes.io/proxy-read-timeout: "60"
    nginx.ingress.kubernetes.io/proxy-send-timeout: "60"
spec:
  tls:
  - hosts:
    - api.bluefly.io
    secretName: bfapi-tls
  rules:
  - host: api.bluefly.io
    http:
      paths:
      - path: /
        pathType: Prefix
        backend:
          service:
            name: bfapi
            port:
              number: 80
```

## Database Deployment

### PostgreSQL

```yaml
# kubernetes/database/postgresql.yaml
apiVersion: apps/v1
kind: StatefulSet
metadata:
  name: postgresql
  namespace: bluefly
spec:
  serviceName: postgresql
  replicas: 1
  selector:
    matchLabels:
      app: postgresql
  template:
    metadata:
      labels:
        app: postgresql
    spec:
      containers:
      - name: postgresql
        image: postgres:14-alpine
        ports:
        - containerPort: 5432
          name: postgresql
        env:
        - name: POSTGRES_USER
          valueFrom:
            secretKeyRef:
              name: postgresql-secrets
              key: username
        - name: POSTGRES_PASSWORD
          valueFrom:
            secretKeyRef:
              name: postgresql-secrets
              key: password
        - name: POSTGRES_DB
          value: bluefly
        volumeMounts:
        - name: data
          mountPath: /var/lib/postgresql/data
        resources:
          requests:
            cpu: 500m
            memory: 1Gi
          limits:
            cpu: 1000m
            memory: 2Gi
        livenessProbe:
          exec:
            command:
            - pg_isready
            - -U
            - postgres
          initialDelaySeconds: 30
          periodSeconds: 10
        readinessProbe:
          exec:
            command:
            - pg_isready
            - -U
            - postgres
          initialDelaySeconds: 5
          periodSeconds: 5
      volumes:
      - name: data
        persistentVolumeClaim:
          claimName: postgresql-data
```

### Redis

```yaml
# kubernetes/database/redis.yaml
apiVersion: apps/v1
kind: StatefulSet
metadata:
  name: redis
  namespace: bluefly
spec:
  serviceName: redis
  replicas: 3
  selector:
    matchLabels:
      app: redis
  template:
    metadata:
      labels:
        app: redis
    spec:
      containers:
      - name: redis
        image: redis:7-alpine
        ports:
        - containerPort: 6379
          name: redis
        command:
        - redis-server
        - /etc/redis/redis.conf
        volumeMounts:
        - name: config
          mountPath: /etc/redis
        - name: data
          mountPath: /data
        resources:
          requests:
            cpu: 100m
            memory: 256Mi
          limits:
            cpu: 200m
            memory: 512Mi
        livenessProbe:
          exec:
            command:
            - redis-cli
            - ping
          initialDelaySeconds: 30
          periodSeconds: 10
        readinessProbe:
          exec:
            command:
            - redis-cli
            - ping
          initialDelaySeconds: 5
          periodSeconds: 5
      volumes:
      - name: config
        configMap:
          name: redis-config
      - name: data
        persistentVolumeClaim:
          claimName: redis-data
```

## Monitoring Deployment

### Prometheus

```yaml
# kubernetes/monitoring/prometheus.yaml
apiVersion: monitoring.coreos.com/v1
kind: ServiceMonitor
metadata:
  name: bfapi
  namespace: bluefly
spec:
  selector:
    matchLabels:
      app: bfapi
  endpoints:
  - port: http
    path: /metrics
    interval: 15s
    scrapeTimeout: 10s
```

### Grafana

```yaml
# kubernetes/monitoring/grafana.yaml
apiVersion: v1
kind: ConfigMap
metadata:
  name: grafana-dashboards
  namespace: bluefly
data:
  bfapi.json: |
    {
      "annotations": {
        "list": []
      },
      "editable": true,
      "fiscalYearStartMonth": 0,
      "graphTooltip": 0,
      "links": [],
      "liveNow": false,
      "panels": [
        {
          "datasource": {
            "type": "prometheus",
            "uid": "prometheus"
          },
          "fieldConfig": {
            "defaults": {
              "color": {
                "mode": "palette-classic"
              },
              "custom": {
                "axisCenteredZero": false,
                "axisColorMode": "text",
                "axisLabel": "",
                "axisPlacement": "auto",
                "barAlignment": 0,
                "drawStyle": "line",
                "fillOpacity": 10,
                "gradientMode": "none",
                "hideFrom": {
                  "legend": false,
                  "tooltip": false,
                  "viz": false
                },
                "lineInterpolation": "linear",
                "lineWidth": 1,
                "pointSize": 5,
                "scaleDistribution": {
                  "type": "linear"
                },
                "showPoints": "never",
                "spanNulls": false,
                "stacking": {
                  "group": "A",
                  "mode": "none"
                },
                "thresholdsStyle": {
                  "mode": "off"
                }
              },
              "mappings": [],
              "thresholds": {
                "mode": "absolute",
                "steps": [
                  {
                    "color": "green",
                    "value": null
                  },
                  {
                    "color": "red",
                    "value": 80
                  }
                ]
              },
              "unit": "short"
            },
            "overrides": []
          },
          "gridPos": {
            "h": 8,
            "w": 12,
            "x": 0,
            "y": 0
          },
          "id": 1,
          "options": {
            "legend": {
              "calcs": [],
              "displayMode": "list",
              "placement": "bottom",
              "showLegend": true
            },
            "tooltip": {
              "mode": "single",
              "sort": "none"
            }
          },
          "targets": [
            {
              "datasource": {
                "type": "prometheus",
                "uid": "prometheus"
              },
              "expr": "rate(bf_agent_executions_total[5m])",
              "refId": "A"
            }
          ],
          "title": "Agent Executions",
          "type": "timeseries"
        }
      ],
      "refresh": "5s",
      "schemaVersion": 38,
      "style": "dark",
      "tags": [],
      "templating": {
        "list": []
      },
      "time": {
        "from": "now-6h",
        "to": "now"
      },
      "timepicker": {},
      "timezone": "",
      "title": "BFAPI Dashboard",
      "uid": "bfapi",
      "version": 1,
      "weekStart": ""
    }
```

## Deployment Process

### CI/CD Pipeline

```yaml
# .gitlab-ci.yml
stages:
  - test
  - build
  - deploy

variables:
  DOCKER_REGISTRY: registry.bluefly.io
  DOCKER_IMAGE: $DOCKER_REGISTRY/bfapi
  KUBE_NAMESPACE: bluefly

test:
  stage: test
  image: node:18-alpine
  script:
    - npm ci
    - npm run test
    - npm run lint

build:
  stage: build
  image: docker:20.10
  services:
    - docker:20.10-dind
  script:
    - docker build -t $DOCKER_IMAGE:$CI_COMMIT_SHA .
    - docker push $DOCKER_IMAGE:$CI_COMMIT_SHA
    - docker tag $DOCKER_IMAGE:$CI_COMMIT_SHA $DOCKER_IMAGE:latest
    - docker push $DOCKER_IMAGE:latest

deploy:
  stage: deploy
  image: dtzar/helm-kubectl:3.8.0
  script:
    - helm upgrade --install bfapi ./helm/bfapi
      --set image.tag=$CI_COMMIT_SHA
      --namespace $KUBE_NAMESPACE
      --wait
      --timeout 5m
  environment:
    name: production
  only:
    - main
```

### Helm Chart

```yaml
# helm/bfapi/values.yaml
replicaCount: 3

image:
  repository: registry.bluefly.io/bfapi
  tag: latest
  pullPolicy: Always

service:
  type: ClusterIP
  port: 80

ingress:
  enabled: true
  className: nginx
  hosts:
    - host: api.bluefly.io
      paths:
        - path: /
          pathType: Prefix
  tls:
    - secretName: bfapi-tls
      hosts:
        - api.bluefly.io

resources:
  requests:
    cpu: 100m
    memory: 256Mi
  limits:
    cpu: 500m
    memory: 512Mi

env:
  - name: NODE_ENV
    value: production
  - name: API_URL
    value: https://api.bluefly.io
  - name: MCP_URL
    value: https://mcp.bluefly.io

secrets:
  - name: DATABASE_URL
    secretName: bfapi-secrets
    key: database-url
  - name: REDIS_URL
    secretName: bfapi-secrets
    key: redis-url
  - name: JWT_SECRET
    secretName: bfapi-secrets
    key: jwt-secret
```

## Best Practices

1. **Deployment**
   - Use rolling updates
   - Set resource limits
   - Configure health checks
   - Use secrets management

2. **Scaling**
   - Use horizontal scaling
   - Configure auto-scaling
   - Monitor resources
   - Set up alerts

3. **Monitoring**
   - Collect metrics
   - Set up dashboards
   - Configure alerts
   - Log aggregation

4. **Security**
   - Use network policies
   - Configure RBAC
   - Manage secrets
   - Regular updates

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Development Guide](../development/README)
- [Security Guide](../security/README)
- [Operations Guide](../operations/README)
- [Testing Guide](../testing/README) 
