---
id: development-memory
---
# Memory Systems

<!-- title: Memory Systems -->
<!-- description: Developer guide for memory systems, caching, and vector storage in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

This guide covers the memory systems used in the LLM Platform Platform, including caching, vector storage, and memory management for AI agents.

## Architecture

### Core Components

```typescript
// Memory Interface
interface Memory {
  type: MemoryType;
  config: MemoryConfig;
  
  initialize(): Promise<void>;
  shutdown(): Promise<void>;
  get(key: string): Promise<any>;
  set(key: string, value: any, ttl?: number): Promise<void>;
  delete(key: string): Promise<void>;
  clear(): Promise<void>;
}

// Memory Types
enum MemoryType {
  CACHE = 'cache',
  VECTOR = 'vector',
  DOCUMENT = 'document',
  CONVERSATION = 'conversation'
}

// Memory Configuration
interface MemoryConfig {
  provider: string;
  connection: {
    host?: string;
    port?: number;
    url?: string;
    credentials?: {
      username?: string;
      password?: string;
      apiKey?: string;
    };
  };
  options: {
    ttl?: number;
    maxSize?: number;
    compression?: boolean;
    encryption?: boolean;
    [key: string]: any;
  };
}

// Vector Store Interface
interface VectorStore extends Memory {
  add(vectors: Vector[]): Promise<void>;
  search(query: Vector, options: SearchOptions): Promise<SearchResult[]>;
  delete(ids: string[]): Promise<void>;
}

// Document Store Interface
interface DocumentStore extends Memory {
  add(documents: Document[]): Promise<void>;
  search(query: SearchQuery): Promise<Document[]>;
  update(id: string, document: Partial<Document>): Promise<void>;
  delete(ids: string[]): Promise<void>;
}
```

### Directory Structure

```
bfmemory/
├── src/
│   ├── stores/           # Memory store implementations
│   │   ├── cache/       # Cache stores (Redis, Memcached)
│   │   ├── vector/      # Vector stores (Pinecone, Weaviate)
│   │   ├── document/    # Document stores (Elasticsearch)
│   │   └── hybrid/      # Hybrid memory systems
│   ├── providers/       # Provider integrations
│   │   ├── redis/      # Redis provider
│   │   ├── pinecone/   # Pinecone provider
│   │   └── elastic/    # Elasticsearch provider
│   ├── utils/          # Utility functions
│   │   ├── vector/     # Vector operations
│   │   ├── cache/      # Cache utilities
│   │   └── security/   # Security utilities
│   └── types/          # Type definitions
├── config/             # Memory configurations
├── tests/             # Test suite
└── docs/              # Documentation
```

## Memory Store Implementation

### Base Memory Store

```typescript
// src/stores/base-memory.ts
import { Memory, MemoryType, MemoryConfig } from '../types';
import { Logger } from '../utils/logger';
import { Metrics } from '../utils/metrics';

export abstract class BaseMemory implements Memory {
  protected logger: Logger;
  protected metrics: Metrics;
  
  constructor(
    public readonly type: MemoryType,
    protected config: MemoryConfig
  ) {
    this.logger = new Logger(`memory:${type}`);
    this.metrics = new Metrics(`memory:${type}`);
  }
  
  abstract initialize(): Promise<void>;
  abstract shutdown(): Promise<void>;
  
  async get(key: string): Promise<any> {
    const start = Date.now();
    try {
      const value = await this.getInternal(key);
      this.metrics.recordLatency('get', Date.now() - start);
      return value;
    } catch (error) {
      this.logger.error('Failed to get value', { key, error });
      throw error;
    }
  }
  
  async set(key: string, value: any, ttl?: number): Promise<void> {
    const start = Date.now();
    try {
      await this.setInternal(key, value, ttl);
      this.metrics.recordLatency('set', Date.now() - start);
    } catch (error) {
      this.logger.error('Failed to set value', { key, error });
      throw error;
    }
  }
  
  async delete(key: string): Promise<void> {
    const start = Date.now();
    try {
      await this.deleteInternal(key);
      this.metrics.recordLatency('delete', Date.now() - start);
    } catch (error) {
      this.logger.error('Failed to delete value', { key, error });
      throw error;
    }
  }
  
  async clear(): Promise<void> {
    const start = Date.now();
    try {
      await this.clearInternal();
      this.metrics.recordLatency('clear', Date.now() - start);
    } catch (error) {
      this.logger.error('Failed to clear store', { error });
      throw error;
    }
  }
  
  protected abstract getInternal(key: string): Promise<any>;
  protected abstract setInternal(key: string, value: any, ttl?: number): Promise<void>;
  protected abstract deleteInternal(key: string): Promise<void>;
  protected abstract clearInternal(): Promise<void>;
}
```

### Redis Cache Store

```typescript
// src/stores/cache/redis-store.ts
import { BaseMemory } from '../base-memory';
import { RedisProvider } from '../../providers/redis';
import { MemoryType, MemoryConfig } from '../../types';
import { serialize, deserialize } from '../../utils/serialization';

export class RedisStore extends BaseMemory {
  private provider: RedisProvider;
  
  constructor(config: MemoryConfig) {
    super(MemoryType.CACHE, config);
    this.provider = new RedisProvider(config.connection);
  }
  
  async initialize(): Promise<void> {
    await this.provider.connect();
    this.logger.info('Redis store initialized');
  }
  
  async shutdown(): Promise<void> {
    await this.provider.disconnect();
    this.logger.info('Redis store shutdown');
  }
  
  protected async getInternal(key: string): Promise<any> {
    const value = await this.provider.get(key);
    return value ? deserialize(value) : null;
  }
  
  protected async setInternal(key: string, value: any, ttl?: number): Promise<void> {
    const serialized = serialize(value);
    if (ttl) {
      await this.provider.setex(key, ttl, serialized);
    } else {
      await this.provider.set(key, serialized);
    }
  }
  
  protected async deleteInternal(key: string): Promise<void> {
    await this.provider.del(key);
  }
  
  protected async clearInternal(): Promise<void> {
    await this.provider.flushdb();
  }
}
```

### Pinecone Vector Store

```typescript
// src/stores/vector/pinecone-store.ts
import { BaseMemory } from '../base-memory';
import { PineconeProvider } from '../../providers/pinecone';
import { MemoryType, MemoryConfig, Vector, SearchOptions, SearchResult } from '../../types';
import { VectorStore } from '../types';

export class PineconeStore extends BaseMemory implements VectorStore {
  private provider: PineconeProvider;
  
  constructor(config: MemoryConfig) {
    super(MemoryType.VECTOR, config);
    this.provider = new PineconeProvider(config.connection);
  }
  
  async initialize(): Promise<void> {
    await this.provider.initialize();
    this.logger.info('Pinecone store initialized');
  }
  
  async shutdown(): Promise<void> {
    await this.provider.cleanup();
    this.logger.info('Pinecone store shutdown');
  }
  
  async add(vectors: Vector[]): Promise<void> {
    const start = Date.now();
    try {
      await this.provider.upsert(vectors);
      this.metrics.recordLatency('add', Date.now() - start);
    } catch (error) {
      this.logger.error('Failed to add vectors', { error });
      throw error;
    }
  }
  
  async search(query: Vector, options: SearchOptions): Promise<SearchResult[]> {
    const start = Date.now();
    try {
      const results = await this.provider.query({
        vector: query.values,
        topK: options.limit,
        filter: options.filter,
        namespace: options.namespace
      });
      
      this.metrics.recordLatency('search', Date.now() - start);
      return results.map(result => ({
        id: result.id,
        score: result.score,
        metadata: result.metadata
      }));
    } catch (error) {
      this.logger.error('Failed to search vectors', { error });
      throw error;
    }
  }
  
  async delete(ids: string[]): Promise<void> {
    const start = Date.now();
    try {
      await this.provider.delete(ids);
      this.metrics.recordLatency('delete', Date.now() - start);
    } catch (error) {
      this.logger.error('Failed to delete vectors', { error });
      throw error;
    }
  }
  
  protected async getInternal(key: string): Promise<any> {
    const vector = await this.provider.fetch([key]);
    return vector[0] || null;
  }
  
  protected async setInternal(key: string, value: Vector, ttl?: number): Promise<void> {
    await this.provider.upsert([{
      id: key,
      values: value.values,
      metadata: value.metadata
    }]);
  }
  
  protected async deleteInternal(key: string): Promise<void> {
    await this.provider.delete([key]);
  }
  
  protected async clearInternal(): Promise<void> {
    await this.provider.deleteAll();
  }
}
```

## Memory Management

### Memory Registry

```typescript
// src/registry/memory-registry.ts
import { Memory, MemoryType } from '../types';

export class MemoryRegistry {
  private stores: Map<string, Memory> = new Map();
  
  register(name: string, store: Memory): void {
    if (this.stores.has(name)) {
      throw new Error(`Memory store ${name} already registered`);
    }
    
    this.stores.set(name, store);
  }
  
  unregister(name: string): void {
    if (!this.stores.has(name)) {
      throw new Error(`Memory store ${name} not found`);
    }
    
    this.stores.delete(name);
  }
  
  get(name: string): Memory {
    const store = this.stores.get(name);
    if (!store) {
      throw new Error(`Memory store ${name} not found`);
    }
    
    return store;
  }
  
  getByType(type: MemoryType): Memory[] {
    return Array.from(this.stores.values())
      .filter(store => store.type === type);
  }
  
  async initialize(): Promise<void> {
    for (const store of this.stores.values()) {
      await store.initialize();
    }
  }
  
  async shutdown(): Promise<void> {
    for (const store of this.stores.values()) {
      await store.shutdown();
    }
  }
}
```

### Memory Manager

```typescript
// src/manager/memory-manager.ts
import { MemoryRegistry } from '../registry/memory-registry';
import { memoryConfigs } from '../../config/memory';
import { RedisStore } from '../stores/cache/redis-store';
import { PineconeStore } from '../stores/vector/pinecone-store';

export class MemoryManager {
  private registry: MemoryRegistry;
  
  constructor() {
    this.registry = new MemoryRegistry();
  }
  
  async initialize(): Promise<void> {
    // Initialize cache store
    const cache = new RedisStore(memoryConfigs.cache);
    await cache.initialize();
    this.registry.register('cache', cache);
    
    // Initialize vector store
    const vector = new PineconeStore(memoryConfigs.vector);
    await vector.initialize();
    this.registry.register('vector', vector);
    
    // Initialize other stores...
  }
  
  async shutdown(): Promise<void> {
    await this.registry.shutdown();
  }
  
  getRegistry(): MemoryRegistry {
    return this.registry;
  }
}
```

## Configuration

```typescript
// config/memory.ts
import { MemoryConfig } from '../src/types';

export const memoryConfigs: Record<string, MemoryConfig> = {
  cache: {
    provider: 'redis',
    connection: {
      host: process.env.REDIS_HOST,
      port: parseInt(process.env.REDIS_PORT || '6379'),
      credentials: {
        password: process.env.REDIS_PASSWORD
      }
    },
    options: {
      ttl: 3600,
      maxSize: 1000,
      compression: true,
      encryption: true
    }
  },
  
  vector: {
    provider: 'pinecone',
    connection: {
      apiKey: process.env.PINECONE_API_KEY,
      environment: process.env.PINECONE_ENVIRONMENT
    },
    options: {
      namespace: 'default',
      dimension: 1536,
      metric: 'cosine'
    }
  },
  
  document: {
    provider: 'elasticsearch',
    connection: {
      url: process.env.ELASTICSEARCH_URL,
      credentials: {
        username: process.env.ELASTICSEARCH_USERNAME,
        password: process.env.ELASTICSEARCH_PASSWORD
      }
    },
    options: {
      index: 'documents',
      shards: 3,
      replicas: 1
    }
  }
};
```

## Usage Examples

### Caching Model Responses

```typescript
// src/utils/model-cache.ts
import { MemoryRegistry } from '../registry/memory-registry';
import { ModelInput, ModelOutput } from '../types';

export class ModelCache {
  constructor(private registry: MemoryRegistry) {}
  
  async getCachedResponse(input: ModelInput): Promise<ModelOutput | null> {
    const cache = this.registry.get('cache');
    const key = this.generateCacheKey(input);
    return cache.get(key);
  }
  
  async cacheResponse(input: ModelInput, output: ModelOutput): Promise<void> {
    const cache = this.registry.get('cache');
    const key = this.generateCacheKey(input);
    await cache.set(key, output, 3600); // Cache for 1 hour
  }
  
  private generateCacheKey(input: ModelInput): string {
    return `model:${input.model}:${JSON.stringify(input.parameters)}`;
  }
}
```

### Vector Search

```typescript
// src/utils/vector-search.ts
import { MemoryRegistry } from '../registry/memory-registry';
import { Vector, SearchOptions } from '../types';

export class VectorSearch {
  constructor(private registry: MemoryRegistry) {}
  
  async searchSimilar(query: Vector, options: SearchOptions): Promise<SearchResult[]> {
    const vector = this.registry.get('vector') as VectorStore;
    return vector.search(query, {
      ...options,
      filter: {
        ...options.filter,
        status: 'active'
      }
    });
  }
  
  async addToIndex(vectors: Vector[]): Promise<void> {
    const vector = this.registry.get('vector') as VectorStore;
    await vector.add(vectors);
  }
}
```

## Testing

### Memory Store Tests

```typescript
// tests/stores/redis-store.test.ts
import { RedisStore } from '../../src/stores/cache/redis-store';
import { memoryConfigs } from '../../config/memory';

describe('RedisStore', () => {
  let store: RedisStore;
  
  beforeAll(async () => {
    store = new RedisStore(memoryConfigs.cache);
    await store.initialize();
  });
  
  afterAll(async () => {
    await store.shutdown();
  });
  
  it('should set and get values', async () => {
    const key = 'test:key';
    const value = { data: 'test' };
    
    await store.set(key, value);
    const result = await store.get(key);
    
    expect(result).toEqual(value);
  });
  
  it('should handle TTL', async () => {
    const key = 'test:ttl';
    const value = { data: 'test' };
    
    await store.set(key, value, 1); // 1 second TTL
    await new Promise(resolve => setTimeout(resolve, 1100));
    
    const result = await store.get(key);
    expect(result).toBeNull();
  });
});
```

### Vector Store Tests

```typescript
// tests/stores/pinecone-store.test.ts
import { PineconeStore } from '../../src/stores/vector/pinecone-store';
import { memoryConfigs } from '../../config/memory';

describe('PineconeStore', () => {
  let store: PineconeStore;
  
  beforeAll(async () => {
    store = new PineconeStore(memoryConfigs.vector);
    await store.initialize();
  });
  
  afterAll(async () => {
    await store.shutdown();
  });
  
  it('should add and search vectors', async () => {
    const vectors = [
      {
        id: 'test:1',
        values: [0.1, 0.2, 0.3],
        metadata: { type: 'test' }
      }
    ];
    
    await store.add(vectors);
    
    const results = await store.search(
      { values: [0.1, 0.2, 0.3] },
      { limit: 1 }
    );
    
    expect(results).toHaveLength(1);
    expect(results[0].id).toBe('test:1');
  });
});
```

## Best Practices

1. **Memory Management**
   - Use appropriate store types
   - Implement proper error handling
   - Monitor memory usage
   - Implement cleanup strategies
   - Use compression when appropriate

2. **Caching**
   - Set appropriate TTLs
   - Implement cache invalidation
   - Use cache prefixes
   - Monitor cache hit rates
   - Handle cache failures

3. **Vector Storage**
   - Choose appropriate dimensions
   - Implement proper indexing
   - Use efficient search algorithms
   - Monitor vector quality
   - Handle vector updates

4. **Security**
   - Encrypt sensitive data
   - Implement access control
   - Secure connections
   - Monitor access patterns
   - Regular security audits

## Related Documentation

- [Model Integration](./models)
- [Testing Guide](./testing)
- [Deployment Guide](../operations/deployment)
- [Security Guide](../operations/security) 
