---
id: development-npm-publishing-and-ci
---
# NPM Publishing & GitLab CI Guide

## NPM Publishing Summary

### Successfully Published
-  **@bluefly/native-ai**: Published version 0.2.0
-  **@bluefly/api-client**: Updated from 0.1.4 to 1.0.2
  - Fixed duplicate GraphQLError export
  - Added missing recordLog method to TelemetryClient
  - Fixed CircuitBreaker export

### Packages Needing Fixes Before Publishing

#### @bluefly/secure-project
- **Local version**: 1.0.0
- **npm version**: Not published
- **Issues**:
  - Multiple TypeScript compilation errors
  - Missing type definitions
  - Issues with module imports
- **Fix needed**: Fix TypeScript errors and install missing type definitions

#### @bluefly/llm
- **Local version**: 1.1.0
- **npm version**: 1.0.0
- **Issues**:
  - Missing dependency: `@bluefly/cli/config/ConfigManager`
  - Multiple TypeScript errors
  - Build script failures
- **Fix needed**: Fix dependencies and TypeScript errors

#### @bluefly/rfp
- **Local version**: 1.0.1
- **npm version**: 0.1.0
- **Issues**:
  - ESLint configuration error in `.eslintrc.js`
  - Module not defined error
- **Fix needed**: Fix ESLint configuration to work with ESM

## How to Fix and Publish

For each package, follow these steps:

1. Fix the identified issues (dependency, TypeScript, ESLint, etc.).
2. Run the validation script: `npm run validate:publish`.
3. If successful, publish using: `npm publish --access public`.

For packages with persistent build issues, you might consider:
- Temporarily modifying the `prepublishOnly` script to bypass problematic checks.
- Using the `--ignore-scripts` flag (with caution): `npm publish --access public --ignore-scripts`.
- Publishing a temporary version and planning a proper fix for the next release.

## Publishing Command Reference

```bash
# Basic publishing command
npm publish --access public

# Skip running scripts (use with caution!)
npm publish --access public --ignore-scripts

# Publish a specific tag
npm publish --access public --tag beta

# Dry run to test publishing
npm publish --access public --dry-run
```

## Dependencies to Install

Here are the missing dependencies identified:
```bash
npm install --save-dev @bluefly/cli
npm install --save-dev @types/uuid @types/js-yaml
```

---

## GitLab CI Configuration (Standardized Node.js Template)

Below is a standardized GitLab CI configuration (using the npm_golden_ci_package component) for Node.js projects. (You can copy this into your project's `.gitlab-ci.yml` file.)

```yaml
# Standardized GitLab CI Configuration for Node.js Projects
# Using the npm_golden_ci_package component

include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/nodejs/npm_golden_ci_package@main
    inputs:
      node_version: "20"
      npm_token: "$NPM_TOKEN"  # Set in CI/CD variables
      release_branch: "main"
      package_manager: "npm"  # Options: npm, yarn, pnpm
      public_package: true
      enable_security_scan: true
      enable_build_cache: true
      min_test_coverage: 90
      enable_dependency_audit: true
      enable_bundle_analysis: true
      enable_api_docs: true
      enable_performance_benchmarks: true
      enable_memory_leak_checks: false
      enable_type_checking: true
      enable_code_quality: true
      enable_commit_lint: true
      enable_changelog: true

variables:
  # Project-specific variables (customize as needed)
  NODE_ENV: "test"
  
  # If using Docker
  # DOCKER_IMAGE: "$CI_REGISTRY_IMAGE/[YOUR_SERVICE_NAME]"
  
  # If using databases
  # DATABASE_URL: "postgresql://postgres:postgres@postgres:5432/test_db"
  # REDIS_URL: "redis://redis:6379"
  
  # API testing
  # API_PORT: "3000"
  # API_HOST: "0.0.0.0"
  
  # Performance metrics
  # MAX_RESPONSE_TIME_MS: "300"
  # MIN_THROUGHPUT_RPS: "50"
  # MAX_ERROR_RATE: "0.02"

# Optional: Add custom jobs specific to your project

# Example OpenAPI validation (uncomment if needed)
# openapi_validation:
#   stage: validate
#   image: node:$[[ inputs.node_version ]]-alpine
#   script:
#     - npm ci
#     - npx @stoplight/spectral-cli lint openapi.yaml
#   artifacts:
#     paths:
#       - openapi-validation-report.json
#   rules:
#     - if: $CI_PIPELINE_SOURCE == "merge_request_event" || $CI_COMMIT_BRANCH == "main"
#       changes:
#         paths:
#           - openapi.yaml
#           - openapi/*.yaml

# Example performance testing (uncomment if needed)
# api_performance_test:
#   stage: benchmark
#   image: node:$[[ inputs.node_version ]]-alpine
#   script:
#     - npm ci
#     - npm run test:performance
#   artifacts:
#     paths:
#       - performance-results.json
#     reports:
#       metrics: performance-results.json
#   rules:
#     - if: $CI_COMMIT_BRANCH == "main"

# Example deployment (uncomment if needed)
# deploy_staging:
#   stage: deploy
#   image: alpine:latest
#   script:
#     - echo "Deploying to staging..."
#   environment:
#     name: staging
#     url: https://staging.example.com
#   rules:
#     - if: $CI_COMMIT_BRANCH == "staging"
#
# deploy_production:
#   stage: deploy
#   image: alpine:latest
#   script:
#     - echo "Deploying to production..."
#   environment:
#     name: production
#     url: https://example.com
#   rules:
#     - if: $CI_COMMIT_BRANCH == "main"
#       when: manual

```

---

*This guide is part of the LLM Platform AI Platform documentation hub (bfdocs). For further details, refer to the [bfdocs](bfdocs/) project.* 
