---
id: development-tdd-coverage
---
# Test Coverage Best Practices Guide

## Introduction

This guide establishes standards and best practices for achieving and maintaining high test coverage across all LLM Platform projects. It aligns with our strict Test-Driven Development (TDD) methodology and provides practical guidance for developers.

## Core Testing Principles

### 1. Test-First Development

All development in the LLM Platform must follow strict TDD:

1. **RED phase**: Write failing tests that define expected behavior
2. **GREEN phase**: Implement minimal code to make tests pass
3. **REFACTOR phase**: Optimize and clean up code while keeping tests passing

```typescript
// Example: RED phase - write the test first
describe('KnowledgeGraphManager', () => {
  test('should create a relationship between two nodes', () => {
    const manager = new KnowledgeGraphManager();
    const node1 = manager.createNode('concept', 'caching');
    const node2 = manager.createNode('technology', 'redis');
    
    const relationship = manager.createRelationship(node1.id, node2.id, 'implements');
    
    expect(relationship).toBeDefined();
    expect(relationship.source).toBe(node1.id);
    expect(relationship.target).toBe(node2.id);
    expect(relationship.type).toBe('implements');
  });
});

// GREEN phase - implement the code
class KnowledgeGraphManager {
  nodes = new Map();
  relationships = new Map();
  
  createNode(type, label) {
    const id = `node-${Date.now()}-${Math.random().toString(36).substr(2, 9)}`;
    const node = { id, type, label };
    this.nodes.set(id, node);
    return node;
  }
  
  createRelationship(sourceId, targetId, type) {
    const id = `rel-${Date.now()}-${Math.random().toString(36).substr(2, 9)}`;
    const relationship = { id, source: sourceId, target: targetId, type };
    this.relationships.set(id, relationship);
    return relationship;
  }
}

// REFACTOR phase - optimize while tests still pass
class KnowledgeGraphManager {
  nodes = new Map();
  relationships = new Map();
  
  createNode(type, label, properties = {}) {
    const id = this._generateId('node');
    const node = { id, type, label, properties };
    this.nodes.set(id, node);
    return node;
  }
  
  createRelationship(sourceId, targetId, type, properties = {}) {
    if (!this.nodes.has(sourceId) || !this.nodes.has(targetId)) {
      throw new Error('Source or target node does not exist');
    }
    
    const id = this._generateId('rel');
    const relationship = { 
      id, 
      source: sourceId, 
      target: targetId, 
      type,
      properties 
    };
    this.relationships.set(id, relationship);
    return relationship;
  }
  
  _generateId(prefix) {
    return `${prefix}-${Date.now()}-${Math.random().toString(36).substr(2, 9)}`;
  }
}
```

### 2. Coverage Requirements

All projects must maintain the following minimum coverage thresholds:

| Component Type | Statements | Branches | Functions | Lines |
|----------------|------------|----------|-----------|-------|
| Core Services  | 95%        | 90%      | 95%       | 95%   |
| APIs           | 90%        | 85%      | 90%       | 90%   |
| UI Components  | 90%        | 80%      | 90%       | 90%   |
| Utilities      | 95%        | 90%      | 95%       | 95%   |
| Overall        | 90%        | 85%      | 90%       | 90%   |

### 3. Test Types by Project

Each project requires specific types of tests:

| Project         | Unit Tests | Integration Tests | API Tests | UI Tests | E2E Tests |
|-----------------|------------|-------------------|-----------|----------|-----------|
| bfapi           |          |                 |         | ❌       |         |
| bfllm           |          |                 |         | ❌       | ❌        |
| llm-mcp           |          |                 |         | ❌       | ❌        |
| bfrfp           |          |                 |         | ❌       | ❌        |
| llm-ui            |          |                 | ❌        |        | ❌        |
| bfcicomponents  |          |                 | ❌        | ❌       |         |
| llm_platform_recipe |     |                 |         |        |         |
| native_aijs     |          |                 |         | ❌       | ❌        |

## Test Organization Standards

### Directory Structure

```
project-root/
├── __tests__/               # Root-level tests (project-wide)
│   ├── unit/                # Unit tests
│   ├── integration/         # Integration tests
│   └── e2e/                 # End-to-end tests
├── src/
│   ├── component1/
│   │   ├── __tests__/       # Component-specific tests
│   │   │   ├── unit/        
│   │   │   └── integration/ 
│   │   └── ...
│   └── ...
├── tests/                   # For PHP projects (e.g., llm_platform_recipe)
│   ├── src/
│   │   ├── Unit/
│   │   ├── Kernel/
│   │   └── Functional/
```

### Naming Conventions

- TypeScript/JavaScript: `*.test.ts`, `*.test.tsx`, `*.test.js`
- PHP: `*Test.php`
- Test helpers: `*.test.helpers.ts`
- Test fixtures: `__fixtures__/*.ts`
- Mock data: `__mocks__/*.ts`

## Testing Best Practices by Language

### TypeScript/JavaScript (Jest)

#### 1. Basic Test Structure

```typescript
describe('ComponentName', () => {
  // Setup and teardown
  beforeAll(() => {
    // One-time setup before all tests
  });
  
  afterAll(() => {
    // One-time cleanup after all tests
  });
  
  beforeEach(() => {
    // Setup before each test
  });
  
  afterEach(() => {
    // Cleanup after each test
  });
  
  // Test cases
  test('should perform specific action', () => {
    // Arrange
    const component = new Component();
    
    // Act
    const result = component.method();
    
    // Assert
    expect(result).toBe(expectedValue);
  });
  
  // Grouping related tests
  describe('when specific condition', () => {
    test('should behave in certain way', () => {
      // ...
    });
  });
});
```

#### 2. Mocking Dependencies

```typescript
// Mocking modules
jest.mock('../../services/apiService', () => ({
  fetchData: jest.fn().mockResolvedValue({ data: 'mocked data' }),
  saveData: jest.fn().mockResolvedValue({ success: true })
}));

// Mocking class implementation
jest.mock('../../services/Logger', () => {
  return {
    Logger: jest.fn().mockImplementation(() => ({
      info: jest.fn(),
      error: jest.fn(),
      warn: jest.fn(),
      debug: jest.fn()
    }))
  };
});

// Spying on methods
test('should call logger when error occurs', () => {
  const logger = new Logger();
  const spy = jest.spyOn(logger, 'error');
  
  try {
    throwError();
  } catch (err) {
    logger.error(err);
  }
  
  expect(spy).toHaveBeenCalled();
});
```

#### 3. Testing Asynchronous Code

```typescript
// Using async/await
test('should fetch data asynchronously', async () => {
  const data = await service.fetchData();
  expect(data).toEqual(expectedData);
});

// Testing promises
test('should handle promise rejection', () => {
  return expect(service.fetchWithError()).rejects.toThrow('Error message');
});

// Using done callback (avoid if possible, prefer async/await)
test('should call callback when complete', (done) => {
  service.processWithCallback(result => {
    expect(result).toBe(expectedValue);
    done();
  });
});
```

#### 4. Testing React Components

```tsx
// Basic component rendering
test('should render component', () => {
  render(<Button>Click me</Button>);
  expect(screen.getByText('Click me')).toBeInTheDocument();
});

// Testing user interactions
test('should handle click events', () => {
  const handleClick = jest.fn();
  render(<Button onClick={handleClick}>Click me</Button>);
  
  fireEvent.click(screen.getByText('Click me'));
  
  expect(handleClick).toHaveBeenCalledTimes(1);
});

// Testing state changes
test('should update state on user interaction', () => {
  render(<Counter />);
  
  const incrementButton = screen.getByText('+');
  fireEvent.click(incrementButton);
  
  expect(screen.getByText('Count: 1')).toBeInTheDocument();
});

// Testing async state updates (important: use act)
test('should load data and update state', async () => {
  // Mock fetch response
  jest.spyOn(global, 'fetch').mockResolvedValue({
    json: jest.fn().mockResolvedValue([{ id: 1, name: 'Item 1' }]),
    ok: true
  } as any);
  
  // Always wrap renders with async operations in act
  await act(async () => {
    render(<DataList />);
  });
  
  // Wait for data to load
  await waitFor(() => {
    expect(screen.getByText('Item 1')).toBeInTheDocument();
  });
});
```

### PHP (PHPUnit)

#### 1. Basic Test Structure

```php
<?php

namespace Drupal\Tests\llm_platform_recipe\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\llm_platform_recipe\Service\ExampleService;

/**
 * @coversDefaultClass \Drupal\llm_platform_recipe\Service\ExampleService
 * @group llm_platform_recipe
 */
class ExampleServiceTest extends UnitTestCase {

  /**
   * The example service under test.
   *
   * @var \Drupal\llm_platform_recipe\Service\ExampleService
   */
  protected $service;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->service = new ExampleService();
  }

  /**
   * @covers ::processData
   */
  public function testProcessData() {
    $input = 'test data';
    $result = $this->service->processData($input);
    $this->assertEquals('PROCESSED: test data', $result);
  }

  /**
   * @covers ::validateInput
   * @dataProvider provideValidationData
   */
  public function testValidateInput($input, $expected) {
    $result = $this->service->validateInput($input);
    $this->assertEquals($expected, $result);
  }

  /**
   * Data provider for testValidateInput.
   */
  public function provideValidationData() {
    return [
      'valid input' => ['valid data', TRUE],
      'empty input' => ['', FALSE],
      'invalid input' => ['invalid*data', FALSE],
    ];
  }

}
```

#### 2. Mocking Drupal Services

```php
<?php

namespace Drupal\Tests\llm_platform_recipe\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\llm_platform_recipe\Service\ConfigBasedService;

/**
 * @coversDefaultClass \Drupal\llm_platform_recipe\Service\ConfigBasedService
 * @group llm_platform_recipe
 */
class ConfigBasedServiceTest extends UnitTestCase {

  /**
   * The config factory mock.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $configFactory;

  /**
   * The service under test.
   *
   * @var \Drupal\llm_platform_recipe\Service\ConfigBasedService
   */
  protected $service;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    
    // Create a mock config factory
    $this->configFactory = $this->createMock(ConfigFactoryInterface::class);
    
    // Create a mock config object
    $config = $this->createMock(ImmutableConfig::class);
    
    // Configure the config mock to return specific values
    $config->expects($this->any())
      ->method('get')
      ->willReturnMap([
        ['api.key', 'test_api_key'],
        ['timeout', 30],
        ['features.advanced', TRUE],
      ]);
    
    // Configure the config factory to return our config mock
    $this->configFactory->expects($this->any())
      ->method('get')
      ->with('llm_platform_recipe.settings')
      ->willReturn($config);
    
    // Instantiate the service with the mocked dependencies
    $this->service = new ConfigBasedService($this->configFactory);
  }

  /**
   * @covers ::getApiKey
   */
  public function testGetApiKey() {
    $this->assertEquals('test_api_key', $this->service->getApiKey());
  }

  /**
   * @covers ::isFeatureEnabled
   */
  public function testIsFeatureEnabled() {
    $this->assertTrue($this->service->isFeatureEnabled('advanced'));
    $this->assertFalse($this->service->isFeatureEnabled('nonexistent'));
  }

}
```

#### 3. Testing Controllers and Forms

```php
<?php

namespace Drupal\Tests\llm_platform_recipe\Unit\Controller;

use Drupal\Tests\UnitTestCase;
use Drupal\llm_platform_recipe\Controller\DashboardController;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Symfony\Component\HttpFoundation\Request;

/**
 * @coversDefaultClass \Drupal\llm_platform_recipe\Controller\DashboardController
 * @group llm_platform_recipe
 */
class DashboardControllerTest extends UnitTestCase {

  /**
   * The controller to test.
   *
   * @var \Drupal\llm_platform_recipe\Controller\DashboardController
   */
  protected $controller;

  /**
   * The mocked renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    
    // Create a container builder
    $container = new ContainerBuilder();
    
    // Set up the string translation service
    $translator = $this->getStringTranslationStub();
    $container->set('string_translation', $translator);
    
    // Set up a renderer mock
    $this->renderer = $this->createMock('\Drupal\Core\Render\RendererInterface');
    $container->set('renderer', $this->renderer);
    
    // Set the container
    \Drupal::setContainer($container);
    
    // Create the controller
    $this->controller = new DashboardController();
  }

  /**
   * @covers ::dashboard
   */
  public function testDashboard() {
    $request = new Request();
    $result = $this->controller->dashboard($request);
    
    // Test the build array structure
    $this->assertIsArray($result);
    $this->assertArrayHasKey('#theme', $result);
    $this->assertEquals('llm_platform_recipe_dashboard', $result['#theme']);
    
    // Ensure it contains the expected sections
    $this->assertArrayHasKey('#models', $result);
    $this->assertArrayHasKey('#metrics', $result);
    $this->assertArrayHasKey('#attached', $result);
    
    // Check for attached libraries
    $this->assertArrayHasKey('library', $result['#attached']);
    $this->assertContains('llm_platform_recipe/dashboard', $result['#attached']['library']);
  }

}
```

## Testing Tools & Configuration

### Jest Configuration (TypeScript Projects)

```javascript
// jest.config.js
module.exports = {
  preset: 'ts-jest',
  testEnvironment: 'node', // Use 'jsdom' for UI components
  moduleFileExtensions: ['ts', 'tsx', 'js', 'json'],
  transform: {
    '^.+\\.(ts|tsx)$': 'ts-jest'
  },
  testMatch: [
    '**/__tests__/**/*.test.(ts|tsx|js)',
    '**/?(*.)+(spec|test).(ts|tsx|js)'
  ],
  coverageDirectory: 'coverage',
  collectCoverageFrom: [
    'src/**/*.(ts|tsx|js)',
    '!src/**/*.d.ts',
    '!src/**/index.ts',
    '!src/types/**',
    '!**/node_modules/**'
  ],
  coverageReporters: ['text', 'lcov', 'clover'],
  coverageThreshold: {
    global: {
      branches: 85,
      functions: 90,
      lines: 90,
      statements: 90
    }
  }
};
```

### PHPUnit Configuration (PHP Projects)

```xml
<!-- phpunit.xml -->
<?xml version="1.0" encoding="UTF-8"?>
<phpunit bootstrap="vendor/autoload.php"
         colors="true"
         verbose="true">
  <php>
    <ini name="error_reporting" value="E_ALL"/>
    <ini name="display_errors" value="On"/>
    <ini name="memory_limit" value="-1"/>
  </php>
  <testsuites>
    <testsuite name="Unit">
      <directory>./tests/src/Unit</directory>
    </testsuite>
    <testsuite name="Kernel">
      <directory>./tests/src/Kernel</directory>
    </testsuite>
    <testsuite name="Functional">
      <directory>./tests/src/Functional</directory>
    </testsuite>
  </testsuites>
  <coverage processUncoveredFiles="true">
    <include>
      <directory suffix=".php">./src</directory>
    </include>
    <exclude>
      <directory>./vendor</directory>
      <directory>./tests</directory>
    </exclude>
    <report>
      <html outputDirectory="coverage/html"/>
      <clover outputFile="coverage/clover.xml"/>
      <text outputFile="php://stdout" showUncoveredFiles="true" showOnlySummary="false"/>
    </report>
  </coverage>
</phpunit>
```

### CI/CD Pipeline Configuration

```yaml
# Generic .gitlab-ci.yml coverage enforcement
test:coverage:
  stage: test
  script:
    - npm ci # or composer install for PHP projects
    - npm run test:coverage:enforce
  coverage: '/Lines\s*:\s*([0-9.]+)%/'
  artifacts:
    paths:
      - coverage/
    reports:
      coverage_report:
        coverage_format: cobertura
        path: coverage/clover.xml
  only:
    - merge_requests
  rules:
    - if: $CI_MERGE_REQUEST_ID
```

## Testing Strategy By Component Type

### 1. API & Service Testing

```typescript
// Example API controller test
describe('ModelController', () => {
  let controller: ModelController;
  let modelService: ModelService;
  
  beforeEach(() => {
    // Mock dependencies
    modelService = {
      listModels: jest.fn().mockResolvedValue([
        { id: 'model1', name: 'Model 1' },
        { id: 'model2', name: 'Model 2' }
      ]),
      getModel: jest.fn().mockImplementation((id) => {
        return Promise.resolve({ id, name: `Model ${id.split('model')[1]}` });
      }),
      createModel: jest.fn().mockImplementation((data) => {
        return Promise.resolve({ id: 'new-model', ...data });
      })
    } as any;
    
    controller = new ModelController(modelService);
  });
  
  describe('GET /models', () => {
    test('should return all models', async () => {
      const req = {} as Request;
      const res = {
        json: jest.fn(),
        status: jest.fn().mockReturnThis()
      } as any;
      
      await controller.listModels(req, res);
      
      expect(modelService.listModels).toHaveBeenCalled();
      expect(res.json).toHaveBeenCalledWith([
        { id: 'model1', name: 'Model 1' },
        { id: 'model2', name: 'Model 2' }
      ]);
    });
    
    test('should handle errors', async () => {
      const req = {} as Request;
      const res = {
        json: jest.fn(),
        status: jest.fn().mockReturnThis()
      } as any;
      
      // Mock service to throw error
      modelService.listModels = jest.fn().mockRejectedValue(
        new Error('Database error')
      );
      
      await controller.listModels(req, res);
      
      expect(res.status).toHaveBeenCalledWith(500);
      expect(res.json).toHaveBeenCalledWith({ 
        error: 'Failed to fetch models' 
      });
    });
  });
});
```

### 2. React Component Testing

```tsx
// Example React component test with proper act() usage
describe('UserProfile', () => {
  test('should load and display user data', async () => {
    // Mock fetch user data
    jest.spyOn(global, 'fetch').mockResolvedValue({
      ok: true,
      json: jest.fn().mockResolvedValue({
        name: 'John Doe',
        email: 'john@example.com',
        role: 'Admin'
      })
    } as any);
    
    // Render component with act() to properly handle state updates
    await act(async () => {
      render(<UserProfile userId="123" />);
    });
    
    // Verify the component renders user data correctly
    expect(screen.getByText('John Doe')).toBeInTheDocument();
    expect(screen.getByText('john@example.com')).toBeInTheDocument();
    expect(screen.getByText('Admin')).toBeInTheDocument();
    
    // Clean up
    jest.restoreAllMocks();
  });
  
  test('should handle loading state', () => {
    // Mock slow fetch
    jest.spyOn(global, 'fetch').mockImplementation(
      () => new Promise(resolve => setTimeout(resolve, 1000))
    );
    
    render(<UserProfile userId="123" />);
    
    // Loading state should be displayed
    expect(screen.getByText('Loading...')).toBeInTheDocument();
    
    // Clean up
    jest.restoreAllMocks();
  });
  
  test('should handle error state', async () => {
    // Mock fetch failure
    jest.spyOn(global, 'fetch').mockRejectedValue(
      new Error('Failed to load user')
    );
    
    await act(async () => {
      render(<UserProfile userId="123" />);
    });
    
    // Error message should be displayed
    expect(screen.getByText('Error loading user data')).toBeInTheDocument();
    
    // Clean up
    jest.restoreAllMocks();
  });
});
```

### 3. ML Model & Inference Testing

```typescript
// Example ML component test
describe('ModelInferenceService', () => {
  let service: ModelInferenceService;
  
  beforeEach(() => {
    // Mock TensorFlow or other ML dependencies
    jest.mock('@tensorflow/tfjs', () => ({
      loadLayersModel: jest.fn().mockResolvedValue({
        predict: jest.fn().mockReturnValue({
          dataSync: jest.fn().mockReturnValue(new Float32Array([0.1, 0.2, 0.7])),
          dispose: jest.fn()
        }),
        dispose: jest.fn()
      }),
      dispose: jest.fn(),
      tensor: jest.fn().mockReturnValue({
        expandDims: jest.fn().mockReturnThis(),
        dispose: jest.fn()
      })
    }));
    
    service = new ModelInferenceService();
  });
  
  test('should load model', async () => {
    await service.loadModel('model-path');
    
    expect(require('@tensorflow/tfjs').loadLayersModel)
      .toHaveBeenCalledWith('model-path');
    expect(service.isModelLoaded()).toBe(true);
  });
  
  test('should make predictions', async () => {
    await service.loadModel('model-path');
    
    const result = await service.predict('This is a test input');
    
    expect(result).toEqual([0.1, 0.2, 0.7]);
    expect(require('@tensorflow/tfjs').tensor).toHaveBeenCalled();
  });
  
  test('should handle prediction errors', async () => {
    await service.loadModel('model-path');
    
    // Mock prediction failure
    require('@tensorflow/tfjs').loadLayersModel().predict = 
      jest.fn().mockImplementation(() => {
        throw new Error('Prediction failed');
      });
    
    await expect(service.predict('test input'))
      .rejects.toThrow('Prediction failed');
  });
  
  test('should properly dispose resources', async () => {
    await service.loadModel('model-path');
    
    await service.predict('test input');
    await service.unloadModel();
    
    const tfjs = require('@tensorflow/tfjs');
    const mockModel = tfjs.loadLayersModel();
    
    expect(mockModel.dispose).toHaveBeenCalled();
    expect(service.isModelLoaded()).toBe(false);
  });
});
```

### 4. Integration Testing

```typescript
// Example integration test between services
describe('RFP Generation Integration', () => {
  // Set up test environment
  let llmService: LLMService;
  let documentService: DocumentService;
  let rfpGenerator: RFPGenerator;
  
  beforeAll(async () => {
    // Create real services (not mocks) for integration testing
    llmService = new LLMService({
      endpoint: 'http://localhost:3001/api/llm',
      timeout: 1000
    });
    
    documentService = new DocumentService({
      storageBasePath: './test-data/documents'
    });
    
    // Use real services for integration
    rfpGenerator = new RFPGenerator(llmService, documentService);
    
    // Prepare test data
    await documentService.saveDocument('req1', {
      content: 'Test RFP requirements document',
      metadata: { type: 'requirements' }
    });
  });
  
  afterAll(async () => {
    // Clean up test data
    await documentService.deleteDocument('req1');
  });
  
  test('should generate RFP response from requirements document', async () => {
    // Configure mock for external service call only
    jest.spyOn(llmService, 'generateContent').mockImplementation(async (prompt) => {
      // Simulate LLM behavior but controlled for testing
      if (prompt.includes('executive summary')) {
        return 'This is a mock executive summary';
      } else if (prompt.includes('technical approach')) {
        return 'This is a mock technical approach';
      }
      return 'Generic response';
    });
    
    // Exercise the full workflow
    const result = await rfpGenerator.generateFromDocument('req1', {
      sections: ['executiveSummary', 'technicalApproach']
    });
    
    // Verify integration results
    expect(result.executiveSummary).toBe('This is a mock executive summary');
    expect(result.technicalApproach).toBe('This is a mock technical approach');
    
    // Verify service interaction (integration point)
    expect(llmService.generateContent).toHaveBeenCalledTimes(2);
  });
});
```

## Common Testing Pitfalls & Solutions

### 1. Async Testing Issues

#### Problem
```typescript
// Missing await with act() causes warnings
test('should update state', () => {
  render(<AsyncComponent />);
  // Warning: React state update happening during render
});
```

#### Solution
```typescript
// Proper async testing with act()
test('should update state', async () => {
  await act(async () => {
    render(<AsyncComponent />);
  });
  
  // Now you can make assertions after state updates
});
```

### 2. Mock Cleanup

#### Problem
```typescript
// Leaking mocks between tests
beforeEach(() => {
  jest.mock('./service');
});

// Later tests affected by this mock
```

#### Solution
```typescript
// Proper mock cleanup
afterEach(() => {
  jest.resetAllMocks();
});

// For module mocks
afterAll(() => {
  jest.restoreAllMocks();
});
```

### 3. Coverage Blind Spots

#### Problem
- Testing the "happy path" only
- Missing edge cases and error handling

#### Solution
```typescript
// Test error paths and edge cases
test('should handle empty input', async () => {
  await expect(service.process('')).rejects.toThrow('Input required');
});

test('should handle null values', async () => {
  const result = await service.process(null);
  expect(result).toBeNull();
});

test('should handle maximum size limits', async () => {
  const largeInput = 'a'.repeat(10000);
  await expect(service.process(largeInput)).rejects.toThrow('Exceeds max size');
});
```

### 4. Database Testing

#### Problem
- Tests that rely on database state are brittle
- Tests can interfere with each other

#### Solution
```typescript
// Use in-memory database for tests
beforeAll(async () => {
  await initializeInMemoryDatabase();
});

// Reset database between tests
beforeEach(async () => {
  await clearDatabase();
  await seedTestData();
});

// Close connections after tests
afterAll(async () => {
  await closeDatabase();
});
```

## CI/CD Integration

### 1. Pull Request Validation

Configure CI to run tests and enforce coverage on all PRs:

```yaml
# .gitlab-ci.yml
stages:
  - test
  - build

test:
  stage: test
  script:
    - npm ci
    - npm run lint
    - npm run test:coverage:enforce
  coverage: '/Lines\s*:\s*([0-9.]+)%/'
  artifacts:
    paths:
      - coverage/
    reports:
      coverage_report:
        coverage_format: cobertura
        path: coverage/clover.xml
  only:
    - merge_requests
```

### 2. Coverage History Tracking

```yaml
# .gitlab-ci.yml
coverage:report:
  stage: test
  script:
    - npm ci
    - npm run test:coverage
    - node scripts/compare-coverage.js $CI_COMMIT_REF_NAME
  artifacts:
    paths:
      - coverage-report/
  only:
    - merge_requests
```

### 3. Scheduled Test Quality Checks

```yaml
# .gitlab-ci.yml
test:quality:
  stage: test
  script:
    - npm ci
    - npm run test:mutation
    - npm run test:complexity
  artifacts:
    paths:
      - quality-report/
  only:
    - schedules
```

## Advanced Testing Techniques

### 1. Property-Based Testing

```typescript
import fc from 'fast-check';

describe('String utilities', () => {
  test('reverse function should be reversible', () => {
    fc.assert(
      fc.property(fc.string(), str => {
        // Any string reversed twice should equal the original string
        expect(reverse(reverse(str))).toBe(str);
      })
    );
  });
});
```

### 2. Snapshot Testing

```typescript
describe('Button component', () => {
  test('primary button matches snapshot', () => {
    const { container } = render(<Button variant="primary">Click me</Button>);
    expect(container).toMatchSnapshot();
  });
  
  test('secondary button matches snapshot', () => {
    const { container } = render(<Button variant="secondary">Click me</Button>);
    expect(container).toMatchSnapshot();
  });
});
```

### 3. Visual Regression Testing

```typescript
describe('Dashboard UI', () => {
  test('dashboard renders correctly', async () => {
    render(<Dashboard />);
    
    // Wait for data to load
    await screen.findByText('Recent Activity');
    
    // Take screenshot and compare with baseline
    expect(await takeScreenshot('dashboard')).toMatchImageSnapshot();
  });
});
```

### 4. Load & Performance Testing

```typescript
describe('API performance', () => {
  test('should handle 100 concurrent requests', async () => {
    const startTime = Date.now();
    
    // Generate 100 concurrent requests
    const promises = Array(100).fill(0).map(() => 
      api.get('/endpoint')
    );
    
    // Wait for all to complete
    const results = await Promise.all(promises);
    const endTime = Date.now();
    
    // Check performance
    const totalTime = endTime - startTime;
    const averageTime = totalTime / 100;
    
    expect(averageTime).toBeLessThan(200); // Average < 200ms
    expect(results.every(r => r.status === 200)).toBe(true);
  });
});
```

## Documentation Standards

### Test Coverage Reports

All projects should generate and maintain standardized coverage reports:

1. HTML coverage report for developer reference
2. LCOV format for CI/CD integration
3. Cobertura XML for GitLab CI integration

Coverage reports should be generated on CI runs and attached as artifacts.

### Test Documentation

Each test file should include:

1. Brief description of what's being tested
2. Any special setup requirements
3. External dependencies or mocks needed

Example:
```typescript
/**
 * Tests for the ModelManager class that handles LLM model registration, 
 * loading, and inference requests.
 * 
 * Dependencies:
 * - TensorFlow.js (mocked)
 * - ModelRegistry service (mocked)
 * 
 * Test data:
 * - Sample model configs in __fixtures__/models.json
 */
describe('ModelManager', () => {
  // ...
});
```

## Conclusion

Following these best practices will ensure high test coverage, maintainable tests, and reliable software across all LLM Platform projects. Remember that tests are a critical part of our codebase and should be treated with the same care and quality standards as production code.

The test coverage roadmap for each project, along with detailed test coverage plans, provides a clear path to achieving our coverage goals. Together, we'll build a robust, well-tested platform that delivers enterprise-grade reliability.
