---
id: development-testing
---
# Testing Framework

<!-- title: Testing Framework -->
<!-- description: Developer guide for testing strategies, test types, and CI integration in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

This guide covers the testing framework used in the LLM Platform Platform, including unit testing, integration testing, end-to-end testing, and CI/CD integration.

## Architecture

### Core Components

```typescript
// Test Suite Interface
interface TestSuite {
  name: string;
  description: string;
  tests: Test[];
  setup?(): Promise<void>;
  teardown?(): Promise<void>;
}

// Test Interface
interface Test {
  name: string;
  description: string;
  fn: () => Promise<void>;
  timeout?: number;
  retries?: number;
  tags?: string[];
}

// Test Runner Interface
interface TestRunner {
  run(suite: TestSuite): Promise<TestResult>;
  runAll(suites: TestSuite[]): Promise<TestResult[]>;
  stop(): Promise<void>;
}

// Test Result Interface
interface TestResult {
  suite: string;
  test: string;
  status: TestStatus;
  duration: number;
  error?: Error;
  logs: TestLog[];
  metadata: Record<string, any>;
}

// Test Reporter Interface
interface TestReporter {
  onSuiteStart(suite: TestSuite): void;
  onSuiteEnd(result: TestResult): void;
  onTestStart(test: Test): void;
  onTestEnd(result: TestResult): void;
  onError(error: Error): void;
  generateReport(results: TestResult[]): Promise<TestReport>;
}
```

### Directory Structure

```
bftest/
├── src/
│   ├── core/            # Core testing framework
│   │   ├── runner/     # Test runner implementation
│   │   ├── reporter/   # Test reporters
│   │   └── utils/      # Testing utilities
│   ├── suites/         # Test suites
│   │   ├── unit/      # Unit tests
│   │   ├── integration/# Integration tests
│   │   ├── e2e/       # End-to-end tests
│   │   └── performance/# Performance tests
│   ├── fixtures/       # Test fixtures
│   ├── mocks/         # Mock implementations
│   └── types/         # Type definitions
├── config/             # Test configurations
├── reports/           # Test reports
└── docs/              # Documentation
```

## Test Implementation

### Base Test Suite

```typescript
// src/core/base-suite.ts
import { TestSuite, Test, TestResult } from '../types';
import { Logger } from '../utils/logger';
import { Metrics } from '../utils/metrics';

export abstract class BaseTestSuite implements TestSuite {
  protected logger: Logger;
  protected metrics: Metrics;
  protected results: TestResult[] = [];
  
  constructor(
    public readonly name: string,
    public readonly description: string,
    public readonly tests: Test[]
  ) {
    this.logger = new Logger(`test:${name}`);
    this.metrics = new Metrics(`test:${name}`);
  }
  
  async setup(): Promise<void> {
    this.logger.info('Setting up test suite');
    await this.setupInternal();
  }
  
  async teardown(): Promise<void> {
    this.logger.info('Tearing down test suite');
    await this.teardownInternal();
  }
  
  protected abstract setupInternal(): Promise<void>;
  protected abstract teardownInternal(): Promise<void>;
  
  protected async runTest(test: Test): Promise<TestResult> {
    const start = Date.now();
    const result: TestResult = {
      suite: this.name,
      test: test.name,
      status: 'pending',
      duration: 0,
      logs: [],
      metadata: {}
    };
    
    try {
      this.logger.info(`Running test: ${test.name}`);
      await test.fn();
      
      result.status = 'passed';
      result.duration = Date.now() - start;
      this.metrics.recordSuccess(test.name, result.duration);
    } catch (error) {
      result.status = 'failed';
      result.duration = Date.now() - start;
      result.error = error;
      this.metrics.recordFailure(test.name, error);
      
      this.logger.error(`Test failed: ${test.name}`, { error });
    }
    
    this.results.push(result);
    return result;
  }
}
```

### Unit Test Suite

```typescript
// src/suites/unit/model-suite.ts
import { BaseTestSuite } from '../../core/base-suite';
import { Model } from '../../types';
import { ModelRegistry } from '../../registry';

export class ModelTestSuite extends BaseTestSuite {
  private registry: ModelRegistry;
  
  constructor() {
    super(
      'Model Tests',
      'Unit tests for model implementations',
      [
        {
          name: 'should load model',
          description: 'Test model loading',
          fn: () => this.testModelLoading(),
          timeout: 5000
        },
        {
          name: 'should generate text',
          description: 'Test text generation',
          fn: () => this.testTextGeneration(),
          timeout: 10000
        },
        {
          name: 'should handle errors',
          description: 'Test error handling',
          fn: () => this.testErrorHandling(),
          timeout: 5000
        }
      ]
    );
    
    this.registry = new ModelRegistry();
  }
  
  protected async setupInternal(): Promise<void> {
    await this.registry.initialize();
  }
  
  protected async teardownInternal(): Promise<void> {
    await this.registry.shutdown();
  }
  
  private async testModelLoading(): Promise<void> {
    const model = await this.registry.get('gpt-4');
    expect(model).toBeDefined();
    expect(model.name).toBe('gpt-4');
  }
  
  private async testTextGeneration(): Promise<void> {
    const model = await this.registry.get('gpt-4');
    const output = await model.generate({
      capability: 'text-generation',
      parameters: {
        prompt: 'Hello, world!',
        temperature: 0.7,
        maxTokens: 100
      }
    });
    
    expect(output.text).toBeDefined();
    expect(output.metadata.model).toBe('gpt-4');
  }
  
  private async testErrorHandling(): Promise<void> {
    const model = await this.registry.get('gpt-4');
    
    await expect(model.generate({
      capability: 'invalid',
      parameters: {}
    })).rejects.toThrow('Unsupported capability');
  }
}
```

### Integration Test Suite

```typescript
// src/suites/integration/api-suite.ts
import { BaseTestSuite } from '../../core/base-suite';
import { APIServer } from '../../server';
import { ModelRegistry } from '../../registry';
import { MemoryRegistry } from '../../registry';

export class APITestSuite extends BaseTestSuite {
  private server: APIServer;
  private modelRegistry: ModelRegistry;
  private memoryRegistry: MemoryRegistry;
  
  constructor() {
    super(
      'API Integration Tests',
      'Integration tests for API endpoints',
      [
        {
          name: 'should handle model requests',
          description: 'Test model API endpoints',
          fn: () => this.testModelEndpoints(),
          timeout: 15000
        },
        {
          name: 'should handle memory operations',
          description: 'Test memory API endpoints',
          fn: () => this.testMemoryEndpoints(),
          timeout: 10000
        },
        {
          name: 'should handle errors',
          description: 'Test API error handling',
          fn: () => this.testErrorHandling(),
          timeout: 5000
        }
      ]
    );
    
    this.server = new APIServer();
    this.modelRegistry = new ModelRegistry();
    this.memoryRegistry = new MemoryRegistry();
  }
  
  protected async setupInternal(): Promise<void> {
    await this.modelRegistry.initialize();
    await this.memoryRegistry.initialize();
    await this.server.start();
  }
  
  protected async teardownInternal(): Promise<void> {
    await this.server.stop();
    await this.modelRegistry.shutdown();
    await this.memoryRegistry.shutdown();
  }
  
  private async testModelEndpoints(): Promise<void> {
    const response = await fetch('http://localhost:3000/api/v1/models/generate', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify({
        model: 'gpt-4',
        capability: 'text-generation',
        parameters: {
          prompt: 'Hello, world!',
          temperature: 0.7,
          maxTokens: 100
        }
      })
    });
    
    expect(response.status).toBe(200);
    const data = await response.json();
    expect(data.text).toBeDefined();
    expect(data.metadata.model).toBe('gpt-4');
  }
  
  private async testMemoryEndpoints(): Promise<void> {
    // Test memory operations
    const response = await fetch('http://localhost:3000/api/v1/memory/cache', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify({
        key: 'test:key',
        value: { data: 'test' },
        ttl: 3600
      })
    });
    
    expect(response.status).toBe(200);
    
    const getResponse = await fetch('http://localhost:3000/api/v1/memory/cache/test:key');
    expect(getResponse.status).toBe(200);
    const data = await getResponse.json();
    expect(data.data).toBe('test');
  }
  
  private async testErrorHandling(): Promise<void> {
    const response = await fetch('http://localhost:3000/api/v1/models/generate', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify({
        model: 'invalid',
        capability: 'text-generation',
        parameters: {}
      })
    });
    
    expect(response.status).toBe(400);
    const error = await response.json();
    expect(error.code).toBe('MODEL_NOT_FOUND');
  }
}
```

## Test Runner

```typescript
// src/core/runner/test-runner.ts
import { TestRunner, TestSuite, TestResult } from '../types';
import { Logger } from '../utils/logger';
import { Metrics } from '../utils/metrics';
import { TestReporter } from '../reporter';

export class TestRunner implements TestRunner {
  private logger: Logger;
  private metrics: Metrics;
  private reporter: TestReporter;
  private running: boolean = false;
  
  constructor(reporter: TestReporter) {
    this.logger = new Logger('test:runner');
    this.metrics = new Metrics('test:runner');
    this.reporter = reporter;
  }
  
  async run(suite: TestSuite): Promise<TestResult[]> {
    if (this.running) {
      throw new Error('Test runner is already running');
    }
    
    this.running = true;
    const results: TestResult[] = [];
    
    try {
      this.logger.info(`Running test suite: ${suite.name}`);
      this.reporter.onSuiteStart(suite);
      
      // Setup suite
      if (suite.setup) {
        await suite.setup();
      }
      
      // Run tests
      for (const test of suite.tests) {
        this.reporter.onTestStart(test);
        
        const result = await this.runTest(suite, test);
        results.push(result);
        
        this.reporter.onTestEnd(result);
        
        if (result.status === 'failed' && test.retries) {
          for (let i = 0; i < test.retries; i++) {
            this.logger.info(`Retrying test: ${test.name} (attempt ${i + 1})`);
            
            const retryResult = await this.runTest(suite, test);
            results.push(retryResult);
            
            this.reporter.onTestEnd(retryResult);
            
            if (retryResult.status === 'passed') {
              break;
            }
          }
        }
      }
      
      // Teardown suite
      if (suite.teardown) {
        await suite.teardown();
      }
      
      this.reporter.onSuiteEnd({
        suite: suite.name,
        status: this.getSuiteStatus(results),
        duration: this.getSuiteDuration(results),
        results
      });
    } catch (error) {
      this.logger.error('Test suite failed', { error });
      this.reporter.onError(error);
      throw error;
    } finally {
      this.running = false;
    }
    
    return results;
  }
  
  async runAll(suites: TestSuite[]): Promise<TestResult[][]> {
    const results: TestResult[][] = [];
    
    for (const suite of suites) {
      const suiteResults = await this.run(suite);
      results.push(suiteResults);
    }
    
    return results;
  }
  
  async stop(): Promise<void> {
    if (!this.running) {
      return;
    }
    
    this.running = false;
    this.logger.info('Stopping test runner');
  }
  
  private async runTest(suite: TestSuite, test: Test): Promise<TestResult> {
    const start = Date.now();
    const result: TestResult = {
      suite: suite.name,
      test: test.name,
      status: 'pending',
      duration: 0,
      logs: [],
      metadata: {}
    };
    
    try {
      const timeout = test.timeout || 5000;
      await Promise.race([
        test.fn(),
        new Promise((_, reject) => 
          setTimeout(() => reject(new Error('Test timeout')), timeout)
        )
      ]);
      
      result.status = 'passed';
      result.duration = Date.now() - start;
      this.metrics.recordSuccess(test.name, result.duration);
    } catch (error) {
      result.status = 'failed';
      result.duration = Date.now() - start;
      result.error = error;
      this.metrics.recordFailure(test.name, error);
      
      this.logger.error(`Test failed: ${test.name}`, { error });
    }
    
    return result;
  }
  
  private getSuiteStatus(results: TestResult[]): TestStatus {
    if (results.some(r => r.status === 'failed')) {
      return 'failed';
    }
    if (results.some(r => r.status === 'pending')) {
      return 'pending';
    }
    return 'passed';
  }
  
  private getSuiteDuration(results: TestResult[]): number {
    return results.reduce((total, r) => total + r.duration, 0);
  }
}
```

## Test Reporter

```typescript
// src/core/reporter/json-reporter.ts
import { TestReporter, TestSuite, Test, TestResult, TestReport } from '../types';
import { Logger } from '../utils/logger';
import { writeFile } from 'fs/promises';
import { join } from 'path';

export class JSONReporter implements TestReporter {
  private logger: Logger;
  private results: TestResult[] = [];
  
  constructor(private outputDir: string) {
    this.logger = new Logger('test:reporter:json');
  }
  
  onSuiteStart(suite: TestSuite): void {
    this.logger.info(`Starting suite: ${suite.name}`);
  }
  
  onSuiteEnd(result: TestResult): void {
    this.logger.info(`Finished suite: ${result.suite}`);
    this.results.push(result);
  }
  
  onTestStart(test: Test): void {
    this.logger.info(`Starting test: ${test.name}`);
  }
  
  onTestEnd(result: TestResult): void {
    this.logger.info(`Finished test: ${result.test} (${result.status})`);
  }
  
  onError(error: Error): void {
    this.logger.error('Test reporter error', { error });
  }
  
  async generateReport(results: TestResult[]): Promise<TestReport> {
    const report: TestReport = {
      timestamp: new Date().toISOString(),
      summary: {
        total: results.length,
        passed: results.filter(r => r.status === 'passed').length,
        failed: results.filter(r => r.status === 'failed').length,
        pending: results.filter(r => r.status === 'pending').length,
        duration: results.reduce((total, r) => total + r.duration, 0)
      },
      suites: this.groupBySuite(results),
      errors: this.collectErrors(results)
    };
    
    // Write report to file
    const reportPath = join(this.outputDir, `report-${Date.now()}.json`);
    await writeFile(reportPath, JSON.stringify(report, null, 2));
    
    return report;
  }
  
  private groupBySuite(results: TestResult[]): Record<string, TestResult[]> {
    return results.reduce((groups, result) => {
      const suite = result.suite;
      if (!groups[suite]) {
        groups[suite] = [];
      }
      groups[suite].push(result);
      return groups;
    }, {} as Record<string, TestResult[]>);
  }
  
  private collectErrors(results: TestResult[]): Error[] {
    return results
      .filter(r => r.status === 'failed' && r.error)
      .map(r => r.error!);
  }
}
```

## CI Integration

### GitLab CI Configuration

```yaml
# .gitlab-ci.yml
image: node:18

variables:
  NODE_ENV: test
  TEST_REPORTS_DIR: test-reports

stages:
  - test
  - report

test:unit:
  stage: test
  script:
    - npm install
    - npm run test:unit
  artifacts:
    paths:
      - $TEST_REPORTS_DIR
    reports:
      junit: $TEST_REPORTS_DIR/junit.xml

test:integration:
  stage: test
  services:
    - redis:latest
    - elasticsearch:latest
  variables:
    REDIS_HOST: redis
    ELASTICSEARCH_URL: http://elasticsearch:9200
  script:
    - npm install
    - npm run test:integration
  artifacts:
    paths:
      - $TEST_REPORTS_DIR
    reports:
      junit: $TEST_REPORTS_DIR/junit.xml

test:e2e:
  stage: test
  services:
    - redis:latest
    - elasticsearch:latest
    - pinecone:latest
  variables:
    REDIS_HOST: redis
    ELASTICSEARCH_URL: http://elasticsearch:9200
    PINECONE_ENVIRONMENT: test
  script:
    - npm install
    - npm run test:e2e
  artifacts:
    paths:
      - $TEST_REPORTS_DIR
    reports:
      junit: $TEST_REPORTS_DIR/junit.xml

generate:report:
  stage: report
  script:
    - npm install
    - npm run test:report
  artifacts:
    paths:
      - $TEST_REPORTS_DIR
    reports:
      junit: $TEST_REPORTS_DIR/junit.xml
```

## Best Practices

1. **Test Organization**
   - Group tests by type
   - Use descriptive test names
   - Keep tests independent
   - Use appropriate test types
   - Maintain test fixtures

2. **Test Implementation**
   - Follow AAA pattern (Arrange, Act, Assert)
   - Use meaningful assertions
   - Handle async operations
   - Implement proper cleanup
   - Use test doubles when needed

3. **Test Coverage**
   - Aim for high coverage
   - Focus on critical paths
   - Test edge cases
   - Test error conditions
   - Monitor coverage trends

4. **Performance**
   - Keep tests fast
   - Use appropriate timeouts
   - Implement parallel execution
   - Optimize test setup
   - Monitor test duration

## Related Documentation

- [Model Integration](./models)
- [Memory Systems](./memory)
- [Deployment Guide](../operations/deployment)
- [Security Guide](../operations/security)
