---
id: guides-AI_DOCUMENTATION_GUIDE
---
# AI-Powered Documentation Generation Guide

This guide shows how to use AI (Ollama, OpenAI, or Anthropic) to automatically generate, enhance, and maintain your documentation in GitLab CI/CD.

## Overview

The AI Documentation Generator can:
- **Create** - Generate comprehensive documentation from code
- **Enhance** - Improve existing documentation with additional insights
- **Review** - Analyze documentation quality and suggest improvements
- **Translate** - Convert documentation to different languages (coming soon)

## Quick Start

### 1. Local Usage with Ollama (Free)

```bash
# Install Ollama
curl -fsSL https://ollama.com/install.sh | sh

# Pull a model (mixtral is great for code)
ollama pull mixtral:latest
ollama pull nomic-embed-text  # For embeddings

# Run the documentation generator
cd bfdocs
node scripts/ai-doc-generator.js
```

### 2. GitLab CI Integration

Add to your `.gitlab-ci.yml`:

```yaml
include:
  - component: gitlab.bluefly.io/ecosystem/bfcicomponents/ai-documentation-generator@main
    inputs:
      provider: ollama  # or openai, anthropic, auto
      mode: enhance     # or create, review
      file_patterns: "**/*.{js,ts,php,py}"
      doc_output_dir: "docs/ai-generated"
```

### 3. With Environment Variables

```yaml
variables:
  AI_PROVIDER: "openai"
  OPENAI_API_KEY: $OPENAI_API_KEY  # Set in GitLab CI/CD settings

ai-docs:
  extends: .ai-doc-generate
  stage: documentation
```

## Configuration Options

### Providers

#### Ollama (Recommended for Self-Hosted)
```yaml
ai-docs:
  extends: .ai-doc-generate
  variables:
    AI_PROVIDER: "ollama"
    OLLAMA_MODEL: "mixtral:latest"  # or codellama, llama2, etc.
```

**Available Models**:
- `mixtral:latest` - Best for comprehensive documentation
- `codellama:latest` - Optimized for code understanding
- `llama2:latest` - Good general purpose
- `mistral:latest` - Fast and efficient

#### OpenAI
```yaml
ai-docs:
  extends: .ai-doc-generate
  variables:
    AI_PROVIDER: "openai"
    OPENAI_MODEL: "gpt-4"  # or gpt-4-turbo, gpt-3.5-turbo
    OPENAI_API_KEY: $OPENAI_API_KEY
```

#### Anthropic (Claude)
```yaml
ai-docs:
  extends: .ai-doc-generate
  variables:
    AI_PROVIDER: "anthropic"
    ANTHROPIC_MODEL: "claude-3-opus-20240229"  # or claude-3-sonnet, claude-3-haiku
    ANTHROPIC_API_KEY: $ANTHROPIC_API_KEY
```

#### Auto Mode
Automatically selects the best available provider:
```yaml
ai-docs:
  extends: .ai-doc-generate
  variables:
    AI_PROVIDER: "auto"  # Tries: Ollama → OpenAI → Anthropic
```

### Modes

#### Create Mode
Generates new documentation from scratch:
```yaml
ai-docs-create:
  extends: .ai-doc-generate
  variables:
    UPDATE_MODE: "create"
  rules:
    - if: '$CI_COMMIT_MESSAGE =~ /\[create-docs\]/'
```

#### Enhance Mode (Default)
Improves existing documentation:
```yaml
ai-docs-enhance:
  extends: .ai-doc-generate
  variables:
    UPDATE_MODE: "enhance"
  rules:
    - if: '$CI_PIPELINE_SOURCE == "merge_request_event"'
```

#### Review Mode
Reviews documentation quality:
```yaml
ai-docs-review:
  extends: .ai-doc-generate
  variables:
    UPDATE_MODE: "review"
  when: manual  # Run manually to avoid API costs
```

## Advanced Usage

### Custom File Patterns

```yaml
ai-docs:
  extends: .ai-doc-generate
  variables:
    FILE_PATTERNS: "src/**/*.{ts,tsx},lib/**/*.js,packages/*/src/**/*.{js,ts}"
```

### Caching for Performance

```yaml
ai-docs:
  extends: .ai-doc-generate
  cache:
    key: "ai-embeddings-${CI_COMMIT_REF_SLUG}"
    paths:
      - .embeddings-cache/
      - .ollama-models/
```

### Automatic Merge Requests

```yaml
ai-docs-with-mr:
  extends: .ai-doc-generate
  variables:
    COMMIT_CHANGES: "true"
  after_script:
    - |
      if [ -n "$(git status --porcelain)" ]; then
        git push origin HEAD:ai-docs-$CI_COMMIT_SHORT_SHA
        curl -X POST "${CI_API_V4_URL}/projects/${CI_PROJECT_ID}/merge_requests" \
          -H "PRIVATE-TOKEN: ${PROJECT_ACCESS_TOKEN}" \
          -H "Content-Type: application/json" \
          -d '{
            "source_branch": "ai-docs-'${CI_COMMIT_SHORT_SHA}'",
            "target_branch": "'${CI_DEFAULT_BRANCH}'",
            "title": "AI Documentation Updates",
            "description": "Automated documentation updates generated by AI"
          }'
      fi
```

### Integration with Existing Docs

```javascript
// In your package.json
{
  "scripts": {
    "docs:ai": "node scripts/ai-doc-generator.js",
    "docs:ai:create": "AI_MODE=create npm run docs:ai",
    "docs:ai:enhance": "AI_MODE=enhance npm run docs:ai",
    "docs:ai:review": "AI_MODE=review npm run docs:ai",
    "docs:build": "npm run docs:ai:enhance && vuepress build docs"
  }
}
```

## Best Practices

### 1. Start with Create Mode
Generate initial documentation for undocumented code:
```bash
AI_MODE=create FILE_PATTERNS="src/**/*.js" npm run docs:ai
```

### 2. Use Enhance Mode in CI/CD
Automatically improve docs on every merge request:
```yaml
stages:
  - test
  - documentation
  - review

enhance-docs:
  extends: .ai-doc-generate
  stage: documentation
  variables:
    UPDATE_MODE: "enhance"
  rules:
    - if: '$CI_PIPELINE_SOURCE == "merge_request_event"'
```

### 3. Schedule Regular Reviews
```yaml
scheduled-doc-review:
  extends: .ai-doc-generate
  variables:
    UPDATE_MODE: "review"
  only:
    - schedules
```

### 4. Combine with VuePress
```yaml
pages:
  stage: deploy
  dependencies:
    - ai-docs
  script:
    - npm run docs:build  # VuePress build
    - cp -r docs/ai-generated public/ai
  artifacts:
    paths:
      - public
```

## Example Output

The AI generator creates comprehensive documentation like this:

```markdown
# user.service.ts

**File**: `src/services/user.service.ts`  
**Language**: typescript  
**Generated**: 2025-01-06T10:30:00Z  
**AI Model**: ollama/mixtral  

## Overview

The UserService class provides comprehensive user management functionality including authentication, authorization, profile management, and user data operations. It implements the Repository pattern and integrates with the authentication system.

## API Reference

### Class: UserService

#### Methods

**createUser(userData: CreateUserDto): Promise<User>**
Creates a new user with the provided data.
- Validates input data
- Hashes password using bcrypt
- Sends welcome email
- Returns created user object

*Example:*
```typescript
const user = await userService.createUser({
  email: 'user@example.com',
  password: 'securepass123',
  name: 'John Doe'
});
```

[... continues with full documentation ...]
```

## Monitoring and Costs

### Token Usage Tracking
The generator reports token usage in the artifacts:
```json
{
  "filesProcessed": 45,
  "tokensUsed": 125000,
  "estimatedCost": "$2.50",
  "provider": "openai",
  "model": "gpt-4"
}
```

### Cost Optimization
- Use Ollama for free, unlimited generation
- Use `gpt-3.5-turbo` for lower costs
- Enable caching to avoid re-processing
- Use file patterns to limit scope

## Troubleshooting

### Ollama Connection Issues
```yaml
ai-docs:
  extends: .ai-doc-setup
  before_script:
    - ollama serve &
    - sleep 10  # Give Ollama time to start
    - ollama list  # Verify models
```

### Memory Issues
For large codebases:
```yaml
ai-docs:
  extends: .ai-doc-generate
  variables:
    CHUNK_SIZE: "1000"  # Smaller chunks
    BATCH_SIZE: "10"    # Process fewer files at once
```

### API Rate Limits
Add retry logic:
```yaml
ai-docs:
  extends: .ai-doc-generate
  retry:
    max: 2
    when:
      - runner_system_failure
      - stuck_or_timeout_failure
```

## Security Considerations

1. **API Keys**: Store in GitLab CI/CD variables (masked)
2. **Code Privacy**: Use Ollama for sensitive code
3. **Access Control**: Limit who can trigger AI jobs
4. **Output Review**: Always review AI-generated content

## Next Steps

1. **Install Ollama locally** and test the generator
2. **Add to your GitLab CI/CD** pipeline
3. **Customize prompts** in `ai-doc-generator.js`
4. **Integrate with your docs site** (VuePress, Docusaurus, etc.)

The AI Documentation Generator is now ready to transform your codebase into comprehensive, maintainable documentation! 🚀
