---
id: guides-VALIDATION_MIGRATION_GUIDE
---
# Validation Utilities Migration Guide

This guide provides step-by-step instructions for migrating existing validation logic in your application to the new shared validation utilities.

## Why Migrate?

The new validation utilities provide several advantages over custom validation implementations:

1. **Comprehensive Coverage**: Pre-built validators for common data types and formats
2. **Type Safety**: Full TypeScript support with proper type definitions
3. **Standardization**: Consistent validation behavior across all projects
4. **Schema Validation**: Easy validation of complex objects
5. **Detailed Error Messages**: Informative error messages for validation failures
6. **Reduced Code Duplication**: Centralized validation logic reduces duplication
7. **Maintainability**: Single source of truth for validation rules

## Migration Steps

### Step 1: Install or Update Dependencies

Ensure you have the latest version of the shared library installed:

```bash
npm install @bluefly/llmcli@latest --save
```

### Step 2: Import the Validation Utilities

Replace your custom validation imports with the shared validation utilities:

```typescript
// Before
import { validateEmail, validateUrl } from './utils/validators';

// After
import { validationUtils } from '@bluefly/llmcli/shared';
```

You can also import specific validation modules if needed:

```typescript
import { 
  string as validateString,
  number as validateNumber,
  object as validateObject
} from '@bluefly/llmcli/shared/utils/validation-utils';
```

### Step 3: Replace Simple Validation Functions

Replace individual validation functions with their shared counterparts:

```typescript
// Before
const isValidEmail = validateEmail(email);
const isValidUrl = validateUrl(url);
const isPositiveNumber = number > 0;

// After
const isValidEmail = validationUtils.string.isEmail(email);
const isValidUrl = validationUtils.string.isUrl(url);
const isPositiveNumber = validationUtils.number.isPositive(number);
```

### Step 4: Migrate Object Validation

Replace custom object validation with schema-based validation:

```typescript
// Before
function validateConfig(config) {
  if (!config) return false;
  if (typeof config.baseUrl !== 'string') return false;
  if (!isValidUrl(config.baseUrl)) return false;
  if (config.timeout !== undefined && (typeof config.timeout !== 'number' || config.timeout < 0)) return false;
  if (config.retries !== undefined && (typeof config.retries !== 'number' || config.retries < 0)) return false;
  return true;
}

// After
function validateConfig(config) {
  const result = validationUtils.object.matchesSchema(config, {
    baseUrl: { 
      type: 'string', 
      required: true,
      custom: (value) => ({
        valid: validationUtils.string.isUrl(value),
        errors: validationUtils.string.isUrl(value) ? undefined : ['Not a valid URL']
      })
    },
    timeout: { 
      type: 'number', 
      required: false,
      min: 0
    },
    retries: { 
      type: 'number', 
      required: false,
      min: 0 
    }
  });
  
  return result.valid;
}
```

### Step 5: Migrate Array Validation

Replace custom array validation with the shared array validation utilities:

```typescript
// Before
function validateUserList(users) {
  if (!Array.isArray(users)) return false;
  if (users.length === 0) return false;
  return users.every(user => validateUser(user));
}

// After
function validateUserList(users) {
  if (!validationUtils.array.isArray(users)) return false;
  if (!validationUtils.array.minLength(users, 1)) return false;
  
  // Option 1: Using allValid
  return validationUtils.array.allValid(users, user => validateUser(user));
  
  // Option 2: Using validateObjects with a schema
  const result = validationUtils.array.validateObjects(users, userSchema);
  return result.valid;
}
```

### Step 6: Implement Custom Validators

Migrate custom validation logic to use the validator creation utilities:

```typescript
// Before
function validatePassword(password) {
  if (typeof password !== 'string') return false;
  if (password.length < 8) return false;
  if (!/[A-Z]/.test(password)) return false; // Has uppercase
  if (!/[0-9]/.test(password)) return false; // Has number
  if (!/[^A-Za-z0-9]/.test(password)) return false; // Has special character
  return true;
}

// After
const validatePassword = validationUtils.createValidator<string>(
  (password) => typeof password === 'string',
  (password) => validationUtils.string.hasMinLength(password, 8),
  (password) => /[A-Z]/.test(password), // Has uppercase
  (password) => /[0-9]/.test(password), // Has number
  (password) => /[^A-Za-z0-9]/.test(password) // Has special character
);

// Usage
const result = validatePassword(password);
if (!result.valid) {
  console.error('Password validation failed:', result.errors);
}
```

### Step 7: Integrate with Error Handling

Replace custom error handling with the shared error handling system:

```typescript
// Before
function processUserData(userData) {
  if (!validateUser(userData)) {
    throw new Error('Invalid user data');
  }
  // Process user data...
}

// After
import { createError } from '@bluefly/llmcli/shared';

function processUserData(userData) {
  try {
    validationUtils.assertValid(
      userData,
      [user => validateUser(user).valid],
      'Invalid user data'
    );
    // Process user data...
  } catch (error) {
    // The error is already properly formatted
    throw error;
  }
}

// Alternative using object schema validation
function processUserData(userData) {
  const result = validationUtils.object.matchesSchema(userData, userSchema);
  
  if (!result.valid) {
    throw createError(
      `Invalid user data: ${result.errors?.join('; ')}`,
      'VALIDATION_ERROR'
    );
  }
  
  // Process user data...
}
```

## Common Migration Patterns

### Email Validation

```typescript
// Before
const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
const isValidEmail = emailRegex.test(email);

// After
const isValidEmail = validationUtils.string.isEmail(email);
```

### URL Validation

```typescript
// Before
function isValidUrl(url) {
  try {
    new URL(url);
    return true;
  } catch {
    return false;
  }
}

// After
const isValidUrl = validationUtils.string.isUrl(url);
```

### Form Validation

```typescript
// Before
function validateForm(formData) {
  const errors = {};
  
  if (!formData.name) errors.name = 'Name is required';
  if (!emailRegex.test(formData.email)) errors.email = 'Invalid email';
  if (!formData.password || formData.password.length < 8) errors.password = 'Password must be at least 8 characters';
  
  return {
    valid: Object.keys(errors).length === 0,
    errors
  };
}

// After
function validateForm(formData) {
  return validationUtils.object.matchesSchema(formData, {
    name: { 
      type: 'string', 
      required: true,
      message: 'Name is required'
    },
    email: { 
      type: 'string', 
      required: true,
      custom: (value) => ({
        valid: validationUtils.string.isEmail(value),
        errors: validationUtils.string.isEmail(value) ? undefined : ['Invalid email']
      })
    },
    password: { 
      type: 'string', 
      required: true,
      minLength: 8,
      message: 'Password must be at least 8 characters'
    }
  });
}
```

## Advanced Migration Scenarios

### API Request Validation

```typescript
// Before
function validateApiRequest(request) {
  // Complex validation logic
  // ...
}

// After
const apiRequestSchema = {
  method: { type: 'string', required: true, enum: ['GET', 'POST', 'PUT', 'DELETE'] },
  endpoint: { type: 'string', required: true },
  headers: { type: 'object', required: false },
  body: { type: 'object', required: false },
  timeout: { type: 'number', required: false, min: 0 }
};

function validateApiRequest(request) {
  return validationUtils.object.matchesSchema(request, apiRequestSchema);
}
```

### Complex Configuration Validation

```typescript
// Before
function validateConfig(config) {
  // Complex validation with nested properties
  // ...
}

// After
const configSchema = {
  services: {
    type: 'object',
    required: true,
    custom: (value) => {
      // Check if it has required services
      const required = ['api', 'auth', 'storage'];
      const missing = required.filter(svc => !value[svc]);
      
      return {
        valid: missing.length === 0,
        errors: missing.length ? [`Missing required services: ${missing.join(', ')}`] : undefined
      };
    }
  },
  'services.api.url': { type: 'string', required: true },
  'services.api.timeout': { type: 'number', required: false, min: 0 },
  'services.auth.providers': { 
    type: 'object',
    required: true,
    custom: (value) => {
      const hasProvider = Object.keys(value).length > 0;
      return {
        valid: hasProvider,
        errors: hasProvider ? undefined : ['At least one auth provider must be configured']
      };
    }
  }
};

function validateConfig(config) {
  // Flatten the config for deep property validation
  const flatConfig = validationUtils.objectUtils.flatten(config);
  return validationUtils.object.matchesSchema(flatConfig, configSchema);
}
```

## Testing Your Migration

After migrating to the shared validation utilities, make sure to test thoroughly:

1. **Unit Tests**: Update your unit tests to use the new validation utilities
2. **Integration Tests**: Verify that validation integrates correctly with other components
3. **Edge Cases**: Test boundary conditions and edge cases to ensure validation is working as expected

## Backward Compatibility

The shared validation utilities maintain backward compatibility with the existing validation functions:

```typescript
// Legacy validation (still works)
const isValidEmail = stringUtils.isValidEmail('user@example.com');

// New validation (preferred)
const isValidEmail = validationUtils.string.isEmail('user@example.com');
```

## Getting Help

If you encounter issues during migration, refer to the comprehensive documentation:

- [Validation Utilities Documentation](./bfdocs/shared/VALIDATION_UTILITIES)
- [Shared Utilities Update](./bfdocs/shared/SHARED_UTILITIES_UPDATE)

For specific questions or assistance, contact the shared utilities maintenance team.
