---
id: integration-patterns
---
# Integration Patterns for AI Modules

This document describes the integration patterns and best practices for the various AI modules in this repository. It explains how the modules interact, the dependency patterns, and how to use the standardized interfaces for cross-module communication.

## Table of Contents

1. [Integration Overview](#integration-overview)
2. [Service-Oriented Architecture](#service-oriented-architecture)
3. [Module Dependency Strategy](#module-dependency-strategy)
4. [Capability-Based Integration](#capability-based-integration)
5. [Cross-Module Communication](#cross-module-communication)
6. [Bridge Modules and Services](#bridge-modules-and-services)
7. [Integration Patterns in Action](#integration-patterns-in-action)
8. [Best Practices](#best-practices)

## Integration Overview

The AI modules in this repository are designed to work together while maintaining independence. They follow these core principles:

1. **Loose Coupling**: Modules depend on interfaces, not implementations
2. **Capability Detection**: Runtime discovery of available features
3. **Graceful Degradation**: Fallback behavior when dependencies are missing
4. **Optional Dependencies**: Minimize hard dependencies between modules
5. **Interface Standardization**: Common interfaces for similar functionalities

## Service-Oriented Architecture

The modules follow a service-oriented architecture with these key components:

1. **Interface Module**: `llm_platform_interfaces` provides standardized interfaces
2. **Service Locator**: Central service for discovering capabilities and services
3. **Bridge Services**: Integration points between different modules
4. **Event System**: Cross-module communication through events
5. **Registry Services**: Centralized registries for models, agents, etc.

## Module Dependency Strategy

### Hard Dependencies vs. Optional Dependencies

Modules use a combination of hard dependencies and optional dependencies:

```yaml
# Hard dependencies - absolutely required for functionality
dependencies:
  - drupal:system
  - llm:llm_platform_interfaces  # Standardized interfaces

# Optional dependencies - enhance functionality but not strictly required
optional_dependencies:
  - llm:llm                      # LLM platform integration
  - ai_swarm_intelligence:ai_swarm_intelligence  # Swarm intelligence
  - field_eca_bridge:field_eca_bridge            # Field-level ECA
```

### Interface-Only Dependencies

Modules depend on the `llm_platform_interfaces` module, which provides:

1. Base interfaces like `AiPlatformInterface`
2. Specialized interfaces for agents, models, events, etc.
3. Bridge interfaces for cross-module integration
4. Service discovery through `ServiceLocator`

## Capability-Based Integration

Modules define their capabilities and look for capabilities in other modules, rather than checking for the existence of specific modules.

### Defining Capabilities

```yaml
# Integration capabilities
integration_capabilities:
  - api_normalization          # API data normalization
  - openapi_compliance         # OpenAPI 3.1 compliance
  - schema_validation          # Schema-based validation
```

### Discovering Capabilities

```php
// Using ServiceLocator to find services with capabilities
$serviceLocator = \Drupal::service('llm_platform_interfaces.service_locator');

if ($serviceLocator->hasCapability('text_processing')) {
  // Use text processing capability
  $serviceId = $serviceLocator->getBestServiceForCapability('text_processing');
  $service = \Drupal::service($serviceId);
}
```

## Cross-Module Communication

### Event-Based Communication

Modules communicate through events rather than direct method calls:

```php
// Dispatching an event
$dispatcher = \Drupal::service('llm_platform_interfaces.event_dispatcher');
$event = $dispatcher->createEvent('process_complete', 'my_module', [
  'result' => $result,
]);
$dispatcher->dispatch('ai.process_complete', $event);

// Subscribing to events
public function getSubscribedEvents() {
  return [
    'ai.process_complete' => ['onProcessComplete', 100],
  ];
}
```

### Service Discovery

Modules discover services implementing specific interfaces:

```php
// Finding services implementing an interface
$serviceLocator = \Drupal::service('llm_platform_interfaces.service_locator');
$serviceIds = $serviceLocator->getServicesByInterface(
  'Drupal\llm_platform_interfaces\Agent\AiAgentInterface'
);

// Getting the best implementation
$agent = $serviceLocator->getBestImplementation(
  'Drupal\llm_platform_interfaces\Agent\AiAgentInterface'
);
```

## Bridge Modules and Services

Bridge services connect different modules:

### Swarm-LLM Bridge

```php
// Using the Swarm-LLM bridge
$bridge = \Drupal::service('llm_platform_interfaces.swarm_llm_bridge');
$swarm = $bridge->createLlmSwarm([
  'name' => 'My Swarm',
  'agentTypes' => ['llm', 'code_genesis'],
]);
$response = $bridge->processWithLlmSwarm($swarm, $request);
```

### Field-LLM Bridge

```php
// Using the Field-LLM bridge
$bridge = \Drupal::service('llm_platform_interfaces.field_llm_bridge');
$processedValue = $bridge->processField($entity, $fieldName, $options);
```

## Integration Patterns in Action

### Example: AI Field Processing

This example shows how field processing integrates across modules:

1. `field_eca_bridge` provides field-level events and conditions
2. `llm` provides language model processing
3. `ai_swarm_intelligence` provides collaborative processing
4. Integration happens through the `ServiceLocator` and interfaces

```php
// Field ECA Bridge event subscriber
public function onFieldChange(FieldChangeEvent $event) {
  $serviceLocator = \Drupal::service('llm_platform_interfaces.service_locator');
  
  // Use AI processing if available
  if ($serviceLocator->hasCapability('field_processing')) {
    $processor = $serviceLocator->getBestServiceForCapability('field_processing');
    $processor->processField($event->getEntity(), $event->getFieldName());
  }
}
```

### Example: LLM and Swarm Integration

This example shows how LLM and Swarm Intelligence modules integrate:

1. `llm` provides agent implementations
2. `ai_swarm_intelligence` provides swarm orchestration
3. Integration happens through the `AiSwarmLlmBridgeInterface`

```php
// Swarm-LLM bridge usage
$bridge = \Drupal::service('llm_platform_interfaces.swarm_llm_bridge');

// Get LLM agents
$llmAgents = $serviceLocator->getServicesByTag('ai_agent_provider');

// Create a swarm with LLM agents
$swarm = $bridge->createLlmSwarm([
  'name' => 'Content Analysis Swarm',
  'agentTypes' => ['text_analyzer', 'sentiment_analyzer'],
]);

// Process content with the swarm
$response = $bridge->processWithLlmSwarm($swarm, [
  'content' => $content,
  'options' => ['detailed_analysis' => true],
]);
```

## Best Practices

### 1. Focus on Capabilities, Not Modules

```php
// BAD: Checking for module existence
if (\Drupal::moduleHandler()->moduleExists('llm')) {
  // Use LLM module
}

// GOOD: Checking for capability
if ($serviceLocator->hasCapability('text_processing')) {
  // Use text processing capability
}
```

### 2. Implement Graceful Degradation

```php
// Service with graceful degradation
public function processContent($content, $options = []) {
  $serviceLocator = \Drupal::service('llm_platform_interfaces.service_locator');
  
  // Try advanced processing first
  if ($serviceLocator->hasCapability('advanced_text_processing')) {
    $processor = $serviceLocator->getBestServiceForCapability('advanced_text_processing');
    return $processor->process($content, $options);
  }
  
  // Fall back to basic processing
  if ($serviceLocator->hasCapability('basic_text_processing')) {
    $processor = $serviceLocator->getBestServiceForCapability('basic_text_processing');
    return $processor->process($content, $options);
  }
  
  // Final fallback to simple processing
  return $this->basicProcess($content, $options);
}
```

### 3. Use Service Tags

```yaml
services:
  my_module.my_service:
    class: Drupal\my_module\MyService
    arguments: ['@database', '@cache.default']
    tags:
      - { name: ai_model_provider, capabilities: ['text_processing', 'code_generation'] }
```

### 4. Use Events for Cross-Module Communication

```php
// Instead of direct method calls
$result = $otherModule->processData($data);

// Use events
$event = new DataProcessEvent($data);
$this->eventDispatcher->dispatch('ai.data_process', $event);
$result = $event->getResult();
```

### 5. Implement Common Interfaces

```php
// Implement standardized interfaces
class MyService implements AiPlatformInterface, AiConfigurableInterface {
  // Implementation of interface methods
}
```
