---
id: integration-service-naming-conventions
---
# Service Naming Conventions

This document outlines the standardized naming conventions for services across all AI integration modules to ensure consistency and improve maintainability.

## Current Service Naming Analysis

After analyzing the services across multiple modules, we've identified several patterns and inconsistencies:

### Module Prefixes:
- **LLM Module**: Uses `llm.` prefix (e.g., `llm.platform`, `llm.service`)
- **AI Swarm Intelligence**: Uses `ai_swarm.` prefix (e.g., `ai_swarm.orchestrator`)
- **Field ECA Bridge**: Uses `field_eca_bridge.` prefix (e.g., `field_eca_bridge.event_manager`)
- **LangChain**: Uses `langchain.` prefix (e.g., `langchain.service`)

### Service Categories:
Services tend to fall into these categories:
1. **Core services**: Main module functionality
2. **Integration services**: Connect with other modules
3. **Manager services**: Manage a collection of related functions
4. **Bridge services**: Provide standardized cross-module communication
5. **Engine services**: Provide specialized processing capabilities
6. **Utility services**: Provide supporting functionality

### Inconsistencies:
1. Variations in naming for similar concepts (e.g., `llm.platform` vs `langchain.service` as main services)
2. Inconsistent patterns for integration services
3. Mixed use of underscores and dots
4. Varying levels of specificity in service names

## Standardized Naming Convention

To address these inconsistencies, we propose the following standardized naming convention:

### 1. Module Prefix

Always use the module's machine name as the prefix, followed by a period:
- `llm.`
- `ai_swarm_intelligence.` (or `ai_swarm.` for brevity)
- `field_eca_bridge.`
- `langchain.`

### 2. Service Category

After the module prefix, use a category identifier:

- **Core Services**: `core` (e.g., `llm.core`)
- **Managers**: `manager.[type]` (e.g., `llm.manager.model`)
- **Engines**: `engine.[type]` (e.g., `ai_swarm.engine.emergence`)
- **Integrations**: `integration.[module]` (e.g., `field_eca_bridge.integration.llm`)
- **Bridges**: `bridge.[module]` (e.g., `llm.bridge.swarm`)
- **Utilities**: `util.[type]` (e.g., `llm.util.cache`)
- **Services**: `service.[type]` (e.g., `langchain.service.provider`)

### 3. Specific Service Functions

For further clarity, add descriptive terms for specific functions:

- **Registry**: `registry.[type]` (e.g., `llm.registry.model`)
- **Factory**: `factory.[type]` (e.g., `ai_swarm.factory.agent`)
- **Validator**: `validator.[type]` (e.g., `field_eca_bridge.validator.field`)
- **Processor**: `processor.[type]` (e.g., `llm.processor.content`)
- **Analyzer**: `analyzer.[type]` (e.g., `ai_swarm.analyzer.behavior`)
- **Dispatcher**: `dispatcher.[type]` (e.g., `field_eca_bridge.dispatcher.event`)

### 4. Integration and Bridge Services

For services that bridge between modules, use this consistent pattern:
`[module_a].bridge.[module_b]` or `[module_a].integration.[module_b]`

Examples:
- `llm.bridge.swarm` (Bridge from LLM to AI Swarm)
- `ai_swarm.bridge.llm` (Bridge from AI Swarm to LLM)
- `field_eca_bridge.integration.llm` (Integration from Field ECA Bridge to LLM)

### 5. Backward Compatibility

When renaming services, maintain backward compatibility using Drupal's service alias feature:

```yaml
# New standardized name
llm.manager.model:
  class: Drupal\llm\Service\ModelManager
  # ...arguments

# Backward compatibility alias
llm.model_manager:
  alias: llm.manager.model
  deprecated: 'Use llm.manager.model instead. Will be removed in next version.'
```

## Examples of Standardized Service Names

### LLM Module:
- `llm.core` (Main service, replacing `llm.platform`)
- `llm.manager.model` (Manages LLM models)
- `llm.processor.content` (Processes content with LLMs)
- `llm.bridge.swarm` (Bridge to AI Swarm)

### AI Swarm Intelligence:
- `ai_swarm.core` (Main service, replacing `ai_swarm.orchestrator`)
- `ai_swarm.engine.emergence` (Emergence detection engine)
- `ai_swarm.factory.agent` (Creates agents)
- `ai_swarm.bridge.llm` (Bridge to LLM)

### Field ECA Bridge:
- `field_eca_bridge.core` (Main service)
- `field_eca_bridge.manager.event` (Manages field events)
- `field_eca_bridge.bridge.swarm` (Bridge to AI Swarm)

### LangChain:
- `langchain.core` (Main service, replacing `langchain.service`)
- `langchain.manager.provider` (Manages LangChain providers)
- `langchain.integration.mcp` (Integration with MCP)

## Implementation Plan

1. Document current service names and their proposed standardized versions
2. Create aliases for all renamed services to maintain backward compatibility
3. Update documentation to reflect new naming conventions
4. Gradually transition to new service names in code
5. Update any tests that depend on specific service names

## Conclusion

This standardized naming convention will improve code readability, maintainability, and cross-module integration. It provides clear guidelines for naming new services and refactoring existing ones while ensuring backward compatibility.
