---
id: model-registry-api-README
---
# Model Registry API

This is a production-ready implementation of the Model Registry API for the LLM Platform AI Platform. It provides endpoints for managing models, creating deployments, and running inference.

## Test-Driven Development (TDD)

This project strictly follows Test-Driven Development principles:

1. **RED Phase**: We write failing tests first that define the expected behavior
2. **GREEN Phase**: We implement minimal code to make tests pass
3. **REFACTOR Phase**: We optimize code while maintaining passing tests

Our TDD workflow ensures:
- High test coverage (95%+)
- Reliable, well-defined behaviors
- Clean, maintainable code
- Regression prevention

To contribute to this project, please follow the TDD approach:

## Features

- Complete implementation of `/models` and `/deployments` endpoints
- JWT authentication
- Input validation
- Rate limiting
- Error handling
- Database integration
- Logging
- Security best practices

## Prerequisites

- Node.js 16+
- MySQL 8.0+

## Installation

1. Clone the repository
2. Install dependencies

```bash
npm install
```

3. Create a `.env` file (you can copy from `.env.example`)

```bash
cp .env.example .env
```

4. Update the `.env` file with your configuration

5. Run the database migrations

```bash
# Make sure your MySQL server is running
# Import the SQL files from the migrations directory
mysql -u your_user -p your_database < database_schemas/migrations/01_create_llm_platform_tables.sql
mysql -u your_user -p your_database < database_schemas/migrations/02_insert_sample_data.sql
mysql -u your_user -p your_database < database_schemas/migrations/03_create_deployments_table.sql
```

## Running the Application

### Development Mode

```bash
npm run dev
```

### Production Mode

```bash
npm start
```

## API Endpoints

### Authentication

```
POST /v1/auth
```

Request body:

```json
{
  "username": "user@example.com",
  "password": "secure_password"
}
```

Response:

```json
{
  "token": "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."
}
```

### List Models

```
GET /v1/models
```

Query parameters:

- `provider` (optional): Filter by provider
- `is_active` (optional): Filter by active status
- `limit` (optional): Limit results (default: 20, max: 100)
- `offset` (optional): Offset for pagination (default: 0)

### Create Deployment

```
POST /v1/deployments
```

Request body:

```json
{
  "name": "production-gpt4",
  "model_id": 1,
  "model_version_id": 3,
  "environment": "production",
  "scaling": {
    "min_replicas": 2,
    "max_replicas": 10,
    "target_cpu_utilization": 70
  },
  "config": {
    "timeout_seconds": 60,
    "max_tokens": 8192,
    "temperature": 0.7
  }
}
```

## Security Features

- JWT-based authentication
- Rate limiting to prevent abuse
- Input validation to prevent malicious data
- Security headers using Helmet
- Parameterized queries to prevent SQL injection
- Error handling that doesn't expose sensitive information

## Testing

```bash
# Run all tests
npm test

# Run tests with coverage
npm run test:coverage
```

## Rate Limiting

The API implements rate limiting to ensure fair usage and system stability:

- Authentication endpoints: 10 requests per minute per IP address
- Standard endpoints: 60 requests per minute per authenticated user
- Bulk operations: 10 requests per minute per authenticated user

## Error Handling

The API uses standard HTTP status codes and returns consistent error responses:

```json
{
  "error": "error_type",
  "message": "Human-readable error message",
  "details": [
    {
      "field": "field_name",
      "message": "Field-specific error message"
    }
  ]
}
```

## Database Schema

The API uses the following database schema:

### Models Table

Stores the core model information.

### ModelVersions Table

Stores different versions of each model.

### Deployments Table

Stores model deployments.

## License

MIT
