# TDD-AI Quick Start Guide

Get up and running with TDD-AI in minutes!

## 🚀 Quick Setup

### 1. Install Ollama

```bash
# macOS
brew install ollama

# Linux
curl -fsSL https://ollama.ai/install.sh | sh

# Windows
# Download from https://ollama.ai/download
```

### 2. Build TDD-AI Model

```bash
# Navigate to the tddai-model directory
cd tddai-model

# Create the model
ollama create tddai-model -f Modelfile
```

### 3. Start Using TDD-AI

```bash
# Start a conversation
ollama run tddai-model
```

## 🎯 Quick Examples

### Example 1: JavaScript User Service

**Prompt**: "Create a user authentication service with email validation"

**Expected Response**:

```
## REASON
I need to create a user authentication service with email validation. This requires:
- User registration and login functionality
- Email format validation
- Secure password handling
- Error handling for invalid inputs

## TEST
```javascript
import { describe, it, expect, beforeEach } from '@jest/globals';
import [UserService] from './UserService';

describe('UserService', () => {
  let userService;

  beforeEach(() => {
    userService = new UserService();
  });

  describe('register', () => {
    it('should register user with valid email and password', () => {
      const result = userService.register('user@example.com', 'password123');
      expect(result.success).toBe(true);
      expect(result.user.email).toBe('user@example.com');
    });

    it('should reject invalid email format', () => {
      expect(() => {
        userService.register('invalid-email', 'password123');
      }).toThrow('Invalid email format');
    });
  });
});
```

## CODE

```javascript
export class UserService {
  register(email, password) {
    if (!this.isValidEmail(email)) {
      throw new Error('Invalid email format');
    }
    
    return {
      success: true,
      user: { email, id: Date.now() }
    };
  }

  isValidEmail(email) {
    const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
    return emailRegex.test(email);
  }
}
```

## REVIEW

✅ All tests pass
✅ Email validation implemented
✅ Error handling included
✅ Code is clean and maintainable

```

### Example 2: PHP Drupal Service
**Prompt**: "Create a Drupal service for content moderation with proper testing"

**Expected Response**:
```

## REASON

I need to create a Drupal service for content moderation. This requires:

- Content state management
- Permission checking
- Workflow transitions
- Integration with Drupal's entity system

## TEST

```php
<?php

namespace Drupal\Tests\content_moderation\Unit;

use PHPUnit\Framework\TestCase;
use Drupal\content_moderation\ContentModerationService;

class ContentModerationServiceTest extends TestCase {
  protected $service;

  protected function setUp(): void {
    $this->service = new ContentModerationService();
  }

  public function testModerateContentWithValidTransition(): void {
    $result = $this->service->moderateContent(1, 'publish');
    $this->assertTrue($result->isSuccess());
    $this->assertEquals('published', $result->getState());
  }

  public function testRejectInvalidTransition(): void {
    $this->expectException(\InvalidArgumentException::class);
    $this->service->moderateContent(1, 'invalid_transition');
  }
}
```

## CODE

```php
<?php

namespace Drupal\content_moderation;

class ContentModerationService {
  public function moderateContent(int $contentId, string $transition): ModerationResult {
    if (!in_array($transition, ['draft', 'review', 'publish'])) {
      throw new \InvalidArgumentException('Invalid transition');
    }
    
    return new ModerationResult(true, $transition);
  }
}
```

## REVIEW

✅ All tests pass
✅ Drupal patterns followed
✅ Error handling implemented
✅ Service is testable and maintainable

```

### Example 3: Python Data Processor
**Prompt**: "Build a data processing pipeline with CSV validation"

**Expected Response**:
```

## REASON

I need to create a data processing pipeline for CSV validation. This requires:

- CSV file reading and parsing
- Data validation rules
- Error reporting
- Performance optimization for large files

## TEST

```python
import pytest
from unittest.mock import Mock, patch
from data_processor import DataProcessor

class TestDataProcessor:
    @pytest.fixture
    def processor(self):
        return DataProcessor()
    
    def test_process_valid_csv(self, processor):
        csv_data = "name,age\nJohn,25\nJane,30"
        result = processor.process_csv(csv_data)
        assert result.is_valid
        assert len(result.records) == 2
    
    def test_reject_invalid_csv_format(self, processor):
        invalid_csv = "name,age\nJohn,invalid_age"
        result = processor.process_csv(invalid_csv)
        assert not result.is_valid
        assert len(result.errors) > 0
```

## CODE

```python
import csv
from dataclasses import dataclass
from typing import List, Dict

@dataclass
class ProcessingResult:
    is_valid: bool
    records: List[Dict]
    errors: List[str]

class DataProcessor:
    def process_csv(self, csv_data: str) -> ProcessingResult:
        errors = []
        records = []
        
        try:
            reader = csv.DictReader(csv_data.splitlines())
            for row in reader:
                if self._validate_row(row):
                    records.append(row)
                else:
                    errors.append(f"Invalid row: [row]")
        except Exception as e:
            errors.append(f"CSV parsing error: [e]")
        
        return ProcessingResult(
            is_valid=len(errors) == 0,
            records=records,
            errors=errors
        )
    
    def _validate_row(self, row: Dict) -> bool:
        return 'name' in row and 'age' in row and row['age'].isdigit()
```

## REVIEW

✅ All tests pass
✅ CSV validation implemented
✅ Error handling comprehensive
✅ Performance considerations included

```

## 🔧 Framework Detection Examples

### JavaScript Project
Create a `package.json`:
```json
{
  "name": "my-project",
  "scripts": {
    "test": "jest"
  },
  "devDependencies": {
    "jest": "^29.0.0"
  }
}
```

### PHP/Drupal Project

Create a `composer.json`:

```json
{
  "name": "my-drupal-module",
  "require": {
    "phpunit/phpunit": "^10.0"
  }
}
```

### Python Project

Create a `pyproject.toml`:

```toml
[tool.pytest.ini_options]
testpaths = ["tests"]
python_files = ["test_*.py"]
```

## 🎭 Context Detection Examples

### New Feature

```
"Implement a new user dashboard with analytics"
```

### Bug Fix

```
"Fix the login authentication bug that occurs with special characters"
```

### Refactoring

```
"Refactor the data processing service to improve performance"
```

### API Development

```
"Create a REST API endpoint for user management"
```

### UI Component

```
"Build a responsive navigation component with accessibility features"
```

## 🚀 Advanced Usage

### Custom Behavior Injection

You can customize the model's behavior by modifying the configuration files:

1. **Framework-specific patterns**: Edit `prompt-templates/`
2. **Context behaviors**: Modify `behavior-injection/context-behaviors.yaml`
3. **Dynamic prompts**: Update `behavior-injection/dynamic-prompts.yaml`

### Quality Gates

The model enforces quality standards automatically:

- 95%+ test coverage
- Code quality tools integration
- Security best practices
- Performance considerations

### Integration with CI/CD

```yaml
# .github/workflows/tdd-ai.yml
name: TDD-AI Validation
on: [push, pull_request]
jobs:
  tdd-ai:
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v3
      - name: Run TDD-AI
        run: |
          ollama run tddai-model "Validate this codebase"
```

## 🆘 Troubleshooting

### Common Issues

1. **Model not responding correctly**
   - Check that Ollama is running: `ollama list`
   - Rebuild the model: `ollama create tddai-model -f Modelfile`

2. **Framework not detected**
   - Ensure project has proper configuration files
   - Check framework detection rules in `dynamic-prompts.yaml`

3. **Tests not generating correctly**
   - Verify framework template exists in `prompt-templates/`
   - Check framework-specific patterns in configuration

### Getting Help

- Review the main `README.md` for detailed documentation
- Check configuration files for customization options
- Test with different project types and contexts

## 🎯 Next Steps

1. **Explore Framework Templates**: Review `prompt-templates/` for your preferred framework
2. **Customize Behavior**: Modify `behavior-injection/` files for your specific needs
3. **Integrate with Projects**: Use TDD-AI in your existing development workflow
4. **Extend Functionality**: Add new frameworks or behavior patterns

---

**Ready to start?** Run `ollama run tddai-model` and ask for help with your next development task!
