# TDD-AI Model Documentation

> Comprehensive technical documentation for the TDD-AI model project and generic model registry component.

## 📚 Documentation Structure

### [User Guide](./user-guide/)
- [Getting Started](./user-guide/getting-started.md) - Quick start guide and installation
- [Model Usage](./user-guide/model-usage.md) - How to use the TDD-AI model
- [TDD Workflow](./user-guide/tdd-workflow.md) - Understanding the AI-powered TDD workflow
- [Examples](./user-guide/examples.md) - Practical examples and use cases
- [Troubleshooting](./user-guide/troubleshooting.md) - Common issues and solutions

### [Development](./development/)
- [Architecture](./development/architecture.md) - System architecture and design patterns
- [Modelfile Reference](./development/modelfile-reference.md) - Complete Modelfile specification
- [Integration Configuration](./development/integration-config.md) - tddai-integration.yaml reference
- [LLM Ecosystem](./development/llm-ecosystem.md) - LLM ecosystem integration details
- [Testing](./development/testing.md) - Testing strategies and validation

### [Components](./components/)
- [Model Registry Component](./components/model-registry.md) - Generic model registry component
- [AI Capabilities](./components/ai-capabilities.md) - AI-powered features and capabilities
- [Workflow Engine](./components/workflow-engine.md) - Multi-step reasoning framework
- [Quality Assessment](./components/quality-assessment.md) - Quality metrics and scoring

### [API](./api/)
- [Model API](./api/model-api.md) - Model interaction and API endpoints
- [TDD-AI Commands](./api/tddai-commands.md) - @bluefly/tddai command reference
- [LLMCLI Integration](./api/llmcli-integration.md) - LLMCLI command reference
- [GitLab Registry API](./api/gitlab-registry-api.md) - GitLab ML Model Registry integration

### [Integration](./integration/)
- [LLM Ecosystem](./integration/llm-ecosystem.md) - Integration with Bluefly LLM projects
- [Drupal Integration](./integration/drupal-integration.md) - Drupal module integration
- [Framework Support](./integration/framework-support.md) - Jest, PHPUnit, React support
- [Development Tools](./integration/development-tools.md) - llmcli, DDEV, GitLab CI/CD

### [Deployment](./deployment/)
- [GitLab CI/CD](./deployment/gitlab-cicd.md) - CI/CD pipeline configuration
- [Model Registry](./deployment/model-registry.md) - Model registration and deployment
- [Quality Gates](./deployment/quality-gates.md) - Quality assurance and validation
- [Monitoring](./deployment/monitoring.md) - Health checks and observability
- [Observability & Tracing](./deployment/observability.md) — Langfuse, Helicone, OpenLLMetry integration and configuration

## 🚀 Quick Start

### Prerequisites
- Node.js 18+ and npm
- PHP 8.1+ and Composer (for Drupal integration)
- Ollama (for local model inference)
- GitLab with ML Model Registry enabled

### Installation
```bash
# Install TDD-AI framework
npm install -g @bluefly/tddai@latest

# Build TDD-AI model
cd tddai-model
ollama create tddai-model -f Modelfile

# Setup for LLM ecosystem
cd /Users/flux423/Sites/LLM/_CommonNPM/llm-gateway
tddai setup --type node --framework jest
tddai preset apply --name enterprise
```

### Basic Usage
```bash
# Start TDD-AI model
ollama run tddai-model

# Generate tests
tddai ai generate-tests -f userService.js --function authenticateUser

# Validate project
tddai validate --strict --coverage 95
```

## 🔧 Configuration

### Model Configuration
- [Modelfile](./development/modelfile-reference.md) - Model definition and parameters
- [Integration Config](./development/integration-config.md) - tddai-integration.yaml
- [Quality Standards](./components/quality-assessment.md) - Quality metrics and thresholds

### LLM Ecosystem Integration
- [Core Projects](./integration/llm-ecosystem.md) - llm-gateway, llm-ui, llm-mcp, etc.
- [DEMO Project](./integration/drupal-integration.md) - Drupal 11.x with custom modules
- [LLMCLI Integration](./api/llmcli-integration.md) - AI-powered CLI toolkit

## 📊 Quality Standards

- **Test Coverage**: 95%+ for all new code
- **TypeScript**: Strict mode enforcement
- **Drupal**: Coding standards compliance
- **Security**: Vulnerability scanning and compliance
- **Performance**: Response time < 200ms, memory < 512MB
- **Accessibility**: WCAG 2.2 AA compliance

## 🔄 TDD Workflow

1. **REASON**: AI-powered requirement analysis with context awareness
2. **TEST**: AI-powered test design using SmartTestGenerator
3. **CODE**: AI-powered implementation with code generation
4. **REVIEW**: AI-powered validation with code analysis
5. **INTEGRATION**: AI-powered compatibility assurance
6. **OPTIMIZATION**: AI-powered performance optimization

## 🏗️ Architecture

### Core Components
- **TDD-AI Model**: Production-ready AI-powered TDD assistant
- **Generic Model Registry**: GitLab ML Model Registry integration
- **AI Capabilities**: SmartTestGenerator, TestOptimizer, PatternLearner
- **Workflow Engine**: Multi-step reasoning framework
- **Quality Assessment**: Automated quality scoring and validation

### Integration Points
- **@bluefly/tddai**: TDD-AI framework integration
- **LLMCLI**: AI-powered CLI toolkit
- **GitLab CI/CD**: Automated testing and deployment
- **Ollama**: Local LLM inference
- **LLM Ecosystem**: Bluefly LLM projects integration

## 📈 Performance Metrics

- **Response Time**: < 2 seconds for model interactions
- **Accuracy**: 95%+ for TDD workflow guidance
- **Coverage**: Comprehensive TDD workflow support
- **Integration**: 100% LLM ecosystem compatibility
- **Quality**: 80/100+ quality assessment score

## 🔒 Security & Compliance

- **Input Validation**: Comprehensive input sanitization
- **Error Handling**: Graceful error recovery and fallbacks
- **Security Patterns**: Following secure_project patterns
- **Compliance**: Government compliance features
- **Access Control**: Authentication and authorization

## 📝 Contributing

1. Follow the TDD workflow: REASON → TEST → CODE → REVIEW → INTEGRATION → OPTIMIZATION
2. Ensure 95%+ test coverage
3. Follow LLM ecosystem patterns
4. Use AI-powered analysis and optimization
5. Validate with @bluefly/tddai and llmcli commands

## 📄 License

This project is licensed under the MIT License - see the [LICENSE](../LICENSE) file for details.

## 🆘 Support

For support and questions:
- Check the [@bluefly/tddai documentation](https://github.com/bluefly/tddai)
- Review [llmcli help commands](./api/llmcli-integration.md)
- Consult the [LLM ecosystem patterns](./integration/llm-ecosystem.md)
- Use [AI-powered analysis tools](./components/ai-capabilities.md)

---

**Last Updated**: January 27, 2025  
**Version**: 2.0.0  
**Documentation Version**: 1.0.0

# Documentation Index

This directory contains all project documentation, organized by topic and module.

## Structure
- `api/` — API reference and OpenAPI docs
- `components/` — Component-level documentation
- `deployment/` — Deployment guides and status
- `development/` — Architecture and development practices
- `integration/` — Integration guides
- `user-guide/` — End-user and onboarding guides

## Contribution Guide
- Add new docs in the appropriate subdirectory.
- Keep documentation modular and up to date.
- Follow accessibility and clarity best practices. 