# Getting Started with TDD-AI Model

> Quick start guide for setting up and using the TDD-AI model for production-ready AI-powered test-driven development.

## Prerequisites

### System Requirements

- **Operating System**: macOS, Linux, or Windows (WSL2)
- **Node.js**: 18.0.0 or higher
- **npm**: 9.0.0 or higher
- **PHP**: 8.1.0 or higher (for Drupal integration)
- **Composer**: 2.0.0 or higher (for Drupal integration)
- **Git**: 2.30.0 or higher
- **Docker**: 20.10.0 or higher (optional, for containerized deployment)

### Required Software

- **Ollama**: For local LLM inference
- **GitLab**: With ML Model Registry enabled
- **DDEV**: For local Drupal development (optional)

## Installation

### Step 1: Install TDD-AI Framework

```bash
# Install the TDD-AI framework globally
npm install -g @bluefly/tddai@latest

# Verify installation
tddai --version
```

### Step 2: Install Ollama

```bash
# macOS (using Homebrew)
brew install ollama

# Linux
curl -fsSL https://ollama.ai/install.sh | sh

# Windows (WSL2)
curl -fsSL https://ollama.ai/install.sh | sh

# Start Ollama service
ollama serve
```

### Step 3: Build TDD-AI Model

```bash
# Clone the TDD-AI model repository
git clone https://gitlab.bluefly.io/llm/models/tddai-model.git
cd tddai-model

# Build the model with Ollama
ollama create tddai-model -f Modelfile

# Verify model creation
ollama list
```

### Step 4: Setup LLM Ecosystem Integration

```bash
# Navigate to LLM ecosystem
cd /Users/flux423/Sites/LLM

# Setup llm-gateway project
cd _CommonNPM/llm-gateway
tddai setup --type node --framework jest
tddai preset apply --name enterprise

# Setup llm-ui project
cd ../llm-ui
tddai setup --type react --framework jest
tddai preset apply --name react

# Setup DEMO Drupal project
cd ../../DEMO
tddai setup --type drupal --framework phpunit
tddai preset apply --name drupal
```

### Step 5: Configure AI Integration

```bash
# Setup AI integration for all projects
tddai ai setup-all

# Verify LLMCLI installation
llmcli --version

# Check ecosystem status
llmcli ecosystem status
```

## Configuration

### Environment Variables

Create a `.env` file in your project root:

```bash
# TDD-AI Configuration
TDDAI_MODEL_NAME=tddai-model
TDDAI_VALIDATION_LEVEL=comprehensive
TDDAI_COVERAGE_THRESHOLD=95

# LLM Ecosystem Paths
LLM_ECOSYSTEM_ROOT=/Users/flux423/Sites/LLM
LLM_GATEWAY_PATH=/Users/flux423/Sites/LLM/_CommonNPM/llm-gateway
LLM_UI_PATH=/Users/flux423/Sites/LLM/_CommonNPM/llm-ui
DEMO_PROJECT_PATH=/Users/flux423/Sites/LLM/DEMO

# GitLab Configuration
GITLAB_URL=https://gitlab.bluefly.io
GITLAB_PROJECT_ID=your-project-id
GITLAB_TOKEN=your-gitlab-token

# AI Provider Configuration (optional)
OPENAI_API_KEY=your-openai-key
ANTHROPIC_API_KEY=your-anthropic-key
```

### Model Configuration

The TDD-AI model is configured through several key files:

1. **Modelfile**: Model definition and parameters
2. **tddai-integration.yaml**: Integration configuration
3. **.gitlab-ci.yml**: CI/CD pipeline configuration

## Quick Start Examples

### Basic Model Usage

```bash
# Start the TDD-AI model
ollama run tddai-model

# Test the model with a simple prompt
echo "Write a Jest test for a user authentication function" | ollama run tddai-model
```

### TDD Workflow Example

```bash
# 1. REASON: Analyze requirements
echo "I need to create a user authentication service" | ollama run tddai-model

# 2. TEST: Generate tests
tddai ai generate-tests -f userService.js --function authenticateUser

# 3. CODE: Implement the service
# (Model will guide you through implementation)

# 4. REVIEW: Validate implementation
tddai validate --strict --coverage 95

# 5. INTEGRATION: Check ecosystem compatibility
llmcli ecosystem health

# 6. OPTIMIZATION: Optimize performance
tddai ai optimize-tests -d ./tests --analyze-only
```

### LLM Ecosystem Integration

```bash
# Check ecosystem status
llmcli ecosystem status

# Validate specific project
cd /Users/flux423/Sites/LLM/_CommonNPM/llm-gateway
tddai validate --strict --coverage 95

# Generate tests for LLM project
tddai ai generate-tests -f apiGateway.ts --edge-cases --security --performance

# Analyze code quality
llmcli ai analyze ./src --type quality --recursive
```

### Drupal Integration

```bash
# Navigate to DEMO project
cd /Users/flux423/Sites/LLM/DEMO

# Validate Drupal module
tddai drupal validate-ai-tdd -m web/modules/custom/llm

# Generate tests for Drupal service
tddai ai generate-tests -f llm.module.php --function generateResponse

# Check Drupal ecosystem
llmcli drupal stats --detailed
```

## Validation and Testing

### Model Validation

```bash
# Validate model configuration
tddai validate --config tddai-integration.yaml

# Run comprehensive validation
tddai validate --strict --coverage 95

# Check model quality
tddai ai smart-analyze -f Modelfile --report json
```

### Quality Assessment

```bash
# Assess code quality
tddai ai learn-patterns -p ./src --save-patterns

# Optimize tests
tddai ai optimize-tests -d ./tests --parallel --cache

# Monitor compliance
tddai monitor --interval 30
```

### GitLab Integration

```bash
# Register model in GitLab ML Model Registry
# (This happens automatically via CI/CD pipeline)

# Check model registry status
curl -H "PRIVATE-TOKEN: $GITLAB_TOKEN" \
  "$GITLAB_URL/api/v4/projects/$GITLAB_PROJECT_ID/ml/model_registry"
```

## Troubleshooting

### Common Issues

**Model not found:**

```bash
# Rebuild the model
ollama create tddai-model -f Modelfile

# Check available models
ollama list
```

**TDDAI command not found:**

```bash
# Reinstall TDDAI
npm install -g @bluefly/tddai@latest

# Check installation
which tddai
```

**LLM ecosystem path issues:**

```bash
# Verify paths
ls -la /Users/flux423/Sites/LLM/_CommonNPM/

# Update configuration
tddai ai setup-all --standards .tddai/standards.yaml
```

### Performance Optimization

```bash
# Optimize model performance
ollama run tddai-model --num-ctx 8192 --num-thread 8

# Monitor resource usage
tddai ai optimize-tests --analyze-only --performance
```

## Next Steps

1. **Read the Documentation**: Explore the [full documentation](../README.md)
2. **Try Examples**: Check out [practical examples](./examples.md)
3. **Learn TDD Workflow**: Understand the [AI-powered TDD workflow](./tdd-workflow.md)
4. **Explore Integration**: Learn about [LLM ecosystem integration](../integration/llm-ecosystem.md)
5. **Deploy to Production**: Follow the [deployment guide](../deployment/gitlab-cicd.md)

## Support

- **Documentation**: [Full documentation](../README.md)
- **Issues**: [GitLab Issues](https://gitlab.bluefly.io/llm/models/tddai-model/-/issues)
- **Community**: [Bluefly AI Community](https://community.bluefly.io)
- **Email**: ai@bluefly.io

---

**Last Updated**: January 27, 2025  
**Version**: 2.0.0
