---
id: modules-development-drupal_migration_plus-ENHANCED_MIGRATION_GUIDE
---
# Enhanced Migration Guide with Drupal Migration Plus and Rector

This guide provides a comprehensive approach to significantly enhance your Drupal migration using Drupal Migration Plus and Rector.

## 1. Setting Up Drupal Migration Plus

### Install Required Dependencies

```bash
# Ensure required modules are installed
composer require drupal/migrate_plus:^6.0
composer require drupal/eca:^1.0
composer require drupal/jsonapi_extras:^3.0
composer require drupal/ai:^1.0
```

### Enable Core and Submodules

```bash
# Enable core module
drush en drupal_migration_plus -y

# Enable source-specific submodule (choose based on your source CMS)
drush en drupal_migration_plus_wordpress -y  # For WordPress
# OR
drush en drupal_migration_plus_drupal7 -y    # For Drupal 7
# OR
drush en drupal_migration_plus_joomla -y     # For Joomla
# OR
drush en drupal_migration_plus_sitecore -y   # For Sitecore

# Enable Experience Builder for visual field mapping
drush en drupal_migration_plus_experience_builder -y

# Enable AI integration for intelligent mapping
drush en drupal_migration_plus_ai -y

# Enable theme migration support
drush en drupal_migration_plus_themes -y
```

### Configure Database Connection

Add the following to your `settings.php` file:

```php
// For Drupal 7 source
$databases['migrate']['default'] = [
  'database' => 'source_db_name',
  'username' => 'db_user',
  'password' => 'db_password',
  'prefix' => '',
  'host' => 'localhost',
  'port' => '3306',
  'namespace' => 'Drupal\\Core\\Database\\Driver\\mysql',
  'driver' => 'mysql',
];

// File paths for Drupal 7
$settings['migrate_source_base_path'] = '/path/to/drupal7/files';
$settings['migrate_source_private_file_path'] = '/path/to/drupal7/private/files';

// For WordPress source (if applicable)
$databases['wordpress_migrate']['default'] = [
  'database' => 'wordpress_db',
  'username' => 'db_user',
  'password' => 'db_password',
  'prefix' => 'wp_',
  'host' => 'localhost',
  'port' => '3306',
  'driver' => 'mysql',
];
$settings['wordpress_migrate_files_directory'] = '/path/to/wordpress/wp-content/uploads';
```

## 2. Leveraging Rector for Code Modernization

### Configure Rector for Drupal-specific Rules

Create an enhanced `rector.php` configuration file:

```php
<?php

declare(strict_types=1);

use DrupalRector\Set\Drupal9SetList;
use DrupalRector\Set\Drupal10SetList;
use DrupalRector\Set\Drupal11SetList;
use Rector\Config\RectorConfig;
use Rector\Set\ValueObject\LevelSetList;
use Rector\Set\ValueObject\SetList;

return static function (RectorConfig $rectorConfig): void {
    // Paths to process
    $rectorConfig->paths([
        // Your custom code
        __DIR__ . '/src',
        __DIR__ . '/modules',
        __DIR__ . '/tests/src',
        // Include migration-specific code
        __DIR__ . '/migrations',
    ]);

    // Skip certain files
    $rectorConfig->skip([
        __DIR__ . '/vendor',
        __DIR__ . '/node_modules',
    ]);

    // PHP version target
    $rectorConfig->phpVersion(\Rector\ValueObject\PhpVersion::PHP_83);

    // Basic rule sets
    $rectorConfig->sets([
        LevelSetList::UP_TO_PHP_83,
        SetList::CODE_QUALITY,
        SetList::DEAD_CODE,
        SetList::EARLY_RETURN,
        SetList::TYPE_DECLARATION,
        
        // Drupal-specific sets
        Drupal9SetList::DRUPAL_9,
        Drupal10SetList::DRUPAL_10,
        Drupal11SetList::DRUPAL_11,
    ]);

    // Performance settings
    $rectorConfig->parallel();
    $rectorConfig->importNames();
    $rectorConfig->cacheDirectory(__DIR__ . '/.rector-cache');
};
```

### Run Rector to Fix Deprecated Code

```bash
# Analyze code first (dry run)
vendor/bin/rector process --dry-run

# Apply fixes
vendor/bin/rector process

# You can also run on specific directories
vendor/bin/rector process src/SourceAdapter --dry-run
```

## 3. Migration Workflow with Drupal Migration Plus

### Step 1: Source Analysis and Discovery

Access the migration dashboard at `/admin/migrate/plus/recipe-onboarding` and start the source analysis process:

1. Select your source system type
2. Enter connection details
3. Click "Analyze Source"

This will scan your source system and provide:
- Content type mapping suggestions
- Field structure analysis
- Media and file detection
- Entity relationships mapping
- Theme and configuration analysis

### Step 2: Field Mapping with Experience Builder

Access the field mapping interface at `/admin/migrate/plus/field-mapping`:

1. Select source content type
2. Select target content type
3. Use the visual drag-and-drop interface to map fields
4. Review AI-suggested mappings (highlighted in blue)
5. Configure transformations for complex field types

For Drupal-to-Drupal migrations, use the specialized interface at `/admin/migrate/plus/field-mapping/drupal-to-drupal`.

### Step 3: Configure ECA Workflows

Set up automated migration workflows with ECA:

1. Go to `/admin/config/workflow/eca`
2. Create new workflow or use pre-configured templates
3. Configure events for migration triggers
4. Set up conditions for validation rules
5. Define actions for post-migration tasks

Example workflows:
- Pre-migration validation
- Post-migration content cleanup
- Media processing and optimization
- URL path generation
- Taxonomy term deduplication

### Step 4: Test Migration

Run a test migration with a limited dataset:

1. From the migration dashboard, select "Test Migration"
2. Set batch size (recommended: 50 items)
3. Select specific content to migrate
4. Run the test migration
5. Review results and logs

### Step 5: Execute Full Migration

Once satisfied with test results:

1. Return to the migration dashboard
2. Configure parallel processing settings (threads, batch size)
3. Set up error handling preferences
4. Start the full migration process
5. Monitor progress in real-time

### Step 6: Post-Migration Validation

Verify migration success:

1. Run built-in validation checks
2. Check entity references integrity
3. Verify media and file migrations
4. Test multilingual content (if applicable)
5. Run performance audits

## 4. Advanced Features

### Parallel Processing for Large Datasets

For large migrations, configure parallel processing:

1. Go to `/admin/config/migrate/plus/parallel-settings`
2. Set number of worker threads (based on server capacity)
3. Configure batch size per thread
4. Set memory limits and timeouts
5. Enable distributed queue for multi-server processing

### AI-Enhanced Field Mapping

Leverage AI for complex mapping scenarios:

1. Go to `/admin/config/migrate/plus/ai-settings`
2. Configure AI provider (OpenAI/Claude)
3. Set intelligence level for mapping suggestions
4. Enable content analysis for improved mapping
5. Configure learning from mapping patterns

### Theme Migration with Computer Vision

For visual theme migration:

1. Go to `/admin/config/migrate/plus/theme-migration`
2. Upload screenshots or provide source site URL
3. The system will analyze visual design elements
4. Extract color schemes, typography, and layout
5. Generate Drupal theme with matching visual design

### Custom Migration Plugins

For specialized migration needs:

1. Create custom source plugins in `src/Plugin/migrate/source/`
2. Create custom process plugins in `src/Plugin/migrate/process/`
3. Configure through the UI or YAML configuration
4. Integrate with the Recipe system for reusability

## 5. Troubleshooting Common Issues

### Memory Limitations

If encountering memory errors:

```php
// In settings.php
ini_set('memory_limit', '2G');
```

Also, reduce batch size and enable garbage collection.

### Timeout Issues

For long-running migrations:

```php
// In settings.php
ini_set('max_execution_time', '3600');
```

Consider using the parallel processing feature for better performance.

### Database Connection Issues

Verify database credentials and ensure the source database is accessible from the migration environment.

### Field Mapping Errors

Check for:
- Missing fields in target content types
- Field type incompatibilities
- Entity reference configuration
- File and media path configuration

## 6. Best Practices

1. **Always backup** your source and target databases before migration
2. Use **version control** for all custom migration code
3. Run Rector **before and after** major migration steps
4. Implement **comprehensive testing** for migration results
5. Use the **Recipe system** for reusable migration patterns
6. Enable **detailed logging** during migration process
7. Validate **entity relationships** post-migration
8. Use **ECA workflows** for automated validation and cleanup
9. Implement **rollback capabilities** for failed migrations
10. Document all **custom transformations** and special handling

---

This guide provides a comprehensive approach to enhancing your migration using the powerful combination of Drupal Migration Plus and Rector.
