---
id: modules-development-drupal_migration_plus-INSTALLATION
---
# Installation Guide

This guide provides instructions for installing and configuring the Drupal Migration Plus module.

## Requirements

Before installing, ensure your system meets these requirements:

- Drupal 10.1 or later
- PHP 8.1 or later
- MySQL 8.0+ or MariaDB 10.6+
- Composer (recommended for dependency management)
- Required modules:
  - Migrate (Drupal core)
  - Migrate Drupal (Drupal core)
  - Migrate Drupal UI (Drupal core)
  - Migrate Plus
  - Migrate Tools

### Recommended Server Configuration

- Memory: 512MB minimum, 2GB recommended
- PHP max_execution_time: 300 seconds minimum
- PHP memory_limit: 512MB minimum
- MySQL max_allowed_packet: 64MB

## Installation Methods

### Using Composer (Recommended)

1. Add the Drupal package repository if you haven't already:

```bash
composer config repositories.drupal composer https://packages.drupal.org/8
```

2. Require the module and its dependencies:

```bash
composer require drupal/drupal_migration_plus drupal/migrate_plus drupal/migrate_tools
```

3. Enable the module and its dependencies:

```bash
drush en drupal_migration_plus -y
```

### Manual Installation

1. Download the module from [drupal.org/project/drupal_migration_plus](https://www.drupal.org/project/drupal_migration_plus)
2. Download the required dependencies:
   - [Migrate Plus](https://www.drupal.org/project/migrate_plus)
   - [Migrate Tools](https://www.drupal.org/project/migrate_tools)
3. Extract all modules to your modules directory (e.g., `modules/contrib/`)
4. Enable the module through the UI:
   - Navigate to Administration » Extend
   - Check "Drupal Migration Plus"
   - Click "Install"
5. Alternatively, enable using Drush:

```bash
drush en drupal_migration_plus -y
```

## Post-Installation Configuration

### Basic Configuration

1. Set Permissions:
   - Navigate to Administration » People » Permissions
   - Configure appropriate permissions:
     - "Administer Drupal Migration Plus": Full access to migration features
     - "Access Migration Plus Reports": View migration reports and status

2. Configure Migration Settings:
   - Navigate to Administration » Configuration » Development » Migration Plus Settings
   - Configure default migration settings
   - Set up source and destination mapping preferences
   - Configure performance settings

3. Source Database Configuration:
   - Add source database configuration to `settings.php`:

```php
// Source database configuration
$databases['migrate']['default'] = [
  'database' => 'source_db',
  'username' => 'db_user',
  'password' => 'db_password',
  'host' => 'localhost',
  'port' => '3306',
  'driver' => 'mysql',
  'prefix' => '',
  'charset' => 'utf8mb4',
  'collation' => 'utf8mb4_general_ci',
];

// File path configuration
$settings['migrate_source_base_path'] = '/path/to/source/files';
$settings['migrate_source_private_file_path'] = '/path/to/private/files';
```

### Advanced Configuration

#### Performance Optimization

For large migrations, add to `settings.php`:

```php
// Increase memory for migrations
ini_set('memory_limit', '2G');
ini_set('max_execution_time', 0);

// Migration-specific cache configuration
$settings['cache']['bins']['drupal_migration_plus'] = 'cache.backend.database';

// Enable migration logging
$config['system.logging']['error_level'] = 'verbose';
```

#### Database Optimization

For large migrations, optimize your database configuration:

```sql
-- MySQL configuration recommendations
SET GLOBAL innodb_buffer_pool_size = 2G;
SET GLOBAL max_allowed_packet = 64M;
SET GLOBAL innodb_log_file_size = 512M;
```

## Recommended Modules

For enhanced functionality, consider installing these additional modules:

- **Migrate Upgrade**: For upgrading from previous Drupal versions
- **Migrate Source CSV**: For CSV migration sources
- **Migrate Source JSON**: For JSON migration sources
- **JSON:API**: For API-based migrations
- **Devel**: For debugging migrations

## Verification

### Check Module Status

Verify installation via Drush:

```bash
drush pml | grep migration_plus
```

Expected output:
```
drupal_migration_plus          Enabled
```

### Verify UI Access

Navigate to Administration » Migration to ensure you can access the migration dashboard.

## Troubleshooting

### Common Issues

#### Module Not Found

**Error**: "The drupal_migration_plus module is not found"

**Solution**:
```bash
# Clear cache and rebuild
drush cr
```

#### Database Connection Issues

**Error**: "Migration source database connection failed"

**Solution**:
1. Verify database credentials in `settings.php`
2. Test database connection:
```bash
drush sql:connect --database=migrate
```

#### Memory Issues

**Error**: "Fatal error: Allowed memory size exhausted"

**Solution**:
1. Increase PHP memory limit in `php.ini`:
```ini
memory_limit = 2G
```
2. For specific migrations, increase in `settings.php`:
```php
ini_set('memory_limit', '4G');
```

### Getting Help

If you continue to experience issues:

1. Check the [troubleshooting guide](troubleshooting) for more detailed information
2. Visit the [issue queue](https://www.drupal.org/project/issues/drupal_migration_plus) on Drupal.org
3. Join the [Drupal Migration group](https://groups.drupal.org/migration) for community support

## Next Steps

After successful installation:

1. Create your first migration recipe
2. Configure source and destination mappings
3. Run test migrations with small datasets
4. Review the [User Guide](user-guide) for detailed usage instructions

---

**Note**: Always test migrations in a development environment before running them on production systems.
