---
id: modules-development-langchain-ECA_CONDITIONS
---
# LangChain: ECA Condition Plugins

This document provides detailed information about the ECA condition plugins provided by the LangChain module. These condition plugins enhance the Event-Condition-Action (ECA) framework by adding LangChain-specific conditions that can be used in workflow automation and AI pipeline orchestration.

## Available Condition Plugins

### Chain Success Rate

**ID**: `chain_success_rate`

Evaluates if a LangChain chain or agent meets a specified success rate threshold. This condition allows you to create workflows based on the performance metrics of your AI chains and agents.

**Configuration Options**:
- **Target type**: What to check for success rate:
  - Chain: Check a specific chain
  - Agent: Check a specific agent
  - Category: Check all chains in a category
  - All chains: Check all chains in the system
- **Chain ID**: The ID of the specific chain to check (if target type is "Chain")
- **Agent ID**: The ID of the specific agent to check (if target type is "Agent")
- **Chain category**: The category of chains to check (if target type is "Category")
- **Time period**: The time period to consider for success rate calculation:
  - Last hour
  - Last 6 hours
  - Last 24 hours
  - Last 7 days
  - Last 30 days
- **Minimum executions**: The minimum number of executions required for the condition to be valid
- **Metric type**: The type of metric to check:
  - Success rate (%)
  - Average execution time (seconds)
  - Error rate (%)
  - Total executions
- **Threshold**: The threshold value to compare against
- **Operator**: The operator to use when comparing the metric with the threshold
- **Use token replacement**: Whether to use token replacement for chain ID, agent ID, and category values

**Use Cases**:
- Automatically adjust chain routing based on performance metrics
- Implement fallback paths when chains experience high error rates
- Create alerts when execution times exceed thresholds
- Identify and remediate poorly performing AI chains
- Schedule maintenance based on usage patterns
- Implement adaptive load balancing across models
- Enforce quality standards for AI-driven processes

## Integration with ECA Workflow

To use these conditions in your ECA workflows:

1. Navigate to the ECA configuration page (`/admin/config/workflow/eca`)
2. Create a new ECA model or edit an existing one
3. Add an event that should trigger your workflow
4. Add the chain success rate condition
5. Configure the condition with the appropriate chain and metric settings
6. Add actions to execute when the condition is met

## Example Configurations

### Check Chain Success Rate

```yaml
conditions:
  condition_1:
    plugin: chain_success_rate
    configuration:
      target_type: chain
      chain_id: document_analyzer
      time_period: 24h
      min_executions: 10
      metric_type: success_rate
      success_threshold: 90
      operator: '>='
```

### Monitor Execution Time

```yaml
conditions:
  condition_1:
    plugin: chain_success_rate
    configuration:
      target_type: chain
      chain_id: real_time_translator
      time_period: 1h
      min_executions: 5
      metric_type: average_execution_time
      success_threshold: 2.0
      operator: '<'
```

### Check Category Performance

```yaml
conditions:
  condition_1:
    plugin: chain_success_rate
    configuration:
      target_type: category
      category: text_generation
      time_period: 7d
      min_executions: 50
      metric_type: error_rate
      success_threshold: 5
      operator: '<'
```

## Programmatic Usage

These conditions can also be used programmatically in custom code:

```php
// Example of programmatically checking chain success rate
$condition_manager = \Drupal::service('plugin.manager.eca.condition');
$condition = $condition_manager->createInstance('chain_success_rate', [
  'target_type' => 'chain',
  'chain_id' => 'document_analyzer',
  'time_period' => '24h',
  'min_executions' => 10,
  'metric_type' => 'success_rate',
  'success_threshold' => 90,
  'operator' => '>=',
]);
$condition->setEvent($event);
$result = $condition->evaluate();
```

## Understanding Chain Performance Metrics

LangChain tracks several performance metrics for chains and agents:

### Success Rate
The percentage of chain executions that completed successfully without errors.

### Error Rate
The percentage of chain executions that resulted in errors.

### Average Execution Time
The average time (in seconds) it takes to execute the chain from start to finish.

### Total Executions
The total number of times the chain has been executed in the specified time period.

## Execution History and Storage

Performance metrics are calculated from execution history, which is stored in two locations:

1. **Primary Storage**: The LangChain service's execution history
2. **Fallback Storage**: State API storage as a backup

The condition will automatically fall back to State API storage if the primary storage is unavailable.

## Chain Categories

Chains can be organized into categories for easier management:

- **text_generation**: General text generation chains
- **chat**: Interactive conversation chains
- **document_processing**: Document analysis and extraction chains
- **data_analysis**: Data processing and analysis chains
- **search**: Search enhancement chains
- **code**: Code generation and analysis chains

Categories can be used to track performance across related chains.

## Time Period Considerations

When selecting a time period:

1. **Shorter periods** (1h, 6h) are useful for detecting immediate issues
2. **Medium periods** (24h) provide a balanced view of performance
3. **Longer periods** (7d, 30d) help identify long-term trends

Ensure your time period contains enough executions to be statistically significant.

## Minimum Executions

The minimum executions setting prevents the condition from evaluating chains with insufficient data.

This is important because:
- Small sample sizes can lead to misleading success rates
- A single error in a small sample can skew the error rate dramatically
- Meaningful performance patterns require sufficient data points

## Metric Selection Guidelines

Choose metrics based on what's most important for your use case:

- **Success Rate**: Best for mission-critical chains where reliability is essential
- **Error Rate**: Useful for detecting problematic chains
- **Execution Time**: Important for real-time or user-facing applications
- **Total Executions**: Helpful for usage-based routing or load balancing

## Threshold Selection

When setting thresholds:

1. Start with baseline measurements of current performance
2. Set realistic thresholds based on observed performance
3. Gradually increase thresholds as chain performance improves
4. Consider different thresholds for different chain categories

## Troubleshooting

If you encounter issues with chain success rate conditions:

1. Verify that the chain or agent exists in the system
2. Check that there is sufficient execution history available
3. Ensure the minimum executions setting is appropriate
4. Review ECA logs for detailed information about condition evaluation

For more complex issues, enable debug mode in LangChain settings to get detailed information about metric calculations.

## Performance Optimization

For large-scale deployments:

1. Use targeted conditions rather than checking all chains
2. Select appropriate time periods to limit the amount of data processed
3. Consider scheduled evaluations instead of event-triggered checks for resource-intensive metrics
4. Implement caching for frequently checked metrics

## Integration with Monitoring Systems

Chain performance metrics can be integrated with external monitoring systems:

1. Use ECA actions to send metrics to external systems
2. Create scheduled evaluations to maintain performance dashboards
3. Implement automated alerts for performance degradation
