---
id: modules-development-langchain-deployment-DEPLOYMENT
---
# LangChain Deployment Guide

This guide provides detailed instructions for deploying the LangChain Drupal module in various environments, from development to high-availability production setups.

## Table of Contents

- [Prerequisites](#prerequisites)
- [Deployment Methods](#deployment-methods)
  - [Automated Deployment Script](#automated-deployment-script)
  - [Docker Development Environment](#docker-development-environment)
  - [Manual Installation](#manual-installation)
- [Environment-Specific Configurations](#environment-specific-configurations)
  - [Development](#development)
  - [Staging](#staging)
  - [Production](#production)
  - [Performance-Optimized](#performance-optimized)
  - [High-Availability](#high-availability)
- [Post-Deployment Validation](#post-deployment-validation)
- [Troubleshooting](#troubleshooting)
- [Maintenance and Upgrades](#maintenance-and-upgrades)
- [Advanced Configuration](#advanced-configuration)

## Prerequisites

Before deploying the LangChain module, ensure your environment meets the following requirements:

- PHP 8.2 or higher
- Drupal 10 or 11
- Composer
- Drush
- Required PHP extensions: json, pdo, pdo_mysql, gd, xml, mbstring, curl, openssl
- Recommended: Redis for caching
- Recommended: A vector database (e.g., Meilisearch, Elasticsearch, or Pinecone)
- API keys for your LLM providers (e.g., OpenAI, Anthropic)

## Deployment Methods

### Automated Deployment Script

The module provides a comprehensive deployment script (`deploy-langchain.sh`) that automates the installation and configuration process.

#### Basic Usage

```bash
# Deploy with default settings (development environment)
./deploy-langchain.sh

# Deploy to production
./deploy-langchain.sh --environment production

# Deploy to staging with custom Drupal root
./deploy-langchain.sh --environment staging --drupal-root /var/www/html

# Deploy with specific options
./deploy-langchain.sh --environment production --skip-tests --skip-optional
```

#### Available Options

Run `./deploy-langchain.sh --help` for a full list of options. Key options include:

- `--environment` (or `-e`): Set the deployment environment (dev, staging, production, performance, ha)
- `--drupal-root` (or `-d`): Set the Drupal root path
- `--skip-dependencies`: Skip dependency installation
- `--skip-enable`: Skip module enabling
- `--skip-validation`: Skip post-installation validation
- `--skip-configuration`: Skip configuration setup
- `--skip-optional`: Skip optional dependencies
- `--skip-tests`: Skip running tests

### Docker Development Environment

The module includes a Docker setup for development purposes, providing a complete environment with all dependencies.

#### Starting the Docker Environment

```bash
# Start the development environment
docker-compose up -d

# Access the application at http://localhost:8080
# Default admin credentials: admin/admin

# Stop the environment
docker-compose down

# Stop and remove volumes
docker-compose down -v
```

#### Docker Services

The Docker environment includes:

- Drupal with PHP 8.2
- MySQL 8.0
- Redis 7
- Meilisearch (Vector DB)
- Langfuse (Observability)
- LLM Platform service
- Traefik for routing
- PHPMyAdmin for database management

### Manual Installation

If you prefer to install manually, follow these steps:

1. Install the module in your Drupal site:
   ```bash
   composer require bluefly/langchain
   drush en langchain
   ```

2. Configure the module settings:
   - Visit `/admin/config/services/langchain/settings`
   - Set up API keys via the Key module
   - Configure providers and default settings

3. Install optional dependencies as needed:
   ```bash
   drush en eca search_api views views_bulk_operations token
   ```

4. Clear caches:
   ```bash
   drush cr
   ```

## Environment-Specific Configurations

The module provides optimized configurations for different environments. These are managed by the deployment script and stored in `config/deployment/[environment]/`.

### Development

The development environment focuses on:
- Mock providers for testing without real API costs
- Debugging tools enabled
- Verbose logging
- Development PHP settings
- Test data and examples

To configure manually:
```bash
cp config/deployment/dev/config.yml /path/to/your/drupal/config/sync/
drush cim -y
```

### Staging

The staging environment balances between development and production:
- Real providers but with restricted usage
- Moderate caching
- Warning-level logging
- Performance optimizations enabled
- Security features active

To configure manually:
```bash
cp config/deployment/staging/config.yml /path/to/your/drupal/config/sync/
drush cim -y
```

### Production

The production environment prioritizes:
- Security
- Reliability
- Performance
- Monitoring
- Error recovery

Configuration focuses on:
- Redis caching
- OpCache optimization
- Circuit breakers for API failures
- Rate limiting
- Production-grade error handling

To configure manually:
```bash
cp config/deployment/production/config.yml /path/to/your/drupal/config/sync/
drush cim -y
```

### Performance-Optimized

The performance-optimized environment maximizes throughput and minimizes latency:
- Aggressive caching strategies
- Advanced Redis configuration
- Asset optimization
- Memory allocation optimization
- Response compression
- Batch processing

To configure manually:
```bash
cp config/deployment/performance/config.yml /path/to/your/drupal/config/sync/
drush cim -y
```

### High-Availability

The high-availability environment ensures fault tolerance and continuous operation:
- Multi-node clustering
- Redis Sentinel/Cluster
- RabbitMQ for reliable queues
- Service discovery with Consul
- Load balancing with HAProxy
- Distributed tracing
- Automatic failover

To configure manually:
```bash
cp config/deployment/ha/config.yml /path/to/your/drupal/config/sync/
drush cim -y
```

## Post-Deployment Validation

After deployment, verify that the module is working correctly:

1. Check that the module is enabled:
   ```bash
   drush pm:list | grep langchain
   ```

2. Verify services are available:
   ```bash
   drush ev "return \Drupal::hasService('langchain.service') ? 'OK' : 'FAIL';"
   ```

3. Access the admin interface:
   - Visit `/admin/config/services/langchain`
   - Check that you can create and manage chains and agents

4. Test API endpoints:
   ```bash
   curl -X GET http://your-site.com/api/langchain/orchestrate/providers \
     -H "Content-Type: application/json" \
     -H "Authorization: Bearer YOUR_TOKEN"
   ```

5. For Docker deployments, use the health endpoint:
   ```bash
   curl http://localhost:8080/health
   ```

## Troubleshooting

### Common Issues

1. **Missing Dependencies**
   - Check PHP version: `php -v`
   - Verify Drupal version: `drush status`
   - Check installed modules: `drush pm:list`

2. **API Connection Issues**
   - Verify API keys are correctly set in the Key module
   - Check network connectivity to API providers
   - Look for timeout errors in the logs

3. **Performance Problems**
   - Check Redis connectivity and performance
   - Verify OpCache settings
   - Monitor memory usage and PHP limits

4. **Docker Issues**
   - Check container status: `docker-compose ps`
   - View logs: `docker-compose logs [service]`
   - Verify networking between containers

### Logs and Debugging

Key log locations:
- Drupal logs: `drush watchdog:show`
- PHP logs: Check your PHP configuration
- Apache/Nginx logs: Check your web server configuration
- Docker logs: `docker-compose logs`

Enable debug mode for more verbose logging:
```bash
drush config:set langchain.settings debug_mode 1 -y
```

## Maintenance and Upgrades

### Regular Maintenance

1. Check for updates:
   ```bash
   composer outdated "bluefly/*"
   ```

2. Monitor API usage and costs:
   - Visit `/admin/reports/langchain/usage`
   - Check your LLM provider dashboards

3. Optimize cache and performance:
   ```bash
   drush cache:rebuild
   ```

4. Perform security updates promptly:
   ```bash
   composer update bluefly/langchain --with-dependencies
   ```

### Upgrading

1. Back up your site:
   ```bash
   drush sql:dump > langchain_before_upgrade.sql
   ```

2. Update the module:
   ```bash
   composer update bluefly/langchain
   ```

3. Run database updates:
   ```bash
   drush updatedb
   ```

4. Import new configuration:
   ```bash
   drush config:import
   ```

5. Clear caches:
   ```bash
   drush cache:rebuild
   ```

## Advanced Configuration

### Custom Providers

Create custom LLM providers by:
1. Creating a new class that implements `LangChainProviderInterface`
2. Registering it with the `@LangChainProvider` annotation
3. Clearing caches to discover the new provider

Example:
```php
/**
 * @LangChainProvider(
 *   id = "custom_provider",
 *   label = @Translation("Custom Provider"),
 *   description = @Translation("A custom LLM provider.")
 * )
 */
class CustomProvider extends LangChainProviderBase {
  // Implementation here
}
```

### Environment Variables

Key environment variables:
- `OPENAI_API_KEY`: API key for OpenAI
- `LANGFUSE_API_KEY`: API key for Langfuse observability
- `REDIS_HOST`: Redis host for caching
- `VECTOR_DB_URL`: URL for vector database

For Docker, these can be set in `.env` or passed directly to docker-compose.

### Performance Tuning

For high-traffic sites:
1. Increase PHP memory limits
2. Optimize Redis configuration
3. Consider using a CDN for static assets
4. Enable OpCache with optimal settings
5. Use a dedicated vector database server

### Security Recommendations

1. Use environment variables for API keys, not hardcoded values
2. Restrict access to LangChain admin pages
3. Enable rate limiting for API endpoints
4. Set up monitoring and alerts for unusual activity
5. Use HTTPS for all connections
6. Regularly audit API usage and costs
