---
id: modules-development-langchain-plugin-tests
---
# Plugin System Tests

This document provides information on testing the plugin systems in the LangChain module.

## Table of Contents

- [Overview](#overview)
- [Plugin Systems](#plugin-systems)
- [Testing Approach](#testing-approach)
- [Unit Testing Plugin Managers](#unit-testing-plugin-managers)
- [Unit Testing Plugins](#unit-testing-plugins)
- [Kernel Testing Plugin Discovery](#kernel-testing-plugin-discovery)
- [Common Test Patterns](#common-test-patterns)
- [Test Coverage Requirements](#test-coverage-requirements)

## Overview

The LangChain module uses Drupal's plugin system extensively to provide extensibility and flexibility. This document outlines how to test these plugin systems to ensure they function correctly and maintain compatibility with third-party extensions.

## Plugin Systems

The LangChain module includes three main plugin systems:

1. **Tool Plugin System**: Provides tools that agents can use to interact with Drupal and external systems
2. **Node Plugin System**: Provides nodes that can be used in workflow graphs
3. **Condition Plugin System**: Provides conditions for determining edge traversal in workflow graphs

## Testing Approach

Each plugin system requires three levels of testing:

1. **Unit Tests for Plugin Managers**: Test that plugin managers correctly handle plugin discovery, instantiation, and configuration
2. **Unit Tests for Plugins**: Test that individual plugins function correctly in isolation
3. **Kernel Tests for Plugin Discovery**: Test that plugins are correctly discovered and instantiated in a real Drupal environment

## Unit Testing Plugin Managers

Plugin managers handle plugin discovery, instantiation, and configuration. Unit tests for plugin managers focus on testing these responsibilities in isolation.

### Example: Tool Plugin Manager Test

```php
namespace Drupal\Tests\langchain\Unit\Plugin;

use Drupal\Tests\UnitTestCase;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\langchain_langgraph\Plugin\ToolPluginManager;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * @coversDefaultClass \Drupal\langchain_langgraph\Plugin\ToolPluginManager
 * @group langchain
 */
class ToolPluginManagerTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface|\Prophecy\Prophecy\ProphecyInterface
   */
  protected $cacheBackend;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface|\Prophecy\Prophecy\ProphecyInterface
   */
  protected $moduleHandler;

  /**
   * The plugin namespaces.
   *
   * @var \Traversable
   */
  protected $namespaces;

  /**
   * The plugin manager.
   *
   * @var \Drupal\langchain_langgraph\Plugin\ToolPluginManager
   */
  protected $manager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->cacheBackend = $this->prophesize(CacheBackendInterface::class);
    $this->moduleHandler = $this->prophesize(ModuleHandlerInterface::class);
    $this->moduleHandler->alter('langchain_agent_tool_info', [], '')->willReturn(NULL);

    $this->namespaces = new \ArrayObject([
      'Drupal\langchain_langgraph\Plugin\AgentTool' => $this->root . '/modules/custom/langchain/modules/langchain_langgraph/src/Plugin/AgentTool',
    ]);

    $this->manager = new ToolPluginManager(
      $this->namespaces,
      $this->cacheBackend->reveal(),
      $this->moduleHandler->reveal()
    );
  }

  /**
   * @covers ::__construct
   */
  public function testConstruction() {
    $this->assertInstanceOf(ToolPluginManager::class, $this->manager);
  }

  /**
   * @covers ::getToolsByCategory
   */
  public function testGetToolsByCategory() {
    // Test implementation details...
  }

  /**
   * @covers ::getToolOptions
   */
  public function testGetToolOptions() {
    // Test implementation details...
  }

  /**
   * @covers ::createInstance
   */
  public function testCreateInstance() {
    // Test implementation details...
  }
}
```

### Test Cases for Plugin Managers

1. **Construction**: Test that the manager can be instantiated
2. **Plugin Discovery**: Test that the manager can discover plugin definitions
3. **Plugin Instantiation**: Test that the manager can create plugin instances
4. **Error Handling**: Test that the manager handles invalid plugin IDs correctly
5. **Utility Methods**: Test any utility methods provided by the manager

## Unit Testing Plugins

Individual plugins should be tested in isolation to verify they function correctly with various inputs and configurations.

### Example: Search Tool Plugin Test

```php
namespace Drupal\Tests\langchain\Unit\Plugin\AgentTool;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\langchain_langgraph\Plugin\AgentTool\SearchTool;
use Drupal\Tests\UnitTestCase;
use Prophecy\PhpUnit\ProphecyTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @coversDefaultClass \Drupal\langchain_langgraph\Plugin\AgentTool\SearchTool
 * @group langchain
 */
class SearchToolTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection|\Prophecy\Prophecy\ProphecyInterface
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\Prophecy\Prophecy\ProphecyInterface
   */
  protected $entityTypeManager;

  /**
   * The search tool plugin.
   *
   * @var \Drupal\langchain_langgraph\Plugin\AgentTool\SearchTool
   */
  protected $plugin;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->database = $this->prophesize(Connection::class);
    $this->entityTypeManager = $this->prophesize(EntityTypeManagerInterface::class);

    $pluginDefinition = [
      'id' => 'search',
      'name' => 'Search',
      'description' => 'Search for content in the site.',
      'category' => 'content',
      'parameters' => [
        'properties' => [
          'query' => [
            'type' => 'string',
            'description' => 'The search query',
          ],
          'type' => [
            'type' => 'string',
            'description' => 'The content type to search (optional)',
            'enum' => ['node', 'user', 'taxonomy_term', 'all'],
          ],
          'limit' => [
            'type' => 'integer',
            'description' => 'Maximum number of results to return',
            'default' => 10,
          ],
        ],
        'required' => ['query'],
      ],
    ];

    $this->plugin = new SearchTool(
      [],
      'search',
      $pluginDefinition,
      $this->database->reveal(),
      $this->entityTypeManager->reveal()
    );
  }

  /**
   * @covers ::create
   */
  public function testCreate() {
    // Test implementation details...
  }

  /**
   * @covers ::validate
   */
  public function testValidate() {
    // Test implementation details...
  }

  /**
   * @covers ::execute
   */
  public function testExecute() {
    // Test implementation details...
  }
}
```

### Test Cases for Plugins

1. **Creation**: Test that the plugin can be instantiated via the factory method
2. **Validation**: Test that the plugin correctly validates inputs
3. **Execution**: Test that the plugin executes correctly with valid inputs
4. **Error Handling**: Test that the plugin handles invalid inputs and errors gracefully
5. **Getters**: Test any getter methods provided by the plugin

## Kernel Testing Plugin Discovery

Kernel tests verify that plugins are correctly discovered and instantiated in a real Drupal environment.

### Example: Tool Plugin Discovery Test

```php
namespace Drupal\Tests\langchain\Kernel\Plugin;

use Drupal\KernelTests\KernelTestBase;
use Drupal\langchain_langgraph\Plugin\ToolPluginInterface;

/**
 * Tests the discovery of tool plugins in the system.
 *
 * @group langchain
 */
class ToolPluginDiscoveryTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'langchain',
    'langchain_langgraph',
  ];

  /**
   * The tool plugin manager.
   *
   * @var \Drupal\langchain_langgraph\Plugin\ToolPluginManagerInterface
   */
  protected $toolPluginManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig(['langchain']);
    $this->toolPluginManager = $this->container->get('plugin.manager.langchain_agent_tool');
  }

  /**
   * Tests that the system can discover tool plugins.
   */
  public function testToolPluginDiscovery() {
    // Get all tool plugin definitions.
    $definitions = $this->toolPluginManager->getDefinitions();
    
    // Verify we have at least the core plugins.
    $this->assertNotEmpty($definitions, 'Tool plugin definitions should not be empty.');
    
    // Verify we have the expected core plugins.
    $expected_plugins = ['search', 'create_content'];
    foreach ($expected_plugins as $plugin_id) {
      $this->assertArrayHasKey($plugin_id, $definitions, "Tool plugin '$plugin_id' should be discovered.");
    }
    
    // Test that we can instantiate the plugins.
    foreach ($definitions as $plugin_id => $definition) {
      $plugin = $this->toolPluginManager->createInstance($plugin_id);
      $this->assertInstanceOf(ToolPluginInterface::class, $plugin);
      $this->assertEquals($plugin_id, $plugin->getPluginId());
      $this->assertEquals($definition['name'], $plugin->getName());
      $this->assertEquals($definition['description'], $plugin->getDescription());
    }
  }

  /**
   * Tests error handling for invalid plugin IDs.
   */
  public function testInvalidPluginId() {
    // Try to instantiate a non-existent plugin.
    $this->expectException('\Drupal\Component\Plugin\Exception\PluginNotFoundException');
    $this->toolPluginManager->createInstance('non_existent_plugin');
  }
}
```

### Test Cases for Plugin Discovery

1. **Plugin Definitions**: Test that plugin definitions are correctly discovered
2. **Plugin Instantiation**: Test that plugins can be instantiated from definitions
3. **Error Handling**: Test that the system handles invalid plugin IDs correctly
4. **Plugin Alterations**: Test that plugin definitions can be altered by other modules

## Common Test Patterns

### Testing Plugin Configuration Validation

```php
/**
 * Tests configuration validation for the LLM node plugin.
 */
public function testConfigurationValidation() {
  // Valid configuration.
  $valid_config = [
    'provider_id' => 'openai',
    'model' => 'gpt-3.5-turbo',
    'system_prompt' => 'You are a helpful assistant.',
    'options' => [
      'temperature' => 0.7,
      'max_tokens' => 500,
    ],
  ];
  
  $errors = LlmNode::validateConfiguration($valid_config);
  $this->assertEmpty($errors, 'Valid configuration should not produce errors.');
  
  // Invalid configuration - missing required fields.
  $invalid_config = [
    // Missing provider_id and model.
    'system_prompt' => 'You are a helpful assistant.',
  ];
  
  $errors = LlmNode::validateConfiguration($invalid_config);
  $this->assertNotEmpty($errors, 'Invalid configuration should produce errors.');
  $this->assertContains('Provider ID is required.', $errors);
  $this->assertContains('Model is required.', $errors);
  
  // Invalid configuration - invalid field type.
  $invalid_type_config = [
    'provider_id' => 'openai',
    'model' => 'gpt-3.5-turbo',
    'options' => 'not an array', // Should be an array.
  ];
  
  $errors = LlmNode::validateConfiguration($invalid_type_config);
  $this->assertNotEmpty($errors);
  $this->assertContains('Options must be a valid JSON object.', $errors);
}
```

### Testing Plugin Execution with Various Inputs

```php
/**
 * @dataProvider executeDataProvider
 */
public function testExecute($input, $expected_result, $setup_callback = NULL) {
  // Allow for custom setup if needed.
  if ($setup_callback) {
    $setup_callback($this);
  }
  
  $result = $this->plugin->execute($input);
  
  if (isset($expected_result['success'])) {
    $this->assertEquals($expected_result['success'], $result['success']);
  }
  
  if (isset($expected_result['error'])) {
    $this->assertArrayHasKey('error', $result);
    $this->assertStringContainsString($expected_result['error'], $result['error']);
  }
  
  if (isset($expected_result['count'])) {
    $this->assertEquals($expected_result['count'], $result['count']);
  }
}

/**
 * Data provider for testExecute.
 */
public function executeDataProvider() {
  return [
    'valid_input' => [
      // Input.
      [
        'query' => 'test',
        'type' => 'node',
        'limit' => 5,
      ],
      // Expected result.
      [
        'success' => TRUE,
        'count' => 1,
      ],
      // Setup callback.
      function ($test) {
        // Mock database query for success case.
        $test->setupDatabaseMockForSuccess();
      },
    ],
    'invalid_input' => [
      // Input.
      [
        'query' => '',  // Empty query.
      ],
      // Expected result.
      [
        'error' => 'Search query is required',
      ],
    ],
    'database_error' => [
      // Input.
      [
        'query' => 'test',
        'type' => 'node',
      ],
      // Expected result.
      [
        'error' => 'Search failed',
      ],
      // Setup callback.
      function ($test) {
        // Mock database query to throw exception.
        $test->setupDatabaseMockForError();
      },
    ],
  ];
}
```

### Testing Plugin Form Building

```php
/**
 * Tests form building for the keyword condition plugin.
 */
public function testBuildConfigurationForm() {
  $form = [];
  $form_state = [];
  
  $form_elements = $this->condition->buildConfigurationForm($form, $form_state);
  
  // Verify form structure.
  $this->assertIsArray($form_elements);
  $this->assertArrayHasKey('source_field', $form_elements);
  $this->assertArrayHasKey('keywords', $form_elements);
  $this->assertArrayHasKey('match_mode', $form_elements);
  $this->assertArrayHasKey('case_sensitive', $form_elements);
  
  // Verify default values.
  $this->assertEquals('response', $form_elements['source_field']['#default_value']);
  $this->assertEquals(implode("\n", ['success', 'completed']), $form_elements['keywords']['#default_value']);
  $this->assertEquals('any', $form_elements['match_mode']['#default_value']);
  $this->assertEquals(FALSE, $form_elements['case_sensitive']['#default_value']);
  
  // Verify required fields.
  $this->assertTrue($form_elements['source_field']['#required']);
  $this->assertTrue($form_elements['keywords']['#required']);
  $this->assertTrue($form_elements['match_mode']['#required']);
}
```

## Test Coverage Requirements

For each plugin system, the following test coverage is required:

### Tool Plugin System

1. **Plugin Manager Tests**:
   - Test plugin discovery
   - Test `getToolsByCategory()` method
   - Test `getToolOptions()` method
   - Test plugin instantiation
   - Test error handling

2. **Tool Plugin Tests**:
   - Test plugin instantiation
   - Test `validate()` method with valid and invalid inputs
   - Test `execute()` method with various inputs
   - Test error handling and recovery
   - Test `getName()`, `getDescription()`, `getParameters()`, `getCategory()`, and `getMetadata()` methods

3. **Plugin Discovery Tests**:
   - Test plugin discovery in a Drupal environment
   - Test plugin instantiation from discovered definitions
   - Test error handling for invalid plugin IDs

### Node Plugin System

1. **Plugin Manager Tests**:
   - Test plugin discovery
   - Test `getOptions()` method
   - Test plugin instantiation
   - Test error handling

2. **Node Plugin Tests**:
   - Test plugin instantiation
   - Test `process()` method with various inputs
   - Test error handling in processing
   - Test configuration validation
   - Test `getInputSchema()`, `getOutputSchema()`, and `updatesState()` methods

3. **Plugin Discovery Tests**:
   - Test plugin discovery in a Drupal environment
   - Test plugin instantiation from discovered definitions
   - Test error handling for invalid plugin IDs

### Condition Plugin System

1. **Plugin Manager Tests**:
   - Test plugin discovery
   - Test `getOptions()` method
   - Test plugin instantiation
   - Test error handling

2. **Condition Plugin Tests**:
   - Test plugin instantiation
   - Test `evaluate()` method with various inputs
   - Test configuration validation
   - Test default configuration

3. **Plugin Discovery Tests**:
   - Test plugin discovery in a Drupal environment
   - Test plugin instantiation from discovered definitions
   - Test error handling for invalid plugin IDs
