---
id: monitoring-alerting
---
# Monitoring and Alerting Guide

<!-- title: Monitoring and Alerting Guide -->
<!-- description: Practical guide for monitoring and alerting in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers monitoring and alerting for the LLM Platform Platform, including metrics collection, alert rules, notification channels, and incident management.

## Metrics Collection

### Metrics Service

```typescript
// src/monitoring/metrics/service.ts
import { PrometheusClient } from 'prom-client';
import { MetricsConfig, MetricValue, MetricQuery } from './types';
import { MetricsStorage } from './storage';
import { MetricsAnalyzer } from './analyzer';

export class MetricsService {
  constructor(
    private prometheus: PrometheusClient,
    private storage: MetricsStorage,
    private analyzer: MetricsAnalyzer,
    private options: MetricsOptions
  ) {}

  async recordMetric(
    metric: MetricValue
  ): Promise<void> {
    const startTime = Date.now();

    try {
      // Record metric
      await this.recordPrometheusMetric(metric);

      // Store metric
      await this.storage.storeMetric(metric);

      // Analyze metric
      await this.analyzer.analyzeMetric(metric);

      // Record metrics
      this.recordMetricsOperation('record', startTime);
    } catch (error) {
      // Record error
      this.recordMetricsError('record', error, startTime);
      throw error;
    }
  }

  async queryMetrics(
    query: MetricQuery
  ): Promise<MetricValue[]> {
    const startTime = Date.now();

    try {
      // Query metrics
      const metrics = await this.storage.queryMetrics(query);

      // Record metrics
      this.recordMetricsOperation('query', startTime);

      return metrics;
    } catch (error) {
      // Record error
      this.recordMetricsError('query', error, startTime);
      throw error;
    }
  }

  async getMetricStats(
    metric: string,
    timeRange: {
      start: Date;
      end: Date;
    }
  ): Promise<MetricStats> {
    const startTime = Date.now();

    try {
      // Get stats
      const stats = await this.analyzer.getMetricStats(
        metric,
        timeRange
      );

      // Record metrics
      this.recordMetricsOperation('stats', startTime);

      return stats;
    } catch (error) {
      // Record error
      this.recordMetricsError('stats', error, startTime);
      throw error;
    }
  }

  private async recordPrometheusMetric(
    metric: MetricValue
  ): Promise<void> {
    const { name, value, labels, type } = metric;

    switch (type) {
      case 'counter':
        this.prometheus.Counter.inc({ name, labels }, value);
        break;

      case 'gauge':
        this.prometheus.Gauge.set({ name, labels }, value);
        break;

      case 'histogram':
        this.prometheus.Histogram.observe(
          { name, labels },
          value
        );
        break;

      case 'summary':
        this.prometheus.Summary.observe(
          { name, labels },
          value
        );
        break;

      default:
        throw new Error(`Unknown metric type: ${type}`);
    }
  }

  private recordMetricsOperation(
    operation: string,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.prometheus.Counter.inc(
      {
        name: 'metrics_operations_total',
        labels: {
          operation,
          environment: this.options.environment,
          service: this.options.service,
          version: this.options.version
        }
      },
      1
    );

    this.prometheus.Histogram.observe(
      {
        name: 'metrics_operation_duration_seconds',
        labels: {
          operation,
          environment: this.options.environment,
          service: this.options.service,
          version: this.options.version
        }
      },
      duration / 1000
    );
  }

  private recordMetricsError(
    operation: string,
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.prometheus.Counter.inc(
      {
        name: 'metrics_errors_total',
        labels: {
          operation,
          error: error.name,
          environment: this.options.environment,
          service: this.options.service,
          version: this.options.version
        }
      },
      1
    );

    this.prometheus.Histogram.observe(
      {
        name: 'metrics_error_duration_seconds',
        labels: {
          operation,
          error: error.name,
          environment: this.options.environment,
          service: this.options.service,
          version: this.options.version
        }
      },
      duration / 1000
    );
  }
}

interface MetricsOptions {
  environment: string;
  service: string;
  version: string;
}

interface MetricValue {
  name: string;
  value: number;
  labels: Record<string, string>;
  type: 'counter' | 'gauge' | 'histogram' | 'summary';
  timestamp: Date;
}

interface MetricQuery {
  name?: string;
  labels?: Record<string, string>;
  timeRange?: {
    start: Date;
    end: Date;
  };
  limit?: number;
  offset?: number;
}

interface MetricStats {
  count: number;
  sum: number;
  min: number;
  max: number;
  avg: number;
  p95: number;
  p99: number;
}
```

### Alert Rules

```typescript
// src/monitoring/alert/rules.ts
import { AlertRule, AlertCondition, AlertAction } from './types';
import { MetricsService } from '../metrics/service';
import { NotificationService } from '../notification/service';

export class AlertRuleService {
  constructor(
    private metrics: MetricsService,
    private notifications: NotificationService,
    private options: AlertOptions
  ) {}

  async evaluateRules(): Promise<void> {
    const startTime = Date.now();

    try {
      // Get rules
      const rules = await this.getRules();

      // Evaluate rules
      for (const rule of rules) {
        await this.evaluateRule(rule);
      }

      // Record metrics
      this.recordRuleEvaluation(startTime);
    } catch (error) {
      // Record error
      this.recordRuleError(error, startTime);
      throw error;
    }
  }

  private async evaluateRule(
    rule: AlertRule
  ): Promise<void> {
    const startTime = Date.now();

    try {
      // Check conditions
      const triggered = await this.checkConditions(
        rule.conditions
      );

      if (triggered) {
        // Execute actions
        await this.executeActions(rule.actions);

        // Record alert
        this.recordAlert(rule);
      }

      // Record metrics
      this.recordRuleCheck(rule, triggered, startTime);
    } catch (error) {
      // Record error
      this.recordRuleCheckError(rule, error, startTime);
      throw error;
    }
  }

  private async checkConditions(
    conditions: AlertCondition[]
  ): Promise<boolean> {
    // Check all conditions
    const results = await Promise.all(
      conditions.map((condition) =>
        this.checkCondition(condition)
      )
    );

    // All conditions must be true
    return results.every((result) => result);
  }

  private async checkCondition(
    condition: AlertCondition
  ): Promise<boolean> {
    const { metric, operator, threshold, duration } =
      condition;

    // Get metric stats
    const stats = await this.metrics.getMetricStats(
      metric,
      {
        start: new Date(Date.now() - duration),
        end: new Date()
      }
    );

    // Check condition
    switch (operator) {
      case 'gt':
        return stats.avg > threshold;

      case 'gte':
        return stats.avg >= threshold;

      case 'lt':
        return stats.avg < threshold;

      case 'lte':
        return stats.avg <= threshold;

      case 'eq':
        return stats.avg === threshold;

      case 'ne':
        return stats.avg !== threshold;

      default:
        throw new Error(
          `Unknown operator: ${operator}`
        );
    }
  }

  private async executeActions(
    actions: AlertAction[]
  ): Promise<void> {
    // Execute all actions
    await Promise.all(
      actions.map((action) =>
        this.executeAction(action)
      )
    );
  }

  private async executeAction(
    action: AlertAction
  ): Promise<void> {
    const { type, config } = action;

    switch (type) {
      case 'notification':
        await this.notifications.sendNotification(
          config
        );
        break;

      case 'webhook':
        await this.executeWebhook(config);
        break;

      case 'script':
        await this.executeScript(config);
        break;

      default:
        throw new Error(
          `Unknown action type: ${type}`
        );
    }
  }

  private recordRuleEvaluation(
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'alert_rule_evaluation_duration_seconds',
      value: duration / 1000,
      labels: {
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }

  private recordRuleError(
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'alert_rule_error_duration_seconds',
      value: duration / 1000,
      labels: {
        error: error.name,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }

  private recordRuleCheck(
    rule: AlertRule,
    triggered: boolean,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'alert_rule_check_duration_seconds',
      value: duration / 1000,
      labels: {
        rule: rule.id,
        triggered: String(triggered),
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }

  private recordRuleCheckError(
    rule: AlertRule,
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'alert_rule_check_error_duration_seconds',
      value: duration / 1000,
      labels: {
        rule: rule.id,
        error: error.name,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }

  private recordAlert(rule: AlertRule): void {
    this.metrics.recordMetric({
      name: 'alert_triggered_total',
      value: 1,
      labels: {
        rule: rule.id,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'counter',
      timestamp: new Date()
    });
  }
}

interface AlertOptions {
  environment: string;
  service: string;
  version: string;
}

interface AlertRule {
  id: string;
  name: string;
  description: string;
  conditions: AlertCondition[];
  actions: AlertAction[];
  enabled: boolean;
}

interface AlertCondition {
  metric: string;
  operator: 'gt' | 'gte' | 'lt' | 'lte' | 'eq' | 'ne';
  threshold: number;
  duration: number;
}

interface AlertAction {
  type: 'notification' | 'webhook' | 'script';
  config: any;
}
```

### Notification Service

```typescript
// src/monitoring/notification/service.ts
import { Notification, NotificationChannel } from './types';
import { MetricsService } from '../metrics/service';

export class NotificationService {
  constructor(
    private metrics: MetricsService,
    private options: NotificationOptions
  ) {}

  async sendNotification(
    notification: Notification
  ): Promise<void> {
    const startTime = Date.now();

    try {
      // Get channels
      const channels = await this.getChannels(
        notification
      );

      // Send to channels
      await Promise.all(
        channels.map((channel) =>
          this.sendToChannel(channel, notification)
        )
      );

      // Record metrics
      this.recordNotificationSent(
        notification,
        channels,
        startTime
      );
    } catch (error) {
      // Record error
      this.recordNotificationError(
        notification,
        error,
        startTime
      );
      throw error;
    }
  }

  private async getChannels(
    notification: Notification
  ): Promise<NotificationChannel[]> {
    // Get channels from database
    const channels = await this.options.channelRepository.findByNotification(
      notification
    );

    return channels;
  }

  private async sendToChannel(
    channel: NotificationChannel,
    notification: Notification
  ): Promise<void> {
    const { type, config } = channel;

    switch (type) {
      case 'email':
        await this.sendEmail(config, notification);
        break;

      case 'slack':
        await this.sendSlack(config, notification);
        break;

      case 'webhook':
        await this.sendWebhook(config, notification);
        break;

      default:
        throw new Error(
          `Unknown channel type: ${type}`
        );
    }
  }

  private recordNotificationSent(
    notification: Notification,
    channels: NotificationChannel[],
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'notification_sent_total',
      value: 1,
      labels: {
        type: notification.type,
        channels: channels
          .map((c) => c.type)
          .join(','),
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'counter',
      timestamp: new Date()
    });

    this.metrics.recordMetric({
      name: 'notification_sent_duration_seconds',
      value: duration / 1000,
      labels: {
        type: notification.type,
        channels: channels
          .map((c) => c.type)
          .join(','),
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }

  private recordNotificationError(
    notification: Notification,
    error: Error,
    startTime: number
  ): void {
    const duration = Date.now() - startTime;

    this.metrics.recordMetric({
      name: 'notification_error_total',
      value: 1,
      labels: {
        type: notification.type,
        error: error.name,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'counter',
      timestamp: new Date()
    });

    this.metrics.recordMetric({
      name: 'notification_error_duration_seconds',
      value: duration / 1000,
      labels: {
        type: notification.type,
        error: error.name,
        environment: this.options.environment,
        service: this.options.service,
        version: this.options.version
      },
      type: 'histogram',
      timestamp: new Date()
    });
  }
}

interface NotificationOptions {
  environment: string;
  service: string;
  version: string;
  channelRepository: ChannelRepository;
}

interface ChannelRepository {
  findByNotification(
    notification: Notification
  ): Promise<NotificationChannel[]>;
}

interface Notification {
  type: string;
  title: string;
  message: string;
  severity: 'info' | 'warning' | 'error' | 'critical';
  metadata?: any;
}

interface NotificationChannel {
  id: string;
  type: 'email' | 'slack' | 'webhook';
  config: any;
}
```

## Best Practices

1. **Metrics Collection**
   - Define metrics
   - Set labels
   - Choose types
   - Store metrics
   - Analyze metrics

2. **Alert Rules**
   - Define rules
   - Set thresholds
   - Choose operators
   - Set duration
   - Test rules

3. **Notifications**
   - Choose channels
   - Set templates
   - Group alerts
   - Set severity
   - Test notifications

4. **Incident Management**
   - Track incidents
   - Set priorities
   - Assign owners
   - Document resolution
   - Review incidents

## Related Documentation

- [Monitoring Guide](../monitoring/README)
- [Security Guide](../security/README)
- [API Guide](../api/README)
- [Performance Guide](../performance/README)
- [Deployment Guide](../deployment/README) 
