---
id: monitoring-observability
---
# Monitoring and Observability Guide

<!-- title: Monitoring and Observability Guide -->
<!-- description: Practical guide for monitoring and observability in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical monitoring and observability implementation for the LLM Platform Platform, including metrics collection, distributed tracing, logging, and alerting.

## Metrics Collection

### Prometheus Integration

```typescript
// src/monitoring/metrics.service.ts
@Injectable()
export class MetricsService {
  private readonly logger = new Logger(MetricsService.name);
  private readonly prometheus: PrometheusService;
  
  constructor() {
    this.prometheus = new PrometheusService({
      defaultMetrics: {
        enabled: true,
        config: {
          prefix: 'bfapi_'
        }
      }
    });
    
    // Register custom metrics
    this.registerCustomMetrics();
  }
  
  private registerCustomMetrics(): void {
    // Request metrics
    this.prometheus.registerCounter({
      name: 'http_requests_total',
      help: 'Total HTTP requests',
      labelNames: ['method', 'path', 'status']
    });
    
    this.prometheus.registerHistogram({
      name: 'http_request_duration_seconds',
      help: 'HTTP request duration',
      labelNames: ['method', 'path'],
      buckets: [0.1, 0.5, 1, 2, 5]
    });
    
    // Agent metrics
    this.prometheus.registerCounter({
      name: 'agent_executions_total',
      help: 'Total agent executions',
      labelNames: ['agent_id', 'model', 'status']
    });
    
    this.prometheus.registerHistogram({
      name: 'agent_execution_duration_seconds',
      help: 'Agent execution duration',
      labelNames: ['agent_id', 'model'],
      buckets: [1, 5, 10, 30, 60]
    });
    
    // Model metrics
    this.prometheus.registerCounter({
      name: 'model_tokens_total',
      help: 'Total tokens processed',
      labelNames: ['model', 'type']
    });
    
    this.prometheus.registerGauge({
      name: 'model_concurrent_requests',
      help: 'Current concurrent model requests',
      labelNames: ['model']
    });
  }
  
  recordHttpRequest(
    method: string,
    path: string,
    status: number,
    duration: number
  ): void {
    this.prometheus.http_requests_total.inc({
      method,
      path,
      status: status.toString()
    });
    
    this.prometheus.http_request_duration_seconds.observe(
      { method, path },
      duration
    );
  }
  
  recordAgentExecution(
    agentId: string,
    model: string,
    status: string,
    duration: number
  ): void {
    this.prometheus.agent_executions_total.inc({
      agent_id: agentId,
      model,
      status
    });
    
    this.prometheus.agent_execution_duration_seconds.observe(
      { agent_id: agentId, model },
      duration
    );
  }
  
  recordModelTokens(
    model: string,
    type: 'input' | 'output',
    count: number
  ): void {
    this.prometheus.model_tokens_total.inc({
      model,
      type
    }, count);
  }
  
  updateConcurrentRequests(
    model: string,
    delta: number
  ): void {
    this.prometheus.model_concurrent_requests.inc({
      model
    }, delta);
  }
}

// src/interceptors/metrics.interceptor.ts
@Injectable()
export class MetricsInterceptor implements NestInterceptor {
  constructor(
    private metricsService: MetricsService,
    private logger: Logger
  ) {}
  
  intercept(
    context: ExecutionContext,
    next: CallHandler
  ): Observable<any> {
    const request = context.switchToHttp().getRequest();
    const { method, path } = request;
    const startTime = Date.now();
    
    return next.handle().pipe(
      tap({
        next: (response) => {
          const duration = (Date.now() - startTime) / 1000;
          this.metricsService.recordHttpRequest(
            method,
            path,
            response?.status || 200,
            duration
          );
        },
        error: (error) => {
          const duration = (Date.now() - startTime) / 1000;
          this.metricsService.recordHttpRequest(
            method,
            path,
            error?.status || 500,
            duration
          );
        }
      })
    );
  }
}
```

### Metrics Dashboard

```typescript
// src/monitoring/dashboard.service.ts
@Injectable()
export class DashboardService {
  private readonly logger = new Logger(DashboardService.name);
  private readonly grafana: GrafanaService;
  
  constructor(
    private configService: ConfigService
  ) {
    this.grafana = new GrafanaService({
      url: this.configService.get('GRAFANA_URL'),
      token: this.configService.get('GRAFANA_TOKEN')
    });
  }
  
  async createDashboard(): Promise<void> {
    const dashboard = {
      dashboard: {
        id: null,
        title: 'BFAPI Dashboard',
        tags: ['bfapi', 'api'],
        timezone: 'browser',
        refresh: '10s',
        schemaVersion: 30,
        version: 0,
        panels: [
          // Request rate panel
          {
            title: 'Request Rate',
            type: 'graph',
            gridPos: { x: 0, y: 0, w: 12, h: 8 },
            targets: [{
              expr: 'sum(rate(http_requests_total[5m])) by (status)',
              legendFormat: '{{status}}'
            }]
          },
          
          // Error rate panel
          {
            title: 'Error Rate',
            type: 'graph',
            gridPos: { x: 12, y: 0, w: 12, h: 8 },
            targets: [{
              expr: 'sum(rate(http_requests_total{status=~"5.."}[5m])) / sum(rate(http_requests_total[5m]))',
              legendFormat: 'error rate'
            }]
          },
          
          // Latency panel
          {
            title: 'Latency',
            type: 'graph',
            gridPos: { x: 0, y: 8, w: 12, h: 8 },
            targets: [{
              expr: 'histogram_quantile(0.95, sum(rate(http_request_duration_seconds_bucket[5m])) by (le))',
              legendFormat: 'p95'
            }]
          },
          
          // Agent executions panel
          {
            title: 'Agent Executions',
            type: 'graph',
            gridPos: { x: 12, y: 8, w: 12, h: 8 },
            targets: [{
              expr: 'sum(rate(agent_executions_total[5m])) by (model, status)',
              legendFormat: '{{model}} - {{status}}'
            }]
          }
        ]
      },
      overwrite: true
    };
    
    try {
      await this.grafana.createDashboard(dashboard);
      this.logger.log('Dashboard created successfully');
    } catch (error) {
      this.logger.error('Failed to create dashboard:', error);
      throw error;
    }
  }
}
```

## Distributed Tracing

### OpenTelemetry Integration

```typescript
// src/monitoring/tracing.service.ts
@Injectable()
export class TracingService {
  private readonly logger = new Logger(TracingService.name);
  private readonly tracer: Tracer;
  
  constructor(
    private configService: ConfigService
  ) {
    const provider = new NodeTracerProvider({
      resource: new Resource({
        'service.name': 'bfapi',
        'service.version': this.configService.get('APP_VERSION')
      })
    });
    
    // Configure exporters
    provider.addSpanProcessor(
      new BatchSpanProcessor(
        new OTLPTraceExporter({
          url: this.configService.get('OTEL_EXPORTER_URL')
        })
      )
    );
    
    // Register provider
    provider.register();
    
    this.tracer = provider.getTracer('bfapi');
  }
  
  startSpan(
    name: string,
    options?: SpanOptions
  ): Span {
    return this.tracer.startSpan(name, options);
  }
  
  async traceAsync<T>(
    name: string,
    fn: (span: Span) => Promise<T>,
    options?: SpanOptions
  ): Promise<T> {
    const span = this.startSpan(name, options);
    
    try {
      const result = await fn(span);
      span.end();
      return result;
    } catch (error) {
      span.recordException(error);
      span.setStatus({
        code: SpanStatusCode.ERROR,
        message: error.message
      });
      span.end();
      throw error;
    }
  }
}

// src/interceptors/tracing.interceptor.ts
@Injectable()
export class TracingInterceptor implements NestInterceptor {
  constructor(
    private tracingService: TracingService,
    private logger: Logger
  ) {}
  
  intercept(
    context: ExecutionContext,
    next: CallHandler
  ): Observable<any> {
    const request = context.switchToHttp().getRequest();
    const { method, url, headers } = request;
    
    return this.tracingService.traceAsync(
      `${method} ${url}`,
      async (span) => {
        // Add request context
        span.setAttributes({
          'http.method': method,
          'http.url': url,
          'http.user_agent': headers['user-agent'],
          'http.request_id': headers['x-request-id']
        });
        
        // Create child span for handler
        return next.handle().pipe(
          tap({
            next: (response) => {
              span.setAttributes({
                'http.status_code': response?.status || 200
              });
            },
            error: (error) => {
              span.setAttributes({
                'http.status_code': error?.status || 500,
                'error.type': error.name,
                'error.message': error.message
              });
            }
          })
        ).toPromise();
      }
    );
  }
}
```

## Logging

### Structured Logging

```typescript
// src/monitoring/logging.service.ts
@Injectable()
export class LoggingService {
  private readonly logger: Logger;
  
  constructor(
    private configService: ConfigService
  ) {
    // Configure Winston logger
    this.logger = createLogger({
      level: this.configService.get('LOG_LEVEL'),
      format: combine(
        timestamp(),
        json()
      ),
      defaultMeta: {
        service: 'bfapi',
        environment: this.configService.get('NODE_ENV')
      },
      transports: [
        // Console transport
        new transports.Console({
          format: combine(
            colorize(),
            timestamp(),
            printf(({ timestamp, level, message, ...meta }) => {
              return `${timestamp} ${level}: ${message} ${
                Object.keys(meta).length ? JSON.stringify(meta) : ''
              }`;
            })
          )
        }),
        
        // File transport
        new transports.File({
          filename: 'logs/error.log',
          level: 'error',
          maxsize: 5242880, // 5MB
          maxFiles: 5
        }),
        
        // Elasticsearch transport
        new ElasticsearchTransport({
          level: 'info',
          index: 'bfapi-logs',
          clientOpts: {
            node: this.configService.get('ELASTICSEARCH_URL'),
            auth: {
              username: this.configService.get('ELASTICSEARCH_USER'),
              password: this.configService.get('ELASTICSEARCH_PASSWORD')
            }
          }
        })
      ]
    });
  }
  
  log(
    level: string,
    message: string,
    meta?: any
  ): void {
    this.logger.log(level, message, {
      ...meta,
      timestamp: new Date().toISOString()
    });
  }
  
  error(
    message: string,
    error?: Error,
    meta?: any
  ): void {
    this.logger.error(message, {
      ...meta,
      error: error ? {
        message: error.message,
        stack: error.stack,
        name: error.name
      } : undefined,
      timestamp: new Date().toISOString()
    });
  }
  
  async searchLogs(
    query: LogSearchQuery
  ): Promise<LogSearchResult> {
    try {
      const { body } = await this.elastic.search({
        index: 'bfapi-logs',
        body: {
          query: {
            bool: {
              must: [
                { term: { level: query.level } },
                { range: {
                  timestamp: {
                    gte: query.startDate,
                    lte: query.endDate
                  }
                }}
              ],
              filter: query.filters
            }
          },
          sort: [{ timestamp: 'desc' }],
          from: query.offset,
          size: query.limit
        }
      });
      
      return {
        total: body.hits.total.value,
        hits: body.hits.hits.map(hit => hit._source),
        aggregations: body.aggregations
      };
    } catch (error) {
      this.error('Log search failed', error);
      throw error;
    }
  }
}

// src/interceptors/logging.interceptor.ts
@Injectable()
export class LoggingInterceptor implements NestInterceptor {
  constructor(
    private loggingService: LoggingService,
    private logger: Logger
  ) {}
  
  intercept(
    context: ExecutionContext,
    next: CallHandler
  ): Observable<any> {
    const request = context.switchToHttp().getRequest();
    const { method, url, ip, user } = request;
    const requestId = request.headers['x-request-id'];
    
    this.loggingService.log('info', 'Incoming request', {
      method,
      url,
      ip,
      user: user?.id,
      requestId
    });
    
    const startTime = Date.now();
    
    return next.handle().pipe(
      tap({
        next: (response) => {
          const duration = Date.now() - startTime;
          this.loggingService.log('info', 'Request completed', {
            method,
            url,
            status: response?.status || 200,
            duration,
            requestId
          });
        },
        error: (error) => {
          const duration = Date.now() - startTime;
          this.loggingService.error('Request failed', error, {
            method,
            url,
            status: error?.status || 500,
            duration,
            requestId
          });
        }
      })
    );
  }
}
```

## Alerting

### Alert Manager Configuration

```yaml
# kubernetes/monitoring/alertmanager.yaml
apiVersion: monitoring.coreos.com/v1
kind: AlertmanagerConfig
metadata:
  name: bfapi-alerts
  namespace: monitoring
spec:
  route:
    groupBy: ['alertname', 'severity']
    groupWait: 30s
    groupInterval: 5m
    repeatInterval: 4h
    receiver: 'slack-notifications'
    routes:
    - match:
        severity: critical
      receiver: 'pagerduty-critical'
      continue: true
    - match:
        severity: warning
      receiver: 'slack-notifications'
  
  receivers:
  - name: 'slack-notifications'
    slackConfigs:
    - channel: '#bfapi-alerts'
      apiURL:
        name: slack-webhook
        key: url
      sendResolved: true
      title: '{{ template "slack.title" . }}'
      text: '{{ template "slack.text" . }}'
  
  - name: 'pagerduty-critical'
    pagerDutyConfigs:
    - serviceKey:
        name: pagerduty-service-key
        key: key
      sendResolved: true
      description: '{{ template "pagerduty.description" . }}'
      client: 'BFAPI'
      clientURL: 'https://api.bluefly.io'
      severity: '{{ .CommonLabels.severity }}'

---
apiVersion: v1
kind: Secret
metadata:
  name: slack-webhook
  namespace: monitoring
type: Opaque
stringData:
  url: ${SLACK_WEBHOOK_URL}

---
apiVersion: v1
kind: Secret
metadata:
  name: pagerduty-service-key
  namespace: monitoring
type: Opaque
stringData:
  key: ${PAGERDUTY_SERVICE_KEY}
```

### Alert Rules

```yaml
# kubernetes/monitoring/prometheus-rules.yaml
apiVersion: monitoring.coreos.com/v1
kind: PrometheusRule
metadata:
  name: bfapi-alerts
  namespace: monitoring
spec:
  groups:
  - name: bfapi
    rules:
    # High error rate
    - alert: HighErrorRate
      expr: |
        sum(rate(http_requests_total{status=~"5.."}[5m]))
        /
        sum(rate(http_requests_total[5m])) > 0.05
      for: 5m
      labels:
        severity: critical
      annotations:
        summary: High error rate
        description: Error rate is above 5% for 5 minutes
    
    # High latency
    - alert: HighLatency
      expr: |
        histogram_quantile(0.95, sum(rate(http_request_duration_seconds_bucket[5m])) by (le)) > 1
      for: 5m
      labels:
        severity: warning
      annotations:
        summary: High latency
        description: 95th percentile latency is above 1s for 5 minutes
    
    # High memory usage
    - alert: HighMemoryUsage
      expr: |
        container_memory_usage_bytes{container="bfapi"} > 400Mi
      for: 5m
      labels:
        severity: warning
      annotations:
        summary: High memory usage
        description: Container memory usage is above 400Mi for 5 minutes
    
    # High CPU usage
    - alert: HighCPUUsage
      expr: |
        rate(container_cpu_usage_seconds_total{container="bfapi"}[5m]) > 0.8
      for: 5m
      labels:
        severity: warning
      annotations:
        summary: High CPU usage
        description: CPU usage is above 80% for 5 minutes
    
    # Service down
    - alert: ServiceDown
      expr: |
        up{job="bfapi"} == 0
      for: 1m
      labels:
        severity: critical
      annotations:
        summary: Service down
        description: BFAPI service is down
    
    # High agent error rate
    - alert: HighAgentErrorRate
      expr: |
        sum(rate(agent_executions_total{status="error"}[5m]))
        /
        sum(rate(agent_executions_total[5m])) > 0.1
      for: 5m
      labels:
        severity: warning
      annotations:
        summary: High agent error rate
        description: Agent error rate is above 10% for 5 minutes
```

## Best Practices

1. **Metrics Collection**
   - Use appropriate metrics
   - Label consistently
   - Set proper buckets
   - Monitor cardinality
   - Regular cleanup

2. **Distributed Tracing**
   - Propagate context
   - Sample appropriately
   - Add useful attributes
   - Monitor trace volume
   - Set proper timeouts

3. **Logging**
   - Use structured logging
   - Include context
   - Set proper levels
   - Rotate logs
   - Monitor log volume

4. **Alerting**
   - Set proper thresholds
   - Use appropriate severity
   - Include useful context
   - Avoid alert fatigue
   - Regular review

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Development Guide](../development/README)
- [Deployment Guide](../deployment/README)
- [Operations Guide](../operations/README)
- [Security Guide](../security/README) 
