---
id: operations-monitoring
---
# Monitoring and Operations Guide

<!-- title: Monitoring and Operations Guide -->
<!-- description: Practical guide for monitoring and operations in the LLM Platform Platform -->
<!-- lastUpdated: 2024-03-19 -->

## Overview

This guide covers practical monitoring and operations for the LLM Platform Platform, including metrics collection, logging, alerting, and incident response.

## Metrics Collection

### Prometheus Configuration

```yaml
# prometheus/values.yaml
server:
  global:
    scrape_interval: 15s
    evaluation_interval: 15s
  
  scrape_configs:
    - job_name: 'bfapi'
      static_configs:
        - targets: ['bfapi:3000']
      metrics_path: '/metrics'
      
    - job_name: 'llm-mcp'
      static_configs:
        - targets: ['llm-mcp:3001']
      metrics_path: '/metrics'
      
    - job_name: 'node-exporter'
      static_configs:
        - targets: ['node-exporter:9100']
```

### Custom Metrics

```typescript
// src/metrics/agent.metrics.ts
import { Counter, Histogram } from 'prom-client';

export const agentExecutions = new Counter({
  name: 'bf_agent_executions_total',
  help: 'Total number of agent executions',
  labelNames: ['agent_id', 'model', 'status']
});

export const agentLatency = new Histogram({
  name: 'bf_agent_execution_seconds',
  help: 'Agent execution latency in seconds',
  labelNames: ['agent_id', 'model'],
  buckets: [0.1, 0.5, 1, 2, 5, 10, 30]
});

export const agentTokens = new Counter({
  name: 'bf_agent_tokens_total',
  help: 'Total number of tokens processed',
  labelNames: ['agent_id', 'model', 'type']
});
```

### Metric Collection

```typescript
// src/services/agent.service.ts
@Injectable()
export class AgentService {
  async execute(
    agentId: string,
    input: string,
    config: any
  ): Promise<any> {
    const start = Date.now();
    
    try {
      const result = await this.modelService.execute(
        agentId,
        input,
        config
      );
      
      // Record metrics
      agentExecutions.inc({
        agent_id: agentId,
        model: config.model,
        status: 'success'
      });
      
      agentLatency.observe(
        {
          agent_id: agentId,
          model: config.model
        },
        (Date.now() - start) / 1000
      );
      
      agentTokens.inc({
        agent_id: agentId,
        model: config.model,
        type: 'output'
      }, result.tokens);
      
      return result;
    } catch (error) {
      // Record error
      agentExecutions.inc({
        agent_id: agentId,
        model: config.model,
        status: 'error'
      });
      
      throw error;
    }
  }
}
```

## Logging

### Logging Configuration

```typescript
// src/config/logging.config.ts
import { WinstonModule } from 'nest-winston';
import * as winston from 'winston';

export const loggingConfig = WinstonModule.forRoot({
  transports: [
    new winston.transports.Console({
      format: winston.format.combine(
        winston.format.timestamp(),
        winston.format.json()
      )
    }),
    new winston.transports.File({
      filename: 'logs/error.log',
      level: 'error',
      format: winston.format.combine(
        winston.format.timestamp(),
        winston.format.json()
      )
    }),
    new winston.transports.File({
      filename: 'logs/combined.log',
      format: winston.format.combine(
        winston.format.timestamp(),
        winston.format.json()
      )
    })
  ]
});
```

### Structured Logging

```typescript
// src/services/agent.service.ts
@Injectable()
export class AgentService {
  private readonly logger = new Logger(AgentService.name);
  
  async execute(
    agentId: string,
    input: string,
    config: any
  ): Promise<any> {
    this.logger.log({
      message: 'Executing agent',
      agent_id: agentId,
      model: config.model,
      input_length: input.length
    });
    
    try {
      const result = await this.modelService.execute(
        agentId,
        input,
        config
      );
      
      this.logger.log({
        message: 'Agent execution successful',
        agent_id: agentId,
        model: config.model,
        output_length: result.output.length,
        tokens: result.tokens
      });
      
      return result;
    } catch (error) {
      this.logger.error({
        message: 'Agent execution failed',
        agent_id: agentId,
        model: config.model,
        error: error.message,
        stack: error.stack
      });
      
      throw error;
    }
  }
}
```

## Alerting

### Alert Rules

```yaml
# prometheus/rules/alerts.yaml
groups:
  - name: bfapi
    rules:
      - alert: HighErrorRate
        expr: rate(bf_agent_executions_total{status="error"}[5m]) > 0.1
        for: 5m
        labels:
          severity: critical
        annotations:
          summary: High error rate detected
          description: Error rate is above 10% for 5 minutes
      
      - alert: HighLatency
        expr: histogram_quantile(0.95, rate(bf_agent_execution_seconds_bucket[5m])) > 5
        for: 5m
        labels:
          severity: warning
        annotations:
          summary: High latency detected
          description: 95th percentile latency is above 5 seconds
      
      - alert: HighTokenUsage
        expr: rate(bf_agent_tokens_total[1h]) > 1000000
        for: 5m
        labels:
          severity: warning
        annotations:
          summary: High token usage detected
          description: Token usage is above 1M per hour
```

### Alert Notifications

```yaml
# alertmanager/config.yaml
receivers:
  - name: 'slack'
    slack_configs:
      - channel: '#alerts'
        send_resolved: true
        title: '{{ template "slack.title" . }}'
        text: '{{ template "slack.text" . }}'
  
  - name: 'email'
    email_configs:
      - to: 'ops@bluefly.io'
        send_resolved: true
        headers:
          subject: '{{ template "email.subject" . }}'
        html: '{{ template "email.html" . }}'

route:
  group_by: ['alertname', 'agent_id']
  group_wait: 30s
  group_interval: 5m
  repeat_interval: 4h
  receiver: 'slack'
  routes:
    - match:
        severity: critical
      receiver: 'email'
```

## Incident Response

### Runbooks

1. **High Error Rate**
   ```markdown
   ## High Error Rate Incident
   
   ### Symptoms
   - Error rate above 10% for 5 minutes
   - Increased latency
   - Failed requests
   
   ### Investigation
   1. Check logs for error patterns
   2. Review recent deployments
   3. Check model provider status
   4. Monitor system resources
   
   ### Resolution
   1. Rollback if deployment-related
   2. Scale resources if needed
   3. Update error handling
   4. Document incident
   ```

2. **High Latency**
   ```markdown
   ## High Latency Incident
   
   ### Symptoms
   - 95th percentile latency above 5 seconds
   - Increased response times
   - Timeout errors
   
   ### Investigation
   1. Check system resources
   2. Review recent changes
   3. Analyze slow queries
   4. Check external dependencies
   
   ### Resolution
   1. Scale resources
   2. Optimize queries
   3. Update caching
   4. Document incident
   ```

### Incident Management

1. **Process**
   ```markdown
   ## Incident Management Process
   
   1. **Detection**
      - Monitor alerts
      - Review metrics
      - Check logs
   
   2. **Response**
      - Acknowledge incident
      - Assign owner
      - Start communication
   
   3. **Investigation**
      - Gather information
      - Analyze root cause
      - Document findings
   
   4. **Resolution**
      - Implement fix
      - Verify solution
      - Update documentation
   
   5. **Post-mortem**
      - Review timeline
      - Identify improvements
      - Update runbooks
   ```

2. **Communication**
   ```markdown
   ## Incident Communication
   
   ### Internal
   - Slack: #incidents
   - Email: incidents@bluefly.io
   - Status page: status.bluefly.io
   
   ### External
   - Status page updates
   - Customer notifications
   - Public post-mortem
   ```

## Best Practices

1. **Monitoring**
   - Collect relevant metrics
   - Set appropriate thresholds
   - Use meaningful labels
   - Regular review

2. **Logging**
   - Use structured logging
   - Include context
   - Set appropriate levels
   - Regular rotation

3. **Alerting**
   - Avoid alert fatigue
   - Use meaningful thresholds
   - Include runbooks
   - Regular review

4. **Incident Response**
   - Follow process
   - Document everything
   - Learn from incidents
   - Update runbooks

## Related Documentation

- [Architecture Guide](../architecture/README)
- [Deployment Guide](../deployment/README)
- [Security Guide](../security/README)
- [Development Guide](../development/README)
- [Testing Guide](../testing/README) 
