---
id: operations-monitoring
---
# Troubleshooting Guide

This document provides solutions for common issues encountered when working with the LLM Platform AI Platform.

## General Troubleshooting Process

1. **Identify the problem**: Determine which component or service is experiencing issues
2. **Check logs**: Review relevant logs for error messages
3. **Verify configuration**: Ensure all configuration settings are correct
4. **Test in isolation**: Test the problematic component in isolation
5. **Review documentation**: Check documentation for known issues and solutions
6. **Seek help**: If unable to resolve, reach out to the development team

## API Gateway (bfapi) Issues

### Connection Refused

**Problem**: Unable to connect to the API Gateway.

**Solutions**:
1. Verify the service is running: `npm run dev` in the bfapi directory
2. Check the port configuration (default: 3000)
3. Ensure no firewall is blocking the connection
4. Verify the correct URL is being used

### Authentication Failures

**Problem**: API returns 401 Unauthorized errors.

**Solutions**:
1. Check that the JWT token is valid and not expired
2. Verify the JWT secret matches between services
3. Ensure the request includes the Authorization header with the Bearer token
4. Check user permissions for the requested resource

### Rate Limiting Issues

**Problem**: API returns 429 Too Many Requests errors.

**Solutions**:
1. Reduce the frequency of requests
2. Implement request batching to reduce the number of API calls
3. Check the rate limit configuration in environment variables
4. Consider requesting a higher rate limit for production use

## LLM Inference (bfllm) Issues

### Model Loading Failures

**Problem**: Unable to load AI models.

**Solutions**:
1. Check that model files exist in the expected location
2. Verify sufficient disk space and memory
3. Ensure required dependencies are installed
4. Check model registry configuration in `config/service.json`

### Slow Inference Performance

**Problem**: AI model inference is taking too long.

**Solutions**:
1. Monitor system resources to identify bottlenecks
2. Consider using a smaller or optimized model
3. Implement request batching for multiple inputs
4. Increase hardware resources (CPU, RAM, GPU)
5. Enable caching for common requests

### Out of Memory Errors

**Problem**: Service crashes with out of memory errors.

**Solutions**:
1. Reduce batch size for processing
2. Implement streaming responses for large outputs
3. Use models with smaller parameter sizes
4. Increase container memory limits
5. Enable memory optimization features

## MCP Protocol (llm-mcp) Issues

### gRPC Connection Issues

**Problem**: Unable to establish gRPC connections.

**Solutions**:
1. Verify the gRPC server is running
2. Check network configuration (ports, firewalls)
3. Ensure TLS certificates are valid if using secure connections
4. Verify protocol buffer definitions match between client and server

### Tool Registration Failures

**Problem**: Tools fail to register with the MCP server.

**Solutions**:
1. Check tool registration configuration
2. Verify tool manifest format is correct
3. Ensure tool authentication credentials are valid
4. Check MCP server logs for specific error messages

## Content Engine (bfrfp) Issues

### Document Generation Failures

**Problem**: AI-generated documents have errors or fail to generate.

**Solutions**:
1. Check input parameters for document generation
2. Verify document templates are correctly formatted
3. Ensure sufficient token limits for the document size
4. Check for rate limiting or quota issues with AI providers

### Performance Issues with Large Documents

**Problem**: Processing large documents is very slow.

**Solutions**:
1. Implement chunking for large documents
2. Use parallel processing for document sections
3. Optimize document parsing and generation algorithms
4. Consider using a more powerful model for complex documents

## Drupal Integration Issues

### Recipe Application Failures

**Problem**: The LLM Platform recipe fails to apply properly.

**Solutions**:
1. Check Drupal error logs for specific issues
2. Verify Drupal version compatibility (Drupal 11 required)
3. Ensure all required dependencies are installed
4. Clear Drupal cache after configuration changes
5. Run `drush recipe:apply llm_platform --verbose` for detailed output

### Module Conflicts

**Problem**: Conflicts between Drupal modules.

**Solutions**:
1. Check module compatibility matrix
2. Update all modules to latest versions
3. Disable conflicting modules
4. Apply patches if available
5. Check hook implementations for conflicts

## Development Environment Issues

### Build Failures

**Problem**: npm run build fails with errors.

**Solutions**:
1. Check for TypeScript compilation errors
2. Verify all dependencies are installed (`npm install`)
3. Check for conflicting dependency versions
4. Clear node_modules and reinstall (`rm -rf node_modules && npm install`)
5. Check build configuration in tsconfig.json and webpack.config.js

### Test Failures

**Problem**: Tests are failing.

**Solutions**:
1. Run tests with verbose output for more details
2. Check for environment configuration issues
3. Ensure all dependencies are installed
4. Verify mocks and fixtures are properly set up
5. Run specific failing tests in isolation

## Deployment Issues

### Docker Build Failures

**Problem**: Docker image build fails.

**Solutions**:
1. Check Dockerfile for errors
2. Verify all required files are included in the build context
3. Ensure sufficient disk space
4. Check for network issues when pulling base images
5. Verify build arguments are correctly provided

### Kubernetes Deployment Issues

**Problem**: Pods fail to start or are in CrashLoopBackOff state.

**Solutions**:
1. Check pod logs: `kubectl logs <pod-name>`
2. Verify resource limits are appropriate
3. Check for configuration issues in deployment YAML
4. Ensure secrets and config maps are properly created
5. Verify image pull credentials if using private registry

## Database Issues

### Connection Failures

**Problem**: Unable to connect to the database.

**Solutions**:
1. Verify database credentials
2. Check database server is running and accessible
3. Ensure firewall allows connections
4. Check connection string format
5. Verify database user has appropriate permissions

### Performance Issues

**Problem**: Database queries are slow.

**Solutions**:
1. Optimize SQL queries
2. Add appropriate indexes
3. Implement query caching
4. Consider database scaling options
5. Monitor database load and resource usage

## Security Issues

### SSL/TLS Certificate Problems

**Problem**: SSL certificate errors or warnings.

**Solutions**:
1. Verify certificate is valid and not expired
2. Ensure certificate chain is complete
3. Check certificate domain matches the service domain
4. Verify certificate file permissions
5. Reload service after certificate changes

### API Key Management Issues

**Problem**: API keys are not working or are exposed.

**Solutions**:
1. Regenerate API keys if compromised
2. Use environment variables for storing API keys
3. Implement API key rotation policies
4. Verify API key permissions and scopes
5. Use secrets management solutions for production

## Monitoring and Logging Issues

### Missing Logs

**Problem**: Expected logs are not appearing.

**Solutions**:
1. Check log level configuration
2. Verify log storage location is writable
3. Ensure logging service is running
4. Check for log rotation issues
5. Verify log format configuration

### Alerting Problems

**Problem**: Alerts are not firing or are firing incorrectly.

**Solutions**:
1. Check alert rules configuration
2. Verify alerting service is running
3. Test alert notification channels
4. Check for silenced or muted alerts
5. Verify metric collection is working properly

## Performance Optimization

### High CPU Usage

**Problem**: Services using excessive CPU.

**Solutions**:
1. Identify CPU-intensive operations
2. Implement caching for expensive computations
3. Optimize algorithms and data structures
4. Consider horizontal scaling
5. Profile code to identify specific bottlenecks

### Memory Leaks

**Problem**: Service memory usage grows over time.

**Solutions**:
1. Use memory profiling tools to identify leaks
2. Check for unclosed resources (file handles, connections)
3. Implement periodic restart strategies
4. Update dependencies with known memory leak fixes
5. Review object lifecycle management in code

## Getting Additional Help

If you're unable to resolve an issue using this guide:

1. **Check existing issues**: Search the project issue tracker for similar problems
2. **Gather information**: Collect logs, error messages, and steps to reproduce
3. **Create a detailed report**: Include all relevant information
4. **Contact the development team**: Submit your issue report
5. **Join community channels**: Discuss in Slack or Discord channels

## Common Error Codes and Solutions

| Error Code | Description | Solution |
|------------|-------------|----------|
| `ERR_CONN_REFUSED` | Connection refused | Check service is running and port is correct |
| `ERR_INVALID_TOKEN` | Invalid JWT token | Regenerate token or check authentication |
| `ERR_RATE_LIMIT` | Rate limit exceeded | Reduce request frequency or request higher limits |
| `ERR_MODEL_NOT_FOUND` | AI model not found | Verify model ID and availability |
| `ERR_INSUFFICIENT_TOKENS` | Token quota exceeded | Purchase additional tokens or optimize usage |
| `ERR_INVALID_CONFIG` | Invalid configuration | Check configuration format and required fields |
| `ERR_DB_CONN` | Database connection error | Verify credentials and database availability |
| `ERR_OUT_OF_MEMORY` | Out of memory error | Increase memory allocation or optimize usage |
| `ERR_TIMEOUT` | Request timeout | Increase timeout setting or optimize performance |
| `ERR_PERMISSION_DENIED` | Permission denied | Check user roles and permissions |
