# Apple FM Integration Guide

This guide covers integration of the `@bluefly/apple-fm` library into various platforms and frameworks, with a focus on Drupal integration.

## Table of Contents

- [Drupal Integration](#drupal-integration)
- [WordPress Integration](#wordpress-integration)
- [Framework Integration](#framework-integration)
- [Microservices Architecture](#microservices-architecture)
- [Security Considerations](#security-considerations)

## Drupal Integration

### Module Setup

1. **Create Module Structure**

```bash
cd web/modules/custom
mkdir -p apple_fm_integration/{src/{Controller,Service,Plugin/Field/{FieldFormatter,FieldWidget}},config/install,templates}
```

2. **Module Info File** (`apple_fm_integration.info.yml`)

```yaml
name: 'Apple FM Integration'
type: module
description: 'Integrates Apple Foundation Models for AI-powered content generation'
core_version_requirement: ^10.0 || ^11.0
package: AI
dependencies:
  - drupal:field
  - drupal:node
  - drupal:user
```

### Service Configuration

1. **Service Definition** (`apple_fm_integration.services.yml`)

```yaml
services:
  apple_fm.client:
    class: Drupal\apple_fm_integration\Service\AppleFMService
    arguments: 
      - '@config.factory'
      - '@logger.factory'
      - '@cache.default'
      - '@http_client'
    tags:
      - { name: 'http_client_middleware' }

  apple_fm.privacy_manager:
    class: Drupal\apple_fm_integration\Service\PrivacyManager
    arguments:
      - '@current_user'
      - '@user.data'
      - '@config.factory'
```

2. **Service Implementation** (`src/Service/AppleFMService.php`)

```php
<?php

namespace Drupal\apple_fm_integration\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use GuzzleHttp\ClientInterface;

class AppleFMService {
  
  protected $configFactory;
  protected $logger;
  protected $cache;
  protected $httpClient;
  protected $client;

  public function __construct(
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    CacheBackendInterface $cache,
    ClientInterface $http_client
  ) {
    $this->configFactory = $config_factory;
    $this->logger = $logger_factory->get('apple_fm');
    $this->cache = $cache;
    $this->httpClient = $http_client;
    $this->initializeClient();
  }

  protected function initializeClient() {
    $config = $this->configFactory->get('apple_fm_integration.settings');
    
    // Initialize the Node.js bridge
    $this->client = new \stdClass();
    $this->client->apiKey = $config->get('api_key');
    $this->client->privacyMode = $config->get('privacy_mode') ?? 'standard';
    $this->client->endpoint = $config->get('endpoint') ?? 'http://localhost:3000/apple-fm';
  }

  public function generate($prompt, array $options = []) {
    $cache_key = 'apple_fm:' . md5($prompt . serialize($options));
    
    // Check cache first
    if ($cached = $this->cache->get($cache_key)) {
      return $cached->data;
    }

    try {
      $response = $this->httpClient->post($this->client->endpoint . '/generate', [
        'json' => [
          'prompt' => $prompt,
          'options' => array_merge([
            'maxTokens' => 200,
            'temperature' => 0.7,
          ], $options),
          'privacyMode' => $this->client->privacyMode,
        ],
        'headers' => [
          'Authorization' => 'Bearer ' . $this->client->apiKey,
          'X-Privacy-Mode' => $this->client->privacyMode,
        ],
      ]);

      $data = json_decode($response->getBody(), TRUE);
      
      // Cache for 1 hour
      $this->cache->set($cache_key, $data, time() + 3600);
      
      return $data;
    }
    catch (\Exception $e) {
      $this->logger->error('Apple FM generation failed: @message', [
        '@message' => $e->getMessage(),
      ]);
      throw $e;
    }
  }

  public function embed($texts, array $options = []) [// Similar implementation for embeddings]

  public function moderate($content, array $categories = []) [// Similar implementation for moderation]
}
```

### Field Integration

1. **Field Formatter** (`src/Plugin/Field/FieldFormatter/AppleFMFormatter.php`)

```php
<?php

namespace Drupal\apple_fm_integration\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'apple_fm_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "apple_fm_formatter",
 *   label = @Translation("Apple FM Enhanced"),
 *   field_types = {
 *     "text_long",
 *     "text_with_summary"
 *   }
 * )
 */
class AppleFMFormatter extends FormatterBase {

  public static function defaultSettings() {
    return [
      'enhancement_type' => 'summary',
      'max_tokens' => 150,
      'cache_duration' => 3600,
    ] + parent::defaultSettings();
  }

  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['enhancement_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Enhancement Type'),
      '#options' => [
        'summary' => $this->t('Summary'),
        'keywords' => $this->t('Keywords'),
        'questions' => $this->t('Questions'),
        'translation' => $this->t('Translation'),
      ],
      '#default_value' => $this->getSetting('enhancement_type'),
    ];

    $elements['max_tokens'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum Tokens'),
      '#default_value' => $this->getSetting('max_tokens'),
      '#min' => 50,
      '#max' => 1000,
    ];

    return $elements;
  }

  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $service = \Drupal::service('apple_fm.client');

    foreach ($items as $delta => $item) {
      $text = $item->value;
      
      try {
        $prompt = $this->buildPrompt($text);
        $enhanced = $service->generate($prompt, [
          'maxTokens' => $this->getSetting('max_tokens'),
        ]);

        $elements[$delta] = [
          '#theme' => 'apple_fm_enhanced_text',
          '#original' => $text,
          '#enhanced' => $enhanced['text'],
          '#type' => $this->getSetting('enhancement_type'),
          '#attached' => [
            'library' => ['apple_fm_integration/formatter'],
          ],
        ];
      }
      catch (\Exception $e) {
        $elements[$delta] = [
          '#markup' => $text,
        ];
      }
    }

    return $elements;
  }

  protected function buildPrompt($text) {
    $type = $this->getSetting('enhancement_type');
    
    switch ($type) {
      case 'summary':
        return "Summarize the following text in 2-3 sentences: " . $text;
      case 'keywords':
        return "Extract 5-7 key topics from this text: " . $text;
      case 'questions':
        return "Generate 3 thoughtful questions about this text: " . $text;
      default:
        return $text;
    }
  }
}
```

2. **Field Widget** (`src/Plugin/Field/FieldWidget/AppleFMWidget.php`)

```php
<?php

namespace Drupal\apple_fm_integration\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'apple_fm_widget' widget.
 *
 * @FieldWidget(
 *   id = "apple_fm_widget",
 *   label = @Translation("Apple FM AI Assistant"),
 *   field_types = {
 *     "text_long",
 *     "text_with_summary"
 *   }
 * )
 */
class AppleFMWidget extends WidgetBase {

  public static function defaultSettings() {
    return [
      'suggestions_enabled' => TRUE,
      'auto_complete' => FALSE,
      'grammar_check' => TRUE,
    ] + parent::defaultSettings();
  }

  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element['value'] = $element + [
      '#type' => 'textarea',
      '#default_value' => $items[$delta]->value ?? '',
      '#attributes' => [
        'class' => ['apple-fm-enhanced-textarea'],
        'data-suggestions' => $this->getSetting('suggestions_enabled') ? 'true' : 'false',
        'data-grammar' => $this->getSetting('grammar_check') ? 'true' : 'false',
      ],
    ];

    $element['ai_tools'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['apple-fm-tools']],
    ];

    $element['ai_tools']['improve'] = [
      '#type' => 'button',
      '#value' => $this->t('Improve Writing'),
      '#ajax' => [
        'callback' => [$this, 'improveWriting'],
        'wrapper' => 'apple-fm-widget-' . $delta,
      ],
      '#attributes' => ['class' => ['apple-fm-improve-btn']],
    ];

    $element['ai_tools']['expand'] = [
      '#type' => 'button',
      '#value' => $this->t('Expand'),
      '#ajax' => [
        'callback' => [$this, 'expandText'],
        'wrapper' => 'apple-fm-widget-' . $delta,
      ],
    ];

    $element['ai_tools']['summarize'] = [
      '#type' => 'button',
      '#value' => $this->t('Summarize'),
      '#ajax' => [
        'callback' => [$this, 'summarizeText'],
        'wrapper' => 'apple-fm-widget-' . $delta,
      ],
    ];

    $element['#attached']['library'][] = 'apple_fm_integration/widget';
    
    return $element;
  }

  public function improveWriting(array &$form, FormStateInterface $form_state) [// AJAX callback for improving writing]
}
```

### Routes and Controllers

1. **Routing** (`apple_fm_integration.routing.yml`)

```yaml
apple_fm_integration.api:
  path: '/api/apple-fm/[action]'
  defaults:
    _controller: '\Drupal\apple_fm_integration\Controller\AppleFMController::handleAction'
    _title: 'Apple FM API'
  methods: [POST]
  requirements:
    _permission: 'use apple fm'
    action: 'generate|embed|moderate'

apple_fm_integration.settings:
  path: '/admin/config/ai/apple-fm'
  defaults:
    _form: '\Drupal\apple_fm_integration\Form\AppleFMSettingsForm'
    _title: 'Apple FM Settings'
  requirements:
    _permission: 'administer apple fm'
```

2. **Controller** (`src/Controller/AppleFMController.php`)

```php
<?php

namespace Drupal\apple_fm_integration\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AppleFMController extends ControllerBase {

  protected $appleFMService;
  protected $privacyManager;

  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->appleFMService = $container->get('apple_fm.client');
    $instance->privacyManager = $container->get('apple_fm.privacy_manager');
    return $instance;
  }

  public function handleAction(Request $request, $action) {
    // Check privacy settings
    if (!$this->privacyManager->canPerformAction($action)) {
      return new JsonResponse([
        'error' => 'Privacy settings prevent this action',
      ], 403);
    }

    $data = json_decode($request->getContent(), TRUE);

    try {
      switch ($action) {
        case 'generate':
          $result = $this->appleFMService->generate(
            $data['prompt'],
            $data['options'] ?? []
          );
          break;
        
        case 'embed':
          $result = $this->appleFMService->embed(
            $data['texts'],
            $data['options'] ?? []
          );
          break;
        
        case 'moderate':
          $result = $this->appleFMService->moderate(
            $data['content'],
            $data['categories'] ?? []
          );
          break;
        
        default:
          return new JsonResponse(['error' => 'Invalid action'], 400);
      }

      return new JsonResponse($result);
    }
    catch (\Exception $e) {
      return new JsonResponse([
        'error' => $e->getMessage(),
      ], 500);
    }
  }
}
```

### JavaScript Integration

1. **Library Definition** (`apple_fm_integration.libraries.yml`)

```yaml
formatter:
  version: 1.x
  css:
    theme:
      css/apple-fm-formatter.css: {}
  js:
    js/apple-fm-formatter.js: {}
  dependencies:
    - core/drupal
    - core/jquery

widget:
  version: 1.x
  css:
    theme:
      css/apple-fm-widget.css: {}
  js:
    js/apple-fm-widget.js: {}
  dependencies:
    - core/drupal
    - core/jquery
    - core/drupal.ajax
```

2. **Widget JavaScript** (`js/apple-fm-widget.js`)

```javascript
(function ($, Drupal) {
  'use strict';

  Drupal.behaviors.appleFMWidget = {
    attach: function (context, settings) {
      once('apple-fm-widget', '.apple-fm-enhanced-textarea', context).forEach(function (textarea) {
        const $textarea = $(textarea);
        const suggestions = $textarea.data('suggestions');
        const grammar = $textarea.data('grammar');

        if (suggestions) {
          initializeSuggestions($textarea);
        }

        if (grammar) {
          initializeGrammarCheck($textarea);
        }
      });
    }
  };

  function initializeSuggestions($textarea) {
    let debounceTimer;
    
    $textarea.on('input', function () {
      clearTimeout(debounceTimer);
      debounceTimer = setTimeout(function () {
        const text = $textarea.val();
        if (text.length > 50) {
          fetchSuggestions(text, $textarea);
        }
      }, 1000);
    });
  }

  function fetchSuggestions(text, $textarea) {
    $.ajax({
      url: '/api/apple-fm/generate',
      method: 'POST',
      data: JSON.stringify({
        prompt: `Suggest the next sentence for: [text]`,
        options: {
          maxTokens: 50,
          temperature: 0.8
        }
      }),
      contentType: 'application/json',
      success: function (response) {
        showSuggestion($textarea, response.text);
      }
    });
  }

  function showSuggestion($textarea, suggestion) {
    const $suggestionBox = $('<div class="apple-fm-suggestion">')
      .text(suggestion)
      .insertAfter($textarea);

    $suggestionBox.on('click', function () {
      $textarea.val($textarea.val() + ' ' + suggestion);
      $suggestionBox.remove();
    });

    // Auto-hide after 10 seconds
    setTimeout(function () {
      $suggestionBox.fadeOut(function () {
        $(this).remove();
      });
    }, 10000);
  }

})(jQuery, Drupal);
```

### Configuration Management

1. **Settings Form** (`src/Form/AppleFMSettingsForm.php`)

```php
<?php

namespace Drupal\apple_fm_integration\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class AppleFMSettingsForm extends ConfigFormBase {

  protected function getEditableConfigNames() {
    return ['apple_fm_integration.settings'];
  }

  public function getFormId() {
    return 'apple_fm_settings_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('apple_fm_integration.settings');

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
      '#description' => $this->t('Your Apple FM API key'),
    ];

    $form['privacy_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Privacy Mode'),
      '#options' => [
        'standard' => $this->t('Standard'),
        'strict' => $this->t('Strict'),
        'offline' => $this->t('Offline'),
      ],
      '#default_value' => $config->get('privacy_mode') ?? 'standard',
      '#description' => $this->t('Select the privacy level for API interactions'),
    ];

    $form['endpoint'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Endpoint'),
      '#default_value' => $config->get('endpoint') ?? 'http://localhost:3000/apple-fm',
      '#description' => $this->t('The endpoint for the Apple FM bridge service'),
    ];

    $form['cache_duration'] = [
      '#type' => 'number',
      '#title' => $this->t('Cache Duration (seconds)'),
      '#default_value' => $config->get('cache_duration') ?? 3600,
      '#min' => 0,
      '#description' => $this->t('How long to cache API responses'),
    ];

    return parent::buildForm($form, $form_state);
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('apple_fm_integration.settings')
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('privacy_mode', $form_state->getValue('privacy_mode'))
      ->set('endpoint', $form_state->getValue('endpoint'))
      ->set('cache_duration', $form_state->getValue('cache_duration'))
      ->save();

    parent::submitForm($form, $form_state);
  }
}
```

2. **Default Configuration** (`config/install/apple_fm_integration.settings.yml`)

```yaml
api_key: ''
privacy_mode: 'standard'
endpoint: 'http://localhost:3000/apple-fm'
cache_duration: 3600
allowed_content_types:
  - article
  - page
  - blog_post
max_requests_per_hour: 1000
```

### Node.js Bridge Service

For production deployments, create a Node.js bridge service:

```javascript
// bridge-service.js
const express = require('express');
const [AppleFMClient] = require('@bluefly/apple-fm');

const app = express();
app.use(express.json());

const client = new AppleFMClient({
  apiKey: process.env.APPLE_FM_API_KEY,
  privacyMode: process.env.PRIVACY_MODE || 'standard'
});

app.post('/apple-fm/generate', async (req, res) => {
  try {
    const { prompt, options } = req.body;
    const response = await client.generate({ prompt, ...options });
    res.json(response);
  } catch (error) {
    res.status(500).json({ error: error.message });
  }
});

app.post('/apple-fm/embed', async (req, res) => {
  try {
    const { texts, options } = req.body;
    const response = await client.embed({ texts, ...options });
    res.json(response);
  } catch (error) {
    res.status(500).json({ error: error.message });
  }
});

app.listen(3000, () => {
  console.log('Apple FM bridge service running on port 3000');
});
```

### Drush Commands

Create custom Drush commands for CLI operations:

```php
<?php

namespace Drupal\apple_fm_integration\Commands;

use Drush\Commands\DrushCommands;

/**
 * Apple FM Drush commands.
 */
class AppleFMCommands extends DrushCommands {

  /**
   * Generate text using Apple FM.
   *
   * @command apple-fm:generate
   * @aliases afg
   * @param string $prompt The prompt to generate from
   * @option max-tokens Maximum number of tokens
   * @usage apple-fm:generate "Write a haiku about Drupal"
   */
  public function generate($prompt, $options = ['max-tokens' => 100]) {
    $service = \Drupal::service('apple_fm.client');
    
    try {
      $response = $service->generate($prompt, [
        'maxTokens' => $options['max-tokens'],
      ]);
      
      $this->output()->writeln($response['text']);
    }
    catch (\Exception $e) {
      $this->logger()->error($e->getMessage());
    }
  }

  /**
   * Bulk process content with Apple FM.
   *
   * @command apple-fm:bulk-process
   * @aliases afbp
   * @param string $entity_type Entity type to process
   * @param string $bundle Bundle to process
   * @option field Field to process
   * @option action Action to perform (summarize, translate, etc)
   */
  public function bulkProcess($entity_type, $bundle, $options = ['field' => 'body', 'action' => 'summarize']) [// Implementation for bulk processing]
}
```

## WordPress Integration

### Plugin Structure

```php
<?php
/**
 * Plugin Name: Apple FM Integration
 * Description: Integrates Apple Foundation Models for AI-powered content
 * Version: 1.0.0
 */

class AppleFMWordPress {
  
  private $client;
  
  public function __construct() {
    add_action('init', [$this, 'init']);
    add_action('admin_menu', [$this, 'add_admin_menu']);
    add_filter('the_content', [$this, 'enhance_content']);
  }
  
  public function init() {
    // Initialize Apple FM client
    $this->client = new AppleFMBridge(
      get_option('apple_fm_api_key'),
      get_option('apple_fm_privacy_mode', 'standard')
    );
  }
  
  public function enhance_content($content) {
    if (!is_single() || !get_option('apple_fm_auto_enhance')) {
      return $content;
    }
    
    // Add AI-generated summary
    $summary = $this->client->summarize($content);
    
    return sprintf(
      '<div class="ai-summary">%s</div>%s',
      $summary,
      $content
    );
  }
}

new AppleFMWordPress();
```

## Framework Integration

### Laravel

```php
// ServiceProvider
namespace App\Providers;

use Illuminate\Support\ServiceProvider;
use App\Services\AppleFMService;

class AppleFMServiceProvider extends ServiceProvider {
    public function register() {
        $this->app->singleton(AppleFMService::class, function ($app) {
            return new AppleFMService(
                config('services.apple_fm.key'),
                config('services.apple_fm.privacy_mode')
            );
        });
    }
}

// Controller
namespace App\Http\Controllers;

use App\Services\AppleFMService;

class ContentController extends Controller {
    protected $appleFM;
    
    public function __construct(AppleFMService $appleFM) {
        $this->appleFM = $appleFM;
    }
    
    public function enhance(Request $request) {
        $enhanced = $this->appleFM->generate(
            $request->input('prompt'),
            ['maxTokens' => 200]
        );
        
        return response()->json($enhanced);
    }
}
```

### Next.js

```typescript
// pages/api/apple-fm/[action].ts
import { NextApiRequest, NextApiResponse } from 'next';
import [AppleFMClient] from '@bluefly/apple-fm';

const client = new AppleFMClient({
  apiKey: process.env.APPLE_FM_API_KEY!,
  privacyMode: 'standard'
});

export default async function handler(
  req: NextApiRequest,
  res: NextApiResponse
) [const { action] = req.query;
  
  if (req.method !== 'POST') {
    return res.status(405).json({ error: 'Method not allowed' });
  }
  
  try {
    let result;
    
    switch (action) {
      case 'generate':
        result = await client.generate(req.body);
        break;
      case 'embed':
        result = await client.embed(req.body);
        break;
      default:
        return res.status(404).json({ error: 'Unknown action' });
    }
    
    res.status(200).json(result);
  } catch (error) {
    res.status(500).json({ error: error.message });
  }
}
```

## Microservices Architecture

### Docker Compose Setup

```yaml
version: '3.8'

services:
  apple-fm-bridge:
    build: ./apple-fm-bridge
    environment:
      - APPLE_FM_API_KEY=[APPLE_FM_API_KEY]
      - PRIVACY_MODE=standard
      - REDIS_URL=redis://redis:6379
    ports:
      - "3000:3000"
    depends_on:
      - redis
      
  redis:
    image: redis:alpine
    ports:
      - "6379:6379"
      
  drupal:
    build: ./drupal
    environment:
      - APPLE_FM_ENDPOINT=http://apple-fm-bridge:3000
    ports:
      - "8080:80"
    depends_on:
      - apple-fm-bridge
```

### Kubernetes Deployment

```yaml
apiVersion: apps/v1
kind: Deployment
metadata:
  name: apple-fm-bridge
spec:
  replicas: 3
  selector:
    matchLabels:
      app: apple-fm-bridge
  template:
    metadata:
      labels:
        app: apple-fm-bridge
    spec:
      containers:
      - name: apple-fm-bridge
        image: your-registry/apple-fm-bridge:latest
        env:
        - name: APPLE_FM_API_KEY
          valueFrom:
            secretKeyRef:
              name: apple-fm-secret
              key: api-key
        - name: PRIVACY_MODE
          value: "strict"
        ports:
        - containerPort: 3000
        resources:
          requests:
            memory: "256Mi"
            cpu: "250m"
          limits:
            memory: "512Mi"
            cpu: "500m"
---
apiVersion: v1
kind: Service
metadata:
  name: apple-fm-bridge
spec:
  selector:
    app: apple-fm-bridge
  ports:
    - port: 3000
      targetPort: 3000
  type: ClusterIP
```

## Security Considerations

### API Key Management

1. **Environment Variables**: Never hardcode API keys
2. **Key Rotation**: Implement regular key rotation
3. **Access Control**: Use role-based permissions
4. **Audit Logging**: Log all API usage

### Privacy Compliance

1. **Data Retention**: Implement automatic cache expiration
2. **User Consent**: Track consent for AI processing
3. **Data Anonymization**: Remove PII before processing
4. **Regional Compliance**: Respect data residency requirements

### Rate Limiting

```php
// Drupal example
class RateLimiter {
  public function check($user_id, $action) {
    $key = "apple_fm:rate_limit:$user_id:$action";
    $count = \Drupal::cache()->get($key);
    
    if ($count && $count->data >= 100) {
      throw new RateLimitException('Rate limit exceeded');
    }
    
    \Drupal::cache()->set($key, ($count->data ?? 0) + 1, time() + 3600);
  }
}
```

## Best Practices

1. **Always cache responses** to reduce API calls
2. **Implement fallbacks** for when the service is unavailable
3. **Monitor usage** and set up alerts for anomalies
4. **Use async processing** for bulk operations
5. **Implement proper error handling** with user-friendly messages
6. **Test privacy modes** thoroughly before production
7. **Document all customizations** for future maintenance

## Troubleshooting

### Common Issues

1. **Connection Refused**: Check if bridge service is running
2. **Authentication Failed**: Verify API key and permissions
3. **Rate Limit Exceeded**: Implement caching and throttling
4. **Privacy Mode Conflicts**: Ensure consistent settings across services

### Debug Mode

Enable debug logging in Drupal:

```php
// settings.php
$settings['apple_fm_debug'] = TRUE;
$config['apple_fm_integration.settings']['debug_mode'] = TRUE;
```

### Health Checks

Implement health check endpoints:

```javascript
// Bridge service
app.get('/health', (req, res) => {
  res.json({
    status: 'healthy',
    version: '1.0.0',
    privacy_mode: client.privacyMode,
    uptime: process.uptime()
  });
});
```