# Extensibility & Integration Guide for AI Agents

This document provides guidance for AI agents on handling new project types, plugins, and integrations within the LLM Platform AI Platform ecosystem.

##  Understanding the Modular Architecture

The LLM Platform AI Platform follows a modular, extensible architecture where:

1. **Each directory is a separate project**: Not a monorepo, but a collection of independent projects
2. **Projects communicate via well-defined APIs**: OpenAPI-specified interfaces
3. **Extension points exist at multiple levels**: From core services to plugins
4. **New integrations follow standardized patterns**: For consistency and interoperability

## 🚦 Core Extension Principles

When working with extensions and integrations, always follow these core principles:

1. **Respect project boundaries**: Never assume cross-project dependencies
2. **Follow the OpenAPI-first approach**: Define interfaces before implementation
3. **Maintain backward compatibility**: Add, don't modify or remove
4. **Test-first development**: Write tests for extensions before implementing
5. **Document extension points**: Make integration clear for future developers

## 🔄 Types of Extensions in the Platform

### 1. Service-Level Extensions

Adding new capabilities to existing services:

| Service | Extension Points | How to Extend |
|---------|-----------------|---------------|
| bfapi | Endpoints, Middleware, Authentication Providers | Add to `src/routes/`, `src/middleware/`, or `src/auth/` |
| bfllm | Models, Training Pipelines, Agent Types | Add to `src/models/`, `src/training/`, or `src/agents/` |
| llm-mcp | Tools, Resource Types, Protocol Extensions | Add to `src/tools/`, `src/resources/`, or `src/protocol/` |
| bfrfp | Content Processors, Renderers, Data Sources | Add to `src/processors/`, `src/renderers/`, or `src/sources/` |

### 2. Plugin Systems

Several services support formal plugin architectures:

- **llmcli Plugins**: Command extensions for the CLI tool
- **bfllm Model Providers**: New model backends and providers
- **llm-mcp Tool Providers**: New tool implementations for agents
- **llm_platform_recipe Modules**: Drupal modules that extend the platform

### 3. Integration Points

Cross-service integrations occur through:

- **API Gateways**: Centralized by bfapi
- **Event Systems**: For asynchronous communication
- **Shared Protocols**: Such as the Model Context Protocol (MCP)
- **Configuration**: Via shared configuration systems

## 📋 Adding New Project Types

When introducing a completely new project type to the platform:

### 1. Project Structure

```
/new-project-name/
├── README.md          # Project documentation
├── CLAUDE.md          # AI agent guidance specific to this project
├── src/               # Source code
├── __tests__/         # Test files
├── docs/              # Documentation
├── package.json       # Dependencies and scripts
├── tsconfig.json      # TypeScript configuration
└── openapi.yaml       # API specification
```

### 2. Required Files

Every new project must include:

- **README.md**: Project overview and getting started guide
- **CLAUDE.md**: AI agent guidance specific to this project
- **openapi.yaml**: If the project exposes APIs
- **package.json**: With standardized scripts (dev, build, test, lint)
- **tsconfig.json**: For TypeScript projects, extending from base config

### 3. Integration Requirements

New projects must:

- Define clear API contracts if they expose services
- Include comprehensive test coverage (95%+)
- Document extension points and integration patterns
- Follow the platform's security and authentication standards
- Implement health checks and monitoring

### 4. Example: Adding a New Service

```typescript
// 1. Define the OpenAPI specification first
// openapi.yaml

// 2. Create service implementation
// src/index.ts
import express from 'express';
import [OpenApiValidator] from 'express-openapi-validator';

const app = express();

// Validate requests/responses against OpenAPI spec
app.use(OpenApiValidator.middleware({
  apiSpec: './openapi.yaml',
  validateRequests: true,
  validateResponses: true,
}));

// Implement endpoints defined in OpenAPI spec
app.get('/resources', (req, res) => [// Implementation here]);

export default app;

// 3. Write tests that validate against the OpenAPI spec
// __tests__/api.test.ts
describe('API', () => {
  test('GET /resources should return valid response', async () => {
    // Test implementation
  });
});
```

## 🔌 Creating Plugins

The platform supports several plugin systems. Here's how to work with each:

### llmcli Plugins

1. **Plugin Structure**:
   ```
   plugin-name/
   ├── index.js          # Plugin entry point
   ├── commands/         # Custom commands
   ├── package.json      # Plugin metadata and dependencies
   └── README.md         # Plugin documentation
   ```

2. **Plugin Registration**:
   ```javascript
   // index.js
   module.exports = {
     name: 'my-plugin',
     version: '1.0.0',
     commands: [
       require('./commands/my-command')
     ],
     hooks: {
       beforeCommand: (command) => {
         // Hook implementation
       }
     }
   };
   ```

3. **Command Implementation**:
   ```javascript
   // commands/my-command.js
   module.exports = {
     name: 'my-command',
     description: 'My custom command',
     options: [
       { name: '--option', description: 'An option for the command' }
     ],
     action: async (args, context) => [// Command implementation]
   };
   ```

### bfllm Model Providers

1. **Provider Structure**:
   ```
   src/models/providers/my-provider/
   ├── index.ts            # Provider entry point
   ├── client.ts           # API client implementation
   ├── types.ts            # Type definitions
   ├── transformer.ts      # Input/output transformers
   └── __tests__/          # Provider tests
   ```

2. **Provider Implementation**:
   ```typescript
   // index.ts
   import [ModelProvider] from '../../types';
   import [MyProviderClient] from './client';
   import { transformInput, transformOutput } from './transformer';
   
   export class MyModelProvider implements ModelProvider {
     id = 'my-provider';
     supportedModels = ['model-a', 'model-b'];
     
     async initialize(config) {
       this.client = new MyProviderClient(config);
     }
     
     async generateText(input) {
       const formattedInput = transformInput(input);
       const result = await this.client.generate(formattedInput);
       return transformOutput(result);
     }
     
     // Other required methods
   }
   ```

3. **Provider Registration**:
   ```typescript
   // src/models/registry.ts
   import [MyModelProvider] from './providers/my-provider';
   
   export function registerProviders() {
     modelRegistry.register(new MyModelProvider());
   }
   ```

### llm-mcp Tool Providers

1. **Tool Structure**:
   ```
   src/tools/my-tool/
   ├── index.ts         # Tool entry point
   ├── executor.ts      # Tool execution logic
   ├── schema.ts        # Tool schema definition
   └── __tests__/       # Tool tests
   ```

2. **Tool Implementation**:
   ```typescript
   // index.ts
   import [Tool] from '../types';
   import [execute] from './executor';
   import [schema] from './schema';
   
   export const myTool: Tool = {
     name: 'my-tool',
     description: 'A tool that does something useful',
     schema,
     execute,
   };
   ```

3. **Tool Registration**:
   ```typescript
   // src/tools/registry.ts
   import [myTool] from './my-tool';
   
   export function registerTools() {
     toolRegistry.register(myTool);
   }
   ```

### llm_platform_recipe Modules

1. **Module Structure**:
   ```
   modules/my_module/
   ├── my_module.info.yml       # Module information
   ├── my_module.module         # Module hooks
   ├── my_module.services.yml   # Service definitions
   ├── my_module.routing.yml    # Routing configuration
   ├── src/                     # PHP classes
   └── tests/                   # Module tests
   ```

2. **Module Information**:
   ```yaml
   # my_module.info.yml
   name: My Module
   type: module
   description: 'A module that extends the LLM Platform.'
   package: LLM Platform
   core_version_requirement: ^11
   dependencies:
     - llm_platform_recipe:llm_platform_recipe
   ```

3. **Service Definition**:
   ```yaml
   # my_module.services.yml
   services:
     my_module.service:
       class: Drupal\my_module\Service\MyService
       arguments: ['@llm_platform_recipe.service']
   ```

## 🔗 Third-Party Integrations

When integrating with external systems and third-party services:

### 1. Adapter Pattern

Always use the adapter pattern to abstract third-party APIs:

```typescript
// src/adapters/third-party-service.ts
export interface ThirdPartyServiceAdapter {
  initialize(config: ThirdPartyConfig): Promise<void>;
  performAction(input: ActionInput): Promise<ActionResult>;
  // Other methods
}

export class ThirdPartyServiceAdapterImpl implements ThirdPartyServiceAdapter {
  private client: ThirdPartyClient;
  
  async initialize(config: ThirdPartyConfig): Promise<void> {
    this.client = new ThirdPartyClient(config);
    await this.client.connect();
  }
  
  async performAction(input: ActionInput): Promise<ActionResult> {
    // Transform platform format to third-party format
    const thirdPartyInput = this.transformInput(input);
    
    // Call third-party API
    const thirdPartyResult = await this.client.action(thirdPartyInput);
    
    // Transform result back to platform format
    return this.transformOutput(thirdPartyResult);
  }
  
  private transformInput(input: ActionInput): ThirdPartyInput [// Implementation]
  
  private transformOutput(output: ThirdPartyOutput): ActionResult [// Implementation]
}
```

### 2. Configuration

Store third-party service configuration securely:

```typescript
// src/config/third-party.ts
export interface ThirdPartyConfig {
  apiKey: string;
  endpoint: string;
  timeout: number;
  // Other configuration options
}

// Access via configuration system
const config = configManager.get<ThirdPartyConfig>('services.thirdParty');
```

### 3. Error Handling

Implement proper error handling for third-party services:

```typescript
try {
  const result = await thirdPartyAdapter.performAction(input);
  return result;
} catch (error) {
  if (error instanceof ThirdPartyRateLimitError) {
    // Handle rate limiting
    logger.warn('Rate limit exceeded for third-party service');
    throw new ServiceError('RATE_LIMIT_EXCEEDED', 'Please try again later');
  } else if (error instanceof ThirdPartyAuthError) {
    // Handle authentication errors
    logger.error('Authentication failed for third-party service');
    throw new ServiceError('AUTH_ERROR', 'Service authentication failed');
  } else {
    // Handle other errors
    logger.error('Unknown error from third-party service', error);
    throw new ServiceError('SERVICE_ERROR', 'An error occurred with the service');
  }
}
```

##  Event-Based Integration

The platform uses an event system for loosely coupled integrations:

### 1. Event Definition

```typescript
// src/events/types.ts
export interface ModelTrainingStartedEvent {
  type: 'model.training.started';
  modelId: string;
  timestamp: number;
  params: {
    // Training parameters
  };
}

export type PlatformEvent = 
  | ModelTrainingStartedEvent
  | ModelTrainingCompletedEvent
  | // Other event types
```

### 2. Event Publishing

```typescript
// src/services/training.ts
import [eventBus] from '../events';

export class TrainingService {
  async startTraining(modelId: string, params: TrainingParams): Promise<void> {
    // Start training process
    
    // Publish event
    eventBus.publish({
      type: 'model.training.started',
      modelId,
      timestamp: Date.now(),
      params,
    });
  }
}
```

### 3. Event Subscription

```typescript
// src/monitoring/training-monitor.ts
import [eventBus] from '../events';

export class TrainingMonitor {
  initialize(): void {
    eventBus.subscribe('model.training.started', this.handleTrainingStarted.bind(this));
    eventBus.subscribe('model.training.completed', this.handleTrainingCompleted.bind(this));
  }
  
  private handleTrainingStarted(event: ModelTrainingStartedEvent): void [// Implementation]
  
  private handleTrainingCompleted(event: ModelTrainingCompletedEvent): void [// Implementation]
}
```

##  Best Practices for Integration

### 1. Versioning

All integrations should follow semantic versioning:

- **MAJOR**: Breaking changes
- **MINOR**: New features, backward compatible
- **PATCH**: Bug fixes, backward compatible

### 2. Testing Integration Points

Always write comprehensive tests for integration points:

```typescript
// __tests__/integration/third-party.test.ts
describe('ThirdPartyIntegration', () => {
  let adapter: ThirdPartyServiceAdapter;
  let mockServer: MockServer;
  
  beforeAll(async () => {
    // Set up mock server
    mockServer = new MockServer();
    await mockServer.start();
    
    // Configure adapter to use mock server
    adapter = new ThirdPartyServiceAdapterImpl();
    await adapter.initialize({
      endpoint: mockServer.url,
      apiKey: 'test-key',
      timeout: 1000,
    });
  });
  
  afterAll(async () => {
    await mockServer.stop();
  });
  
  test('performAction should correctly transform input and output', async () => [// Arrange
    const input = { /* test input */];
    mockServer.addMockResponse('/action', [/* mock response */]);
    
    // Act
    const result = await adapter.performAction(input);
    
    // Assert
    expect(result).toEqual([/* expected output */]);
    expect(mockServer.getLastRequest('/action')).toEqual([/* expected request */]);
  });
  
  test('should handle rate limiting errors', async () => [// Arrange
    const input = { /* test input */];
    mockServer.addMockError('/action', {
      status: 429,
      body: { error: 'Rate limit exceeded' },
    });
    
    // Act & Assert
    await expect(adapter.performAction(input)).rejects.toThrow('RATE_LIMIT_EXCEEDED');
  });
});
```

### 3. Documentation

Document all integration points thoroughly:

```typescript
/**
 * Adapter for interacting with the ThirdParty service.
 * 
 * This adapter provides a standardized interface for the platform to interact with
 * ThirdParty's API, handling authentication, error mapping, and data transformation.
 * 
 * @example
 * ```typescript
 * const adapter = new ThirdPartyServiceAdapterImpl();
 * await adapter.initialize({
 *   apiKey: process.env.THIRD_PARTY_API_KEY,
 *   endpoint: 'https://api.thirdparty.com/v1',
 *   timeout: 5000,
 * });
 * 
 * const result = await adapter.performAction({
 *   param1: 'value1',
 *   param2: 'value2',
 * });
 * ```
 * 
 * @see https://docs.thirdparty.com/api/v1 for the ThirdParty API documentation
 */
export class ThirdPartyServiceAdapterImpl implements ThirdPartyServiceAdapter [// Implementation]
```

### 4. Configuration Management

Use a centralized configuration system:

```typescript
// src/config/index.ts
import [ConfigManager] from '../lib/config';

export const configManager = new ConfigManager({
  services: {
    thirdParty: {
      apiKey: process.env.THIRD_PARTY_API_KEY,
      endpoint: process.env.THIRD_PARTY_ENDPOINT || 'https://api.thirdparty.com/v1',
      timeout: parseInt(process.env.THIRD_PARTY_TIMEOUT || '5000', 10),
    },
    // Other service configurations
  },
  // Other configuration sections
});
```

### 5. Health Checks

Implement health checks for all integrations:

```typescript
// src/health/third-party.ts
import { HealthCheck, HealthStatus } from '../types/health';
import [thirdPartyAdapter] from '../adapters/third-party';

export class ThirdPartyHealthCheck implements HealthCheck {
  async check(): Promise<HealthStatus> {
    try {
      await thirdPartyAdapter.ping();
      
      return {
        status: 'healthy',
        service: 'third-party',
        timestamp: Date.now(),
      };
    } catch (error) {
      return {
        status: 'unhealthy',
        service: 'third-party',
        error: error.message,
        timestamp: Date.now(),
      };
    }
  }
}
```

## 📋 AI Agent Checklist for Extensions and Integrations

When working with extensions and integrations:

1. ** Respect Project Boundaries**: Treat each project as independent
2. ** Follow the OpenAPI-First Approach**: Define interfaces before implementation
3. ** Use the Adapter Pattern**: Abstract third-party services
4. ** Implement Error Handling**: Handle all error scenarios
5. ** Write Comprehensive Tests**: Test all integration points
6. ** Document Integration Points**: Provide clear documentation
7. ** Implement Health Checks**: Monitor integration health
8. ** Follow Security Best Practices**: Secure all integration points
9. ** Use Event-Based Integration**: For loose coupling when appropriate
10. ** Follow the Platform's Patterns**: Be consistent with existing code

## 🔄 Common Integration Scenarios

### 1. Adding a New LLM Provider

```typescript
// 1. Create provider adapter
// src/models/providers/new-provider/index.ts
import [ModelProvider] from '../../types';

export class NewProviderAdapter implements ModelProvider [// Implementation]

// 2. Register provider
// src/models/registry.ts
import [NewProviderAdapter] from './providers/new-provider';

modelRegistry.register(new NewProviderAdapter());

// 3. Add provider configuration
// src/config/providers.ts
export const providerConfig = {
  'new-provider': {
    apiKey: process.env.NEW_PROVIDER_API_KEY,
    endpoint: process.env.NEW_PROVIDER_ENDPOINT,
    // Other configuration
  },
  // Other providers
};
```

### 2. Adding a New Tool for Agents

```typescript
// 1. Define tool schema
// src/tools/new-tool/schema.ts
export const schema = {
  type: 'object',
  properties: {
    input: {
      type: 'string',
      description: 'The input for the tool',
    },
  },
  required: ['input'],
};

// 2. Implement tool executor
// src/tools/new-tool/executor.ts
export async function execute(params, context) {
  // Implementation
  return {
    result: 'Success',
  };
}

// 3. Register tool
// src/tools/new-tool/index.ts
import [Tool] from '../types';
import [schema] from './schema';
import [execute] from './executor';

export const newTool: Tool = {
  name: 'new-tool',
  description: 'A new tool for agents',
  schema,
  execute,
};

// src/tools/registry.ts
import [newTool] from './new-tool';

toolRegistry.register(newTool);
```

### 3. Integrating with a Third-Party API

```typescript
// 1. Create adapter
// src/adapters/third-party-api.ts
export class ThirdPartyApiAdapter [// Implementation]

// 2. Create service
// src/services/third-party.ts
import [ThirdPartyApiAdapter] from '../adapters/third-party-api';

export class ThirdPartyService {
  private adapter: ThirdPartyApiAdapter;
  
  constructor(config) {
    this.adapter = new ThirdPartyApiAdapter(config);
  }
  
  // Service methods
}

// 3. Expose via API
// src/routes/third-party.ts
import [Router] from 'express';
import [ThirdPartyService] from '../services/third-party';

const router = Router();
const service = new ThirdPartyService(config.thirdParty);

router.get('/third-party/resources', async (req, res) => {
  try {
    const resources = await service.getResources(req.query);
    res.json(resources);
  } catch (error) [// Error handling]
});

export default router;
```

---

By following this guide, AI agents will be able to consistently and effectively work with extensions and integrations in the LLM Platform AI Platform, ensuring that new components maintain the platform's architecture, standards, and best practices.