# Quick Reference Guide

This document provides in-context help, code snippets, and links to key documentation for AI agents and human contributors working with the LLM Platform AI Platform codebase.

## 📋 Table of Contents

- [Common Commands](#common-commands)
- [Code Snippets](#code-snippets)
- [Project Structure](#project-structure)
- [Key Documentation](#key-documentation)
- [Workflow Guides](#workflow-guides)
- [Troubleshooting](#troubleshooting)

##  Common Commands

### Build & Development

```bash
# Navigate to specific project directory first!
cd [project_directory]

# Development server
npm run dev

# Build project
npm run build

# Watch mode
npm run build:watch

# Production build
npm run build:prod
```

### Testing

```bash
# Run tests
npm test

# Watch tests
npm run test:watch

# Test coverage
npm run test:coverage

# Run specific tests
npm test -- -t "test name pattern"

# Drupal tests
cd llm_platform_recipe
./vendor/bin/phpunit
```

### Linting & Formatting

```bash
# Lint code
npm run lint

# Fix linting issues
npm run lint:fix

# Format code
npm run format

# Check formatting
npm run format:check

# TypeScript type checking
npm run typecheck
```

### Docker & Deployment

```bash
# Build Docker image
npm run docker:build

# Run Docker container
npm run docker:run

# Deploy to Kubernetes
npm run helm:upgrade

# Deploy to staging
npm run deploy:staging
```

## 💻 Code Snippets

### API Endpoint (Express)

```typescript
// src/controllers/modelController.ts
import { Request, Response } from 'express';
import [ModelService] from '../services/modelService';
import [logger] from '../utils/logger';

export class ModelController {
  constructor(private modelService: ModelService) {}
  
  async getModelById(req: Request, res: Response): Promise<void> {
    try {
      const modelId = req.params.id;
      const model = await this.modelService.getModelById(modelId);
      res.json(model);
    } catch (error) {
      logger.error('Error retrieving model:', error);
      res.status(500).json({ error: 'Internal server error' });
    }
  }
  
  async createModel(req: Request, res: Response): Promise<void> {
    try {
      const model = await this.modelService.createModel(req.body);
      res.status(201).json(model);
    } catch (error) {
      logger.error('Error creating model:', error);
      res.status(500).json({ error: 'Internal server error' });
    }
  }
}

// src/routes/modelRoutes.ts
import [Router] from 'express';
import [ModelController] from '../controllers/modelController';
import { authenticate, validate } from '../middleware';

export function createModelRoutes(modelController: ModelController): Router {
  const router = Router();
  
  router.get('/models/:id', authenticate, modelController.getModelById.bind(modelController));
  router.post('/models', authenticate, validate('Model'), modelController.createModel.bind(modelController));
  
  return router;
}
```

### React Component

```tsx
// src/components/ModelSelector.tsx
import React from 'react';
import { Select, FormLabel, Box, Text } from '@chakra-ui/react';

interface Model {
  id: string;
  name: string;
  description?: string;
}

interface ModelSelectorProps {
  models: Model[];
  selectedModelId: string | null;
  onChange: (modelId: string) => void;
  isDisabled?: boolean;
  label?: string;
}

export const ModelSelector: React.FC<ModelSelectorProps> = ({
  models,
  selectedModelId,
  onChange,
  isDisabled = false,
  label = 'Select a model'
}) => {
  if (!models || models.length === 0) {
    return <Text color="gray.500">No models available</Text>;
  }

  return (
    <Box mb=[4]>
      <FormLabel htmlFor="model-selector">[label]</FormLabel>
      <Select
        id="model-selector"
        value=[selectedModelId || '']
        onChange={(e) => onChange(e.target.value)}
        isDisabled=[isDisabled]
        data-testid="model-selector"
      >
        <option value="" disabled>Select a model</option>
        {models.map((model) => (
          <option key={model.id} value=[model.id]>
            [model.name]
          </option>
        ))}
      </Select>
    </Box>
  );
};

// Test file
// src/components/__tests__/ModelSelector.test.tsx
import { render, screen, fireEvent } from '@testing-library/react';
import [ModelSelector] from '../ModelSelector';

describe('ModelSelector', () => {
  const mockModels = [
    { id: 'model1', name: 'Model 1' },
    { id: 'model2', name: 'Model 2' }
  ];
  
  it('renders correctly with models', () => [render(
      <ModelSelector
        models={mockModels]
        selectedModelId=[null]
        onChange=[jest.fn()]
      />
    );
    
    expect(screen.getByText('Select a model')).toBeInTheDocument();
    expect(screen.getByText('Model 1')).toBeInTheDocument();
    expect(screen.getByText('Model 2')).toBeInTheDocument();
  });
  
  it('calls onChange when selection changes', () => {
    const handleChange = jest.fn();
    
    render(
      <ModelSelector
        models={mockModels}
        selectedModelId=[null]
        onChange=[handleChange]
      />
    );
    
    fireEvent.change(screen.getByTestId('model-selector'), {
      target: { value: 'model2' }
    });
    
    expect(handleChange).toHaveBeenCalledWith('model2');
  });
});
```

### Service Class

```typescript
// src/services/modelService.ts
import [ModelRepository] from '../repositories/modelRepository';
import { Model, ModelCreateInput } from '../types';
import [ModelNotFoundError] from '../errors';
import [logger] from '../utils/logger';

export class ModelService {
  constructor(private modelRepository: ModelRepository) {}
  
  async getModelById(id: string): Promise<Model> {
    logger.debug(`Getting model with ID: item`);
    
    const model = await this.modelRepository.findById(id);
    
    if (!model) {
      throw new ModelNotFoundError(`Model with ID item not found`);
    }
    
    return model;
  }
  
  async createModel(input: ModelCreateInput): Promise<Model> {
    logger.debug('Creating new model', { input });
    
    // Validate input
    if (!input.name) {
      throw new Error('Model name is required');
    }
    
    // Create model
    const model = await this.modelRepository.create({
      ...input,
      createdAt: new Date().toISOString()
    });
    
    return model;
  }
}

// Test file
// src/services/__tests__/modelService.test.ts
import [ModelService] from '../modelService';
import [ModelRepository] from '../../repositories/modelRepository';
import [ModelNotFoundError] from '../../errors';

describe('ModelService', () => {
  let modelService: ModelService;
  let mockModelRepository: jest.Mocked<ModelRepository>;
  
  beforeEach(() => {
    mockModelRepository = {
      findById: jest.fn(),
      create: jest.fn()
    } as unknown as jest.Mocked<ModelRepository>;
    
    modelService = new ModelService(mockModelRepository);
  });
  
  describe('getModelById', () => {
    it('returns model when found', async () => {
      const mockModel = { id: 'test-id', name: 'Test Model' };
      mockModelRepository.findById.mockResolvedValue(mockModel);
      
      const result = await modelService.getModelById('test-id');
      
      expect(result).toEqual(mockModel);
      expect(mockModelRepository.findById).toHaveBeenCalledWith('test-id');
    });
    
    it('throws ModelNotFoundError when model not found', async () => {
      mockModelRepository.findById.mockResolvedValue(null);
      
      await expect(modelService.getModelById('test-id'))
        .rejects.toThrow(ModelNotFoundError);
    });
  });
});
```

### Drupal Module

```php
<?php

/**
 * @file
 * Contains llm_platform_recipe.module.
 */

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Implements hook_help().
 */
function llm_platform_recipe_help($route_name, RouteMatchInterface $route_match) {
  switch ($route_name) {
    case 'help.page.llm_platform_recipe':
      $output = '';
      $output .= '<h3>' . t('About') . '</h3>';
      $output .= '<p>' . t('LLM Platform Recipe provides integration with AI services.') . '</p>';
      return $output;
  }
}

/**
 * Implements hook_entity_presave().
 */
function llm_platform_recipe_entity_presave(EntityInterface $entity) {
  // Process entities before saving
  if ($entity->getEntityTypeId() === 'node' && $entity->bundle() === 'article') {
    // Get the AI service
    $ai_service = \Drupal::service('llm_platform_recipe.ai_service');
    
    // Process content with AI
    if ($entity->hasField('body') && !$entity->get('body')->isEmpty()) {
      $body = $entity->get('body')->value;
      $enhanced_body = $ai_service->enhanceContent($body);
      $entity->set('body', $enhanced_body);
    }
  }
}
```

## 🏢 Project Structure

Each project in the platform follows this basic structure:

```
project/
├── config/              # Configuration files
├── dist/                # Compiled output
├── docs/                # Documentation
├── src/                 # Source code
│   ├── api/             # API-related code
│   ├── components/      # UI components
│   ├── services/        # Business logic
│   ├── repositories/    # Data access
│   ├── utils/           # Utility functions
│   └── index.ts         # Main entry point
├── __tests__/           # Test files
│   ├── unit/            # Unit tests
│   ├── integration/     # Integration tests
│   └── e2e/             # End-to-end tests
├── package.json         # Dependencies and scripts
├── tsconfig.json        # TypeScript configuration
└── README.md            # Project documentation
```

##  Key Documentation

### Project Documentation

| Project | Documentation | Description |
|---------|---------------|-------------|
| **Overall** | [CLAUDE.md](./CLAUDE.md) | Main guide for AI agents working in this repo |
| **bfapi** | [README.md](./bfapi/README.md) | API Gateway documentation |
| **bfllm** | [README.md](./bfllm/README.md) | LLM Inference Engine documentation |
| **llm-mcp** | [README.md](./llm-mcp/README.md) | MCP Protocol Server documentation |
| **bfrfp** | [README.md](./bfrfp/README.md) | Content Engine documentation |
| **llm_platform_recipe** | [README.md](./llm_platform_recipe/README.md) | Drupal Recipe documentation |

### Architecture & Design Documents

| Document | Path | Description |
|----------|------|-------------|
| Architecture Overview | [docs/architecture.md](./docs/architecture.md) | High-level architecture |
| API Interfaces | [bfapi/docs/api.md](./bfapi/docs/api.md) | API interface documentation |
| LLM Integration | [bfllm/docs/integration.md](./bfllm/docs/integration.md) | LLM integration guide |
| Drupal Integration | [docs/DRUPAL_INTEGRATION.md](./docs/DRUPAL_INTEGRATION.md) | Drupal integration guide |
| OpenAPI Guidelines | [OPENAPI_GUIDELINES.md](./OPENAPI_GUIDELINES.md) | OpenAPI development standards |
| TDD Methodology | [TDD_CICD_GUIDE.md](./TDD_CICD_GUIDE.md) | Test-Driven Development guide |

### Standards & Guidelines

| Document | Path | Description |
|----------|------|-------------|
| AI Agent Onboarding | [AI_AGENT_ONBOARDING.md](./AI_AGENT_ONBOARDING.md) | Onboarding for AI agents |
| Project Integration | [PROJECT_INTEGRATION_GUIDE.md](./PROJECT_INTEGRATION_GUIDE.md) | Guide for integrating new projects |
| Agent Automation | [AGENT_AUTOMATION_GUIDE.md](./AGENT_AUTOMATION_GUIDE.md) | Guide for agent-driven automation |
| OpenAPI Tools | [OPENAPI_TOOLS.md](./OPENAPI_TOOLS.md) | OpenAPI tools and resources |

## 🔄 Workflow Guides

### Adding a New API Endpoint

1. **Create Tests First**
   ```bash
   # Create test file
   touch __tests__/api/newEndpoint.test.ts
   ```

2. **Write failing tests**
   ```typescript
   // Write tests for the new endpoint
   describe('POST /api/v1/new-endpoint', () => {
     it('should create resource when valid input is provided', async () => {
       // Test implementation
     });
   });
   ```

3. **Create controller and route files**
   ```bash
   # Create controller and route files
   touch src/api/newEndpoint/newEndpoint.controller.ts
   touch src/api/newEndpoint/newEndpoint.routes.ts
   ```

4. **Implement controller and routes**
   ```typescript
   // Implement just enough to make tests pass
   ```

5. **Update OpenAPI specification**
   ```yaml
   # Add new endpoint to OpenAPI spec
   /new-endpoint:
     post:
       # OpenAPI definition
   ```

6. **Validate everything**
   ```bash
   # Run tests
   npm test
   
   # Validate OpenAPI spec
   npm run openapi:validate
   
   # Run linting
   npm run lint
   ```

### Creating a New React Component

1. **Create Tests First**
   ```bash
   # Create test file
   touch src/components/__tests__/NewComponent.test.tsx
   ```

2. **Write failing tests**
   ```tsx
   // Write tests for the new component
   describe('NewComponent', () => {
     it('should render correctly', () => {
       // Test implementation
     });
   });
   ```

3. **Create component file**
   ```bash
   # Create component file
   touch src/components/NewComponent.tsx
   ```

4. **Implement component**
   ```tsx
   // Implement just enough to make tests pass
   ```

5. **Validate everything**
   ```bash
   # Run tests
   npm test
   
   # Run linting
   npm run lint
   
   # Type check
   npm run typecheck
   ```

### Creating a New Drupal Module

1. **Create module skeleton**
   ```bash
   # Create module directory
   mkdir -p modules/custom/my_module
   
   # Create basic files
   touch modules/custom/my_module/my_module.info.yml
   touch modules/custom/my_module/my_module.module
   ```

2. **Create module info file**
   ```yaml
   # my_module.info.yml
   name: My Module
   type: module
   description: 'Description of my module'
   package: Custom
   core_version_requirement: ^9 || ^10
   dependencies:
     - llm_platform_recipe:llm_platform_recipe
   ```

3. **Create tests**
   ```bash
   # Create test directory
   mkdir -p modules/custom/my_module/tests/src/Unit
   
   # Create test file
   touch modules/custom/my_module/tests/src/Unit/MyModuleTest.php
   ```

4. **Implement tests**
   ```php
   <?php
   
   namespace Drupal\Tests\my_module\Unit;
   
   use Drupal\Tests\UnitTestCase;
   
   /**
    * Tests the functionality of my module.
    *
    * @group my_module
    */
   class MyModuleTest extends UnitTestCase [// Test implementation]
   ```

5. **Implement module functionality**
   ```php
   <?php
   
   /**
    * @file
    * Contains my_module.module.
    */
   
   // Module implementation
   ```

6. **Run tests**
   ```bash
   # Run PHPUnit tests
   ./vendor/bin/phpunit modules/custom/my_module
   ```

## 🛠️ Troubleshooting

### Common Issues and Solutions

#### Build Errors

**Issue**: Build fails with TypeScript errors
```
TS2307: Cannot find module './some-module' or its corresponding type declarations.
```

**Solution**:
1. Check import path - ensure the module exists
2. Check `tsconfig.json` for correct path configuration
3. Run `npm install` to ensure all dependencies are installed

#### Test Failures

**Issue**: Tests are failing with timeout errors
```
Timeout - Async callback was not invoked within the 5000 ms timeout specified by jest.setTimeout.
```

**Solution**:
1. Increase Jest timeout in `jest.config.js`:
```javascript
module.exports = {
  // ...
  testTimeout: 10000,
};
```
2. Check for async operations that might not be resolving

#### Dependency Issues

**Issue**: `npm install` fails with peer dependency warnings
```
npm WARN some-package@1.0.0 requires a peer of other-package@^2.0.0 but none is installed
```

**Solution**:
1. Install the peer dependency manually:
```bash
npm install other-package@^2.0.0 --save-dev
```
2. Update your package to use compatible versions

#### Authentication Issues

**Issue**: API requests failing with 401 Unauthorized

**Solution**:
1. Check if your JWT token is valid and not expired
2. Verify authentication headers are set correctly
3. Check server logs for more details about authentication failure

#### Database Connection Issues

**Issue**: Cannot connect to database

**Solution**:
1. Verify database credentials in `.env` file
2. Check if database server is running
3. Test connection with direct database client
4. Check network connectivity between service and database

### Getting Help

If you're stuck, here are resources for getting help:

1. **Project Documentation**: Check the project's README and docs directory
2. **Issue Tracker**: Search for similar issues in the GitLab/GitHub issue tracker
3. **Team Chat**: Ask in the team's communication channel
4. **Knowledge Base**: Check the internal knowledge base for solutions
5. **Logging**: Enable verbose logging for more diagnostic information

## 🔄 Regular Tasks

### Dependency Updates

```bash
# Check for outdated dependencies
npm outdated

# Update dependencies
npm update

# Update a specific dependency
npm install package-name@latest
```

### Performance Profiling

```bash
# Run performance tests
npm run test:performance

# Generate performance report
npm run analyze:performance
```

### Security Scanning

```bash
# Run security audit
npm audit

# Fix security issues
npm audit fix

# Run security scan
npm run security:scan
```

##  Further Reading

- [TypeScript Documentation](https://www.typescriptlang.org/docs/)
- [React Documentation](https://reactjs.org/docs/getting-started.html)
- [Express.js Documentation](https://expressjs.com/)
- [Jest Testing Framework](https://jestjs.io/docs/getting-started)
- [Drupal Documentation](https://www.drupal.org/docs)
- [OpenAPI Specification](https://spec.openapis.org/oas/v3.1.0)

---

This Quick Reference Guide provides essential information for working with the LLM Platform AI Platform codebase. For more detailed information, refer to the specific project documentation.