# AI Streaming Integration Guide

This document explains how to use the AI Streaming components implemented in the llm-mcp npm project.

## Overview

The AI Streaming system provides a robust framework for streaming AI responses with support for:

- Stream response handling
- State management
- Metrics collection and thresholds
- Error handling
- Provider integration

## Components

### Core Interfaces

```typescript
import { 
  AiOperationState, 
  AiStreamingService, 
  StreamingMetrics, 
  ChunkHandler, 
  StreamCallback 
} from '@llm-mcp/services/ai-streaming';
```

### Implementation Classes

```typescript
import { 
  AiStreamingServiceImpl, 
  StreamingState, 
  StreamingMetricsImpl 
} from '@llm-mcp/services/ai-streaming';
```

## Usage Examples

### Basic Streaming Response

```typescript
import [AiStreamingServiceImpl] from '@llm-mcp/services/ai-streaming';
import [getLogger] from '@llm-mcp/services/logger';
import [stateManager] from '@llm-mcp/services/state-manager';
import [configManager] from '@llm-mcp/services/config-manager';

// Create streaming service
const streamingService = new AiStreamingServiceImpl(
  getLogger('ai-streaming'),
  stateManager,
  configManager
);

// Create a streaming response
app.get('/stream-response', (req, res) => {
  return streamingService.createStreamingResponse((chunkHandler) => {
    // Send chunks with artificial delay
    chunkHandler('Hello', { tokensUsed: 1 });
    setTimeout(() => chunkHandler(' world', { tokensUsed: 1 }), 500);
    setTimeout(() => chunkHandler('!', { tokensUsed: 0 }), 1000);
  }, {
    headers: {
      'X-Custom-Header': 'custom-value',
    },
  });
});
```

### Chat Completion with Streaming

```typescript
import [AiStreamingServiceImpl] from '@llm-mcp/services/ai-streaming';
import [getLogger] from '@llm-mcp/services/logger';
import [stateManager] from '@llm-mcp/services/state-manager';
import [configManager] from '@llm-mcp/services/config-manager';
import [OpenAIProvider] from '@llm-mcp/providers/openai-provider';

// Create streaming service
const streamingService = new AiStreamingServiceImpl(
  getLogger('ai-streaming'),
  stateManager,
  configManager
);

// Create provider
const openai = new OpenAIProvider(
  configManager,
  getLogger('openai'),
  stateManager,
  streamingService
);

// Create a streaming chat endpoint
app.post('/chat-stream', async (req, res) => [const { messages] = req.body;
  return openai.chatStreamed(messages, {
    model: 'gpt-4',
    temperature: 0.7,
  });
});
```

### Implementing a Custom Provider

```typescript
import [AbstractAiProvider] from '@llm-mcp/providers/abstract-ai-provider';
import [ConfigManager] from '@llm-mcp/services/config-manager';
import [StateManager] from '@llm-mcp/services/state-manager';
import [Logger] from 'winston';
import [AiStreamingService] from '@llm-mcp/services/ai-streaming';
import [Response] from 'express';

export class CustomProvider extends AbstractAiProvider {
  constructor(
    configManager: ConfigManager,
    logger: Logger,
    stateManager: StateManager,
    streamingService: AiStreamingService
  ) {
    super('custom-provider', configManager, logger, stateManager, streamingService);
  }

  public getCapabilities(): string[] {
    return [
      'text_generation',
      'streaming',
      'chat',
    ];
  }

  public async generate(prompt: string, options: Record<string, any> = {}): Promise<Record<string, any>> {
    try {
      // Implementation for text generation
      const response = 'Generated text...';
      
      return {
        text: response,
        status: 'success',
      };
    } catch (error: any) {
      return this.handleError(error, 'generate', { prompt });
    }
  }

  public generateStreamed(prompt: string, options: Record<string, any> = {}): Response {
    return this.createStreamingResponse((chunkHandler) => {
      try {
        // Implementation for streaming generation
        chunkHandler('Streaming ', { tokensUsed: 1 });
        setTimeout(() => chunkHandler('text...', { tokensUsed: 1 }), 500);
      } catch (error: any) {
        this.logger.error('Error in streaming generation:', error);
        throw error;
      }
    });
  }

  public async chat(messages: any[], options: Record<string, any> = {}): Promise<Record<string, any>> {
    try {
      // Implementation for chat completion
      const response = 'Chat response...';
      
      return {
        text: response,
        message: {
          role: 'assistant',
          content: response,
        },
        status: 'success',
      };
    } catch (error: any) {
      return this.handleError(error, 'chat', { messages });
    }
  }

  public chatStreamed(messages: any[], options: Record<string, any> = {}): Response {
    return this.createStreamingResponse((chunkHandler) => {
      try {
        // Implementation for streaming chat
        chunkHandler('Chat ', { tokensUsed: 1 });
        setTimeout(() => chunkHandler('response...', { tokensUsed: 1 }), 500);
      } catch (error: any) {
        this.logger.error('Error in streaming chat:', error);
        throw error;
      }
    });
  }
}
```

## Integration with Drupal

### API Bridge

The npm package includes a bridge for Drupal integration:

```typescript
import [createDrupalContextBridge] from '@llm-mcp/bridges/drupal-context-bridge';

const bridge = createDrupalContextBridge({
  apiUrl: 'https://example.com/api',
  apiKey: 'your-api-key',
  timeout: 30000,
});

// Execute tools in Drupal context
const result = await bridge.executeInDrupalContext('tool_name', {
  param1: 'value1',
  param2: 'value2',
});
```

### Setting Up for Drupal Integration

1. Install the npm package in your Node.js project:

```bash
npm install @llm-mcp
```

2. Configure the integration in your Node.js application:

```typescript
import [configureBridge] from '@llm-mcp/bridges';

configureBridge({
  drupal: {
    apiUrl: process.env.DRUPAL_API_URL,
    apiKey: process.env.DRUPAL_API_KEY,
    contextSyncInterval: 5000,
  },
});
```

3. Enable the integration in your Drupal module's settings:

```php
$config = \Drupal::configFactory()->getEditable('drupal_native_ai_nativeaijs.settings');
$config->set('service_url', 'http://localhost:3000')
  ->set('enabled', TRUE)
  ->save();
```

## Advanced Configuration

### Metrics Thresholds

Configure thresholds for token usage and cost:

```typescript
configManager.set('ai-streaming.metrics-thresholds', {
  maxTokens: 2000000,  // 2 million tokens
  maxCost: 500,        // $500
});
```

### Custom Metrics

Add custom metrics to track specific usage patterns:

```typescript
const metrics = streamingService.createStreamingMetrics();
metrics.setTokensUsed(150);
metrics.setLatency(450);
metrics.setCost(0.002);
metrics.setCustomMetric('promptComplexity', 0.75);
metrics.setCustomMetric('domainSpecific', true);
```

## Error Handling

The streaming service provides robust error handling:

```typescript
try [// Your streaming code] catch (error) {
  // The streaming service will automatically:
  // 1. Update the state with error information
  // 2. Send an error response to the client
  // 3. Log the error with context
}
```