# AI Streaming and Vector Integration

This document provides a comprehensive guide to using the AI streaming, vector operations, and function calling capabilities that have been integrated from Drupal Native AI into the llm-mcp npm package.

## Table of Contents

- [Overview](#overview)
- [Core Components](#core-components)
- [Setup and Installation](#setup-and-installation)
- [Basic Usage](#basic-usage)
- [Advanced Usage](#advanced-usage)
- [Integration with Drupal](#integration-with-drupal)
- [Error Handling](#error-handling)
- [Performance Considerations](#performance-considerations)
- [Security Considerations](#security-considerations)

## Overview

The AI Streaming and Vector Integration provides a unified framework for:

1. **Streaming AI Responses**: Real-time streaming of AI-generated content
2. **Vector Operations**: Embedding generation, storage, and semantic search
3. **Function Calling**: Registration and execution of functions with AI models
4. **Provider Management**: Standardized interface for multiple AI providers

This integration reduces code duplication between Node.js and Drupal implementations while providing a robust, type-safe interface for AI operations.

## Core Components

### AI Streaming

- **AiOperationState**: Tracks the state of streaming operations
- **StreamingMetrics**: Collects and manages metrics for streaming operations
- **AiStreamingService**: Creates and manages streaming responses

### Vector Operations

- **VectorOperations**: Interface for vector database operations
- **VectorOperationsService**: Implementation of vector operations
- **VectorBridge**: Bridge to various vector database providers

### Function Calling

- **FunctionDefinition**: Defines callable functions and their parameters
- **FunctionManager**: Registers and executes functions
- **FunctionCall**: Represents a call to a function by an AI model

### Provider Framework

- **AiProvider**: Interface for AI providers
- **AbstractAiProvider**: Base class for AI providers
- **Provider Capabilities**: System for advertising and selecting providers based on capabilities

## Setup and Installation

### Prerequisites

- Node.js 20.x or later
- TypeScript 5.x or later
- Access to AI model APIs (OpenAI, Claude, etc.)
- Access to vector database (optional, for vector operations)

### Installation

```bash
npm install @bluefly/mcp
```

### Configuration

Create a configuration file (e.g., `config.json`):

```json
{
  "ai-streaming": {
    "metrics-thresholds": {
      "maxTokens": 1000000,
      "maxCost": 500
    }
  },
  "ai-providers": {
    "openai": {
      "enabled": true,
      "priority": 10,
      "default": true,
      "apiKey": "your-api-key"
    }
  },
  "vector-operations": {
    "endpoint": "http://localhost:3001/api/vectors",
    "useQdrant": false,
    "dimensions": 1536
  }
}
```

## Basic Usage

### Initializing Services

```typescript
import { 
  AiStreamingServiceImpl, 
  VectorOperationsService,
  FunctionManagerImpl,
  ConfigManager,
  StateManager 
} from '@bluefly/mcp';

// Initialize services
const logger = createLogger({ component: 'app' });
const configManager = new ConfigManager('path/to/config.json');
const stateManager = new StateManager();

// Create streaming service
const streamingService = new AiStreamingServiceImpl(
  logger, 
  stateManager, 
  configManager
);

// Create vector operations service
const vectorOperations = new VectorOperationsService(
  logger,
  configManager
);

// Create function manager
const functionManager = new FunctionManagerImpl(
  logger,
  configManager
);
```

### Using AI Streaming

```typescript
// In an Express route
app.get('/stream-demo', (req, res) => {
  return streamingService.createStreamingResponse((chunkHandler) => {
    // Send chunks with artificial delay
    chunkHandler('Hello', { tokensUsed: 1 });
    setTimeout(() => chunkHandler(' world', { tokensUsed: 1 }), 500);
    setTimeout(() => chunkHandler('!', { tokensUsed: 0 }), 1000);
  });
});
```

### Using Vector Operations

```typescript
// Generate embeddings
const embeddings = await vectorOperations.createEmbeddings('This is a test text');

// Store vectors
await vectorOperations.storeVectors([
  {
    id: 'vector-1',
    values: embeddings.data[0].embedding,
    metadata: { text: 'This is a test text' }
  }
], 'test-collection');

// Search for similar vectors
const searchResults = await vectorOperations.searchVectors(
  embeddings.data[0].embedding,
  'test-collection',
  { limit: 5 }
);

// Perform RAG
const ragResults = await vectorOperations.performRag(
  'What is a test?',
  'test-collection'
);
```

### Using Function Calling

```typescript
// Define a function
const calculateFunction = {
  name: 'calculate',
  description: 'Perform a calculation',
  parameters: {
    type: 'object',
    properties: {
      operation: {
        type: 'string',
        enum: ['add', 'subtract', 'multiply', 'divide'],
        description: 'The operation to perform'
      },
      a: { type: 'number', description: 'First number' },
      b: { type: 'number', description: 'Second number' }
    },
    required: ['operation', 'a', 'b']
  }
};

// Register the function
functionManager.registerFunction(calculateFunction);

// Register the implementation
functionManager.registerFunctionImplementation('calculate', async (args) => {
  const { operation, a, b } = args;
  
  switch (operation) {
    case 'add': return { result: a + b };
    case 'subtract': return { result: a - b };
    case 'multiply': return { result: a * b };
    case 'divide': 
      if (b === 0) throw new Error('Division by zero');
      return { result: a / b };
    default:
      throw new Error('Unknown operation');
  }
});

// Execute the function
const result = await functionManager.executeFunction('calculate', {
  operation: 'add',
  a: 2,
  b: 3
});
// result.result.result = 5
```

## Advanced Usage

### Combining Streaming, Vector Operations, and Function Calling

```typescript
app.post('/rag-stream', (req, res) => {
  const { query, collection } = req.body;
  
  return streamingService.createStreamingResponse(async (chunkHandler) => {
    try {
      // Execute RAG function
      const ragResult = await functionManager.executeFunction('performRag', {
        query,
        collection,
        limit: 5
      });
      
      if (ragResult.status === 'error') {
        chunkHandler(`Error: [ragResult.error]`, { error: ragResult.error });
        return;
      }
      
      // Stream response
      chunkHandler('Based on the search results:\n\n', { tokensUsed: 5 });
      
      // Send context chunks
      const contextParts = ragResult.result.context.split('. ');
      for (const part of contextParts) {
        // Add a small delay to simulate typing
        await new Promise(resolve => setTimeout(resolve, 300));
        chunkHandler(part + '. ', { tokensUsed: part.length / 4 });
      }
      
      // Add sources
      chunkHandler('\n\nSources:\n', { tokensUsed: 2 });
      
      for (const source of ragResult.result.sources) {
        chunkHandler(`- [source.metadata.title || source.id]\n`, { tokensUsed: 2 });
      }
    } catch (error: any) {
      chunkHandler(`Error: [error.message]`, { error: error.message });
    }
  });
});
```

### Creating Custom AI Providers

```typescript
import [AbstractAiProvider] from '@bluefly/mcp';

class CustomAiProvider extends AbstractAiProvider {
  constructor(
    configManager,
    logger,
    stateManager,
    streamingService
  ) {
    super('custom-provider', configManager, logger, stateManager, streamingService);
  }
  
  public getCapabilities(): string[] {
    return ['text_generation', 'streaming', 'chat', 'function_calling'];
  }
  
  public async generate(prompt: string, options?: Record<string, any>): Promise<Record<string, any>> {
    // Implement generation logic
    return {
      text: 'Generated text...',
      usage: { total_tokens: 10 }
    };
  }
  
  public generateStreamed(prompt: string, options?: Record<string, any>) {
    return this.createStreamingResponse((chunkHandler) => {
      // Implement streaming logic
      chunkHandler('Generated ', { tokensUsed: 1 });
      setTimeout(() => chunkHandler('text...', { tokensUsed: 1 }), 500);
    });
  }
  
  public async chat(messages: any[], options?: Record<string, any>): Promise<Record<string, any>> {
    // Implement chat logic
    return {
      message: { role: 'assistant', content: 'Response...' },
      usage: { total_tokens: 10 }
    };
  }
  
  public chatStreamed(messages: any[], options?: Record<string, any>) {
    return this.createStreamingResponse((chunkHandler) => {
      // Implement chat streaming logic
      chunkHandler('Response...', { tokensUsed: 2 });
    });
  }
}
```

## Integration with Drupal

### Setting Up Node.js Bridge

```typescript
import [createDrupalContextBridge] from '@bluefly/mcp';

// Create and configure the bridge
const drupalBridge = createDrupalContextBridge({
  apiUrl: 'https://example.com/api',
  apiKey: 'your-api-key',
  timeout: 30000,
});
```

### Using in Drupal

```php
<?php

// In Drupal PHP code
$streamingService = \Drupal::service('drupal_native_ai.streaming_service');

// Create a streaming response
$response = $streamingService->createStreamingResponse(function($chunkHandler) {
  // Send chunks
  $chunkHandler->sendChunk('Hello from Drupal!', ['tokensUsed' => 4]);
  
  // Add a delay
  sleep(1);
  
  // Send another chunk
  $chunkHandler->sendChunk(' More content...', ['tokensUsed' => 2]);
});

// Return the response
return $response;
```

## Error Handling

### Handling Streaming Errors

```typescript
app.get('/error-example', (req, res) => {
  return streamingService.createStreamingResponse((chunkHandler) => {
    try {
      // Attempt something that might fail
      if (Math.random() > 0.5) {
        throw new Error('Random error occurred');
      }
      
      chunkHandler('Success!', { tokensUsed: 1 });
    } catch (error) {
      // Error will be automatically handled by the streaming service
      throw error;
    }
  });
});
```

### Resilient Function Execution

```typescript
// Resilient function execution
async function executeWithRetry(functionName, args, maxRetries = 3) {
  for (let attempt = 1; attempt <= maxRetries; attempt++) {
    try {
      const result = await functionManager.executeFunction(functionName, args);
      if (result.status === 'success') {
        return result;
      }
      
      if (attempt < maxRetries) {
        await new Promise(resolve => setTimeout(resolve, 1000 * attempt));
      }
    } catch (error) {
      if (attempt === maxRetries) {
        throw error;
      }
      
      await new Promise(resolve => setTimeout(resolve, 1000 * attempt));
    }
  }
  
  throw new Error(`Failed to execute function [functionName] after [maxRetries] attempts`);
}
```

## Performance Considerations

### Vector Operations

- Use batch operations for inserting multiple vectors
- Set appropriate chunk sizes for document indexing
- Adjust limits and thresholds for vector searches based on use case
- Consider using a dedicated vector database for production

### Streaming

- Set appropriate timeouts for streaming operations
- Monitor token usage and implement rate limiting
- Use connection pooling for external API calls
- Consider implementing backpressure handling for large responses

### Function Calling

- Cache repetitive function calls
- Implement rate limiting for expensive functions
- Use function categories to organize and selectively expose functions

## Security Considerations

### Function Calling

- Validate all function inputs
- Implement function permissions/scopes
- Never execute arbitrary code in function handlers
- Sanitize function outputs before displaying

### Vector Operations

- Implement access controls for vector collections
- Validate embedding inputs for potential injection
- Be cautious with metadata storage (avoid storing sensitive data)

### Streaming

- Implement request validation
- Set appropriate token usage thresholds
- Monitor for abuse patterns
- Sanitize outputs to prevent XSS