# LLM-MCP Docker Deployment Guide

This guide explains how to deploy LLM-MCP using Docker containers.

## Quick Start

### Using GitLab CI/CD

LLM-MCP uses BFCIComponents for all CI/CD operations. The pipeline automatically:

1. Builds multi-architecture Docker images (amd64, arm64)
2. Runs security scans
3. Pushes to GitLab Container Registry
4. Deploys to staging/production

The entire CI/CD configuration is just:

```yaml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/platforms/docker/llm-mcp-docker-deploy@main
    inputs:
      service_name: "llm-mcp"
      # ... other configuration
```

### Manual Deployment

1. **Pull the Docker image:**
   ```bash
   docker pull registry.gitlab.bluefly.io/llm/llm-mcp:latest
   ```

2. **Run with Docker:**
   ```bash
   docker run -d \
     --name llm-mcp \
     -p 8080:8080 \
     -e NODE_ENV=production \
     registry.gitlab.bluefly.io/llm/llm-mcp:latest
   ```

3. **Or use Docker Compose:**
   ```bash
   docker-compose -f docker-compose.production.yml up -d
   ```

## Deployment Options

### Option 1: Standalone Docker Container

Use the deployment script:

```bash
./scripts/deploy-docker.sh \
  --image registry.gitlab.bluefly.io/llm/llm-mcp \
  --tag latest \
  --port 8080
```

### Option 2: Docker Compose Stack

The production docker-compose includes:
- LLM-MCP server
- Redis for caching
- Qdrant for vector storage
- Optional Nginx proxy
- Optional Prometheus/Grafana monitoring

```bash
# Basic deployment
docker-compose -f docker-compose.production.yml up -d

# With monitoring
docker-compose -f docker-compose.production.yml --profile monitoring up -d

# With proxy
docker-compose -f docker-compose.production.yml --profile with-proxy up -d
```

### Option 3: Kubernetes/Helm

```bash
# Using kubectl
kubectl apply -f k8s/

# Using Helm
helm install llm-mcp ./llm-mcp-chart/
```

## Configuration

### Environment Variables

Create a `.env.production` file:

```env
# Node Configuration
NODE_ENV=production
LOG_LEVEL=info

# MCP Configuration
MCP_SERVER_PORT=8080
MCP_VERSION=2025

# Redis Configuration
REDIS_HOST=redis
REDIS_PORT=6379
REDIS_PASSWORD=your-redis-password

# Qdrant Configuration
QDRANT_HOST=qdrant
QDRANT_PORT=6333

# Service URLs
BFAPI_URL=http://bfapi:3001
BFLLM_URL=http://bfllm:3002

# Security
API_KEY=your-api-key
JWT_SECRET=your-jwt-secret
```

### Docker Registry Authentication

For private registries:

```bash
# GitLab Registry
docker login registry.gitlab.bluefly.io

# AWS ECR
aws ecr get-login-password | docker login --username AWS --password-stdin $ECR_URL

# Docker Hub
docker login
```

## Health Checks

LLM-MCP provides comprehensive health endpoints:

- `/health` - Basic health check
- `/health/liveness` - Kubernetes liveness probe
- `/health/readiness` - Kubernetes readiness probe

Test health:
```bash
curl http://localhost:8080/health
```

## Monitoring

### Prometheus Metrics

LLM-MCP exposes metrics at `/metrics`:

```yaml
# prometheus.yml
scrape_configs:
  - job_name: 'llm-mcp'
    static_configs:
      - targets: ['llm-mcp:8080']
```

### Logging

Container logs:
```bash
docker logs llm-mcp

# Follow logs
docker logs -f llm-mcp

# Last 100 lines
docker logs --tail 100 llm-mcp
```

## Troubleshooting

### Container Won't Start

1. Check logs:
   ```bash
   docker logs llm-mcp
   ```

2. Verify image:
   ```bash
   docker images | grep llm-mcp
   ```

3. Check resources:
   ```bash
   docker stats llm-mcp
   ```

### Health Check Failures

1. Test endpoint directly:
   ```bash
   docker exec llm-mcp wget -O- http://localhost:8080/health
   ```

2. Check dependencies:
   ```bash
   docker-compose ps
   ```

### Network Issues

1. Verify network:
   ```bash
   docker network ls
   docker network inspect bluefly-network
   ```

2. Test connectivity:
   ```bash
   docker exec llm-mcp ping redis
   ```

## Security Best Practices

1. **Use specific image tags** (not `latest` in production)
2. **Run as non-root user** (already configured in Dockerfile)
3. **Use secrets management** for sensitive data
4. **Enable security scanning** in CI/CD
5. **Use read-only filesystem** where possible:
   ```bash
   docker run --read-only --tmpfs /tmp llm-mcp
   ```

## Backup and Recovery

### Backup Volumes

```bash
# Backup Qdrant data
docker run --rm \
  -v llm-mcp_qdrant-data:/data \
  -v $(pwd):/backup \
  alpine tar czf /backup/qdrant-backup.tar.gz -C /data .

# Backup Redis data
docker exec llm-mcp-redis redis-cli BGSAVE
```

### Restore Volumes

```bash
# Restore Qdrant data
docker run --rm \
  -v llm-mcp_qdrant-data:/data \
  -v $(pwd):/backup \
  alpine tar xzf /backup/qdrant-backup.tar.gz -C /data
```

## Updates and Rollbacks

### Update to New Version

```bash
# Pull new image
docker pull registry.gitlab.bluefly.io/llm/llm-mcp:v1.2.0

# Stop old container
docker stop llm-mcp

# Start new container
docker run -d --name llm-mcp-new \
  -p 8080:8080 \
  registry.gitlab.bluefly.io/llm/llm-mcp:v1.2.0

# Verify health
curl http://localhost:8080/health

# Remove old container
docker rm llm-mcp
docker rename llm-mcp-new llm-mcp
```

### Rollback

```bash
# Stop current container
docker stop llm-mcp

# Start previous version
docker run -d --name llm-mcp \
  -p 8080:8080 \
  registry.gitlab.bluefly.io/llm/llm-mcp:v1.1.0
```

## Performance Tuning

### Docker Resources

```bash
# Set resource limits
docker run -d \
  --name llm-mcp \
  --memory="4g" \
  --cpus="2" \
  -p 8080:8080 \
  registry.gitlab.bluefly.io/llm/llm-mcp:latest
```

### Node.js Optimization

```bash
# Set Node.js memory
docker run -d \
  --name llm-mcp \
  -e NODE_OPTIONS="--max-old-space-size=3072" \
  -p 8080:8080 \
  registry.gitlab.bluefly.io/llm/llm-mcp:latest
```

## Support

For issues or questions:
1. Check container logs
2. Review health endpoint response
3. Consult the [main documentation](./README.md)
4. Open an issue in GitLab