# LangChain Integration Documentation

## Overview

This document describes the comprehensive LangChain integration between Drupal and the llm-mcp Python bridge service, enabling enterprise-grade AI capabilities through a secure, scalable architecture.

## Architecture

```
┌─────────────────────┐     ┌─────────────────────┐     ┌─────────────────────┐
│   Drupal Frontend   │────▶│   llm-mcp Service     │────▶│  Python Bridge      │
│  (ai_provider_      │     │  (TypeScript/Node)  │     │  (FastAPI Server)   │
│    langchain)       │◀────│                     │◀────│                     │
└─────────────────────┘     └─────────────────────┘     └─────────────────────┘
         │                           │                            │
         │                           │                            │
         ▼                           ▼                            ▼
┌─────────────────────┐     ┌─────────────────────┐     ┌─────────────────────┐
│   MySQL Database    │     │   Redis Cache       │     │   LangChain Core    │
│  (Chain Storage)    │     │  (Performance)      │     │  (Python Ecosystem) │
└─────────────────────┘     └─────────────────────┘     └─────────────────────┘
```

## Components

### 1. Drupal Module (`ai_provider_langchain`)

**Location**: `/web/modules/custom/ai_provider_langchain/`

**Key Features**:
- Admin UI for chain management
- Database schema for persistent storage
- Security audit and compliance validation
- Real-time execution monitoring

**Controllers**:
- `ChainController`: Chain management interface
- `AgentController`: Agent configuration and deployment
- `VectorController`: Vector store operations
- `AnalyticsController`: Performance and usage analytics

### 2. llm-mcp TypeScript Service

**Location**: `/Users/flux423/Sites/LLM/_CommonNPM/llm-mcp/`

**Python Bridge Service**: `src/services/python-bridge/python-bridge.service.ts`

**Features**:
- Subprocess management for Python FastAPI server
- HTTP/WebSocket communication
- Health monitoring and auto-restart
- Request/response handling with error recovery

### 3. Python Bridge Server

**Location**: `/Users/flux423/Sites/LLM/_CommonNPM/llm-mcp/src/python/bridge_server.py`

**Components**:
- `LangChainExecutor`: Handles all LangChain operations
- `ModelManager`: LLM model lifecycle management
- `VectorStoreManager`: Embedding and vector operations
- `ChainRegistry`: Dynamic chain registration and execution

**Supported Operations**:
- Chat completions with streaming
- Chain execution with memory
- Agent orchestration
- Vector similarity search
- Tool calling and function execution

## Installation

### Prerequisites

1. **System Requirements**:
   - PHP 8.1+
   - Node.js 18+
   - Python 3.9+
   - MySQL 5.7+
   - Redis (optional, for caching)

2. **Python Dependencies**:
   ```bash
   pip install langchain langchain-community langchain-openai \
               langchain-anthropic fastapi uvicorn websockets \
               pydantic python-dotenv aiohttp numpy faiss-cpu
   ```

### Setup Steps

1. **Install Drupal Dependencies**:
   ```bash
   ddev composer require drupal/search_api drupal/search_api_solr \
                        drupal/search_api_solr_dense_vector
   ```

2. **Enable Required Modules**:
   ```bash
   ddev drush en -y search_api search_api_solr \
                    search_api_solr_dense_vector \
                    ai ai_provider_langchain \
                    mcp_client_extras
   ```

3. **Configure Environment**:
   ```bash
   # In .env file
   OPENAI_API_KEY=your-key-here
   ANTHROPIC_API_KEY=your-key-here
   PYTHON_BRIDGE_PORT=8001
   PYTHON_BRIDGE_HOST=localhost
   ```

4. **Start Services**:
   ```bash
   # Terminal 1: Start llm-mcp service
   cd /Users/flux423/Sites/LLM/_CommonNPM/llm-mcp
   npm run dev
   
   # Terminal 2: Python bridge starts automatically via llm-mcp
   ```

## Usage

### Admin Interface

1. **Access LangChain Admin**:
   - Navigate to `/admin/ai/langchain/chains`
   - Create and manage chains
   - Monitor execution logs

2. **Create a Chain**:
   ```php
   // Example: Creating a simple Q&A chain
   $chain = [
     'name' => 'QA Chain',
     'type' => 'qa_retrieval',
     'config' => [
       'model' => 'gpt-4',
       'temperature' => 0.7,
       'vector_store' => 'faiss',
       'k' => 4
     ]
   ];
   ```

3. **Execute Operations**:
   ```php
   // Via Drupal service
   $langchain = \Drupal::service('ai_provider_langchain.chain_builder');
   $result = $langchain->executeChain('qa-chain-id', [
     'input' => 'What is the capital of France?'
   ]);
   ```

### API Endpoints

**Base URL**: `http://localhost:8001`

1. **Execute Chain**:
   ```bash
   POST /execute
   {
     "operation": "chain",
     "chain_id": "qa-chain",
     "inputs": {"question": "What is AI?"},
     "config": {"temperature": 0.5}
   }
   ```

2. **Chat Completion**:
   ```bash
   POST /execute
   {
     "operation": "chat",
     "messages": ["role": "user", "content": "Hello"],
     "config": {"model": "gpt-4", "stream": true}
   }
   ```

3. **Vector Search**:
   ```bash
   POST /execute
   {
     "operation": "vector",
     "action": "search",
     "query": "machine learning",
     "k": 5
   }
   ```

## Security Considerations

1. **API Key Management**:
   - Store keys in environment variables
   - Use Drupal's Key module for production
   - Rotate keys regularly

2. **Access Control**:
   - Permission-based access to chains
   - Audit logging for all operations
   - Rate limiting on API endpoints

3. **Data Privacy**:
   - Encrypted storage for sensitive data
   - GDPR compliance features
   - Data retention policies

## Monitoring & Debugging

1. **Logs**:
   - Drupal: `/admin/reports/dblog`
   - llm-mcp: Console output
   - Python: `/tmp/langchain_bridge.log`

2. **Health Checks**:
   ```bash
   # Check Python bridge
   curl http://localhost:8001/health
   
   # Check llm-mcp service
   curl http://localhost:3030/health
   ```

3. **Performance Metrics**:
   - Available at `/admin/ai/langchain/analytics`
   - Redis cache statistics
   - Token usage tracking

## Troubleshooting

### Common Issues

1. **Module Installation Fails**:
   ```bash
   # Clear cache and retry
   ddev drush cr
   ddev drush en -y ai_provider_langchain
   ```

2. **Python Bridge Not Starting**:
   ```bash
   # Check Python dependencies
   pip list | grep langchain
   
   # Manual start for debugging
   cd /path/to/llm-mcp/src/python
   python bridge_server.py
   ```

3. **Connection Errors**:
   - Verify ports are not in use
   - Check firewall settings
   - Ensure services are running

## Advanced Configuration

### Custom Chain Types

1. **Register Custom Chain**:
   ```python
   # In bridge_server.py
   chain_registry.register_chain(
       "custom_chain",
       CustomChainClass,
       config_schema=CustomConfigSchema
   )
   ```

2. **Drupal Integration**:
   ```php
   // In custom module
   function mymodule_langchain_chain_types_alter(&$types) {
     $types['custom_chain'] = [
       'label' => t('Custom Chain'),
       'class' => 'Drupal\mymodule\Chain\CustomChain',
     ];
   }
   ```

### Performance Optimization

1. **Caching Strategy**:
   - Enable Redis for vector cache
   - Configure TTL appropriately
   - Use batch operations

2. **Scaling**:
   - Run multiple Python workers
   - Use load balancer for API
   - Implement queue for async operations

## Contributing

1. **Code Standards**:
   - Follow Drupal coding standards
   - Use TypeScript strict mode
   - Python: Follow PEP 8

2. **Testing**:
   - PHPUnit for Drupal
   - Jest for TypeScript
   - Pytest for Python

3. **Documentation**:
   - Update this guide for new features
   - Add inline code comments
   - Create user tutorials

## Support

- **Issues**: GitHub repository
- **Documentation**: `/docs` directory
- **Community**: Drupal Slack #ai channel

---

Last Updated: January 2025
Version: 1.0.0