# LangGraph Tool Calling Guide

## Overview

This guide demonstrates how to build and use agents with tool calling capabilities using LangGraph in the LLM-MCP project. LangGraph is a library for building stateful, multi-actor applications with LLMs, and it provides excellent support for tool calling patterns.

## Architecture

### Key Components

1. **StateGraph**: The core graph structure that manages agent state and execution flow
2. **Tools**: Functions that the agent can call to perform specific tasks
3. **Nodes**: Processing steps in the graph (analyze, call tools, process results, generate response)
4. **Edges**: Define the flow between nodes, including conditional routing

### State Management

The agent maintains state across tool calls with the following structure:

```typescript
interface AgentState {
  messages: BaseMessage[];        // Conversation history
  currentTask: string;           // Current task being processed
  toolCalls: ToolCallRecord[];   // Record of all tool calls made
  finalResult?: any;             // Final result of the execution
  error?: string;                // Error tracking
}
```

## Tool Types

### 1. Dynamic Tools
Simple tools with string input/output:

```typescript
const weatherTool = new DynamicTool({
  name: 'get_weather',
  description: 'Get the current weather for a location',
  func: async (location: string) => {
    // Implementation
    return JSON.stringify(weatherData);
  },
});
```

### 2. Structured Tools
Tools with schema validation using Zod:

```typescript
const calculatorTool = new StructuredTool({
  name: 'calculator',
  description: 'Perform mathematical calculations',
  schema: z.object({
    operation: z.enum(['add', 'subtract', 'multiply', 'divide']),
    a: z.number(),
    b: z.number(),
  }),
  func: async ({ operation, a, b }) => [// Implementation with type safety],
});
```

## Graph Workflow

The LangGraph workflow follows this pattern:

```
Start → Analyze Request → Call Tool(s) → Process Results → Generate Response → End
                ↓              ↓              ↓                    ↓
            Error Handler ←────┴──────────────┴────────────────────┘
```

### Node Descriptions

1. **analyze_request**: Examines the user request and determines which tools to use
2. **call_tool**: Executes the selected tools with appropriate parameters
3. **process_result**: Analyzes tool outputs and decides if more tools are needed
4. **generate_response**: Creates the final response based on all tool results
5. **handle_error**: Gracefully handles any errors in the workflow

## Usage Examples

### Basic Usage

```typescript
import [ToolCallingExampleAgent] from './agents/tool-calling-example.agent';

const agent = new ToolCallingExampleAgent();

// Execute a simple task
const result = await agent.execute('What is the weather in New York?');
console.log(result);
```

### Streaming Responses

```typescript
// Stream execution for real-time updates
for await (const event of agent.stream('Calculate 25 * 4 and check weather in Tokyo')) {
  console.log('Event:', event);
}
```

### Adding Custom Tools

```typescript
// Create a custom tool
const customTool = new StructuredTool({
  name: 'analyze_sentiment',
  description: 'Analyze sentiment of text',
  schema: z.object({
    text: z.string(),
    language: z.string().optional(),
  }),
  func: async ({ text, language }) => {
    // Custom implementation
    return JSON.stringify({
      sentiment: 'positive',
      confidence: 0.85,
      language: language || 'en',
    });
  },
});

// Add to agent
agent.addTool(customTool);
```

## Built-in Tools

The example agent includes these tools:

1. **get_weather**: Retrieves weather information for a location
2. **calculator**: Performs mathematical operations
3. **query_database**: Executes database queries
4. **web_search**: Searches the web for information
5. **send_email**: Sends email messages

## Advanced Patterns

### Tool Chaining
Tools can be chained together in complex workflows:

```typescript
const result = await agent.execute(`
  1. Search for "best restaurants in Paris"
  2. For each restaurant, check the weather at its location
  3. Calculate the average price in USD
  4. Email the results to travel@example.com
`);
```

### Conditional Tool Usage
The agent can make decisions based on tool results:

```typescript
const result = await agent.execute(`
  Check the weather in London.
  If it's raining, search for indoor activities.
  If it's sunny, search for outdoor activities.
  Then calculate the budget for recommended activities.
`);
```

### Error Recovery
The agent handles tool failures gracefully:

```typescript
// Even if a tool fails, the agent continues with available information
const result = await agent.execute('Search for "error" and calculate 10/0');
// The agent will handle the search error and division by zero gracefully
```

## Best Practices

### 1. Tool Design
- Keep tools focused on a single responsibility
- Use structured tools for complex inputs
- Return JSON for structured data
- Include error handling in tool implementations

### 2. Error Handling
- Tools should throw meaningful errors
- The agent will catch and handle tool errors
- Always provide fallback behavior

### 3. Performance
- Tools should be async for non-blocking execution
- Use streaming for long-running tasks
- Implement timeouts for external API calls

### 4. Testing
- Test tools independently
- Mock external dependencies
- Verify error handling paths
- Test tool combinations

## Configuration

### Environment Variables
```bash
# For OpenAI models
OPENAI_API_KEY=your-api-key

# For Anthropic models
ANTHROPIC_API_KEY=your-api-key

# For Hugging Face (free tier)
HUGGINGFACEHUB_API_KEY=your-api-key
```

### Model Selection
The agent automatically selects the best available model:
1. Prefers GPT-4 if available
2. Falls back to Claude Sonnet
3. Uses free Hugging Face models as last resort

## Integration with LLM-MCP

The tool-calling agent integrates seamlessly with the existing LLM-MCP infrastructure:

1. **MCP Protocol**: Tools can be exposed as MCP resources
2. **Vector Store**: Tool results can be stored and searched
3. **Multi-Agent**: Multiple tool-calling agents can collaborate
4. **Monitoring**: Tool usage is tracked and monitored

## Troubleshooting

### Common Issues

1. **No API Keys**: The agent will use mock responses if no API keys are configured
2. **Tool Not Found**: Ensure tools are added before execution
3. **Graph Not Initialized**: The graph is built automatically on instantiation
4. **Streaming Errors**: Check for async iterator support in your environment

### Debug Mode

Enable verbose logging:
```typescript
const agent = new ToolCallingExampleAgent();
// The agent already has verbose mode enabled by default
```

## Further Reading

- [LangGraph Documentation](https://langchain-ai.github.io/langgraph/)
- [LangChain Tools Guide](https://python.langchain.com/docs/modules/agents/tools/)
- [LLM-MCP Architecture](./ARCHITECTURE.md)
- [MCP Protocol Integration](./MCP_INTEGRATION.md)