# OpenAPI Validation in LLM-MCP

This document describes the OpenAPI validation system implemented in the LLM-MCP server. The validation system ensures that all API requests and responses conform to the standardized OpenAPI specification.

## Overview

The OpenAPI validation system includes:

1. **Standardized OpenAPI Specification**: A comprehensive OpenAPI 3.1 specification that defines all LLM-MCP endpoints, request/response schemas, and error formats.

2. **Request Validation**: Validates incoming requests against the OpenAPI specification, ensuring that path parameters, query parameters, headers, and request bodies conform to the defined schemas.

3. **Response Validation**: Validates outgoing responses against the OpenAPI specification, ensuring that response bodies conform to the defined schemas.

4. **Validation Middleware**: Express middleware that integrates with the LLM-MCP server to validate requests and responses at runtime.

5. **Validation Utilities**: Tools for checking OpenAPI compliance during development and testing.

## Configuration

OpenAPI validation can be configured using environment variables or the configuration object:

| Environment Variable | Configuration Key | Description | Default |
|---------------------|-------------------|-------------|---------|
| `OPENAPI_PATH` | `openapi.path` | Path to the OpenAPI specification file | `openapi-standardized.yaml` |
| `OPENAPI_VALIDATION` | `openapi.validation` | Enable/disable OpenAPI validation | `true` |
| `OPENAPI_STRICT_MODE` | `openapi.strictMode` | Enable strict mode for validation | `false` |
| `DISABLE_OPENAPI_VALIDATION` | `openapi.disableValidation` | Disable validation (overrides other settings) | `false` (but defaults to `true` in production) |

## Usage

### Validation Middleware

The OpenAPI validation middleware is automatically initialized in the LLM-MCP server's `setupMiddleware` method:

```typescript
// Initialize OpenAPI validation middleware
if (config.openapi?.validation !== false) [try {
    const { initializeOpenApiValidation] = await import('../middleware/openapi-validation.js');
    await initializeOpenApiValidation(this.app);
    logger.info('OpenAPI validation middleware initialized');
  } catch (error) {
    logger.warn('Failed to initialize OpenAPI validation middleware:', error);
    // Non-blocking error - server will continue without OpenAPI validation
  }
}
```

### Validating OpenAPI Compliance

To check if your OpenAPI specification is valid and compliant, use the validation script:

```bash
npm run validate:openapi
```

This script will:
1. Validate the OpenAPI specification for correctness
2. Test API endpoints against the specification
3. Generate a validation report

### Programmatic Validation

You can also validate requests and responses programmatically:

```typescript
import [checkOpenApiCompliance] from '../middleware/openapi-validation.js';

// Check if a request/response pair complies with the OpenAPI spec
const { valid, errors } = await checkOpenApiCompliance(req, res, {
  specPath: 'path/to/openapi.yaml',
  strictMode: true
});

if (!valid) {
  console.error('OpenAPI compliance errors:', errors);
}
```

## Development Guidelines

When developing new endpoints or modifying existing ones:

1. **Update the OpenAPI Specification First**: Always update `openapi-standardized.yaml` to reflect the changes.

2. **Test Specification Validity**: Run `npm run validate:openapi` to ensure the specification is valid.

3. **Use the Specification for Development**: Use the OpenAPI specification as a reference when implementing endpoints.

4. **Ensure Compliance**: Verify that your implementation complies with the specification using the validation tools.

## Error Handling

When validation fails:

- **Request Validation**: Returns a 400 Bad Request response with details about the validation errors.
- **Response Validation**: In development mode, returns a 500 Internal Server Error response with details about the validation errors. In production mode, logs the error but still returns the original response to avoid blocking operations.

## Best Practices

1. **Maintain Clear Documentation**: Keep the OpenAPI specification up-to-date with clear descriptions and examples.

2. **Follow Schema Standards**: Use consistent naming and schema patterns across the API.

3. **Test Validation During Development**: Regularly validate your API against the specification during development.

4. **Version the API**: Use the OpenAPI `info.version` property to track API versions and maintain backward compatibility.

5. **Generate Client SDKs**: Use the OpenAPI specification to generate client SDKs for different programming languages.