# llm-mcp - Model Context Protocol Server

**Production-grade MCP server** orchestrating tool and resource access across the Enterprise AI Platform. Enables standardized AI assistant interactions with models, APIs, and Drupal workflows.

## Architecture Integration

llm-mcp serves as a central hub in the AI platform ecosystem with bidirectional integration:

```
Client Applications -> bfapi -> [bfllm, llm-mcp, Drupal]
bfllm <-> llm-mcp (bidirectional)
llm-mcp <-> Drupal (llm_eca modules)
llmcli -> [bfapi, bfllm, llm-mcp, BFCIComponents]
```

### Integration Points

- **bfllm to llm-mcp**: 
  - bfllm registers plugins with llm-mcp's registry
  - Exposes model capabilities via MCP protocol
  - Provides standardized model access

- **llm-mcp to Drupal**:
  - Connects to llm_eca and llm_eca_claude modules
  - Uses Event-Condition-Action pattern for content events
  - Follows OpenAPI specifications for integration

<!-- 
INTEGRATION INSTRUCTIONS:
1. All tools must be registered following the MCP protocol standards
2. Tool schemas must be fully typed with proper input/output definitions
3. When exposing models from bfllm, ensure all capabilities are properly registered
4. For Drupal integration, implement the ECA pattern triggers and listeners
5. Authentication between components must use JWT with proper token validation
-->

## Features

- **MCP Tool Integration**: Exposes functionality from LLM Platform components as MCP tools
- **Resource Access**: Provides context resources from connected systems
- **AI Streaming**: Real-time AI response streaming with metrics tracking
- **Vector Operations**: Semantic search and embedding management
- **Cross-Component Integration**: Enables workflows that span multiple LLM Platform systems
- **TypeScript Implementation**: Built with TypeScript for type safety and IDE support

## Getting Started

### Prerequisites

- Node.js 14+
- Access to bfapi, bfllm and/or Drupal installations

### Installation

```bash
# Clone the repository
git clone https://github.com/LLM Platform/llm-mcp.git
cd llm-mcp

# Install dependencies
npm install

# Build the project
npm run build
```

### Configuration

Create a `.env` file in the root directory with the following variables:

```
# Server configuration
PORT=3000

# API integrations
bfapi_URL=http://localhost:3001
bfllm_URL=http://localhost:3002
DRUPAL_URL=http://localhost/drupal

# Authentication
DRUPAL_CLIENT_ID=your_drupal_client_id
DRUPAL_CLIENT_SECRET=your_drupal_client_secret

# Logging
LOG_LEVEL=info
```

### Usage

Start the server:

```bash
npm start
```

For development with auto-reloading:

```bash
npm run dev
```

## Tools and Resources

### bfapi Integration

Provides MCP tools generated from the bfapi OpenAPI specification. These tools allow AI assistants to:

- Query API endpoints
- Perform CRUD operations on resources
- Access documentation and metadata

### bfllm Integration

Enables interaction with LLM Platform's LLM capabilities:

- Run inference on fine-tuned models
- Access model metadata and capabilities
- Utilize specialized models for different tasks

### LLM Platform_insights Integration

Connects to the Drupal LLM Platform_insights module to provide:

- Content analytics and statistics
- Popular and recent content
- Content search capabilities

## Tool Registration Examples

### Basic Tool Registration

Register a tool with standardized schema and execution logic:

```typescript
import [registerTool] from '@ac/mcp-core';

registerTool({
  name: 'content_search',
  description: 'Search for content across the Acme AI Platform',
  version: '1.0.0',
  
  input_schema: {
    type: 'object',
    properties: {
      query: {
        type: 'string',
        description: 'The search query'
      },
      content_type: {
        type: 'string',
        enum: ['article', 'page', 'product'],
        description: 'Filter by content type'
      },
      limit: {
        type: 'number',
        description: 'Maximum number of results',
        default: 10
      }
    },
    required: ['query']
  },
  
  output_schema: {
    type: 'object',
    properties: {
      results: {
        type: 'array',
        items: {
          type: 'object',
          properties: {
            id: { type: 'string' },
            title: { type: 'string' },
            url: { type: 'string' },
            summary: { type: 'string' },
            content_type: { type: 'string' }
          }
        }
      },
      total_count: { type: 'number' }
    }
  },
  
  execute: async (input) => {
    // Implementation of search functionality
    // For example, making a request to bfapi or Drupal
    
    // Sample implementation:
    const response = await fetch(`[process.env.DRUPAL_URL]/api/content-search`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'Authorization': `Bearer [await getDrupalToken()]`
      },
      body: JSON.stringify(input)
    });
    
    const data = await response.json();
    
    return {
      results: data.items.map(item => ({
        id: item.id,
        title: item.title,
        url: item.url,
        summary: item.summary,
        content_type: item.type
      })),
      total_count: data.total
    };
  }
});
```

### bfllm Model Tool Registration

Register a model from bfllm as an MCP tool:

```typescript
import [registerTool] from '@ac/mcp-core';
import { getModelInfo, runInference } from '@ac/llm-client';

// Register a model as an MCP tool
const registerModelTool = async (modelId) => {
  // Fetch model information from bfllm
  const modelInfo = await getModelInfo(modelId);
  
  // Register the model as a tool
  registerTool({
    name: `model.[modelId]`,
    description: modelInfo.description || `Run inference on [modelId] model`,
    version: modelInfo.version || '1.0.0',
    
    input_schema: {
      type: 'object',
      properties: {
        prompt: {
          type: 'string',
          description: 'The input prompt for the model'
        },
        parameters: {
          type: 'object',
          properties: {
            max_tokens: { 
              type: 'number', 
              description: 'Maximum tokens to generate',
              default: 100
            },
            temperature: { 
              type: 'number', 
              description: 'Sampling temperature',
              default: 0.7
            },
            top_p: { 
              type: 'number', 
              description: 'Nucleus sampling parameter',
              default: 1.0
            },
            frequency_penalty: { 
              type: 'number', 
              default: 0.0 
            },
            presence_penalty: { 
              type: 'number', 
              default: 0.0 
            }
          }
        }
      },
      required: ['prompt']
    },
    
    output_schema: {
      type: 'object',
      properties: {
        completion: { type: 'string' },
        usage: {
          type: 'object',
          properties: {
            prompt_tokens: { type: 'number' },
            completion_tokens: { type: 'number' },
            total_tokens: { type: 'number' }
          }
        }
      }
    },
    
    execute: async (input) => {
      // Use the bfllm client to run inference
      const result = await runInference(modelId, input.prompt, input.parameters);
      
      return {
        completion: result.text,
        usage: result.usage
      };
    }
  });
};

// Example usage: Register all models from bfllm
const registerAllModels = async () => {
  const models = await getAllModels();
  
  for (const model of models) {
    await registerModelTool(model.id);
  }
  
  console.log(`Registered [models.length] models as MCP tools`);
};
```

## Development

### Adding New Tools

Create a new tool implementation in `tools/your-tool.js`:

```javascript
module.exports = {
  name: 'your_tool_name',
  description: 'Description of what your tool does',
  
  input_schema: {
    type: 'object',
    properties: {
      // Define your input parameters here
    },
    required: ['param1']
  },
  
  execute: async (input) => {
    // Implement your tool's functionality
    return {
      // Return your result
    };
  }
};
```

### Adding New Resources

Create a new resource implementation in `resources/your-resource.js`:

```javascript
module.exports = {
  uri: 'your/resource/path',
  description: 'Description of what your resource provides',
  
  fetch: async () => {
    // Implement your resource's retrieval logic
    return {
      // Return your resource data
    };
  }
};
```

## Using with llmcli

llmcli provides commands for managing MCP servers:

```bash
# Initialize from OpenAPI spec
llmcli mcp init openapi --name bfapi-mcp --output ./llm-mcp --port 3001

# Add tools for LLM Platform_insights
llmcli mcp tool:create LLM Platform_insights --server ./llm-mcp

# Start the server
llmcli mcp start ./llm-mcp
```

## OpenAPI Registry Publishing

After every successful CI run on main/master, the canonical OpenAPI spec (`openapi.yaml`) is automatically published to the central Apicurio registry.

- **Registry URL:** `https://apicurio.example.com/apis/projects/<project-name>/versions/<branch-or-tag>`
- **How to browse:** Visit the Apicurio UI or use the API to fetch the latest spec for this project.
- **How to use:**
  - For SDK generation, docs, or federation, always use the published spec from the registry.
  - Example fetch:
    ```sh
    curl -H "Authorization: Bearer <token>" \
      https://apicurio.example.com/apis/projects/LLM-MCP/versions/main
    ```
- **CI Job:** See `.gitlab-ci.yml` for the `openapi-publish` job that automates this process.

> The registry ensures all clients and services use the latest, validated API contract for maximum interoperability.

## Documentation Structure

- **[Documentation Home](docs/index.md)** - Complete documentation index
- **[API Documentation](docs/api/)** - API reference and endpoints
- **[Architecture](docs/architecture/)** - System design and configuration
- **[Deployment Guides](docs/deployment/)** - Deployment instructions
- **[Integration Guides](docs/integration/)** - Integration with other services
- **[Usage Examples](docs/examples/)** - Example code and scenarios
- **[Setup & Configuration](docs/guides/)** - Setup and configuration guides
- **[Troubleshooting](docs/troubleshooting/)** - Troubleshooting guides
- **[Changelog](docs/reference/changelog.md)** - Release notes

## Workflow Engine Standardization

See [WORKFLOW_ENGINE.md](WORKFLOW_ENGINE.md) for workflow engine rationale, migration plan, and example code.

## Observability & Monitoring

See [OBSERVABILITY.md](OBSERVABILITY.md) for observability quickstart, best practices, and example code.

## Changelog

See [CHANGELOG.md](CHANGELOG.md) for the full changelog.

## License

MIT