# LLM-MCP Security Hardening Guide

This guide provides comprehensive security hardening recommendations for deploying LLM-MCP in production environments.

## Table of Contents

1. [Container Security](#container-security)
2. [Network Security](#network-security)
3. [Authentication & Authorization](#authentication--authorization)
4. [Data Protection](#data-protection)
5. [Secrets Management](#secrets-management)
6. [Monitoring & Auditing](#monitoring--auditing)
7. [Security Checklist](#security-checklist)

## Container Security

### 1. Non-Root User

The LLM-MCP Docker image runs as a non-root user (UID 1001):

```dockerfile
USER 1001:1001
```

### 2. Read-Only Filesystem

Deploy containers with read-only root filesystem:

```yaml
# docker-compose.yml
services:
  llm-mcp:
    read_only: true
    tmpfs:
      - /tmp
      - /app/logs
```

```yaml
# Kubernetes
securityContext:
  readOnlyRootFilesystem: true
  runAsNonRoot: true
  runAsUser: 1001
  allowPrivilegeEscalation: false
  capabilities:
    drop:
      - ALL
```

### 3. Image Scanning

Enable automated security scanning in CI/CD:

```yaml
security:scan:
  stage: security
  script:
    - trivy image --severity HIGH,CRITICAL --exit-code 1 $IMAGE_NAME
```

### 4. Distroless Images

Consider using distroless base images:

```dockerfile
FROM gcr.io/distroless/nodejs20-debian12
COPY --from=builder /app/dist /app
COPY --from=builder /app/node_modules /app/node_modules
```

## Network Security

### 1. TLS/SSL Configuration

Always use TLS for external communications:

```nginx
# nginx.conf
server {
    listen 443 ssl http2;
    ssl_protocols TLSv1.2 TLSv1.3;
    ssl_ciphers ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256;
    ssl_prefer_server_ciphers off;
    ssl_session_cache shared:SSL:10m;
    ssl_session_timeout 10m;
    ssl_stapling on;
    ssl_stapling_verify on;
}
```

### 2. Network Policies

Implement Kubernetes NetworkPolicies:

```yaml
apiVersion: networking.k8s.io/v1
kind: NetworkPolicy
metadata:
  name: llm-mcp-network-policy
spec:
  podSelector:
    matchLabels:
      app: llm-mcp
  policyTypes:
    - Ingress
    - Egress
  ingress:
    - from:
        - namespaceSelector:
            matchLabels:
              name: ingress-nginx
      ports:
        - protocol: TCP
          port: 8080
  egress:
    - to:
        - namespaceSelector: {}
      ports:
        - protocol: TCP
          port: 443  # HTTPS only
```

### 3. Service Mesh

Consider using Istio or Linkerd for:
- mTLS between services
- Traffic policies
- Circuit breaking
- Rate limiting

```yaml
# Istio PeerAuthentication
apiVersion: security.istio.io/v1beta1
kind: PeerAuthentication
metadata:
  name: default
spec:
  mtls:
    mode: STRICT
```

### 4. API Gateway Security

Configure rate limiting and authentication:

```yaml
# Kong/NGINX rate limiting
limit_req_zone $binary_remote_addr zone=api:10m rate=10r/s;
limit_req zone=api burst=20 nodelay;
```

## Authentication & Authorization

### 1. JWT Configuration

Use strong JWT settings:

```javascript
const jwtOptions = {
  algorithm: 'RS256',
  expiresIn: '15m',
  issuer: 'llm-mcp',
  audience: 'llm-mcp-api',
  keyid: process.env.JWT_KEY_ID
};
```

### 2. OAuth2/OIDC Integration

```javascript
// OAuth2 configuration
const oauth2Config = {
  authorizationURL: 'https://auth.example.com/oauth/authorize',
  tokenURL: 'https://auth.example.com/oauth/token',
  clientID: process.env.OAUTH_CLIENT_ID,
  clientSecret: process.env.OAUTH_CLIENT_SECRET,
  callbackURL: 'https://api.example.com/auth/callback',
  scope: ['openid', 'profile', 'email'],
  state: true,
  pkce: true
};
```

### 3. API Key Management

- Rotate API keys regularly
- Use different keys for different environments
- Implement key versioning
- Monitor key usage

```javascript
// API key validation
const validateApiKey = async (key) => {
  const hashedKey = crypto.createHash('sha256').update(key).digest('hex');
  const validKey = await db.query('SELECT * FROM api_keys WHERE key_hash = $1 AND expires_at > NOW()', [hashedKey]);
  
  if (!validKey) {
    throw new UnauthorizedError('Invalid or expired API key');
  }
  
  // Log usage
  await db.query('INSERT INTO api_key_usage (key_id, used_at, ip_address) VALUES ($1, NOW(), $2)', 
    [validKey.id, req.ip]);
};
```

### 4. RBAC Implementation

```yaml
# Kubernetes RBAC
apiVersion: rbac.authorization.k8s.io/v1
kind: Role
metadata:
  name: llm-mcp-role
rules:
  - apiGroups: [""]
    resources: ["configmaps", "secrets"]
    verbs: ["get", "list"]
    resourceNames: ["llm-mcp-config", "llm-mcp-secrets"]
```

## Data Protection

### 1. Encryption at Rest

```yaml
# Kubernetes StorageClass with encryption
apiVersion: storage.k8s.io/v1
kind: StorageClass
metadata:
  name: encrypted-ssd
provisioner: kubernetes.io/aws-ebs
parameters:
  type: gp3
  encrypted: "true"
  kmsKeyId: "arn:aws:kms:region:account:key/key-id"
```

### 2. Database Security

```sql
-- PostgreSQL Row Level Security
ALTER TABLE sensitive_data ENABLE ROW LEVEL SECURITY;

CREATE POLICY user_data_policy ON sensitive_data
  FOR ALL
  TO application_role
  USING (user_id = current_setting('app.current_user_id')::uuid);
```

### 3. Vector Database Security

```javascript
// Qdrant with API key
const qdrantClient = new QdrantClient({
  url: process.env.QDRANT_URL,
  apiKey: process.env.QDRANT_API_KEY,
  https: true
});
```

### 4. Data Sanitization

```javascript
// Input sanitization
const sanitizeInput = (input) => {
  return validator.escape(
    validator.stripLow(
      validator.trim(input)
    )
  );
};

// Output filtering
const filterSensitiveData = (data) => {
  const sensitive = ['password', 'token', 'secret', 'key', 'ssn'];
  return _.omit(data, sensitive);
};
```

## Secrets Management

### 1. Kubernetes Secrets

```yaml
# Sealed Secrets
apiVersion: bitnami.com/v1alpha1
kind: SealedSecret
metadata:
  name: llm-mcp-secrets
spec:
  encryptedData:
    api-key: AgA3kZ9... # encrypted value
```

### 2. HashiCorp Vault Integration

```javascript
const vault = require('node-vault')({
  endpoint: process.env.VAULT_ADDR,
  token: process.env.VAULT_TOKEN
});

const getSecret = async (path) => [const { data] = await vault.read(`secret/data//path/to/resource`);
  return data.data;
};
```

### 3. AWS Secrets Manager

```javascript
const AWS = require('aws-sdk');
const secretsManager = new AWS.SecretsManager();

const getSecret = async (secretName) => {
  const data = await secretsManager.getSecretValue({ SecretId: secretName }).promise();
  return JSON.parse(data.SecretString);
};
```

### 4. Environment Variable Security

```bash
# Never commit .env files
echo ".env*" >> .gitignore

# Use encrypted environment files
openssl enc -aes-256-cbc -salt -in .env -out .env.enc
```

## Monitoring & Auditing

### 1. Security Event Logging

```javascript
// Audit logging
const auditLog = (event) => {
  logger.info({
    type: 'AUDIT',
    event: event.type,
    user: event.user,
    ip: event.ip,
    timestamp: new Date().toISOString(),
    details: event.details
  });
};

// Log security events
app.use((req, res, next) => {
  if (req.path.startsWith('/auth') || req.path.startsWith('/api')) {
    auditLog({
      type: 'API_ACCESS',
      user: req.user?.id,
      ip: req.ip,
      details: {
        method: req.method,
        path: req.path,
        userAgent: req.get('user-agent')
      }
    });
  }
  next();
});
```

### 2. Intrusion Detection

```yaml
# Falco rules for container runtime security
- rule: Unauthorized Process in Container
  desc: Detect unauthorized process execution
  condition: >
    container and
    not proc.name in (allowed_processes) and
    not container.image.repository in (trusted_images)
  output: >
    Unauthorized process started in container
    (user=%user.name command=%proc.cmdline container=%container.name)
```

### 3. Security Metrics

```javascript
// Prometheus metrics for security monitoring
const securityMetrics = {
  authFailures: new promClient.Counter({
    name: 'auth_failures_total',
    help: 'Total number of authentication failures',
    labelNames: ['reason']
  }),
  
  suspiciousRequests: new promClient.Counter({
    name: 'suspicious_requests_total',
    help: 'Total number of suspicious requests detected',
    labelNames: ['type']
  })
};
```

### 4. SIEM Integration

```javascript
// Send security events to SIEM
const sendToSIEM = async (event) => {
  await fetch(process.env.SIEM_ENDPOINT, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      'Authorization': `Bearer [process.env.SIEM_TOKEN]`
    },
    body: JSON.stringify({
      ...event,
      source: 'llm-mcp',
      severity: calculateSeverity(event)
    })
  });
};
```

## Security Checklist

### Pre-Deployment

- [ ] All container images scanned for vulnerabilities
- [ ] Secrets removed from code and configuration files
- [ ] TLS certificates valid and properly configured
- [ ] Network policies defined and tested
- [ ] RBAC policies implemented
- [ ] Security headers configured (HSTS, CSP, etc.)
- [ ] Input validation implemented
- [ ] Rate limiting configured
- [ ] Audit logging enabled

### Runtime Security

- [ ] Container runtime security (Falco/Sysdig)
- [ ] File integrity monitoring
- [ ] Network traffic monitoring
- [ ] Anomaly detection configured
- [ ] Incident response plan documented
- [ ] Backup encryption verified
- [ ] Access logs regularly reviewed
- [ ] Security patches applied promptly

### Compliance

- [ ] Data residency requirements met
- [ ] Encryption standards compliance (FIPS 140-2)
- [ ] Access control audit trail
- [ ] Data retention policies implemented
- [ ] GDPR/CCPA compliance verified
- [ ] Security assessment completed
- [ ] Penetration testing performed
- [ ] Disaster recovery plan tested

### Monitoring

- [ ] Security dashboard configured
- [ ] Alerting rules defined
- [ ] Log aggregation configured
- [ ] Metrics collection enabled
- [ ] Threat intelligence feeds integrated
- [ ] Vulnerability scanning scheduled
- [ ] Security KPIs defined
- [ ] Regular security reviews scheduled

## Security Headers

Add these headers to all responses:

```javascript
app.use(helmet({
  contentSecurityPolicy: {
    directives: {
      defaultSrc: ["'self'"],
      styleSrc: ["'self'", "'unsafe-inline'"],
      scriptSrc: ["'self'"],
      imgSrc: ["'self'", "data:", "https:"],
      connectSrc: ["'self'"],
      fontSrc: ["'self'"],
      objectSrc: ["'none'"],
      mediaSrc: ["'self'"],
      frameSrc: ["'none'"],
    },
  },
  hsts: {
    maxAge: 31536000,
    includeSubDomains: true,
    preload: true
  }
}));
```

## Additional Resources

- [OWASP Top 10](https://owasp.org/www-project-top-ten/)
- [CIS Docker Benchmark](https://www.cisecurity.org/benchmark/docker)
- [Kubernetes Security Best Practices](https://kubernetes.io/docs/concepts/security/)
- [NIST Cybersecurity Framework](https://www.nist.gov/cyberframework)