# Vector Database Configuration Guide

This guide provides comprehensive documentation for configuring and using vector databases with the LLM Platform AI Platform. Vector databases are essential for storing and retrieving embeddings efficiently, enabling semantic search, recommendation systems, and other AI-powered features.

## Supported Vector Databases

The platform supports the following vector database providers:

| Provider      | Description                                      | Use Cases                                  | Configuration Complexity |
|---------------|--------------------------------------------------|--------------------------------------------|-----------------------|
| Pinecone      | Cloud-based vector database with high scalability | Production deployments, high scalability    | Medium                |
| Qdrant        | High-performance vector similarity search engine | Production/development, self-hosted option | Low                   |
| Weaviate      | Knowledge graph with vector capabilities         | Complex data relationships                 | Medium                |
| Elasticsearch | Full-text search with vector support             | Hybrid search combining text and vectors   | Medium                |
| PGVector      | Vector extension for PostgreSQL                  | Integration with existing PostgreSQL DBs   | Medium                |
| Redis Vector  | Vector functionality in Redis                    | Low-latency, caching scenarios            | Low                   |
| FAISS         | Facebook AI Similarity Search                    | Local deployments, no external dependency | Low                   |
| Local         | File-based vector storage                        | Development, testing                      | Very Low              |
| Memory        | In-memory vector storage                         | Testing, ephemeral usage                  | Very Low              |
| Drupal        | Integration with Drupal's vector storage         | Drupal-based applications                 | Low                   |

## Common Configuration Parameters

All vector database configurations require these common parameters:

| Parameter   | Type    | Required | Description                                               |
|-------------|---------|----------|-----------------------------------------------------------|
| type        | string  | Yes      | The vector database type (see supported providers above)  |
| collection  | string  | Yes      | Collection or index name                                 |
| dimensions  | integer | Yes      | Number of dimensions for vector embeddings              |
| url         | string  | Varies   | Connection URL (not needed for local/memory)            |
| apiKey      | string  | Varies   | API key for authentication (if needed)                  |
| metric      | string  | No       | Metric type: 'cosine', 'euclidean', or 'dot' (default: 'cosine') |
| hybridSearch| boolean | No       | Whether to enable hybrid search (text + vector)         |

## Embedding Models and Dimensions

Vector dimensions must match the embedding model used. Here are common models and their dimensions:

| Model                                   | Dimensions | Provider      |
|-----------------------------------------|------------|---------------|
| text-embedding-ada-002                  | 1536       | OpenAI        |
| text-embedding-3-small                  | 1536       | OpenAI        |
| text-embedding-3-large                  | 3072       | OpenAI        |
| bfllm-embeddings-en                     | 1024       | LLM Platform LLM   |
| bfllm-embeddings-multi                  | 1536       | LLM Platform LLM   |
| bfllm-embeddings-image                  | 2048       | LLM Platform LLM   |
| sentence-transformers/all-mpnet-base-v2 | 768        | Hugging Face  |
| native-ai-embeddings-base               | 768        | Native AI JS  |
| native-ai-embeddings-large              | 1536       | Native AI JS  |

## Provider-Specific Configuration

### Pinecone

```typescript
{
  type: 'pinecone',
  collection: 'my-index', // Pinecone index name
  dimensions: 1536,
  url: 'https://my-index-project.svc.us-east1.pinecone.io',
  apiKey: 'pinecone-api-key',
  options: {
    environment: 'us-east1', // Required: Pinecone environment
    projectId: 'my-project-id', // Required: Pinecone project ID
    namespace: 'my-namespace' // Optional: Namespace for partitioning
  }
}
```

**Notes:**
- Pinecone requires an account and API key from [pinecone.io](https://pinecone.io)
- Environments are region-specific (e.g., 'us-east1', 'eu-west1')
- Namespaces allow data partitioning within an index

### Qdrant

```typescript
{
  type: 'qdrant',
  collection: 'my-collection',
  dimensions: 1536,
  url: 'http://localhost:6333', // or your Qdrant Cloud URL
  apiKey: 'optional-api-key', // Required for Qdrant Cloud
  metric: 'cosine' // Optional: 'cosine', 'euclidean', or 'dot'
}
```

**Notes:**
- Qdrant is available as a self-hosted solution or through [Qdrant Cloud](https://qdrant.tech)
- Collection names must contain only letters, numbers, underscores, and hyphens

### Weaviate

```typescript
{
  type: 'weaviate',
  collection: 'MyCollection',
  dimensions: 1536,
  url: 'http://localhost:8080',
  apiKey: 'optional-api-key',
  options: {
    schemaClass: 'MyClass', // Required: Weaviate schema class name
    textKey: 'content' // Optional: Property name for text storage
  }
}
```

**Notes:**
- Weaviate schema classes must start with a capital letter and contain only alphanumeric characters
- Weaviate is available as a self-hosted solution or through [Weaviate Cloud](https://weaviate.io)

### Elasticsearch

```typescript
{
  type: 'elasticsearch',
  collection: 'my-index-prefix',
  dimensions: 1536,
  url: 'http://localhost:9200',
  apiKey: 'optional-api-key',
  options: {
    index: 'my-vector-index', // Required: Elasticsearch index name
    textField: 'content', // Optional: Field name for text storage
    refreshInterval: '1s' // Optional: Index refresh interval
  }
}
```

**Notes:**
- Elasticsearch index names must be lowercase without spaces
- Refresh interval must be a valid Elasticsearch time value (e.g., '1s', '30s', '1m')
- Requires Elasticsearch 7.11+ with the vector search capability

### PGVector

```typescript
{
  type: 'pgvector',
  collection: 'my-collection',
  dimensions: 1536,
  url: 'postgres://username:password@localhost:5432/database',
  options: {
    tableName: 'embeddings', // Required: Database table name
    vectorColumnName: 'embedding', // Optional: Column for vector data
    textColumnName: 'content', // Optional: Column for text data
    metadataColumnName: 'metadata' // Optional: Column for metadata
  }
}
```

**Notes:**
- Requires PostgreSQL with the pgvector extension installed
- Table and column names must follow SQL naming conventions
- The vector column should be defined with the vector data type matching the dimensions

### Redis Vector

```typescript
{
  type: 'redis',
  collection: 'my-collection',
  dimensions: 1536,
  url: 'redis://localhost:6379',
  options: {
    keyPrefix: 'vector:', // Optional: Prefix for Redis keys
    vectorKeyPrefix: 'embedding:' // Optional: Prefix for vector keys
  }
}
```

**Notes:**
- Requires Redis Stack or Redis with the RediSearch and RedisJSON modules
- Key prefixes can only contain letters, numbers, underscores, colons, periods, and hyphens
- Redis URLs should be in the format: `redis://[[username:]password@]host[:port][/database-number]`

### FAISS

```typescript
{
  type: 'faiss',
  collection: 'my-collection',
  dimensions: 1536,
  options: {
    indexPath: '/path/to/faiss/index' // Optional: Path to save/load FAISS index
  }
}
```

**Notes:**
- FAISS runs locally in the Node.js process
- Recommended for smaller datasets or testing environments
- The index can be persisted to disk for reuse across restarts

### Local Storage

```typescript
{
  type: 'local',
  collection: 'my-collection',
  dimensions: 1536,
  options: {
    storagePath: '/path/to/storage', // Optional: Path to store vectors
    persistToFile: true // Optional: Whether to persist vectors to disk
  }
}
```

**Notes:**
- Simple file-based storage for development and testing
- Not recommended for production use with large datasets

### Memory Storage

```typescript
{
  type: 'memory',
  collection: 'my-collection',
  dimensions: 1536
}
```

**Notes:**
- In-memory storage for testing and development
- Data is lost when the application restarts
- Most appropriate for unit testing and short-lived applications

### Drupal Integration

```typescript
{
  type: 'drupal',
  collection: 'my-collection',
  dimensions: 1536,
  url: 'http://drupal-site.com',
  apiKey: 'drupal-api-key',
  options: {
    endpoint: '/api/vector-database' // Required: API endpoint
  }
}
```

**Notes:**
- Requires the Drupal Vector Database module
- Uses Drupal's entity system for vector storage
- Provides integration with Drupal content entities

## Error Handling and Validation

The platform includes built-in validation for vector database configurations. Errors are reported with specific codes and descriptive messages to help diagnose and fix issues:

| Error Code             | Description                                  |
|------------------------|----------------------------------------------|
| CONFIGURATION          | Invalid or missing configuration parameters  |
| DATABASE_CONNECTION    | Unable to connect to the vector database     |
| DATABASE_OPERATION     | Error during database operation              |
| AUTHENTICATION         | Authentication failure                       |
| INVALID_INPUT          | Invalid input parameters                     |
| NOT_FOUND              | Requested resource not found                 |

When encountering configuration errors, check:

1. Required fields for your chosen vector database
2. Connection details (URL, API keys)
3. Dimension values match your embedding model
4. Field formats match the provider's requirements

## Best Practices

1. **Match Dimensions**: Ensure vector dimensions match your embedding model.
2. **Secure API Keys**: Store API keys securely using environment variables or a secrets manager.
3. **Meaningful Collections**: Use descriptive collection/index names for better organization.
4. **Performance Tuning**: Use appropriate metrics and settings for your use case:
   - Use 'cosine' for most text embedding similarity searches
   - Use 'euclidean' for spatial data or when working with L2-normalized embeddings
   - Use 'dot' product for optimized performance with certain types of models
5. **Test Configurations**: Validate configurations before deploying to production.

## Advanced Configuration

### Hybrid Search

Hybrid search combines vector similarity with text-based search for more accurate results:

```typescript
{
  type: 'qdrant', // or any supported type
  collection: 'my-collection',
  dimensions: 1536,
  url: 'http://localhost:6333',
  hybridSearch: true // Enable hybrid search capabilities
}
```

### Replication Configuration

For high availability and redundancy, configure replication to multiple vector databases:

```typescript
{
  database: {
    type: 'qdrant',
    collection: 'my-collection',
    dimensions: 1536,
    url: 'http://primary-qdrant:6333'
  },
  replication: {
    enabled: true,
    targets: [
      {
        type: 'qdrant',
        collection: 'my-collection',
        dimensions: 1536,
        url: 'http://replica-qdrant:6333'
      }
    ],
    mode: 'async', // or 'sync'
    batchSize: 100
  }
}
```

### Caching Configuration

Configure caching to improve performance for frequently accessed vectors:

```typescript
{
  database: {
    type: 'pinecone',
    collection: 'my-index',
    dimensions: 1536,
    url: 'https://my-index-project.svc.us-east1.pinecone.io',
    apiKey: 'pinecone-api-key'
  },
  cache: {
    enabled: true,
    maxItems: 10000, // Maximum number of cached items
    ttl: 300 // Time-to-live in seconds
  }
}
```

## Troubleshooting

Common issues and solutions:

1. **Connection Errors**:
   - Verify URL is correct and accessible from your environment
   - Check network connectivity and firewall settings
   - Ensure API keys have correct permissions

2. **Dimension Mismatches**:
   - Ensure database dimensions match your embedding model
   - Check for truncated vectors in your processing pipeline

3. **Performance Issues**:
   - Use appropriate index settings for your vector database
   - Consider sharding for large collections
   - Enable caching for frequently accessed vectors

4. **Authentication Failures**:
   - Verify API keys are correct and not expired
   - Check if authentication method is supported
   - Ensure proper credentials for your environment

## Migration Guide

When migrating between vector database providers:

1. Export vectors from the source database
2. Transform data to match the target provider's format
3. Import vectors to the target database
4. Validate vector counts and perform sample queries
5. Update your application configuration

Example migration script structure:
```typescript
// Migration script outline
async function migrateVectorDatabase(sourceConfig, targetConfig) {
  // Initialize source and target adapters
  const sourceAdapter = createDatabaseAdapter(sourceConfig);
  const targetAdapter = createDatabaseAdapter(targetConfig);
  
  // Get all vectors from source
  const vectors = await getAllVectors(sourceAdapter);
  
  // Import to target
  await importVectors(targetAdapter, vectors);
  
  // Validate migration
  await validateMigration(sourceAdapter, targetAdapter);
}
```