# API Reference

This document provides a comprehensive reference for the LLM-MCP (LLM Platform Model Context Protocol) API. It covers all available endpoints, request/response formats, authentication, error handling, and examples. The API allows you to interact with the LLM-MCP platform programmatically, enabling tool registration, execution, vector operations, and agent management.

## Table of Contents

- [Overview](#overview)
- [Base URLs](#base-urls)
- [Authentication](#authentication)
- [Tool Registry API](#tool-registry-api)
- [Tool Execution API](#tool-execution-api)
- [Resource API](#resource-api)
- [Vector API](#vector-api)
- [Agent API](#agent-api)
- [Health API](#health-api)
- [Error Handling](#error-handling)
- [Rate Limiting](#rate-limiting)
- [OpenAPI Specification](#openapi-specification)
- [Client SDKs](#client-sdks)

## Overview

The LLM-MCP API provides a standardized interface for:

- Registering and discovering AI tools
- Executing tools with standard inputs and outputs
- Accessing shared resources
- Managing vector embeddings
- Coordinating agent workflows
- Monitoring service health

The API follows REST principles and uses JSON for request and response payloads. It also provides a gRPC interface for high-performance tool execution.

## Base URLs

### REST API

```
Production: https://api.llm-mcp.bluefly.io/v1
Staging: https://api-staging.llm-mcp.bluefly.io/v1
Development: http://localhost:3001/v1
```

### gRPC API

```
Production: grpc://api.llm-mcp.bluefly.io:50051
Staging: grpc://api-staging.llm-mcp.bluefly.io:50051
Development: grpc://localhost:50051
```

## Authentication

The LLM-MCP API supports multiple authentication methods:

### JWT Authentication

Send a JWT token in the Authorization header:

```
Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
```

### API Key Authentication

Send an API key in the X-API-Key header:

```
X-API-Key: your-api-key
```

### OAuth2 Authentication

The API supports OAuth2 authentication with the following endpoints:

- **Token Endpoint**: `/oauth/token`
- **Authorize Endpoint**: `/oauth/authorize`
- **Supported Grant Types**: `authorization_code`, `client_credentials`, `refresh_token`

## Tool Registry API

The Tool Registry API allows you to register, discover, and manage tools.

### List All Tools

```
GET /tools
```

**Query Parameters**:

| Parameter | Type   | Description                        |
|-----------|--------|------------------------------------|
| category  | string | Filter tools by category           |
| tag       | string | Filter tools by tag                |
| q         | string | Search term for tool name or description |
| limit     | number | Maximum number of tools to return  |
| offset    | number | Offset for pagination              |

**Response**:

```json
{
  "tools": [
    {
      "id": "content_search",
      "name": "Content Search",
      "description": "Search for content across the platform",
      "version": "1.0.0",
      "category": "search",
      "tags": ["content", "search"],
      "input_schema": {
        "type": "object",
        "properties": {
          "query": {
            "type": "string",
            "description": "The search query"
          },
          "limit": {
            "type": "number",
            "description": "Maximum number of results",
            "default": 10
          }
        },
        "required": ["query"]
      },
      "output_schema": {
        "type": "object",
        "properties": {
          "results": {
            "type": "array",
            "items": {
              "type": "object",
              "properties": {
                "id": { "type": "string" },
                "title": { "type": "string" },
                "url": { "type": "string" },
                "summary": { "type": "string" }
              }
            }
          },
          "total_count": { "type": "number" }
        }
      }
    }
  ],
  "total_count": 1,
  "limit": 10,
  "offset": 0
}
```

### Get Tool by ID

```
GET /tools/[tool_id]
```

**Path Parameters**:

| Parameter | Type   | Description              |
|-----------|--------|--------------------------|
| tool_id   | string | The ID of the tool to get |

**Response**:

```json
{
  "id": "content_search",
  "name": "Content Search",
  "description": "Search for content across the platform",
  "version": "1.0.0",
  "category": "search",
  "tags": ["content", "search"],
  "input_schema": {
    "type": "object",
    "properties": {
      "query": {
        "type": "string",
        "description": "The search query"
      },
      "limit": {
        "type": "number",
        "description": "Maximum number of results",
        "default": 10
      }
    },
    "required": ["query"]
  },
  "output_schema": {
    "type": "object",
    "properties": {
      "results": {
        "type": "array",
        "items": {
          "type": "object",
          "properties": {
            "id": { "type": "string" },
            "title": { "type": "string" },
            "url": { "type": "string" },
            "summary": { "type": "string" }
          }
        }
      },
      "total_count": { "type": "number" }
    }
  }
}
```

### Register Tool

```
POST /tools
```

**Request Body**:

```json
{
  "id": "content_search",
  "name": "Content Search",
  "description": "Search for content across the platform",
  "version": "1.0.0",
  "category": "search",
  "tags": ["content", "search"],
  "input_schema": {
    "type": "object",
    "properties": {
      "query": {
        "type": "string",
        "description": "The search query"
      },
      "limit": {
        "type": "number",
        "description": "Maximum number of results",
        "default": 10
      }
    },
    "required": ["query"]
  },
  "output_schema": {
    "type": "object",
    "properties": {
      "results": {
        "type": "array",
        "items": {
          "type": "object",
          "properties": {
            "id": { "type": "string" },
            "title": { "type": "string" },
            "url": { "type": "string" },
            "summary": { "type": "string" }
          }
        }
      },
      "total_count": { "type": "number" }
    }
  }
}
```

**Response**:

```json
{
  "id": "content_search",
  "status": "registered",
  "message": "Tool registered successfully"
}
```

### Update Tool

```
PUT /tools/[tool_id]
```

**Path Parameters**:

| Parameter | Type   | Description              |
|-----------|--------|--------------------------|
| tool_id   | string | The ID of the tool to update |

**Request Body**:

Same as the register tool request body.

**Response**:

```json
{
  "id": "content_search",
  "status": "updated",
  "message": "Tool updated successfully"
}
```

### Delete Tool

```
DELETE /tools/[tool_id]
```

**Path Parameters**:

| Parameter | Type   | Description              |
|-----------|--------|--------------------------|
| tool_id   | string | The ID of the tool to delete |

**Response**:

```json
{
  "id": "content_search",
  "status": "deleted",
  "message": "Tool deleted successfully"
}
```

## Tool Execution API

The Tool Execution API allows you to execute tools with standard inputs and receive outputs.

### Execute Tool

```
POST /tools/[tool_id]/execute
```

**Path Parameters**:

| Parameter | Type   | Description                  |
|-----------|--------|------------------------------|
| tool_id   | string | The ID of the tool to execute |

**Request Body**:

The request body must conform to the tool's input schema. For example:

```json
{
  "query": "artificial intelligence",
  "limit": 5
}
```

**Response**:

The response body will conform to the tool's output schema. For example:

```json
{
  "results": [
    {
      "id": "article-1",
      "title": "Introduction to AI",
      "url": "https://example.com/articles/introduction-to-ai",
      "summary": "An introduction to artificial intelligence."
    },
    {
      "id": "article-2",
      "title": "Machine Learning Basics",
      "url": "https://example.com/articles/machine-learning-basics",
      "summary": "The basics of machine learning."
    }
  ],
  "total_count": 2
}
```

### Execute Tool Asynchronously

```
POST /tools/[tool_id]/execute-async
```

**Path Parameters**:

| Parameter | Type   | Description                  |
|-----------|--------|------------------------------|
| tool_id   | string | The ID of the tool to execute |

**Request Body**:

Same as the synchronous execute tool request body.

**Response**:

```json
{
  "task_id": "task-123",
  "status": "in_progress",
  "message": "Tool execution started",
  "status_url": "/tasks/task-123"
}
```

### Check Asynchronous Task Status

```
GET /tasks/[task_id]
```

**Path Parameters**:

| Parameter | Type   | Description                 |
|-----------|--------|-----------------------------|
| task_id   | string | The ID of the task to check |

**Response**:

```json
{
  "task_id": "task-123",
  "status": "completed",
  "result": {
    // Tool execution result (conforms to the tool's output schema)
  },
  "created_at": "2023-06-01T12:00:00Z",
  "completed_at": "2023-06-01T12:00:05Z"
}
```

## Resource API

The Resource API allows you to access shared resources.

### List Resources

```
GET /resources
```

**Query Parameters**:

| Parameter | Type   | Description                         |
|-----------|--------|-------------------------------------|
| type      | string | Filter resources by type            |
| tag       | string | Filter resources by tag             |
| q         | string | Search term for resource name or description |
| limit     | number | Maximum number of resources to return |
| offset    | number | Offset for pagination                |

**Response**:

```json
{
  "resources": [
    {
      "id": "api-docs",
      "name": "API Documentation",
      "description": "Documentation for the API",
      "type": "documentation",
      "tags": ["api", "documentation"],
      "uri": "/resources/api-docs",
      "metadata": {
        "format": "markdown",
        "version": "1.0.0"
      }
    }
  ],
  "total_count": 1,
  "limit": 10,
  "offset": 0
}
```

### Get Resource by ID

```
GET /resources/[resource_id]
```

**Path Parameters**:

| Parameter  | Type   | Description                   |
|------------|--------|-------------------------------|
| resource_id | string | The ID of the resource to get |

**Response**:

```json
{
  "id": "api-docs",
  "name": "API Documentation",
  "description": "Documentation for the API",
  "type": "documentation",
  "tags": ["api", "documentation"],
  "uri": "/resources/api-docs",
  "metadata": {
    "format": "markdown",
    "version": "1.0.0"
  },
  "content": "# API Documentation\n\n..."
}
```

### Register Resource

```
POST /resources
```

**Request Body**:

```json
{
  "id": "api-docs",
  "name": "API Documentation",
  "description": "Documentation for the API",
  "type": "documentation",
  "tags": ["api", "documentation"],
  "metadata": {
    "format": "markdown",
    "version": "1.0.0"
  },
  "content": "# API Documentation\n\n..."
}
```

**Response**:

```json
{
  "id": "api-docs",
  "status": "registered",
  "message": "Resource registered successfully",
  "uri": "/resources/api-docs"
}
```

### Update Resource

```
PUT /resources/[resource_id]
```

**Path Parameters**:

| Parameter  | Type   | Description                      |
|------------|--------|----------------------------------|
| resource_id | string | The ID of the resource to update |

**Request Body**:

Same as the register resource request body.

**Response**:

```json
{
  "id": "api-docs",
  "status": "updated",
  "message": "Resource updated successfully",
  "uri": "/resources/api-docs"
}
```

### Delete Resource

```
DELETE /resources/[resource_id]
```

**Path Parameters**:

| Parameter  | Type   | Description                      |
|------------|--------|----------------------------------|
| resource_id | string | The ID of the resource to delete |

**Response**:

```json
{
  "id": "api-docs",
  "status": "deleted",
  "message": "Resource deleted successfully"
}
```

## Vector API

The Vector API allows you to store, search, and manage vector embeddings.

### Store Vector

```
POST /vectors
```

**Request Body**:

```json
{
  "id": "vector-1",
  "vector": [0.1, 0.2, 0.3, 0.4],
  "metadata": {
    "text": "Example text",
    "source": "example.com",
    "tags": ["example", "vector"]
  }
}
```

**Response**:

```json
{
  "id": "vector-1",
  "status": "stored",
  "message": "Vector stored successfully"
}
```

### Get Vector by ID

```
GET /vectors/[vector_id]
```

**Path Parameters**:

| Parameter | Type   | Description                |
|-----------|--------|----------------------------|
| vector_id | string | The ID of the vector to get |

**Response**:

```json
{
  "id": "vector-1",
  "vector": [0.1, 0.2, 0.3, 0.4],
  "metadata": {
    "text": "Example text",
    "source": "example.com",
    "tags": ["example", "vector"]
  }
}
```

### Search Vectors

```
POST /vectors/search
```

**Request Body**:

```json
{
  "query_vector": [0.1, 0.2, 0.3, 0.4],
  "top_k": 5,
  "metadata_filter": {
    "tags": "example"
  }
}
```

**Response**:

```json
{
  "results": [
    {
      "id": "vector-1",
      "score": 1.0,
      "vector": [0.1, 0.2, 0.3, 0.4],
      "metadata": {
        "text": "Example text",
        "source": "example.com",
        "tags": ["example", "vector"]
      }
    }
  ]
}
```

### Delete Vector

```
DELETE /vectors/[vector_id]
```

**Path Parameters**:

| Parameter | Type   | Description                   |
|-----------|--------|-------------------------------|
| vector_id | string | The ID of the vector to delete |

**Response**:

```json
{
  "id": "vector-1",
  "status": "deleted",
  "message": "Vector deleted successfully"
}
```

### Batch Operations

#### Batch Store Vectors

```
POST /vectors/batch
```

**Request Body**:

```json
{
  "vectors": [
    {
      "id": "vector-1",
      "vector": [0.1, 0.2, 0.3, 0.4],
      "metadata": {
        "text": "Example text 1",
        "source": "example.com",
        "tags": ["example", "vector"]
      }
    },
    {
      "id": "vector-2",
      "vector": [0.2, 0.3, 0.4, 0.5],
      "metadata": {
        "text": "Example text 2",
        "source": "example.com",
        "tags": ["example", "vector"]
      }
    }
  ]
}
```

**Response**:

```json
{
  "status": "stored",
  "message": "2 vectors stored successfully",
  "vectors": [
    {
      "id": "vector-1",
      "status": "stored"
    },
    {
      "id": "vector-2",
      "status": "stored"
    }
  ]
}
```

#### Batch Get Vectors

```
POST /vectors/batch/get
```

**Request Body**:

```json
{
  "ids": ["vector-1", "vector-2"]
}
```

**Response**:

```json
{
  "vectors": [
    {
      "id": "vector-1",
      "vector": [0.1, 0.2, 0.3, 0.4],
      "metadata": {
        "text": "Example text 1",
        "source": "example.com",
        "tags": ["example", "vector"]
      }
    },
    {
      "id": "vector-2",
      "vector": [0.2, 0.3, 0.4, 0.5],
      "metadata": {
        "text": "Example text 2",
        "source": "example.com",
        "tags": ["example", "vector"]
      }
    }
  ]
}
```

#### Batch Delete Vectors

```
POST /vectors/batch/delete
```

**Request Body**:

```json
{
  "ids": ["vector-1", "vector-2"]
}
```

**Response**:

```json
{
  "status": "deleted",
  "message": "2 vectors deleted successfully",
  "vectors": [
    {
      "id": "vector-1",
      "status": "deleted"
    },
    {
      "id": "vector-2",
      "status": "deleted"
    }
  ]
}
```

## Agent API

The Agent API allows you to manage agent workflows.

### Create Agent

```
POST /agents
```

**Request Body**:

```json
{
  "id": "agent-1",
  "name": "Content Creation Agent",
  "description": "Agent for creating and managing content",
  "tools": ["content_search", "content_generator", "content_analyzer"],
  "config": {
    "max_iterations": 10,
    "timeout": 60,
    "memory": true
  }
}
```

**Response**:

```json
{
  "id": "agent-1",
  "status": "created",
  "message": "Agent created successfully"
}
```

### Get Agent by ID

```
GET /agents/[agent_id]
```

**Path Parameters**:

| Parameter | Type   | Description               |
|-----------|--------|---------------------------|
| agent_id  | string | The ID of the agent to get |

**Response**:

```json
{
  "id": "agent-1",
  "name": "Content Creation Agent",
  "description": "Agent for creating and managing content",
  "tools": ["content_search", "content_generator", "content_analyzer"],
  "config": {
    "max_iterations": 10,
    "timeout": 60,
    "memory": true
  }
}
```

### List Agents

```
GET /agents
```

**Query Parameters**:

| Parameter | Type   | Description                       |
|-----------|--------|-----------------------------------|
| name      | string | Filter agents by name             |
| tool      | string | Filter agents by tool             |
| limit     | number | Maximum number of agents to return |
| offset    | number | Offset for pagination              |

**Response**:

```json
{
  "agents": [
    {
      "id": "agent-1",
      "name": "Content Creation Agent",
      "description": "Agent for creating and managing content",
      "tools": ["content_search", "content_generator", "content_analyzer"],
      "config": {
        "max_iterations": 10,
        "timeout": 60,
        "memory": true
      }
    }
  ],
  "total_count": 1,
  "limit": 10,
  "offset": 0
}
```

### Update Agent

```
PUT /agents/[agent_id]
```

**Path Parameters**:

| Parameter | Type   | Description                  |
|-----------|--------|------------------------------|
| agent_id  | string | The ID of the agent to update |

**Request Body**:

Same as the create agent request body.

**Response**:

```json
{
  "id": "agent-1",
  "status": "updated",
  "message": "Agent updated successfully"
}
```

### Delete Agent

```
DELETE /agents/[agent_id]
```

**Path Parameters**:

| Parameter | Type   | Description                  |
|-----------|--------|------------------------------|
| agent_id  | string | The ID of the agent to delete |

**Response**:

```json
{
  "id": "agent-1",
  "status": "deleted",
  "message": "Agent deleted successfully"
}
```

### Execute Agent

```
POST /agents/[agent_id]/execute
```

**Path Parameters**:

| Parameter | Type   | Description                  |
|-----------|--------|------------------------------|
| agent_id  | string | The ID of the agent to execute |

**Request Body**:

```json
{
  "input": {
    "query": "Create an article about artificial intelligence"
  },
  "config": {
    "max_iterations": 5,
    "timeout": 30
  }
}
```

**Response**:

```json
{
  "task_id": "task-123",
  "status": "in_progress",
  "message": "Agent execution started",
  "status_url": "/tasks/task-123"
}
```

## Health API

The Health API allows you to check the health of the LLM-MCP service.

### Get Health Status

```
GET /health
```

**Response**:

```json
{
  "status": "healthy",
  "version": "1.0.0",
  "uptime": 3600,
  "timestamp": "2023-06-01T12:00:00Z",
  "components": {
    "api": {
      "status": "healthy",
      "message": "API is functioning normally"
    },
    "database": {
      "status": "healthy",
      "message": "Database connection is stable"
    },
    "vector_store": {
      "status": "healthy",
      "message": "Vector store is operational"
    }
  }
}
```

### Get Component Health Status

```
GET /health/component-name
```

**Path Parameters**:

| Parameter | Type   | Description                    |
|-----------|--------|--------------------------------|
| component | string | The component to check health for |

**Response**:

```json
{
  "status": "healthy",
  "message": "API is functioning normally",
  "details": {
    "requests_per_second": 10,
    "average_response_time": 100,
    "error_rate": 0.01
  }
}
```

## Error Handling

The LLM-MCP API uses standard HTTP status codes to indicate the success or failure of an API request. In addition to the status code, the response body will contain more information about the error.

### Error Response Format

```json
{
  "error": {
    "code": "invalid_request",
    "message": "The request was invalid",
    "details": [
      {
        "field": "query",
        "message": "Query is required"
      }
    ],
    "request_id": "req-123"
  }
}
```

### Common Error Codes

| Status Code | Error Code        | Description                                          |
|-------------|-------------------|------------------------------------------------------|
| 400         | invalid_request   | The request was invalid                              |
| 401         | unauthorized      | Authentication is required                           |
| 403         | forbidden         | The authenticated user does not have access          |
| 404         | not_found         | The requested resource was not found                 |
| 409         | conflict          | The request conflicts with the current state         |
| 422         | validation_error  | The request body failed validation                    |
| 429         | too_many_requests | The rate limit has been exceeded                     |
| 500         | internal_error    | An internal server error occurred                    |
| 503         | service_unavailable | The service is temporarily unavailable             |

## Rate Limiting

The LLM-MCP API implements rate limiting to prevent abuse and ensure fair usage. Rate limits are applied per API key or user.

### Rate Limit Headers

The API includes the following headers in responses to help you manage rate limits:

```
X-RateLimit-Limit: 100
X-RateLimit-Remaining: 99
X-RateLimit-Reset: 1622548800
```

- `X-RateLimit-Limit`: The maximum number of requests allowed in the current time window
- `X-RateLimit-Remaining`: The number of requests remaining in the current time window
- `X-RateLimit-Reset`: The time at which the current rate limit window resets (Unix timestamp)

### Rate Limit Exceeded Response

When the rate limit is exceeded, the API returns a 429 Too Many Requests response:

```json
{
  "error": {
    "code": "too_many_requests",
    "message": "Rate limit exceeded. Please try again later.",
    "details": {
      "limit": 100,
      "remaining": 0,
      "reset": 1622548800
    },
    "request_id": "req-123"
  }
}
```

## OpenAPI Specification

The LLM-MCP API is fully documented using the OpenAPI 3.0 specification. You can access the specification at:

```
/openapi.yaml
```

You can also view the interactive API documentation at:

```
/api-docs
```

## Client SDKs

LLM-MCP provides client SDKs for various programming languages to simplify API integration:

- **JavaScript/TypeScript**: [@bluefly/llm-mcp-client](https://www.npmjs.com/package/@bluefly/llm-mcp-client)
- **Python**: [llm-mcp-client](https://pypi.org/project/llm-mcp-client/)
- **PHP**: [bluefly/llm-mcp-client](https://packagist.org/packages/bluefly/llm-mcp-client)
- **Java**: [io.bluefly.llm-mcp.client](https://mvnrepository.com/artifact/io.bluefly.llm-mcp/client)
- **Go**: [github.com/bluefly/llm-mcp-client-go](https://github.com/bluefly/llm-mcp-client-go)
- **Ruby**: [llm-mcp-client](https://rubygems.org/gems/llm-mcp-client)

### JavaScript/TypeScript Example

```javascript
import [BfmcpClient] from '@bluefly/llm-mcp-client';

// Create a client
const client = new BfmcpClient({
  baseUrl: 'https://api.llm-mcp.bluefly.io/v1',
  apiKey: 'your-api-key'
});

// List tools
async function listTools() {
  const tools = await client.tools.list();
  console.log(tools);
}

// Execute a tool
async function executeContentSearch() {
  const result = await client.tools.execute('content_search', {
    query: 'artificial intelligence',
    limit: 5
  });
  console.log(result);
}

// Store a vector
async function storeVector() {
  const result = await client.vectors.store({
    id: 'vector-1',
    vector: [0.1, 0.2, 0.3, 0.4],
    metadata: {
      text: 'Example text',
      source: 'example.com',
      tags: ['example', 'vector']
    }
  });
  console.log(result);
}
```

### Python Example

```python
from llm-mcp_client import BfmcpClient

# Create a client
client = BfmcpClient(
    base_url='https://api.llm-mcp.bluefly.io/v1',
    api_key='your-api-key'
)

# List tools
def list_tools():
    tools = client.tools.list()
    print(tools)

# Execute a tool
def execute_content_search():
    result = client.tools.execute('content_search', {
        'query': 'artificial intelligence',
        'limit': 5
    })
    print(result)

# Store a vector
def store_vector():
    result = client.vectors.store({
        'id': 'vector-1',
        'vector': [0.1, 0.2, 0.3, 0.4],
        'metadata': {
            'text': 'Example text',
            'source': 'example.com',
            'tags': ['example', 'vector']
        }
    })
    print(result)
```

For more examples and detailed documentation on using the client SDKs, refer to the SDK-specific documentation.

## See Also

- [Getting Started Guide](../guides/getting-started.md) - Step-by-step guide to getting started with LLM-MCP
- [Tool Registration Guide](../guides/tool-registration.md) - Detailed guide for registering and managing tools
- [Vector Operations Guide](../guides/vector-operations.md) - Comprehensive guide to vector storage and retrieval
- [Basic Usage Examples](../examples/basic-usage.md) - Code examples for common LLM-MCP operations
- [gRPC API Documentation](grpc-api.md) - Documentation for the gRPC API
- [OpenAPI Specification](../../openapi.yaml) - Full OpenAPI specification