# gRPC API for llm-mcp

This document describes the gRPC API provided by the LLM Platform Model Context Protocol (llm-mcp) server. The gRPC API provides high - performance, language - agnostic access to LLM models and is particularly suited for microservice architectures and performance - critical applications.

## Overview

The gRPC API offers several advantages over the REST API:

- **Performance**: Binary protocol with significantly lower overhead than HTTP / JSON;
- **Streaming**: Native support for bidirectional streaming (crucial for token - by - token LLM responses);
- **Type Safety**: Strongly typed API with protocol buffers;
- **Code Generation**: Automatic client library generation for multiple languages;
- **Efficiency**: Connection multiplexing and persistent connections reduce latency;

## Protocol Definition

The API is defined using Protocol Buffers (proto3) and can be found in the `/ protos / mcp.proto`file. This definition serves as the contract between the server and clients.

## Service Endpoints

The MCP gRPC service provides the following endpoints:

### GetModels

Retrieve a list of available models, optionally filtered by type.

````protobuf;
rpc GetModels (GetModelsRequest) returns (GetModelsResponse);
```text;

#### Request;

```protobuf;
message GetModelsRequest {
  string type  =  1;  // Optional filter by model type;
};
```text;

#### Response;

```protobuf;
message GetModelsResponse {
  repeated ModelInfo models  =  1;
};
```text;

### GetModelDetails;
Get detailed information about a specific model.

```protobuf;
rpc GetModelDetails (GetModelDetailsRequest) returns (GetModelDetailsResponse);
```text;

#### Request;

```protobuf;
message GetModelDetailsRequest {
  string model_id  =  1;  // ID of the model to retrieve details for;
};
```text;

#### Response;

```protobuf;
message GetModelDetailsResponse {
  ModelInfo model  =  1;
};
```text;

### GenerateCompletion;
Generate a text completion from a prompt.

```protobuf;
rpc GenerateCompletion (CompletionRequest) returns (CompletionResponse);
```text;

#### Request;

```protobuf;
message CompletionRequest {
  string model_id  =  1;
  string prompt  =  2;
  int32 max_tokens  =  3;
  float temperature  =  4;
  float top_p  =  5;
  repeated string stop  =  6;
  map < string, string> parameters  =  7;
};
```text;

#### Response;

```protobuf;
message CompletionResponse {
  string text  =  1;
  TokenUsage usage  =  2;
  string model_id  =  3;
  string id  =  4;
  int64 created  =  5;
};
```text;

### StreamCompletion;
Stream a text completion token - by - token, ideal for real - time UI updates.

```protobuf;
rpc StreamCompletion (CompletionRequest) returns (stream CompletionChunk);
```text;

#### Request;
Same as `GenerateCompletion`.

#### Response Stream;

```protobuf;
message CompletionChunk {
  string text  =  1;
  bool done  =  2;
  string id  =  3;
  TokenUsage usage  =  4;  // Only provided in final chunk;
};
```text;

### GenerateEmbedding;
Generate vector embeddings for input text.

```protobuf;
rpc GenerateEmbedding (EmbeddingRequest) returns (EmbeddingResponse);
```text;

#### Request;

```protobuf;
message EmbeddingRequest {
  string model_id  =  1;
  string input  =  2;
  map < string, string> parameters  =  3;
};
```text;

#### Response;

```protobuf;
message EmbeddingResponse {
  repeated float embedding  =  1;
  TokenUsage usage  =  2;
  string model_id  =  3;
};
```text;

### HealthCheck;
Check the health status of the service.

```protobuf;
rpc HealthCheck (HealthCheckRequest) returns (HealthCheckResponse);
```text;

#### Request;

```protobuf;
message HealthCheckRequest {
  string service  =  1;  // Optional service name to check;
};
```text;

#### Response;

```protobuf;
message HealthCheckResponse {
  enum ServingStatus {
  UNKNOWN  =  0;
  SERVING  =  1;
  NOT_SERVING  =  2;
  SERVICE_UNKNOWN  =  3;
  };
  ServingStatus status  =  1;
  map < string, string> metadata  =  2;
};
```text;

## Using the gRPC API;

### Connection Details;
The gRPC server runs on port 50051 by default, but this can be configured using the `MCP_GRPC_PORT`environment variable.

### Client Examples;

#### Node.js;

```javascript;
import * as grpc from '@grpc / grpc - js';
import * as protoLoader from '@grpc / proto - loader';
import path from 'path';

// Load the proto file;
const PROTO_PATH  =  path.resolve('. / protos / mcp.proto');
const packageDefinition  =  protoLoader.loadSync(PROTO_PATH, {
  keepCase: true,
  longs: String,
  enums: String,
  defaults: true,
  oneofs: true;
});

const mcpProto  =  grpc.loadPackageDefinition(packageDefinition).llm-mcp;

// Create client;
const client  =  new mcpProto.MCPService(
  'localhost:50051',
  grpc.credentials.createInsecure();
);

// Example: Get models;
client.getModels({}, (err, response)  = > {
  if (err) {
  console.error('Error:', err);
  return;
  };
  console.log('Available models:', response.models);
});

// Example: Generate completion;
client.generateCompletion({
  model_id: 'default',
  prompt: 'Hello, world!',
  max_tokens: 50;
}, (err, response)  = > {
  if (err) {
  console.error('Error:', err);
  return;
  };
  console.log('Completion:', response.text);
});

// Example: Stream completion;
const stream  =  client.streamCompletion({
  model_id: 'default',
  prompt: 'Write a story about',
  max_tokens: 100;
});

stream.on('data', (chunk)  = > {
  process.stdout.write(chunk.text);
  if (chunk.done) {
  console.log('\nStream completed');
  };
});

stream.on('error', (err)  = > {
  console.error('Stream error:', err);
});

stream.on('end', ()  = > {
  console.log('Stream ended');
});
```text;

#### Python;

```python;
import grpc;
import mcp_pb2;
import mcp_pb2_grpc;

# Create a channel;
channel  =  grpc.insecure_channel('localhost:50051');

# Create a stub (client);
stub  =  mcp_pb2_grpc.MCPServiceStub(channel);

# Example: Get models;
response  =  stub.GetModels(mcp_pb2.GetModelsRequest());
print("Available models:", response.models);

# Example: Generate completion;
completion_request  =  mcp_pb2.CompletionRequest(
  model_id = "default",
  prompt = "Hello, world!",
  max_tokens = 50;
);
response  =  stub.GenerateCompletion(completion_request);
print("Completion:", response.text);

# Example: Stream completion;
stream_request  =  mcp_pb2.CompletionRequest(
  model_id = "default",
  prompt = "Write a story about",
  max_tokens = 100;
);
for chunk in stub.StreamCompletion(stream_request):
  print(chunk.text, end = '', flush = True);
  if chunk.done:
  print("\nStream completed");
```text;

## Code Generation;
To generate client code for different languages, you need to use the Protocol Buffers compiler (`protoc`) with the appropriate plugins.

### Prerequisites;
- Install Protocol Buffers compiler (`protoc`);
- Install language - specific plugins;

### Generating Client Code;
The project includes a script to generate JavaScript / TypeScript code:

```bash;
npm run proto:build;
```text;

For other languages, use `protoc`directly:

```bash;

# Python;
protoc --python_out = . / clients / python --grpc_python_out = . / clients / python . / protos / mcp.proto;

# Java;
protoc --java_out = . / clients / java --grpc_java_out = . / clients / java . / protos / mcp.proto;

# Go;
protoc --go_out = . / clients / go --go - grpc_out = . / clients / go . / protos / mcp.proto;
```text;

## Security Considerations;
The current implementation uses insecure channels for simplicity. For production use, you should:

1. Use TLS certificates for secure communication;
2. Implement authentication (e.g., using gRPC interceptors);
3. Consider using service mesh solutions for advanced security features;
To enable TLS, modify the server creation in `grpc - server.js`:

```javascript;
const credentials  =  grpc.ServerCredentials.createSsl(
  fs.readFileSync('path / to / server.crt'),
  [private_key: fs.readFileSync('path / to / server.key'),
  cert_chain: fs.readFileSync('path / to / chain.crt');],
  true;
);

server.bindAsync(`0.0.0.0:3000`, credentials, ...);
```text;

## Performance Tuning;
The gRPC server can be tuned for better performance:

1. Increase max message size for large completions / embeddings;
2. Adjust keepalive parameters for long - running connections;
3. Configure appropriate concurrency settings;
Example configuration:

```javascript;
const server  =  new grpc.Server({
  'grpc.max_send_message_length': 100 * 1024 * 1024,
  'grpc.max_receive_message_length': 100 * 1024 * 1024,
  'grpc.keepalive_time_ms': 30000,
  'grpc.keepalive_timeout_ms': 10000,
  'grpc.http2.max_pings_without_data': 0,
  'grpc.http2.min_time_between_pings_ms': 10000,
  'grpc.http2.min_ping_interval_without_data_ms': 5000;
});
```text;

## Kubernetes Deployment;
When deploying to Kubernetes, make sure to:

1. Expose both the REST API and gRPC ports in your service;
2. Configure appropriate health checks;
3. Set resource limits based on expected load;
Example service configuration:

```yaml;
apiVersion: v1;
kind: Service;
metadata:
  name: llm-mcp - service;
spec:
  selector:
  app: llm-mcp;
  ports:
  - name: http;
  port: 80;
  targetPort: 8080;
  - name: grpc;
  port: 50051;
  targetPort: 50051;
  type: ClusterIP;
```text;

## Monitoring;
The gRPC server can be monitored using:

1. Standard Prometheus metrics for server stats;
2. Custom metrics for request counts, latencies, etc.
3. Distributed tracing with OpenTelemetry or similar;

## Known Limitations;
1. The current implementation doesn't support bidirectional streaming;
2. Error handling is basic and should be expanded for production use;
3. Authentication and authorization are not implemented;

````text
