# MCP Configuration Documentation

This document summarizes the MCP server configuration for the bluefly.io LLM ecosystem, strictly enforcing OpenAPI 3.1.0, modular architecture, and GitLab-native CI/CD as per platform rules.

## Overview

The MCP configuration defines two MCP servers:

1. **Global MCP Server**
   - **ID:** mcp-global
   - **Description:** Central MCP server orchestrating GitLab-native workflows across the LLM stack. Handles production deployments, agent orchestration, and workflow management.
   - **Type:** central
   - **Communication:** SSE (Server-Sent Events) via URI `https://mcp.bluefly.io/api/sse`
   - **OpenAPI Spec:** `https://mcp.bluefly.io/openapi.json` (OpenAPI 3.1.0, Spectral-validated)
   - **Host:** `https://mcp.bluefly.io`
   - **Auth:** Token-based authentication using environment variable `MCP_GLOBAL_TOKEN` (JWT RS256, OAuth2, MFA)
   - **Projects:** All core LLM projects and Drupal modules (see below)
   - **Endpoints:** 
     - Agent registration and management
     - Workflow orchestration
     - Model evaluation and metrics
     - GitLab CI/CD integration
     - Health monitoring
     - LangChain tool integration
     - OpenAPI validation (Spectral, Redocly)
     - Security scanning
   - **Observability:** 
     - Prometheus metrics
     - Jaeger tracing
     - Langfuse analytics
     - Elasticsearch logging
     - Sentry error tracking
     - OpenTelemetry integration
   - **Deployment:** 
     - Kubernetes with autoscaling
     - Resource limits and quotas
     - Network policies
     - Helm chart deployment
     - GitLab CI/CD pipelines (validate, test, build, deploy, security)
   - **Security:**
     - JWT authentication, OAuth2, MFA
     - Rate limiting (100 RPM, burst 200)
     - Request validation, output sanitization
     - Audit logging
     - Encryption (AES-256-GCM, 30d key rotation)
     - Security headers, CSRF protection
   - **GitLab Integration:**
     - Webhook management
     - Pipeline orchestration
     - MR templates
     - Security scanning
     - Container registry
     - Package registry
   - **LangChain Tools:**
     - Redis memory store
     - Vector embeddings
     - Agent profiles
     - Workflow templates
     - Model registry

2. **Local MCP Server**
   - **ID:** mcp-local
   - **Description:** Local MCP for development, testing, and command-line usage. Provides a lightweight environment for local development and testing of agents and workflows.
   - **Type:** local
   - **Command:** `node examples/mcp-model-evaluator-server.js`
   - **OpenAPI Spec:** `http://localhost:5314/openapi.json` (OpenAPI 3.1.0, Spectral-validated)
   - **Host:** `http://localhost:5314`
   - **Auth:** Development token (`local-dev-token`)
   - **Projects:** Same as global server
   - **Endpoints:** Development versions of global endpoints
   - **Observability:** 
     - Local logging
     - Development metrics
     - Debug tracing
   - **Deployment:** 
     - Node.js with nodemon
     - Docker Compose for services
     - Local Kubernetes (optional)
   - **Security:**
     - Development token
     - Local rate limiting (1000 RPM, burst 2000)
     - Basic validation
   - **LangChain Tools:**
     - Local memory store
     - Development embeddings
     - Test agent profiles

## Managed Projects (as of 2025)

### LLM Projects
- llmcli (Command Layer)
- bfapi (API Gateway)
- llm-gateway-sdk (API Client)
- BFCIComponents (CI/CD Base)
- BFDOCS (Documentation)
- bfllm (Inference Engine)
- llm-mcp (MCP Server)
- acrfp (RFP Processing)
- acui (Frontend)

### Drupal Modules
- llm_platform (core, admin, insights, workflow, api, integrations)
- api_normalization
- secure_drupal (with all submodules)
- experience_builder_converter
- ddev_alternative_services
- ai_project_assistant
- llm_ui_library
- mcp_client_extras

## OpenAPI Enforcement & Validation

- **All endpoints, methods, parameters, responses, and schemas must be defined in OpenAPI 3.1.0**
- **Spectral and Redocly validation is required in CI/CD for every project**
- **Security schemes:** BearerAuth (JWT), ApiKeyAuth (for dev/local)
- **Error responses:** RFC 7807 Problem+JSON
- **Reusable schemas:** All schemas must be modular and referenced
- **Linting:** `llmcli openapi validate` and `npm run validate:openapi` are required in all pipelines
- **SDK Generation:** Client SDKs and explorers must be generated from OpenAPI

## Security & Performance Standards

- **Rate limiting:** Enforced per project and endpoint
- **Token validation:** JWT/OAuth2 for production, dev token for local
- **Request validation:** All input/output must be sanitized
- **Audit logging:** All actions logged and queryable
- **Encryption:** AES-256-GCM, 30-day key rotation
- **Security headers:** Strict-Transport-Security, X-Content-Type-Options, X-Frame-Options, X-XSS-Protection
- **CSRF protection:** Required for all state-changing endpoints
- **Performance:**
  - Cache efficiency (tags, contexts)
  - Queue processing (workers, batch size)
  - Database optimization (indexes, joins)
  - Memory management (garbage collection)
  - Response time targets (<100ms API, <2s page load)

## CI/CD & GitLab Integration

- **Pipelines:** All projects use GitLab CI/CD with validate, test, build, deploy, and security stages
- **Containerization:** Docker/Helm for all services
- **Monitoring:** Prometheus, Grafana, Jaeger, Sentry, OpenTelemetry
- **Documentation:** All modules must have README, API, architecture, and security docs
- **Branch protection:** Enforced for all MCP-managed repos

## Testing and Validation

- **Global Server:**
  - OpenAPI 3.1.0 validation (Spectral, Redocly)
  - Security scanning (SAST, dependency, container)
  - Performance/load/integration/chaos testing
- **Local Server:**
  - Unit/integration testing
  - Local OpenAPI contract testing
  - Model/agent evaluation

---

This documentation is always kept in sync with:
- `.cursor/mcp.json` (MCP configuration)
- `separate-projects-map.json` (Project map)
- Platform rules: rule-llm-project-fixes, rule-llm-project-overview
- Project-level OpenAPI specs
- GitLab CI/CD configuration