# LLM-MCP Deployment Guide

Complete guide for deploying LLM-MCP using Docker, Kubernetes, and CI/CD pipelines.

## 🚀 Quick Start

### Using GitLab CI/CD (Recommended)

```yaml
# .gitlab-ci.yml
include:
  - component: gitlab.bluefly.io/llm/bfcicomponents/components/platforms/docker/llm-mcp-docker-deploy@main
    inputs:
      service_name: "llm-mcp"
      node_version: "20"
      enable_multi_arch: true
      health_check_path: "/health"
```

Push to GitLab and the pipeline will automatically build and deploy.

### Manual Docker Deployment

```bash
# Clone and deploy
git clone https://gitlab.bluefly.io/llm/llm-mcp.git
cd llm-mcp

# Deploy with Docker Compose
docker-compose -f docker-compose.production.yml up -d

# Or use the deployment script
./scripts/deploy-docker.sh --tag latest --port 8080
```

## 📋 Deployment Options

### 1. Docker Compose (Simple)

Perfect for single-server deployments:

```bash
# Production deployment with all services
docker-compose -f docker-compose.production.yml up -d

# With monitoring stack
docker-compose -f docker-compose.production.yml --profile monitoring up -d

# Check status
docker-compose ps
docker-compose logs -f llm-mcp
```

### 2. Kubernetes (Scalable)

For production clusters:

```bash
# Create namespace
kubectl apply -f k8s/namespace.yaml

# Deploy secrets and configs
kubectl apply -f k8s/secret.yaml
kubectl apply -f k8s/configmap.yaml

# Deploy application
kubectl apply -f k8s/

# Check deployment
kubectl -n bluefly-ai get pods
kubectl -n bluefly-ai logs -f deployment/llm-mcp
```

### 3. Helm Chart (Advanced)

```bash
# Add custom values
cat > values.production.yaml <<EOF
replicaCount: 3
image:
  repository: registry.gitlab.bluefly.io/llm/llm-mcp
  tag: latest
ingress:
  enabled: true
  hostname: mcp.example.com
resources:
  limits:
    cpu: 2000m
    memory: 4Gi
EOF

# Install
helm install llm-mcp ./llm-mcp-chart -f values.production.yaml
```

## 🔧 Configuration

### Environment Variables

Copy and customize the environment file:

```bash
cp .env.example .env.production

# Edit with your values
nano .env.production
```

Key configurations:

```env
# Server
NODE_ENV=production
MCP_SERVER_PORT=8080

# Database
DATABASE_URL=postgresql://user:pass@host:5432/llm-mcp
REDIS_HOST=redis
QDRANT_HOST=qdrant

# Authentication
JWT_SECRET=your-secret-key
API_KEY=your-api-key

# LLM APIs (optional)
OPENAI_API_KEY=sk-...
ANTHROPIC_API_KEY=sk-ant-...
```

### SSL/TLS Certificates

For HTTPS, mount your certificates:

```yaml
# docker-compose.yml
volumes:
  - ./ssl/cert.pem:/etc/nginx/ssl/cert.pem:ro
  - ./ssl/key.pem:/etc/nginx/ssl/key.pem:ro
```

## 🏗️ Infrastructure Components

### Required Services

1. **Redis** - Caching and session storage
2. **Qdrant** - Vector database for embeddings
3. **PostgreSQL** - Main database (optional)

### Optional Services

1. **Nginx** - Reverse proxy with SSL termination
2. **Prometheus** - Metrics collection
3. **Grafana** - Metrics visualization

## 📊 Monitoring

### Health Checks

- `/health` - Basic health status
- `/health/liveness` - Kubernetes liveness probe
- `/health/readiness` - Kubernetes readiness probe
- `/metrics` - Prometheus metrics

### Grafana Dashboard

1. Access Grafana at `http://localhost:3000`
2. Default credentials: `admin/admin`
3. Import dashboard from `monitoring/grafana/dashboards/`

### Alerts

Configure alerts in `monitoring/alert-rules.yml`:
- Service down
- High error rate
- High response time
- Resource usage

## 🔒 Security

### Production Checklist

- [ ] Use specific image tags (not `latest`)
- [ ] Enable TLS/SSL
- [ ] Configure firewall rules
- [ ] Set strong passwords
- [ ] Enable audit logging
- [ ] Configure rate limiting
- [ ] Set up backup schedule
- [ ] Review security hardening guide

See [SECURITY_HARDENING.md](docs/SECURITY_HARDENING.md) for details.

## 💾 Backup & Recovery

### Automated Backups

```bash
# Schedule daily backups
crontab -e
0 2 * * * /app/scripts/backup.sh

# Manual backup
./scripts/backup.sh

# Backup to S3
S3_BUCKET=my-backups ./scripts/backup.sh
```

### Restore from Backup

```bash
# List available backups
ls -la /backup/

# Restore specific backup
./scripts/restore.sh /backup/llm-mcp_backup_20240115_020000.tar.gz

# Restore from S3
./scripts/restore.sh s3://my-backups/llm-mcp_backup_20240115_020000.tar.gz
```

## 🚦 Load Balancing

### Nginx Configuration

Already included in `nginx/conf.d/llm-mcp.conf`:
- Rate limiting
- Connection limits
- Health check routing
- WebSocket support

### Kubernetes Ingress

```yaml
# k8s/ingress.yaml configured with:
- TLS termination
- Rate limiting
- CORS headers
- Security policies
```

## 📈 Scaling

### Horizontal Scaling

```bash
# Docker Swarm
docker service scale llm-mcp=5

# Kubernetes
kubectl -n bluefly-ai scale deployment/llm-mcp --replicas=5

# Or use HPA (auto-scaling)
kubectl -n bluefly-ai apply -f k8s/hpa.yaml
```

### Vertical Scaling

Update resource limits:

```yaml
# docker-compose.yml
deploy:
  resources:
    limits:
      cpus: '4'
      memory: 8G

# Kubernetes
resources:
  limits:
    cpu: 4000m
    memory: 8Gi
```

## 🔧 Troubleshooting

### Common Issues

1. **Container won't start**
   ```bash
   docker logs llm-mcp
   kubectl -n bluefly-ai describe pod llm-mcp-xxx
   ```

2. **Health check failing**
   ```bash
   curl http://localhost:8080/health
   docker exec llm-mcp wget -O- http://localhost:8080/health
   ```

3. **Connection errors**
   ```bash
   # Check network
   docker network ls
   docker network inspect bluefly-network
   
   # Test connectivity
   docker exec llm-mcp ping redis
   ```

4. **Performance issues**
   ```bash
   # Check resources
   docker stats llm-mcp
   kubectl -n bluefly-ai top pod
   
   # Check metrics
   curl http://localhost:8080/metrics
   ```

### Debug Mode

```bash
# Run with debug logging
LOG_LEVEL=debug docker-compose up

# Interactive shell
docker exec -it llm-mcp sh
kubectl -n bluefly-ai exec -it deployment/llm-mcp -- sh
```

## 📞 Support

- Documentation: [/docs](./docs)
- Issues: [GitLab Issues](https://gitlab.bluefly.io/llm/llm-mcp/issues)
- Security: security@bluefly.io

## 🎯 Next Steps

1. Review [SECURITY_HARDENING.md](docs/SECURITY_HARDENING.md)
2. Set up monitoring dashboards
3. Configure automated backups
4. Plan disaster recovery
5. Schedule security audits

---

For CI/CD component development, see [BFCIComponents](https://gitlab.bluefly.io/llm/bfcicomponents).