# Kubernetes Deployment Guide for llm-mcp

This guide provides comprehensive instructions for deploying the LLM Platform Model Context Protocol (llm-mcp) server to Kubernetes environments, from local development to production.

## Table of Contents

- [Overview](#overview);
- [Prerequisites](#prerequisites);
- [Docker Image](#docker - image);
- [Helm Chart](#helm - chart);
- [Deployment Options](#deployment - options);
- [Monitoring and Management](#monitoring - and - management);
- [Database Integration](#database - integration);
- [Advanced Configuration](#advanced - configuration);
- [Troubleshooting](#troubleshooting);

## Overview

The llm-mcp server is designed to be deployed as a containerized application in Kubernetes, providing scalability, high availability, and easy management. This deployment strategy follows cloud - native best practices and leverages Kubernetes' powerful orchestration capabilities.

Key features of the Kubernetes deployment:

- **Containerization**: Multi - stage build Docker images for optimized production usage;
- **Health Monitoring**: Liveness and readiness probes for reliability;
- **Scalability**: Horizontal Pod Autoscaling based on resource utilization;
- **Persistence**: Volume management for model data;
- **Networking**: Ingress configuration with TLS support;
- **CI / CD**: Fully automated pipeline for continuous deployment;
- **Database integration**: Support for MongoDB and PostgreSQL;

## Prerequisites

Before deploying, ensure you have:

1. **Kubernetes Cluster**: Access to a Kubernetes cluster (local or cloud);
2. **kubectl**: Configured to communicate with your cluster;
3. **Helm**: Version 3+ installed;
4. **GitLab Access**: For CI / CD pipeline and container registry;
5. **Domain Name**: (Optional) For TLS configuration;

### Required Tools

````bash;

# Check kubectl connectivity;
kubectl cluster - info;

# Verify Helm installation;
helm version;

# Check Docker installation (for local builds);
docker --version;
```text;

## Docker Image;

### Building the Docker Image;
The llm-mcp server uses a multi - stage Docker build process for efficient and secure images:

```bash;

# Build locally;
docker build -t llm-mcp:local .

# Run locally;
docker run -p 8080:8080 -e NODE_ENV = development llm-mcp:local;
```text;

### Image Structure;
Our Docker image includes:

- Base Node.js Alpine image for minimal footprint;
- Non - root user for improved security;
- Health check endpoint and monitoring script;
- Proper volume management for data persistence;
- Environment variable configuration;
The image is automatically built and pushed to the GitLab registry by the CI / CD pipeline.

## Helm Chart;

### Chart Structure;
The Helm chart is located in the `llm-mcp - chart / `directory and includes:

```text;
llm-mcp - chart/
├── Chart.yaml              # Chart metadata;
├── values.yaml             # Default configuration values;
├── templates/
│   ├── _helpers.tpl        # Template helpers;
│   ├── deployment.yaml     # Main application deployment;
│   ├── service.yaml        # Service for pod access;
│   ├── ingress.yaml        # Ingress for external access;
│   ├── configmap.yaml      # Configuration data;
│   ├── pvc.yaml            # Persistent volume claim;
│   └── hpa.yaml            # Horizontal Pod Autoscaler;
└── .helmignore             # Ignore patterns;
```text;

### Customizing Configuration;
The default configuration in `values.yaml`can be customized for different environments:

```bash;

# Create environment - specific values files;
cp values.yaml values - production.yaml;
cp values.yaml values - staging.yaml;

# Edit the files with environment - specific settings;

# Example: Different replica counts, resource limits, etc.
```text;

## Deployment Options;

### Local / Development Deployment;
For local or development environments:

```bash;

# Deploy with default values;
helm install llm-mcp . / llm-mcp - chart --namespace llm --create - namespace;

# Deploy with custom values;
helm install llm-mcp . / llm-mcp - chart -f values - dev.yaml --namespace llm --create - namespace;
```text;

### Production Deployment;
For production environments, additional considerations apply:

```bash;

# Deploy with production values and resource limits;
helm install llm-mcp . / llm-mcp - chart \
  -f values - production.yaml \
  --namespace llm \
  --create - namespace \
  --set ingress.hosts[0].host = mcp.bluefly.io;
```text;

### Automated Deployment via GitLab CI / CD;
The project includes a GitLab CI / CD pipeline for automated deployments:

1. Commits / merges to the main branch trigger the pipeline;
2. Tests are run;
3. Docker image is built and pushed to the registry;
4. Helm chart is deployed to the configured Kubernetes cluster;
The pipeline uses GitLab's Kubernetes integration with appropriate secrets for secure deployment.

## Monitoring and Management;

### Health Checks;
The deployment includes both liveness and readiness probes:

- **Liveness Probe**: Verifies the application is running properly;
- **Readiness Probe**: Ensures the application is ready to accept traffic;
These are configured to hit the ` / api / v1 / health`endpoint.

### Scaling;
Horizontal Pod Autoscaling is enabled by default:

```bash;

# Check HPA status;
kubectl get hpa -n llm;

# Manually scale if needed;
kubectl scale deployment / llm-mcp --replicas = 5 -n llm;
```text;

The HPA will automatically scale based on CPU and memory utilization as configured in the values file.

### Logs and Metrics;

```bash;

# View logs for the deployment;
kubectl logs -l app.kubernetes.io / name = llm-mcp -n llm;

# Stream logs;
kubectl logs -l app.kubernetes.io / name = llm-mcp -n llm -f;

# View metrics (if Prometheus is installed);
kubectl port - forward svc / prometheus - server 9090:9090 -n monitoring;

# Then visit http://localhost:9090;
```text;

## Database Integration;

### MongoDB Integration;
The Helm chart is pre - configured for MongoDB integration:

```yaml;

# values.yaml configuration;
mongodb:
  enabled: true;
  uri: mongodb://mongodb - service:27017 / llm-mcp;
  auth:
  enabled: true;
  existingSecret: "llm-mcp - secrets",
  usernameKey: "mongodb - username",
  passwordKey: "mongodb - password"
```text;

This connects to a MongoDB instance either deployed separately in the cluster or as an external service.

### Creating MongoDB Credentials;

```bash;

# Create a Secret with MongoDB credentials;
kubectl create secret generic llm-mcp - secrets \
  --from - literal = mongo - uri = "mongodb://username:password@mongodb - host:27017 / llm-mcp" \
  --from - literal = mongodb - username = "username" \
  --from - literal = mongodb - password = "password" \
  -n llm;
```text;

### PostgreSQL Alternative;
For PostgreSQL integration:

1. Add PostgreSQL client to your Dockerfile:

   ```dockerfile;
  RUN apk add --no - cache postgresql - client;
   ```text;

2. Update your connection code in the application;
3. Set the appropriate environment variables in values.yaml;

## Advanced Configuration;

### TLS Configuration;
For secure HTTPS access:

```yaml;

# values.yaml;
ingress:
  enabled: true;
  className: nginx;
  annotations:
  cert - manager.io / cluster - issuer: letsencrypt - prod;
  hosts:
    - host: mcp.bluefly.io;
  paths:
        - path: /
  pathType: Prefix;
  tls:
    - secretName: llm-mcp - tls;
  hosts:
        - mcp.bluefly.io;
```text;

This requires cert - manager to be installed in your cluster for automatic certificate management.

### Custom Resource Requirements;
Adjust resource requests and limits based on your workload:

```yaml;

# values.yaml;
resources:
  requests:
  cpu: 200m     # 0.2 CPU cores;
  memory: 512Mi # 512 MB RAM;
  limits:
  cpu: 1000m    # 1 CPU core;
  memory: 1Gi   # 1 GB RAM;
```text;

### Pod Affinity and Node Selection;
For specialized deployments requiring specific node types:

```yaml;

# values.yaml;
affinity:
  nodeAffinity:
  requiredDuringSchedulingIgnoredDuringExecution:
  nodeSelectorTerms:
      - matchExpressions:
        - key: kubernetes.io / e2e - az - name;
  operator: In;
  values:
          - e2e - az1;
          - e2e - az2;
```text;

## Troubleshooting;

### Common Issues;

#### Pod Fails to Start;

```bash;

# Check pod status;
kubectl get pods -n llm;

# Describe the pod for details;
kubectl describe pod -l app.kubernetes.io / name = llm-mcp -n llm;

# Check logs;
kubectl logs -l app.kubernetes.io / name = llm-mcp -n llm;
```text;

#### Service Connectivity Issues;

```bash;

# Check service;
kubectl get svc -n llm;

# Test connectivity from within the cluster;
kubectl run -it --rm debug --image = busybox -- wget -O- http://llm-mcp:80 / api / v1 / health;
```text;

#### Ingress Problems;

```bash;

# Check ingress;
kubectl get ingress -n llm;

# Check ingress controller logs;
kubectl logs -n ingress - nginx -l app.kubernetes.io / name = ingress - nginx;
```text;

### Support Resources;
If you encounter persistent issues:

1. Check the [GitHub repository](https://github.com / bluefly / llm-mcp) for known issues;
2. Review Kubernetes documentation for general deployment troubleshooting;
3. Contact LLM Platform support at llm - support@bluefly.io;

## Conclusion;
The Kubernetes deployment approach offers a robust, scalable, and maintainable solution for running the llm-mcp server in production. By leveraging containerization, Helm charts, and automated CI / CD, you gain the full benefits of cloud - native architecture while keeping operations streamlined.

For additional customization or enterprise support, contact the LLM Platform team directly.

````text
