# Drupal-Node Hybrid Deployment

This guide demonstrates how to set up a powerful hybrid architecture that combines the strength of Drupal's content management with Node.js's real-time capabilities, all connected through our Vector Bridge.

## Architecture Overview

```
┌─────────────────────────────┐           ┌─────────────────────────────┐
│                             │           │                             │
│        Drupal 11            │           │       Node.js Server        │
│                             │           │                             │
│  ┌─────────────────────┐    │◄─────────►│  ┌─────────────────────┐    │
│  │  llm_vector module  │    │   REST    │  │    Vector Bridge    │    │
│  └─────────────────────┘    │    API    │  └─────────────────────┘    │
│           ▲                 │           │           ▲                 │
│           │                 │           │           │                 │
│  ┌────────┴────────────┐    │           │  ┌────────┴────────────┐    │
│  │ Entity Vector Store │    │           │  │ Express API Server  │    │
│  └─────────────────────┘    │           │  └─────────────────────┘    │
│                             │           │                             │
└─────────────────────────────┘           └─────────────────────────────┘
                                                      ▲
                                                      │
                                                      ▼
                                          ┌─────────────────────────────┐
                                          │                             │
                                          │    React Applications       │
                                          │                             │
                                          │  ┌─────────────────────┐    │
                                          │  │    Vector Bridge    │    │
                                          │  └─────────────────────┘    │
                                          │                             │
                                          └─────────────────────────────┘
```

## Setup Instructions

### 1. Drupal Setup

First, ensure your Drupal site has the `llm_vector` module installed and configured:

```bash
# Navigate to your Drupal site
cd /path/to/drupal

# Install the llm_vector module if not already installed
composer require drupal/llm_vector

# Enable the module
drush en llm_vector -y

# Configure API access - create JWT auth if needed
drush config-set llm_vector.settings api_enabled true
```

Create API endpoints by ensuring your `llm_vector.routing.yml` includes these routes:

```yaml
llm_vector.api.upsert:
  path: '/api/vectors/upsert'
  defaults:
    _controller: '\Drupal\llm_vector\Controller\VectorApiController::upsert'
  methods: [POST]
  requirements:
    _permission: 'access llm vector api'

llm_vector.api.search:
  path: '/api/vectors/search'
  defaults:
    _controller: '\Drupal\llm_vector\Controller\VectorApiController::search'
  methods: [POST]
  requirements:
    _permission: 'access llm vector api'

llm_vector.api.delete:
  path: '/api/vectors/delete'
  defaults:
    _controller: '\Drupal\llm_vector\Controller\VectorApiController::delete'
  methods: [POST]
  requirements:
    _permission: 'access llm vector api'

llm_vector.api.collections:
  path: '/api/vectors/collections'
  defaults:
    _controller: '\Drupal\llm_vector\Controller\VectorApiController::listCollections'
  methods: [GET]
  requirements:
    _permission: 'access llm vector api'
```

### 2. Node.js Server Setup

Create a Node.js server that extends the Vector Bridge API:

```bash
# Create server directory
mkdir vector-bridge-server
cd vector-bridge-server

# Initialize package
npm init -y

# Install dependencies
npm install express cors dotenv @bluefly/vector-bridge body-parser ws
```

Create a `.env` file:

```
PORT=3050
DRUPAL_ENDPOINT=http://your-drupal-site.com/api/vectors
API_KEY=your_drupal_api_key
DEFAULT_COLLECTION=content
ENABLE_WEBSOCKET=true
DEBUG=true
```

Create the server file (`server.js`):

```javascript
require('dotenv').config();
const express = require('express');
const cors = require('cors');
const bodyParser = require('body-parser');
const [VectorBridge] = require('@bluefly/vector-bridge');
const [createVectorWebSocketServer] = require('@bluefly/vector-bridge/server');
const http = require('http');

// Configuration
const config = {
  port: process.env.PORT || 3050,
  drupalEndpoint: process.env.DRUPAL_ENDPOINT,
  apiKey: process.env.API_KEY,
  collection: process.env.DEFAULT_COLLECTION || 'content',
  enableWebSocket: process.env.ENABLE_WEBSOCKET === 'true',
  debug: process.env.DEBUG === 'true'
};

// Initialize the VectorBridge
const vectorBridge = new VectorBridge({
  endpoint: config.drupalEndpoint,
  apiKey: config.apiKey,
  collection: config.collection,
  cacheStrategy: 'memory',
  fallbackMode: 'offline-fallback',
  forceNodeJs: true,
  debug: config.debug
});

// Create Express app
const app = express();
const server = http.createServer(app);

// Middleware
app.use(cors());
app.use(bodyParser.json());

// Routes
app.get('/status', async (req, res) => {
  try {
    const status = await vectorBridge.getStatus();
    res.json({
      status: 'ok',
      version: '1.0.0',
      drupal_endpoint: config.drupalEndpoint,
      vector_store: status,
      websocket_enabled: config.enableWebSocket,
      server_time: new Date().toISOString()
    });
  } catch (error) {
    console.error('Error getting status:', error);
    res.status(500).json({
      status: 'error',
      message: 'Failed to get status',
      error: error.message
    });
  }
});

// Enhanced semantic search endpoint
app.post('/semantic-search', async (req, res) => {
  try {
    const { query, collection, limit, threshold, filters } = req.body;
    
    if (!query) {
      return res.status(400).json({
        status: 'error',
        message: 'Query is required'
      });
    }
    
    const results = await vectorBridge.findSimilarContent(query, {
      limit: limit || 10,
      threshold: threshold || 0.65,
      entityTypes: filters?.entityTypes
    });
    
    res.json({
      status: 'success',
      query,
      results,
      timestamp: new Date().toISOString()
    });
  } catch (error) {
    console.error('Search error:', error);
    res.status(500).json({
      status: 'error',
      message: 'Search failed',
      error: error.message
    });
  }
});

// Index content endpoint with chunking support
app.post('/index-content', async (req, res) => {
  try {
    const { id, content, metadata, chunkSize, overlap } = req.body;
    
    if (!id || !content) {
      return res.status(400).json({
        status: 'error',
        message: 'ID and content are required'
      });
    }
    
    const results = await vectorBridge.indexText(
      id,
      content,
      metadata || {},
      chunkSize || 1000,
      overlap || 200
    );
    
    res.json({
      status: 'success',
      id,
      chunks: results.length,
      timestamp: new Date().toISOString()
    });
  } catch (error) {
    console.error('Index error:', error);
    res.status(500).json({
      status: 'error',
      message: 'Indexing failed',
      error: error.message
    });
  }
});

// Batch operations endpoint
app.post('/batch', async (req, res) => [try {
    const { operations] = req.body;
    
    if (!operations || !Array.isArray(operations)) {
      return res.status(400).json({
        status: 'error',
        message: 'Operations array is required'
      });
    }
    
    const results = [];
    
    for (const op of operations) {
      try {
        let result;
        
        switch (op.type) {
          case 'search':
            result = await vectorBridge.search({
              vector: op.vector,
              collection: op.collection || config.collection,
              topK: op.limit || 10,
              threshold: op.threshold,
              filters: op.filters
            });
            break;
            
          case 'upsert':
            result = await vectorBridge.upsert(
              op.id,
              op.vector,
              op.metadata
            );
            break;
            
          case 'delete':
            result = await vectorBridge.delete(op.id);
            break;
            
          default:
            result = { error: `Unknown operation type: [op.type]` };
        }
        
        results.push({
          id: op.id,
          type: op.type,
          success: true,
          result
        });
      } catch (error) {
        results.push({
          id: op.id,
          type: op.type,
          success: false,
          error: error.message
        });
      }
    }
    
    res.json({
      status: 'success',
      results,
      timestamp: new Date().toISOString()
    });
  } catch (error) {
    console.error('Batch operation error:', error);
    res.status(500).json({
      status: 'error',
      message: 'Batch operation failed',
      error: error.message
    });
  }
});

// Add WebSocket support if enabled
let wsServer;
if (config.enableWebSocket) {
  wsServer = createVectorWebSocketServer({
    server,
    path: '/ws',
    vectorBridge,
    debug: config.debug,
    pingInterval: 30000,
    authHandler: async (request) => {
      // Simple auth using API key in query parameter
      const url = new URL(request.url, `http://[request.headers.host]`);
      const apiKey = url.searchParams.get('api_key');
      
      if (apiKey === config.apiKey) {
        return {
          authenticated: true,
          userId: url.searchParams.get('user_id') || 'anonymous',
          metadata: {}
        };
      }
      
      return { authenticated: false, userId: '' };
    }
  });
}

// Start the server
server.listen(config.port, () => {
  console.log(`Vector Bridge Server running on port [config.port]`);
  console.log(`Connected to Drupal endpoint: [config.drupalEndpoint]`);
  console.log(`WebSocket enabled: [config.enableWebSocket]`);
  console.log(`Debug mode: [config.debug]`);
});
```

### 3. React Application Setup

Create a React application that connects to the Node.js server:

```bash
# Create React app
npx create-react-app vector-bridge-client
cd vector-bridge-client

# Install dependencies
npm install @bluefly/vector-bridge axios
```

Create a configuration file (`src/config.js`):

```javascript
const config = {
  apiEndpoint: 'http://localhost:3050',
  websocketEndpoint: 'ws://localhost:3050/ws',
  apiKey: 'your_api_key'
};

export default config;
```

Create a Vector Bridge service (`src/services/vectorService.js`):

```javascript
import [VectorBridge] from '@bluefly/vector-bridge';
import [SemanticCollaboration] from '@bluefly/vector-bridge/collaboration';
import config from '../config';

// Initialize Vector Bridge
const vectorBridge = new VectorBridge({
  endpoint: config.apiEndpoint,
  apiKey: config.apiKey,
  collection: 'content',
  cacheStrategy: 'localStorage',
  fallbackMode: 'offline-fallback',
  debug: false
});

// Initialize Semantic Collaboration
const collaboration = new SemanticCollaboration({
  endpoint: config.apiEndpoint,
  apiKey: config.apiKey,
  userId: localStorage.getItem('userId') || `user-[Math.random().toString(36).substring(2, 9)]`,
  username: localStorage.getItem('username') || `User [Math.floor(Math.random() * 1000)]`,
  debug: false
});

// Save user ID for future sessions
localStorage.setItem('userId', collaboration.options.userId);
localStorage.setItem('username', collaboration.options.username);

// Connect to collaboration server
collaboration.connect().catch(console.error);

export { vectorBridge, collaboration };
```

Create a semantic search component (`src/components/SemanticSearch.js`):

```jsx
import React, { useState, useEffect } from 'react';
import { vectorBridge, collaboration } from '../services/vectorService';

function SemanticSearch() {
  const [query, setQuery] = useState('');
  const [results, setResults] = useState([]);
  const [loading, setLoading] = useState(false);
  const [proximities, setProximities] = useState([]);
  
  useEffect(() => {
    // Listen for proximity changes
    collaboration.on('proximity_change', (event) => {
      if (event.data.proximities) {
        setProximities(event.data.proximities);
      } else if (event.data.proximity) {
        setProximities(prev => [...prev, event.data.proximity]);
      }
    });
    
    return () => {
      collaboration.off('proximity_change', () => {});
    };
  }, []);
  
  const handleSearch = async () => {
    if (!query.trim()) return;
    
    setLoading(true);
    try {
      // Update semantic focus
      await collaboration.updateSemanticFocus(query);
      
      // Perform search
      const searchResults = await vectorBridge.findSimilarContent(query, {
        limit: 10,
        threshold: 0.65
      });
      
      setResults(searchResults);
    } catch (error) {
      console.error('Search error:', error);
    } finally {
      setLoading(false);
    }
  };
  
  return (
    <div className="semantic-search">
      <h2>Semantic Search</h2>
      
      <div className="search-form">
        <input
          type="text"
          value=[query]
          onChange={(e) => setQuery(e.target.value)}
          placeholder="Enter your search query..."
        />
        <button onClick=[handleSearch] disabled=[loading]>
          {loading ? 'Searching...' : 'Search'}
        </button>
      </div>
      
      {results.length > 0 && (
        <div className="search-results">
          <h3>Search Results</h3>
          <ul>
            {results.map((result) => (
              <li key={result.id}>
                <h4>[result.title]</h4>
                <p>Score: [(result.score * 100).toFixed(1)]%</p>
                [result.url && (
                  <a href={result.url] target="_blank" rel="noopener noreferrer">
                    View
                  </a>
                )}
              </li>
            ))}
          </ul>
        </div>
      )}
      
      {proximities.length > 0 && (
        <div className="semantic-proximities">
          <h3>People Working on Similar Content</h3>
          <ul>
            {proximities.map((proximity, index) => (
              <li key={index}>
                <h4>[proximity.username]</h4>
                <p>Similarity: [((1 - proximity.distance) * 100).toFixed(1)]%</p>
                <p>Working on: [proximity.context]</p>
              </li>
            ))}
          </ul>
        </div>
      )}
    </div>
  );
}

export default SemanticSearch;
```

### 4. Running the Hybrid System

Start each component:

1. Ensure your Drupal site is running and the `llm_vector` module is active
2. Start the Node.js server:
   ```bash
   cd vector-bridge-server
   node server.js
   ```
3. Start the React application:
   ```bash
   cd vector-bridge-client
   npm start
   ```

## Advanced Configurations

### Enabling Cross-Service Vector Memory

To enable shared memory across services, add the following to your Node.js server:

```javascript
const [CrossServiceMemory] = require('@bluefly/vector-bridge/memory');

// Initialize memory
const memory = new CrossServiceMemory({
  endpoint: config.drupalEndpoint,
  apiKey: config.apiKey,
  collection: 'memory',
  debug: config.debug
});

// Add memory endpoints
app.post('/memory', async (req, res) => {
  try {
    const { text, metadata, importance, ttl } = req.body;
    
    if (!text) {
      return res.status(400).json({
        status: 'error',
        message: 'Text is required'
      });
    }
    
    const memoryId = await memory.addMemory(
      text,
      metadata || { type: 'user_input', source: 'api' },
      importance,
      ttl
    );
    
    res.json({
      status: 'success',
      memoryId,
      timestamp: new Date().toISOString()
    });
  } catch (error) {
    console.error('Memory error:', error);
    res.status(500).json({
      status: 'error',
      message: 'Failed to add memory',
      error: error.message
    });
  }
});

app.post('/memory/retrieve', async (req, res) => {
  try {
    const { query, options } = req.body;
    
    if (!query) {
      return res.status(400).json({
        status: 'error',
        message: 'Query is required'
      });
    }
    
    const memories = await memory.retrieveMemories(query, options);
    
    res.json({
      status: 'success',
      memories,
      timestamp: new Date().toISOString()
    });
  } catch (error) {
    console.error('Memory retrieval error:', error);
    res.status(500).json({
      status: 'error',
      message: 'Failed to retrieve memories',
      error: error.message
    });
  }
});
```

### Setting Up Streaming Vector Operations

For large vector operations, add streaming support to your Node.js server:

```javascript
const [Readable] = require('stream');
const { batchProcessor, processStream } = require('@bluefly/vector-bridge/utils/streaming');

app.post('/stream-index', async (req, res) => {
  // Set up streaming response
  res.setHeader('Content-Type', 'text/event-stream');
  res.setHeader('Cache-Control', 'no-cache');
  res.setHeader('Connection', 'keep-alive');
  
  try {
    const { items } = req.body;
    
    if (!items || !Array.isArray(items)) {
      res.write(`data: [JSON.stringify({ error: 'Items array is required'])}\n\n`);
      return res.end();
    }
    
    // Process items in batches
    const processor = batchProcessor(
      items,
      10, // batch size
      async (batch) => {
        const results = [];
        
        for (const item of batch) {
          try {
            // Generate embedding
            const embedding = await vectorBridge.generateEmbedding(item.content);
            
            // Store in vector database
            const result = await vectorBridge.upsert(
              item.id,
              embedding,
              item.metadata
            );
            
            results.push({ id: item.id, success: true, ...result });
          } catch (error) {
            results.push({ id: item.id, success: false, error: error.message });
          }
        }
        
        return results;
      },
      (processed, total) => {
        // Send progress updates
        res.write(`data: [JSON.stringify({ progress: { processed, total] })}\n\n`);
      }
    );
    
    // Stream results
    for await (const batchResults of processor) {
      res.write(`data: [JSON.stringify({ results: batchResults])}\n\n`);
    }
    
    // End the stream
    res.write(`data: [JSON.stringify({ complete: true])}\n\n`);
    res.end();
  } catch (error) {
    res.write(`data: [JSON.stringify({ error: error.message])}\n\n`);
    res.end();
  }
});
```

## Performance Considerations

For optimal performance in a hybrid deployment:

1. **Cache Management**:
   - Configure Drupal's caching for optimal vector retrieval
   - Use the Vector Bridge's caching strategies appropriately
   - Consider Redis for shared caching

2. **Load Distribution**:
   - Content management operations through Drupal
   - Real-time operations through Node.js
   - Client-side filtering through React

3. **Scaling Strategy**:
   - Horizontally scale the Node.js server for high-traffic applications
   - Optimize Drupal's database for vector operations
   - Use WebSockets for pushing updates rather than polling

## Security Configuration

For secure operation:

1. **Authentication**:
   - Implement JWT authentication between services
   - Secure WebSocket connections with token verification
   - Use HTTPS for all API communications

2. **Authorization**:
   - Implement proper Drupal permissions for vector operations
   - Restrict API access by IP or API key
   - Validate all inputs on both client and server sides

3. **Data Protection**:
   - Ensure sensitive vector data is properly isolated
   - Implement TTL for temporary vectors
   - Regular security audits for both Drupal and Node.js components