# LLM-MCP (LLM Platform Model Context Protocol)

LLM-MCP is the orchestration and agent registry server for the Acme AI Platform. It provides a unified protocol for tool registration, vector operations, and model management. This documentation hub contains comprehensive guides, references, and examples for working with the LLM-MCP system.

## Table of Contents

- [Documentation Standards](#documentation-standards)
- [Quick Start](#quick-start)
- [Architecture](#architecture)
- [Documentation Sections](#documentation-sections)
- [Contributing](#contributing)
- [References](#references)

## Documentation Standards

- [Documentation Standards](DOCUMENTATION_STANDARDS.md) - Style guide and standards for all documentation
- [Documentation Cross References](DOCUMENTATION_CROSS_REFERENCES.md) - Mapping of relationships between docs
- [Documentation Progress Summary](DOCUMENTATION_PROGRESS_SUMMARY.md) - Current progress and status

## Quick Start

Get started with LLM-MCP in minutes:

1. **Install dependencies:**
   ```sh
   npm install
   ```
2. **Start the server:**
   ```sh
   npm run start
   ```
3. **Access the OpenAPI explorer:**
   - [http://localhost:8080/api/docs](http://localhost:8080/api/docs)

## Architecture

LLM-MCP integrates with other platform components through REST APIs and standard protocols:

```mermaid
graph TD;
  A[Drupal llm_platform_mcp UI] -->|OpenAPI, REST| B(llm-mcp Server);
  B -->|OpenAPI, REST| C(GitLab CI/CD);
  C -->|Status API| D[Drupal UI: Deploy Status];
```

Key architectural components:

- **Agent actions** in Drupal trigger llm-mcp endpoints and GitLab pipelines
- **OpenAPI contracts** are validated in CI/CD
- **Pipeline/job status** is surfaced in the UI

## Documentation Sections

The documentation is organized into these main sections:

### API & Usage

- [API Documentation](api/index.md) - API references and specifications
- [Examples](examples/index.md) - Usage scenarios and code examples
- [Basic Usage Examples](examples/basic-usage.md) - Comprehensive code examples for common operations
- [Integration](integration/index.md) - Integration with other platform components

### Architecture & Design

- [Architecture Documentation](architecture/index.md) - Architecture overview and design
- [MCP Configuration](architecture/mcp-config.md) - MCP server configuration
- [Advanced Configuration](configuration/index.md) - Comprehensive configuration documentation
- [Performance Optimization](performance/index.md) - Performance tuning and optimization guides
- [Security Best Practices](security/index.md) - Security hardening and best practices
- [Custom Tool Development](tools/index.md) - Developing and deploying custom tools

### Guides & Setup

- [Getting Started Guide](guides/getting-started.md) - Comprehensive step-by-step guide for new users
- [Tool Registration Guide](guides/tool-registration.md) - Detailed guide for registering and using tools
- [Vector Operations Guide](guides/vector-operations.md) - Detailed guide for vector storage and retrieval
- [AI Streaming Integration](guides/ai-streaming-integration.md) - Guide for real-time AI response streaming
- [AI Streaming & Vector Integration](AI_STREAMING_VECTOR_INTEGRATION.md) - Comprehensive guide for using AI streaming, vector operations, and function calling
- [Function Calling Integration](guides/function-calling-integration.md) - Guide for integrating function calling capabilities
- [Setup Guide](guides/setup.md) - Detailed installation and setup
- [Configuration Guide](guides/configuration.md) - Configuration options
- [Cursor Directory Integration](guides/cursor-directory-integration.md) - Integration with Cursor Directory

### Operations & Deployment

- [Deployment Documentation](deployment/index.md) - Deployment instructions and environments
- [Monitoring and Logging](monitoring/index.md) - Monitoring, logging, and observability guides
- [Troubleshooting](troubleshooting/index.md) - Troubleshooting guides

### Reference

- [Reference Documentation](reference/index.md) - Reference materials and specifications
- [Changelog](reference/changelog.md) - Release notes and version history

## Common Use Cases

LLM-MCP supports these common use cases:

- **Tool Registration and Discovery**: Register custom tools and make them available for AI agents
- **Vector Operations**: Store, retrieve, and search vector embeddings for semantic search
- **AI Streaming**: Real-time AI response streaming with metrics tracking
- **Function Calling**: Register and execute functions with AI models
- **Retrieval Augmented Generation (RAG)**: Implement RAG workflows with vector operations
- **Multi-Provider Support**: Abstract multiple AI providers behind a unified interface
- **Agent Orchestration**: Coordinate multiple AI agents and tools
- **GitLab CI/CD Integration**: Automate deployment and validation through pipelines
- **Drupal Integration**: Connect LLM-MCP with Drupal-based UIs

## Contributing

To contribute to LLM-MCP documentation:

- See [README.md](README.md) for contribution guidelines
- Follow the [Documentation Standards](DOCUMENTATION_STANDARDS.md)
- See [changelog.md](reference/changelog.md) for release notes

## References

- [OpenAPI Spec](../openapi.yaml) - OpenAPI specification for LLM-MCP endpoints
- [GitLab API Docs](https://docs.gitlab.bluefly..io/ee/api/pipelines.html) - GitLab API documentation
- [Spectral Linter](https://github.com/stoplightio/spectral) - OpenAPI linting tool
- [Redocly](https://redocly.com/docs/cli/) - OpenAPI documentation generation

---

**For questions or contributions, see the Acme AI Platform documentation hub or contact the platform team.**- [DIRECT_MCP_TOOL_EXECUTION.md](direct_mcp_tool_execution.md)
- [IMPLEMENTATION_PROGRESS.md](implementation_progress.md)
- [IMPLEMENTATION_SUMMARY.md](implementation_summary.md)
- [SHARED_TYPES_IMPLEMENTATION_SUMMARY.md](shared_types_implementation_summary.md)
- [TYPESCRIPT_IMPLEMENTATION_SUMMARY.md](typescript_implementation_summary.md)
