# LLM-MCP and MCP Client Extras Integration

This document describes the integration between the LLM-MCP service and the Drupal MCP Client Extras module, including architecture, API endpoints, testing strategy, and deployment considerations.

## Overview

The integration between LLM-MCP and MCP Client Extras enables seamless communication between the Node.js MCP services and the Drupal-based CMS, providing a robust platform for AI tool registration, execution, and monitoring within the Drupal ecosystem.

### Key Features

- Bidirectional tool registration and discovery
- Tool execution proxying
- Context synchronization and propagation
- Federation of MCP servers for multi-system orchestration
- Health monitoring and reporting
- Event notification and logging

## Architecture

The integration follows a client-server architecture with both components acting as both client and server depending on the operation:

```
┌────────────────┐                           ┌────────────────┐
│                │                           │                │
│                │◄──── Tool Registration ───┤                │
│                │                           │                │
│                │                           │                │
│     LLM-MCP      │───── Tool Discovery ─────►│  MCP Client    │
│    Service     │                           │    Extras      │
│                │◄─── Tool Execution ───────┤    Module      │
│                │                           │                │
│                │                           │                │
│                │────── Health Status ─────►│                │
│                │                           │                │
└────────────────┘                           └────────────────┘
       ▲                                            ▲
       │                                            │
       │                                            │
       │                                            │
       ▼                                            ▼
┌────────────────┐                           ┌────────────────┐
│                │                           │                │
│   LLM Models   │                           │    Drupal      │
│  and Services  │                           │    Content     │
│                │                           │                │
└────────────────┘                           └────────────────┘
```

### Components

1. **LLM-MCP Service**
   - Core MCP protocol implementation
   - Tool registry and execution
   - Connection pooling and management
   - Health monitoring

2. **MCP Client Extras Module**
   - Drupal module for MCP integration
   - Admin interface for MCP configuration
   - Server management
   - Workflow automation

3. **McpClientExtrasAdapter**
   - Integration adapter in LLM-MCP
   - Handles API communication
   - Provides error handling and retries
   - Manages server registration

4. **DrupalContextBridge**
   - Handles context synchronization
   - Provides fallback mechanisms
   - Manages Drupal-specific context enrichment

## API Endpoints

The MCP Client Extras module exposes the following API endpoints:

| Endpoint | Method | Description |
|----------|--------|-------------|
| `/api/mcp/health` | GET | Health check endpoint |
| `/api/mcp/status` | GET | Get current status of MCP Client Extras |
| `/api/mcp/servers/register` | POST | Register a new MCP server |
| `/api/mcp/tools/register` | POST | Register a new tool |
| `/api/mcp/tools/discover` | GET | Discover available tools |
| `/api/mcp/tools/execute` | POST | Execute a tool |
| `/api/mcp/context/sync` | POST | Synchronize context |
| `/api/mcp/federation/status` | GET | Get federation status |
| `/api/mcp/notifications/tool-execution` | POST | Notify tool execution |

### Example: Tool Registration

```json
// POST /api/mcp/tools/register
{
  "name": "summarize",
  "description": "Summarize text content",
  "version": "1.0.0",
  "category": "content",
  "inputSchema": {
    "type": "object",
    "properties": {
      "text": { "type": "string" },
      "max_length": { "type": "integer" }
    },
    "required": ["text"]
  },
  "metadata": {
    "tags": ["content", "nlp"],
    "permissions": ["use_summarize_tool"],
    "author": "LLM-MCP Team"
  }
}
```

### Example: Tool Execution

```json
// POST /api/mcp/tools/execute
{
  "tool_name": "summarize",
  "parameters": {
    "text": "Lorem ipsum dolor sit amet, consectetur adipiscing elit...",
    "max_length": 100
  },
  "context": {
    "correlation_id": "request-123",
    "user_id": "user-456",
    "session_id": "session-789"
  },
  "execution_options": {
    "timeout": 5000,
    "priority": "normal",
    "cache": true
  }
}
```

## Integration Implementation

The integration is implemented in the `McpClientExtrasAdapter` class in LLM-MCP, which handles all communication with the MCP Client Extras module.

Key methods include:

- `initialize()`: Initialize the adapter and register the LLM-MCP server
- `registerTool()`: Register a tool with MCP Client Extras
- `discoverTools()`: Discover tools from MCP Client Extras
- `executeTool()`: Execute a tool through MCP Client Extras
- `syncContext()`: Synchronize context with MCP Client Extras
- `getFederationStatus()`: Get federation status from MCP Client Extras
- `notifyToolExecution()`: Notify MCP Client Extras of tool execution
- `healthCheck()`: Check health of MCP Client Extras

## Testing

The integration includes comprehensive testing to ensure reliability:

1. **Unit Tests**
   - Test adapter methods with mocked HTTP client
   - Verify error handling and edge cases
   - Test context synchronization fallback

2. **Integration Tests**
   - Test with mock MCP Client Extras server
   - Verify API contract compliance
   - Test end-to-end workflows

3. **API Integration Tests**
   - Test with real MCP Client Extras module (optional)
   - Verify real-world integration

### Running Tests

To run the integration tests with the mock server:

```bash
./scripts/run-integration-tests.sh
```

To run with the real MCP Client Extras module:

```bash
./scripts/run-integration-tests.sh --api
```

## Configuration

The integration is configured through the `McpClientExtrasAdapter` constructor:

```typescript
const adapter = new McpClientExtrasAdapter({
  drupalBaseUrl: 'http://localhost:8080',
  apiEndpoint: '/api/mcp',
  apiKey: 'your-api-key',
  timeout: 5000,
  retryAttempts: 3,
  cacheTtl: 300
}, drupalBridge);
```

### Configuration Parameters

| Parameter | Description | Default |
|-----------|-------------|---------|
| `drupalBaseUrl` | Base URL of the Drupal site | Required |
| `apiEndpoint` | MCP API endpoint path | `/api/mcp` |
| `apiKey` | API key for authentication | Required |
| `timeout` | Request timeout in milliseconds | 5000 |
| `retryAttempts` | Number of retry attempts | 3 |
| `cacheTtl` | Cache TTL in seconds | 300 |
| `webhookUrl` | Webhook URL for notifications | Optional |

## Deployment Considerations

When deploying the integrated solution, consider the following:

1. **Network Connectivity**
   - Ensure LLM-MCP can reach Drupal and vice versa
   - Configure firewall rules appropriately
   - Use TLS for secure communication

2. **Authentication**
   - Generate and securely store API keys
   - Use OAuth2 for production environments
   - Implement IP-based restrictions if needed

3. **Scaling**
   - Configure connection pooling for high load
   - Implement rate limiting for tool execution
   - Monitor resource usage on both sides

4. **Monitoring**
   - Monitor health checks from both services
   - Set up alerts for connectivity issues
   - Log all integration events for debugging

5. **Fallback Mechanisms**
   - Implement circuit breakers for temporary failures
   - Cache tool registry for offline operation
   - Use local execution for critical tools

## Troubleshooting

Common issues and solutions:

### Connection Issues

If you encounter connection issues between LLM-MCP and MCP Client Extras:

1. Verify network connectivity with `curl` or similar tools
2. Check that the Drupal server is running and accessible
3. Verify API endpoint configuration
4. Check logs for connection errors

### Authentication Failures

If authentication fails:

1. Verify API key is correct
2. Check that the API key is active in MCP Client Extras
3. Verify permissions associated with the API key
4. Check for IP restrictions in the Drupal site

### Tool Execution Errors

If tool execution fails:

1. Verify the tool is registered in both systems
2. Check permissions for the executing user
3. Verify tool parameters match the expected schema
4. Look for execution timeout issues

## Performance Optimization

To optimize performance of the integration:

1. **Connection Pooling**
   - Enable connection pooling in the adapter
   - Configure appropriate pool size based on load
   - Set reasonable connection TTL

2. **Caching**
   - Cache tool discovery results
   - Enable result caching for idempotent tools
   - Use Redis or similar for shared caching

3. **Batch Operations**
   - Use batch operations for multiple tool registrations
   - Combine context synchronization when possible
   - Implement bulk notifications for high-volume events

## Security Considerations

To ensure secure integration:

1. **Authentication**
   - Use strong API keys or OAuth2 tokens
   - Rotate keys regularly
   - Implement IP-based restrictions

2. **Data Protection**
   - Encrypt sensitive data in transit with TLS
   - Sanitize inputs to prevent injection attacks
   - Implement context isolation between requests

3. **Access Control**
   - Enforce least privilege principle
   - Implement fine-grained permissions for tools
   - Audit all cross-system operations

## Future Enhancements

Planned enhancements for the integration:

1. **Bidirectional Federation**
   - Enhanced discovery across multiple MCP servers
   - Dynamic routing based on server capabilities
   - Load balancing across federated servers

2. **Advanced Context Management**
   - Hierarchical context propagation
   - Context versioning and conflict resolution
   - Selective context synchronization

3. **Performance Improvements**
   - Binary protocol for high-performance operations
   - Streaming results for long-running tools
   - Predictive prefetching for common operations

## Conclusion

The integration between LLM-MCP and MCP Client Extras provides a robust foundation for AI-powered applications in Drupal. By following the guidelines in this document, you can ensure a reliable, secure, and high-performance integration between these systems.