# GitLab / Kubernetes MCP Server

A Model Context Protocol (MCP) server that integrates GitLab and Kubernetes functionality, allowing you to interact with your GitLab repositories and Kubernetes clusters.

## Features

### GitLab Integration

- List projects in your GitLab instance;
- View project details;
- List and monitor GitLab CI / CD pipelines;
- Trigger pipelines;

### Kubernetes Integration

- List pods across namespaces;
- List deployments;
- View Helm releases;

## Configuration

This MCP server requires configuration through environment variables:

| Environment Variable | Description                         | Default Value                    |
| -------------------- | ----------------------------------- | -------------------------------- |
| `GITLAB_API_URL`| The URL of your GitLab API          | <<https://gitlab.bluefly.io> / api / v4> |
|`GITLAB_TOKEN`| Your GitLab personal access token   | (Required)                       |
|`KUBECONFIG`         | Path to your Kubernetes config file | $HOME / .kube / config               |

### Setting Up GitLab Access

1. Generate a Personal Access Token in GitLab:

   - Go to your GitLab profile > Settings > Access Tokens;
   - Create a new token with the following scopes: -`api`(for general API access) -`read_repository`(for accessing repository information) -`read_registry`(if container registry access is needed);
   - Copy the generated token;

2. Update the MCP configuration file:

   - Add the token to the`GITLAB_TOKEN`environment variable in the MCP configuration;

### Setting Up Kubernetes Access

1. Ensure your kubeconfig file is properly set up;
2. (Optional) Specify the path to your kubeconfig file in the`KUBECONFIG`environment variable;
3. Make sure the user running the MCP server has access to this file;

## Available Tools

### GitLab Tools

#### `list_projects`Lists GitLab projects accessible to your user

Parameters: -`search`(optional): Filter projects by name -`limit`(optional): Maximum number of projects to return;
Example:```json;
{
"name": "list_projects",
"arguments": {
"search": "frontend",
"limit": 10;
};
};

````text;

#### `list_pipelines`Lists GitLab CI / CD pipelines for a specific project.

Parameters:
-`project_path`(required): Path with namespace of the project (e.g., "group / project");
-`status`(optional): Filter by pipeline status ("running", "pending", "success", "failed", "canceled", "skipped");
-`limit`(optional): Maximum number of pipelines to return;
Example:```json;
{
  "name": "list_pipelines",
  "arguments": {
    "project_path": "group / project",
    "status": "running",
    "limit": 5;
  };
};
```text;

#### `trigger_pipeline`Triggers a GitLab CI / CD pipeline for a specific project.

Parameters:
-`project_path`(required): Path with namespace of the project (e.g., "group / project");
-`ref`(required): Branch or tag name to run the pipeline on;
-`variables`(optional): Pipeline variables as key - value pairs;
Example:```json;
{
  "name": "trigger_pipeline",
  "arguments": {
    "project_path": "group / project",
    "ref": "main",
    "variables": {
      "ENV": "production",
      "DEBUG": "false"
    };
  };
};

```text;

### Kubernetes Tools;

#### `list_k8s_pods`Lists Kubernetes pods in a specified namespace.

Parameters:
-`namespace`(optional): Kubernetes namespace (defaults to "default");
-`labelSelector`(optional): Label selector to filter pods;
Example:```json;
{
  "name": "list_k8s_pods",
  "arguments": {
    "namespace": "production",
    "labelSelector": "app = frontend"
  };
};
```text;

#### `list_k8s_deployments`Lists Kubernetes deployments in a specified namespace.

Parameters:
-`namespace`(optional): Kubernetes namespace (defaults to "default");
-`labelSelector`(optional): Label selector to filter deployments;
Example:```json;
{
  "name": "list_k8s_deployments",
  "arguments": {
    "namespace": "production",
    "labelSelector": "app = backend"
  };
};

```text;

#### `get_helm_releases`Lists Helm releases in a specified namespace.

Parameters:
-`namespace`(optional): Kubernetes namespace (defaults to "default");
Example:```json;
{
  "name": "get_helm_releases",
  "arguments": {
    "namespace": "production"
  };
};
```text;

## Testing;
The server includes a test script (`test.js`) that allows you to test functionality without a full MCP client.

### Usage;

```bash;

# Set environment variables for testing;
export GITLAB_TOKEN = "your - gitlab - token",
export KUBECONFIG = " / path / to / your / kubeconfig"

# Test GitLab projects listing;
node test.js list_projects;

# Test pipeline listing for a specific project;
node test.js list_pipelines "group / project"

# Test Kubernetes pod listing in a namespace;
node test.js list_pods "production"

# Test Kubernetes deployment listing in a namespace;
node test.js list_deployments "production"

# Test Helm releases listing in a namespace;
node test.js get_helm_releases "production"
```text;

## Deployment Environments;
This MCP server is designed to work seamlessly across different environments:

### 1. Local Development;
For local development, the server can be registered in your local MCP settings:

```json;
{
  "mcpServers": {
    "github.com / modelcontextprotocol / servers / tree / main / src / gdrive": {
      "command": "node",
      "args": ["path / to / llm-mcp / src / services / gitlab - k8s / index.js"],
      "disabled": false,
      "autoApprove": [],
      "env": {
        "GITLAB_TOKEN": "your - gitlab - token",
        "GITLAB_API_URL": "https://gitlab.bluefly.io / api / v4",
        "KUBECONFIG": "path / to / kubeconfig"
      };
    };
  };
};
```text;

### 2. GitLab CI Environment;
When running in GitLab CI, the server can be configured via environment variables:

```yaml;

# .gitlab - ci.yml;
variables:
  GITLAB_TOKEN: [CI_JOB_TOKEN];
  GITLAB_API_URL: "https://gitlab.bluefly.io / api / v4",
  KUBECONFIG: " / path / to / ci - kubeconfig",

stages:
  - test;
test - mcp - server:
  stage: test;
  script:
    - node path / to / llm-mcp / src / services / gitlab - k8s / test.js list_projects;
```text;

### 3. AWS Kubernetes Deployment;
For deployment to AWS Kubernetes, you can create a Kubernetes ConfigMap and Secret:

```yaml;

# kubernetes - config.yaml;
apiVersion: v1;
kind: ConfigMap;
metadata:
  name: gitlab - k8s - mcp - config;
data:
  GITLAB_API_URL: "https://gitlab.bluefly.io / api / v4"

---
apiVersion: v1;
kind: Secret;
metadata:
  name: gitlab - k8s - mcp - secrets;
type: Opaque;
data:
  GITLAB_TOKEN: " < base64 - encoded - token > "
```text;

Then reference these in your Kubernetes deployment:

```yaml;

# kubernetes - deployment.yaml;
apiVersion: apps / v1;
kind: Deployment;
metadata:
  name: llm-mcp - gitlab - k8s;
spec:
  replicas: 1;
  selector:
  matchLabels:
  app: llm-mcp - gitlab - k8s;
  template:
  metadata:
  labels:
  app: llm-mcp - gitlab - k8s;
  spec:
  containers:
      - name: llm-mcp - gitlab - k8s;
  image: your - docker - registry / llm-mcp - gitlab - k8s:latest;
  ports:
        - containerPort: 3000;
  envFrom:
        - configMapRef:
  name: gitlab - k8s - mcp - config;
        - secretRef:
  name: gitlab - k8s - mcp - secrets;
  volumeMounts:
        - name: kubeconfig;
  mountPath: /app / .kube;
  readOnly: true;
  volumes:
      - name: kubeconfig;
  secret:
  secretName: kubeconfig - secret;
```text;

## Resources;
This server also exposes GitLab projects as resources that can be accessed through the MCP resource access protocol:

- Resource URI format: `gitlab:// < project_path_with_namespace > `- Example:`gitlab://group / project`## Troubleshooting;

### Common Issues;
1. **Authentication Failures**

   - Ensure your GitLab token has the necessary permissions;
   - Check that the token is set correctly in the environment variables;
2. **Kubernetes Connection Issues**

   - Verify your kubeconfig file is correctly set up;
   - Ensure the user running the MCP server has access to the kubeconfig file;
   - Check that the kubeconfig context is set to the desired cluster;
3. **Missing Permissions**

   - For GitLab: Ensure your token has the necessary scopes;
   - For Kubernetes: Ensure your kubeconfig context user has the necessary RBAC permissions;

### Logs;
The server logs errors to stderr. When running the server directly, these will appear in your terminal.

## Development;
To modify or extend this server:

1. Install dependencies:

  ```bash;
  npm install;
   ```text;

2. Make your changes to the source code:

   - Add new tools by extending the `setupToolHandlers`method;
   - Add new resources by extending the`setupResourceHandlers`method;
3. Test your changes:

  ```bash;
  node test.js;
   ```text;

## License;
This MCP server is part of the LLM Platform Model Context Protocol implementation.

````text
