# LLM-MCP Troubleshooting Guide

This guide provides solutions for common issues you might encounter when working with the LLM Platform Model Context Protocol (LLM-MCP) platform.

## Diagnostics and Health Checks

Before diving into specific issues, use these commands to verify the health of your LLM-MCP installation:

### Server Health Check

```bash
# Check the health endpoint
curl http://localhost:3001/health

# Check server logs
tail -f logs/llm-mcp.log

# Check service status
systemctl status llm-mcp    # If using systemd
pm2 status               # If using PM2
docker ps | grep llm-mcp   # If using Docker
```

### Configuration Validation

```bash
# Validate your configuration file
npm run validate-config -- /path/to/config.json

# Check environment variables
env | grep LLM-MCP_
```

## Common Issues and Solutions

### Connection Issues

#### Connection Refused Error

**Symptoms**:
- "Connection refused" error when connecting to LLM-MCP server
- Client can't establish connection to server

**Possible Causes**:
1. LLM-MCP server is not running
2. Port is already in use
3. Firewall blocking connection
4. Incorrect connection URL

**Solutions**:

1. **Verify server is running**:
   ```bash
   ps aux | grep llm-mcp
   # or
   docker ps | grep llm-mcp
   ```

2. **Check if port is in use**:
   ```bash
   lsof -i :3001
   # or
   netstat -tuln | grep 3001
   ```

3. **Check firewall settings**:
   ```bash
   # Linux
   sudo iptables -L | grep 3001
   
   # macOS
   sudo pfctl -sr | grep 3001
   ```

4. **Verify connection URL**:
   - Ensure protocol, host and port are correct (e.g., `http://localhost:3001`)
   - Try using IP address instead of hostname

#### Connection Timeout

**Symptoms**:
- Connection attempts timeout
- Slow or no response from server

**Solutions**:
1. Check network latency:
   ```bash
   ping server-hostname
   ```

2. Verify server resources are not exhausted:
   ```bash
   top -u llm-mcp_user
   ```

3. Check logs for high load or memory issues:
   ```bash
   grep "memory" logs/llm-mcp.log
   ```

### Authentication Issues

#### Authentication Failed

**Symptoms**:
- "Authentication failed" or "Invalid credentials" errors
- "Unauthorized" (401) HTTP responses

**Possible Causes**:
1. Incorrect API key or credentials
2. Expired token
3. Missing permissions
4. Clock skew between client and server

**Solutions**:

1. **Verify API key**:
   ```bash
   # Test with curl
   curl -H "Authorization: Bearer YOUR_API_KEY" http://localhost:3001/health
   ```

2. **Check token expiration**:
   - For JWT tokens, decode and check expiration:
   ```bash
   # Decode JWT token (requires jq and base64)
   echo "YOUR_TOKEN" | cut -d "." -f 2 | base64 -d | jq
   ```

3. **Check permissions**:
   - Review the permissions assigned to your API key in admin console
   - Ensure the token has the necessary scopes

4. **Verify clock synchronization**:
   ```bash
   # Check system time
   date
   
   # Sync time with NTP server
   sudo ntpdate -u pool.ntp.org
   ```

### Storage Connection Issues

#### MongoDB Connection Errors

**Symptoms**:
- "MongoError: failed to connect to server"
- "MongoNetworkError: connection closed"

**Solutions**:

1. **Verify MongoDB is running**:
   ```bash
   systemctl status mongodb
   # or
   mongo --eval "db.adminCommand('ping')"
   ```

2. **Check connection string**:
   - Ensure format is correct: `mongodb://username:password@host:port/database`
   - Test connection with mongo shell:
   ```bash
   mongo "mongodb://username:password@host:port/database"
   ```

3. **Check network access**:
   ```bash
   telnet mongodb-host 27017
   ```

#### Redis Connection Issues

**Symptoms**:
- "Error: Redis connection to localhost:6379 failed"
- "ECONNREFUSED" errors

**Solutions**:

1. **Verify Redis is running**:
   ```bash
   redis-cli ping
   ```

2. **Check Redis configuration**:
   ```bash
   redis-cli info server
   ```

3. **Test connection manually**:
   ```bash
   redis-cli -h host -p port -a password
   ```

### Tool Registration Issues

#### Tool Registration Fails

**Symptoms**:
- "Tool registration failed" errors
- 400 or 500 status codes during registration

**Possible Causes**:
1. Invalid tool schema
2. Duplicate tool ID
3. Insufficient permissions

**Solutions**:

1. **Validate tool schema**:
   - Ensure tool definition conforms to JSON Schema
   - Check for required fields (id, name, input_schema, etc.)
   ```bash
   npm run validate-tool -- /path/to/tool-definition.json
   ```

2. **Check for duplicate IDs**:
   ```bash
   # List all registered tools
   curl http://localhost:3001/tools
   ```

3. **Review permissions**:
   - Ensure the client has tool registration permissions

### Tool Execution Issues

#### Tool Execution Timeout

**Symptoms**:
- "Execution timeout" errors
- Long running tool executions fail

**Solutions**:

1. **Increase execution timeout**:
   - Update configuration in `config.json`:
   ```json
   {
     "execution": {
       "timeout": 60000  // 60 seconds
     }
   }
   ```

2. **Optimize tool implementation**:
   - Look for slow database queries
   - Check for blocking I/O operations
   - Consider implementing caching

#### Tool Returns Error

**Symptoms**:
- Tool execution returns error
- Error messages in execution response

**Solutions**:

1. **Check tool implementation**:
   - Debug the tool's function implementation
   - Add logging to identify the failure point

2. **Validate input data**:
   - Ensure input conforms to the tool's input schema
   - Check for missing required fields

3. **Check for resource constraints**:
   - Verify sufficient memory and CPU
   - Check for rate limiting on external APIs

### gRPC API Issues

#### gRPC Connection Errors

**Symptoms**:
- "Failed to connect to gRPC server"
- "Deadline exceeded" errors

**Solutions**:

1. **Verify gRPC server is running**:
   ```bash
   # Check if server is listening on gRPC port
   lsof -i :3002
   ```

2. **Test with grpcurl**:
   ```bash
   grpcurl -plaintext localhost:3002 list
   ```

3. **Check TLS configuration** (if using secure gRPC):
   - Verify client has correct certificates
   - Check server certificate is valid

### Performance Issues

#### Slow Response Times

**Symptoms**:
- API requests take longer than expected
- Operations timeout or are sluggish

**Solutions**:

1. **Check server load**:
   ```bash
   top
   # or
   htop
   ```

2. **Monitor memory usage**:
   ```bash
   free -m
   ```

3. **Optimize database queries**:
   - Add appropriate indexes
   - Review query performance

4. **Scale resources**:
   - Increase container resources
   - Add more replicas if using Kubernetes

#### High Memory Usage

**Symptoms**:
- Server uses excessive memory
- Out of memory errors

**Solutions**:

1. **Check for memory leaks**:
   - Use Node.js heap snapshots
   ```bash
   node --inspect=9229 index.js
   # Then connect with Chrome DevTools
   ```

2. **Adjust memory limits**:
   - Update Node.js memory limits:
   ```bash
   NODE_OPTIONS="--max-old-space-size=4096" npm start
   ```

3. **Enable garbage collection logs**:
   ```bash
   NODE_OPTIONS="--trace-gc" npm start
   ```

## Debugging Techniques

### Enabling Debug Logs

LLM-MCP uses the `debug` package for detailed logging. Enable it with environment variables:

```bash
# Enable all LLM-MCP debug logs
DEBUG=llm-mcp:* npm start

# Enable specific components
DEBUG=llm-mcp:server,llm-mcp:tool-registry npm start
```

### Using CLI Diagnostic Tools

The LLM-MCP CLI provides diagnostic tools:

```bash
# Install LLM-MCP CLI globally
npm install -g @bluefly/llm-mcp-cli

# Run diagnostics
llm-mcp-cli diagnose --server http://localhost:3001

# Test tool execution
llm-mcp-cli execute --tool-id "weather_lookup" --input '{"location":"New York"}'
```

### Monitoring Metrics

LLM-MCP exposes Prometheus metrics at `/metrics`. Use these for monitoring:

```bash
# Get metrics
curl http://localhost:3001/metrics
```

Set up Grafana dashboards for visualization using our template:

```bash
# Import dashboard to Grafana
curl -X POST -H "Content-Type: application/json" -d @llm-mcp-dashboard.json http://grafana:3000/api/dashboards/db
```

## Advanced Troubleshooting

### Database Repairs

If MongoDB data becomes corrupted:

```bash
# Repair MongoDB database
mongod --repair --dbpath /path/to/db
```

For Redis issues:

```bash
# Check Redis memory stats
redis-cli info memory

# Clear Redis cache if needed
redis-cli FLUSHDB
```

### Network Diagnostics

For complex network issues:

```bash
# Trace network path
traceroute server-hostname

# Capture network traffic
sudo tcpdump -i any port 3001 -w llm-mcp.pcap
```

### Process Profiling

For performance investigations:

```bash
# Install clinic.js
npm install -g clinic

# Run with profiling
clinic doctor -- node index.js
```

## Getting Help

If you're still experiencing issues after following this guide:

1. **Check Documentation**:
   - Review the [Getting Started Guide](../guides/getting-started.md)
   - Check the [API Reference](../api/api-reference.md)

2. **Search GitHub Issues**:
   - Search existing issues at https://github.com/bluefly/llm-mcp/issues

3. **Submit a GitHub Issue**:
   - Include error messages
   - Attach server logs
   - Describe your environment (OS, Node.js version, etc.)
   - Share steps to reproduce

4. **Contact Support**:
   - Email: support@bluefly.ai
   - Include your license key or account ID
   - See [SysAdmin Email Guide](../sysadmin-email.md) for template

## Prevention Tips

### Backup and Recovery

Regularly backup your LLM-MCP data:

```bash
# Backup MongoDB
mongodump --uri "mongodb://localhost:27017/llm-mcp" --out /backup/path

# Backup Redis (if using persistence)
redis-cli SAVE
cp /var/lib/redis/dump.rdb /backup/path/
```

### Monitoring Best Practices

Set up proactive monitoring:

1. **Configure health checks**:
   ```bash
   # Add to crontab
   */5 * * * * curl -f http://localhost:3001/health || ./alert.sh
   ```

2. **Set up metrics alerts**:
   - Configure Prometheus alerts for:
     - High error rates
     - Slow response times
     - Memory usage thresholds
     - Connection failures

3. **Log monitoring**:
   - Use ELK Stack or similar for log aggregation
   - Set up alerts for error patterns

## Appendix

### Log File Locations

- **Server Logs**: `logs/llm-mcp.log`
- **Error Logs**: `logs/error.log`
- **Access Logs**: `logs/access.log`

### Configuration File Reference

```json
{
  "server": {
    "port": 3001,
    "host": "0.0.0.0",
    "grpcPort": 3002
  },
  "redis": {
    "host": "localhost",
    "port": 6379,
    "password": "",
    "db": 0
  },
  "storage": {
    "type": "mongodb",
    "uri": "mongodb://localhost:27017/llm-mcp",
    "options": {
      "useNewUrlParser": true,
      "useUnifiedTopology": true
    }
  },
  "security": {
    "apiKey": "",
    "jwtSecret": "",
    "allowedOrigins": ["*"]
  },
  "logging": {
    "level": "info",
    "file": "logs/llm-mcp.log",
    "maxSize": "10m",
    "maxFiles": 5
  }
}
```

### Common Error Codes

| Error Code | Description | Possible Solution |
|------------|-------------|-------------------|
| `AUTH_001` | Authentication failed | Check API key or JWT token |
| `AUTH_002` | Token expired | Refresh your token |
| `DB_001` | Database connection failed | Check MongoDB/Redis connection |
| `TOOL_001` | Tool not found | Verify tool ID exists |
| `TOOL_002` | Tool registration failed | Check tool schema |
| `EXEC_001` | Tool execution failed | Debug tool implementation |
| `EXEC_002` | Execution timeout | Increase timeout or optimize tool |