# Vector Bridge Integration Guide

This document provides guidance on integrating the Vector Bridge library into the LLM Platform AI Platform ecosystem components.

## Overview

Vector Bridge provides a unified interface for vector operations (embeddings, storage, retrieval, and search) across all platform components. It serves as the integration layer between various vector database backends and AI model providers, ensuring consistent vector operations across the entire platform.

## Key Features

- **Unified API**: Consistent interface across all platform components
- **Multi-modal Support**: Text, image, and other data type embeddings
- **Database Flexibility**: Works with local and remote vector databases
- **High Performance**: Optimized for large-scale vector operations
- **Cross-Platform**: Works in Node.js, browser, and Drupal environments
- **Replication and Synchronization**: Maintain consistent vector data across systems

## Integration Points

Vector Bridge integrates with the following platform components:

### 1. LLM-MCP Integration

The LLM-MCP service uses Vector Bridge to provide vector operations through MCP tools:

```typescript
// In llm-mcp/src/plugins/vector-plugin.ts
import [VectorBridge] from '../vector-bridge-index';

class VectorPlugin {
  // Vector operations exposed as MCP tools
  get tools() {
    return {
      'vector_store': this.storeVectorTool.bind(this),
      'vector_search': this.searchVectorTool.bind(this),
      'vector_get': this.getVectorTool.bind(this),
      'vector_delete': this.deleteVectorTool.bind(this),
      'vector_list': this.listVectorsTool.bind(this),
      'text_embed': this.embedTextTool.bind(this),
      'semantic_search': this.semanticSearchTool.bind(this),
      'similarity_compare': this.similarityCompareTool.bind(this),
      'vector_index_text': this.indexTextTool.bind(this),
      'vector_batch_upsert': this.batchUpsertTool.bind(this),
      'vector_status': this.vectorStatusTool.bind(this),
    };
  }
}
```

### 2. BFAPI Integration

BFAPI uses Vector Bridge for its Context Embedding service:

```typescript
// In bfapi/src/adapters/vector-bridge-adapter.ts
import [VectorBridge] from 'vector-bridge';

export class VectorBridgeAdapter {
  private vectorBridge: VectorBridge;
  
  constructor(config: VectorBridgeAdapterConfig = {}) {
    this.vectorBridge = new VectorBridge({
      endpoint: config.endpoint,
      collection: config.collection,
      apiKey: config.apiKey,
      dimensions: config.dimensions,
      debug: config.debug,
    });
  }
  
  // Vector operations for the Context Embedding service
  async generateEmbedding(text: string): Promise<number[]> {
    return await this.vectorBridge.generateEmbedding(text);
  }
  
  async searchByText(text: string, limit: number = 10, threshold?: number): Promise<Array<{
    id: string;
    similarity: number;
    data: any;
    metadata: any;
  }>> {
    return await this.vectorBridge.searchText(text, {
      limit,
      threshold,
    });
  }
  
  // Other vector operations...
}
```

### 3. BFLLM Integration

BFLLM uses Vector Bridge for embedding generation and storage:

```typescript
// Example in bfllm/src/embeddings/VectorBridgeService.ts
import [VectorBridge] from 'vector-bridge';

export class VectorBridgeService {
  private vectorBridge: VectorBridge;
  
  constructor() {
    this.vectorBridge = new VectorBridge({
      collection: 'bfllm-embeddings',
    });
  }
  
  async generateEmbedding(text: string, modelName: string = 'default'): Promise<number[]> {
    return await this.vectorBridge.generateEmbedding(text, { model: modelName });
  }
  
  // Other embedding operations...
}
```

### 4. Drupal Integration (llm_platform_recipe)

The Drupal recipe integrates with Vector Bridge for content embeddings:

```php
<?php
// Example in llm_platform_recipe/src/Service/VectorBridgeService.php

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\ClientInterface;

/**
 * Provides integration with the Vector Bridge service.
 */
class VectorBridgeService {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructor.
   */
  public function __construct(ClientInterface $http_client, ConfigFactoryInterface $config_factory) {
    $this->httpClient = $http_client;
    $this->config = $config_factory->get('llm_platform_recipe.vector_settings');
  }

  /**
   * Generate an embedding for the given text.
   */
  public function generateEmbedding($text) {
    $response = $this->httpClient->post($this->config->get('vector_bridge_endpoint') . '/api/embeddings', [
      'json' => [
        'text' => $text,
      ],
      'headers' => [
        'Authorization' => 'Bearer ' . $this->config->get('vector_bridge_api_key'),
      ],
    ]);
    
    $data = json_decode((string) $response->getBody(), TRUE);
    return $data['embedding'];
  }
  
  // Other vector operations...
}
```

## Configuration

Vector Bridge can be configured at both the global and component-specific levels:

### Global Configuration

```typescript
// In app initialization
import [initializeVectorBridge] from 'vector-bridge';

await initializeVectorBridge({
  endpoint: process.env.VECTOR_BRIDGE_ENDPOINT || 'http://localhost:3001/api/vectors',
  apiKey: process.env.VECTOR_API_KEY,
  defaultCollection: 'default',
  embedDimensions: 384,
  providers: {
    bfllm: {
      enabled: true,
      modelName: 'bfllm/text-embedding',
    },
    openai: {
      enabled: true,
      modelName: 'text-embedding-3-small',
    },
    local: {
      enabled: false,
      modelPath: './models/embedding-model',
    },
  },
});
```

### Component-Specific Configuration

Each component can customize its Vector Bridge configuration:

```typescript
// In LLM-MCP
const vectorBridge = new VectorBridge({
  collection: 'mcp-vectors',
  dimensions: 1536,
  useCache: true,
  cacheOptions: {
    maxSize: 1000,
    ttl: 3600000, // 1 hour
  },
});
```

## Database Adapters

Vector Bridge supports multiple vector database backends:

1. **Qdrant Adapter**: High-performance vector database
2. **Memory Adapter**: In-memory vector storage for development and testing
3. **Drupal Adapter**: Integrates with Drupal's database for vector storage

Example configuration:

```typescript
// Using Qdrant
const vectorBridge = new VectorBridge({
  database: 'qdrant',
  qdrantUrl: 'http://localhost:6333',
  qdrantApiKey: 'your-api-key',
  collection: 'my-vectors',
});

// Using in-memory storage
const vectorBridge = new VectorBridge({
  database: 'memory',
  persistToFile: true,
  persistPath: './vector-data.json',
});
```

## Embedding Providers

Vector Bridge supports multiple embedding providers:

1. **NativeAIJS Adapter**: Uses native_aijs for embeddings
2. **BFLLM Adapter**: Uses BFLLM service for embeddings
3. **OpenAI Adapter**: Uses OpenAI API for embeddings

Example configuration:

```typescript
// Set the provider priority
vectorBridge.setConfig('providers', {
  bfllm: {
    priority: 1, // Highest priority
    modelName: 'bfllm/text-embedding',
  },
  openai: {
    priority: 2, // Fallback
    modelName: 'text-embedding-3-small',
  },
});
```

## Multi-modal Support

Vector Bridge supports multi-modal embeddings:

```typescript
// Text embeddings
const textEmbedding = await vectorBridge.generateEmbedding('This is a text example');

// Image embeddings
const imageEmbedding = await vectorBridge.generateImageEmbedding('/path/to/image.jpg');

// Multi-modal search
const results = await vectorBridge.search({
  text: 'sunset over mountains',
  image: '/path/to/query-image.jpg',
  limit: 10,
});
```

## Replication and Synchronization

Vector Bridge supports replication and synchronization between different vector stores:

```typescript
// Replicate vectors from one collection to another
await vectorBridge.replicate({
  source: {
    collection: 'source-collection',
  },
  target: {
    collection: 'target-collection',
    database: 'qdrant',
    qdrantUrl: 'http://remote-qdrant:6333',
  },
});

// Synchronize vectors between collections
await vectorBridge.synchronize({
  collections: ['collection-1', 'collection-2'],
  strategy: 'newest-wins',
});
```

## Error Handling

Vector Bridge uses the Either monad pattern for robust error handling:

```typescript
import [isLeft] from 'fp-ts/Either';

const result = await vectorBridge.safeSearch({
  text: 'example query',
});

if (isLeft(result)) {
  console.error('Search failed:', result.left);
} else {
  console.log('Search results:', result.right);
}
```

## Monitoring and Telemetry

Vector Bridge includes monitoring and telemetry features:

```typescript
// Enable telemetry
vectorBridge.enableTelemetry({
  metricsEndpoint: 'http://metrics-server:9090',
  logLevel: 'info',
  sampleRate: 0.1, // Sample 10% of operations
});

// Get vector operations metrics
const metrics = await vectorBridge.getMetrics();
console.log('Vector operations:', metrics.operations);
console.log('Average latency:', metrics.avgLatencyMs);
```

## Conclusion

Vector Bridge provides a unified interface for vector operations across the entire LLM Platform AI Platform ecosystem. By integrating with Vector Bridge, components can leverage consistent vector operations with high performance and flexibility.