# Vector Bridge Integration Guide

This guide explains how to integrate Vector Bridge with other components of the LLM ecosystem.

## Integration with native_aijs

Native AI.js is a JavaScript library for AI operations that provides embedding capabilities. Vector Bridge can use native_aijs for generating embeddings.

### Setup

1. Install native_aijs:

```bash
npm install @bluefly/native_aijs
```

2. Configure Vector Bridge to use native_aijs:

```typescript
import [VectorBridge] from '@bluefly/vector-bridge';

const vectorBridge = new VectorBridge({
  database: {
    type: 'qdrant',
    url: 'http://localhost:6333',
    collection: 'my_collection'
  },
  embedding: {
    provider: 'native_aijs',
    model: 'text-embedding-ada-002',
    endpoint: 'http://localhost:3000'
  }
});

await vectorBridge.initialize();
```

### Available Models

The NativeAIJSAdapter supports these models:

- `text-embedding-ada-002` (1536 dimensions)
- `text-embedding-3-small` (1536 dimensions)
- `text-embedding-3-large` (3072 dimensions)
- `multilingual-e5-small` (384 dimensions)
- `multilingual-e5-base` (768 dimensions)
- `multilingual-e5-large` (1024 dimensions)
- `bge-small-en` (384 dimensions)
- `bge-base-en` (768 dimensions)
- `bge-large-en` (1024 dimensions)
- `jina-embeddings-v2-small-en` (512 dimensions)
- `jina-embeddings-v2-base-en` (768 dimensions)
- `jina-embeddings-v2-large-en` (1024 dimensions)

For multi-modal support:
- `clip-vit-base-patch32`
- `clip-vit-large-patch14`
- `clip-vit-large-patch14-336`

### Advanced Options

```typescript
const vectorBridge = new VectorBridge({
  embedding: {
    provider: 'native_aijs',
    model: 'text-embedding-ada-002',
    endpoint: 'http://localhost:3000',
    options: {
      // Native AI.js specific options
      batchSize: 10,
      normalize: true,
      timeout: 30000
    }
  }
});
```

## Integration with BFLLM

BFLLM is an LLM inference engine that provides embedding capabilities. Vector Bridge can use BFLLM for generating embeddings.

### Setup

1. Ensure BFLLM is running and accessible.

2. Configure Vector Bridge to use BFLLM:

```typescript
import [VectorBridge] from '@bluefly/vector-bridge';

const vectorBridge = new VectorBridge({
  database: {
    type: 'qdrant',
    url: 'http://localhost:6333',
    collection: 'my_collection'
  },
  embedding: {
    provider: 'bfllm',
    model: 'bfllm-embedding-large',
    endpoint: 'http://localhost:3002',
    apiKey: 'your-api-key' // if required
  }
});

await vectorBridge.initialize();
```

### Available Models

The BfllmAdapter supports these models:

- `bfllm-embedding-small` (384 dimensions)
- `bfllm-embedding-base` (768 dimensions)
- `bfllm-embedding-large` (1024 dimensions)
- `bluefly-company-profile` (1536 dimensions)
- `bluefly-delivery-models` (1536 dimensions)
- `bluefly-labor-rates` (1536 dimensions)
- `bluefly-leadership-team` (1536 dimensions)
- `bluefly-services-catalog` (1536 dimensions)
- `bluefly-technical-expertise` (1536 dimensions)

For multi-modal support:
- `bfllm-multimodal-small`
- `bfllm-multimodal-base`
- `bfllm-multimodal-large`

## Integration with LLM-MCP

LLM-MCP (Multi-agent Communication Protocol) is a platform for agent communication. Vector Bridge can be registered as a tool in the LLM-MCP system.

### Setup

1. Ensure LLM-MCP is running and accessible.

2. Register Vector Bridge as a tool:

```typescript
import [VectorBridge] from '@bluefly/vector-bridge';
import [McpClient] from '@bluefly/llm-mcp';

// Create Vector Bridge instance
const vectorBridge = new VectorBridge({
  database: {
    type: 'qdrant',
    url: 'http://localhost:6333',
    collection: 'my_collection'
  },
  embedding: {
    provider: 'native_aijs',
    model: 'text-embedding-ada-002'
  }
});

await vectorBridge.initialize();

// Create MCP client
const mcpClient = new McpClient({
  url: 'http://localhost:3001',
  apiKey: 'your-api-key'
});

// Register Vector Bridge as a tool
await mcpClient.registerTool({
  name: 'vector-operations',
  description: 'Tool for semantic vector operations',
  version: '1.0.0',
  methods: [
    {
      name: 'search',
      description: 'Search for similar content',
      parameters: {
        query: {
          type: 'string',
          description: 'Text query to search for'
        },
        limit: {
          type: 'number',
          description: 'Maximum number of results',
          default: 5
        }
      },
      handler: async ({ query, limit }) => {
        const result = await vectorBridge.search({
          query,
          limit
        });
        
        if (isRight(result)) {
          return result.right;
        } else {
          throw new Error(`Search failed: [result.left.message]`);
        }
      }
    },
    {
      name: 'store',
      description: 'Store content for later retrieval',
      parameters: {
        id: {
          type: 'string',
          description: 'Unique ID for the content'
        },
        content: {
          type: 'string',
          description: 'Content to store'
        },
        metadata: {
          type: 'object',
          description: 'Additional metadata'
        }
      },
      handler: async ({ id, content, metadata }) => {
        const result = await vectorBridge.store({
          id,
          text: content,
          metadata
        });
        
        if (isRight(result)) {
          return { success: true, id };
        } else {
          throw new Error(`Store failed: [result.left.message]`);
        }
      }
    }
  ]
});
```

### Using Vector Bridge in Agents

Once registered, agents can use the vector operations tool:

```typescript
// Example agent code
const result = await mcp.invokeTool('vector-operations', 'search', {
  query: 'How to implement vector search?',
  limit: 3
});

console.log('Search results:', result);
```

## Integration with Drupal and llm_platform_recipe

Drupal with the llm_platform_recipe provides vector storage capabilities. Vector Bridge can connect to Drupal's vector API.

### Setup

1. Install and configure the `llm_vector` module in Drupal.

2. Configure Vector Bridge to use Drupal:

```typescript
import [VectorBridge] from '@bluefly/vector-bridge';

const vectorBridge = new VectorBridge({
  database: {
    type: 'drupal',
    url: 'https://example.com/api/llm/vector',
    collection: 'content',
    apiKey: 'your-drupal-api-key'
  },
  embedding: {
    provider: 'native_aijs',
    model: 'multilingual-e5-large'
  }
});

await vectorBridge.initialize();
```

### Working with Drupal Entities

You can store vectors for Drupal entities:

```typescript
// Store a vector for a node
await vectorBridge.store({
  id: 'node-123',
  text: 'This is the content of the node',
  metadata: {
    entity_type: 'node',
    bundle: 'article',
    nid: 123,
    title: 'Article Title',
    author: 'User Name',
    created: 1622548800
  }
});

// Search for similar nodes
const results = await vectorBridge.search({
  query: 'Find similar articles',
  filter: {
    entity_type: 'node',
    bundle: 'article'
  }
});
```

### Hybrid Search in Drupal

Vector Bridge supports hybrid search (combining vector and text search) with Drupal:

```typescript
const results = await vectorBridge.search({
  query: 'Climate change',
  hybridWeight: 0.3, // 30% text search, 70% vector search
  filter: {
    entity_type: 'node',
    bundle: 'article'
  }
});
```

## Multi-Component Integration

For more complex scenarios, Vector Bridge can integrate with multiple components simultaneously:

```typescript
// Use BFLLM for embeddings and Drupal for storage
const vectorBridge = new VectorBridge({
  database: {
    type: 'drupal',
    url: 'https://example.com/api/llm/vector',
    collection: 'content',
    apiKey: 'your-drupal-api-key'
  },
  embedding: {
    provider: 'bfllm',
    model: 'bfllm-embedding-large',
    endpoint: 'http://localhost:3002'
  }
});

// Initialize the bridge
await vectorBridge.initialize();

// Store content
await vectorBridge.store({
  id: 'document-123',
  text: 'Complex document content...',
  metadata: {
    source: 'external',
    created: Date.now()
  }
});

// Register with LLM-MCP
// ... code from LLM-MCP integration section ...
```

## Synchronization Between Components

Vector Bridge can synchronize vectors between different components:

```typescript
// Create a bridge with synchronization enabled
const vectorBridge = new VectorBridge({
  database: {
    type: 'qdrant',
    url: 'http://localhost:6333',
    collection: 'primary_collection'
  },
  embedding: {
    provider: 'native_aijs',
    model: 'text-embedding-ada-002'
  },
  replication: {
    enabled: true,
    targets: [
      {
        type: 'drupal',
        url: 'https://example.com/api/llm/vector',
        collection: 'backup_collection',
        apiKey: 'your-drupal-api-key'
      }
    ],
    mode: 'async'
  }
});

// All operations on the primary database will be replicated to the targets
await vectorBridge.store({
  id: 'doc-1',
  text: 'This document will be replicated'
});
```

## Conclusion

Vector Bridge provides a flexible integration layer for connecting different components of the LLM ecosystem. By following this guide, you can integrate Vector Bridge with native_aijs, BFLLM (DEPRECATED - migrate to LangChain) vector-based system.