# Vector Bridge Usage Guide

This guide provides examples and best practices for using Vector Bridge in your applications.

## Installation

Install Vector Bridge using npm:

```bash
npm install @bluefly/vector-bridge
```

For optional dependencies:

```bash
# For TensorFlow support
npm install @tensorflow/tfjs-node

# For Pinecone support
npm install @pinecone-database/pinecone

# For WebSocket support
npm install ws socket.io-client
```

## Basic Usage

### Initialize Vector Bridge

```typescript
import [VectorBridge] from '@bluefly/vector-bridge';

// Create a new instance
const vectorBridge = new VectorBridge({
  database: {
    type: 'qdrant',
    url: 'http://localhost:6333',
    collection: 'my_collection'
  },
  embedding: {
    provider: 'native_aijs',
    model: 'text-embedding-ada-002'
  }
});

// Initialize the bridge
await vectorBridge.initialize();
```

### Store Vectors

Store a vector with text:

```typescript
const result = await vectorBridge.store({
  id: 'doc-1',
  text: 'This is a sample document about vector search.',
  metadata: {
    source: 'example',
    author: 'LLM Platform',
    created: Date.now()
  }
});

if (isRight(result)) {
  console.log('Vector stored successfully');
} else {
  console.error('Failed to store vector:', result.left.message);
}
```

Store a vector with an image:

```typescript
const result = await vectorBridge.store({
  id: 'image-1',
  image: {
    data: 'base64-encoded-image-data',
    mimeType: 'image/jpeg'
  },
  metadata: {
    source: 'example',
    author: 'LLM Platform',
    created: Date.now()
  }
});
```

### Search for Similar Vectors

Search by text:

```typescript
const results = await vectorBridge.search({
  query: 'How does vector search work?',
  limit: 5,
  threshold: 0.7
});

if (isRight(results)) {
  console.log('Search results:', results.right);
} else {
  console.error('Search failed:', results.left.message);
}
```

Search by vector:

```typescript
const results = await vectorBridge.search({
  queryVector: [0.1, 0.2, 0.3, ...],
  limit: 5,
  threshold: 0.7
});
```

Search with filters:

```typescript
const results = await vectorBridge.search({
  query: 'How does vector search work?',
  filter: {
    source: 'example',
    created: { $gt: Date.now() - 86400000 } // Last 24 hours
  }
});
```

### Retrieve and Delete Vectors

Retrieve a vector:

```typescript
const result = await vectorBridge.retrieve('doc-1');

if (isRight(result)) {
  console.log('Retrieved vector:', result.right);
} else {
  console.error('Retrieval failed:', result.left.message);
}
```

Delete a vector:

```typescript
const result = await vectorBridge.delete('doc-1');

if (isRight(result)) {
  console.log('Vector deleted successfully');
} else {
  console.error('Deletion failed:', result.left.message);
}
```

Delete multiple vectors:

```typescript
const result = await vectorBridge.deleteBatch(['doc-1', 'doc-2', 'doc-3']);
```

### Event Handling

Subscribe to events:

```typescript
// Listen for store events
vectorBridge.on('store', (data) => {
  console.log('Vector stored:', data.id);
});

// Listen for search events
vectorBridge.on('search', (data) => {
  console.log('Search performed:', data.query, 'Results:', data.results);
});

// Unsubscribe when done
vectorBridge.off('store', myHandler);
```

### Cleanup

Close connections when done:

```typescript
await vectorBridge.close();
```

## Advanced Usage

### Batch Operations

Store multiple vectors at once:

```typescript
const items = [
  {
    id: 'doc-1',
    text: 'First document',
    metadata: { category: 'example' }
  },
  {
    id: 'doc-2',
    text: 'Second document',
    metadata: { category: 'example' }
  },
  {
    id: 'doc-3',
    text: 'Third document',
    metadata: { category: 'example' }
  }
];

const result = await vectorBridge.storeBatch(items);
```

### Vector Utilities

Vector Bridge provides utilities for working with vectors:

```typescript
import {
  calculateCosineSimilarity,
  normalizeVector,
  splitTextIntoChunks
} from '@bluefly/vector-bridge';

// Calculate similarity between two vectors
const similarity = calculateCosineSimilarity([0.1, 0.2, 0.3], [0.2, 0.3, 0.4]);

// Normalize a vector
const normalized = normalizeVector([1, 2, 3]);

// Split text into chunks for embedding
const chunks = splitTextIntoChunks('Long text...', 1000, 200);
```

### Custom Configurations

#### Memory Cache Configuration

Enable caching for better performance:

```typescript
const vectorBridge = new VectorBridge({
  database: { ... },
  embedding: [...],
  cache: {
    enabled: true,
    maxItems: 10000,
    ttl: 3600 // 1 hour
  }
});
```

#### Telemetry Configuration

Enable telemetry for monitoring:

```typescript
const vectorBridge = new VectorBridge({
  database: { ... },
  embedding: [...],
  telemetry: {
    enabled: true,
    endpoint: 'https://telemetry.example.com/collect',
    samplingRate: 0.1 // Sample 10% of operations
  }
});
```

#### Logging Configuration

Configure logging:

```typescript
const vectorBridge = new VectorBridge({
  database: { ... },
  embedding: [...],
  logging: {
    level: 'debug', // 'debug', 'info', 'warn', 'error'
    pretty: true // Pretty-print logs
  }
});
```

### Integration with Drupal

Connect to Drupal's vector storage:

```typescript
const vectorBridge = new VectorBridge({
  database: {
    type: 'drupal',
    url: 'https://example.com/api/llm/vector',
    collection: 'content',
    apiKey: 'your-api-key'
  },
  embedding: {
    provider: 'native_aijs',
    model: 'multilingual-e5-large'
  }
});
```

### Integration with BFLLM

Use BFLLM for embeddings:

```typescript
const vectorBridge = new VectorBridge({
  database: {
    type: 'qdrant',
    url: 'http://localhost:6333',
    collection: 'my_collection'
  },
  embedding: {
    provider: 'bfllm',
    model: 'bfllm-embedding-large',
    endpoint: 'http://localhost:3002'
  }
});
```

### Integration with native_aijs

Use native_aijs for embeddings:

```typescript
const vectorBridge = new VectorBridge({
  database: {
    type: 'qdrant',
    url: 'http://localhost:6333',
    collection: 'my_collection'
  },
  embedding: {
    provider: 'native_aijs',
    model: 'text-embedding-ada-002',
    endpoint: 'http://localhost:3000'
  }
});
```

## Best Practices

### Error Handling

Always handle errors properly:

```typescript
import { isLeft, isRight } from 'fp-ts/lib/Either';

const result = await vectorBridge.store({
  id: 'doc-1',
  text: 'Sample document'
});

if (isLeft(result)) {
  const error = result.left;
  
  if (error.retryable) {
    // Implement retry logic
    console.warn(`Retryable error: [error.message]`);
    // Retry after delay...
  } else {
    console.error(`Non-retryable error: [error.message]`);
    // Handle permanent failure
  }
}
```

### Connection Management

Initialize and close connections properly:

```typescript
let vectorBridge;

try [vectorBridge = new VectorBridge({ ...]);
  await vectorBridge.initialize();
  
  // Use vectorBridge...
  
} catch (error) {
  console.error('Failed to initialize Vector Bridge:', error);
} finally {
  if (vectorBridge) {
    await vectorBridge.close();
  }
}
```

### Performance Optimization

- Use batch operations for bulk processing
- Enable caching for frequently accessed vectors
- Use appropriate chunk sizes for text
- Monitor performance with telemetry

### Security Considerations

- Store API keys securely, never hardcode them
- Validate user inputs before sending to Vector Bridge
- Use HTTPS for all remote connections
- Implement proper authentication for your vector database

## Troubleshooting

### Common Issues

**Connection failures**:
- Check that the database URL is correct
- Verify that the database is running
- Check network connectivity
- Verify that API keys are correct

**Embedding generation failures**:
- Check that the embedding provider is available
- Verify that the model ID is correct
- Check if text is too long for the model

**Search failures**:
- Verify that the collection exists
- Check that the query vector has the correct dimensions
- Verify that filters are correctly formatted

### Debugging

Enable debug logging for more information:

```typescript
const vectorBridge = new VectorBridge({
  database: { ... },
  embedding: [...],
  logging: {
    level: 'debug',
    pretty: true
  }
});
```

Check database status:

```typescript
const status = await vectorBridge.status();
if (isRight(status)) {
  console.log('Database status:', status.right);
}
```

## Conclusion

Vector Bridge provides a powerful and flexible way to work with vector embeddings across different platforms. By following this guide, you should be able to integrate it into your applications and leverage its capabilities for semantic search, recommendation systems, and more.