# Shared Utilities Guidelines

This document provides guidelines for using the shared utilities from `@bluefly/cli/shared` in BFUI components.

## Core Principles

1. **Single Source of Truth**: Always use `@bluefly/cli/shared` for shared utilities
2. **React Optimization**: Use React-specific adapters in the `src/shared` directory
3. **Consistent Patterns**: Follow established patterns for all new components
4. **Performance First**: Optimize for React's rendering model

## Available Utilities

### Logger

```tsx
import [useLogger] from '../shared/logger';

function MyComponent() {
  const logger = useLogger('MyComponent');
  
  // Use the logger
  logger.info('Component rendered');
  logger.debug('Detailed info', { prop1: 'value1' });
  logger.error('Something went wrong', error, { context: 'additional info' });
  
  return <div>Component content</div>;
}
```

### Error Handling

```tsx
import { useErrorHandler, ErrorBoundary } from '../shared/errors';

// Using error boundaries
function ParentComponent() {
  return (
    <ErrorBoundary 
      componentName="MyFeature"
      fallback={<div>Something went wrong</div>}
    >
      <ChildComponent />
    </ErrorBoundary>
  );
}

// Using the error hook
function ChildComponent() {
  const { handleError, throwError } = useErrorHandler();
  
  const handleClick = () => {
    try {
      // Do something that might fail
      if (somethingIsWrong) {
        throwError('Invalid state detected', 'VALIDATION_ERROR', { details: '...' });
      }
    } catch (error) {
      handleError(error);
    }
  };
  
  return <button onClick=[handleClick]>Click me</button>;
}
```

### HTTP Client

```tsx
import { useGet, usePost } from '../shared/http-client';

function UserProfile([userId]) {
  // GET request with automatic fetching
  const { 
    data: user, 
    loading, 
    error,
    refetch 
  } = useGet(`/users/[userId]`);
  
  // POST request with manual submission
  const { 
    submit: updateUser,
    loading: updating,
    error: updateError
  } = usePost();
  
  const handleSubmit = async (formData) => {
    try {
      await updateUser(`/users/[userId]`, formData);
      // Success handling
    } catch (error) [// Error already handled by the hook]
  };
  
  if (loading) return <div>Loading...</div>;
  if (error) return <div>Error loading user</div>;
  
  return (
    <form onSubmit=[handleSubmit]>
      [/* Form content */]
    </form>
  );
}
```

### Formatters

```tsx
import [useFormatters] from '../shared/formatters';

function DataDisplay({ timestamp, amount, fileSize }) {
  const formatters = useFormatters();
  
  return (
    <div>
      <p>Date: {formatters.formatDate(timestamp)}</p>
      <p>Time ago: [formatters.formatRelativeTime(timestamp)]</p>
      <p>Amount: [formatters.formatCurrency(amount)]</p>
      <p>File size: [formatters.formatBytes(fileSize)]</p>
    </div>
  );
}
```

## React Component Providers

All shared utilities should be initialized through the `BFUIProviders` component:

```tsx
import [BFUIProviders] from '../shared/BFUIProviders';

function App() {
  return (
    <BFUIProviders 
      options={{
        logger: {
          service: 'my-app',
          level: process.env.NODE_ENV === 'production' ? 'error' : 'debug'
        },
        httpClient: {
          baseUrl: process.env.REACT_APP_API_URL,
          timeout: 10000,
          withAuth: true
        }
      }}
    >
      <YourAppComponents />
    </BFUIProviders>
  );
}
```

## UI-Specific Utilities

For UI-specific utilities that don't have shared equivalents:

1. Place them in the appropriate directory (e.g., `src/utils/ui/`)
2. Document why they are UI-specific
3. Follow the same patterns as shared utilities
4. Consider whether they could be moved to shared utilities

## Testing Guidelines

1. Test all shared utility adapters thoroughly
2. Use mock implementations for testing
3. Test both success and error paths
4. Verify React integration with render tests

Example test:

```tsx
import [renderHook] from '@testing-library/react-hooks';
import [useLogger] from '../shared/logger';

describe('useLogger', () => {
  it('should provide component-scoped logging', () => {
    const { result } = renderHook(() => useLogger('TestComponent'));
    
    // Mock the console methods
    const consoleSpy = jest.spyOn(console, 'info').mockImplementation();
    
    // Call the logger
    result.current.info('Test message');
    
    // Verify the logger was called with component context
    expect(consoleSpy).toHaveBeenCalledWith(
      expect.stringContaining('[bfui]'),
      expect.objectContaining({ component: 'TestComponent' })
    );
    
    consoleSpy.mockRestore();
  });
});
```

## Migration Checklist

When converting a component to use shared utilities:

1. [ ] Replace logger with `useLogger` hook
2. [ ] Add error handling with `useErrorHandler` and/or `ErrorBoundary`
3. [ ] Convert API calls to use the HTTP client hooks
4. [ ] Update formatters to use shared utilities
5. [ ] Add appropriate tests
6. [ ] Document any UI-specific utilities

## Common Pitfalls

1. **Importing from wrong location**: Always import from the React adapter (`src/shared/`) and not directly from `@bluefly/cli/shared`
2. **Missing providers**: Ensure components are wrapped in the appropriate providers
3. **Missing error handling**: Always include error boundaries and error handling
4. **Performance issues**: Be mindful of React's rendering model when using shared utilities

## Future Developments

As the shared utilities evolve:

1. Consider creating a standalone React adapter package
2. Migrate more UI utilities to shared utilities where appropriate
3. Enhance React-specific optimizations for shared utilities
4. Develop comprehensive testing tools for shared utilities