# Accessibility System

## Overview

The BFUI Accessibility System is a comprehensive framework for creating, testing, and validating accessible components in React applications. It provides tools for real-time accessibility analysis, issue reporting, and automated fix suggestions.

## Key Features

- **Component Analysis**: Automatically analyze React components for accessibility issues
- **WCAG Validation**: Check compliance with WCAG 2.1 guidelines (A, AA, AAA levels)
- **Real-time Feedback**: Receive immediate feedback on accessibility issues
- **Fix Suggestions**: Get automated suggestions for fixing accessibility issues
- **Documentation**: Generate accessibility reports for documentation
- **TypeScript Support**: Full type safety with comprehensive interfaces
- **Minimal Dependencies**: Self-contained with no external accessibility packages

## Installation

The accessibility system is included in the BFUI package:

```bash
npm install @bluefly/bfui
```

## Basic Usage

### Provider Setup

Wrap your application with the `AccessibilityProvider` to enable the accessibility system:

```tsx
import [AccessibilityProvider] from '@bluefly/bfui';

function App() {
  return (
    <AccessibilityProvider>
      <YourApp />
    </AccessibilityProvider>
  );
}
```

### Using the Hook

Use the `useAccessibility` hook to access the accessibility system in your components:

```tsx
import [useAccessibility] from '@bluefly/bfui';

function YourComponent() {
  const { 
    analyzeComponent, 
    results, 
    isAnalyzing, 
    applyFix, 
    generateReport, 
    clearResults 
  } = useAccessibility();
  
  // Component implementation...
}
```

### Analyzing Components

Analyze components for accessibility issues:

```tsx
import [useAccessibility] from '@bluefly/bfui';
import { useRef, useEffect } from 'react';

function YourComponent() {
  const { analyzeComponent, results } = useAccessibility();
  const componentRef = useRef(null);
  
  useEffect(() => {
    if (componentRef.current) {
      // Analyze the component
      analyzeComponent(componentRef.current);
    }
  }, []);
  
  return (
    <div>
      <div ref=[componentRef]>
        [/* Component to analyze */]
        <button onClick={() => alert('Clicked!')}>
          Click Me
        </button>
      </div>
      
      [/* Display accessibility issues */]
      {results.length > 0 && (
        <div className="accessibility-issues">
          <h3>Accessibility Issues:</h3>
          <ul>
            {results.map((issue, index) => (
              <li key={index}>[issue.message]</li>
            ))}
          </ul>
        </div>
      )}
    </div>
  );
}
```

## Advanced Configuration

### Provider Options

Configure the accessibility system with custom options:

```tsx
import { AccessibilityProvider, AccessibilityLevels, SeverityLevels } from '@bluefly/bfui';

function App() {
  return (
    <AccessibilityProvider
      options=[// WCAG Conformance level: 'A', 'AA', or 'AAA'
        conformance: AccessibilityLevels.AA,
        
        // Minimum severity level to report: 'error', 'warning', or 'info'
        severity: SeverityLevels.WARNING,
        
        // Whether to use AI for analysis (if available)
        useAI: true,
        
        // Whether to generate fix suggestions
        generateFixes: true,
        
        // Additional options...
        componentName: 'MyApp',
        filePath: 'src/components/MyApp.tsx',]
    >
      <YourApp />
    </AccessibilityProvider>
  );
}
```

### Handling Results

Process and display accessibility results:

```tsx
function AccessibilityIssueList() {
  const { results, applyFix } = useAccessibility();
  
  // Group issues by severity
  const errorIssues = results.filter(issue => issue.severity === 'error');
  const warningIssues = results.filter(issue => issue.severity === 'warning');
  const infoIssues = results.filter(issue => issue.severity === 'info');
  
  const handleApplyFix = async (issue) => {
    try {
      await applyFix(issue);
      alert('Fix applied successfully!');
    } catch (error) {
      console.error('Error applying fix:', error);
      alert('Failed to apply fix. See console for details.');
    }
  };
  
  return (
    <div className="accessibility-issues">
      {errorIssues.length > 0 && (
        <div className="error-issues">
          <h3>Critical Issues:</h3>
          <ul>
            {errorIssues.map((issue, index) => (
              <li key={index} className="severity-error">
                <strong>[issue.criterion]:</strong> [issue.message]
                {issue.fix && (
                  <button onClick={() => handleApplyFix(issue)}>
                    Apply Fix
                  </button>
                )}
              </li>
            ))}
          </ul>
        </div>
      )}
      
      {warningIssues.length > 0 && (
        <div className="warning-issues">
          <h3>Warnings:</h3>
          <ul>
            {warningIssues.map((issue, index) => (
              <li key={index} className="severity-warning">
                <strong>[issue.criterion]:</strong> [issue.message]
                {issue.fix && (
                  <button onClick={() => handleApplyFix(issue)}>
                    Apply Fix
                  </button>
                )}
              </li>
            ))}
          </ul>
        </div>
      )}
      
      {infoIssues.length > 0 && (
        <div className="info-issues">
          <h3>Suggestions:</h3>
          <ul>
            {infoIssues.map((issue, index) => (
              <li key={index} className="severity-info">
                <strong>[issue.criterion]:</strong> [issue.message]
                {issue.fix && (
                  <button onClick={() => handleApplyFix(issue)}>
                    Apply Fix
                  </button>
                )}
              </li>
            ))}
          </ul>
        </div>
      )}
    </div>
  );
}
```

### Generating Reports

Generate accessibility reports for documentation:

```tsx
async function generateAccessibilityReport() [const { generateReport] = useAccessibility();
  
  try {
    const report = await generateReport();
    
    // Do something with the report
    console.log(report);
    
    // Example: Save to file (requires backend integration)
    // await saveToFile(report, 'accessibility-report.md');
    
    // Example: Display in UI
    setReportContent(report);
    
    return report;
  } catch (error) {
    console.error('Error generating report:', error);
    return null;
  }
}
```

## Components

### AccessibilityChecker

A component for displaying accessibility analysis:

```tsx
import [AccessibilityChecker] from '@bluefly/bfui';

function YourComponent() {
  return (
    <div>
      <h2>Accessibility Check</h2>
      <AccessibilityChecker
        component={<YourComponentToAnalyze />}
        source=[`<YourComponentToAnalyze />`]
      />
    </div>
  );
}
```

### AccessibilityPreview

A component for previewing how content might appear to users with different abilities:

```tsx
import [AccessibilityPreview] from '@bluefly/bfui';

function YourComponent() {
  const [previewMode, setPreviewMode] = useState('normal');
  
  return (
    <div>
      <h2>Accessibility Preview</h2>
      <div>
        <button onClick={() => setPreviewMode('normal')}>Normal</button>
        <button onClick={() => setPreviewMode('colorblind')}>Colorblind</button>
        <button onClick={() => setPreviewMode('high-contrast')}>High Contrast</button>
        <button onClick={() => setPreviewMode('screen-reader')}>Screen Reader</button>
      </div>
      <AccessibilityPreview
        component=[<YourComponentToPreview />]
        mode=[previewMode]
      />
    </div>
  );
}
```

### AccessibilityScore

A component for displaying an accessibility score:

```tsx
import [AccessibilityScore] from '@bluefly/bfui';

function YourComponent() [const { results] = useAccessibility();
  
  return (
    <div>
      <h2>Accessibility Score</h2>
      <AccessibilityScore results=[results] />
    </div>
  );
}
```

## Type Definitions

### AccessibilityIssue

```typescript
interface AccessibilityIssue {
  criterion: string;
  message: string;
  severity: 'error' | 'warning' | 'info';
  element?: string;
  impact?: 'minor' | 'moderate' | 'serious' | 'critical';
  details?: string;
  fix?: string;
}
```

### AccessibilityResult

```typescript
interface AccessibilityResult {
  score: number;
  issues: AccessibilityIssue[];
  wcagCriteria: string[];
  passes?: any[];
  violations?: any[];
  incomplete?: any[];
  inapplicable?: any[];
}
```

### AccessibilityOptions

```typescript
interface AccessibilityOptions {
  componentName?: string;
  filePath?: string;
  conformance?: 'A' | 'AA' | 'AAA';
  severity?: 'error' | 'warning' | 'info';
  useAI?: boolean;
  generateFixes?: boolean;
}
```

### AccessibilityContextValue

```typescript
interface AccessibilityContextValue {
  results: AccessibilityIssue[];
  isAnalyzing: boolean;
  analyzeComponent: (source: any, options?: AccessibilityOptions) => Promise<AccessibilityResult>;
  applyFix: (issue: AccessibilityIssue) => Promise<void>;
  generateReport: () => Promise<string>;
  clearResults: () => void;
}
```

## Best Practices

### Integration with Development Workflow

1. **Use during development**
   Integrate accessibility checking during component development

2. **Include in CI/CD pipeline**
   Add automated accessibility testing to your CI/CD pipeline

3. **Regular audits**
   Perform regular accessibility audits of your application

### Handling Issues

1. **Prioritize by severity**
   Address critical issues first

2. **Test fixes with assistive technology**
   Verify fixes with screen readers and other assistive technology

3. **Document accessibility features**
   Include accessibility features in component documentation

## Common Accessibility Checks

The system checks for common accessibility issues, including:

### Content

- Missing alternative text for images
- Improper heading structure
- Missing document language
- Insufficient color contrast
- Text size too small

### Interaction

- Missing keyboard accessibility
- Missing focus indicators
- Touch target size too small
- Timing restrictions
- Complex gestures without alternatives

### Navigation

- Missing skip links
- Inaccessible navigation
- Missing page regions
- Missing breadcrumbs
- Poorly structured links

### Forms

- Missing form labels
- Missing form validation
- Missing error messages
- Missing fieldset and legend
- Complex forms without guidance

## Common WCAG Criteria

The system validates against common WCAG 2.1 criteria, including:

- **1.1.1 Non-text Content** (A)
- **1.3.1 Info and Relationships** (A)
- **1.3.2 Meaningful Sequence** (A)
- **1.4.3 Contrast (Minimum)** (AA)
- **1.4.4 Resize Text** (AA)
- **2.1.1 Keyboard** (A)
- **2.4.3 Focus Order** (A)
- **2.4.6 Headings and Labels** (AA)
- **3.3.2 Labels or Instructions** (A)
- **4.1.2 Name, Role, Value** (A)

## Troubleshooting

### Common Issues

1. **False positives**
   - Verify with manual testing
   - Adjust conformance level if needed

2. **Performance issues**
   - Limit analysis to specific components
   - Use in development environment only

3. **Integration issues**
   - Ensure provider is at the top level
   - Check for context conflicts

## Resources

### WCAG Guidelines

- [WCAG 2.1 Guidelines](https://www.w3.org/TR/WCAG21/)
- [WCAG Quick Reference](https://www.w3.org/WAI/WCAG21/quickref/)

### Accessibility Testing Tools

- [axe](https://www.deque.com/axe/)
- [WAVE](https://wave.webaim.org/)
- [Lighthouse](https://developers.google.com/web/tools/lighthouse)

### Screen Readers

- [NVDA](https://www.nvaccess.org/) (Windows)
- [JAWS](https://www.freedomscientific.com/products/software/jaws/) (Windows)
- [VoiceOver](https://www.apple.com/accessibility/mac/vision/) (macOS)
- [TalkBack](https://support.google.com/accessibility/android/answer/6283677) (Android)

## License

The BFUI Accessibility System is licensed under the MIT License.