# Accessible Component Library

## Overview

The Accessible Component Library is a collection of WCAG 2.1 AA compliant React components designed to provide a seamless and inclusive user experience for all users, including those with disabilities. These components work with both JavaScript and TypeScript projects and include built-in accessibility analysis.

## Key Features

- **Accessibility First**: WCAG 2.1 AA compliant components with ARIA attributes
- **Keyboard Navigation**: Full keyboard support for all interactive elements
- **Screen Reader Support**: Optimized for screen readers and assistive technologies
- **Real-time Accessibility Analysis**: Built-in validation and reporting
- **TypeScript Support**: Full type safety with comprehensive interfaces
- **Dark Mode Compatible**: Support for light and dark color schemes
- **Responsive Design**: Mobile-first approach for all screen sizes
- **Minimal Dependencies**: Self-contained with no external accessibility packages

## Installation

The accessible components are included in the BFUI package:

```bash
npm install @bluefly/bfui
```

## Usage

Import components directly from the package:

```tsx
import { 
  AccessibleButton, 
  AccessibleInput, 
  AccessibleSelect 
} from '@bluefly/bfui';

function App() {
  return (
    <div>
      <AccessibleInput 
        label="Email Address" 
        type="email" 
        required 
      />
      <AccessibleButton variant="primary">
        Submit
      </AccessibleButton>
    </div>
  );
}
```

## Accessibility System

The components use a built-in accessibility system that provides real-time analysis and validation.

```tsx
import { 
  AccessibilityProvider, 
  useAccessibility 
} from '@bluefly/bfui';

function App() {
  return (
    <AccessibilityProvider>
      <YourComponents />
    </AccessibilityProvider>
  );
}

function YourComponent() {
  const { analyzeComponent, results } = useAccessibility();
  
  // You can analyze any component for accessibility issues
  useEffect(() => {
    analyzeComponent(myComponentRef.current);
  }, []);
  
  // Display any accessibility issues
  return (
    <div>
      {results.length > 0 && (
        <div className="accessibility-issues">
          <h3>Accessibility Issues:</h3>
          <ul>
            {results.map((issue, index) => (
              <li key={index}>[issue.message]</li>
            ))}
          </ul>
        </div>
      )}
    </div>
  );
}
```

## Components

### AccessibleButton

A versatile button component with built-in accessibility features.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| variant | 'primary' \| 'secondary' \| 'danger' \| 'success' \| 'warning' | 'primary' | Visual style variant |
| size | 'sm' \| 'md' \| 'lg' | 'md' | Size variant |
| isLoading | boolean | false | Whether the button is in a loading state |
| disabled | boolean | false | Whether the button is disabled |
| leftIcon | ReactNode | - | Icon element to display before button text |
| rightIcon | ReactNode | - | Icon element to display after button text |
| fullWidth | boolean | false | Whether button takes full width of container |
| className | string | '' | Additional CSS classes |
| children | ReactNode | required | Button content |
| ...rest | ButtonHTMLAttributes<HTMLButtonElement> | - | Native button attributes |

#### Examples

```tsx
// Basic button
<AccessibleButton>Click Me</AccessibleButton>

// Primary variant with loading state
<AccessibleButton variant="primary" isLoading>
  Loading...
</AccessibleButton>

// Disabled button
<AccessibleButton disabled>
  Cannot Click
</AccessibleButton>

// Button with icons
<AccessibleButton 
  leftIcon=[<span>←</span>] 
  rightIcon=[<span>→</span>]
>
  Navigate
</AccessibleButton>

// Full width button
<AccessibleButton fullWidth variant="success">
  Save Changes
</AccessibleButton>
```

### AccessibleInput

An accessible input component with built-in validation and helper text.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| label | string | required | Label for the input field |
| helperText | string | - | Helper text to display below the input |
| error | string | - | Error message to display when validation fails |
| isLoading | boolean | false | Whether to show a loading state |
| startIcon | ReactNode | - | Icon to display at the start of the input |
| endIcon | ReactNode | - | Icon to display at the end of the input |
| showCounter | boolean | false | Whether to show a character counter |
| maxLength | number | - | Maximum number of characters allowed |
| validate | (value: string) => string \| undefined | - | Custom validation function |
| analyzeOnMount | boolean | true | Whether to analyze accessibility on mount |
| wrapperClassName | string | '' | Custom class name for the input wrapper |
| inputClassName | string | '' | Custom class name for the input element |
| labelClassName | string | '' | Custom class name for the label |
| helperTextClassName | string | '' | Custom class name for the helper text |
| errorClassName | string | '' | Custom class name for the error message |
| ...rest | InputHTMLAttributes<HTMLInputElement> | - | Native input attributes |

#### Examples

```tsx
// Basic input
<AccessibleInput 
  label="Username" 
  placeholder="Enter your username" 
/>

// Input with helper text
<AccessibleInput 
  label="Email" 
  type="email" 
  helperText="We'll never share your email" 
/>

// Input with error state
<AccessibleInput 
  label="Password" 
  type="password" 
  error="Password must be at least 8 characters" 
/>

// Input with character counter
<AccessibleInput 
  label="Bio" 
  maxLength=[100] 
  showCounter 
/>

// Input with icons
<AccessibleInput 
  label="Search" 
  startIcon=[<SearchIcon />] 
  endIcon=[<ClearIcon />] 
/>

// Input with custom validation
<AccessibleInput 
  label="Age" 
  type="number" 
  validate={(value) => {
    const age = Number(value);
    if (isNaN(age)) return "Please enter a valid number";
    if (age < 18) return "You must be at least 18 years old";
    if (age > 120) return "Please enter a valid age";
    return undefined;
  }}
/>
```

### AccessibleSelect

An accessible select component with search functionality and grouped options.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| label | string | required | Label for the select field |
| options | SelectOption[] | required | Options for the select field |
| helperText | string | - | Helper text to display below the select |
| error | string | - | Error message to display when validation fails |
| isLoading | boolean | false | Whether to show a loading state |
| startIcon | ReactNode | - | Icon to display at the start of the select |
| endIcon | ReactNode | - | Icon to display at the end of the select |
| searchable | boolean | false | Whether to show a search input for filtering options |
| placeholder | string | 'Select an option' | Placeholder text when no option is selected |
| validate | (value: string) => string \| undefined | - | Custom validation function |
| analyzeOnMount | boolean | true | Whether to analyze accessibility on mount |
| wrapperClassName | string | '' | Custom class name for the select wrapper |
| selectClassName | string | '' | Custom class name for the select element |
| labelClassName | string | '' | Custom class name for the label |
| helperTextClassName | string | '' | Custom class name for the helper text |
| errorClassName | string | '' | Custom class name for the error message |
| dropdownClassName | string | '' | Custom class name for the dropdown |
| searchClassName | string | '' | Custom class name for the search input |
| optionClassName | string | '' | Custom class name for the option |
| groupClassName | string | '' | Custom class name for the option group |
| onChange | (value: string, option: SelectOption) => void | - | Callback when the selected value changes |
| ...rest | SelectHTMLAttributes<HTMLSelectElement> | - | Native select attributes |

#### Examples

```tsx
// Basic select
<AccessibleSelect 
  label="Country" 
  options={[
    { value: "us", label: "United States" },
    { value: "ca", label: "Canada" },
    { value: "mx", label: "Mexico" },
  ]} 
/>

// Searchable select
<AccessibleSelect 
  label="State" 
  searchable
  options={[
    { value: "ny", label: "New York" },
    { value: "ca", label: "California" },
    { value: "tx", label: "Texas" },
    // ... more states
  ]} 
/>

// Grouped options
<AccessibleSelect 
  label="Fruit" 
  options={[
    { value: "apple", label: "Apple", group: "Common" },
    { value: "banana", label: "Banana", group: "Common" },
    { value: "orange", label: "Orange", group: "Common" },
    { value: "dragonfruit", label: "Dragon Fruit", group: "Exotic" },
    { value: "starfruit", label: "Star Fruit", group: "Exotic" },
  ]} 
/>

// With option descriptions
<AccessibleSelect 
  label="Plan" 
  options={[
    { value: "basic", label: "Basic", description: "$10/month - Basic features" },
    { value: "pro", label: "Pro", description: "$20/month - Pro features" },
    { value: "enterprise", label: "Enterprise", description: "Contact sales for pricing" },
  ]} 
/>

// Disabled option
<AccessibleSelect 
  label="Version" 
  options={[
    { value: "1.0", label: "Version 1.0" },
    { value: "2.0", label: "Version 2.0" },
    { value: "3.0", label: "Version 3.0", isDisabled: true },
  ]} 
/>
```

### AccessibleCheckbox

An accessible checkbox component with support for indeterminate state.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| label | string | required | Label for the checkbox |
| helperText | string | - | Helper text to display below the checkbox |
| error | string | - | Error message to display when validation fails |
| checked | boolean | - | Whether the checkbox is checked |
| defaultChecked | boolean | - | Default checked state |
| indeterminate | boolean | false | Whether the checkbox is indeterminate |
| required | boolean | false | Whether the checkbox is required |
| disabled | boolean | false | Whether the checkbox is disabled |
| analyzeOnMount | boolean | true | Whether to analyze accessibility on mount |
| wrapperClassName | string | '' | Custom class name for the checkbox wrapper |
| inputClassName | string | '' | Custom class name for the checkbox input |
| labelClassName | string | '' | Custom class name for the label |
| helperTextClassName | string | '' | Custom class name for the helper text |
| errorClassName | string | '' | Custom class name for the error message |
| onChange | (event: React.ChangeEvent<HTMLInputElement>) => void | - | Callback when the checkbox changes |
| ...rest | InputHTMLAttributes<HTMLInputElement> | - | Native input attributes |

#### Examples

```tsx
// Basic checkbox
<AccessibleCheckbox 
  label="I agree to the terms and conditions" 
/>

// Required checkbox
<AccessibleCheckbox 
  label="I agree to the terms and conditions" 
  required 
/>

// Checkbox with helper text
<AccessibleCheckbox 
  label="Subscribe to newsletter" 
  helperText="We'll send you weekly updates" 
/>

// Disabled checkbox
<AccessibleCheckbox 
  label="Admin access" 
  disabled 
/>

// Indeterminate checkbox (for "select all" patterns)
<AccessibleCheckbox 
  label="Select all" 
  indeterminate 
/>

// Checkbox with error
<AccessibleCheckbox 
  label="I agree to the terms" 
  error="You must agree to continue" 
/>
```

### AccessibleRadio

An accessible radio button group component.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| label | string | required | The label for the radio group |
| name | string | required | The name attribute for the radio inputs |
| options | RadioOption[] | required | The options for the radio group |
| value | string | - | The currently selected value |
| defaultValue | string | - | The default value |
| helperText | string | - | Helper text to display below the radio group |
| error | string | - | Error message to display |
| isRequired | boolean | false | Whether the radio group is required |
| isDisabled | boolean | false | Whether the radio group is disabled |
| isHorizontal | boolean | false | Whether to display the radio group in a horizontal layout |
| className | string | '' | Custom class name for the radio group |
| optionsClassName | string | '' | Custom class name for the radio options container |
| optionClassName | string | '' | Custom class name for individual radio options |
| onChange | (value: string) => void | - | Callback when the selected value changes |
| onBlur | () => void | - | Callback when the radio group loses focus |
| ariaDescribedby | string | - | Custom ARIA describedby ID |
| ...rest | InputHTMLAttributes<HTMLInputElement> | - | Native input attributes |

#### Examples

```tsx
// Basic radio group
<AccessibleRadio 
  label="Gender" 
  name="gender" 
  options={[
    { label: "Male", value: "male" },
    { label: "Female", value: "female" },
    { label: "Non-binary", value: "non-binary" },
    { label: "Prefer not to say", value: "not-specified" },
  ]} 
/>

// Horizontal layout
<AccessibleRadio 
  label="Rating" 
  name="rating" 
  isHorizontal
  options={[
    { label: "1", value: "1" },
    { label: "2", value: "2" },
    { label: "3", value: "3" },
    { label: "4", value: "4" },
    { label: "5", value: "5" },
  ]} 
/>

// With descriptions
<AccessibleRadio 
  label="Shipping Method" 
  name="shipping" 
  options={[
    { label: "Standard", value: "standard", description: "3-5 business days" },
    { label: "Express", value: "express", description: "1-2 business days" },
    { label: "Same-day", value: "same-day", description: "Delivered today", isDisabled: true },
  ]} 
/>

// Required radio group
<AccessibleRadio 
  label="T-Shirt Size" 
  name="size" 
  isRequired
  options={[
    { label: "Small", value: "s" },
    { label: "Medium", value: "m" },
    { label: "Large", value: "l" },
    { label: "X-Large", value: "xl" },
  ]} 
/>
```

### AccessibleTabs

An accessible tabs component with keyboard navigation.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| id | string | 'accessible-tabs' | Unique identifier for the tabs component |
| items | TabItem[] | required | Array of tab items |
| selectedTabId | string | - | Currently selected tab ID |
| defaultSelectedTabId | string | - | Default selected tab ID |
| onTabChange | (tabId: string) => void | - | Callback when tab selection changes |
| isDisabled | boolean | false | Whether the tabs are disabled |
| isLoading | boolean | false | Whether the tabs are loading |
| className | string | '' | Custom class name for the tabs container |
| tabListClassName | string | '' | Custom class name for the tab list |
| tabClassName | string | '' | Custom class name for individual tabs |
| tabPanelClassName | string | '' | Custom class name for the tab panel |
| showDescriptions | boolean | false | Whether to show tab descriptions |
| showIcons | boolean | false | Whether to show tab icons |
| isVertical | boolean | false | Whether to use vertical tabs |
| isCompact | boolean | false | Whether to use compact tabs |
| isFilled | boolean | false | Whether to use filled tabs |
| isOutlined | boolean | false | Whether to use outlined tabs |
| isPill | boolean | false | Whether to use pill tabs |
| isAnimated | boolean | false | Whether to use animated tabs |
| ariaLabel | string | - | Custom ARIA label for the tab list |
| ariaDescribedby | string | - | Custom ARIA describedby for the tab list |
| tabPanelAriaLabel | string | - | Custom ARIA label for the tab panel |
| tabPanelAriaDescribedby | string | - | Custom ARIA describedby for the tab panel |
| showAccessibilityIssues | boolean | true | Whether to show accessibility issues |

#### Examples

```tsx
// Basic tabs
<AccessibleTabs
  items={[
    { id: 'tab1', label: 'Tab 1', content: <div>Content for Tab 1</div> },
    { id: 'tab2', label: 'Tab 2', content: <div>Content for Tab 2</div> },
    { id: 'tab3', label: 'Tab 3', content: <div>Content for Tab 3</div> },
  ]}
/>

// Tabs with icons and descriptions
<AccessibleTabs
  showIcons
  showDescriptions
  items={[
    { 
      id: 'profile', 
      label: 'Profile', 
      description: 'View and edit your profile',
      icon: <UserIcon />,
      content: <ProfileContent /> 
    },
    { 
      id: 'settings', 
      label: 'Settings', 
      description: 'Adjust your preferences',
      icon: <GearIcon />,
      content: <SettingsContent /> 
    },
    { 
      id: 'notifications', 
      label: 'Notifications', 
      description: 'Manage your notifications',
      icon: <BellIcon />,
      content: <NotificationsContent /> 
    },
  ]}
/>

// Vertical tabs
<AccessibleTabs
  isVertical
  items={[
    { id: 'tab1', label: 'Section 1', content: <div>Content for Section 1</div> },
    { id: 'tab2', label: 'Section 2', content: <div>Content for Section 2</div> },
    { id: 'tab3', label: 'Section 3', content: <div>Content for Section 3</div> },
  ]}
/>

// Pill tabs
<AccessibleTabs
  isPill
  items={[
    { id: 'day', label: 'Day', content: <DayView /> },
    { id: 'week', label: 'Week', content: <WeekView /> },
    { id: 'month', label: 'Month', content: <MonthView /> },
  ]}
/>

// Tabs with callback
<AccessibleTabs
  items={[
    { id: 'tab1', label: 'Tab 1', content: <div>Content for Tab 1</div> },
    { id: 'tab2', label: 'Tab 2', content: <div>Content for Tab 2</div> },
  ]}
  onTabChange={(tabId) => console.log(`Tab changed to [tabId]`)}
/>
```

### AccessibleModal

An accessible modal dialog with focus trap and keyboard navigation.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| isOpen | boolean | required | Whether the modal is open |
| onClose | () => void | required | Callback when the modal is closed |
| title | string | required | Title for the modal |
| children | ReactNode | required | Content of the modal |
| description | string | - | Description text for the modal |
| showCloseButton | boolean | true | Whether to show the close button |
| className | string | '' | Additional CSS class for the modal |
| overlayClassName | string | '' | Additional CSS class for the modal overlay |

#### Examples

```tsx
// Basic modal
const [isOpen, setIsOpen] = useState(false);

<AccessibleButton onClick={() => setIsOpen(true)}>
  Open Modal
</AccessibleButton>

<AccessibleModal
  isOpen=[isOpen]
  onClose={() => setIsOpen(false)}
  title="Example Modal"
>
  <p>This is the modal content.</p>
  <AccessibleButton onClick={() => setIsOpen(false)}>
    Close
  </AccessibleButton>
</AccessibleModal>

// Modal with description
<AccessibleModal
  isOpen=[isOpen]
  onClose={() => setIsOpen(false)}
  title="Terms of Service"
  description="Please read our terms of service carefully"
>
  <div className="modal-content">
    <p>Terms content goes here...</p>
  </div>
  <div className="modal-footer">
    <AccessibleButton 
      variant="secondary" 
      onClick={() => setIsOpen(false)}
    >
      Decline
    </AccessibleButton>
    <AccessibleButton 
      variant="primary" 
      onClick={() => {
        acceptTerms();
        setIsOpen(false);
      }}
    >
      Accept
    </AccessibleButton>
  </div>
</AccessibleModal>

// Form in modal
<AccessibleModal
  isOpen=[isOpen]
  onClose={() => setIsOpen(false)}
  title="Create Account"
>
  <form onSubmit=[handleSubmit]>
    <AccessibleInput 
      label="Email" 
      type="email" 
      required 
    />
    <AccessibleInput 
      label="Password" 
      type="password" 
      required 
    />
    <div className="modal-footer">
      <AccessibleButton 
        type="button" 
        variant="secondary" 
        onClick={() => setIsOpen(false)}
      >
        Cancel
      </AccessibleButton>
      <AccessibleButton 
        type="submit" 
        variant="primary"
      >
        Create Account
      </AccessibleButton>
    </div>
  </form>
</AccessibleModal>
```

## AccessibilityDemo

The `AccessibilityDemo` component provides a showcase of the accessible components with real-time accessibility validation.

```tsx
import [AccessibilityDemo] from '@bluefly/bfui';

function App() {
  return (
    <div className="container">
      <h1>Accessibility Demo</h1>
      <AccessibilityDemo />
    </div>
  );
}
```

## Best Practices

### Form Components

1. **Always include labels**
   ```tsx
   // Good
   <AccessibleInput label="Email" />
   
   // Avoid
   <AccessibleInput placeholder="Email" />
   ```

2. **Use helper text for guidance**
   ```tsx
   <AccessibleInput 
     label="Password" 
     type="password" 
     helperText="Password must be at least 8 characters" 
   />
   ```

3. **Group related form controls**
   ```tsx
   <div role="group" aria-labelledby="address-group">
     <h3 id="address-group">Shipping Address</h3>
     <AccessibleInput label="Street Address" />
     <AccessibleInput label="City" />
     <AccessibleSelect 
       label="State" 
       options=[stateOptions] 
     />
     <AccessibleInput label="Zip Code" />
   </div>
   ```

### Keyboard Navigation

1. **Test with keyboard only**
   Ensure all components can be operated using only the keyboard

2. **Use logical tab order**
   Components should be navigable in a logical sequence

3. **Provide keyboard shortcuts for complex interactions**
   ```tsx
   <AccessibleTabs
     items=[tabItems]
     keyboardShortcuts=[next: 'ArrowRight',
       previous: 'ArrowLeft',
       first: 'Home',
       last: 'End',]
   />
   ```

### Screen Reader Support

1. **Test with actual screen readers**
   NVDA, JAWS, VoiceOver, etc.

2. **Use semantic HTML**
   Prefer native elements over ARIA when possible

3. **Provide text alternatives for non-text content**
   ```tsx
   <img src="logo.png" alt="Company Logo" />
   ```

## Debugging Accessibility Issues

The accessibility system provides tools for debugging accessibility issues:

```tsx
// Component with accessibility debugging
function DebugAccessibility() {
  const [showIssues, setShowIssues] = useState(true);
  const { analyzeComponent, results } = useAccessibility();
  const componentRef = useRef(null);
  
  useEffect(() => {
    if (componentRef.current) {
      analyzeComponent(componentRef.current);
    }
  }, []);
  
  return (
    <div>
      <div ref=[componentRef]>
        [/* Component to analyze */]
        <img src="chart.png" />
      </div>
      
      <AccessibleCheckbox
        label="Show Accessibility Issues"
        checked=[showIssues]
        onChange={(e) => setShowIssues(e.target.checked)}
      />
      
      {showIssues && results.length > 0 && (
        <div className="accessibility-issues">
          <h3>Accessibility Issues:</h3>
          <ul>
            {results.map((issue, index) => (
              <li key={index} className=[`severity-[issue.severity]`]>
                <strong>[issue.criterion]:</strong> [issue.message]
                {issue.fix && (
                  <div className="fix-suggestion">
                    <strong>Suggested Fix:</strong> {issue.fix}
                  </div>
                )}
              </li>
            ))}
          </ul>
        </div>
      )}
    </div>
  );
}
```

## Accessibility Levels

The library supports different levels of accessibility conformance:

```tsx
import [AccessibilityLevels] from '@bluefly/bfui';

<AccessibilityProvider
  options=[conformance: AccessibilityLevels.AA, // 'A', 'AA', or 'AAA'
    severity: 'error', // 'error', 'warning', or 'info'
    useAI: true,
    generateFixes: true,]
>
  <YourApp />
</AccessibilityProvider>
```

## Browser Support

The accessible components support all modern browsers:

- Chrome (latest 2 versions)
- Firefox (latest 2 versions)
- Safari (latest 2 versions)
- Edge (latest 2 versions)

## License

The BFUI Accessible Component Library is licensed under the MIT License.