# Accessibility API Reference

## Overview

This document provides a comprehensive API reference for the BFUI Accessibility System, including all components, hooks, and types.

## Table of Contents

- [Context & Provider](#context--provider)
- [Hooks](#hooks)
- [Components](#components)
- [Types & Interfaces](#types--interfaces)
- [Constants](#constants)
- [Utilities](#utilities)

## Context & Provider

### AccessibilityContext

The React context that provides accessibility functionality to components.

```typescript
const AccessibilityContext = createContext<AccessibilityContextValue | null>(null);
```

### AccessibilityProvider

The provider component that makes the accessibility system available to its children.

```typescript
interface AccessibilityProviderProps {
  children: React.ReactNode;
  options?: AccessibilityOptions;
}

const AccessibilityProvider: React.FC<AccessibilityProviderProps> = ({
  children,
  options = {
    conformance: 'AA',
    severity: 'error',
    useAI: true,
    generateFixes: true,
  },
}) => [// Implementation...];
```

#### Props

| Name | Type | Default | Description |
|------|------|---------|-------------|
| children | React.ReactNode | required | The child components that will have access to the accessibility context |
| options | AccessibilityOptions | see above | Configuration options for the accessibility system |

#### Example

```tsx
import [AccessibilityProvider] from '@bluefly/bfui';

function App() {
  return (
    <AccessibilityProvider
      options=[conformance: 'AA',
        severity: 'warning',
        useAI: true,
        generateFixes: true,]
    >
      <YourApp />
    </AccessibilityProvider>
  );
}
```

## Hooks

### useAccessibility

Hook that provides access to the accessibility context.

```typescript
function useAccessibility(): AccessibilityContextValue;
```

#### Return Value

Returns an object with the following properties:

| Name | Type | Description |
|------|------|-------------|
| results | AccessibilityIssue[] | Array of accessibility issues found |
| isAnalyzing | boolean | Whether the system is currently analyzing |
| analyzeComponent | (source: any, options?: AccessibilityOptions) => Promise<AccessibilityResult> | Function to analyze a component for accessibility issues |
| applyFix | (issue: AccessibilityIssue) => Promise<void> | Function to apply a suggested fix for an issue |
| generateReport | () => Promise<string> | Function to generate an accessibility report |
| clearResults | () => void | Function to clear the current results |

#### Example

```tsx
import [useAccessibility] from '@bluefly/bfui';

function MyComponent() {
  const { analyzeComponent, results, isAnalyzing } = useAccessibility();
  const componentRef = useRef(null);
  
  useEffect(() => {
    if (componentRef.current) {
      analyzeComponent(componentRef.current);
    }
  }, []);
  
  return (
    <div>
      <div ref=[componentRef]>
        [/* Component to analyze */]
      </div>
      
      {isAnalyzing ? (
        <p>Analyzing...</p>
      ) : (
        results.length > 0 && (
          <div>
            <h3>Accessibility Issues:</h3>
            <ul>
              {results.map((issue, index) => (
                <li key={index}>[issue.message]</li>
              ))}
            </ul>
          </div>
        )
      )}
    </div>
  );
}
```

## Components

### AccessibilityChecker

Component that displays accessibility analysis for a given component.

```typescript
interface AccessibilityCheckerProps {
  component: React.ReactNode;
  source: string;
}

const AccessibilityChecker: React.FC<AccessibilityCheckerProps> = ({
  component,
  source
}) => [// Implementation...];
```

#### Props

| Name | Type | Description |
|------|------|-------------|
| component | React.ReactNode | The component to analyze |
| source | string | The source code of the component (for display purposes) |

#### Example

```tsx
import [AccessibilityChecker] from '@bluefly/bfui';

function Demo() {
  return (
    <div>
      <h2>Accessibility Analysis</h2>
      <AccessibilityChecker
        component={<Button>Click Me</Button>}
        source=[`<Button>Click Me</Button>`]
      />
    </div>
  );
}
```

### AccessibilityPreview

Component that provides different viewing modes for accessibility testing.

```typescript
interface AccessibilityPreviewProps {
  component: React.ReactNode;
  mode?: 'normal' | 'colorblind' | 'high-contrast' | 'screen-reader';
}

const AccessibilityPreview: React.FC<AccessibilityPreviewProps> = ({
  component,
  mode = 'normal'
}) => [// Implementation...];
```

#### Props

| Name | Type | Default | Description |
|------|------|---------|-------------|
| component | React.ReactNode | required | The component to preview |
| mode | 'normal' \| 'colorblind' \| 'high-contrast' \| 'screen-reader' | 'normal' | The preview mode to use |

#### Example

```tsx
import [AccessibilityPreview] from '@bluefly/bfui';

function Demo() {
  const [mode, setMode] = useState('normal');
  
  return (
    <div>
      <div className="controls">
        <button onClick={() => setMode('normal')}>Normal</button>
        <button onClick={() => setMode('colorblind')}>Colorblind</button>
        <button onClick={() => setMode('high-contrast')}>High Contrast</button>
        <button onClick={() => setMode('screen-reader')}>Screen Reader</button>
      </div>
      
      <AccessibilityPreview
        component=[<YourComponent />]
        mode=[mode]
      />
    </div>
  );
}
```

### AccessibilityScore

Component that displays an accessibility score based on issues found.

```typescript
interface AccessibilityScoreProps {
  results: AccessibilityIssue[];
}

const AccessibilityScore: React.FC<AccessibilityScoreProps> = ([results]) => [// Implementation...];
```

#### Props

| Name | Type | Description |
|------|------|-------------|
| results | AccessibilityIssue[] | Array of accessibility issues to calculate the score from |

#### Example

```tsx
import [AccessibilityScore] from '@bluefly/bfui';

function Demo() [const { results] = useAccessibility();
  
  return (
    <div>
      <h2>Accessibility Score</h2>
      <AccessibilityScore results=[results] />
    </div>
  );
}
```

## Types & Interfaces

### AccessibilityIssue

Represents a single accessibility issue.

```typescript
interface AccessibilityIssue {
  criterion: string;
  message: string;
  severity: 'error' | 'warning' | 'info';
  element?: string;
  impact?: 'minor' | 'moderate' | 'serious' | 'critical';
  details?: string;
  fix?: string;
}
```

| Property | Type | Description |
|----------|------|-------------|
| criterion | string | The WCAG criterion that the issue relates to |
| message | string | A descriptive message about the issue |
| severity | 'error' \| 'warning' \| 'info' | The severity level of the issue |
| element | string? | The element or component with the issue |
| impact | 'minor' \| 'moderate' \| 'serious' \| 'critical'? | The impact level of the issue |
| details | string? | Additional details about the issue |
| fix | string? | A suggested fix for the issue |

### AccessibilityResult

Represents the result of an accessibility analysis.

```typescript
interface AccessibilityResult {
  score: number;
  issues: AccessibilityIssue[];
  wcagCriteria: string[];
  passes?: any[];
  violations?: any[];
  incomplete?: any[];
  inapplicable?: any[];
}
```

| Property | Type | Description |
|----------|------|-------------|
| score | number | The overall accessibility score (0-100) |
| issues | AccessibilityIssue[] | Array of accessibility issues found |
| wcagCriteria | string[] | Array of WCAG criteria that were checked |
| passes | any[]? | Array of checks that passed |
| violations | any[]? | Array of checks that failed |
| incomplete | any[]? | Array of checks that couldn't be completed |
| inapplicable | any[]? | Array of checks that weren't applicable |

### AccessibilityOptions

Configuration options for the accessibility system.

```typescript
interface AccessibilityOptions {
  componentName?: string;
  filePath?: string;
  conformance?: 'A' | 'AA' | 'AAA';
  severity?: 'error' | 'warning' | 'info';
  useAI?: boolean;
  generateFixes?: boolean;
}
```

| Property | Type | Description |
|----------|------|-------------|
| componentName | string? | The name of the component being analyzed |
| filePath | string? | The file path of the component |
| conformance | 'A' \| 'AA' \| 'AAA'? | The WCAG conformance level to check against |
| severity | 'error' \| 'warning' \| 'info'? | The minimum severity level to report |
| useAI | boolean? | Whether to use AI for analysis |
| generateFixes | boolean? | Whether to generate fix suggestions |

### AccessibilitySystemInterface

Interface for the accessibility system implementation.

```typescript
interface AccessibilitySystemInterface {
  checkContrast: (foreground: string, background: string) => boolean;
  validateAria: (element: HTMLElement) => string[];
  checkKeyboardNavigation: () => boolean;
  runA11yAudit: (component: any) => Promise<AccessibilityResult>;
}
```

| Method | Parameters | Return Type | Description |
|--------|------------|-------------|-------------|
| checkContrast | (foreground: string, background: string) | boolean | Checks if the contrast ratio meets WCAG requirements |
| validateAria | (element: HTMLElement) | string[] | Validates ARIA attributes on an element |
| checkKeyboardNavigation | () | boolean | Checks if keyboard navigation is properly implemented |
| runA11yAudit | (component: any) | Promise<AccessibilityResult> | Runs a full accessibility audit on a component |

### AccessibilityContextValue

Interface for the value provided by the accessibility context.

```typescript
interface AccessibilityContextValue {
  results: AccessibilityIssue[];
  isAnalyzing: boolean;
  analyzeComponent: (source: any, options?: AccessibilityOptions) => Promise<AccessibilityResult>;
  applyFix: (issue: AccessibilityIssue) => Promise<void>;
  generateReport: () => Promise<string>;
  clearResults: () => void;
}
```

| Property | Type | Description |
|----------|------|-------------|
| results | AccessibilityIssue[] | Array of accessibility issues found |
| isAnalyzing | boolean | Whether the system is currently analyzing |
| analyzeComponent | (source: any, options?: AccessibilityOptions) => Promise<AccessibilityResult> | Function to analyze a component for accessibility issues |
| applyFix | (issue: AccessibilityIssue) => Promise<void> | Function to apply a suggested fix for an issue |
| generateReport | () => Promise<string> | Function to generate an accessibility report |
| clearResults | () => void | Function to clear the current results |

## Constants

### AccessibilityLevels

Constants for WCAG conformance levels.

```typescript
export const AccessibilityLevels = {
  A: 'A',
  AA: 'AA',
  AAA: 'AAA',
};
```

| Value | Description |
|-------|-------------|
| A | WCAG Level A conformance |
| AA | WCAG Level AA conformance |
| AAA | WCAG Level AAA conformance |

### SeverityLevels

Constants for issue severity levels.

```typescript
export const SeverityLevels = {
  ERROR: 'error',
  WARNING: 'warning',
  INFO: 'info',
};
```

| Value | Description |
|-------|-------------|
| ERROR | Critical issues that must be fixed |
| WARNING | Issues that should be addressed |
| INFO | Informational issues or suggestions |

## Utilities

### calculateScore

Utility function that calculates an accessibility score based on issues.

```typescript
function calculateScore(issues: AccessibilityIssue[]): number;
```

#### Parameters

| Name | Type | Description |
|------|------|-------------|
| issues | AccessibilityIssue[] | Array of accessibility issues |

#### Return Value

Returns a number between 0 and 100 representing the accessibility score.

#### Example

```typescript
import [calculateScore] from '@bluefly/bfui/utils';

const issues = [
  { criterion: '1.1.1', message: 'Image missing alt text', severity: 'error' },
  { criterion: '1.4.3', message: 'Low contrast', severity: 'warning' },
];

const score = calculateScore(issues); // Returns a score like 80
```

### generateA11yReport

Utility function that generates a comprehensive accessibility report.

```typescript
function generateA11yReport(result: AccessibilityResult): string;
```

#### Parameters

| Name | Type | Description |
|------|------|-------------|
| result | AccessibilityResult | The result of an accessibility analysis |

#### Return Value

Returns a string containing a formatted accessibility report.

#### Example

```typescript
import [generateA11yReport] from '@bluefly/bfui/utils';

const result = {
  score: 85,
  issues: [
    { criterion: '1.1.1', message: 'Image missing alt text', severity: 'error' },
    { criterion: '1.4.3', message: 'Low contrast', severity: 'warning' },
  ],
  wcagCriteria: ['1.1.1', '1.4.3', '2.1.1', '2.4.6', '3.3.2', '4.1.2'],
};

const report = generateA11yReport(result);
console.log(report);
```

### suggestFix

Utility function that suggests fixes for common accessibility issues.

```typescript
function suggestFix(issue: AccessibilityIssue): string | null;
```

#### Parameters

| Name | Type | Description |
|------|------|-------------|
| issue | AccessibilityIssue | The accessibility issue to suggest a fix for |

#### Return Value

Returns a string with a suggested fix, or null if no suggestion is available.

#### Example

```typescript
import [suggestFix] from '@bluefly/bfui/utils';

const issue = {
  criterion: '1.1.1',
  message: 'Image missing alt text',
  severity: 'error',
  element: '<img src="logo.png">',
};

const fix = suggestFix(issue);
console.log(fix); // '<img src="logo.png" alt="Logo">'
```