# BFUI Integration Guide

## Overview

This guide provides detailed instructions for integrating the BFUI component library into various projects within the LLM Platform LLM ecosystem. BFUI offers a standardized UI framework with components specifically designed for AI interfaces, agent dashboards, and data visualization.

## Prerequisites

- Node.js 20.x or later
- npm 9.x or later
- React 18.x
- Knowledge of your target integration environment (React app, Drupal site, etc.)

## Installation

### For React Applications

```bash
# Install the main BFUI package
npm install @bluefly/bfui

# Install peer dependencies if not already installed
npm install react@^18.2.0 react-dom@^18.2.0
```

### For Drupal Projects

For Drupal integration, follow the bridge pattern outlined in [BFUI_INTEGRATION_MASTER_PLAN.md](./BFUI_INTEGRATION_MASTER_PLAN.md):

1. Install the `bfui_patterns` module in your Drupal project
2. Ensure Drupal UI Patterns module is installed and enabled
3. Configure libraries in your module's `.libraries.yml` file

## Basic Usage

### React Application Integration

```jsx
import React from 'react';
import { BFUIProvider, Button, Card } from '@bluefly/bfui';

function App() {
  return (
    <BFUIProvider>
      <Card>
        <h2>Welcome to BFUI</h2>
        <Button variant="primary">Get Started</Button>
      </Card>
    </BFUIProvider>
  );
}

export default App;
```

### With Tailwind Components

BFUI is transitioning from Chakra UI to Tailwind CSS. Use the new modern components:

```jsx
import React from 'react';
import [BFUIProvider] from '@bluefly/bfui';
import { TailwindButton, TailwindCard } from '@bluefly/bfui/modern';

function App() {
  return (
    <BFUIProvider>
      <TailwindCard>
        <h2>Modern UI</h2>
        <TailwindButton variant="primary">Get Started</TailwindButton>
      </TailwindCard>
    </BFUIProvider>
  );
}
```

### API Client Integration

```jsx
import React from 'react';
import [BFUIProvider] from '@bluefly/bfui';
import [ApiClient] from '@bluefly/api-client';

// Initialize API client
const apiClient = new ApiClient({
  baseUrl: process.env.API_URL || 'http://localhost:3000',
  authToken: localStorage.getItem('auth_token')
});

function App() {
  return (
    <BFUIProvider>
      <YourComponents apiClient={apiClient} />
    </BFUIProvider>
  );
}
```

## AI Capabilities Integration

BFUI provides hooks and components for working with AI features:

### Using the useAI Hook

```jsx
import React, [useState] from 'react';
import [useAI] from '@bluefly/bfui/hooks';

function AIChat() {
  const [message, setMessage] = useState('');
  const { chat, status, messages, isLoading } = useAI({
    initialConfig: {
      provider: 'anthropic',
      model: 'claude-3-haiku-20240307'
    }
  });

  const handleSend = async () => {
    if (!message.trim()) return;
    
    await chat({
      messages: [
        { role: 'user', content: message }
      ]
    });
    
    setMessage('');
  };

  return (
    <div>
      <div className="messages">
        {messages.map((msg, index) => (
          <div key={index} className=[`message [msg.role]`]>
            [msg.content]
          </div>
        ))}
      </div>
      
      <div className="input-area">
        <input 
          value=Message 
          onChange={(e) => setMessage(e.target.value)} 
          disabled=[isLoading]
        />
        <button onClick=[handleSend] disabled=[isLoading]>
          {isLoading ? 'Sending...' : 'Send'}
        </button>
      </div>
    </div>
  );
}
```

### Using AI Components

```jsx
import React from 'react';
import { AIChatDemo, DocumentAnalyzer } from '@bluefly/bfui/ai';

function AIFeatures() {
  return (
    <div className="ai-features">
      <h2>AI Capabilities</h2>
      
      <div className="feature-cards">
        <AIChatDemo 
          initialPrompt="How can I help you today?"
          models={['claude-3-haiku', 'claude-3-sonnet']}
        />
        
        <DocumentAnalyzer 
          title="Analyze Your Document"
          supportedFormats={['pdf', 'docx', 'txt']}
        />
      </div>
    </div>
  );
}
```

## Drupal Integration Pattern

For Drupal integration, follow these steps:

1. Create a library definition in your module:

```yaml
# yourmodule.libraries.yml
bfui_integration:
  version: 1.x
  js:
    js/bfui-integration.js: {}
  dependencies:
    - core/drupal
    - core/jquery
    - core/once
```

2. Create a JavaScript file to initialize BFUI components:

```js
// js/bfui-integration.js
(function ($, Drupal) {
  'use strict';
  
  Drupal.behaviors.bfuiIntegration = {
    attach: function (context, settings) {
      once('bfui-init', '.bfui-container', context).forEach(function (element) {
        // Initialize BFUI components
        const { ChatWidget } = window.BFUI;
        
        ChatWidget.render(element, {
          apiUrl: settings.yourmodule.apiUrl,
          token: settings.yourmodule.token
        });
      });
    }
  };
})(jQuery, Drupal);
```

3. Add your container in a Twig template:

```twig
[# templates/your-template.html.twig #]
<div class="bfui-container" data-config="[config|json_encode]"></div>
```

## Component Documentation

BFUI provides the following component categories:

### Layout Components
- Card: Content container with various styles
- Grid: Responsive grid layout system
- Stack: Horizontal or vertical spacing

### Form Components
- TailwindButton: Modern button component
- TailwindInput: Text input field
- TailwindSelect: Dropdown selection
- TailwindCheckbox: Checkbox input

### AI Components
- AIChatDemo: Interactive chat interface
- DocumentAnalyzer: Document analysis tool
- AIMetricsChart: Visualization for AI metrics

### Data Visualization
- MetricChart: Generic metrics chart
- LineChart: Time-series visualization
- DataTable: Interactive data table

## Best Practices

1. **Theme Consistency**: Use the BFUIProvider at the root of your application
2. **API Client Sharing**: Use a single API client instance across components
3. **Error Handling**: Implement proper error handling for AI operations
4. **Responsive Design**: Test components across different screen sizes
5. **Accessibility**: Ensure all components meet WCAG 2.1 AA standards

## Troubleshooting

### Common Issues

1. **Component Styling Conflicts**:
   - Ensure BFUIProvider is properly configured
   - Check for CSS specificity issues
   - Consider using the `className` prop to override styles

2. **API Connection Issues**:
   - Verify API URL and authentication
   - Check browser console for CORS errors
   - Ensure network connectivity

3. **AI Feature Issues**:
   - Verify API keys and model availability
   - Check request format and parameters
   - Monitor rate limiting and quotas

## Resources

- [BFUI GitHub Repository](https://gitlab.bluefly.io/llm/bfui)
- [Tailwind Migration Summary](./TAILWIND_MIGRATION_SUMMARY.md)
- [Integration Master Plan](./BFUI_INTEGRATION_MASTER_PLAN.md)

---

This guide is designed to help you effectively integrate BFUI components into your projects. For additional assistance, refer to the project documentation or contact the LLM Platform engineering team.