# Modern Component Library

## Overview

The Modern Component Library is a collection of React components built with Tailwind CSS, providing a modern, flexible, and performance-focused alternative to the legacy Chakra UI-based components.

## Key Features

- **Tailwind CSS**: Utility-first CSS framework for rapid UI development
- **TypeScript**: Full type safety with comprehensive prop interfaces
- **Accessibility**: WCAG 2.1 AA compliant components
- **Performance**: Optimized bundle size and runtime performance
- **Theming**: CSS variable-based theming system
- **Testing**: Comprehensive test coverage

## Installation

The modern components are included in the BFUI package:

```bash
npm install @bluefly/bfui
```

## Usage

Import components from the modern path:

```tsx
import { TailwindButton, TailwindCard } from '@bluefly/bfui/modern';

function App() {
  return (
    <TailwindCard title="Welcome">
      <p>This is a modern Tailwind CSS card component.</p>
      <TailwindButton variant="primary">Get Started</TailwindButton>
    </TailwindCard>
  );
}
```

## Components

### TailwindButton

A versatile button component with various styles, sizes, and states.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| variant | 'solid' \| 'outline' \| 'ghost' \| 'link' \| 'unstyled' \| 'primary' \| 'secondary' \| 'danger' | 'solid' | Visual style variant |
| size | 'xs' \| 'sm' \| 'md' \| 'lg' | 'md' | Size variant |
| isLoading | boolean | false | Whether the button is in a loading state |
| isDisabled | boolean | false | Whether the button is disabled |
| loadingText | string | - | Text to display when button is loading |
| leftIcon | ReactNode | - | Icon element to display before button text |
| rightIcon | ReactNode | - | Icon element to display after button text |
| isFullWidth | boolean | false | Whether button takes full width of container |
| className | string | - | Additional CSS classes |
| children | ReactNode | - | Button content |

#### Examples

```tsx
// Basic button
<TailwindButton>Click Me</TailwindButton>

// Primary variant
<TailwindButton variant="primary">Submit</TailwindButton>

// With icon
<TailwindButton leftIcon=[<Icon name="edit" />]>Edit</TailwindButton>

// Loading state
<TailwindButton isLoading loadingText="Saving...">Save</TailwindButton>

// Disabled state
<TailwindButton isDisabled>Unavailable</TailwindButton>

// Full width
<TailwindButton isFullWidth>Full Width</TailwindButton>

// Custom class
<TailwindButton className="shadow-xl">Custom Style</TailwindButton>
```

### TailwindCard

A flexible card component for grouping related content.

#### Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| title | ReactNode | - | Card title displayed in the header |
| content | ReactNode | - | Content to display in the card body |
| footer | ReactNode | - | Content to display in the card footer |
| image | string | - | URL of the image to display |
| imageAlt | string | - | Alternative text for the image |
| variant | 'default' \| 'elevated' \| 'outlined' | 'default' | Visual style variant |
| size | 'small' \| 'medium' \| 'large' | 'medium' | Size variant |
| clickable | boolean | false | Whether the card is clickable |
| hoverable | boolean | false | Whether the card has hover effects |
| url | string | - | URL to navigate to when card is clicked |
| className | string | - | Additional CSS classes |
| children | ReactNode | - | Card content |

#### Examples

```tsx
// Basic card
<TailwindCard title="Card Title">
  This is a card content
</TailwindCard>

// With image
<TailwindCard 
  title="Featured Article"
  image="/images/article.jpg"
  imageAlt="Article thumbnail"
>
  This is a featured article preview
</TailwindCard>

// With footer
<TailwindCard 
  title="Product"
  footer=[<TailwindButton>Add to Cart</TailwindButton>]
>
  Product description goes here
</TailwindCard>

// Clickable card with URL
<TailwindCard 
  title="Blog Post"
  url="/blog/post-1"
  hoverable
>
  Click to read the full post
</TailwindCard>

// Different variants
<TailwindCard variant="elevated" title="Elevated Card">
  This card has a shadow
</TailwindCard>

<TailwindCard variant="outlined" title="Outlined Card">
  This card has a border
</TailwindCard>

// Different sizes
<TailwindCard size="small" title="Small Card">
  Compact content
</TailwindCard>

<TailwindCard size="large" title="Large Card">
  Spacious content
</TailwindCard>
```

## Styling System

### CSS Variables

The modern components use CSS variables for theming. These variables should be defined in your application's root CSS:

```css
:root {
  /* Base colors */
  --background: 0 0% 100%;
  --foreground: 222.2 84% 4.9%;
  
  /* Card colors */
  --card: 0 0% 100%;
  --card-foreground: 222.2 84% 4.9%;
  
  /* Border colors */
  --border: 214.3 31.8% 91.4%;
  
  /* Input colors */
  --input: 214.3 31.8% 91.4%;
  
  /* Primary colors */
  --primary: 222.2 47.4% 11.2%;
  --primary-foreground: 210 40% 98%;
  
  /* Secondary colors */
  --secondary: 210 40% 96.1%;
  --secondary-foreground: 222.2 47.4% 11.2%;
  
  /* Accent colors */
  --accent: 210 40% 96.1%;
  --accent-foreground: 222.2 47.4% 11.2%;
  
  /* Destructive colors */
  --destructive: 0 84.2% 60.2%;
  --destructive-foreground: 210 40% 98%;
  
  /* Muted colors */
  --muted: 210 40% 96.1%;
  --muted-foreground: 215.4 16.3% 46.9%;
  
  /* Popover colors */
  --popover: 0 0% 100%;
  --popover-foreground: 222.2 84% 4.9%;
  
  /* Radius */
  --radius: 0.5rem;
  
  /* Ring */
  --ring: 215 20.2% 65.1%;
}

.dark {
  --background: 222.2 84% 4.9%;
  --foreground: 210 40% 98%;
  
  /* ... other dark mode variables ... */
}
```

### Using the `cn()` Utility

The `cn()` utility helps with conditional class composition:

```tsx
import [cn] from '@bluefly/bfui/utils';

function CustomButton({ isActive, className, ...props }) {
  return (
    <button
      className={cn(
        "base-class",
        isActive && "active-class",
        className
      )}
      [...props]
    />
  );
}
```

## Accessibility

All modern components are designed with accessibility in mind:

- Proper semantic HTML
- ARIA attributes
- Keyboard navigation support
- Focus management
- Color contrast compliance
- Screen reader compatibility

## Browser Support

The modern components support all modern browsers:

- Chrome (latest 2 versions)
- Firefox (latest 2 versions)
- Safari (latest 2 versions)
- Edge (latest 2 versions)

## Best Practices

1. **Responsive Design**: Use Tailwind's responsive modifiers for adaptive layouts
   ```tsx
   <TailwindButton className="w-full md:w-auto">Responsive Button</TailwindButton>
   ```

2. **Dark Mode**: Leverage Tailwind's dark mode support
   ```tsx
   <TailwindCard className="dark:bg-gray-800 dark:text-white">
     Dark mode compatible
   </TailwindCard>
   ```

3. **Composition**: Compose components for complex UIs
   ```tsx
   <TailwindCard>
     <h2 className="text-xl font-bold">Custom Header</h2>
     <div className="mt-4">Custom content here</div>
     <div className="mt-6 flex justify-end">
       <TailwindButton variant="outline" className="mr-2">Cancel</TailwindButton>
       <TailwindButton variant="primary">Confirm</TailwindButton>
     </div>
   </TailwindCard>
   ```

4. **Component Extension**: Extend components for reusable patterns
   ```tsx
   function SuccessButton(props) {
     return (
       <TailwindButton 
         className="bg-green-500 hover:bg-green-600 text-white" 
         {...props} 
       />
     );
   }
   ```

## Troubleshooting

### Common Issues

1. **Tailwind Classes Not Applied**
   - Ensure Tailwind is properly configured in your project
   - Check that PurgeCSS is not removing needed classes

2. **Styling Conflicts**
   - Use more specific selectors or the `!important` modifier
   - Leverage the `cn()` utility for class merging

3. **TypeScript Errors**
   - Ensure you're using the correct prop types
   - Update TypeScript to latest version

## Contributing

To contribute to the modern component library:

1. Follow the project's coding standards
2. Write comprehensive tests
3. Document all props and examples
4. Ensure accessibility compliance
5. Submit a pull request with detailed description

## License

The BFUI Modern Component Library is licensed under the MIT License.