# Card Component

## Overview

The Card component is a flexible container for grouping and displaying related content and actions in a single container. It can include headers, footers, images, and interactive elements.

## Import

```tsx
// Chakra UI version (legacy)
import [Card] from '@bluefly/bfui';

// Tailwind CSS version (modern)
import [TailwindCard] from '@bluefly/bfui/modern';
```

## Usage

### Basic Card

```tsx
<Card id="basic-card" title="Card Title">
  This is the card content.
</Card>
```

### Card with Image

```tsx
<Card
  id="image-card"
  title="Featured Article"
  image="https://example.com/image.jpg"
>
  This is an article preview with an image.
</Card>
```

### Card with Footer

```tsx
<Card
  id="footer-card"
  title="Product"
  footer=[<Button variant="primary">Add to Cart</Button>]
>
  Product description goes here.
</Card>
```

### Interactive Card

```tsx
<Card
  id="interactive-card"
  title="Blog Post"
  clickable
  onClick={() => navigate('/blog/post-1')}
>
  Click to read more about this topic.
</Card>
```

### Card with URL

```tsx
<Card
  id="link-card"
  title="External Resource"
  url="https://example.com/resource"
>
  Click to visit external resource.
</Card>
```

### Card Variants

```tsx
<Card id="default-card" variant="default" title="Default Card">
  Default styling.
</Card>

<Card id="elevated-card" variant="elevated" title="Elevated Card">
  Card with elevation shadow.
</Card>

<Card id="outlined-card" variant="outlined" title="Outlined Card">
  Card with border outline.
</Card>
```

### Card Sizes

```tsx
<Card id="small-card" size="small" title="Small Card">
  Compact content.
</Card>

<Card id="medium-card" size="medium" title="Medium Card">
  Standard content.
</Card>

<Card id="large-card" size="large" title="Large Card">
  Spacious content.
</Card>
```

## Props

### Common Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| id | string | - | Unique identifier for the card |
| title | ReactNode | - | Card title displayed in the header |
| content | ReactNode | - | Content to display in the card body |
| footer | ReactNode | - | Content to display in the card footer |
| image | string | - | URL of the image to display |
| variant | 'default' \| 'elevated' \| 'outlined' | 'default' | Visual style variant |
| size | 'small' \| 'medium' \| 'large' | 'medium' | Size variant |
| clickable | boolean | false | Whether the card is clickable |
| url | string | - | URL to navigate to when card is clicked |
| className | string | - | Additional class names |
| children | ReactNode | - | Card content as children |

### Chakra UI Card Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| image | string | - | URL of the image to display |
| imagePosition | 'top' \| 'center' \| 'bottom' | 'top' | Position of the image |
| imageAlt | string | - | Alternative text for the image |

### Tailwind Card Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| hoverable | boolean | false | Whether the card has hover effects |
| imageAlt | string | - | Alternative text for the image |

## Theming

### Chakra UI Theme Customization

```tsx
// In your Chakra theme file
export const theme = {
  components: {
    Card: {
      baseStyle: {
        container: {
          borderRadius: 'lg',
          boxShadow: 'sm',
        },
        header: {
          paddingBottom: '2',
        },
        body: {
          padding: '4',
        },
        footer: {
          paddingTop: '2',
        },
      },
      variants: {
        custom: {
          container: {
            bg: 'gray.50',
            border: '1px solid',
            borderColor: 'gray.200',
          },
        },
      },
    },
  },
};
```

### Tailwind CSS Customization

```tsx
// Using the className prop
<TailwindCard 
  className="bg-gradient-to-r from-blue-50 to-indigo-50 border-2 border-blue-200"
  title="Custom Card"
>
  Custom styled card content.
</TailwindCard>

// Using Tailwind CSS variables
// In your CSS:
:root {
  --card: 210 40% 98%;
  --card-foreground: 222.2 84% 4.9%;
}
```

## Accessibility

- Cards use semantic HTML structure
- Images include appropriate alt text
- Clickable cards have proper keyboard focus handling
- Interactive elements maintain proper tab order
- Color contrasts meet WCAG standards

## Examples

### Product Card

```tsx
function ProductCard([product]) {
  return (
    <Card
      id={`product-[product.id]`}
      title=[product.name]
      image=[product.imageUrl]
      imageAlt=[product.name]
      footer=[<div className="flex justify-between items-center">
          <span className="font-bold">[product.price.toFixed(2)]</span>
          <Button variant="primary" size="sm">Add to Cart</Button>
        </div>]
    >
      <p>[product.description]</p>
      <div className="mt-2 flex items-center">
        [/* Rating stars */]
        <span className="ml-2 text-sm text-gray-600">
          [product.rating] ([product.reviewCount] reviews)
        </span>
      </div>
    </Card>
  );
}
```

### Blog Post Card

```tsx
function BlogPostCard([post]) {
  return (
    <Card
      id={`post-[post.id]`}
      title=[post.title]
      image=[post.coverImage]
      url=[`/blog/[post.slug]`]
      clickable
      hoverable
      variant="elevated"
    >
      <div>
        <div className="text-sm text-gray-500 mb-2">
          [new Date(post.date).toLocaleDateString()] • [post.readTime] min read
        </div>
        <p>[post.excerpt]</p>
      </div>
      <div className="mt-4 flex items-center">
        <img 
          src=[post.author.avatar] 
          alt=[post.author.name]
          className="w-8 h-8 rounded-full mr-2"
        />
        <span className="text-sm font-medium">[post.author.name]</span>
      </div>
    </Card>
  );
}
```

### Dashboard Widget Card

```tsx
function MetricsCard({ title, value, change, icon }) {
  const isPositive = change >= 0;
  
  return (
    <Card
      id={`metric-[title.toLowerCase().replace(/\s+/g, '-')]`}
      size="small"
      variant={isPositive ? 'default' : 'outlined'}
    >
      <div className="flex justify-between items-center mb-2">
        <h3 className="text-sm text-gray-500">Documentation</h3>
        <span className="text-gray-400">[icon]</span>
      </div>
      <div className="text-2xl font-bold">Value</div>
      <div className={`text-sm [isPositive ? 'text-green-500' : 'text-red-500']`}>
        {isPositive ? '+' : ''}[change]% from last period
      </div>
    </Card>
  );
}
```

### Settings Card

```tsx
function SettingsCard({ title, description, children }) {
  return (
    <Card
      id={`setting-[title.toLowerCase().replace(/\s+/g, '-')]`}
      variant="outlined"
    >
      <div className="flex flex-col md:flex-row md:justify-between md:items-center">
        <div className="mb-4 md:mb-0 md:mr-8">
          <h3 className="text-lg font-medium">Documentation</h3>
          <p className="text-gray-500">Project Description</p>
        </div>
        <div className="flex-shrink-0">
          [children]
        </div>
      </div>
    </Card>
  );
}

// Usage
<SettingsCard
  title="Notifications"
  description="Control how you receive notifications."
>
  <Switch isChecked=[enabled] onChange=[toggleNotifications] />
</SettingsCard>
```

## Best Practices

1. **Keep content concise**: Cards should present information in a focused way
2. **Use consistent spacing**: Maintain consistent padding within and between cards
3. **Appropriate imagery**: Use relevant images that enhance the content
4. **Clear actions**: Make interactive elements obvious and accessible
5. **Responsive design**: Ensure cards stack and resize appropriately on mobile
6. **Content hierarchy**: Use visual hierarchy to guide users through the content

## Migration Guide

### From Chakra UI to Tailwind CSS

```tsx
// Chakra UI Card
<Card
  id="chakra-card"
  title="Chakra Card"
  image="https://example.com/image.jpg"
  imagePosition="top"
  variant="elevated"
  size="medium"
  clickable
  onClick=[handleClick]
  footer=[<Button>Action</Button>]
>
  Card content here.
</Card>

// Equivalent Tailwind Card
<TailwindCard
  title="Tailwind Card"
  image="https://example.com/image.jpg"
  imageAlt="Card image"
  variant="elevated"
  size="medium"
  clickable
  hoverable
  onClick=[handleClick]
  footer=[<TailwindButton>Action</TailwindButton>]
>
  Card content here.
</TailwindCard>
```

## Troubleshooting

### Common Issues

1. **Image sizing problems**
   - Use aspect ratio constraints for consistent sizing
   - Ensure images are appropriately optimized

2. **Layout issues on mobile**
   - Use responsive design techniques
   - Test cards at various viewport sizes

3. **Interactive behavior inconsistencies**
   - Ensure clickable area is appropriate
   - Verify hover and focus states work correctly

## Related Components

- [Panel](./panel.md): Similar to Card but for administrative interfaces
- [Box](./box.md): Simpler layout container without card styling
- [MediaObject](./media-object.md): For image/text layouts
- [Grid](./grid.md): For organizing multiple cards in a layout