/**
 * Example of using the Model Standards Validation utilities
 * from the ReelCode integration
 */

import { 
  validateModelStandards, 
  applyModelStandards, 
  generateModelStandardsConfig 
} from '../../src/components/model-registry/ModelStandardsUtility.js';
import type { ModelCreateRequest } from '../../src/components/model-registry/types.js';

/**
 * Example 1: Basic Validation
 * 
 * This demonstrates how to validate a model configuration
 * against predefined standards
 */
function basicValidationExample() {
  // Sample model configuration
  const model: ModelCreateRequest = {
    name: 'gpt4-enhanced',
    provider: 'openai',
    version: '1.0.0',
    description: 'Enhanced GPT-4 model with custom training',
    parameters: {
      temperature: 0.7,
      max_tokens: 2048,
      top_p: 0.95
    },
    capabilities: ['text-generation', 'code-generation', 'summarization'],
    deployment_config: {
      environment: 'production',
      strategy: 'blue_green',
      replicas: 1, // This will trigger a warning for production
      auto_scaling: false // This will trigger a warning for production
    }
  };
  
  // Validate the model
  const validationResult = validateModelStandards(model);
  
  console.log('Model is valid:', validationResult.valid);
  
  if (validationResult.errors.length > 0) {
    console.error('Validation Errors:');
    validationResult.errors.forEach(error => {
      console.error(`- ${error.rule}: ${error.message}`);
    });
  }
  
  if (validationResult.warnings.length > 0) {
    console.warn('Validation Warnings:');
    validationResult.warnings.forEach(warning => {
      console.warn(`- ${warning.rule}: ${warning.message}`);
    });
  }
}

/**
 * Example 2: Automatically Fixing Issues
 * 
 * This demonstrates how to automatically fix common issues
 * in model configurations
 */
function autoFixExample() {
  // Sample model with issues
  const modelWithIssues: ModelCreateRequest = {
    name: 'test model with spaces!', // Invalid characters
    provider: 'openai',
    version: '1.0', // Not full semver
    description: 'Test model with configuration issues',
    parameters: {
      temperature: 1.5, // Out of range
      max_tokens: -10 // Invalid negative value
    },
    deployment_config: {
      environment: 'production',
      strategy: 'rolling',
      replicas: 1, // Too few for production
      auto_scaling: false // Should be true for production
    }
  };
  
  console.log('Original model with issues:');
  console.log(modelWithIssues);
  
  // Validate to see issues
  const validationBefore = validateModelStandards(modelWithIssues);
  console.log('Issues found:', validationBefore.errors.length + validationBefore.warnings.length);
  
  // Auto-fix issues
  const fixedModel = applyModelStandards(modelWithIssues);
  
  console.log('Fixed model:');
  console.log(fixedModel);
  
  // Validate again to confirm fixes
  const validationAfter = validateModelStandards(fixedModel);
  console.log('Remaining issues:', validationAfter.errors.length + validationAfter.warnings.length);
}

/**
 * Example 3: Generating Standards Config for AI Assistants
 * 
 * This demonstrates how to generate standards configuration
 * for AI assistants like Claude or GitHub Copilot
 */
function generateConfigExample() {
  // Generate standards configuration
  const standardsConfig = generateModelStandardsConfig('bfui-model-registry');
  
  console.log('Standards Configuration for AI Assistants:');
  console.log(JSON.stringify(standardsConfig, null, 2));
  
  // This config could be used to generate a CLAUDE.md file or similar
  const aiAssistantInstructions = `
# Model Registry Standards Configuration

This project uses the following standards for model configurations:

\`\`\`json
${JSON.stringify(standardsConfig, null, 2)}
\`\`\`

Please ensure all model configurations adhere to these standards.
  `;
  
  console.log('AI Assistant Instructions:');
  console.log(aiAssistantInstructions);
}

/**
 * Example 4: Integration with API Validation
 * 
 * This demonstrates how to use the validation in an API context
 */
async function apiValidationExample() {
  // Mock API function to create a model
  async function createModelAPI(modelRequest: ModelCreateRequest) {
    // Validate the model before creating
    const validationResult = validateModelStandards(modelRequest);
    
    if (!validationResult.valid) {
      // Return validation errors
      return {
        success: false,
        errors: validationResult.errors,
        warnings: validationResult.warnings
      };
    }
    
    // If valid, apply fixes for any warnings (optional)
    const fixedModel = applyModelStandards(modelRequest);
    
    // Proceed with model creation (mock implementation)
    const createdModel = {
      id: 'model-123',
      ...fixedModel,
      status: 'active',
      created_at: new Date().toISOString()
    };
    
    return {
      success: true,
      model: createdModel,
      warnings: validationResult.warnings
    };
  }
  
  // Example usage
  const modelRequest = {
    name: 'new-model',
    provider: 'anthropic',
    version: '0.1.0',
    parameters: {
      temperature: 0.9 // This will trigger a warning
    }
  };
  
  const result = await createModelAPI(modelRequest);
  
  if (result.success) {
    console.log('Model created successfully:', result.model);
    
    if (result.warnings && result.warnings.length > 0) {
      console.warn('Warnings during creation:');
      result.warnings.forEach(warning => {
        console.warn(`- ${warning.rule}: ${warning.message}`);
      });
    }
  } else {
    console.error('Failed to create model:');
    result.errors.forEach(error => {
      console.error(`- ${error.rule}: ${error.message}`);
    });
  }
}

// Run the examples
basicValidationExample();
autoFixExample();
generateConfigExample();
apiValidationExample();