# BFUI to Drupal UI Patterns Integration Strategy

This document outlines the strategy for integrating React components from the BFUI project with Drupal's UI Patterns module, while respecting the separation between projects.

## Overview

The integration approach uses the llmcli tool as a bridge between the isolated bfui React components and Drupal modules that implement UI Patterns. This strategy maintains separation of concerns while enabling a cohesive design system across platforms.

## Integration Architecture

```
┌───────────────┐     ┌───────────────┐     ┌───────────────┐
│               │     │               │     │               │
│  BFUI         │     │  llmcli        │     │  Drupal       │
│  React        │────▶│  Command      │────▶│  UI Patterns  │
│  Components   │     │  Generator    │     │  Module       │
│               │     │               │     │               │
└───────────────┘     └───────────────┘     └───────────────┘
```

## Key Benefits

1. **Single Source of Truth**: Component definitions are maintained in the React codebase.
2. **Project Isolation**: Respects the separation between projects.
3. **Design Consistency**: Ensures consistent UI across platforms through shared design tokens.
4. **Automated Workflow**: Reduces manual synchronization efforts.
5. **Developer Experience**: Familiar workflow for both React and Drupal developers.
6. **Interactive Documentation**: Standalone pattern library preview provides visual reference.

## Implementation Details

### Component to Pattern Conversion

The conversion process translates React components into UI Patterns with the following mappings:

| React Component | UI Pattern Equivalent |
|-----------------|------------------------|
| Props Interface | Pattern Fields |
| JSDoc Comments | Field Descriptions |
| Default Props | Field Defaults |
| Component Variants | Pattern Variants |
| Storybook Stories | Additional Pattern Variants |
| Theme Tokens | CSS/SCSS Variables |
| Component Structure | Twig Template |
| CSS-in-JS / Styled Components | CSS File |
| Component Behavior | JS Behaviors File |

### Workflow Integration

1. **Development Flow**:
   - Developers create/update React components in bfui
   - Components are analyzed and converted to UI Patterns using llmcli
   - Generated patterns are integrated into Drupal modules

2. **Continuous Integration**:
   - Pattern generation can be automated in CI pipelines
   - Pattern changes can be reviewed as part of code reviews
   - Version control ensures synchronization across environments

## Using the Tool

### Basic Usage

```bash
# Navigate to a Drupal module directory
cd /path/to/drupal/module

# Generate patterns from all components
llmcli ui-patterns:generate --output ./patterns

# Generate pattern for a specific component
llmcli ui-patterns:generate --component-path /path/to/bfui/src/components/Button.tsx --output ./patterns
```

### Advanced Usage

```bash
# Generate patterns with custom theme name
llmcli ui-patterns:generate --theme my_theme --output ./patterns

# Generate patterns with JavaScript behaviors
llmcli ui-patterns:generate --include-js --output ./patterns

# Generate patterns with design tokens from React theme
llmcli ui-patterns:generate --extract-tokens --theme-path /path/to/theme --output ./patterns

# Generate both CSS and SCSS variables for design tokens
llmcli ui-patterns:generate --token-format both --output ./patterns

# Generate patterns with HTML preview documentation
llmcli ui-patterns:generate --generate-preview --output ./patterns

# Preview generation without writing files
llmcli ui-patterns:generate --dry-run --verbose
```

## Integration Example

### React Component (Input.tsx)

```tsx
import React from 'react';
import { FormControl, FormLabel, Input as ChakraInput } from '@chakra-ui/react';

export interface InputProps {
  /**
   * Label for the input
   */
  label?: string;
  
  /**
   * Error message to display
   */
  error?: string;
  
  /**
   * Whether the input is required
   */
  isRequired?: boolean;
}

export const Input: React.FC<InputProps> = ({
  label,
  error,
  isRequired = false,
  ...rest
}) => {
  return (
    <FormControl isInvalid={!!error} isRequired=[isRequired]>
      [label && <FormLabel>{label]</FormLabel>}
      <ChakraInput [...rest] />
    </FormControl>
  );
};
```

### Generated UI Pattern (bfui.ui_patterns.yml)

```yaml
input:
  label: Input
  description: Input component using Chakra UI with BFUI theme customizations
  fields:
    label:
      type: text
      label: Label
      description: Label for the input
    error:
      type: text
      label: Error
      description: Error message to display
    isRequired:
      type: boolean
      label: Is Required
      description: Whether the input is required
  variants:
    default:
      label: Default
      description: Default input style
    outlined:
      label: Outlined
      description: Input with outlined style
      settings:
        variant: "outlined"
    filled:
      label: Filled
      description: Input with filled background
      settings:
        variant: "filled"
  libraries:
    - bfui/input
```

### Generated Twig Template (pattern-input.html.twig)

```twig
{#
 * @file
 * Input pattern template.
 * Generated from Input.tsx
 *
 * Available variables:
 * - [label]: Label for the input
 * - [error]: Error message to display
 * - [isRequired]: Whether the input is required
 * - [variant]: The selected variant name
 #}

<div class="input[% if variant %] input--[variant][% endif %][% if classes %] [classes][% endif %]">
  [% if label %]
    <label[% if id %] for="item"[% endif %] class="[label_classes|default('')]">[label]</label>
  [% endif %]
  
  <input class="input__field[% if variant %] input__field--[variant][% endif %][field_classes ? ' ' ~ field_classes]"
    [% if id %]id="item"[% endif %]
    [% if name %]name="Name"[% endif %]
    [% if value %]value="Value"[% endif %]
    [% if placeholder %]placeholder="[placeholder]"[% endif %]
    [% if type %]type="Type"[% endif %]
    [% if isRequired %]required[% endif %]
  >
  
  [% if error %]
    <div class="error-message">[error]</div>
  [% elseif helperText %]
    <div class="helper-text">[helperText]</div>
  [% endif %]
</div>
```

## Best Practices

1. **Component Organization**:
   - Group related components in the same directory
   - Use consistent naming conventions across projects
   - Document components thoroughly with JSDoc comments

2. **Pattern Usage**:
   - Use pattern libraries to showcase available patterns
   - Document pattern usage with examples
   - Test patterns in various contexts

3. **Maintenance**:
   - Regenerate patterns when components change
   - Version control patterns alongside components
   - Review generated patterns for accuracy

## Conclusion

This integration strategy provides a sustainable approach to maintaining UI consistency between React and Drupal projects while respecting their separation. By using llmcli as a bridge, we can automate the conversion process and reduce the maintenance burden of keeping UI components synchronized across platforms.