# BFUI Shared Utilities Migration Plan

This document outlines a comprehensive plan for migrating the BFUI project to fully utilize shared utilities from `@bluefly/cli/shared`. It includes an analysis of current implementations, a detailed migration strategy, implementation examples, and a timeline for completion.

## 1. Current State Analysis

BFUI currently has multiple utility implementations with mixed origins:

### 1.1 Logger Implementation

- **Current**: Two separate logger implementations
  - `/src/utils/logger.ts` using `@bluefly/shared-utils`
  - `/src/shared/logger-adapter.tsx` using `@bluefly/cli/shared`
- **Problems**: Duplication, inconsistency, potential compatibility issues

### 1.2 Error Handling

- **Current**: A React-specific error adapter in `/src/shared/error-adapter.tsx`
- **Status**: Already uses `@bluefly/cli/shared` but needs standardization across components

### 1.3 HTTP Client

- **Current**: Uses `@bluefly/api-client` in `/src/services/api.ts`
- **Problems**: Not integrated with the standardized `@bluefly/cli/shared/http` client

### 1.4 Utility Functions

- **Current**: Multiple custom implementations
  - `/src/utils/formatters.ts` - Formatting functions
  - `/src/utils/colorUtils.ts` - Color manipulation
  - `/src/utils/styleUtils.ts` - Styling utilities
  - `/src/utils/responsiveUtils.ts` - Responsive design utilities
  - `/src/utils/vectorUtils.ts` - Vector operations
- **Problems**: Some of these can be replaced with shared utilities, others need React-specific adapters

## 2. Migration Goals

1. **Standardization**: Use `@bluefly/cli/shared` for all shared utilities
2. **React Integration**: Create React-specific adapters for all shared utilities
3. **Consistency**: Ensure consistent patterns across all components
4. **Maintainability**: Reduce duplication and simplify maintenance
5. **Performance**: Optimize React-specific implementations for performance

## 3. Implementation Plan

### 3.1. Phase 1: Core Utility Migration

#### 3.1.1 Logger Implementation

**Action Items:**
1. Remove `/src/utils/logger.ts` implementation using `@bluefly/shared-utils`
2. Enhance `/src/shared/logger-adapter.tsx` to provide full React-compatible functionality
3. Create React hook versions of the logger
4. Add component-specific context to all log messages
5. Update all components to use the standardized logger

**Implementation Example:**

```typescript
// src/shared/logger.ts
import React from 'react';
import { LLM PlatformLogger, LogLevel } from '@bluefly/cli/shared';

// Create a React-specific logger interface
export interface ReactLogger {
  debug(message: string, data?: Record<string, any>): void;
  info(message: string, data?: Record<string, any>): void;
  warn(message: string, data?: Record<string, any>): void;
  error(message: string, error?: Error | any, data?: Record<string, any>): void;
  setLevel(level: LogLevel): void;
  getLevel(): LogLevel;
}

// React logger configuration options
export interface ReactLoggerOptions {
  service?: string;
  component?: string;
  level?: LogLevel;
  pretty?: boolean;
}

// Create the core logger instance
const coreLogger = new LLM PlatformLogger({
  prefix: '[bfui]',
  level: process.env.NODE_ENV === 'development' ? 'debug' : 'info',
  timestamp: true,
  colorize: process.env.NODE_ENV === 'development'
});

// Export base logger for direct use
export const logger = coreLogger;

// Create a React-compatible logger with component context
export function createComponentLogger(
  componentName: string, 
  options: Omit<ReactLoggerOptions, 'component'> = {}
): ReactLogger {
  return {
    debug(message: string, data?: Record<string, any>): void {
      coreLogger.debug(message, { component: componentName, ...data });
    },
    
    info(message: string, data?: Record<string, any>): void {
      coreLogger.info(message, { component: componentName, ...data });
    },
    
    warn(message: string, data?: Record<string, any>): void {
      coreLogger.warn(message, { component: componentName, ...data });
    },
    
    error(message: string, error?: Error | any, data?: Record<string, any>): void {
      coreLogger.error(message, { 
        component: componentName, 
        error: error?.message || error,
        stack: error?.stack,
        ...data 
      });
    },
    
    setLevel(level: LogLevel): void {
      coreLogger.setLevel(level);
    },
    
    getLevel(): LogLevel {
      return coreLogger.getLevel();
    }
  };
}

// React hook for component logging
export function useLogger(componentName: string): ReactLogger {
  // Use ref to ensure stable identity across renders
  const loggerRef = React.useRef<ReactLogger | null>(null);
  
  if (loggerRef.current === null) {
    loggerRef.current = createComponentLogger(componentName);
  }
  
  return loggerRef.current;
}
```

#### 3.1.2 Error Handling

**Action Items:**
1. Enhance the existing error adapter in `/src/shared/error-adapter.tsx`
2. Create a standardized ErrorBoundary component
3. Implement React-specific error hooks
4. Create a standardized error reporting system
5. Integrate with the shared logger

**Implementation Example:**

```typescript
// src/shared/errors.ts
import React from 'react';
import { 
  createError, 
  ERROR_CODES, 
  errorUtils, 
  LLM PlatformError 
} from '@bluefly/cli/shared';
import [logger] from './logger';

// Export core error utilities
export { createError, ERROR_CODES, errorUtils };

// React-specific error codes
export const UI_ERROR_CODES = {
  ...ERROR_CODES,
  COMPONENT_ERROR: 'COMPONENT_ERROR',
  RENDER_ERROR: 'RENDER_ERROR',
  HOOK_ERROR: 'HOOK_ERROR',
  STATE_ERROR: 'STATE_ERROR',
  PROP_ERROR: 'PROP_ERROR',
  CONTEXT_ERROR: 'CONTEXT_ERROR'
};

// React-specific error class
export class UIError extends Error {
  code: string;
  statusCode?: number;
  component?: string;
  details?: Record<string, any>;
  
  constructor(options: {
    message: string;
    code?: string;
    statusCode?: number;
    component?: string;
    details?: Record<string, any>;
  }) {
    super(options.message);
    this.name = 'UIError';
    this.code = options.code || UI_ERROR_CODES.COMPONENT_ERROR;
    this.statusCode = options.statusCode;
    this.component = options.component;
    this.details = options.details;
  }
  
  // Convert to standard LLM PlatformError
  toLLM PlatformError(): LLM PlatformError {
    return createError(
      this.message,
      this.code,
      this.statusCode,
      {
        component: this.component,
        ...this.details
      }
    );
  }
}

// Create component-specific error
export function createComponentError(
  message: string,
  component: string,
  details?: Record<string, any>,
  code?: string
): UIError {
  return new UIError({
    message,
    code: code || UI_ERROR_CODES.COMPONENT_ERROR,
    component,
    details
  });
}

// React error boundary component
interface ErrorBoundaryProps {
  fallback?: React.ReactNode | ((error: UIError) => React.ReactNode);
  onError?: (error: UIError, errorInfo: React.ErrorInfo) => void;
  children: React.ReactNode;
}

interface ErrorBoundaryState {
  hasError: boolean;
  error: UIError | null;
}

export class ErrorBoundary extends React.Component<ErrorBoundaryProps, ErrorBoundaryState> {
  constructor(props: ErrorBoundaryProps) {
    super(props);
    this.state = { 
      hasError: false, 
      error: null 
    };
  }

  static getDerivedStateFromError(error: Error): ErrorBoundaryState {
    // Convert to UIError if it's not already
    const uiError = error instanceof UIError 
      ? error 
      : new UIError({
          message: error.message,
          details: { originalError: error }
        });

    return { hasError: true, error: uiError };
  }

  componentDidCatch(error: Error, errorInfo: React.ErrorInfo): void {
    // Get component name from stack if possible
    const componentName = errorInfo.componentStack
      .split('\n')[1]
      ?.trim()
      .replace(/^in\s+/, '')
      || 'Unknown';
    
    // Convert to UIError
    const uiError = this.state.error || new UIError({
      message: error.message,
      component: componentName,
      details: {
        componentStack: errorInfo.componentStack,
        originalError: error
      }
    });
    
    // Log error
    logger.error('React component error', { 
      error: uiError.message,
      component: componentName,
      stack: errorInfo.componentStack
    });
    
    // Call onError if provided
    if (this.props.onError) {
      this.props.onError(uiError, errorInfo);
    }
  }

  render(): React.ReactNode {
    if (this.state.hasError) {
      if (this.props.fallback) {
        if (typeof this.props.fallback === 'function') {
          return this.props.fallback(this.state.error!);
        }
        return this.props.fallback;
      }
      
      // Default fallback UI
      return (
        <div className="bg-red-50 border border-red-200 text-red-800 p-4 rounded">
          <h3 className="font-semibold">Something went wrong</h3>
          [process.env.NODE_ENV !== 'production' && (
            <>
              <p>{this.state.error?.message]</p>
              <details>
                <summary>Error details</summary>
                <pre className="whitespace-pre-wrap">
                  [this.state.error?.stack]
                </pre>
              </details>
            </>
          )}
        </div>
      );
    }

    return this.props.children;
  }
}

// Custom hook for error handling
export function useErrorHandler(): {
  handleError: (error: any, component?: string) => UIError;
  throwError: (message: string, options?: Record<string, any>) => never;
} {
  const componentName = React.useMemo(() => {
    // Try to get component name from the call stack
    try {
      const err = new Error();
      const stackLines = err.stack?.split('\n') || [];
      
      // Find the first capitalized name in the stack which likely represents a component
      for (let i = 2; i < stackLines.length; i++) {
        const matches = stackLines[i].match(/at ([A-Z][a-zA-Z0-9]+)/);
        if (matches && matches[1]) {
          return matches[1];
        }
      }
      
      return 'UnknownComponent';
    } catch (e) {
      return 'UnknownComponent';
    }
  }, []);
  
  const handleError = React.useCallback((error: any, component?: string): UIError => {
    const uiError = error instanceof UIError
      ? error
      : new UIError({
          message: error?.message || 'An error occurred',
          component: component || componentName,
          details: { originalError: error }
        });

    logger.error('Error handled', { 
      error: uiError.message,
      component: component || componentName,
      stack: uiError.stack
    });
    
    return uiError;
  }, [componentName]);
  
  const throwError = React.useCallback((message: string, options?: Record<string, any>): never => {
    throw new UIError({
      message,
      component: componentName,
      details: options
    });
  }, [componentName]);
  
  return { handleError, throwError };
}
```

#### 3.1.3 HTTP Client

**Action Items:**
1. Create a new HTTP client using `@bluefly/cli/shared/http`
2. Implement React-specific hooks for data fetching
3. Create a caching layer compatible with React
4. Add React Suspense support
5. Implement streamlined error handling

**Implementation Example:**

```typescript
// src/shared/http.ts
import { useState, useEffect, useCallback, useRef } from 'react';
import { 
  HttpClient, 
  HttpRequestConfig,
  HttpResponse,
  createCircuitBreaker,
  retryWithBackoff
} from '@bluefly/cli/shared/http';
import [useLogger] from './logger';
import [createComponentError] from './errors';

// Create the HTTP client with default configuration
const httpClient = new HttpClient({
  baseURL: process.env.REACT_APP_API_URL || '/api',
  timeout: 30000,
  headers: {
    'Content-Type': 'application/json',
    'X-Client-ID': 'bfui'
  }
});

// Add circuit breaker
const circuitBreaker = createCircuitBreaker({
  failureThreshold: 5,
  resetTimeout: 30000
});

httpClient.use(circuitBreaker);

// Add retry capability
httpClient.use(retryWithBackoff({
  maxRetries: 3,
  initialDelay: 1000,
  maxDelay: 10000
}));

// Export the configured client
export [httpClient];

// React hook for data fetching
export function useApi<T = any>(
  url: string,
  options: {
    method?: 'GET' | 'POST' | 'PUT' | 'DELETE' | 'PATCH';
    data?: any;
    params?: Record<string, any>;
    headers?: Record<string, any>;
    immediate?: boolean;
    dependencies?: any[];
    onSuccess?: (data: T) => void;
    onError?: (error: Error) => void;
  } = {}
) {
  const { 
    method = 'GET',
    data,
    params,
    headers,
    immediate = true,
    dependencies = [],
    onSuccess,
    onError
  } = options;
  
  const [result, setResult] = useState<T | null>(null);
  const [loading, setLoading] = useState(immediate);
  const [error, setError] = useState<Error | null>(null);
  
  const logger = useLogger('useApi');
  const requestIdRef = useRef(0);
  
  const execute = useCallback(async (overrideData?: any): Promise<T> => {
    const requestId = ++requestIdRef.current;
    
    try {
      setLoading(true);
      setError(null);
      
      logger.debug(`API request to https://example.com`, { method, params });
      
      const config: HttpRequestConfig = {
        url,
        method,
        params,
        headers,
        ...(data || overrideData ? { data: overrideData || data } : {})
      };
      
      const response = await httpClient.request<T>(config);
      
      // Only update state if this is the latest request
      if (requestId === requestIdRef.current) {
        setResult(response.data);
        setLoading(false);
        
        if (onSuccess) {
          onSuccess(response.data);
        }
      }
      
      return response.data;
    } catch (err: any) {
      // Only update state if this is the latest request
      if (requestId === requestIdRef.current) {
        const error = createComponentError(
          err.message || 'API request failed',
          'useApi',
          { url, method, params }
        );
        
        setError(error);
        setLoading(false);
        
        logger.error(`API request to https://example.com failed`, err);
        
        if (onError) {
          onError(error);
        }
      }
      
      throw err;
    }
  }, [url, method, data, params, headers, onSuccess, onError, logger]);
  
  useEffect(() => {
    if (immediate) {
      execute();
    }
  }, [execute, immediate, ...dependencies]);
  
  return {
    data: result,
    loading,
    error,
    execute,
    refetch: execute
  };
}

// API cache implementation
const apiCache = new Map<string, { data: any; timestamp: number }>();

// React hook for cached API requests
export function useCachedApi<T = any>(
  url: string,
  options: {
    method?: 'GET';
    params?: Record<string, any>;
    headers?: Record<string, any>;
    ttl?: number; // Time to live in milliseconds
    immediate?: boolean;
    dependencies?: any[];
    onSuccess?: (data: T) => void;
    onError?: (error: Error) => void;
  } = {}
) {
  const { 
    ttl = 5 * 60 * 1000, // 5 minutes default
    immediate = true,
    ...apiOptions
  } = options;
  
  const cacheKey = `https://example.com:[JSON.stringify(options.params || [])]`;
  const logger = useLogger('useCachedApi');
  
  // Check cache before making request
  const getCachedResult = useCallback(() => {
    const cached = apiCache.get(cacheKey);
    if (cached && Date.now() - cached.timestamp < ttl) {
      logger.debug(`Cache hit for https://example.com`, { cacheKey });
      return cached.data;
    }
    logger.debug(`Cache miss for https://example.com`, [cacheKey]);
    return null;
  }, [cacheKey, ttl, url, logger]);
  
  // Store result in cache
  const setCachedResult = useCallback((data: T) => {
    apiCache.set(cacheKey, { 
      data, 
      timestamp: Date.now() 
    });
    logger.debug(`Cache set for https://example.com`, [cacheKey]);
  }, [cacheKey, url, logger]);
  
  // Use regular API hook with cache behavior
  const apiResult = useApi<T>(url, {
    ...apiOptions,
    immediate: false,
    onSuccess: (data) => {
      setCachedResult(data);
      if (apiOptions.onSuccess) {
        apiOptions.onSuccess(data);
      }
    }
  });
  
  const [cachedData, setCachedData] = useState<T | null>(() => {
    return getCachedResult() as T | null;
  });
  
  // Execute with cache logic
  const executeWithCache = useCallback(async () => {
    const cached = getCachedResult();
    if (cached) {
      setCachedData(cached);
      return cached;
    } else {
      const result = await apiResult.execute();
      setCachedData(result);
      return result;
    }
  }, [apiResult.execute, getCachedResult, setCachedData]);
  
  // Initial execution
  useEffect(() => {
    if (immediate) {
      executeWithCache();
    }
  }, [executeWithCache, immediate, ...apiOptions.dependencies || []]);
  
  return {
    ...apiResult,
    data: cachedData || apiResult.data,
    execute: executeWithCache,
    refetch: apiResult.execute // Bypass cache on explicit refetch
  };
}
```

### 3.2. Phase 2: Specialized Utilities Migration

#### 3.2.1 Formatter Utilities

**Action Items:**
1. Move formatting functions to shared library
2. Create React-specific formatting hooks
3. Implement internationalization support

**Implementation Example:**

```typescript
// src/shared/formatters.ts
import { 
  formatNumber as sharedFormatNumber,
  formatDate as sharedFormatDate,
  formatCurrency as sharedFormatCurrency
} from '@bluefly/cli/shared/formatters';

// Export core formatters directly
export { 
  sharedFormatNumber as formatNumber,
  sharedFormatDate as formatDate,
  sharedFormatCurrency as formatCurrency
};

/**
 * Format bytes into human-readable sizes
 */
export function formatBytes(bytes: number, precision: number = 2): string {
  if (bytes === 0) return '0 B';

  const k = 1024;
  const sizes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB'];
  const i = Math.floor(Math.log(bytes) / Math.log(k));

  return `[(bytes / Math.pow(k, i)).toFixed(precision)] [sizes[i]]`;
}

/**
 * Format a duration in milliseconds to a human-readable string
 */
export function formatDuration(ms: number): string {
  if (ms < 1000) {
    return `[ms]ms`;
  }

  const seconds = ms / 1000;
  if (seconds < 60) {
    return `[seconds.toFixed(1)]s`;
  }

  const minutes = seconds / 60;
  if (minutes < 60) {
    return `[minutes.toFixed(1)]m`;
  }

  const hours = minutes / 60;
  if (hours < 24) {
    return `[hours.toFixed(1)]h`;
  }

  const days = hours / 24;
  return `[days.toFixed(1)]d`;
}

/**
 * Format a percentage value
 */
export function formatPercentage(value: number, precision: number = 1): string {
  return `[value.toFixed(precision)]%`;
}

/**
 * Format a date relative to now (e.g., "2 hours ago")
 */
export function formatRelativeTime(date: Date | string): string {
  const now = new Date();
  const then = typeof date === 'string' ? new Date(date) : date;
  const diffMs = now.getTime() - then.getTime();

  const rtf = new Intl.RelativeTimeFormat('en', { numeric: 'auto' });

  const seconds = Math.floor(diffMs / 1000);
  const minutes = Math.floor(seconds / 60);
  const hours = Math.floor(minutes / 60);
  const days = Math.floor(hours / 24);
  const months = Math.floor(days / 30);
  const years = Math.floor(months / 12);

  if (years > 0) return rtf.format(-years, 'year');
  if (months > 0) return rtf.format(-months, 'month');
  if (days > 0) return rtf.format(-days, 'day');
  if (hours > 0) return rtf.format(-hours, 'hour');
  if (minutes > 0) return rtf.format(-minutes, 'minute');
  return rtf.format(-seconds, 'second');
}

// React hooks for formatters
import [useMemo] from 'react';

// Hook for number formatting with memoization
export function useNumberFormatter(options: {
  locale?: string;
  style?: 'decimal' | 'currency' | 'percent';
  currency?: string;
  minimumFractionDigits?: number;
  maximumFractionDigits?: number;
} = {}) {
  const formatter = useMemo(() => {
    return new Intl.NumberFormat(options.locale || 'en-US', options);
  }, [
    options.locale,
    options.style,
    options.currency,
    options.minimumFractionDigits,
    options.maximumFractionDigits
  ]);
  
  return (value: number) => formatter.format(value);
}

// Hook for date formatting with memoization
export function useDateFormatter(options: {
  locale?: string;
  dateStyle?: 'full' | 'long' | 'medium' | 'short';
  timeStyle?: 'full' | 'long' | 'medium' | 'short';
} = {}) {
  const formatter = useMemo(() => {
    return new Intl.DateTimeFormat(options.locale || 'en-US', options);
  }, [options.locale, options.dateStyle, options.timeStyle]);
  
  return (date: Date | string) => {
    const dateObj = typeof date === 'string' ? new Date(date) : date;
    return formatter.format(dateObj);
  };
}

// Hook for relative time formatting
export function useRelativeTimeFormatter(locale: string = 'en') {
  const formatter = useMemo(() => {
    return new Intl.RelativeTimeFormat(locale, { numeric: 'auto' });
  }, [locale]);
  
  return formatRelativeTime;
}
```

#### 3.2.2 Color Utilities

**Action Items:**
1. Move color utilities to shared library
2. Create React-specific color hooks
3. Add Tailwind integration

**Implementation Example:**

```typescript
// src/shared/colors.ts
import [colorsUtils] from '@bluefly/cli/shared/utils';
import [useMemo] from 'react';

// Export core color utilities
export const {
  hexToRgb,
  rgbToHex,
  darken,
  lighten,
  getContrastColor
} = colorsUtils;

/**
 * Adjusts the brightness of a hex color
 */
export function adjustBrightness(hex: string, percent: number): string {
  if (percent > 0) {
    return lighten(hex, percent);
  } else {
    return darken(hex, Math.abs(percent));
  }
}

/**
 * Converts a hex color to RGBA format
 */
export function hexToRgba(hex: string, alpha = 1): string {
  const rgb = hexToRgb(hex);
  return `rgba([rgb.r], [rgb.g], [rgb.b], [alpha])`;
}

/**
 * Determines if a color is light or dark
 */
export function isLightColor(hex: string): boolean {
  const rgb = hexToRgb(hex);
  // Calculate perceived brightness using the sRGB color space
  const brightness = (rgb.r * 299 + rgb.g * 587 + rgb.b * 114) / 1000;
  return brightness > 128;
}

// React hooks for color manipulation
export function useColorManipulation() {
  return useMemo(() => ({
    hexToRgb,
    rgbToHex,
    hexToRgba,
    darken,
    lighten,
    adjustBrightness,
    isLightColor,
    getContrastColor
  }), []);
}

// Hook for theme-aware color generation
export function useThemeColors(baseColor: string) {
  return useMemo(() => ({
    base: baseColor,
    light: lighten(baseColor, 20),
    lighter: lighten(baseColor, 40),
    dark: darken(baseColor, 20),
    darker: darken(baseColor, 40),
    contrast: getContrastColor(baseColor),
    rgba: (alpha: number) => hexToRgba(baseColor, alpha)
  }), [baseColor]);
}
```

#### 3.2.3 Vector Utilities

**Action Items:**
1. Move vector utilities to shared library
2. Create React-specific hooks for vector operations
3. Optimize for React rendering

**Implementation Example:**

```typescript
// src/shared/vectors.ts
import [vectorUtils] from '@bluefly/cli/shared/math';
import [useMemo] from 'react';

// Export core vector utilities
export const {
  dotProduct,
  magnitude,
  normalize,
  euclideanDistance,
  cosineSimilarity
} = vectorUtils;

/**
 * Calculate the average of multiple vectors
 */
export function averageVectors(vectors: number[][]): number[] {
  if (vectors.length === 0) {
    throw new Error('Cannot average empty vector array');
  }
  
  const vecLength = vectors[0].length;
  
  // Check that all vectors have the same length
  if (!vectors.every(vec => vec.length === vecLength)) {
    throw new Error('All vectors must have the same length');
  }
  
  // Initialize result vector with zeros
  const result = new Array(vecLength).fill(0);
  
  // Sum up all vectors
  for (const vec of vectors) {
    for (let i = 0; i < vecLength; i++) {
      result[i] += vec[i];
    }
  }
  
  // Divide by count to get average
  return result.map(val => val / vectors.length);
}

/**
 * Calculate the weighted average of multiple vectors
 */
export function weightedAverageVectors(
  vectors: number[][],
  weights: number[]
): number[] {
  if (vectors.length === 0) {
    throw new Error('Cannot average empty vector array');
  }
  
  if (vectors.length !== weights.length) {
    throw new Error('Number of vectors must match number of weights');
  }
  
  const vecLength = vectors[0].length;
  
  // Check that all vectors have the same length
  if (!vectors.every(vec => vec.length === vecLength)) {
    throw new Error('All vectors must have the same length');
  }
  
  // Initialize result vector with zeros
  const result = new Array(vecLength).fill(0);
  
  // Calculate weighted sum
  for (let i = 0; i < vectors.length; i++) {
    const vec = vectors[i];
    const weight = weights[i];
    
    for (let j = 0; j < vecLength; j++) {
      result[j] += vec[j] * weight;
    }
  }
  
  return result;
}

/**
 * Find the nearest neighbors to a query vector
 */
export function findNearestNeighbors(
  queryVector: number[],
  vectorDatabase: number[][],
  k: number = 5,
  similarityFn: (a: number[], b: number[]) => number = cosineSimilarity
): { index: number; similarity: number }[] {
  const similarities = vectorDatabase.map((vec, index) => ({
    index,
    similarity: similarityFn(queryVector, vec)
  }));
  
  // Sort by similarity in descending order
  similarities.sort((a, b) => b.similarity - a.similarity);
  
  // Return top k results
  return similarities.slice(0, k);
}

// React hooks for vector operations
export function useVectorOperations() {
  return useMemo(() => ({
    dotProduct,
    magnitude,
    normalize,
    euclideanDistance,
    cosineSimilarity,
    averageVectors,
    weightedAverageVectors,
    findNearestNeighbors
  }), []);
}

// Hook for nearest neighbor search with memoization
export function useNearestNeighbors<T>(
  items: T[],
  getVector: (item: T) => number[],
  options: {
    k?: number;
    similarityThreshold?: number;
  } = {}
) {
  const { k = 5, similarityThreshold = 0.7 } = options;
  
  // Precompute all vectors for items
  const vectors = useMemo(() => items.map(getVector), [items, getVector]);
  
  // Return search function
  return useMemo(() => {
    return (queryVector: number[]) => {
      const neighbors = findNearestNeighbors(queryVector, vectors, k);
      
      // Filter by similarity threshold
      const filteredNeighbors = neighbors.filter(
        n => n.similarity >= similarityThreshold
      );
      
      // Map back to original items
      return filteredNeighbors.map(n => ({
        item: items[n.index],
        similarity: n.similarity
      }));
    };
  }, [items, vectors, k, similarityThreshold]);
}
```

### 3.3. Phase 3: Component Integration

#### 3.3.1 Create Utility Provider

**Action Items:**
1. Create a UtilityProvider component to provide all shared utilities
2. Implement React Context for shared utilities
3. Create React hooks for all utilities

**Implementation Example:**

```typescript
// src/shared/UtilityProvider.tsx
import React, { createContext, useContext, useMemo } from 'react';
import { logger, ReactLogger, useLogger } from './logger';
import { httpClient, useApi, useCachedApi } from './http';
import { 
  formatNumber, formatDate, formatCurrency, 
  formatBytes, formatDuration, formatPercentage,
  formatRelativeTime
} from './formatters';
import {
  hexToRgb, rgbToHex, hexToRgba, 
  darken, lighten, adjustBrightness,
  isLightColor, getContrastColor,
  useColorManipulation, useThemeColors
} from './colors';
import { ErrorBoundary, useErrorHandler } from './errors';

// Create the shared utilities context
interface SharedUtilitiesContext {
  logger: ReactLogger;
  http: typeof httpClient;
  formatters: {
    formatNumber: typeof formatNumber;
    formatDate: typeof formatDate;
    formatCurrency: typeof formatCurrency;
    formatBytes: typeof formatBytes;
    formatDuration: typeof formatDuration;
    formatPercentage: typeof formatPercentage;
    formatRelativeTime: typeof formatRelativeTime;
  };
  colors: {
    hexToRgb: typeof hexToRgb;
    rgbToHex: typeof rgbToHex;
    hexToRgba: typeof hexToRgba;
    darken: typeof darken;
    lighten: typeof lighten;
    adjustBrightness: typeof adjustBrightness;
    isLightColor: typeof isLightColor;
    getContrastColor: typeof getContrastColor;
  };
}

const UtilitiesContext = createContext<SharedUtilitiesContext | null>(null);

// Provider component for shared utilities
interface UtilityProviderProps {
  children: React.ReactNode;
}

export function UtilityProvider([children]: UtilityProviderProps) {
  const componentLogger = useLogger('UtilityProvider');
  
  // Create the context value with all utilities
  const contextValue = useMemo<SharedUtilitiesContext>(() => ({
    logger: componentLogger,
    http: httpClient,
    formatters: {
      formatNumber,
      formatDate,
      formatCurrency,
      formatBytes,
      formatDuration,
      formatPercentage,
      formatRelativeTime
    },
    colors: {
      hexToRgb,
      rgbToHex,
      hexToRgba,
      darken,
      lighten,
      adjustBrightness,
      isLightColor,
      getContrastColor
    }
  }), [componentLogger]);
  
  return (
    <ErrorBoundary>
      <UtilitiesContext.Provider value=[contextValue]>
        [children]
      </UtilitiesContext.Provider>
    </ErrorBoundary>
  );
}

// Hook for accessing shared utilities
export function useSharedUtilities() {
  const context = useContext(UtilitiesContext);
  if (!context) {
    throw new Error('useSharedUtilities must be used within a UtilityProvider');
  }
  return context;
}

// Export individual utility hooks
export { useLogger, useApi, useCachedApi, useErrorHandler, useColorManipulation, useThemeColors };
```

#### 3.3.2 Update Components to Use Shared Utilities

**Action Items:**
1. Update key components to use shared utilities
2. Create standardized patterns for component implementation
3. Optimize for performance and maintainability

**Implementation Example:**

```typescript
// src/components/Button/Button.tsx
import React from 'react';
import [useLogger] from '../../shared/logger';
import [useColorManipulation] from '../../shared/colors';
import [useErrorHandler] from '../../shared/errors';
import [cn] from '../utils/cn';

export interface ButtonProps extends React.ButtonHTMLAttributes<HTMLButtonElement> {
  variant?: 'primary' | 'secondary' | 'outline' | 'ghost';
  size?: 'sm' | 'md' | 'lg';
  loading?: boolean;
  fullWidth?: boolean;
}

export const Button = React.forwardRef<HTMLButtonElement, ButtonProps>(
  ({ 
    children, 
    variant = 'primary', 
    size = 'md', 
    loading = false,
    fullWidth = false,
    className, 
    disabled,
    onClick,
    ...props 
  }, ref) => {
    const logger = useLogger('Button');
    const { handleError } = useErrorHandler();
    
    const handleClick = (e: React.MouseEvent<HTMLButtonElement>) => {
      try {
        logger.debug('Button clicked', { variant, size, disabled });
        
        if (onClick) {
          onClick(e);
        }
      } catch (error) {
        handleError(error);
      }
    };
    
    return (
      <button
        ref=[ref]
        className={cn(
          'inline-flex items-center justify-center rounded-md font-medium transition-colors',
          'focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-offset-2',
          {
            'bg-primary text-primary-foreground hover:bg-primary/90': variant === 'primary',
            'bg-secondary text-secondary-foreground hover:bg-secondary/90': variant === 'secondary',
            'border border-input bg-background hover:bg-accent hover:text-accent-foreground': variant === 'outline',
            'hover:bg-accent hover:text-accent-foreground': variant === 'ghost',
            'h-9 px-3 text-sm': size === 'sm',
            'h-10 px-4 py-2': size === 'md',
            'h-11 px-8 text-lg': size === 'lg',
            'w-full': fullWidth,
            'opacity-50 cursor-not-allowed': disabled || loading,
          },
          className
        )}
        disabled=[disabled || loading]
        onClick=[handleClick]
        [...props]
      >
        {loading ? (
          <span className="mr-2 inline-block h-4 w-4 animate-spin rounded-full border-2 border-current border-t-transparent" />
        ) : null}
        [children]
      </button>
    );
  }
);

Button.displayName = 'Button';
```

## 4. Migration Roadmap

### 4.1 Phase 1: Preparation and Infrastructure (Week 1)

1. Create the core shared utility adapters
   - Logger implementation
   - Error handling
   - HTTP client
2. Setup unit tests for all adapters
3. Create documentation for the new patterns

### 4.2 Phase 2: Utility Migration (Week 2)

1. Migrate all utility functions to use shared utilities
   - Formatters
   - Color utilities
   - Vector utilities
2. Create React-specific hooks and adapters
3. Test and validate all utility functions

### 4.3 Phase 3: Component Updates (Weeks 3-4)

1. Update core components to use shared utilities
2. Create the UtilityProvider component
3. Refactor API services to use the shared HTTP client
4. Implement error boundaries throughout the application

### 4.4 Phase 4: Testing and Refinement (Week 5)

1. Comprehensive testing of all components
2. Performance optimization
3. Documentation updates
4. Final validation and review

## 5. Implementation Approach

### 5.1 Code Pattern Updates

1. **Standard Import Pattern**:
   ```typescript
   // Before
   import [createLogger] from '@bluefly/shared-utils';
   
   // After
   import { logger, useLogger } from '@bluefly/cli/shared';
   ```

2. **Component Pattern**:
   ```typescript
   // Before
   import React from 'react';
   import logger from '../../utils/logger';
   
   const MyComponent = () => {
     const handleClick = () => {
       try {
         logger.info('Button clicked');
       } catch (e) {
         console.error(e);
       }
     };
     
     return <button onClick=[handleClick]>Click Me</button>;
   };
   
   // After
   import React from 'react';
   import { useLogger, useErrorHandler } from '../../shared';
   
   const MyComponent = () => {
     const logger = useLogger('MyComponent');
     const { handleError } = useErrorHandler();
     
     const handleClick = () => {
       try {
         logger.info('Button clicked');
       } catch (e) {
         handleError(e);
       }
     };
     
     return <button onClick=[handleClick]>Click Me</button>;
   };
   ```

3. **API Pattern**:
   ```typescript
   // Before
   import [apiClient] from '../../services/api';
   
   const fetchData = async () => {
     try {
       const response = await apiClient.get('/endpoint');
       return response.data;
     } catch (e) {
       console.error(e);
     }
   };
   
   // After
   import [useApi] from '../../shared/http';
   
   const MyComponent = () => {
     const { data, loading, error, refetch } = useApi('/endpoint');
     
     // Component using data, loading, error states
     return loading ? <Loader /> : error ? <Error /> : <Data data=[data] />;
   };
   ```

### 5.2 Progressive Implementation

The implementation will follow a progressive approach:

1. **Infrastructure First**: Create the core shared utility adapters
2. **Utility Migration**: Replace all utility functions with shared implementations
3. **Core Components**: Update the most frequently used components
4. **Complete Integration**: Update all remaining components

### 5.3 Testing Strategy

1. **Unit Tests**: Create comprehensive tests for all utility adapters
2. **Integration Tests**: Test component interactions with shared utilities
3. **Performance Tests**: Benchmark before and after migration
4. **Visual Regression Tests**: Ensure UI consistency

## 6. Success Criteria

The migration will be considered successful when:

1. All components use standardized utilities from `@bluefly/cli/shared`
2. No references to `@bluefly/shared-utils` remain
3. All React components use the appropriate React-specific adapters
4. Test coverage is maintained or improved
5. No regressions in functionality or performance
6. Documentation is updated to reflect the new patterns

## 7. Conclusion

This migration plan provides a comprehensive approach to standardizing BFUI on the shared utilities from `@bluefly/cli/shared`. By implementing React-specific adapters and hooks, we ensure that components benefit from shared infrastructure while maintaining React-specific optimizations.

The phased approach allows for incremental implementation and validation, minimizing disruption to ongoing development. The end result will be a more maintainable, consistent, and efficient UI library that fully leverages the shared utilities ecosystem.