# Tooltip Component

A highly customizable, accessible, and animated tooltip component for React applications.

## Features

-  **Smart Positioning**: Uses Popper.js for intelligent positioning and automatic flipping
-  **Multiple Animations**: Fade, scale, slide, or no animation
- ♿ **Accessibility**: WCAG 2.2 AA compliant with proper ARIA attributes
- 🎭 **Theming**: Light, dark, and system theme support
- ⌨️ **Keyboard Navigation**: Full keyboard support
- 🎮 **Multiple Triggers**: Hover, click, focus, or manual control
- 🌈 **Customizable**: Extensive props for styling and behavior
- 📱 **Responsive**: Works well on all screen sizes
-  **Performance**: Optimized with React hooks and proper cleanup

## Installation

```bash
# Core dependencies
npm install @reach/auto-id @react-spring/web react-popper

# Peer dependencies (should already be installed)
npm install react react-dom
```

## Usage

```tsx
import [Tooltip] from './components/overlay/Tooltip';

// Basic usage
<Tooltip content="Simple tooltip">
  <button>Hover me</button>
</Tooltip>

// Advanced usage
<Tooltip
  content=[<div>
      <h3>Rich Content</h3>
      <p>Supports any React node as content</p>
    </div>]
  placement="top-start"
  trigger="click"
  animation="scale"
  interactive
  theme="dark"
  maxWidth=[400]
  onShow={() => console.log('Shown')}
  onHide={() => console.log('Hidden')}
>
  <button>Click me</button>
</Tooltip>
```

## Props

| Prop | Type | Default | Description |
|------|------|---------|-------------|
| `children` | `ReactNode` | Required | The element that triggers the tooltip |
| `content` | `ReactNode` | Required | The tooltip content |
| `placement` | `TooltipPlacement` | `'top'` | Tooltip placement relative to trigger |
| `trigger` | `'hover' \| 'click' \| 'focus' \| 'manual'` | `'hover'` | How the tooltip is triggered |
| `size` | `'sm' \| 'md' \| 'lg'` | `'md'` | Size variant of the tooltip |
| `isOpen` | `boolean` | `undefined` | Controlled open state |
| `onOpenChange` | `(isOpen: boolean) => void` | `undefined` | Callback when open state changes |
| `delay` | `number` | `0` | Delay before showing tooltip (ms) |
| `hideDelay` | `number` | `0` | Delay before hiding tooltip (ms) |
| `className` | `string` | `undefined` | Additional class for trigger element |
| `tooltipClassName` | `string` | `undefined` | Additional class for tooltip element |
| `disabled` | `boolean` | `false` | Disable the tooltip |
| `offset` | `number` | `8` | Offset from trigger element (px) |
| `arrow` | `boolean` | `true` | Show arrow pointer |
| `portal` | `boolean` | `false` | Render tooltip in portal |
| `portalTarget` | `HTMLElement` | `document.body` | Portal target element |
| `ariaLabel` | `string` | `'Tooltip'` | ARIA label for tooltip |
| `animation` | `'fade' \| 'scale' \| 'slide' \| 'none'` | `'fade'` | Animation type |
| `maxWidth` | `number` | `300` | Maximum width of tooltip (px) |
| `interactive` | `boolean` | `false` | Allow hovering over tooltip |
| `strategy` | `'absolute' \| 'fixed'` | `'absolute'` | Positioning strategy |
| `modifiers` | `Array<any>` | `[]` | Popper.js modifiers |
| `onShow` | `() => void` | `undefined` | Callback when tooltip shows |
| `onHide` | `() => void` | `undefined` | Callback when tooltip hides |
| `zIndex` | `number` | `1000` | z-index of tooltip |
| `theme` | `'light' \| 'dark' \| 'system'` | `'system'` | Theme variant |

## Placement Options

```typescript
type TooltipPlacement =
  | 'top'
  | 'bottom'
  | 'left'
  | 'right'
  | 'top-start'
  | 'top-end'
  | 'bottom-start'
  | 'bottom-end'
  | 'left-start'
  | 'left-end'
  | 'right-start'
  | 'right-end';
```

## CSS Classes

The component uses BEM-style class names:

```css
.bf-tooltip
.bf-tooltip--[size]        /* sm, md, lg */
.bf-tooltip--[placement]   /* top, bottom, etc. */
.bf-tooltip--[theme]       /* light, dark, system */
.bf-tooltip--arrow
.bf-tooltip--interactive
.bf-tooltip__arrow
.bf-tooltip__content
.bf-tooltip__trigger
.bf-tooltip__trigger--disabled
```

## Accessibility

- Uses proper ARIA attributes (`role="tooltip"`, `aria-describedby`, etc.)
- Keyboard navigation support (Escape to close)
- Focus management
- Screen reader friendly
- WCAG 2.2 AA compliant

## Browser Support

- Chrome (latest)
- Firefox (latest)
- Safari (latest)
- Edge (latest)
- IE11 (with polyfills)

## Performance

- Uses React hooks for optimal performance
- Proper cleanup of event listeners and timeouts
- Memoized callbacks
- Efficient re-renders
- Portal support for better stacking context

## Contributing

1. Fork the repository
2. Create your feature branch (`git checkout -b feature/amazing-feature`)
3. Commit your changes (`git commit -m 'Add amazing feature'`)
4. Push to the branch (`git push origin feature/amazing-feature`)
5. Open a Pull Request

## License

MIT © LLM Platform.io 