# Quantum Swarm Orchestrator Client

A JavaScript/TypeScript client library for interacting with the Quantum Swarm Orchestrator API from frontend applications. This library enables frontend components to participate in quantum-inspired multi-agent orchestration across distributed Drupal instances.

## Overview

The Quantum Swarm Orchestrator is a revolutionary system that enables multiple quantum thought processes to collaborate across different Drupal instances through federated swarm intelligence. This client library provides a simple interface for frontend applications to interact with this system.

## Key Features

- **Agent Registration & Management**: Register and specialize quantum agents with specific capabilities
- **Cognitive Workflow Creation**: Create and manage cognitive workflows that orchestrate multiple agents
- **Insight Contribution**: Submit insights from agents to cognitive workflows
- **Consensus Building**: Initiate and manage quantum-weighted consensus processes
- **Entanglement Management**: Create and optimize entanglement groups for agent collaboration
- **React Integration**: Includes a React hook for easy integration with React applications

## Installation

```bash
# If you're using npm
npm install --save @bluefly/quantum-swarm-client

# If you're using yarn
yarn add @bluefly/quantum-swarm-client
```

## Basic Usage

```typescript
import QuantumSwarmClient from '@bluefly/quantum-swarm-client';

// Create a client instance
const client = new QuantumSwarmClient({
  drupalBaseUrl: 'https://your-drupal-instance.com',
  apiBasePath: '/api/quantum-swarm'
});

// Register a new agent
const agent = await client.registerAgent({
  id: 'agent-123',
  name: 'Pattern Recognition Agent',
  capabilities: ['pattern_recognition', 'insight_generation', 'consensus_building'],
  quantumAffinity: 0.8,
  entanglementCapacity: 5,
  specializationScores: {
    'pattern_recognition': 0.9,
    'data_analysis': 0.7,
    'insight_generation': 0.8
  }
});

// Create a cognitive workflow
const workflow = await client.createCognitiveWorkflow(
  'Data Analysis Workflow',
  'A cognitive workflow for analyzing complex data patterns',
  ['agent-123', 'agent-456'],
  ['pattern_recognition', 'data_analysis']
);

// Contribute an insight to the workflow
const insight = await client.contributeInsight(
  workflow.id,
  agent.id,
  'emergence',
  {
    title: 'Emergent Pattern Detected',
    description: 'A novel pattern has emerged from the data analysis',
    patternData: { /* ... */ }
  },
  0.85 // confidence level
);

// Initiate a consensus process
const consensus = await client.initiateConsensus(
  workflow.id,
  'Data interpretation approach',
  {
    weightByQuantumAffinity: true,
    threshold: 0.75
  }
);
```

## React Integration

The library includes a React hook for easy integration with React applications:

```typescript
import [useQuantumSwarm] from '@bluefly/quantum-swarm-client';

function QuantumWorkflowComponent() {
  const {
    client,
    agents,
    workflows,
    insights,
    loading,
    error,
    createAgent,
    createWorkflow,
    contributeInsight
  } = useQuantumSwarm({
    drupalBaseUrl: 'https://your-drupal-instance.com',
    autoInitialize: true
  });

  // Create a new agent
  const handleCreateAgent = async () => {
    try {
      const newAgent = await createAgent({
        name: 'New Agent',
        capabilities: ['insight_generation'],
        quantumAffinity: 0.7,
        entanglementCapacity: 3,
        specializationScores: {
          'insight_generation': 0.8
        }
      });
      
      console.log('Created agent:', newAgent);
    } catch (err) {
      console.error('Failed to create agent:', err);
    }
  };

  // Render component
  return (
    <div>
      <h2>Quantum Agents</h2>
      {loading.agents ? (
        <p>Loading agents...</p>
      ) : (
        <ul>
          {agents.map(agent => (
            <li key={agent.id}>[agent.name]</li>
          ))}
        </ul>
      )}
      
      <button onClick=[handleCreateAgent]>Create Agent</button>
      
      {/* Additional UI for workflows, insights, etc. */}
    </div>
  );
}
```

## Advanced Entanglement Management

The library provides advanced entanglement management capabilities through the `EntanglementManager` class:

```typescript
import [EntanglementManager] from '@bluefly/quantum-swarm-client';

// Create an entanglement manager
const entanglementManager = new EntanglementManager(client);

// Calculate compatibility between two agents
const compatibility = entanglementManager.calculateAgentCompatibility(agent1, agent2);

// Create optimal entanglement groups
const groups = entanglementManager.createOptimalEntanglementGroups(
  agents,
  {
    minCompatibilityScore: 0.5,
    preferComplementaryAgents: true,
    maxGroupSize: 4,
    enforceCrossInstanceDiversity: true,
    prioritizeSpecializations: ['pattern_recognition', 'insight_generation']
  }
);

// Optimize existing entanglement groups
const optimizedGroups = entanglementManager.optimizeEntanglementGroups(
  existingGroups,
  allAgents,
  recentInsights
);
```

## API Reference

### QuantumSwarmClient

The main client class for interacting with the Quantum Swarm Orchestrator API.

#### Constructor

```typescript
constructor(config?: QuantumSwarmClientConfig)
```

Configuration options:
- `drupalBaseUrl`: Base URL of the Drupal instance
- `apiBasePath`: Base path for the Quantum Swarm API endpoints
- `headers`: Additional headers to include in API requests
- `timeout`: Request timeout in milliseconds

#### Methods

##### Agent Management

- `registerAgent(agent: Omit<SwarmAgent, 'quantumState' | 'currentEntanglements'>): Promise<SwarmAgent>`
  Registers a new agent with the quantum swarm orchestrator

- `specializeAgent(agentId: string, specializations: Record<string, number>): Promise<SwarmAgent>`
  Specializes an agent for specific types of quantum thought processes

##### Workflow Management

- `createCognitiveWorkflow(name: string, description: string, agentIds?: string[], specializationAreas?: string[]): Promise<CognitiveWorkflow>`
  Creates a new cognitive workflow that orchestrates multiple quantum thought processes

- `getWorkflowStatus(workflowId: string): Promise<CognitiveWorkflow | null>`
  Gets the status of a cognitive workflow

##### Insight Management

- `contributeInsight(workflowId: string, agentId: string, insightType: WorkflowInsight['type'], insightContent: any, confidence?: number): Promise<WorkflowInsight>`
  Contributes an insight from an agent to a cognitive workflow

- `getWorkflowInsights(workflowId: string): Promise<WorkflowInsight[]>`
  Gets all insights from a cognitive workflow

##### Consensus Building

- `initiateConsensus(workflowId: string, topic: string, options?: Partial<ConsensusConfiguration>): Promise<any>`
  Initiates a consensus-building process within a cognitive workflow

##### Entanglement Metrics

- `getEntanglementMetrics(): Promise<any>`
  Gets entanglement metrics for the quantum swarm

### useQuantumSwarm

React hook for interacting with the Quantum Swarm Orchestrator.

```typescript
function useQuantumSwarm(options?: UseQuantumSwarmOptions): UseQuantumSwarmResult
```

Options:
- `autoInitialize`: Whether to automatically fetch agents and workflows on mount
- All options from `QuantumSwarmClientConfig`

Returns:
- `client`: The quantum swarm client instance
- `agents`: Array of registered agents
- `workflows`: Array of cognitive workflows
- `insights`: Array of workflow insights
- `loading`: Loading states for different operations
- `error`: Error states for different operations
- Methods for interacting with the quantum swarm

### EntanglementManager

Class for managing quantum entanglement groups.

```typescript
constructor(client: QuantumSwarmClient)
```

Methods:
- `calculateAgentCompatibility(agent1: SwarmAgent, agent2: SwarmAgent): AgentCompatibility`
  Calculates compatibility between two agents

- `createOptimalEntanglementGroups(agents: SwarmAgent[], options?: EntanglementOptimizationOptions): EntanglementGroup[]`
  Creates optimal entanglement groups from a set of agents

- `calculateCoherenceMetrics(group: EntanglementGroup, insights?: WorkflowInsight[]): EntanglementGroup['coherenceMetrics']`
  Calculates coherence metrics for an entanglement group

- `updateGroupWithInsight(group: EntanglementGroup, insight: WorkflowInsight): EntanglementGroup`
  Updates an entanglement group with a new insight

- `optimizeEntanglementGroups(groups: EntanglementGroup[], agents: SwarmAgent[], insights: WorkflowInsight[]): EntanglementGroup[]`
  Optimizes existing entanglement groups based on performance

## Core Interfaces

### SwarmAgent

```typescript
interface SwarmAgent {
  id: string;
  name: string;
  drupalInstanceId?: string;
  capabilities: string[];
  quantumAffinity: number;
  entanglementCapacity: number;
  currentEntanglements: string[];
  specializationScores: Record<string, number>;
  quantumState: {
    superposition: number;
    coherence: number;
    entanglement: number;
  };
}
```

### CognitiveWorkflow

```typescript
interface CognitiveWorkflow {
  id: string;
  name: string;
  description: string;
  status: 'initializing' | 'in_progress' | 'completed' | 'failed';
  createdAt: Date;
  updatedAt: Date;
  completedAt?: Date;
  agentIds: string[];
  phases: WorkflowPhase[];
  insights: WorkflowInsight[];
  metrics: {
    coherenceLevel: number;
    consensusRate: number;
    emergenceIntensity: number;
    quantumAdvantage: number;
  };
}
```

### WorkflowInsight

```typescript
interface WorkflowInsight {
  id: string;
  type: 'convergence' | 'divergence' | 'emergence' | 'quantum_leap' | 'consensus';
  description: string;
  contributingAgents: string[];
  confidenceScore: number;
  timestamp: Date;
  content: any;
  relatedPatterns: string[];
  verificationStatus: 'unverified' | 'partially_verified' | 'verified';
}
```

### EntanglementGroup

```typescript
interface EntanglementGroup {
  id: string;
  agentIds: string[];
  entanglementStrength: number;
  entanglementType: 'direct' | 'workflow' | 'insight' | 'emergent';
  createdAt: Date;
  lastInteractionAt: Date;
  interactionCount: number;
  sharedInsights: string[];
  coherenceMetrics: {
    phaseAlignment: number;
    informationTransfer: number;
    correlationStrength: number;
  };
}
```

## Integration with Drupal

This client library is designed to work with the Quantum Swarm Orchestrator Drupal module. The module provides the following API endpoints:

- `/api/quantum-swarm/register-agent` (POST): Register a new quantum agent
- `/api/quantum-swarm/create-workflow` (POST): Create a new quantum cognitive workflow
- `/api/quantum-swarm/contribute-insight` (POST): Contribute an insight to a workflow
- `/api/quantum-swarm/initiate-consensus` (POST): Initiate a consensus-building process
- `/api/quantum-swarm/specialize-agent` (POST): Specialize a quantum agent
- `/api/quantum-swarm/workflow/[workflow_id]/status` (GET): Get the status of a workflow
- `/api/quantum-swarm/workflow/[workflow_id]/insights` (GET): Get insights from a workflow
- `/api/quantum-swarm/entanglement/metrics` (GET): Get entanglement metrics

## Error Handling

All methods return promises that may be rejected with errors. It's recommended to use try/catch blocks to handle errors:

```typescript
try {
  const workflow = await client.createCognitiveWorkflow(
    'Data Analysis Workflow',
    'A cognitive workflow for analyzing complex data patterns'
  );
  console.log('Created workflow:', workflow);
} catch (error) {
  console.error('Failed to create workflow:', error.message);
}
```

## License

This library is licensed under the MIT License. See the LICENSE file for details.