# Compliance Integration Guide

This guide describes how the secure-project NPM package provides GDPR, CCPA, and SOC2 compliance features that can be used by Drupal and other platforms.

## Overview

The secure-project package includes a comprehensive ComplianceManager that orchestrates:

- **GDPR** (General Data Protection Regulation) - EU data protection compliance
- **CCPA** (California Consumer Privacy Act) - California privacy compliance  
- **SOC2** (Service Organization Control 2) - Trust service criteria compliance

## Architecture

```
secure-project/
├── src/
│   ├── compliance/
│   │   ├── ComplianceManager.ts     # Main orchestrator
│   │   ├── GDPRCompliance.ts        # GDPR implementation
│   │   ├── CCPACompliance.ts        # CCPA implementation
│   │   └── SOC2Compliance.ts        # SOC2 implementation
│   └── bridge.ts                    # Bridge for external integration
```

## Using from Drupal

The `gov_compliance` module uses the secure-project bridge to access compliance features:

### 1. Bridge Configuration

```php
// In SecureProjectBridgeService.php
$result = $this->callSecureProject('compliance/generateReport', [
  'framework' => 'gdpr',
  'securityStatus' => $securityStatus,
]);
```

### 2. Available Methods

- `compliance/generateReport` - Generate compliance report
- `compliance/handleDataRequest` - Handle GDPR/CCPA data requests
- `compliance/getControls` - Get framework controls
- `compliance/auditConsent` - Audit GDPR consent
- `compliance/frameworks` - List available frameworks

### 3. Starting the Bridge

```bash
# HTTP mode (recommended for production)
cd /path/to/secure-project
npm run bridge:http

# Process mode (for development)
npm run bridge:process
```

## Direct Usage (Node.js)

```typescript
import [ComplianceManager] from '@secure-project/compliance';

// Initialize
const compliance = new ComplianceManager({
  enabledFrameworks: ['gdpr', 'ccpa', 'soc2'],
  defaultFramework: 'gdpr'
});

// Generate report
const report = await compliance.generateComplianceReport('gdpr', {
  data_encryption: true,
  consent_management: true,
  // ... other security status flags
});

// Handle data request
const result = await compliance.handleDataRequest({
  framework: 'gdpr',
  type: 'erasure',
  user_id: '123',
  verified: true
});
```

## Compliance Features

### GDPR Features

- **Data Protection Principles**: All 7 principles implemented
- **Data Subject Rights**: Articles 15-22 (access, rectification, erasure, etc.)
- **Organizational Measures**: DPO, DPIA, breach notification
- **Consent Management**: Explicit consent tracking and audit
- **30-day response timeline** for data requests

### CCPA Features

- **Consumer Rights**: Know, delete, opt-out, non-discrimination
- **Business Obligations**: Privacy notice, verifiable requests
- **Opt-Out Registry**: "Do Not Sell" mechanism
- **45-day response timeline** for consumer requests

### SOC2 Features

- **Trust Service Criteria**: Security, availability, processing integrity, confidentiality, privacy
- **Common Criteria (CC1-CC9)**: Full implementation
- **Evidence Collection**: Automated evidence gathering
- **Control Objectives**: Comprehensive control mapping

## Security Status Mapping

The compliance frameworks expect certain security status flags:

### GDPR
```javascript
{
  consent_management: true,
  data_encryption: true,
  audit_logging: true,
  privacy_by_design: true,
  data_protection_officer: true,
  // ... see GDPRCompliance.ts for full list
}
```

### CCPA
```javascript
{
  opt_out_mechanism: true,
  data_disclosure: true,
  verifiable_requests: true,
  privacy_notice: true,
  // ... see CCPACompliance.ts for full list
}
```

### SOC2
```javascript
{
  security_review: true,
  system_availability: true,
  access_control: true,
  change_management: true,
  // ... see SOC2Compliance.ts for full list
}
```

## API Reference

### ComplianceManager

```typescript
class ComplianceManager {
  // Get available frameworks
  getAvailableFrameworks(): Record<string, string>

  // Generate compliance report
  generateComplianceReport(
    framework: ComplianceFramework,
    securityStatus: any
  ): Promise<ComplianceReport>

  // Handle data requests (GDPR/CCPA)
  handleDataRequest(
    request: DataRequest
  ): Promise<any>

  // Get framework controls
  getComplianceControls(
    framework: ComplianceFramework
  ): ComplianceControls

  // Audit consent (GDPR)
  auditConsentManagement(
    consentData: any
  ): Promise<any>

  // Validate compliance
  validateCompliance(
    framework: ComplianceFramework,
    securityStatus: any
  ): Promise<any>
}
```

## Testing

```bash
# Run tests
npm test

# Run compliance-specific tests
npm test -- compliance/

# Generate coverage report
npm run test:coverage
```

## Production Deployment

1. **Build the package**:
   ```bash
   npm run build
   ```

2. **Start the bridge**:
   ```bash
   # With PM2
   pm2 start scripts/bridge-production.js -- http

   # With systemd
   node scripts/bridge-production.js http
   ```

3. **Configure Drupal**:
   ```php
   // In settings.php
   $config['gov_compliance.bridge']['secure_project.use_http_api'] = TRUE;
   $config['gov_compliance.bridge']['secure_project.api_base_url'] = 'http://localhost:3010';
   ```

## Troubleshooting

### Bridge Connection Issues
```bash
# Check bridge health
curl http://localhost:3010/health

# Check from Drupal
drush ev 'var_dump(\Drupal::service("gov_compliance.secure_project_bridge")->checkConnection());'
```

### Compliance Report Issues
- Ensure all required security status flags are provided
- Check bridge logs for detailed error messages
- Verify framework is supported

### Data Request Issues
- Verify request type is valid for the framework
- Ensure user verification is completed
- Check response timeline requirements

## Contributing

When adding new compliance frameworks:

1. Create new compliance class in `src/compliance/`
2. Add to ComplianceManager
3. Update bridge.ts with new methods
4. Add tests
5. Update documentation

## License

This module is part of the secure-project enterprise security suite.