# Adapter Plugins

This document describes the adapter plugin system in secure_project, which allows integration with different server frameworks and platforms.

## Overview

The adapter plugin system provides a standardized way to integrate secure_project with various server frameworks and platforms. Each adapter implements a common interface, making it easy to use secure_project features with your preferred framework.

## Supported Adapters

The following adapters are included with secure_project:

| Adapter | Framework | Description |
|---------|-----------|-------------|
| Express | Express.js | Integrates with Express.js web framework |
| Koa | Koa.js | Integrates with Koa.js web framework |
| Drupal | Drupal CMS | Integrates with Drupal CMS via bridge |
| Fastify | Fastify | Integrates with Fastify web framework |
| NestJS | NestJS | Integrates with NestJS framework |

## Adapter Interface

All adapters implement the `AdapterPlugin` interface:

```typescript
interface AdapterPlugin {
  // Unique identifier for the adapter
  id: string;
  
  // Display name of the adapter
  name: string;
  
  // Version of the adapter
  version: string;
  
  // Description of the adapter
  description: string;
  
  // The framework this adapter integrates with
  framework: string;
  
  // Initialize the adapter
  initialize(): Promise<void>;
  
  // Create middleware for the specific framework
  createMiddleware(options: any, context?: AdapterContext): any;
}
```

The `AdapterContext` provides additional context for the adapter:

```typescript
interface AdapterContext {
  // Logger instance to use for adapter-specific logging
  logger?: any;
  
  // Whether audit logging is enabled
  audit?: boolean;
  
  // List of features to enable in the adapter
  features?: string[];
  
  // Custom options passed to the adapter
  [key: string]: any;
}
```

## Adapter Lifecycle

The lifecycle of an adapter consists of the following stages:

1. **Registration**: The adapter is registered with the `AdapterManager`
2. **Initialization**: The adapter's `initialize()` method is called
3. **Middleware Creation**: The adapter's `createMiddleware()` method is called when middleware is needed
4. **Unregistration**: The adapter is unregistered when no longer needed

### Registration

Adapters must be registered with the `AdapterManager` before they can be used:

```typescript
import [AdapterManager] from 'secure_project/plugins/AdapterManager';
import [ExpressAdapter] from 'secure_project/plugins/adapters/ExpressAdapter';

const adapterManager = new AdapterManager();
const expressAdapter = new ExpressAdapter();

// Register the adapter
await adapterManager.registerAdapter(expressAdapter);
```

You can also load all adapters from a directory:

```typescript
await adapterManager.loadAdaptersFromDirectory('/path/to/adapters');
```

### Initialization

When an adapter is registered, its `initialize()` method is called. This is where the adapter should set up any required services or resources.

### Middleware Creation

To create middleware for a specific adapter:

```typescript
const middleware = adapterManager.createMiddleware('express', {
  audit: true,
  accessControl: true
});

// Use the middleware with your framework
app.use(middleware);
```

### Unregistration

When an adapter is no longer needed, it can be unregistered:

```typescript
await adapterManager.unregisterAdapter('express');
```

## Using Adapters

### Express.js

```typescript
import express from 'express';
import [createExpressAdapter] from 'secure_project/plugins/adapters/ExpressAdapter';

const app = express();
const secureMiddleware = createExpressAdapter({
  audit: true,
  accessControl: true
});

app.use(secureMiddleware);

app.get('/protected', (req, res) => {
  // Check access using the middleware
  req.secure_project.checkAccess('read', 'resource', '123')
    .then(result => {
      if (result.allowed) {
        res.send('Access granted');
      } else {
        res.status(403).send('Access denied: ' + result.reason);
      }
    });
});
```

### Koa.js

```typescript
import Koa from 'koa';
import [createKoaAdapter] from 'secure_project/plugins/adapters/KoaAdapter';

const app = new Koa();
const secureMiddleware = createKoaAdapter({
  audit: true,
  accessControl: true
});

app.use(secureMiddleware);

app.use(async ctx => {
  // Check access using the middleware
  const result = await ctx.state.secure_project.checkAccess('read', 'resource', '123');
  
  if (result.allowed) {
    ctx.body = 'Access granted';
  } else {
    ctx.status = 403;
    ctx.body = 'Access denied: ' + result.reason;
  }
});
```

### Drupal

```php
<?php
// In your Drupal module

/**
 * Implements hook_init().
 */
function my_module_init() {
  // Connect to secure_project via bridge
  $bridge = secure_project_get_bridge();
  
  // Use bridge methods
  $result = $bridge->validateAccess([
    'userId' => \Drupal::currentUser()->id(),
    'userRoles' => \Drupal::currentUser()->getRoles(),
    'userTenant' => get_user_tenant(),
    'action' => 'read',
    'entityType' => 'node',
    'entityId' => $node->id(),
    'resourceTenant' => get_node_tenant($node),
  ]);
  
  if (!$result['data']['allowed']) {
    throw new AccessDeniedException($result['data']['reason']);
  }
}
```

### Fastify

```typescript
import fastify from 'fastify';
import [createFastifyAdapter] from 'secure_project/plugins/adapters/FastifyAdapter';

const app = fastify();
const securePlugin = createFastifyAdapter({
  audit: true,
  accessControl: true
});

app.register(securePlugin);

app.get('/protected', async (request, reply) => {
  // Check access using the plugin
  const result = await request.secure_project.checkAccess('read', 'resource', '123');
  
  if (result.allowed) {
    return 'Access granted';
  } else {
    reply.code(403);
    return 'Access denied: ' + result.reason;
  }
});
```

### NestJS

```typescript
import [Module] from '@nestjs/common';
import [createNestJSAdapter] from 'secure_project/plugins/adapters/NestJSAdapter';

const { SecureProjectModule, AccessControlGuard } = createNestJSAdapter({
  audit: true,
  accessControl: true
});

@Module({
  imports: [
    SecureProjectModule.forRoot({
      // Additional options
    })
  ],
  controllers: [],
  providers: []
})
export class AppModule {}

// In your controller
@Controller('resources')
export class ResourceController [constructor(@Inject('SECURE_PROJECT_SERVICE') private secureProject) {]
  
  @Get(':id')
  @UseGuards(AccessControlGuard)
  @ResourceType('resource') // Custom decorator to set resource type
  @Action('read') // Custom decorator to set action
  async getResource(@Param('id') id: string) {
    // Access is already checked by the guard
    return { id, data: 'Resource data' };
  }
}
```

## Creating Custom Adapters

You can create custom adapters for other frameworks or platforms by implementing the `AdapterPlugin` interface:

```typescript
import { AdapterPlugin, AdapterContext } from 'secure_project/plugins/AdapterPlugin';

export class MyCustomAdapter implements AdapterPlugin {
  id = 'my-custom';
  name = 'My Custom Adapter';
  version = '1.0.0';
  description = 'Custom adapter for my framework';
  framework = 'my-framework';
  
  async initialize(): Promise<void> {
    // Initialize services
  }
  
  createMiddleware(options: any = {}, context?: AdapterContext): any {
    // Create middleware for your framework
    return function(req, res, next) {
      // Implement middleware logic
      next();
    };
  }
}
```

## Adapter Manager API

The `AdapterManager` class provides the following methods:

### `registerAdapter(adapter: AdapterPlugin): Promise<void>`

Registers an adapter with the manager.

### `unregisterAdapter(adapterId: string): Promise<void>`

Unregisters an adapter by ID.

### `getAdapter(adapterId: string): AdapterPlugin | undefined`

Gets an adapter by ID.

### `getAdapterByFramework(framework: string): AdapterPlugin | undefined`

Gets an adapter by framework name.

### `listAdapters(): AdapterPlugin[]`

Lists all registered adapters.

### `createMiddleware(adapterId: string, options: any, context?: AdapterContext): any`

Creates middleware for a specific adapter.

### `loadAdaptersFromDirectory(directory: string): Promise<AdapterPlugin[]>`

Loads all adapters from a directory.