# Separation of Duties (SoD) Framework

The Separation of Duties module for Drupal implements a robust security framework designed to enforce the principle of separation of duties in organizational workflows. This module helps organizations comply with regulatory requirements like SOX, HIPAA, GDPR, and other compliance frameworks that mandate role segregation for critical operations.

## Overview

Separation of Duties (SoD) is a key principle in internal control mechanisms that prevents fraud and errors by ensuring that no single individual has control over all parts of a critical transaction or process. This module provides:

- **Role Segregation**: Defines and enforces incompatible role combinations
- **Multi-stage Approval Workflows**: Requires approvals from users with specific roles for sensitive operations
- **Audit Logging**: Comprehensive audit trail for all SoD-related actions
- **Policy Management**: Flexible policy framework for defining and enforcing SoD rules
- **Compliance Framework Integration**: Support for multiple regulatory compliance frameworks

## Key Features

### Role Separation Enforcement

- Define incompatible role combinations with reasons
- Enforce role separation at user creation and role assignment time
- Configure enforcement levels (advisory, warning, blocking)
- Maximum roles per user constraints

### Approval Workflows

- Multi-stage approval processes for sensitive operations
- Role-based approver requirements
- Approval request management
- Workflow auditing and evidence collection

### Audit Trail

- Comprehensive logging of all SoD-related actions
- Configurable retention policies
- Evidence generation for compliance audits
- Export capabilities for reporting

### Policy Management

- Flexible policy definition
- Support for multiple compliance frameworks
- Version control for policies
- Policy import/export

## Architecture

The SoD framework is built around these core components:

1. **RoleSeparationEnforcer**: Core service for detecting and preventing incompatible role combinations
2. **WorkflowApprovalManager**: Manages multi-stage approval workflows for sensitive operations
3. **AuditTrailService**: Provides comprehensive audit logging with retention policies
4. **SeparationOfDutiesManager**: Top-level service that coordinates the SoD functionality
5. **Plugin System**: Extensible plugin-based architecture for supporting different compliance frameworks

## Configuration

### Basic Setup

1. Enable the module: `drush en separation_of_duties`
2. Configure settings at: `/admin/config/secure-drupal/separation-of-duties/settings`
3. Define incompatible roles at: `/admin/config/secure-drupal/separation-of-duties/roles`
4. Configure approval workflows at: `/admin/config/secure-drupal/separation-of-duties/workflows`

### Enforcement Levels

- **Advisory**: Logs violations but doesn't prevent actions
- **Warning**: Shows warnings but allows actions to proceed
- **Blocking**: Prevents actions that violate SoD policies

## Integration

### ECA Integration

The module integrates with the [ECA (Event-Condition-Action)](https://www.drupal.org/project/eca) module to provide:

- Event-driven workflow enforcement
- Conditional policy application
- Custom actions for policy violations

### CI/CD Integration

Policies can be versioned as code and integrated with CI/CD pipelines:

- Export policies to YAML configuration
- Version control SoD policies
- Automated policy testing

## Usage Examples

### Role Separation

```php
// Check if roles are compatible
$compatible = \Drupal::service('separation_of_duties.role_enforcer')
  ->areRolesCompatible('financial_initiator', 'financial_approver');

// Get incompatible roles for a role
$incompatible_roles = \Drupal::service('separation_of_duties.role_enforcer')
  ->getIncompatibleRoles('technical_admin');
```

### Approval Workflows

```php
// Create an approval request
$request_id = \Drupal::service('separation_of_duties.workflow_approval_manager')
  ->createApprovalRequest(
    'financial_transaction',
    'payment',
    ['amount' => 5000, 'recipient' => 'Vendor Inc.']
  );

// Approve a request
$approved = \Drupal::service('separation_of_duties.workflow_approval_manager')
  ->approveRequest($request_id, 'Approved after verification');
```

## Extending the Framework

### Custom Compliance Frameworks

Create a custom compliance framework by implementing:

1. A `DutySeparator` plugin
2. A `ComplianceEnforcer` plugin
3. Configuration for your framework

## Troubleshooting

### Common Issues

- **Role conflicts not detected**: Check enforcement level in configuration
- **Approval workflows not triggered**: Verify workflow configuration
- **Audit logs missing**: Check database tables and permissions

## API Reference

See the PHPDoc in the following core services:

- `\Drupal\separation_of_duties\Service\RoleSeparationEnforcer`
- `\Drupal\separation_of_duties\Service\WorkflowApprovalManager`
- `\Drupal\separation_of_duties\Service\AuditTrailService`
- `\Drupal\separation_of_duties\Service\SeparationOfDutiesManager`

## License

This module is licensed under the GPL v2 or later.

## Credits

Developed as part of the Secure Drupal initiative by the Drupal Security Team.