# Secure Drupal Recipe Deployment Guide

## Table of Contents
1. [Prerequisites](#prerequisites)
2. [Installation Methods](#installation-methods)
3. [Production Deployment](#production-deployment)
4. [Configuration](#configuration)
5. [Validation & Testing](#validation--testing)
6. [Troubleshooting](#troubleshooting)
7. [Maintenance](#maintenance)

## Prerequisites

### System Requirements
- **Drupal**: Version 11.0 or higher
- **PHP**: 8.2 or higher
- **Database**: PostgreSQL 13+ or MySQL 8.0+
- **Memory**: Minimum 512MB PHP memory limit
- **Disk Space**: 100MB for modules + audit logs

### Required Tools
- **Composer**: 2.0 or higher
- **Drush**: 12.0 or higher
- **Node.js**: 18.0 or higher (for build tools)
- **Git**: For version control

### Pre-deployment Checklist
- [ ] Backup existing Drupal site
- [ ] Verify PHP version compatibility
- [ ] Ensure database backup exists
- [ ] Review current security configuration
- [ ] Plan maintenance window (30-45 minutes)

## Installation Methods

### Method 1: New Drupal Installation

```bash
# Create new Drupal project
composer create-project drupal/recommended-project:^11.0 secure-site
cd secure-site

# Install Drupal
drush site:install standard \
  --db-url=pgsql://user:pass@localhost/drupal \
  --site-name="Secure Government Site" \
  --account-name=admin \
  --account-pass=temporary_password

# Apply security recipe
drush recipe /path/to/secure_drupal

# Change admin password immediately
drush user:password admin "new_secure_password"
```

### Method 2: Existing Drupal Site

```bash
# Navigate to Drupal root
cd /var/www/drupal

# Put site in maintenance mode
drush state:set system.maintenance_mode 1

# Clear caches
drush cr

# Apply security recipe
drush recipe /path/to/secure_drupal

# Take site out of maintenance mode
drush state:set system.maintenance_mode 0
```

### Method 3: Docker Deployment

```bash
# Clone recipe repository
git clone https://gitlab.bluefly.io/llm/drupal-recipes/secure_drupal.git
cd secure_drupal

# Build and start containers
docker-compose up -d

# Install Drupal in container
docker-compose exec drupal drush site:install

# Apply recipe
docker-compose exec drupal drush recipe /var/www/html/recipes/secure_drupal
```

### Method 4: Automated CI/CD

```yaml
# .gitlab-ci.yml or .github/workflows/deploy.yml
deploy:
  script:
    - drush state:set system.maintenance_mode 1
    - drush recipe $RECIPE_PATH/secure_drupal
    - drush secure-drupal:deploy $COMPLIANCE_LEVEL
    - drush secure-drupal:audit
    - drush state:set system.maintenance_mode 0
```

## Production Deployment

### Step 1: Pre-deployment Preparation

```bash
# Create deployment checklist
cat > deployment-checklist.txt << EOF
[ ] Database backup completed
[ ] File system backup completed
[ ] Configuration export completed
[ ] Maintenance window scheduled
[ ] Team notified
[ ] Rollback plan prepared
EOF

# Export current configuration
drush config:export -y

# Create full site backup
drush sql:dump > backup-$(date +%Y%m%d-%H%M%S).sql
tar -czf files-backup-$(date +%Y%m%d-%H%M%S).tar.gz sites/default/files
```

### Step 2: Deploy Security Recipe

```bash
# Set maintenance mode
drush state:set system.maintenance_mode 1 \
  --maintenance_mode_message="Security updates in progress. Site will be available shortly."

# Apply recipe with specific compliance level
drush recipe /path/to/secure_drupal
drush secure-drupal:deploy fedramp

# Install all security modules
drush secure-drupal:install-modules

# Run initial security audit
drush secure-drupal:audit
```

### Step 3: Configure Security Settings

```bash
# Configure two-factor authentication
drush config:set tfa.settings enabled 1
drush config:set tfa.settings required_roles.administrator administrator

# Set password policy
drush config:set password_policy.password_policy.government_grade password_length 14

# Configure session timeout
drush config:set autologout.settings timeout 900

# Enable HTTPS redirect
drush config:set seckit.settings seckit_ssl.hsts.checkbox 1
```

### Step 4: Post-deployment Validation

```bash
# Run compliance check
drush secure-drupal:compliance all --format=table

# Test critical functionality
drush secure-drupal:audit --format=json > audit-results.json

# Verify security headers
curl -I https://your-site.gov | grep -E "(Strict-Transport-Security|X-Frame-Options|X-Content-Type-Options)"

# Remove maintenance mode
drush state:set system.maintenance_mode 0
```

## Configuration

### Environment-Specific Settings

#### Development Environment
```php
// settings.local.php
$config['autologout.settings']['timeout'] = 0; // Disable for development
$config['tfa.settings']['enabled'] = 0; // Disable 2FA
$config['system.logging']['error_level'] = 'verbose'; // Show all errors
```

#### Staging Environment
```php
// settings.staging.php
$config['autologout.settings']['timeout'] = 1800; // 30 minutes
$config['shield.settings']['shield_enable'] = 1; // Enable HTTP auth
$config['system.logging']['error_level'] = 'some'; // Show some errors
```

#### Production Environment
```php
// settings.production.php
$config['autologout.settings']['timeout'] = 900; // 15 minutes
$config['system.logging']['error_level'] = 'hide'; // Hide all errors
$config['seckit.settings']['seckit_ssl']['hsts']['preload'] = 1; // Enable HSTS preload
```

### Security Headers Configuration

```nginx
# nginx.conf
location / {
    # Security headers
    add_header X-Content-Type-Options "nosniff" always;
    add_header X-Frame-Options "DENY" always;
    add_header X-XSS-Protection "1; mode=block" always;
    add_header Referrer-Policy "strict-origin-when-cross-origin" always;
    add_header Permissions-Policy "geolocation=(), microphone=(), camera=()" always;
    
    # HSTS
    add_header Strict-Transport-Security "max-age=31536000; includeSubDomains; preload" always;
    
    # CSP
    add_header Content-Security-Policy "default-src 'self'; script-src 'self' 'unsafe-inline'; style-src 'self' 'unsafe-inline'; img-src 'self' data: https:; font-src 'self' data:; connect-src 'self'; frame-ancestors 'none';" always;
}
```

### Database Optimization

```sql
-- Optimize audit log tables
ALTER TABLE admin_audit_trail ADD INDEX idx_created (created);
ALTER TABLE admin_audit_trail ADD INDEX idx_uid (uid);
ALTER TABLE login_tracker ADD INDEX idx_timestamp (timestamp);

-- Partition audit logs by month
ALTER TABLE admin_audit_trail PARTITION BY RANGE (UNIX_TIMESTAMP(created)) (
    PARTITION p_2024_01 VALUES LESS THAN (UNIX_TIMESTAMP('2024-02-01')),
    PARTITION p_2024_02 VALUES LESS THAN (UNIX_TIMESTAMP('2024-03-01')),
    -- Add more partitions as needed
);
```

## Validation & Testing

### Security Validation Tests

```bash
# Test 1: Password Policy
drush user:create testuser --password="weak"
# Should fail with password policy error

# Test 2: Session Timeout
# Login and wait 16 minutes, should be logged out

# Test 3: Failed Login Attempts
# Try 4 failed logins, account should be blocked

# Test 4: Security Headers
curl -s -D- https://your-site.gov | grep -E "Strict-Transport-Security|X-Frame-Options"

# Test 5: Two-Factor Authentication
# Login as admin, should require 2FA setup
```

### Compliance Validation

```bash
# Run full compliance suite
drush secure-drupal:compliance all --format=json > compliance-report.json

# Check specific standard
drush secure-drupal:compliance fedramp
drush secure-drupal:compliance hipaa
drush secure-drupal:compliance soc2
drush secure-drupal:compliance nist
```

### Performance Testing

```bash
# Baseline performance
ab -n 1000 -c 10 https://your-site.gov/

# With security features
ab -n 1000 -c 10 -C "SSESS123=authenticated" https://your-site.gov/

# Monitor resource usage
top -b -n 1 | grep -E "(php|mysql|apache2|nginx)"
```

## Troubleshooting

### Common Issues

#### Issue: Module installation fails
```bash
# Solution: Clear composer cache and retry
composer clear-cache
composer require drupal/security_review
drush pm:install security_review
```

#### Issue: Too many authentication failures
```bash
# Solution: Unblock IP address
drush sql:query "DELETE FROM flood WHERE event = 'user.failed_login_ip'"
drush cr
```

#### Issue: Session timeout too aggressive
```bash
# Solution: Temporarily increase timeout
drush config:set autologout.settings timeout 1800
# Remember to revert for compliance
```

#### Issue: 2FA locked out admin
```bash
# Solution: Temporarily disable 2FA
drush config:set tfa.settings enabled 0
# Fix 2FA setup
drush config:set tfa.settings enabled 1
```

### Debug Commands

```bash
# Check module status
drush pm:list --type=module --status=enabled | grep -E "(security|tfa|autologout|password)"

# View current configuration
drush config:get autologout.settings
drush config:get password_policy.password_policy.government_grade
drush config:get tfa.settings

# Check error logs
drush watchdog:show --severity=error --count=50

# Database queries for debugging
drush sql:query "SELECT * FROM admin_audit_trail ORDER BY created DESC LIMIT 10"
```

## Maintenance

### Daily Tasks
```bash
# Check security status
drush secure-drupal:audit

# Review audit logs
drush sql:query "SELECT * FROM admin_audit_trail WHERE created > DATE_SUB(NOW(), INTERVAL 1 DAY)"

# Check for failed login attempts
drush sql:query "SELECT * FROM flood WHERE event = 'user.failed_login_ip'"
```

### Weekly Tasks
```bash
# Update security modules
composer outdated "drupal/*"
composer update drupal/security_review --with-dependencies

# Run security review
drush security-review

# Generate compliance report
drush secure-drupal:compliance all --format=csv > weekly-compliance-$(date +%Y%m%d).csv
```

### Monthly Tasks
```bash
# Full security audit
npm run build
npm run docker:test
npm run docker:compliance

# Rotate audit logs
drush sql:query "DELETE FROM admin_audit_trail WHERE created < DATE_SUB(NOW(), INTERVAL 7 YEAR)"

# Update all dependencies
composer update --with-dependencies
npm update
```

### Quarterly Tasks
```bash
# Compliance certification
drush secure-drupal:compliance all --format=pdf

# Penetration testing
# Contract with security firm for testing

# Review and update security policies
# Update password requirements if needed
# Review user access permissions
```

## Emergency Procedures

### Security Incident Response
```bash
# 1. Enable emergency mode
drush state:set system.maintenance_mode 1
drush config:set shield.settings shield_enable 1

# 2. Lock down the site
drush sql:query "UPDATE users_field_data SET status = 0 WHERE uid > 1"

# 3. Preserve evidence
drush sql:dump > incident-$(date +%Y%m%d-%H%M%S).sql
tar -czf incident-files-$(date +%Y%m%d-%H%M%S).tar.gz sites/default/files

# 4. Review audit logs
drush sql:query "SELECT * FROM admin_audit_trail WHERE created > DATE_SUB(NOW(), INTERVAL 24 HOUR)" > incident-audit.log

# 5. Reset all passwords
drush sql:query "UPDATE users_field_data SET pass = NULL WHERE uid > 0"
```

### Rollback Procedure
```bash
# 1. Restore database
drush sql:drop -y
drush sql:cli < backup-YYYYMMDD-HHMMSS.sql

# 2. Restore files
rm -rf sites/default/files/*
tar -xzf files-backup-YYYYMMDD-HHMMSS.tar.gz

# 3. Clear caches
drush cr

# 4. Verify functionality
drush status
```

## Support & Resources

### Documentation
- [Drupal Security Guide](https://www.drupal.org/security)
- [Government Drupal Community](https://groups.drupal.org/government)
- [FedRAMP Resources](https://www.fedramp.gov/)
- [NIST Cybersecurity Framework](https://www.nist.gov/cyberframework)

### Getting Help
- Security issues: security@drupal.org
- Recipe issues: [GitLab Issues](https://gitlab.bluefly.io/llm/drupal-recipes/secure_drupal/issues)
- Community support: [Drupal Slack #security](https://drupal.slack.com/)