#!/usr/bin/env node

/**
 * Secure Drupal Recipe
 * Government-grade security for Drupal 11 sites
 * 
 * This recipe provides:
 * - FedRAMP compliance
 * - HIPAA compliance
 * - SOC2 compliance
 * - NIST 800-53 compliance
 * - FISMA compliance
 */

const path = require('path');
const { execSync } = require('child_process');

class SecureDrupalRecipe {
  constructor() {
    this.name = 'Secure Drupal';
    this.version = '1.0.0';
    this.description = 'Government-grade security recipe for Drupal 11';
    this.recipePath = __dirname;
  }

  /**
   * Check if running in a Drupal context
   */
  isDrupalEnvironment() {
    try {
      execSync('drush status', { stdio: 'ignore' });
      return true;
    } catch (error) {
      return false;
    }
  }

  /**
   * Apply the recipe to a Drupal site
   */
  apply() {
    console.log(`🔒 Applying ${this.name} Recipe v${this.version}`);
    console.log('=' .repeat(50));

    if (!this.isDrupalEnvironment()) {
      console.error('❌ Error: This recipe must be run from a Drupal site root directory');
      console.error('   Make sure Drush is installed and you are in a Drupal project');
      process.exit(1);
    }

    try {
      console.log('📦 Installing recipe...');
      execSync(`drush recipe ${this.recipePath}`, { stdio: 'inherit' });
      
      console.log('\n✅ Recipe applied successfully!');
      console.log('\n📋 Next steps:');
      console.log('   1. Run security audit: drush security-review');
      console.log('   2. Configure 2FA for admin users');
      console.log('   3. Review security headers in browser developer tools');
      console.log('   4. Test auto-logout functionality (15 minutes)');
      
    } catch (error) {
      console.error('❌ Error applying recipe:', error.message);
      process.exit(1);
    }
  }

  /**
   * Build documentation and reports
   */
  build() {
    console.log(`🏗️  Building ${this.name} Recipe Documentation`);
    console.log('=' .repeat(50));

    try {
      execSync('npm run build', { cwd: this.recipePath, stdio: 'inherit' });
    } catch (error) {
      console.error('❌ Error building documentation:', error.message);
      process.exit(1);
    }
  }

  /**
   * Display recipe information
   */
  info() {
    console.log(`
🔒 ${this.name} Recipe v${this.version}
${this.description}

📋 Features:
- 25+ security modules pre-configured
- Government-grade password policies
- Two-factor authentication (2FA)
- Session timeout (15 minutes)
- Comprehensive audit logging
- HTTPS enforcement
- Security headers (HSTS, CSP, X-Frame-Options)
- Brute force protection
- Field-level encryption

🏛️ Compliance:
- FedRAMP Ready
- HIPAA Compliant
- SOC2 Type II
- NIST 800-53
- FISMA

📦 Commands:
- npm run build     : Generate documentation and reports
- npm run audit     : Run security audit
- npm run compliance: Validate compliance
- npm run configure : Configure modules
- npm run document  : Generate documentation
- npm run clean     : Clean generated files

🚀 Quick Start:
1. cd /path/to/drupal/site
2. drush recipe ${this.recipePath}
3. drush security-review
    `);
  }
}

// CLI handling
if (require.main === module) {
  const recipe = new SecureDrupalRecipe();
  const command = process.argv[2];

  switch (command) {
    case 'apply':
      recipe.apply();
      break;
    case 'build':
      recipe.build();
      break;
    case 'info':
    default:
      recipe.info();
      break;
  }
}

module.exports = SecureDrupalRecipe;