/**
 * Security Audit Tests
 * Comprehensive test suite for secure_drupal recipe
 */

const { execSync } = require('child_process');
const fs = require('fs');
const path = require('path');
const yaml = require('js-yaml');

describe('Secure Drupal Recipe Tests', () => {
  const recipePath = path.join(__dirname, '..');
  const recipeFile = path.join(recipePath, 'recipe.yml');
  let recipe;

  beforeAll(() => {
    // Load recipe
    const recipeContent = fs.readFileSync(recipeFile, 'utf8');
    recipe = yaml.load(recipeContent);
  });

  describe('Recipe Structure', () => {
    test('should have valid recipe.yml', () => {
      expect(recipe).toBeDefined();
      expect(recipe.name).toBe('Secure Drupal');
      expect(recipe.type).toBe('Site');
    });

    test('should include all required security modules', () => {
      const requiredModules = [
        'security_review',
        'password_policy',
        'autologout',
        'tfa',
        'login_security',
        'seckit'
      ];

      requiredModules.forEach(module => {
        expect(recipe.install).toContain(module);
      });
    });

    test('should have government-grade configurations', () => {
      const config = recipe.config.import;
      
      // Password policy checks
      expect(config['password_policy.password_policy.government_grade'].password_length).toBeGreaterThanOrEqual(14);
      expect(config['password_policy.password_policy.government_grade'].password_character_types).toBe(4);
      
      // Session timeout check
      expect(config['autologout.settings'].timeout).toBeLessThanOrEqual(900); // 15 minutes
      
      // 2FA enabled
      expect(config['tfa.settings'].enabled).toBe(true);
    });
  });

  describe('Security Module Configuration', () => {
    test('should configure HSTS properly', () => {
      const seckit = recipe.config.import['seckit.settings'];
      expect(seckit.seckit_ssl.hsts.checkbox).toBe(true);
      expect(seckit.seckit_ssl.hsts.max_age).toBeGreaterThanOrEqual(31536000);
      expect(seckit.seckit_ssl.hsts.preload).toBe(true);
    });

    test('should configure CSP headers', () => {
      const csp = recipe.config.import['csp.settings'];
      expect(csp.enforce).toBe(true);
      expect(csp.policy['frame-ancestors']).toBe("'none'");
      expect(csp.policy['default-src']).toContain("'self'");
    });

    test('should configure encryption', () => {
      const encrypt = recipe.config.import['encrypt.settings'];
      expect(encrypt.default_encryption_profile).toBe('government_aes');
      
      const key = recipe.config.import['key.key.government_aes'];
      expect(key.key_type).toBe('encryption');
      expect(key.key_size).toBe(256);
    });
  });

  describe('Compliance Validation', () => {
    test('should meet FedRAMP requirements', () => {
      const config = recipe.config.import;
      
      // All FedRAMP checks
      expect(config['password_policy.password_policy.government_grade']).toBeDefined();
      expect(config['autologout.settings'].timeout).toBe(900);
      expect(config['tfa.settings'].enabled).toBe(true);
      expect(config['audit_log.settings'].log_user_operations).toBe(true);
      expect(config['encrypt.settings'].default_encryption_profile).toBeDefined();
    });

    test('should meet HIPAA requirements', () => {
      const config = recipe.config.import;
      
      // Access controls
      expect(recipe.install).toContain('field_permissions');
      
      // Transmission security
      expect(config['seckit.settings'].seckit_ssl.hsts.checkbox).toBe(true);
      
      // Audit controls
      expect(config['audit_log.settings']).toBeDefined();
    });

    test('should meet NIST 800-53 requirements', () => {
      const config = recipe.config.import;
      
      // Access control
      expect(recipe.install).toContain('content_access');
      
      // Identification and authentication
      expect(config['tfa.settings']).toBeDefined();
      
      // System and communications protection
      expect(config['csp.settings']).toBeDefined();
    });
  });

  describe('Worker Scripts', () => {
    test('should have all worker files', () => {
      const workers = [
        'worker-01-security-audit.js',
        'worker-02-compliance-validation.js',
        'worker-03-module-configuration.js',
        'worker-04-documentation.js',
        'run-workers.js'
      ];

      workers.forEach(worker => {
        const workerPath = path.join(recipePath, 'workers', worker);
        expect(fs.existsSync(workerPath)).toBe(true);
      });
    });

    test('should generate documentation on build', (done) => {
      // Clean first
      execSync('npm run clean', { cwd: recipePath });
      
      // Run build
      execSync('npm run build', { cwd: recipePath });
      
      // Check generated files
      const generatedFiles = [
        'security-configuration-report.json',
        'compliance-summary.json',
        'IMPLEMENTATION_GUIDE.md'
      ];

      generatedFiles.forEach(file => {
        const filePath = path.join(recipePath, file);
        expect(fs.existsSync(filePath)).toBe(true);
      });

      done();
    }, 30000); // 30 second timeout
  });

  describe('Security Best Practices', () => {
    test('should disable user registration', () => {
      const userSettings = recipe.config.import['user.settings'];
      expect(userSettings.register).toBe('admin_only');
    });

    test('should hide error messages', () => {
      const logging = recipe.config.import['system.logging'];
      expect(logging.error_level).toBe('hide');
    });

    test('should configure flood control', () => {
      const flood = recipe.config.import['flood_control.settings'];
      expect(flood.user_limit).toBeLessThanOrEqual(3);
      expect(flood.ip_limit).toBeLessThanOrEqual(10);
    });

    test('should prevent username enumeration', () => {
      const enumPrevention = recipe.config.import['username_enumeration_prevention.settings'];
      expect(enumPrevention.login_form).toBe(true);
      expect(enumPrevention.reset_form).toBe(true);
    });
  });

  describe('Integration Tests', () => {
    test('should have valid Drush commands', () => {
      const drushFile = path.join(recipePath, 'src/Commands/SecureDrupalCommands.php');
      expect(fs.existsSync(drushFile)).toBe(true);
      
      const drushContent = fs.readFileSync(drushFile, 'utf8');
      expect(drushContent).toContain('secure-drupal:deploy');
      expect(drushContent).toContain('secure-drupal:audit');
      expect(drushContent).toContain('secure-drupal:compliance');
    });

    test('should have drush services configuration', () => {
      const servicesFile = path.join(recipePath, 'drush.services.yml');
      expect(fs.existsSync(servicesFile)).toBe(true);
      
      const servicesContent = fs.readFileSync(servicesFile, 'utf8');
      const services = yaml.load(servicesContent);
      expect(services.services['secure_drupal.commands']).toBeDefined();
    });
  });
});