# secure_project Adapter Plugins

This directory contains adapter plugins for various web frameworks and platforms. These adapters allow secure_project to be integrated with different frameworks while providing a consistent security API.

## Available Adapters

| Adapter | Framework | Description |
|---------|-----------|-------------|
| ExpressAdapter | Express.js | Integration with Express.js web framework |
| KoaAdapter | Koa.js | Integration with Koa.js web framework |
| DrupalAdapter | Drupal CMS | Integration with Drupal CMS via bridge |
| FastifyAdapter | Fastify | Integration with Fastify web framework |
| NestJSAdapter | NestJS | Integration with NestJS framework |

## Using Adapters

Adapters can be used in two ways:

### 1. Direct Usage (Backward Compatible)

```typescript
import [createExpressAdapter] from 'secure_project';
import express from 'express';

const app = express();
const middleware = createExpressAdapter();
app.use(middleware);
```

### 2. Plugin System (Recommended)

```typescript
import { 
  AdapterManager, 
  ExpressAdapter, 
  AdapterConfig, 
  createDefaultConfig 
} from 'secure_project';
import express from 'express';

// Create adapter manager and config
const adapterManager = new AdapterManager();
const config = createDefaultConfig();

// Create and register adapter
const expressAdapter = new ExpressAdapter();
await adapterManager.registerAdapter(expressAdapter);

// Configure adapter
config.updateAdapterConfig('express', {
  audit: true,
  accessControl: true,
  securityAlerts: true,
  dataSeparation: true,
  customSetting: 'test-value'
});

// Create Express app
const app = express();

// Add middleware with config
const middleware = expressAdapter.createMiddleware(
  config.getMergedConfig('express')
);

app.use(middleware);
```

### 3. Dynamic Discovery

```typescript
import { 
  AdapterManager, 
  PluginDiscovery, 
  AdapterConfig, 
  createDefaultConfig 
} from 'secure_project';
import express from 'express';

// Create adapter manager and config
const adapterManager = new AdapterManager();
const config = createDefaultConfig();
const discovery = new PluginDiscovery(adapterManager);

// Discover and register adapters
await discovery.discoverAndRegister([
  '/path/to/adapters'
], {
  includePackages: true
});

// Create Express app
const app = express();

// Add middleware with config
const middleware = adapterManager.createMiddleware('express', 
  config.getMergedConfig('express')
);

app.use(middleware);
```

## Creating Custom Adapters

To create a custom adapter, implement the `AdapterPlugin` interface:

```typescript
import { AdapterPlugin, AdapterContext } from 'secure_project';

export class MyCustomAdapter implements AdapterPlugin {
  id = 'my-custom';
  name = 'My Custom Adapter';
  version = '1.0.0';
  description = 'Custom adapter for my framework';
  framework = 'my-framework';
  
  async initialize(): Promise<void> {
    // Initialize services
  }
  
  createMiddleware(options: any = {}, context?: AdapterContext): any {
    // Create middleware for your framework
    return function(req, res, next) {
      // Implement middleware logic
      next();
    };
  }
}
```

## Adapter Configuration

Each adapter can be configured with both global and adapter-specific options:

```typescript
// Global configuration (applies to all adapters)
config.updateGlobalConfig({
  audit: true,
  accessControl: true,
  securityAlerts: true,
  dataSeparation: true
});

// Adapter-specific configuration (overrides global settings)
config.updateAdapterConfig('express', {
  audit: false, // Override global setting
  customSetting: 'value' // Adapter-specific setting
});
```

## Environment Variables

Adapters can also be configured using environment variables:

```bash
# Global configuration
export SECURE_PROJECT_AUDIT=false
export SECURE_PROJECT_ACCESS_CONTROL=true

# Adapter-specific configuration
export SECURE_PROJECT_ADAPTER_EXPRESS_AUDIT=true
export SECURE_PROJECT_ADAPTER_EXPRESS_CUSTOM_SETTING=value
```

To enable environment variables:

```typescript
const config = createDefaultConfig({
  useEnvironmentVariables: true,
  environmentPrefix: 'SECURE_PROJECT_'
});
```

## Configuration File

Configurations can be loaded from and saved to JSON files:

```typescript
// Load configuration
await config.loadFromFile('secure-project-config.json');

// Save configuration
await config.saveToFile('secure-project-config.json');
```

## Adding New Adapters

To add a new adapter to this directory:

1. Create a new file named `[Framework]Adapter.ts`
2. Implement the `AdapterPlugin` interface
3. Export the adapter class and a factory function
4. Add the adapter to `index.ts` exports

Example:

```typescript
// SvelteKitAdapter.ts
export class SvelteKitAdapter implements AdapterPlugin {
  id = 'sveltekit';
  name = 'SvelteKit';
  version = '1.0.0';
  description = 'SvelteKit adapter for secure_project';
  framework = 'sveltekit';
  
  // Implementation...
}

export function createSvelteKitAdapter(options?: any): any {
  const adapter = new SvelteKitAdapter();
  return adapter.createMiddleware(options);
}
```

Then add to `index.ts`:

```typescript
export { SvelteKitAdapter, createSvelteKitAdapter } from './SvelteKitAdapter';
```