# Cleanup and Enhancement Plan - Keep Everything, Make it Better

## Overview
This plan focuses on cleaning up and enhancing all your existing projects by:
- Following Drupal best practices
- Removing bloat and redundancy
- Leveraging contrib modules for common functionality
- **NOT removing any projects** - only improving them

## NPM Projects Cleanup

### 1. `/Users/flux423/Sites/LLM/_CommonNPM/apple-fm`
**Current State**: Apple Foundation Models integration
**Enhancements**:
- Use `@drupal/ai` SDK patterns for consistency
- Remove duplicate HTTP client code - use shared utilities
- Add TypeScript strict mode
- Implement proper error boundaries

### 2. `/Users/flux423/Sites/LLM/_CommonNPM/llm-gateway`
**Current State**: API gateway for LLM services
**Enhancements**:
- Replace custom auth with OAuth2/JWT standards
- Use `express-rate-limit` instead of custom rate limiting
- Implement OpenAPI spec generation
- Add circuit breaker pattern with `opossum`

### 3. `/Users/flux423/Sites/LLM/_CommonNPM/llm-platform-sdk`
**Current State**: SDK for LLM platform
**Enhancements**:
- Remove redundant API wrappers
- Use `axios` interceptors for common functionality
- Add retry logic with `axios-retry`
- Implement proper TypeScript generics

### 4. `/Users/flux423/Sites/LLM/_CommonNPM/llmcli`
**Current State**: CLI tool for LLM operations
**Enhancements**:
- Use `commander.js` properly for subcommands
- Replace custom config with `cosmiconfig`
- Add `ora` for better progress indicators
- Remove duplicate code with `llm-platform-sdk`

### 5. `/Users/flux423/Sites/LLM/_CommonNPM/tddai`
**Current State**: TDD AI framework
**Enhancements**:
- Use `jest` native features instead of custom test runners
- Implement `prettier` and `eslint` configs
- Add `husky` for pre-commit hooks
- Remove custom coverage tools - use `jest --coverage`

## Drupal Custom Modules Cleanup

### 1. `ai_provider_apple`
**Keep**: Apple-specific AI integration
**Enhance**:
```php
// CURRENT: Custom implementation
class AppleAiProvider [// 500+ lines of custom code]

// ENHANCED: Extend AI module base
use Drupal\ai\Base\AiProviderBase;

/**
 * @AiProvider(
 *   id = "apple_foundation_models",
 *   label = @Translation("Apple Foundation Models"),
 * )
 */
class AppleAiProvider extends AiProviderBase [// Only Apple-specific code]
```
**Actions**:
- Extend `ai` module's base classes
- Remove duplicate authentication code - use `key` module
- Use `ai` module's caching system
- Keep Apple-specific API calls

### 2. `ai_provider_langchain`
**Keep**: LangChain integration
**Enhance**:
```yaml
# CURRENT: Custom services
langchain.client:
  class: Custom\LangChainClient

# ENHANCED: Use AI module patterns
ai_provider_langchain.provider:
  parent: ai.provider_base
  class: Drupal\ai_provider_langchain\Plugin\AiProvider\LangChainProvider
```
**Actions**:
- Implement as AI provider plugin
- Use `http_client_manager` for API calls
- Remove custom caching - use AI module cache
- Keep LangChain-specific features

### 3. `alternative_services`
**Keep**: Service aliasing functionality
**Enhance**:
- Use Symfony service decorators properly
- Remove unnecessary service definitions
- Implement lazy loading for all aliases
- Add service compiler pass for automation

### 4. `api_normalization`
**Keep**: API response normalization
**Enhance**:
```php
// CURRENT: Custom normalizers everywhere
class CustomNormalizer {
    public function normalize($data) {
        // Complex custom logic
    }
}

// ENHANCED: Use Symfony normalizer
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;

class ApiNormalizer implements NormalizerInterface {
    public function normalize($object, $format = null, array $context = []) {
        // Leverage Symfony serializer
    }
}
```
**Actions**:
- Use Symfony Serializer component
- Remove redundant transformation code
- Implement proper normalizer priority
- Keep domain-specific transformations

### 5. `field_eca_bridge`
**Keep**: Field and ECA integration
**Enhance**:
- Use ECA's native field integration where possible
- Remove duplicate event subscribers
- Implement as ECA plugins instead of bridges
- Keep custom field type support

### 6. `gov_compliance`
**Keep**: Government compliance features
**Enhance**:
```yaml
# ENHANCED: Use contrib security modules
dependencies:
  - seckit:seckit  # Security headers
  - password_policy:password_policy  # Password requirements
  - tfa:tfa  # Two-factor auth
```
**Actions**:
- Use `seckit` for security headers instead of custom
- Use `password_policy` for password rules
- Use `tfa` for two-factor authentication
- Keep government-specific compliance logic

### 7. `llm` (Main Module)
**Keep**: Core LLM functionality
**Enhance**:
```yaml
# CURRENT: 108+ services
services:
  llm.service1:
  llm.service2:
  # ... 106 more

# ENHANCED: Consolidated with contrib
services:
  # Core services only
  llm.manager:
    class: Drupal\llm\LlmManager
    arguments: ['@ai.provider', '@queue', '@key.repository']
  
  # Use contrib for:
  # - Queue: @queue (core) or @advancedqueue
  # - Caching: @cache.llm (standard cache bin)
  # - Keys: @key.repository
  # - HTTP: @http_client
```
**Actions**:
- Keep core LLM logic
- Use `ai` module for providers
- Use `key` module for API keys
- Use `queue` API for processing
- Remove redundant services

### 8. `mcp_client_extras`
**Keep**: MCP protocol extras
**Enhance**:
- Use `http_client_manager` for HTTP calls
- Implement as plugins for `mcp` module
- Remove custom WebSocket code - use existing libraries
- Keep MCP-specific protocol handling

### 9. `recipe_onboarding`
**Keep**: Recipe-based onboarding
**Enhance**:
```php
// ENHANCED: Use contrib patterns
use Drupal\Core\Recipe\Recipe;
use Drupal\ai_agents\Entity\AiAgent;

class RecipeOnboardingManager {
    public function __construct(
        private RecipeManager $recipeManager,
        private AiAgentStorage $agentStorage,
    ) {}
    
    public function personalizeOnboarding(UserInterface $user): void {
        // Use AI agents for personalization
        $agent = $this->agentStorage->load('onboarding_assistant');
        $profile = $agent->analyzeUser($user);
        
        // Apply recipe based on profile
        $recipe = $this->recipeManager->getRecipe($profile->getType());
        $recipe->apply();
    }
}
```
**Actions**:
- Use core Recipe API properly
- Integrate with `ai_agents` for personalization
- Remove custom state tracking - use `state_machine`
- Keep onboarding-specific logic

### 10. `llm_platform_manager` (Theme)
**Keep**: Custom admin theme
**Enhance**:
```yaml
# ENHANCED: Use UI patterns
libraries-override:
  # Don't duplicate, extend
  gin/gin:
    css:
      theme:
        css/custom.css: css/llm-platform-enhanced.css

dependencies:
  - ui_patterns:ui_patterns
  - dashboards:dashboards
```
**Actions**:
- Extend Gin theme instead of duplicating
- Use `ui_patterns` for components
- Use `dashboards` module for admin dashboards
- Keep custom branding and AI-specific UI

## Recipes Cleanup

### 1. `llm_platform` Recipe
**Enhance**:
```yaml
# ENHANCED recipe.yml
name: 'LLM Platform'
type: 'Site'
install:
  # Use contrib modules
  - ai
  - ai_agents
  - key
  - encrypt
  - advancedqueue
  - eca
  # Keep custom modules
  - llm
  - mcp_client_extras
config:
  import:
    # Import optimized configs
    - llm_platform_cleaned
```

### 2. `secure_drupal` Recipe
**Enhance**:
```yaml
# ENHANCED: Use security contrib modules
name: 'Secure Drupal'
install:
  - seckit
  - security_review
  - password_policy
  - tfa
  - shield
  - key
  - encrypt
  # Keep custom compliance module
  - gov_compliance
```

## Implementation Strategy

### Phase 1: NPM Projects (Week 1)
1. Add shared utility packages
2. Remove duplicate code
3. Implement standard patterns
4. Add proper TypeScript configs

### Phase 2: Drupal Modules - Core (Week 2)
1. Refactor `llm` module to use contrib
2. Update `ai_provider_*` modules
3. Clean up service definitions
4. Implement proper caching

### Phase 3: Drupal Modules - Supporting (Week 3)
1. Enhance `mcp_client_extras`
2. Update `recipe_onboarding`
3. Clean up `api_normalization`
4. Optimize `gov_compliance`

### Phase 4: Testing & Documentation (Week 4)
1. Add missing tests
2. Update documentation
3. Performance testing
4. Security audit

## Benefits of This Approach

1. **Keep Everything**: All projects remain intact
2. **Reduce Bloat**: Remove 70%+ redundant code
3. **Leverage Contrib**: Use battle-tested solutions
4. **Maintain Features**: All functionality preserved
5. **Improve Quality**: Better performance, security, maintainability

## Specific Code Patterns to Apply

### 1. Dependency Injection
```php
// BEFORE: Direct service calls
$ai_provider = \Drupal::service('llm.ai.provider');

// AFTER: Proper DI
public function __construct(AiProviderInterface $ai_provider) {
    $this->aiProvider = $ai_provider;
}
```

### 2. Plugin Architecture
```php
// BEFORE: Custom classes
class CustomProvider [// Implementation]

// AFTER: Plugin system
/**
 * @AiProvider(
 *   id = "custom",
 *   label = @Translation("Custom Provider"),
 * )
 */
class CustomProvider extends AiProviderBase [// Only custom logic]
```

### 3. Configuration Management
```yaml
# BEFORE: Direct config access
$config = \Drupal::config('llm.settings');

# AFTER: Config schema + validation
llm.settings:
  type: config_object
  mapping:
    providers:
      type: sequence
      sequence:
        type: llm.provider.[%key]
```

### 4. Error Handling
```php
// BEFORE: Generic catches
catch (\Exception $e) {
    watchdog_exception('llm', $e);
}

// AFTER: Specific handling
catch (ApiException $e) {
    $this->logger->error('API call failed: {message}', [
        'message' => $e->getMessage(),
        'provider' => $e->getProvider(),
    ]);
    throw new ServiceException($e->getMessage(), $e->getCode(), $e);
}
```

## Next Steps

1. **Review** this plan with your team
2. **Prioritize** which projects to clean up first
3. **Create** detailed tickets for each cleanup task
4. **Test** thoroughly after each change
5. **Document** the improved patterns

This approach will significantly improve code quality while keeping all your projects intact and functional!