# CLI Validation Commands

This document provides instructions for using the ReelCode CLI validation commands to check your code against defined standards.

## Overview

ReelCode provides a powerful validation system to ensure your code adheres to defined standards. The validation commands allow you to:

1. Validate individual files
2. Validate entire directories
3. Validate complete projects with automatic standards detection

## Prerequisites

Before using the validation commands, you should have:

1. A standards file (YAML or JSON) defining your coding standards
2. Code files to validate (TypeScript, JavaScript, or PHP)

## Command Structure

The validation commands follow this structure:

```bash
tddai validate [command] [options]
```

Available commands:

- `file`: Validate a specific file
- `directory` (or `dir`): Validate all files in a directory
- `project`: Validate a project with automatic standards detection

## Validating a File

To validate a single file against standards:

```bash
tddai validate file path/to/file.ts --standards standards.yaml
```

Options:

- `-s, --standards <path>`: Path to the standards file (default: `./standards.yaml`)
- `-p, --project <id>`: Project ID to use for project-specific standards
- `--include-validators <validators>`: Include only specific validators (comma-separated)
- `--exclude-validators <validators>`: Exclude specific validators (comma-separated)
- `-j, --json`: Output results in JSON format
- `-q, --quiet`: Suppress output except for errors
- `-v, --verbose`: Show verbose output

Example:

```bash
# Validate a TypeScript file
tddai validate file src/components/UserProfile.ts --standards .tddai/standards.yaml

# Validate with verbose output
tddai validate file src/components/UserProfile.ts --verbose

# Validate with specific validators
tddai validate file src/components/UserProfile.ts --include-validators typescript
```

## Validating a Directory

To validate all files in a directory:

```bash
tddai validate directory src/ --standards standards.yaml
```

Options:

- All options from the `file` command, plus:
- `--include <patterns>`: Include only specific file patterns (comma-separated, default: `**/*.ts,**/*.js,**/*.php`)
- `--exclude <patterns>`: Exclude specific file patterns (comma-separated, default: `**/node_modules/**,**/vendor/**,**/dist/**`)
- `--max-files <count>`: Maximum number of files to validate (default: `1000`)
- `--fail-fast`: Stop after first error (default: `false`)

Example:

```bash
# Validate all TypeScript files in src/ directory
tddai validate directory src/ --include "**/*.ts"

# Validate with custom exclude patterns
tddai validate directory src/ --exclude "**/node_modules/**,**/tests/**"

# Validate with a limit on files
tddai validate directory src/ --max-files 100
```

## Validating a Project

The project command automatically detects standards files and validates the entire project:

```bash
tddai validate project [project-path]
```

Options:

- All options from the `directory` command
- Project path defaults to current directory if not specified

Example:

```bash
# Validate current project
tddai validate project

# Validate a specific project
tddai validate project /path/to/project

# Validate with project-specific ID
tddai validate project --project my-project-id
```

The project command searches for standards files in common locations:
- `./standards.yaml` (or specified path)
- `./.tddai/standards.yaml|yml|json`
- `./.cursor/standards.yaml|yml|json`

## Output Formats

### Default Output

By default, the validation commands output results in a human-readable format:

```
 src/components/UserProfile.ts - Valid
❌ src/components/UserForm.ts - Invalid (2 errors, 1 warnings)
  Error: src/components/UserForm.ts:15:3 - Variable "UserData" does not follow camelCase convention (javascript:naming-convention)
  Error: src/components/UserForm.ts:22:5 - Missing return type annotation (typescript:strict-mode)

Validation Summary:
  Total files: 2
  Valid files: 1
  Invalid files: 1
  Total errors: 2
  Total warnings: 1
```

### JSON Output

For integration with other tools, use the `--json` flag to output results in JSON format:

```json
{
  "results": [
    {
      "filePath": "/path/to/src/components/UserProfile.ts",
      "valid": true,
      "errors": [],
      "warnings": []
    },
    {
      "filePath": "/path/to/src/components/UserForm.ts",
      "valid": false,
      "errors": [
        {
          "rule": "javascript:naming-convention",
          "message": "Variable \"UserData\" does not follow camelCase convention",
          "filePath": "/path/to/src/components/UserForm.ts",
          "line": 15,
          "column": 3
        }
      ],
      "warnings": []
    }
  ],
  "summary": {
    "totalFiles": 2,
    "validFiles": 1,
    "invalidFiles": 1,
    "totalErrors": 1,
    "totalWarnings": 0
  }
}
```

## Exit Codes

The validation commands use exit codes to indicate success or failure:

- `0`: All files passed validation
- `1`: At least one file failed validation
- `1`: An error occurred during validation

This allows for easy integration with CI/CD pipelines and build systems.

## Integrating with CI/CD

The validation commands can be easily integrated into your CI/CD pipelines:

### GitLab CI

```yaml
validate:
  stage: test
  script:
    - npm install -g @bluefly/tddai
    - tddai validate project --standards .tddai/standards.yaml
```

### GitHub Actions

```yaml
jobs:
  validate:
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v2
      - uses: actions/setup-node@v2
      - run: npm install -g @bluefly/tddai
      - run: tddai validate project
```

## Troubleshooting

If you encounter issues with validation:

1. Use the `--verbose` flag to get more detailed output
2. Verify that your standards file is valid with `tddai validate-standards`
3. Check that the appropriate validators are registered for your file types
4. Ensure the standards file contains rules for the languages you're validating
5. Use `--include-validators` to isolate specific validators for testing