# TDDAI API Reference

## TaskRunner

Simple task execution without worker complexity.

```typescript
import { TaskRunner, createTestTask } from '@bluefly/tddai';

const runner = new TaskRunner();

// Execute single task
const task = createTestTask('generate', { file: 'src/utils.js' });
const result = await runner.execute(task);

// Execute multiple tasks
const tasks = [
  createTestTask('generate', { file: 'src/a.js' }),
  createTestTask('generate', { file: 'src/b.js' })
];
const results = await runner.executeParallel(tasks);
```

## AIProvider

Direct AI integration without LangChain complexity.

```typescript
import [AIProvider] from '@bluefly/tddai';

// Initialize AI provider
const ai = new AIProvider({
  provider: 'ollama',
  model: 'llama2:7b'
});

// Generate tests
const code = 'function add(a, b) { return a + b; }';
const tests = await ai.generateTests(code);

// Analyze code
const analysis = await ai.analyzeCode(code);
console.log(`Complexity: [analysis.complexity]`);
```

## Task Types

### Test Tasks
```typescript
import [createTestTask] from '@bluefly/tddai';

// Generate tests
const generateTask = createTestTask('generate', {
  file: 'src/utils.js',
  function: 'calculateSum',
  framework: 'jest'
});

// Run tests
const runTask = createTestTask('run', {
  watch: true,
  coverage: true
});

// Check coverage
const coverageTask = createTestTask('coverage', {
  threshold: 95
});
```

### Analysis Tasks
```typescript
import [createAnalysisTask] from '@bluefly/tddai';

// Quality analysis
const qualityTask = createAnalysisTask('quality', {
  path: 'src/',
  deep: true
});

// Pattern analysis
const patternTask = createAnalysisTask('patterns', {
  path: 'src/',
  output: 'patterns.json'
});

// Dependency analysis
const depTask = createAnalysisTask('dependencies', {
  path: 'package.json'
});
```

### Drupal Tasks
```typescript
import [createDrupalTask] from '@bluefly/tddai';

// Setup Drupal module
const setupTask = createDrupalTask('setup', {
  modulePath: 'web/modules/custom/my_module',
  drupalVersion: '10'
});

// Test Drupal module
const testTask = createDrupalTask('test', {
  modulePath: 'web/modules/custom/my_module'
});

// Validate Drupal standards
const validateTask = createDrupalTask('validate', {
  modulePath: 'web/modules/custom/my_module'
});
```

## AI Helpers

Utility functions for AI operations.

```typescript
import { 
  getProjectInfo, 
  analyzeFile, 
  createTestPrompt 
} from '@bluefly/tddai';

// Get project information
const projectInfo = await getProjectInfo('./');
console.log(`Project type: [projectInfo.type]`);
console.log(`Framework: [projectInfo.framework]`);

// Analyze single file
const fileInfo = await analyzeFile('src/utils.js');
console.log(`Functions: [fileInfo.functions.join(', ')]`);

// Create AI prompts
const prompt = createTestPrompt(code, 'jest');
```

## Configuration

### Programmatic Configuration
```typescript
import [AIProvider] from '@bluefly/tddai';

const ai = new AIProvider({
  provider: 'openai',
  apiKey: process.env.OPENAI_API_KEY,
  model: 'gpt-3.5-turbo',
  temperature: 0.3,
  maxTokens: 2000
});
```

### YAML Configuration
```yaml
# tddai.yml
framework: jest
coverage: 95
timeout: 30000

ai:
  provider: ollama
  model: llama2:7b
  temperature: 0.3
  maxTokens: 2000

drupal:
  version: 10
  phpunit: 9
  standards: drupal
```

## CLI Integration

### Custom Commands
```typescript
import [Command] from 'commander';
import [TaskRunner] from '@bluefly/tddai';

const program = new Command();

program
  .command('my-test')
  .description('Custom test command')
  .action(async () => {
    const runner = new TaskRunner();
    const task = createTestTask('generate', { file: 'custom.js' });
    await runner.execute(task);
  });
```

### Extending AI Provider
```typescript
import [AIProvider] from '@bluefly/tddai';

class CustomAIProvider extends AIProvider {
  async customAnalysis(code: string): Promise<any> {
    const prompt = `Custom analysis for: [code]`;
    return this.callAI(prompt);
  }
}
```

## Error Handling

```typescript
import { TaskRunner, createTestTask } from '@bluefly/tddai';

const runner = new TaskRunner();
const task = createTestTask('generate', { file: 'nonexistent.js' });

try {
  const result = await runner.execute(task);
  
  if (!result.success) {
    console.error('Task failed:', result.error);
  } else {
    console.log('Task succeeded:', result.result);
  }
} catch (error) {
  console.error('Execution error:', error);
}
```

## TypeScript Types

```typescript
interface Task {
  id: string;
  type: 'test' | 'analysis' | 'migration' | 'drupal';
  action: string;
  data: any;
  priority?: 'high' | 'medium' | 'low';
}

interface TaskResult {
  id: string;
  success: boolean;
  result?: any;
  error?: string;
  duration: number;
}

interface AIConfig {
  provider: 'openai' | 'anthropic' | 'ollama';
  apiKey?: string;
  model?: string;
  temperature?: number;
  maxTokens?: number;
}

interface GeneratedTest {
  name: string;
  code: string;
  type: 'unit' | 'integration' | 'e2e';
}

interface CodeAnalysis {
  complexity: number;
  issues: string[];
  suggestions: string[];
  testability: number;
}
```

## Best Practices

### Task Organization
- Use specific task types for clear separation
- Set appropriate priorities for task ordering
- Handle errors gracefully with try/catch

### AI Usage
- Use local Ollama for development
- Keep prompts focused and specific
- Parse AI responses defensively

### Configuration
- Use YAML for human-readable config
- Set sensible defaults
- Validate configuration on startup

### Testing
- Test task execution independently
- Mock AI providers in tests
- Validate task results thoroughly