# API Reference

This document provides an overview of the AI Coding Standards Tool's programmatic API. For detailed information on specific classes and methods, refer to the class-specific documentation linked below.

## Core Classes

The API consists of the following core classes:

| Class | Description | Documentation |
|-------|-------------|---------------|
| `StandardsAnalyzer` | Analyzes code against defined standards | [StandardsAnalyzer](standards-analyzer.md) |
| `RuleEngine` | Processes and applies rules to code | [RuleEngine](rule-engine.md) |
| `AIPromptValidator` | Validates AI prompts against standards | [AIPromptValidator](prompt-validator.md) |
| `StandardsComplianceReporter` | Generates compliance reports | [StandardsComplianceReporter](compliance-reporter.md) |
| `StandardsConfiguration` | Manages standards configuration | [StandardsConfiguration](standards-configuration.md) |
| `AIAssistantIntegration` | Base class for AI assistant integrations | [AIAssistantIntegration](ai-assistant-integration.md) |

## Basic Usage

### Initializing the Standards Analyzer

```typescript
import [StandardsAnalyzer] from '@bluefly/ai-coding-standards';

// Initialize with default configuration
const analyzer = new StandardsAnalyzer();

// Or with custom configuration
const analyzerWithConfig = new StandardsAnalyzer({
  configPath: './.custom-standards.json',
  includeDefaultRules: true,
  cacheResults: true
});
```

### Analyzing Code

```typescript
// Analyze a single file
const fileResult = await analyzer.analyzeFile('/path/to/file.js');
console.log(`File compliance: [fileResult.compliant ? 'Compliant' : 'Non-compliant']`);
console.log('Violations:', fileResult.violations);

// Analyze a directory
const directoryResult = await analyzer.analyzeDirectory('/path/to/directory');
console.log(`Overall compliance: [directoryResult.compliant ? 'Compliant' : 'Non-compliant']`);
console.log('Files analyzed:', directoryResult.filesAnalyzed);
console.log('Total violations:', directoryResult.totalViolations);
```

### Working with Rules

```typescript
import { RuleEngine, Rule } from '@bluefly/ai-coding-standards';

// Initialize the rule engine
const ruleEngine = new RuleEngine();

// Add a custom rule
const customRule: Rule = {
  id: 'custom-naming-convention',
  name: 'Custom Naming Convention',
  description: 'Enforces custom naming conventions',
  languages: ['javascript', 'typescript'],
  severity: 'error',
  validate: (code, context) => {
    // Rule implementation
    const violations = [];
    // Check code against rule
    return violations;
  }
};

ruleEngine.addRule(customRule);

// Apply rules to code
const result = await ruleEngine.applyRules(code, {
  filePath: '/path/to/file.js',
  language: 'javascript'
});
```

### Validating AI Prompts

```typescript
import [AIPromptValidator] from '@bluefly/ai-coding-standards';

// Initialize the prompt validator
const promptValidator = new AIPromptValidator({
  standardsPath: './.acs-standards.json'
});

// Validate a prompt
const prompt = `
Create a React component that displays a user profile.
The component should accept a user object as a prop and render the user's name, email, and avatar.
`;

const validationResult = await promptValidator.validatePrompt(prompt, {
  assistant: 'claude',
  task: 'code-generation',
  language: 'typescript'
});

console.log(`Prompt compliance: [validationResult.compliant ? 'Compliant' : 'Non-compliant']`);
console.log('Missing standards context:', validationResult.missingContext);
console.log('Suggested improvements:', validationResult.suggestions);
```

### Generating Reports

```typescript
import [StandardsComplianceReporter] from '@bluefly/ai-coding-standards';

// Initialize the reporter
const reporter = new StandardsComplianceReporter({
  outputFormat: 'html',
  outputDir: './reports'
});

// Generate a report from analysis results
const report = await reporter.generateReport(analysisResults, {
  projectName: 'My Project',
  includeViolations: true,
  includeTrends: true
});

console.log(`Report generated at: [report.path]`);
```

### AI Assistant Integration

```typescript
import [ClaudeIntegration] from '@bluefly/ai-coding-standards';

// Initialize Claude integration
const claude = new ClaudeIntegration({
  apiKey: process.env.CLAUDE_API_KEY,
  standardsPath: './.acs-standards.json'
});

// Generate standards-compliant code
const result = await claude.generateCode({
  prompt: 'Create a function that sorts an array of objects by a specified property',
  language: 'javascript',
  includeStandards: true
});

console.log('Generated code:', result.code);

// Validate the generated code
const validation = await claude.validateGeneratedCode(result.code, {
  language: 'javascript'
});

console.log(`Code compliance: [validation.compliant ? 'Compliant' : 'Non-compliant']`);
```

## Configuration

### StandardsAnalyzer Configuration

```typescript
const analyzerConfig = {
  // Path to standards configuration file
  configPath: './.acs-standards.json',
  
  // Whether to include default rules
  includeDefaultRules: true,
  
  // Whether to cache analysis results
  cacheResults: true,
  
  // Maximum file size to analyze (in bytes)
  maxFileSize: 1000000,
  
  // Patterns to ignore
  ignorePatterns: ['node_modules/**', 'dist/**'],
  
  // Custom plugins to use
  plugins: ['my-custom-plugin'],
  
  // Analysis timeout (in milliseconds)
  timeout: 5000
};

const analyzer = new StandardsAnalyzer(analyzerConfig);
```

### RuleEngine Configuration

```typescript
const ruleEngineConfig = {
  // Whether to apply language-specific rules only
  languageSpecificOnly: false,
  
  // Whether to stop on first violation
  failFast: false,
  
  // Maximum violations to report per rule
  maxViolationsPerRule: 10,
  
  // Custom rule implementations
  customRules: [myCustomRule1, myCustomRule2]
};

const ruleEngine = new RuleEngine(ruleEngineConfig);
```

## Advanced Usage

### Working with Standards Configuration

```typescript
import [StandardsConfiguration] from '@bluefly/ai-coding-standards';

// Load existing configuration
const config = StandardsConfiguration.load('./.acs-standards.json');

// Add a rule
config.addRule('javascript', 'max-len', ['error', { code: 100 }]);

// Extend a standard
config.extendStandard('typescript', 'typescript-recommended');

// Set AI assistant configuration
config.setAIAssistantConfig({
  assistants: ['claude', 'gpt4'],
  contextInclusion: ['standards', 'examples']
});

// Save configuration
config.save('./.acs-standards.json');
```

### Custom Rule Implementation

```typescript
import { Rule, RuleContext, RuleViolation } from '@bluefly/ai-coding-standards';

// Create a custom rule
const customRule: Rule = {
  id: 'require-async-suffix',
  name: 'Require Async Suffix',
  description: 'Async functions should have names ending with "Async"',
  languages: ['javascript', 'typescript'],
  severity: 'error',
  validate: (code: string, context: RuleContext): RuleViolation[] => {
    const violations: RuleViolation[] = [];
    
    // Simple regex to find async functions without Async suffix
    // Note: In a real implementation, you would use a proper parser
    const asyncFnRegex = /async\s+function\s+(\w+)\s*\(/g;
    let match;
    
    while ((match = asyncFnRegex.exec(code)) !== null) {
      const functionName = match[1];
      if (!functionName.endsWith('Async')) {
        violations.push({
          rule: 'require-async-suffix',
          message: `Async function "[functionName]" should end with "Async"`,
          severity: 'error',
          line: context.getLineNumber(match.index),
          column: context.getColumnNumber(match.index),
          fix: {
            description: `Rename to "[functionName]Async"`,
            replacement: {
              text: `async function [functionName]Async(`,
              range: {
                start: match.index,
                end: match.index + match[0].length
              }
            }
          }
        });
      }
    }
    
    return violations;
  }
};
```

### Plugin Development

```typescript
import { Plugin, PluginContext } from '@bluefly/ai-coding-standards';

// Create a custom plugin
const myPlugin: Plugin = {
  name: 'my-custom-plugin',
  version: '1.0.0',
  description: 'A custom plugin for the AI Coding Standards Tool',
  
  // Initialize the plugin
  initialize: (context: PluginContext) => {
    // Register custom rules
    context.registerRule(myCustomRule1);
    context.registerRule(myCustomRule2);
    
    // Register custom formatters
    context.registerFormatter('custom-format', myCustomFormatter);
    
    // Register custom commands
    context.registerCommand('custom-command', myCustomCommand);
    
    return {
      // Cleanup function
      cleanup: () => {
        // Perform cleanup when plugin is unloaded
      }
    };
  }
};
```

## TypeScript Types

The library provides TypeScript definitions for all public APIs. Key types include:

- `Rule`: Defines a standards rule
- `RuleViolation`: Represents a rule violation
- `AnalysisResult`: Result of code analysis
- `ValidationResult`: Result of prompt validation
- `ComplianceReport`: Generated compliance report
- `StandardsConfig`: Standards configuration

Refer to the individual class documentation for detailed type information.

## Error Handling

All API methods that can fail will throw appropriate errors. Common error types include:

- `ConfigurationError`: Error in configuration
- `ValidationError`: Error during validation
- `AnalysisError`: Error during code analysis
- `IntegrationError`: Error during AI assistant integration

Example error handling:

```typescript
try {
  const result = await analyzer.analyzeFile('/path/to/file.js');
  // Process result
} catch (error) {
  if (error instanceof ConfigurationError) {
    console.error('Configuration error:', error.message);
  } else if (error instanceof AnalysisError) {
    console.error('Analysis error:', error.message);
  } else {
    console.error('Unexpected error:', error);
  }
}
```

## Next Steps

- Explore the [Standards Analyzer API](standards-analyzer.md) in detail
- Learn about [Rule Engine](rule-engine.md) capabilities
- See [AI Assistant Integration](ai-assistant-integration.md) options
- Check out [API Examples](../examples/api-usage.md) for more usage patterns