# TDDAI Drupal.org Governance Integration

TDDAI now includes comprehensive Drupal.org governance policy enforcement, ensuring all code automatically meets official Drupal contribution standards during the TDD workflow.

## Overview

The TDDAI framework has been enhanced with deep Drupal.org standards integration based on official governance policy:

- **Coding Standards Committee Compliance**: Automatic enforcement of all Drupal coding standards
- **DrupalCI Equivalent Checks**: Complete local validation before submission
- **TDD Workflow Integration**: Standards enforcement in RED-GREEN-REFACTOR phases
- **Contribution Readiness**: Automated validation for Drupal.org submission

## New CLI Commands

### Drupal Governance Validation
```bash
# Full Drupal.org compliance check
tddai drupal validate-governance --project /path/to/module

# Run TDD workflow with Drupal standards
tddai drupal tdd-workflow --project /path/to/module \
  --test-type kernel \
  --class MyService \
  --description "Tests MyService functionality"

# Enforce standards during specific TDD phase
tddai drupal enforce-standards --project /path/to/module \
  --phase green \
  --auto-fix
```

### Complete DrupalCI Check Suite
```bash
# Equivalent to core/scripts/dev/commit-code-check.sh
tddai drupal core-development-checks --project /path/to/module
```

## Automatic Standards Enforcement

### During TDD Phases

**RED Phase**: 
- Validates existing code meets Drupal standards
- Creates Drupal-compliant test templates
- Ensures proper test hierarchy (Unit/Kernel/Functional)

**GREEN Phase**:
- Auto-fixes coding standards violations during implementation
- Validates dependency injection usage (no static calls)
- Ensures governance requirements are met

**REFACTOR Phase**:
- Final comprehensive Drupal.org compliance validation
- Generates contribution readiness report
- Validates all committee requirements

## Governance Checks Included

### Code Standards
- ✅ **PHPCS**: PHP coding standards (Drupal ruleset)
- ✅ **ESLint**: JavaScript standards
- ✅ **Stylelint**: CSS/SCSS standards  
- ✅ **CSpell**: Spelling checks

### Static Analysis
- ✅ **PHPStan**: Static analysis and type checking
- ✅ **Deprecation Detection**: Legacy code identification

### Testing Requirements
- ✅ **Unit Tests**: PHPUnit unit test coverage
- ✅ **Kernel Tests**: Drupal service integration tests
- ✅ **Functional Tests**: Full browser testing
- ✅ **Coverage Threshold**: Configurable minimum coverage

### Governance Requirements
- ✅ **Naming Conventions**: Drupal-specific naming rules
- ✅ **Documentation**: Proper PHPDoc comments
- ✅ **Type Hints**: Parameter and return types
- ✅ **Dependency Injection**: No `\Drupal::` static calls

## Configuration

Create `.tddai/drupal-governance.json` in your project:

```json
{
  "projectType": "contrib",
  "enforceContribStandards": true,
  "requireTestCoverage": true,
  "minimumTestCoverage": 80,
  "enableCspellCheck": true,
  "enableEslintCheck": true,
  "enableStylelintCheck": true,
  "enablePhpstanAnalysis": true,
  "phpstanLevel": 1,
  "drupalVersion": "10.0",
  "autoFixStandards": true
}
```

## Example: Complete TDD Cycle with Governance

```bash
# 1. Start TDD workflow with governance
tddai drupal tdd-workflow \
  --project ./my_module \
  --test-type kernel \
  --class TokenManager \
  --description "Tests token management functionality" \
  --implementation-file src/Service/TokenManager.php

# This automatically:
# RED: Creates failing test with Drupal standards
# GREEN: Implements code with auto-fixed standards
# REFACTOR: Validates full Drupal.org compliance
```

## Contribution Readiness Report

After the REFACTOR phase, TDDAI generates `CONTRIBUTION_READINESS_REPORT.json`:

```json
{
  "timestamp": "2024-01-15T10:30:00Z",
  "contributionReady": true,
  "drupalCompliance": {
    "compliant": true,
    "codeStandards": {
      "phpcs": true,
      "eslint": true, 
      "stylelint": true,
      "cspell": true
    },
    "staticAnalysis": {
      "phpstan": true,
      "deprecations": true
    },
    "testing": {
      "coverage": 95,
      "meetsThreshold": true
    },
    "governance": {
      "followsNamingConventions": true,
      "hasProperDocumentation": true,
      "usesTypeHints": true,
      "avoidStaticCalls": true
    }
  },
  "recommendations": []
}
```

## Integration with Existing Projects

### Existing LLM Module Validation

```bash
# Validate the current LLM module
cd /path/to/project/web/modules/custom/llm
tddai drupal validate-governance --project .

# Expected results for EnterpriseAiProvider:
# ❌ PHPCS: 248 violations detected
# ❌ Static calls: \Drupal:: calls found
# ✅ Tests: PHPUnit configuration found
# ❌ Coverage: Below 80% threshold
```

### Auto-Fix Existing Code

```bash
# Fix standards violations automatically
tddai drupal enforce-standards --project . --auto-fix --phase refactor

# This will:
# 1. Run vendor/bin/phpcbf --standard=Drupal
# 2. Fix static calls with dependency injection
# 3. Add proper PHPDoc comments
# 4. Ensure type hints are present
```

## Best Practices

### 1. Start New Features with TDD
```bash
# Always start new features with failing test
tddai drupal tdd-workflow --step red \
  --test-type unit \
  --class NewFeature \
  --description "Tests new feature functionality"
```

### 2. Regular Governance Validation
```bash
# Run before commits
tddai drupal validate-governance --project .

# Add to git pre-commit hook
#!/bin/sh
tddai drupal validate-governance --project . || exit 1
```

### 3. CI/CD Integration
```yaml
# .gitlab-ci.yml
drupal:governance:
  stage: validate
  script:
    - npm install -g @bluefly/tddai
    - tddai drupal validate-governance --project .
  rules:
    - if: $CI_PIPELINE_SOURCE == "merge_request_event"
```

## Solving the 248 PHPCS Violations

The current `EnterpriseAiProvider.php` has 248 coding standards violations. TDDAI can fix these automatically:

```bash
cd /path/to/project/web/modules/custom/llm

# Auto-fix all violations
tddai drupal enforce-standards \
  --file src/Service/EnterpriseAiProvider.php \
  --auto-fix \
  --phase refactor

# This will:
# 1. Fix line length violations
# 2. Add missing documentation
# 3. Fix spacing and formatting
# 4. Ensure proper type hints
# 5. Replace static calls with DI
```

## Committee Approval Workflow

For contrib modules requiring Coding Standards committee approval:

```bash
# Generate compliance report for committee review
tddai drupal validate-governance --project . --generate-report

# Creates committee-review.json with:
# - All standards compliance details
# - Test coverage analysis  
# - Governance requirement validation
# - Automated fix recommendations
```

## Integration with Drupal.org

TDDAI ensures your code will pass all Drupal.org automated testing:

### DrupalCI Jobs That Will Pass
- ✅ **PHP CodeSniffer**: No coding standards violations
- ✅ **ESLint**: JavaScript meets standards
- ✅ **Stylelint**: CSS follows guidelines
- ✅ **CSpell**: No spelling errors
- ✅ **PHPStan**: Static analysis passes
- ✅ **PHPUnit**: All tests pass with coverage

### Pre-Submission Checklist
```bash
# Complete pre-submission validation
tddai drupal validate-governance --project . --strict

# Generates checklist:
# ✅ Code standards: PHPCS, ESLint, Stylelint, CSpell
# ✅ Static analysis: PHPStan level 1+
# ✅ Testing: 80%+ coverage, all test types
# ✅ Governance: Naming, docs, types, no static calls
# ✅ Contribution ready: All requirements met
```

This integration ensures that **all code automatically meets Drupal.org standards** without manual intervention, eliminating the common problem of failed automated testing during contribution submissions.