# Git Integration with Claude Code Attribution Control

This document describes the TDDAI git integration feature that provides configurable Claude Code attribution in commit messages.

## Overview

The TDDAI framework now includes comprehensive git operations with configurable commit message formatting. This feature allows you to:

- Control Claude Code attribution in commit messages
- Configure co-author attribution for AI assistance
- Set up automated staging and pre-commit validations
- Customize commit message templates and formatting
- Integrate with TDD workflow validation

## Features

### Configurable Attribution

- **Claude Code Attribution**: Add or remove "Generated with Claude Code" attribution
- **Co-Author Support**: Include "Co-Authored-By: Claude" lines in commits
- **Custom Attribution**: Use custom attribution text instead of defaults
- **Template Support**: Apply commit message templates and formatting

### Pre-commit Validation

- **Test Execution**: Run tests before committing changes
- **TDD Compliance**: Validate TDD compliance before commits
- **File Filtering**: Exclude specific patterns from auto-staging
- **Hook Integration**: Support for skipping or running git hooks

### Workflow Integration

- **Auto-staging**: Automatically stage files based on configuration
- **Quick Commits**: Streamlined commit process with sensible defaults
- **Status Monitoring**: Enhanced git status with TDDAI information
- **Configuration Management**: Persistent configuration with project-level settings

## Installation and Setup

The git integration is included with TDDAI. No additional installation is required.

```bash
# Install TDDAI (if not already installed)
npm install -g @bluefly/tddai

# Initialize git integration in your project
tddai git config --show
```

## CLI Commands

### Configuration Commands

#### Show Current Configuration
```bash
# Display current git configuration
tddai git config --show
```

#### Configure Attribution
```bash
# Disable Claude Code attribution
tddai git config --no-claude-attribution

# Disable co-author line
tddai git config --no-co-author

# Set custom attribution text
tddai git config --custom-attribution "Generated with AI Assistant"

# Set custom co-author format
tddai git config --custom-co-author "Co-Authored-By: AI Bot <ai@example.com>"
```

#### Configure Commit Behavior
```bash
# Enable conventional commits format
tddai git config --conventional-commits

# Set maximum line length
tddai git config --max-line-length 80

# Enable auto-staging
tddai git config --auto-stage

# Set exclude patterns for auto-staging
tddai git config --exclude-patterns "*.log,node_modules/**,dist/**"
```

#### Configure Pre-commit Validation
```bash
# Disable tests before commit
tddai git config --no-tests-before-commit

# Disable TDD validation before commit
tddai git config --no-tdd-validation
```

### Git Operations

#### Status Command
```bash
# Show enhanced git status with TDDAI information
tddai git status
```

#### Commit Commands
```bash
# Create commit with configured attribution
tddai git commit "Add new feature"

# Commit specific files
tddai git commit "Update components" --files "src/component1.ts,src/component2.ts"

# Commit with custom attribution for this commit only
tddai git commit "Fix bug" --custom-attribution "Fixed by AI Assistant"

# Commit without attribution for this commit only
tddai git commit "Minor update" --no-claude-attribution --no-co-author

# Amend previous commit
tddai git commit "Updated commit message" --amend

# Skip pre-commit hooks
tddai git commit "Emergency fix" --skip-hooks
```

#### Quick Commit
```bash
# Quick commit with auto-staging and validation
tddai git quick-commit "Complete feature implementation"

# Quick commit skipping tests
tddai git quick-commit "WIP: Work in progress" --skip-tests

# Quick commit skipping TDD validation
tddai git quick-commit "Documentation update" --skip-validation
```

#### Stage Operations
```bash
# Stage specific files
tddai git stage src/file1.ts src/file2.ts

# Stage files will be done automatically with auto-stage enabled
```

## Configuration File

TDDAI stores git configuration in `.tddai/git.config.json`:

```json
{
  "commit": {
    "includeClaudeAttribution": true,
    "includeCoAuthor": true,
    "useConventionalCommits": false,
    "maxLineLength": 72,
    "customAttribution": null,
    "customCoAuthor": null,
    "messageTemplate": null
  },
  "autoStage": false,
  "excludePatterns": [
    "node_modules/**",
    "*.log",
    ".env*"
  ],
  "runTestsBeforeCommit": true,
  "validateTDDBeforeCommit": true
}
```

## Programmatic Usage

### Basic Usage

```typescript
import [GitService] from '@bluefly/tddai';

// Initialize git service
const gitService = new GitService('/path/to/project');

// Load configuration
await gitService.loadConfig();

// Create a commit
const result = await gitService.commit({
  message: 'Add new feature',
  files: ['src/feature.ts', 'src/feature.test.ts']
});

if (result.success) {
  console.log(`Commit created: [result.hash]`);
  console.log(`Files committed: [result.filesCommitted.join(', ')]`);
}
```

### Advanced Configuration

```typescript
import { GitService, GitConfig } from '@bluefly/tddai';

const gitService = new GitService('/path/to/project');

// Update configuration
gitService.updateConfig({
  commit: {
    includeClaudeAttribution: false,
    customAttribution: 'Generated with Custom AI',
    maxLineLength: 100
  },
  autoStage: true,
  excludePatterns: ['*.log', 'tmp/**']
});

// Save configuration
await gitService.saveConfig();

// Create commit with custom options
const result = await gitService.commit({
  message: 'Implement user authentication',
  config: {
    includeClaudeAttribution: true,
    customAttribution: 'Implemented with AI assistance'
  }
});
```

### Status Monitoring

```typescript
// Get current git status
const status = await gitService.getStatus();

console.log(`Branch: [status.branch]`);
console.log(`Staged files: [status.staged.length]`);
console.log(`Unstaged files: [status.unstaged.length]`);
console.log(`Untracked files: [status.untracked.length]`);
console.log(`Working directory clean: [status.clean]`);
```

## Integration with TDD Workflow

### Automated TDD Compliance

```bash
# Configure strict TDD compliance
tddai git config --no-auto-stage  # Require manual staging
tddai git config --max-line-length 50  # Enforce short commit messages

# Enable all validations
tddai git config  # Uses defaults with tests and TDD validation

# Create commit that will validate TDD compliance
tddai git commit "Add failing test for user service"  # RED phase
tddai git commit "Implement user service to pass tests"  # GREEN phase  
tddai git commit "Refactor user service for clarity"  # REFACTOR phase
```

### Pre-commit Test Execution

```bash
# Tests will run automatically before commit
tddai git commit "Add new feature"  # Will fail if tests don't pass

# Skip tests for non-code changes
tddai git commit "Update documentation" --skip-hooks
```

## Examples

### Example 1: Basic Setup

```bash
# Configure for startup team (less strict)
tddai git config --auto-stage --max-line-length 100

# Create commits with attribution
tddai git commit "Initial project setup"
tddai git commit "Add user registration feature"
```

### Example 2: Enterprise Setup

```bash
# Configure for enterprise (strict compliance)
tddai git config --conventional-commits --max-line-length 50
tddai git config --exclude-patterns "*.log,*.tmp,dist/**,coverage/**"

# All commits will be validated for TDD compliance
tddai git commit "feat: add user authentication"
tddai git commit "test: add unit tests for auth service"
tddai git commit "refactor: improve auth code structure"
```

### Example 3: Custom Attribution

```bash
# Set up custom AI attribution
tddai git config --custom-attribution "✨ Enhanced with AI coding assistant"
tddai git config --custom-co-author "Co-Authored-By: AI Assistant <ai@company.com>"

# Commits will use custom attribution
tddai git commit "Implement advanced search functionality"
```

### Example 4: Disable Attribution

```bash
# Disable all AI attribution
tddai git config --no-claude-attribution --no-co-author

# Or disable for specific commits only
tddai git commit "Fix typo in documentation" --no-claude-attribution --no-co-author
```

## Default Commit Message Format

### With Attribution Enabled (Default)
```
Add user authentication feature

🤖 Generated with [Claude Code](https://claude.ai/code)

Co-Authored-By: Claude <noreply@anthropic.com>
```

### With Custom Attribution
```
Add user authentication feature

✨ Enhanced with AI coding assistant

Co-Authored-By: AI Assistant <ai@company.com>
```

### Without Attribution
```
Add user authentication feature
```

## Troubleshooting

### Configuration Issues

```bash
# Reset to default configuration
rm .tddai/git.config.json
tddai git config --show  # Will show defaults

# Check current configuration
tddai git config --show

# Verify git repository
tddai git status
```

### Pre-commit Failures

```bash
# Skip validations temporarily
tddai git commit "Emergency fix" --skip-hooks

# Check TDD compliance separately
tddai validate --strict

# Run tests manually
npm test
```

### File Staging Issues

```bash
# Check what would be auto-staged
tddai git status

# Manual staging
tddai git stage src/file1.ts src/file2.ts

# Update exclude patterns
tddai git config --exclude-patterns "*.log,tmp/**"
```

## Integration with CI/CD

### GitHub Actions

```yaml
name: TDD Compliance Check
on: [push, pull_request]

jobs:
  validate:
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v4
      - uses: actions/setup-node@v4
      - run: npm install -g @bluefly/tddai
      - run: tddai validate --strict
      - run: tddai git config --show  # Show git configuration
```

### GitLab CI

```yaml
stages:
  - validate

tdd:validate:
  stage: validate
  script:
    - npm install -g @bluefly/tddai
    - tddai validate --strict
    - tddai git config --show
```

## Best Practices

### 1. Team Standardization

```bash
# Create team configuration script
echo '#!/bin/bash
tddai git config --auto-stage
tddai git config --max-line-length 72
tddai git config --exclude-patterns "*.log,node_modules/**,dist/**"
echo "Git configuration standardized for team"
' > setup-git-standards.sh

chmod +x setup-git-standards.sh
```

### 2. Project-Specific Settings

```bash
# Add to package.json scripts
{
  "scripts": {
    "setup:git": "tddai git config --auto-stage --max-line-length 50",
    "commit": "tddai git quick-commit",
    "commit:manual": "tddai git commit"
  }
}
```

### 3. Attribution Control

```bash
# For public repositories (keep attribution)
tddai git config  # Uses defaults

# For private repositories (custom attribution)
tddai git config --custom-attribution "Developed with AI assistance"

# For sensitive projects (no attribution)
tddai git config --no-claude-attribution --no-co-author
```

### 4. Workflow Integration

```bash
# TDD cycle with proper commits
tddai git commit "test: add failing test for feature X"  # RED
tddai git commit "feat: implement feature X"  # GREEN
tddai git commit "refactor: improve feature X implementation"  # REFACTOR
```

## Migration from Standard Git

If you're migrating from standard git workflows:

1. **Install TDDAI**: `npm install -g @bluefly/tddai`
2. **Configure attribution**: `tddai git config --show` to see current settings
3. **Update workflow**: Replace `git commit` with `tddai git commit`
4. **Enable validations**: Keep defaults for TDD compliance
5. **Team training**: Share configuration across team

The git integration maintains full compatibility with standard git operations while adding AI attribution control and TDD validation features.

## Support

For issues with git integration:

1. Check configuration: `tddai git config --show`
2. Verify git repository: `tddai git status`
3. Test with simple commit: `tddai git commit "test message" --no-claude-attribution`
4. Check TDDAI installation: `tddai --version`

The git integration feature provides complete control over Claude Code attribution while maintaining seamless integration with existing TDD workflows and team collaboration processes.