# AI Assistant Integration Guide

This document outlines how the ReelCode AI Standards Management System integrates with different AI assistants to enforce coding standards across your projects.

## Overview

The AI assistant integration framework provides:
1. Standard translation to AI-specific formats
2. Assistant-specific optimizations
3. Bidirectional synchronization between standards and AI configurations
4. Extensible plugin system for new AI assistants

## Supported AI Assistants

### Claude
[Claude](https://claude.ai/code) is Anthropic's AI assistant that can help with coding tasks. ReelCode generates `CLAUDE.md` files that define coding standards and project guidelines for Claude to follow.

### GitHub Copilot
[GitHub Copilot](https://github.com/features/copilot) provides AI-assisted code completion. ReelCode configures Copilot through project-level settings to follow your defined standards.

### Cursor
[Cursor](https://cursor.sh/) is an AI-powered code editor. ReelCode integrates with Cursor through custom rules and configurations.

### OpenAI (ChatGPT)
[OpenAI's ChatGPT](https://openai.com/) can be configured with custom instructions to follow your coding standards.

### Generic Integration
For AI assistants without specific adapters, ReelCode provides generic markdown-based instructions that can be used in prompts.

## Integration Methods

### 1. CLAUDE.md Generation

The most complete integration is with Claude through CLAUDE.md files:

```typescript
import [StandardsManager] from '@bluefly/tddai';

// Initialize standards manager
const manager = new StandardsManager({
  projectId: 'my-project',
  configPath: '.tddai/standards.yaml'
});

// Generate CLAUDE.md
await manager.generateAIConfig('claude', {
  outputPath: 'CLAUDE.md',
  customSections: [
    { 
      title: 'Project Architecture', 
      content: 'This project follows a microservice architecture...' 
    }
  ]
});
```

The generated CLAUDE.md file will contain:
- Coding style conventions from your standards
- Testing requirements
- Documentation standards
- Custom sections you've defined
- Project-specific guidelines

### 2. AI Assistant-Specific Adapters

For each supported AI assistant, ReelCode provides adapter classes:

```typescript
import { 
  AIAssistantAdapter,
  AIAssistantType 
} from '@bluefly/tddai';

// Create adapter for a specific assistant
const claudeAdapter = AIAssistantAdapter.create(AIAssistantType.Claude);
const copilotAdapter = AIAssistantAdapter.create(AIAssistantType.Copilot);

// Load standards
const standards = await loadStandards('.tddai/standards.yaml');

// Generate assistant-specific configuration
const claudeInstructions = await claudeAdapter.generateConfig(standards);
const copilotSettings = await copilotAdapter.generateConfig(standards);

// Apply configuration
await claudeAdapter.applyConfig(claudeInstructions, 'CLAUDE.md');
await copilotAdapter.applyConfig(copilotSettings, '.github/copilot');
```

### 3. Custom Adapters

You can create and register custom adapters for other AI assistants:

```typescript
import [AIAssistantAdapter] from '@bluefly/tddai';

// Create a custom adapter
class MyCustomAssistantAdapter extends AIAssistantAdapter {
  async generateConfig(standards) {
    // Transform standards to your assistant's format
    return `# My Custom Assistant Standards\n...`;
  }
  
  async applyConfig(config, outputPath) [// Apply the configuration to your assistant]
}

// Register the adapter
AIAssistantAdapter.register('my-assistant', MyCustomAssistantAdapter);

// Use the adapter
const myAdapter = AIAssistantAdapter.create('my-assistant');
```

## CLAUDE.md Structure

The CLAUDE.md file is structured to provide clear guidance to Claude:

```markdown
# Project Coding Standards

## Overview
This document defines coding standards for the [Project Name] project.

## TypeScript Conventions
- **Classes**: Use PascalCase (e.g., `UserService`)
- **Interfaces**: Use PascalCase (e.g., `UserProfile`)
- **Variables**: Use camelCase (e.g., `userData`)
- **Constants**: Use UPPER_SNAKE_CASE (e.g., `MAX_RETRY_COUNT`)
- **Strict Mode**: Enabled
- **Indentation**: 2 spaces

## Testing Requirements
- **Coverage**: Minimum 90% test coverage required
- **Methodology**: Test-Driven Development (TDD)
- **Required Tests**: Unit tests, Integration tests

## Documentation
- API documentation using OpenAPI 3.1.0
- JSDoc comments for all public functions
- Required documentation files: README.md, CONTRIBUTING.md

## Project Structure
- Required directories: src/, tests/, docs/
- Component files should be named in PascalCase
- Utility files should be named in camelCase

## [Custom Section Title]
[Custom section content]
```

## Integration with CI/CD

ReelCode integrates with CI/CD systems to validate code against standards:

### GitLab CI

```yaml
# .gitlab-ci.yml
standards:
  stage: validate
  script:
    - npm install -g @bluefly/tddai
    - tddai validate --project-id $CI_PROJECT_NAME
  artifacts:
    paths:
      - standards-report.html
```

### GitHub Actions

```yaml
# .github/workflows/standards.yml
name: Standards Validation
on: [push, pull_request]
jobs:
  validate:
    runs-on: ubuntu-latest
    steps:
      - uses: actions/checkout@v3
      - name: Setup Node.js
        uses: actions/setup-node@v3
        with:
          node-version: '18'
      - name: Install ReelCode
        run: npm install -g @bluefly/tddai
      - name: Validate Standards
        run: tddai validate
      - name: Upload Report
        uses: actions/upload-artifact@v3
        with:
          name: standards-report
          path: standards-report.html
```

## IDE Integration

ReelCode can integrate with IDEs to provide real-time standards enforcement:

### VS Code Extension

The VS Code extension provides:
- Real-time standards validation
- Quick fixes for common issues
- Standards documentation within the editor
- AI assistant configuration management

### JetBrains IDEs

For JetBrains IDEs (WebStorm, PhpStorm, etc.), ReelCode provides:
- Standards validation via Language Server Protocol
- Code inspections based on your standards
- Quick fixes for standards violations

## Interactive Playground

ReelCode includes an interactive playground for testing AI assistant compliance:

```bash
# Start the playground
tddai playground --standards .tddai/standards.yaml
```

The playground allows you to:
- Input code and see how different AI assistants would modify it
- Compare AI assistant behavior with and without standards
- Fine-tune your standards based on AI assistant behavior

## Example Usage

### Basic Integration

```typescript
// Basic integration example
import [StandardsManager] from '@bluefly/tddai';

// Initialize standards manager
const standardsManager = new StandardsManager({
  projectId: 'my-project',
  configPath: '.tddai/standards.yaml'
});

// Generate CLAUDE.md
await standardsManager.generateAIConfig('claude', {
  outputPath: 'CLAUDE.md'
});

// Validate project against standards
const results = await standardsManager.validate();
console.log(`Compliance: [results.valid ? 'Passed' : 'Failed']`);
```

### Advanced Integration

```typescript
// Advanced integration with custom validators and AI adapters
import { StandardsManager, AIAssistantAdapter } from '@bluefly/tddai';
import [CustomValidator] from './validators/custom';

// Initialize standards manager
const standardsManager = new StandardsManager({
  projectId: 'advanced-project',
  configPath: '.tddai/standards.yaml'
});

// Register custom validator
standardsManager.registerValidator('custom', new CustomValidator());

// Create custom AI assistant adapter
class MyAIAdapter extends AIAssistantAdapter {
  async generateConfig(standards) {
    return `# Custom AI Standards\n...`;
  }
}

// Register custom adapter
AIAssistantAdapter.register('my-ai', MyAIAdapter);

// Generate configurations
await standardsManager.generateAIConfig('claude', { outputPath: 'CLAUDE.md' });
await standardsManager.generateAIConfig('my-ai', { outputPath: 'MY_AI.md' });

// Validate with custom options
const results = await standardsManager.validate({
  includeValidators: ['typescript', 'custom'],
  excludePaths: ['node_modules', 'dist']
});
```

## Conclusion

The AI assistant integration framework in ReelCode provides a comprehensive solution for enforcing coding standards across different AI assistants. By generating assistant-specific configurations and providing validation tools, ReelCode ensures that all AI-assisted code follows your organization's standards and best practices.