# Drupal Module Standards

This document collects standards and best practices for Drupal modules, integrating information from prior organization efforts.

## Naming Convention Standardization

Drupal has established coding standards and naming conventions that emphasize clarity, descriptiveness, and consistency. When working with Drupal modules, avoid superlative marketing terms in code, service names, and comments.

### Standardization Principles

1. Improve code readability and maintainability
2. Follow Drupal's established naming conventions
3. Make service and class names more descriptive of their actual functionality
4. Ensure backward compatibility while moving to standardized names

### Examples of Standardized Names

| Original Name (Avoid) | Standardized Name (Recommended) |
|-----------------------|--------------------------------|
| `EnhancedNodeJsBridge` | `AdvancedNodeJsBridge` |
| `UltimateMcpPlatformService` | `McpPlatformService` |
| `RevolutionaryApiService` | `ApiService` |
| `QuantumResistantCryptographyService` | `AdvancedQuantumCryptographyService` |

### Implementation Approach

When standardizing naming conventions:

1. Identify services, classes, and comments with superlative marketing/hype terms
2. Create standardized versions with more descriptive names
3. Maintain backward compatibility through:
   - Service aliases in service definition files
   - Class aliases in PHP files
   - Documentation of original names
4. Update references to renamed services and classes
5. Add update hooks to inform users about the changes
6. Create documentation to record the changes

### Backward Compatibility

Maintain backward compatibility through:

```yaml
# Service alias example (in .services.yml)
original_service_name:
  alias: new_standardized_service_name
  class: NewStandardizedClassName
```

```php
// Class alias example (in PHP file)
class_alias(NewStandardizedClassName::class, 'Original\\Path\\OriginalClassName');

/**
 * Provides advanced functionality.
 *
 * @note Formerly EnhancedClassName - renamed for clarity.
 */
```

## Module Organization

### Submodule Organization

When organizing modules and submodules:

1. Place submodules in a `/modules` directory within the main module
2. Update the main module's `.info.yml` file to include necessary dependencies
3. Maintain consistent naming patterns across submodules
4. Ensure proper routing and service definition in both main and submodules

### Module Merge Best Practices

When merging modules:

1. **Analysis Phase**
   - Analyze module structures and dependencies
   - Identify overlaps between standalone modules and existing submodules
   - Create a comprehensive merge strategy
   - Back up all modules prior to beginning the merge

2. **Implementation Phase**
   - Copy module files into the appropriate submodule directories
   - Update info.yml files for proper dependencies and module hierarchy
   - Update routing, service definitions, and library references
   - Maintain backward compatibility with legacy paths
   - Update documentation to reflect the new structure

3. **Testing Phase**
   - Create a comprehensive test plan for verifying the merge
   - Test all integrated functionality
   - Verify backward compatibility

### Testing Plan for Merged Modules

When testing merged modules, verify:

1. **Installation Testing**
   - Fresh installation works correctly
   - Upgrade from previous version works correctly
   - Submodules are properly discovered

2. **Functionality Testing**
   - All features from both original and merged modules work correctly
   - Integration points between modules function properly
   - Performance is maintained or improved

3. **Filesystem Verification**
   - Directory structure is correct
   - File permissions are correct
   - Symlinks and references are correctly updated

4. **Service Verification**
   - Service registration is correct
   - Plugin discovery works properly
   - Event subscribers are properly registered and triggered

5. **Compatibility Testing**
   - Browser compatibility
   - PHP version compatibility
   - Drupal version compatibility

## Implementation Strategy Example

This example outlines the strategy for merging modules into a main module:

### Directory Structure

```
drupal_module_name/
├── drupal_module_name.info.yml
├── drupal_module_name.module
├── drupal_module_name.services.yml
├── modules/
│   ├── drupal_module_name_submodule_one/
│   │   ├── drupal_module_name_submodule_one.info.yml
│   │   ├── drupal_module_name_submodule_one.module
│   │   └── ...
│   └── drupal_module_name_submodule_two/
│       ├── drupal_module_name_submodule_two.info.yml
│       ├── drupal_module_name_submodule_two.module
│       └── ...
└── ...
```

### Main Module Updates

1. Update the main module's `.info.yml` file:
   ```yaml
   name: 'Main Module Name'
   description: 'Description of the main module'
   type: module
   core_version_requirement: '^9.3 || ^10'
   dependencies:
     - drupal:system
     # Add any other dependencies
   ```

2. Update service definitions in `.services.yml`:
   ```yaml
   services:
     main_module.service:
       class: Drupal\main_module\Service\MainService
       arguments: ['@service_container']
     
     # Legacy service names (for backward compatibility)
     old_service_name:
       alias: main_module.service
   ```

### Backward Compatibility

To ensure a smooth transition when merging modules:

1. Keep legacy directory paths for a transition period
2. Ensure route loading supports both old and new paths
3. Update service definitions to point to the new implementations
4. Add class aliases for renamed classes
5. Properly document the changes for developers

## ReelCode Integration for Drupal Modules

To integrate ReelCode with Drupal module projects:

1. Create a `.tddai/standards.yaml` file in your module directory
2. Run `tddai integrate project drupal /path/to/module`
3. Use `tddai validate project` to check compliance with Drupal coding standards
4. Generate a `CLAUDE.md` file with Drupal-specific guidance

This approach ensures consistent naming conventions, proper module organization, and maintainable code across your Drupal module projects.