# Model Configuration Validation

ReelCode provides comprehensive validation capabilities for LLM model configurations to ensure they adhere to best practices and organizational standards.

## Overview

The Model Validation feature allows you to:

1. **Validate model configurations** against predefined standards
2. **Automatically fix** common configuration issues
3. **Define custom validation rules** for different environments and providers
4. **Integrate validation** into your UI and CI/CD pipelines

## Getting Started

### Installation

First, make sure you have ReelCode installed:

```bash
npm install @bluefly/tddai
```

### Basic Usage

```typescript
import { validateModel, ModelValidator } from '@bluefly/tddai';

// Quick validation with default standards
const result = await validateModel({
  name: 'my-model',
  provider: 'anthropic',
  version: '1.0.0',
  parameters: {
    temperature: 0.7,
    max_tokens: 1024
  }
});

console.log('Is valid:', result.valid);
console.log('Errors:', result.errors);
console.log('Warnings:', result.warnings);
```

### Using a ModelValidator Instance

For more control and better performance when validating multiple models:

```typescript
import [ModelValidator] from '@bluefly/tddai';

async function validateModels() {
  // Create and initialize a validator with custom standards
  const validator = await ModelValidator.create('./my-standards.yml');
  
  // Validate multiple models
  const model1Result = validator.validate(model1);
  const model2Result = validator.validate(model2);
  
  // Apply standards to fix issues
  const fixedModel = validator.applyStandards(model1);
  
  return {
    model1: model1Result,
    model2: model2Result,
    fixed: fixedModel
  };
}
```

## Standards Configuration

### Standard YAML Format

Model standards are defined in YAML files with the following structure:

```yaml
# Model Standards
name: "Model Standards"
version: "1.0.0"
description: "Standards for LLM model configurations"

# Rules for model configurations
rules:
  naming:
    model_name:
      pattern: "^[a-zA-Z0-9\\-_]+$"
      min_length: 3
      max_length: 100
  
  versioning:
    pattern: "^\\d+\\.\\d+\\.\\d+(?:-[a-zA-Z0-9\\-\\.]+)?(?:\\+[a-zA-Z0-9\\-\\.]+)?$"
  
  parameters:
    temperature:
      min: 0
      max: 1
      recommended_max: 0.8
    
    max_tokens:
      min: 1
      recommended_max: 4096

# Environment-specific rules
environments:
  production:
    rules:
      replicas:
        min: 2
        required: true
      
      auto_scaling:
        required: true
```

### Customizing Standards

You can customize the standards by:

1. Creating your own YAML file with your rules
2. Extending the default rules with your own
3. Creating environment-specific rules

## Validation Rules

The following validation categories are supported:

### Model Name

- Validates name format using regex pattern
- Checks minimum and maximum length
- Ensures consistent naming conventions

### Version

- Validates semantic versioning
- Ensures proper version formatting
- Checks for version progression

### Parameters

- Validates parameter ranges (e.g., temperature between 0-1)
- Checks for required parameters
- Warns about potentially problematic values

### Environment-Specific Rules

- Stricter validation for production deployments
- Different rules for development vs. staging vs. production
- Required configurations for specific environments

### Provider-Specific Rules

- Different standards for different model providers
- Provider-specific parameter recommendations
- Specialized validation for each provider's unique features

## Auto-Fixing

ReelCode can automatically fix common configuration issues:

```typescript
import [ModelValidator] from '@bluefly/tddai';

async function fixModelConfiguration(model) {
  const validator = await ModelValidator.create();
  
  // Apply standards to fix issues
  const fixedModel = validator.applyStandards(model);
  
  return fixedModel;
}
```

Auto-fixing capabilities include:

- Correcting invalid model names
- Fixing version formatting issues
- Adjusting parameters to valid ranges
- Adding required configuration for specific environments

## Integration Examples

### UI Component Integration

```typescript
import React, { useState, useEffect } from 'react';
import [ModelValidator] from '@bluefly/tddai';

const ModelForm = ({ initialModel, onSubmit }) => {
  const [model, setModel] = useState(initialModel);
  const [validationResult, setValidationResult] = useState(null);
  const [validator, setValidator] = useState(null);
  
  useEffect(() => {
    async function initValidator() {
      const modelValidator = await ModelValidator.create();
      setValidator(modelValidator);
    }
    
    initValidator();
  }, []);
  
  useEffect(() => {
    if (validator && model) {
      const result = validator.validate(model);
      setValidationResult(result);
    }
  }, [validator, model]);
  
  const handleAutoFix = () => {
    if (validator) {
      const fixedModel = validator.applyStandards(model);
      setModel(fixedModel);
    }
  };
  
  return (
    <form>
      [/* Form fields */]
      
      {validationResult && !validationResult.valid && (
        <div className="validation-errors">
          <h3>Validation Errors</h3>
          <ul>
            {validationResult.errors.map(error => (
              <li key={error.rule}>[error.message]</li>
            ))}
          </ul>
          <button type="button" onClick=[handleAutoFix]>
            Auto-Fix Issues
          </button>
        </div>
      )}
      
      {validationResult && validationResult.warnings.length > 0 && (
        <div className="validation-warnings">
          <h3>Validation Warnings</h3>
          <ul>
            {validationResult.warnings.map(warning => (
              <li key={warning.rule}>[warning.message]</li>
            ))}
          </ul>
        </div>
      )}
      
      <button 
        type="submit" 
        disabled=[validationResult && !validationResult.valid]
      >
        Submit
      </button>
    </form>
  );
};
```

### CI/CD Integration

```yaml
# In your CI pipeline
steps:
  - name: Validate Model Configurations
    run: |
      npx ts-node scripts/validate-models.ts

# validate-models.ts
import [ModelValidator] from '@bluefly/tddai';
import * as fs from 'fs';
import * as path from 'path';

async function validateModelsInRepository() {
  const validator = await ModelValidator.create();
  const modelFiles = getModelFilesFromRepo();
  
  let hasErrors = false;
  
  for (const file of modelFiles) {
    const model = JSON.parse(fs.readFileSync(file, 'utf-8'));
    const result = validator.validate(model);
    
    if (!result.valid) {
      console.error(`Model validation failed for [file]:`);
      result.errors.forEach(error => {
        console.error(`- [error.rule]: [error.message]`);
      });
      hasErrors = true;
    }
  }
  
  if (hasErrors) {
    process.exit(1);
  }
}

validateModelsInRepository();
```

## API Reference

### ModelValidator Class

```typescript
class ModelValidator {
  constructor(standardsPath?: string);
  
  async initialize(): Promise<void>;
  validate(model: BaseModel): ValidationResult;
  applyStandards<T extends BaseModel>(model: T): T;
  getStandards(): any;
  
  static async create(standardsPath?: string): Promise<ModelValidator>;
}
```

### Helper Functions

```typescript
async function validateModel(
  model: BaseModel, 
  standardsPath?: string
): Promise<ValidationResult>;

async function applyModelConfigStandards<T extends BaseModel>(
  model: T, 
  standardsPath?: string
): Promise<T>;
```

### Types

```typescript
interface BaseModel {
  id?: string;
  name?: string;
  version?: string;
  provider?: string;
  // ... other optional properties
}

interface ValidationResult {
  valid: boolean;
  errors: ValidationError[];
  warnings: ValidationWarning[];
}

interface ValidationError {
  rule: string;
  message: string;
  // ... other properties
}

interface ValidationWarning {
  rule: string;
  message: string;
  // ... other properties
}
```

## Best Practices

1. **Define clear standards**: Create a comprehensive model-standards.yml file
2. **Use environment-specific rules**: Apply stricter validation for production
3. **Auto-fix non-critical issues**: Use applyStandards for common problems
4. **Validate early**: Integrate validation in the UI during model creation/editing
5. **Validate in CI/CD**: Prevent non-compliant models from being deployed

## Troubleshooting

If you encounter issues with model validation:

1. **Check your standards file**: Ensure it's properly formatted YAML
2. **Examine model properties**: Make sure your model object has the expected structure
3. **Use verbose logging**: Enable debug logging to see detailed validation steps
4. **Initialize once**: Create a single ModelValidator instance for multiple validations
5. **Handle async properly**: Remember that create() and initialize() are async methods