---
id: shared-SHARED_CONFIGURATION
---
# Shared Configuration System

## Overview

The shared configuration system provides a centralized way to manage configuration across all LLM Platform projects. It supports multiple configuration sources, merging of configuration values, and automatic loading from files and environment variables.

## Features

- **Hierarchical Configuration**: Configuration is loaded from multiple sources in a defined order
- **Multiple Sources**: Supports configuration from files, environment variables, command line, and defaults
- **Automatic Loading**: Automatically loads configuration from standard locations
- **Type Safety**: Full TypeScript support with detailed type definitions
- **Environment Awareness**: Automatically adapts to different environments (development, production, etc.)
- **Deep Merging**: Intelligently merges nested configuration objects

## Configuration Sources

The configuration system loads values from multiple sources in the following order (later sources override earlier ones):

1. **Default Values**: Built-in default configuration
2. **Config Files**: JSON or JS configuration files
3. **Environment Variables**: System environment variables
4. **Command Line**: Arguments passed via CLI
5. **Runtime Overrides**: Values set programmatically during runtime

## Basic Usage

```typescript
import { config } from '@bluefly/llmcli/shared';

// Get a configuration value
const apiUrl = config.get('services.api.url', 'https://default-api.example.com');

// Set a configuration value
config.set('services.api.timeout', 5000);

// Check if a configuration value exists
if (config.has('services.auth.tokenExpiration')) {
  // Use the value
}

// Get all configuration
const allConfig = config.getAll();
```

## Creating a Configuration Instance

```typescript
import { createConfig } from '@bluefly/llmcli/shared';

// Create a custom configuration instance
const serviceConfig = createConfig({
  configDir: '.config',
  configFile: 'service-config.json',
  envPrefix: 'MY_SERVICE_',
  autoload: true,
  mergeSources: true
});

// Use the custom configuration
const servicePort = serviceConfig.get('port', 3000);
```

## Configuration Files

The configuration system automatically loads configuration from JSON files:

```json
// .config/config.json
{
  "services": {
    "api": {
      "url": "https://api.example.com",
      "timeout": 5000,
      "retries": 3
    },
    "auth": {
      "tokenExpiration": 3600,
      "refreshEnabled": true
    }
  },
  "logging": {
    "level": "info",
    "format": "json"
  }
}
```

You can also save configuration back to a file:

```typescript
// Save current configuration to file
config.save();

// Save to a specific file
config.save('/path/to/custom-config.json');
```

## Environment Variables

Environment variables are automatically mapped to configuration properties using the specified prefix:

- Environment variables are converted from `UPPERCASE_SNAKE_CASE` to `lowercase.dot.notation`
- The default prefix is `BF_`, but can be customized when creating a configuration instance
- Arrays and nested objects are not directly supported via environment variables

Examples:
```
BF_SERVICES_API_URL=https://api.example.com
BF_LOGGING_LEVEL=debug
BF_SERVICES_AUTH_TOKEN_EXPIRATION=7200
```

These would map to:
```javascript
{
  "services.api.url": "https://api.example.com",
  "logging.level": "debug",
  "services.auth.token.expiration": "7200"
}
```

## Advanced Usage

### Setting Multiple Values

```typescript
// Set multiple values at once
config.setMultiple({
  'services.api.url': 'https://new-api.example.com',
  'services.api.timeout': 10000,
  'logging.level': 'debug'
}, 'override');
```

### Getting Detailed Configuration

```typescript
// Get detailed configuration with metadata
const detailedConfig = config.getDetailed();

// Access metadata
for (const [key, { value, source, timestamp }] of Object.entries(detailedConfig)) {
  console.log(`${key}: ${value} (from ${source} at ${new Date(timestamp)})`);
}
```

### Environment-Specific Configuration

```typescript
// Load environment-specific configuration
const env = process.env.NODE_ENV || 'development';
config.loadConfigFile(`.config/config.${env}.json`);

// Check the environment
if (config.get('environment') === 'production') {
  // Apply production-specific logic
}
```

## Type Safety

You can use TypeScript to define your configuration schema:

```typescript
interface AppConfig {
  services: {
    api: {
      url: string;
      timeout: number;
      retries: number;
    };
    auth: {
      tokenExpiration: number;
      refreshEnabled: boolean;
    };
  };
  logging: {
    level: 'debug' | 'info' | 'warn' | 'error';
    format: 'json' | 'text';
  };
}

// Get typed configuration
const apiConfig = config.get<AppConfig['services']['api']>('services.api');
const loggingLevel = config.get<AppConfig['logging']['level']>('logging.level', 'info');
```

## Integration with Services

### Logger Integration

```typescript
import { config, logger } from '@bluefly/llmcli/shared';

// Configure logger using shared configuration
const logLevel = config.get('logging.level', 'info');
const logFormat = config.get('logging.format', 'text');

logger.setLevel(logLevel);
logger.setFormat(logFormat);
```

### HTTP Client Integration

```typescript
import { config, HttpClient } from '@bluefly/llmcli/shared';

// Create HTTP client using configuration
const httpClient = new HttpClient({
  baseURL: config.get('services.api.url'),
  timeout: config.get('services.api.timeout', 5000),
  retries: config.get('services.api.retries', 3)
});
```

## Configuration Monitoring

The configuration system can notify you when configuration changes:

```typescript
// Subscribe to configuration changes
const unsubscribe = config.subscribe('services.api', (newValue, oldValue) => {
  console.log(`API configuration changed from ${JSON.stringify(oldValue)} to ${JSON.stringify(newValue)}`);
  
  // Update services that depend on this configuration
  updateApiClients(newValue);
});

// Later, unsubscribe when no longer needed
unsubscribe();
```

## Best Practices

1. **Environment Variables for Sensitive Data**: Use environment variables for sensitive configuration like API keys and secrets.

2. **Default Values**: Always provide sensible default values when getting configuration.

3. **Config Schema**: Define a TypeScript interface for your configuration to ensure type safety.

4. **Configuration Validation**: Validate configuration on startup:

```typescript
function validateConfig() {
  // Required fields
  const requiredFields = ['services.api.url', 'logging.level'];
  for (const field of requiredFields) {
    if (!config.has(field)) {
      throw new Error(`Missing required configuration: ${field}`);
    }
  }
  
  // Validate specific fields
  const apiTimeout = config.get('services.api.timeout');
  if (typeof apiTimeout !== 'number' || apiTimeout <= 0) {
    throw new Error('API timeout must be a positive number');
  }
}

// Call this on startup
validateConfig();
```

5. **Documentation**: Document all configuration options, their default values, and their purpose.

6. **Configuration File Templates**: Provide templates for configuration files with comments.

## Common Configuration Keys

The following configuration keys are standardized across LLM Platform projects:

### Logging

```
logging.level             - Log level (debug, info, warn, error)
logging.format            - Log format (json, text)
logging.timestamp         - Include timestamp in logs (true, false)
logging.colorize          - Colorize logs (true, false)
```

### Services

```
services.<name>.url       - Service URL
services.<name>.timeout   - Request timeout in milliseconds
services.<name>.retries   - Number of retries for failed requests
```

### Authentication

```
auth.tokenExpiration      - Token expiration time in seconds
auth.refreshEnabled       - Enable token refresh (true, false)
auth.refreshThreshold     - Threshold for token refresh in seconds
```

### Environment

```
environment               - Deployment environment (development, staging, production)
service.name              - Service name
service.version           - Service version
```

## Implementation Details

The configuration system is implemented in `@bluefly/llmcli/shared/config/index.ts`. Key components include:

- **SharedConfig Class**: Main class implementing the configuration system
- **ConfigValue Interface**: Stores a configuration value with metadata
- **ConfigOptions Interface**: Options for creating a configuration instance
- **ConfigSource Type**: Enum of possible configuration sources

## Conclusion

The shared configuration system provides a flexible, type-safe way to manage configuration across all LLM Platform projects. By centralizing configuration management, we ensure consistent configuration patterns and reduce duplication.
