---
id: shared-SHARED_UTILITIES_UPDATE
---
# Shared Utilities Update

## Overview

The LLM Platform shared utilities in `@bluefly/llmcli/shared` have been significantly expanded to provide a comprehensive set of utility modules for common operations. These utilities standardize common functionality across all LLM Platform projects and reduce code duplication.

## New Utility Modules

### Date Utilities

A robust set of date and time manipulation functions:

```typescript
import { dateUtils } from '@bluefly/llmcli/shared';

// Format a date
const formattedDate = dateUtils.formatDate(new Date(), 'yyyy-MM-dd HH:mm:ss');

// Add time
const tomorrow = dateUtils.addDays(new Date(), 1);
const nextHour = dateUtils.addHours(new Date(), 1);

// Calculate difference
const diffDays = dateUtils.diffInDays(startDate, endDate);

// Get relative time
const relativeTime = dateUtils.timeAgo(pastDate);
```

### String Utilities

Enhanced string manipulation utilities:

```typescript
import { stringUtils } from '@bluefly/llmcli/shared';

// Case transformations
const camelCased = stringUtils.camelCase('hello-world'); // 'helloWorld'
const kebabCased = stringUtils.kebabCase('helloWorld'); // 'hello-world'
const pascalCased = stringUtils.pascalCase('hello_world'); // 'HelloWorld'

// Formatting
const truncated = stringUtils.truncate('A very long string', 10); // 'A very lon...'
const slug = stringUtils.slugify('Hello, World!'); // 'hello-world'

// Validation
const isValid = stringUtils.isValidEmail('user@example.com');

// HTML handling
const escaped = stringUtils.escapeHtml('<script>alert("XSS")</script>');
```

### ID Utilities

Functions for generating various types of IDs:

```typescript
import { idUtils } from '@bluefly/llmcli/shared';

// Generate identifiers
const uuid = idUtils.generateUuid();
const shortId = idUtils.generateShortId(8);
const correlationId = idUtils.generateCorrelationId();
const requestId = idUtils.generateRequestId('api');

// Validation and formatting
const isValidUuid = idUtils.isUuid(id);
const formattedId = idUtils.formatIdForDisplay(longId, { 
  showFirst: 8, 
  showLast: 4, 
  placeholder: '...' 
});
```

### File System Utilities

Comprehensive file system operations:

```typescript
import { fsUtils } from '@bluefly/llmcli/shared';

// File operations
const exists = fsUtils.fileExists('/path/to/file.txt');
const size = fsUtils.getFileSize('/path/to/file.txt');
const content = fsUtils.readTextFile('/path/to/file.txt');
fsUtils.writeTextFile('/path/to/file.txt', 'Hello World');

// Directory operations
fsUtils.ensureDirectory('/path/to/dir');
const files = fsUtils.listFiles('/path/to/dir', { 
  recursive: true,
  filter: file => file.endsWith('.ts')
});

// Path operations
const normalized = fsUtils.normalizePath('/path//to/./file.txt');
const joined = fsUtils.joinPaths('/path', 'to', 'file.txt');
const extension = fsUtils.getFileExtension('file.txt');
```

### Network Utilities

Tools for working with URLs and network operations:

```typescript
import { networkUtils } from '@bluefly/llmcli/shared';

// URL operations
const url = networkUtils.buildUrl('https://api.example.com', { 
  q: 'search term', 
  page: 1 
});
const joined = networkUtils.joinUrlPaths('https://example.com', 'api', 'v1');
const params = networkUtils.parseUrlParams('https://example.com?q=search&page=2');

// URL parsing and analysis
const parsed = networkUtils.parseUrl('https://example.com/path?q=value#section');
const domain = networkUtils.getDomain('https://sub.example.com');
const isAbsolute = networkUtils.isAbsoluteUrl('/path/to/resource');
```

### Validation Utilities

Comprehensive data validation for various types and formats:

```typescript
import { validationUtils } from '@bluefly/llmcli/shared';

// String validation
const isValidEmail = validationUtils.string.isEmail('user@example.com');
const isValidUrl = validationUtils.string.isUrl('https://example.com');
const isValidUuid = validationUtils.string.isUuid('123e4567-e89b-12d3-a456-426614174000');

// Number validation
const isPositive = validationUtils.number.isPositive(42);
const inRange = validationUtils.number.inRange(25, 0, 100);

// Object validation
const result = validationUtils.object.matchesSchema(config, {
  baseUrl: { type: 'string', required: true },
  timeout: { type: 'number', min: 0 },
  retries: { type: 'number', min: 0, max: 10 }
});

// Array validation
const hasMinLength = validationUtils.array.minLength([1, 2, 3], 2);
const noDuplicates = validationUtils.array.noDuplicates([1, 2, 3, 4]);

// Custom validation
const passwordValidator = validationUtils.createValidator<string>(
  (password) => validationUtils.string.hasMinLength(password, 8),
  (password) => /[A-Z]/.test(password), // Has uppercase
  (password) => /[0-9]/.test(password)  // Has number
);

// Schema-based validation
const validateUser = validationUtils.object.matchesSchema(userData, userSchema);
```

## Integration with Existing Modules

The new utility modules are fully integrated with the existing shared utilities system:

```typescript
import { 
  stringUtils,
  dateUtils,
  idUtils,
  fsUtils,
  networkUtils,
  objectUtils,
  asyncUtils,
  arrayUtils,
  validationUtils
} from '@bluefly/llmcli/shared';
```

## Backward Compatibility

All new utilities maintain backward compatibility with existing implementations:

- Legacy methods like `stringUtils.toCamelCase()` continue to work alongside new methods
- Existing utility functions are preserved with identical behavior
- New functionality extends rather than replaces existing functionality

## Extended Validation

The utilities now include enhanced validation functions:

```typescript
// Email validation
const isValidEmail = stringUtils.isValidEmail('user@example.com');

// URL validation
const isValidUrl = stringUtils.isValidUrl('https://example.com');

// UUID validation
const isValidUuid = idUtils.isUuid('123e4567-e89b-12d3-a456-426614174000');

// Date validation
const isValidDate = dateUtils.isValidDate('2023-01-15');
```

## Asynchronous Utilities

Enhanced async utilities with promise handling:

```typescript
import { asyncUtils } from '@bluefly/llmcli/shared';

// Sleep/delay
await asyncUtils.sleep(1000);

// Retry with exponential backoff
const result = await asyncUtils.retry(
  () => fetchData(),
  3,  // max retries
  300, // base delay
  2,   // multiplier for exponential backoff
  (error, attempt) => error.status !== 404 // retry condition
);

// Timeout
const data = await asyncUtils.timeout(
  slowOperation(),
  5000,
  'Operation timed out'
);

// Debounce
const debouncedFn = asyncUtils.debounce(expensiveOperation, 300);
```

## Features and Benefits

1. **Comprehensive Utilities**: Cover common operations across string manipulation, date handling, ID generation, and more
2. **Type Safety**: Full TypeScript support with detailed type definitions
3. **Consistency**: Standardized implementations across all projects
4. **Performance**: Optimized implementations for common operations
5. **Reduced Bundle Size**: Centralized utilities reduce duplication and bundle size
6. **Improved Maintainability**: Single source of truth for utility functions
7. **Enhanced Developer Experience**: Intuitive API with comprehensive documentation

## Usage

To use these utilities in your project:

1. Make sure `@bluefly/llmcli` is in your dependencies
2. Import the utilities you need:

```typescript
import { 
  dateUtils, 
  stringUtils, 
  idUtils 
} from '@bluefly/llmcli/shared';
```

3. Use the utilities in your code:

```typescript
const formattedDate = dateUtils.formatDate(new Date(), 'yyyy-MM-dd');
const slug = stringUtils.slugify('Hello World!');
const id = idUtils.generateShortId();
```

## Best Practices

1. **Import Specific Utilities**: Import only the utilities you need to keep bundle size small
2. **Use ES Module Imports**: Always include the `.js` extension in import paths
3. **Prefer Dedicated Modules**: Use dedicated utility modules rather than general imports when possible
4. **Check Documentation**: Refer to the utility documentation for detailed usage examples

## Configuration

The utilities are configurable through the shared library's configuration system:

```typescript
import { config } from '@bluefly/llmcli/shared';

// Set configuration values
config.set('utils.date.defaultFormat', 'yyyy-MM-dd');
config.set('utils.id.defaultLength', 10);
```

## Next Steps

1. **Project Migration**: Continue migrating projects to use these shared utilities
2. **Documentation**: Further enhance documentation with more examples
3. **Testing**: Add more comprehensive tests for edge cases
4. **Community Feedback**: Gather feedback from developers using the utilities
5. **Performance Optimization**: Identify and optimize performance bottlenecks
