# LLM Platform Testing Guide

## Overview

This guide provides comprehensive testing strategies and patterns used across all LLM Platform projects. It emphasizes **Test-Driven Development (TDD)** with mandatory coverage requirements.

## Test-Driven Development (TDD) Workflow

### Red-Green-Refactor Cycle

**1. RED Phase - Write Failing Test**
```typescript
// Write test that describes desired behavior
it('should generate text with proper token limit', async () => {
  const result = await provider.generateText('Test prompt', {
    maxTokens: 100
  });
  
  expect(result.tokens_used).toBeLessThanOrEqual(100);
  expect(result.content).toBeDefined();
});
```

**2. GREEN Phase - Write Minimal Code**
```typescript
// Implement just enough to pass the test
async generateText(prompt: string, options: GenerationOptions) {
  const response = await this.client.completions.create({
    prompt,
    max_tokens: options.maxTokens || 1000
  });
  
  return {
    content: response.choices[0].text,
    tokens_used: response.usage.total_tokens
  };
}
```

**3. REFACTOR Phase - Improve Code Quality**
```typescript
// Enhance while keeping tests green
async generateText(prompt: string, options: GenerationOptions): Promise<AIResponse> {
  this.validatePrompt(prompt);
  const sanitizedOptions = this.sanitizeOptions(options);
  
  try {
    const response = await this.client.completions.create({
      prompt,
      ...this.mapOptions(sanitizedOptions)
    });
    
    return this.formatResponse(response);
  } catch (error) {
    throw this.handleProviderError(error);
  }
}
```

## Coverage Requirements

### Minimum Coverage Thresholds

**NPM Packages**: 95% minimum
```json
// jest.config.js
{
  "coverageThreshold": {
    "global": {
      "branches": 95,
      "functions": 95,
      "lines": 95,
      "statements": 95
    }
  }
}
```

**Drupal Modules**: 85% minimum
```xml
<!-- phpunit.xml -->
<coverage>
  <report>
    <html outputDirectory="coverage" lowUpperBound="85" highLowerBound="95"/>
  </report>
</coverage>
```

### Coverage Enforcement

```bash
# NPM Package CI Pipeline
npm run test:coverage
if [ $? -ne 0 ]; then
  echo "Coverage below 95% threshold"
  exit 1
fi

# Drupal Module Testing
vendor/bin/phpunit --coverage-html coverage
```

## Testing Patterns by Component Type

### NPM Package Testing (TypeScript/Jest)

#### Unit Tests
```typescript
// src/__tests__/unit/AIProvider.test.ts
import { AIProvider } from '../../AIProvider';
import { mockClient } from '../__mocks__/client';

describe('AIProvider', () => {
  let provider: AIProvider;
  
  beforeEach(() => {
    provider = new AIProvider({
      client: mockClient,
      apiKey: 'test-key'
    });
  });
  
  afterEach(() => {
    jest.clearAllMocks();
  });
  
  describe('generateText', () => {
    it('should handle successful generation', async () => {
      mockClient.completions.create.mockResolvedValue({
        choices: [{ text: 'Generated content' }],
        usage: { total_tokens: 50 }
      });
      
      const result = await provider.generateText('Test prompt');
      
      expect(result).toEqual({
        content: 'Generated content',
        tokens_used: 50,
        model: 'gpt-4',
        processing_time: expect.any(Number)
      });
    });
    
    it('should retry on transient errors', async () => {
      mockClient.completions.create
        .mockRejectedValueOnce(new Error('Rate limit'))
        .mockResolvedValueOnce({
          choices: [{ text: 'Success after retry' }],
          usage: { total_tokens: 30 }
        });
      
      const result = await provider.generateText('Test prompt');
      
      expect(mockClient.completions.create).toHaveBeenCalledTimes(2);
      expect(result.content).toBe('Success after retry');
    });
  });
});
```

#### Integration Tests
```typescript
// src/__tests__/integration/APIServer.test.ts
import request from 'supertest';
import { app } from '../../app';
import { setupTestDatabase, teardownTestDatabase } from '../test-utils';

describe('API Server Integration', () => {
  beforeAll(async () => {
    await setupTestDatabase();
  });
  
  afterAll(async () => {
    await teardownTestDatabase();
  });
  
  describe('POST /api/generate', () => {
    it('should generate content with valid request', async () => {
      const response = await request(app)
        .post('/api/generate')
        .send({
          prompt: 'Test prompt',
          options: {
            maxTokens: 100,
            temperature: 0.7
          }
        })
        .expect(200);
      
      expect(response.body).toMatchObject({
        content: expect.any(String),
        tokens_used: expect.any(Number),
        processing_time: expect.any(String)
      });
    });
    
    it('should validate request parameters', async () => {
      const response = await request(app)
        .post('/api/generate')
        .send({
          // Missing required prompt
          options: { maxTokens: 100 }
        })
        .expect(400);
      
      expect(response.body.error).toContain('prompt is required');
    });
  });
});
```

### Drupal Module Testing (PHPUnit)

#### Unit Tests
```php
<?php

namespace Drupal\Tests\llm\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\llm\Service\AIProviderService;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Tests the AI Provider Service.
 *
 * @group llm
 * @coversDefaultClass \Drupal\llm\Service\AIProviderService
 */
class AIProviderServiceTest extends UnitTestCase {

  /**
   * The AI provider service.
   *
   * @var \Drupal\llm\Service\AIProviderService
   */
  protected $aiProvider;

  /**
   * Mock config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    
    $this->configFactory = $this->createMock(ConfigFactoryInterface::class);
    $logger = $this->createMock(LoggerChannelFactoryInterface::class);
    
    $this->aiProvider = new AIProviderService(
      $this->configFactory,
      $logger
    );
  }

  /**
   * @covers ::generateText
   */
  public function testGenerateTextWithValidInput() {
    $config = $this->createMock('\Drupal\Core\Config\ImmutableConfig');
    $config->expects($this->any())
      ->method('get')
      ->willReturnMap([
        ['default_provider', 'openai'],
        ['providers.openai.api_key', 'test-key'],
      ]);
    
    $this->configFactory->expects($this->any())
      ->method('get')
      ->with('llm.settings')
      ->willReturn($config);
    
    // Mock HTTP client would be injected here
    $result = $this->aiProvider->generateText('Test prompt', [
      'max_tokens' => 100,
    ]);
    
    $this->assertIsArray($result);
    $this->assertArrayHasKey('content', $result);
  }

  /**
   * @covers ::generateText
   */
  public function testGenerateTextValidatesInput() {
    $this->expectException(\InvalidArgumentException::class);
    $this->expectExceptionMessage('Prompt cannot be empty');
    
    $this->aiProvider->generateText('', []);
  }
}
```

#### Kernel Tests
```php
<?php

namespace Drupal\Tests\llm\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\llm\Entity\AIConversation;

/**
 * Tests AI Conversation entity.
 *
 * @group llm
 */
class AIConversationEntityTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'llm',
    'user',
    'system',
    'field',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    
    $this->installEntitySchema('ai_conversation');
    $this->installEntitySchema('user');
    $this->installConfig(['llm']);
  }

  /**
   * Tests AI conversation creation.
   */
  public function testAIConversationCreation() {
    $conversation = AIConversation::create([
      'title' => 'Test Conversation',
      'prompt' => 'Test prompt',
      'response' => 'Test response',
      'model' => 'gpt-4',
      'tokens_used' => 100,
    ]);
    
    $conversation->save();
    
    $this->assertNotNull($conversation->id());
    $this->assertEquals('Test Conversation', $conversation->getTitle());
    $this->assertEquals(100, $conversation->get('tokens_used')->value);
  }

  /**
   * Tests conversation with field encryption.
   */
  public function testEncryptedFieldStorage() {
    $sensitive_prompt = 'Sensitive information here';
    
    $conversation = AIConversation::create([
      'title' => 'Encrypted Test',
      'prompt' => $sensitive_prompt,
      'field_encrypted_data' => 'Secret API response',
    ]);
    
    $conversation->save();
    
    // Reload to test decryption
    $loaded = AIConversation::load($conversation->id());
    $this->assertEquals($sensitive_prompt, $loaded->get('prompt')->value);
    $this->assertEquals('Secret API response', $loaded->get('field_encrypted_data')->value);
  }
}
```

#### Functional Tests
```php
<?php

namespace Drupal\Tests\llm\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the LLM administration interface.
 *
 * @group llm
 */
class LLMAdminInterfaceTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'llm',
    'user',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests admin configuration form.
   */
  public function testAdminConfigurationForm() {
    $admin_user = $this->drupalCreateUser(['administer llm']);
    $this->drupalLogin($admin_user);
    
    // Test form access
    $this->drupalGet('admin/config/llm/settings');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('LLM Settings');
    
    // Test form submission
    $edit = [
      'default_provider' => 'openai',
      'cache_ttl' => 3600,
      'token_tracking_enabled' => TRUE,
    ];
    
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
    
    // Verify saved values
    $config = $this->config('llm.settings');
    $this->assertEquals('openai', $config->get('default_provider'));
    $this->assertEquals(3600, $config->get('cache_ttl'));
    $this->assertTrue($config->get('token_tracking_enabled'));
  }
}
```

## Testing Best Practices

### Mock External Dependencies

```typescript
// NPM Package Mocking
jest.mock('openai', () => ({
  OpenAI: jest.fn().mockImplementation(() => ({
    completions: {
      create: jest.fn()
    }
  }))
}));

// Drupal Service Mocking
$http_client = $this->prophesize(ClientInterface::class);
$http_client->post(Argument::any(), Argument::any())
  ->willReturn(new Response(200, [], json_encode(['result' => 'success'])));
```

### Test Data Management

```typescript
// TypeScript Test Fixtures
export const testFixtures = {
  validPrompt: 'Generate a technical documentation outline',
  invalidPrompt: '',
  largePrompt: 'x'.repeat(10000),
  
  mockResponses: {
    success: {
      content: 'Generated content',
      tokens_used: 50,
      model: 'gpt-4'
    },
    error: {
      error: 'Rate limit exceeded',
      code: 'rate_limit'
    }
  }
};

// PHP Test Traits
trait AIProviderTestTrait {
  
  protected function createMockAIResponse(array $options = []): array {
    return $options + [
      'content' => 'Mock generated content',
      'tokens_used' => 100,
      'model' => 'gpt-4',
      'processing_time' => 1.5,
    ];
  }
  
  protected function createTestConversation(array $values = []): AIConversation {
    return AIConversation::create($values + [
      'title' => 'Test Conversation',
      'prompt' => 'Test prompt',
      'response' => 'Test response',
    ]);
  }
}
```

### Performance Testing

```typescript
// Jest Performance Testing
describe('Performance Tests', () => {
  it('should process requests within acceptable time', async () => {
    const startTime = performance.now();
    
    await provider.generateText('Test prompt', {
      maxTokens: 100
    });
    
    const duration = performance.now() - startTime;
    expect(duration).toBeLessThan(2000); // 2 second limit
  });
  
  it('should handle concurrent requests efficiently', async () => {
    const promises = Array(10).fill(null).map(() => 
      provider.generateText('Test prompt')
    );
    
    const startTime = performance.now();
    await Promise.all(promises);
    const duration = performance.now() - startTime;
    
    // Should not take 10x single request time
    expect(duration).toBeLessThan(5000);
  });
});
```

## CI/CD Testing Integration

### GitHub Actions / GitLab CI

```yaml
# .gitlab-ci.yml or .github/workflows/test.yml
test:
  stage: test
  script:
    # NPM Package Testing
    - npm ci
    - npm run lint
    - npm run typecheck
    - npm run test:coverage
    - npm run test:integration
    
    # Drupal Module Testing
    - composer install
    - vendor/bin/phpcs
    - vendor/bin/phpstan analyse
    - vendor/bin/phpunit --coverage-text
    
  coverage: '/^\s*Lines:\s*\d+.\d+\%/'
  artifacts:
    reports:
      coverage_report:
        coverage_format: cobertura
        path: coverage/cobertura-coverage.xml
```

### Pre-commit Hooks

```json
// .husky/pre-commit
#!/bin/sh
. "$(dirname "$0")/_/husky.sh"

# Run tests for changed files
npm run test:related -- --passWithNoTests
npm run lint-staged
```

## Testing Utilities

### Custom Assertions

```typescript
// TypeScript Custom Matchers
expect.extend({
  toBeValidAIResponse(received) {
    const pass = 
      received.content !== undefined &&
      received.tokens_used > 0 &&
      received.model !== undefined;
    
    return {
      pass,
      message: () => 
        `expected ${received} to be a valid AI response with content, tokens_used, and model`
    };
  }
});

// Usage
expect(result).toBeValidAIResponse();
```

### Test Helpers

```php
// Drupal Test Helper Service
namespace Drupal\llm\Tests;

class LLMTestHelper {
  
  public static function setupTestEnvironment(): void {
    // Configure test API keys
    \Drupal::configFactory()
      ->getEditable('llm.settings')
      ->set('providers.openai.api_key', 'test-key')
      ->set('providers.openai.endpoint', 'http://mock-api')
      ->save();
    
    // Clear caches
    drupal_flush_all_caches();
  }
  
  public static function mockAIProviderResponse($service_id, $response): void {
    $service = \Drupal::service($service_id);
    $reflection = new \ReflectionClass($service);
    $property = $reflection->getProperty('httpClient');
    $property->setAccessible(true);
    
    $mock_client = \Drupal::service('llm.test.mock_client');
    $mock_client->setResponse($response);
    $property->setValue($service, $mock_client);
  }
}
```

## Debugging Tests

### TypeScript/Jest Debugging

```json
// launch.json for VS Code
{
  "type": "node",
  "request": "launch",
  "name": "Jest Debug",
  "program": "${workspaceFolder}/node_modules/.bin/jest",
  "args": [
    "--runInBand",
    "--no-coverage",
    "${fileBasenameNoExtension}"
  ],
  "console": "integratedTerminal",
  "internalConsoleOptions": "neverOpen"
}
```

### PHP/PHPUnit Debugging

```bash
# Run specific test with verbose output
vendor/bin/phpunit \
  --verbose \
  --debug \
  --filter testAIConversationCreation \
  web/modules/custom/llm/tests/src/Kernel/AIConversationEntityTest.php

# Generate code coverage report
vendor/bin/phpunit \
  --coverage-html coverage \
  --coverage-filter src
```

## Testing Checklist

### Before Committing

- [ ] All tests pass locally
- [ ] Coverage meets minimum threshold (95% NPM, 85% Drupal)
- [ ] No linting errors
- [ ] TypeScript compilation succeeds
- [ ] Integration tests pass
- [ ] Performance benchmarks met

### Test Review Criteria

- [ ] Tests follow TDD red-green-refactor cycle
- [ ] Edge cases and error conditions covered
- [ ] Mocks used appropriately for external dependencies
- [ ] Tests are readable and self-documenting
- [ ] No test interdependencies
- [ ] Reasonable execution time (< 10 seconds for unit tests)

## Common Testing Pitfalls

### Avoid These Anti-patterns

1. **Testing Implementation Instead of Behavior**
   ```typescript
   // Bad: Tests internal implementation
   expect(provider._sanitizeOptions).toHaveBeenCalled();
   
   // Good: Tests observable behavior
   expect(result.content).not.toContain('<script>');
   ```

2. **Overmocking**
   ```typescript
   // Bad: Mocking everything
   jest.mock('../entire-module');
   
   // Good: Mock only external dependencies
   jest.mock('openai');
   ```

3. **Ignoring Async Behavior**
   ```typescript
   // Bad: Not waiting for async operations
   provider.generateText('test');
   expect(cache.get('key')).toBeDefined();
   
   // Good: Properly handling async
   await provider.generateText('test');
   expect(cache.get('key')).toBeDefined();
   ```

This testing guide ensures consistent, high-quality testing practices across all LLM Platform projects. Remember: **No code without tests, no tests without code.**