/**
 * Validation Utilities Examples
 * 
 * This file contains practical examples of using the validation utilities
 * in various real-world scenarios.
 */

import { 
  validationUtils, 
  createError,
  logger 
} from '@bluefly/llmcli/shared';

// ----------------------------------------
// Example 1: Form Validation
// ----------------------------------------

interface UserFormData {
  username: string;
  email: string;
  password: string;
  age?: number;
  role?: string;
}

/**
 * Validates a user registration form
 */
function validateUserForm(formData: UserFormData) {
  const schema = {
    username: { 
      type: 'string', 
      required: true, 
      minLength: 3,
      maxLength: 20,
      pattern: /^[a-zA-Z0-9_-]+$/,
      message: 'Username must be 3-20 characters and only contain letters, numbers, underscores, and dashes'
    },
    email: { 
      type: 'string', 
      required: true,
      custom: (value: string) => ({
        valid: validationUtils.string.isEmail(value),
        errors: validationUtils.string.isEmail(value) ? undefined : ['Invalid email format']
      })
    },
    password: { 
      type: 'string', 
      required: true,
      minLength: 8,
      custom: (value: string) => {
        const hasUppercase = /[A-Z]/.test(value);
        const hasNumber = /[0-9]/.test(value);
        const hasSpecial = /[^A-Za-z0-9]/.test(value);
        
        const errors = [];
        if (!hasUppercase) errors.push('Password must contain at least one uppercase letter');
        if (!hasNumber) errors.push('Password must contain at least one number');
        if (!hasSpecial) errors.push('Password must contain at least one special character');
        
        return {
          valid: hasUppercase && hasNumber && hasSpecial,
          errors: errors.length ? errors : undefined
        };
      }
    },
    age: { 
      type: 'number', 
      required: false,
      min: 13,
      max: 120,
      message: 'Age must be between 13 and 120'
    },
    role: { 
      type: 'string', 
      required: false,
      enum: ['user', 'admin', 'editor'],
      message: 'Role must be one of: user, admin, editor'
    }
  };
  
  return validationUtils.object.matchesSchema(formData, schema);
}

// Usage
const formData: UserFormData = {
  username: 'john_doe',
  email: 'john.doe@example.com',
  password: 'P@ssw0rd',
  age: 30,
  role: 'user'
};

const validationResult = validateUserForm(formData);
if (!validationResult.valid) {
  console.error('Form validation failed:', validationResult.errors);
} else {
  console.log('Form is valid');
}

// ----------------------------------------
// Example 2: API Configuration Validation
// ----------------------------------------

interface ApiConfig {
  baseUrl: string;
  version: string;
  timeout?: number;
  retries?: number;
  authentication?: {
    type: 'apiKey' | 'oauth' | 'jwt';
    key?: string;
    token?: string;
  };
  headers?: Record<string, string>;
}

/**
 * Validates API configuration
 */
function validateApiConfig(config: ApiConfig) {
  // First level validation
  const result = validationUtils.object.matchesSchema(config, {
    baseUrl: { 
      type: 'string', 
      required: true,
      custom: (value: string) => ({
        valid: validationUtils.string.isUrl(value),
        errors: validationUtils.string.isUrl(value) ? undefined : ['Must be a valid URL']
      })
    },
    version: { 
      type: 'string', 
      required: true,
      custom: (value: string) => ({
        valid: validationUtils.string.isSemver(value),
        errors: validationUtils.string.isSemver(value) ? undefined : ['Must be a valid semantic version (e.g., 1.0.0)']
      })
    },
    timeout: { 
      type: 'number', 
      required: false,
      min: 0,
      message: 'Timeout must be a positive number'
    },
    retries: { 
      type: 'number', 
      required: false,
      min: 0,
      max: 10,
      message: 'Retries must be between 0 and 10'
    },
    authentication: { 
      type: 'object', 
      required: false
    },
    headers: { 
      type: 'object', 
      required: false
    }
  });
  
  if (!result.valid) {
    return result;
  }
  
  // Additional validation for authentication if present
  if (config.authentication) {
    const authResult = validationUtils.object.matchesSchema(config.authentication, {
      type: { 
        type: 'string', 
        required: true,
        enum: ['apiKey', 'oauth', 'jwt'],
        message: 'Authentication type must be one of: apiKey, oauth, jwt'
      },
      key: { 
        type: 'string', 
        required: config.authentication.type === 'apiKey'
      },
      token: { 
        type: 'string', 
        required: config.authentication.type === 'oauth' || config.authentication.type === 'jwt'
      }
    });
    
    if (!authResult.valid) {
      return {
        valid: false,
        errors: (result.errors || []).concat(
          (authResult.errors || []).map(err => `authentication: ${err}`)
        )
      };
    }
    
    // Validate token format if present
    if (config.authentication.token) {
      if (config.authentication.type === 'jwt' && 
          !validationUtils.domain.isJwt(config.authentication.token)) {
        return {
          valid: false,
          errors: (result.errors || []).concat(['authentication.token: Not a valid JWT token'])
        };
      }
    }
  }
  
  return { valid: true };
}

// Usage
const apiConfig: ApiConfig = {
  baseUrl: 'https://api.example.com',
  version: '1.0.0',
  timeout: 5000,
  retries: 3,
  authentication: {
    type: 'apiKey',
    key: 'api_key_12345'
  },
  headers: {
    'Content-Type': 'application/json'
  }
};

try {
  const configValidation = validateApiConfig(apiConfig);
  if (!configValidation.valid) {
    throw createError(
      `Invalid API configuration: ${configValidation.errors?.join('; ')}`,
      'CONFIG_VALIDATION_ERROR'
    );
  }
  
  console.log('API configuration is valid');
} catch (error) {
  logger.error('Configuration error', { error });
}

// ----------------------------------------
// Example 3: Custom Validators
// ----------------------------------------

/**
 * Credit card validation
 */
const validateCreditCard = validationUtils.createValidator<string>(
  // Check if it's a string
  (value) => typeof value === 'string',
  
  // Check if it contains only digits
  (value) => /^\d+$/.test(value),
  
  // Check length (most cards are 13-19 digits)
  (value) => value.length >= 13 && value.length <= 19,
  
  // Luhn algorithm validation
  (value) => {
    let sum = 0;
    let shouldDouble = false;
    
    // Loop from right to left
    for (let i = value.length - 1; i >= 0; i--) {
      let digit = parseInt(value.charAt(i));
      
      if (shouldDouble) {
        digit *= 2;
        if (digit > 9) digit -= 9;
      }
      
      sum += digit;
      shouldDouble = !shouldDouble;
    }
    
    return (sum % 10) === 0;
  }
);

// Usage
function processPayment(cardNumber: string, amount: number) {
  // Validate card number
  const cardValidation = validateCreditCard(cardNumber);
  
  if (!cardValidation.valid) {
    throw createError(
      'Invalid credit card number', 
      'PAYMENT_VALIDATION_ERROR'
    );
  }
  
  // Validate amount
  validationUtils.assertValid(
    amount,
    [
      (val) => validationUtils.number.isNumber(val),
      (val) => validationUtils.number.isPositive(val)
    ],
    'Payment amount must be a positive number'
  );
  
  // Process payment...
  console.log(`Processing payment of ${amount} with card ${cardNumber.slice(-4).padStart(cardNumber.length, '*')}`);
}

// ----------------------------------------
// Example 4: Array of Objects Validation
// ----------------------------------------

interface Product {
  id: string;
  name: string;
  price: number;
  category: string;
  inStock: boolean;
}

const productSchema = {
  id: { 
    type: 'string', 
    required: true,
    custom: (value: string) => ({
      valid: validationUtils.string.isUuid(value),
      errors: validationUtils.string.isUuid(value) ? undefined : ['Product ID must be a valid UUID']
    })
  },
  name: { 
    type: 'string', 
    required: true,
    minLength: 3,
    maxLength: 100
  },
  price: { 
    type: 'number', 
    required: true,
    min: 0.01
  },
  category: { 
    type: 'string', 
    required: true,
    enum: ['electronics', 'clothing', 'food', 'books', 'other']
  },
  inStock: { 
    type: 'boolean', 
    required: true
  }
};

/**
 * Validates an array of products
 */
function validateProductList(products: Product[]) {
  // Basic array validation
  if (!validationUtils.array.isArray(products)) {
    return { 
      valid: false, 
      errors: ['Products must be an array'] 
    };
  }
  
  // Check if array is empty
  if (!validationUtils.array.minLength(products, 1)) {
    return { 
      valid: false, 
      errors: ['Product list cannot be empty'] 
    };
  }
  
  // Validate each product against the schema
  return validationUtils.array.validateObjects(products, productSchema);
}

// Usage
const products: Product[] = [
  {
    id: '123e4567-e89b-12d3-a456-426614174000',
    name: 'Smartphone',
    price: 699.99,
    category: 'electronics',
    inStock: true
  },
  {
    id: '223e4567-e89b-12d3-a456-426614174001',
    name: 'T-shirt',
    price: 19.99,
    category: 'clothing',
    inStock: true
  }
];

const productsValidation = validateProductList(products);
if (!productsValidation.valid) {
  console.error('Product list validation failed:', productsValidation.errors);
} else {
  console.log('Product list is valid');
}

// ----------------------------------------
// Example 5: Integration with API Error Handling
// ----------------------------------------

interface ApiRequest {
  endpoint: string;
  method: 'GET' | 'POST' | 'PUT' | 'DELETE';
  data?: any;
  params?: Record<string, string>;
  headers?: Record<string, string>;
}

/**
 * Validate and process an API request
 */
async function processApiRequest(request: ApiRequest) {
  // Validate request
  const result = validationUtils.object.matchesSchema(request, {
    endpoint: { 
      type: 'string', 
      required: true
    },
    method: { 
      type: 'string', 
      required: true,
      enum: ['GET', 'POST', 'PUT', 'DELETE']
    },
    data: { 
      required: false
    },
    params: { 
      type: 'object', 
      required: false
    },
    headers: { 
      type: 'object', 
      required: false
    }
  });
  
  // Handle validation errors
  if (!result.valid) {
    throw createError(
      `Invalid API request: ${result.errors?.join('; ')}`,
      'INVALID_REQUEST',
      { statusCode: 400 }
    );
  }
  
  // Process request based on method
  switch (request.method) {
    case 'GET':
      // Validate URL params if present
      if (request.params) {
        // Additional validation specific to GET params could be done here
      }
      console.log(`Processing GET request to ${request.endpoint}`);
      break;
      
    case 'POST':
    case 'PUT':
      // Data is typically required for POST/PUT
      if (!request.data) {
        throw createError(
          'Request data is required for POST/PUT requests',
          'MISSING_DATA',
          { statusCode: 400 }
        );
      }
      
      // Validate data based on endpoint
      // This would typically call another validation function specific to the endpoint
      console.log(`Processing ${request.method} request to ${request.endpoint}`);
      break;
      
    case 'DELETE':
      console.log(`Processing DELETE request to ${request.endpoint}`);
      break;
  }
  
  // Return mock response
  return {
    status: 200,
    data: { success: true }
  };
}

// Example usage
async function makeApiCall() {
  try {
    const response = await processApiRequest({
      endpoint: '/users',
      method: 'POST',
      data: {
        username: 'johndoe',
        email: 'john@example.com'
      },
      headers: {
        'Content-Type': 'application/json'
      }
    });
    
    console.log('API call successful:', response);
  } catch (error) {
    logger.error('API call failed', { error });
  }
}