/**
 * TDD Test Suite for Model API
 * 
 * RED PHASE: Write failing tests first
 * GREEN PHASE: Implement minimal code to pass tests
 * REFACTOR PHASE: Improve code quality while keeping tests green
 * 
 * TODO: Implementation tasks:
 * - [ ] Add tests for model creation API endpoint
 * - [ ] Test model update functionality
 * - [ ] Implement tests for model version management
 * - [ ] Create tests for bulk operations
 * - [ ] Add performance tests for model queries
 * - [ ] Test advanced filtering and search
 * - [ ] Ensure 95%+ test coverage for all API endpoints
 */

const request = require('supertest');
const app = require('../src/index');
const jwt = require('jsonwebtoken');
const { jwtSecret } = require('../src/config/auth');

// Mock the database so we don't need a real connection
jest.mock('../src/config/database', () => ({
  pool: {
    query: jest.fn(),
    getConnection: jest.fn().mockReturnValue({
      query: jest.fn(),
      beginTransaction: jest.fn(),
      commit: jest.fn(),
      rollback: jest.fn(),
      release: jest.fn()
    })
  },
  testConnection: jest.fn().mockResolvedValue(true)
}));

// Import the mocked database
const { pool } = require('../src/config/database');

// Generate a valid JWT token for testing
const generateToken = () => {
  return jwt.sign(
    { id: 1, email: 'test@example.com', role: 'admin' },
    jwtSecret,
    { expiresIn: '1h' }
  );
};

describe('Model API', () => {
  let token;
  
  beforeEach(() => {
    // Reset mock calls
    jest.clearAllMocks();
    
    // Generate a new token for each test
    token = generateToken();
  });
  
  describe('GET /v1/models', () => {
    it('should return a list of models', async () => {
      // Mock the database response
      const mockModels = [
        {
          id: 1,
          name: 'gpt-4',
          provider: 'openai',
          base_model: 'gpt-4',
          description: 'GPT-4 model with improved reasoning',
          capabilities: JSON.stringify({
            vision: true,
            streaming: true,
            function_calling: true
          }),
          is_active: true,
          created_at: new Date('2025-01-15T12:30:45Z'),
          updated_at: new Date('2025-03-22T09:15:20Z')
        },
        {
          id: 2,
          name: 'claude-3-sonnet',
          provider: 'anthropic',
          base_model: 'claude-3',
          description: 'Claude 3 Sonnet model',
          capabilities: JSON.stringify({
            vision: true,
            streaming: true,
            function_calling: true
          }),
          is_active: true,
          created_at: new Date('2025-02-10T14:25:30Z'),
          updated_at: new Date('2025-04-05T11:40:15Z')
        }
      ];
      
      // Mock count query
      pool.query.mockResolvedValueOnce([[{ total: 2 }]]);
      
      // Mock models query
      pool.query.mockResolvedValueOnce([mockModels]);
      
      // Make the request
      const response = await request(app)
        .get('/v1/models')
        .set('Authorization', `Bearer ${token}`);
      
      // Assert the response
      expect(response.status).toBe(200);
      expect(response.body).toHaveProperty('models');
      expect(response.body).toHaveProperty('pagination');
      expect(response.body.models).toHaveLength(2);
      expect(response.body.models[0].name).toBe('gpt-4');
      expect(response.body.models[1].name).toBe('claude-3-sonnet');
      expect(response.body.pagination.total).toBe(2);
    });
    
    it('should return 401 if no token is provided', async () => {
      const response = await request(app).get('/v1/models');
      
      expect(response.status).toBe(401);
      expect(response.body).toHaveProperty('error', 'unauthorized');
    });
    
    it('should filter models by provider', async () => {
      // Mock count query
      pool.query.mockResolvedValueOnce([[{ total: 1 }]]);
      
      // Mock models query
      pool.query.mockResolvedValueOnce([[
        {
          id: 1,
          name: 'gpt-4',
          provider: 'openai',
          base_model: 'gpt-4',
          description: 'GPT-4 model with improved reasoning',
          capabilities: JSON.stringify({
            vision: true,
            streaming: true,
            function_calling: true
          }),
          is_active: true,
          created_at: new Date('2025-01-15T12:30:45Z'),
          updated_at: new Date('2025-03-22T09:15:20Z')
        }
      ]]);
      
      // Make the request
      const response = await request(app)
        .get('/v1/models?provider=openai')
        .set('Authorization', `Bearer ${token}`);
      
      // Assert the response
      expect(response.status).toBe(200);
      expect(response.body.models).toHaveLength(1);
      expect(response.body.models[0].provider).toBe('openai');
      
      // Verify that the query includes the provider filter
      expect(pool.query).toHaveBeenCalledWith(
        expect.stringContaining('WHERE provider = ?'),
        expect.arrayContaining(['openai'])
      );
    });
  });
  
  describe('GET /v1/models/:id', () => {
    it('should return a model by ID', async () => {
      // Mock the database response
      pool.query.mockResolvedValueOnce([[
        {
          id: 1,
          name: 'gpt-4',
          provider: 'openai',
          base_model: 'gpt-4',
          description: 'GPT-4 model with improved reasoning',
          capabilities: JSON.stringify({
            vision: true,
            streaming: true,
            function_calling: true
          }),
          is_active: true,
          created_at: new Date('2025-01-15T12:30:45Z'),
          updated_at: new Date('2025-03-22T09:15:20Z')
        }
      ]]);
      
      // Make the request
      const response = await request(app)
        .get('/v1/models/1')
        .set('Authorization', `Bearer ${token}`);
      
      // Assert the response
      expect(response.status).toBe(200);
      expect(response.body.id).toBe(1);
      expect(response.body.name).toBe('gpt-4');
      expect(response.body.provider).toBe('openai');
      
      // Verify that the query includes the ID
      expect(pool.query).toHaveBeenCalledWith(
        expect.stringContaining('WHERE id = ?'),
        expect.arrayContaining([1])
      );
    });
    
    it('should return 404 if model is not found', async () => {
      // Mock empty response
      pool.query.mockResolvedValueOnce([[]]);
      
      // Make the request
      const response = await request(app)
        .get('/v1/models/999')
        .set('Authorization', `Bearer ${token}`);
      
      // Assert the response
      expect(response.status).toBe(404);
      expect(response.body).toHaveProperty('error', 'not_found');
    });
  });
});

describe('Deployment API', () => {
  /**
   * TODO: Implementation tasks:
   * - [ ] Add tests for deployment update API endpoint
   * - [ ] Test deployment status transitions
   * - [ ] Implement tests for deployment scaling
   * - [ ] Create tests for deployment metrics
   * - [ ] Add validation tests for deployment configurations
   * - [ ] Test deployment health monitoring
   * - [ ] Ensure 95%+ test coverage for all deployment endpoints
   */
  let token;
  
  beforeEach(() => {
    // Reset mock calls
    jest.clearAllMocks();
    
    // Generate a new token for each test
    token = generateToken();
  });
  
  describe('POST /v1/deployments', () => {
    it('should create a new deployment', async () => {
      // Mock database connection
      const mockConnection = {
        beginTransaction: jest.fn(),
        query: jest.fn(),
        commit: jest.fn(),
        rollback: jest.fn(),
        release: jest.fn()
      };
      
      pool.getConnection.mockResolvedValue(mockConnection);
      
      // Mock model query
      mockConnection.query.mockResolvedValueOnce([[
        {
          id: 1,
          name: 'gpt-4',
          provider: 'openai',
          base_model: 'gpt-4'
        }
      ]]);
      
      // Mock model version query
      mockConnection.query.mockResolvedValueOnce([[
        {
          id: 3,
          model_id: 1,
          version: '1.0'
        }
      ]]);
      
      // Mock check for existing deployment
      mockConnection.query.mockResolvedValueOnce([[]]);
      
      // Mock insert query
      mockConnection.query.mockResolvedValueOnce([{ insertId: 42 }]);
      
      // Mock get deployment query
      mockConnection.query.mockResolvedValueOnce([[
        {
          id: 42,
          name: 'production-gpt4',
          model_id: 1,
          model_version_id: 3,
          status: 'provisioning',
          environment: 'production',
          endpoint: 'https://api.bluefly.io/v1/inference/deployments/production-gpt4',
          scaling: JSON.stringify({
            min_replicas: 2,
            max_replicas: 10,
            target_cpu_utilization: 70
          }),
          config: JSON.stringify({
            timeout_seconds: 60,
            max_tokens: 8192,
            temperature: 0.7
          }),
          created_at: new Date('2025-05-30T15:45:30Z'),
          created_by: 'test@example.com'
        }
      ]]);
      
      // Request body
      const deploymentData = {
        name: 'production-gpt4',
        model_id: 1,
        model_version_id: 3,
        environment: 'production',
        scaling: {
          min_replicas: 2,
          max_replicas: 10,
          target_cpu_utilization: 70
        },
        config: {
          timeout_seconds: 60,
          max_tokens: 8192,
          temperature: 0.7
        }
      };
      
      // Make the request
      const response = await request(app)
        .post('/v1/deployments')
        .set('Authorization', `Bearer ${token}`)
        .send(deploymentData);
      
      // Assert the response
      expect(response.status).toBe(201);
      expect(response.body.id).toBe(42);
      expect(response.body.name).toBe('production-gpt4');
      expect(response.body.model_id).toBe(1);
      expect(response.body.model_version_id).toBe(3);
      expect(response.body.status).toBe('provisioning');
      
      // Verify transaction was committed
      expect(mockConnection.beginTransaction).toHaveBeenCalled();
      expect(mockConnection.commit).toHaveBeenCalled();
      expect(mockConnection.release).toHaveBeenCalled();
    });
    
    it('should return 400 if validation fails', async () => {
      // Request with missing required fields
      const invalidData = {
        name: 'production-gpt4',
        // Missing model_id, model_version_id, etc.
      };
      
      // Make the request
      const response = await request(app)
        .post('/v1/deployments')
        .set('Authorization', `Bearer ${token}`)
        .send(invalidData);
      
      // Assert the response
      expect(response.status).toBe(400);
      expect(response.body).toHaveProperty('error', 'validation_error');
      expect(response.body).toHaveProperty('details');
    });
  });
});