const deploymentService = require('../services/deploymentService');
const logger = require('../utils/logger');

/**
 * Controller for deployment-related endpoints
 */
class DeploymentController {
  /**
   * Create a new deployment
   * @param {Object} req - Express request object
   * @param {Object} res - Express response object
   * @param {Function} next - Express next middleware function
   */
  async createDeployment(req, res, next) {
    try {
      const deploymentData = req.body;
      
      // Log the request
      logger.info('Creating new deployment', {
        name: deploymentData.name,
        model_id: deploymentData.model_id,
        environment: deploymentData.environment,
        user: req.user.email,
        requestId: req.headers['x-request-id'] || 'unknown'
      });
      
      // Create deployment using service
      const deployment = await deploymentService.createDeployment(
        deploymentData,
        req.user.email
      );
      
      // Return the created deployment with 201 status
      res.status(201).json(deployment);
    } catch (error) {
      next(error);
    }
  }
  
  /**
   * Get a deployment by ID
   * @param {Object} req - Express request object
   * @param {Object} res - Express response object
   * @param {Function} next - Express next middleware function
   */
  async getDeploymentById(req, res, next) {
    try {
      const { id } = req.params;
      
      // Log the request
      logger.info(`Retrieving deployment with ID ${id}`, {
        id,
        user: req.user.email,
        requestId: req.headers['x-request-id'] || 'unknown'
      });
      
      // Get deployment from service
      const deployment = await deploymentService.getDeploymentById(parseInt(id, 10));
      
      // Return the deployment
      res.status(200).json(deployment);
    } catch (error) {
      next(error);
    }
  }
  
  /**
   * Get a list of deployments with optional filtering
   * @param {Object} req - Express request object
   * @param {Object} res - Express response object
   * @param {Function} next - Express next middleware function
   */
  async getDeployments(req, res, next) {
    try {
      const { status, environment, model_id, limit, offset } = req.query;
      
      // Log the request
      logger.info('Retrieving deployments', {
        status,
        environment,
        model_id,
        limit,
        offset,
        user: req.user.email,
        requestId: req.headers['x-request-id'] || 'unknown'
      });
      
      // Get deployments from service
      const result = await deploymentService.getDeployments({
        status,
        environment,
        model_id: model_id ? parseInt(model_id, 10) : undefined,
        limit: limit ? parseInt(limit, 10) : 20,
        offset: offset ? parseInt(offset, 10) : 0
      });
      
      // Return the result
      res.status(200).json(result);
    } catch (error) {
      next(error);
    }
  }
}

module.exports = new DeploymentController();