const modelService = require('../services/modelService');
const logger = require('../utils/logger');

/**
 * Controller for model-related endpoints
 */
class ModelController {
  /**
   * Get a list of models with optional filtering
   * @param {Object} req - Express request object
   * @param {Object} res - Express response object
   * @param {Function} next - Express next middleware function
   */
  async getModels(req, res, next) {
    try {
      const { provider, is_active, limit, offset } = req.query;
      
      // Log the request
      logger.info('Retrieving models', {
        provider,
        is_active,
        limit,
        offset,
        user: req.user ? req.user.email : 'anonymous',
        requestId: req.headers['x-request-id'] || 'unknown'
      });
      
      // Get models from service
      const result = await modelService.getModels({
        provider,
        is_active,
        limit: limit ? parseInt(limit, 10) : 20,
        offset: offset ? parseInt(offset, 10) : 0
      });
      
      // Return the result
      res.status(200).json(result);
    } catch (error) {
      next(error);
    }
  }
  
  /**
   * Get a model by ID
   * @param {Object} req - Express request object
   * @param {Object} res - Express response object
   * @param {Function} next - Express next middleware function
   */
  async getModelById(req, res, next) {
    try {
      const { id } = req.params;
      
      // Log the request
      logger.info(`Retrieving model with ID ${id}`, {
        id,
        user: req.user ? req.user.email : 'anonymous',
        requestId: req.headers['x-request-id'] || 'unknown'
      });
      
      // Get model from service
      const model = await modelService.getModelById(parseInt(id, 10));
      
      // Return the model
      res.status(200).json(model);
    } catch (error) {
      next(error);
    }
  }
}

module.exports = new ModelController();