/**
 * Model Registry API - Main Application Entry Point
 * 
 * TODO: Implementation tasks:
 * - [ ] Implement database migrations system
 * - [ ] Add monitoring and health check endpoints
 * - [ ] Set up CI/CD pipeline for automated testing and deployment
 * - [ ] Implement caching for frequently accessed data
 * - [ ] Add rate limiting middleware
 * - [ ] Create API documentation using OpenAPI/Swagger
 * - [ ] Implement performance metrics collection
 */

const express = require('express');
const cors = require('cors');
const helmet = require('helmet');
const dotenv = require('dotenv');
const path = require('path');
const fs = require('fs');

// Load environment variables
dotenv.config();

// Import routes
const modelRoutes = require('./routes/modelRoutes');
const deploymentRoutes = require('./routes/deploymentRoutes');
const authRoutes = require('./routes/authRoutes');

// Import middleware
const { errorHandler, notFoundHandler } = require('./middleware/errorHandler');

// Import database
const { testConnection } = require('./config/database');

// Import logger
const logger = require('./utils/logger');

// Create Express application
const app = express();

// Create logs directory if it doesn't exist
const logsDir = path.join(__dirname, '../logs');
if (!fs.existsSync(logsDir)) {
  fs.mkdirSync(logsDir, { recursive: true });
}

// Set up middleware
app.use(helmet()); // Security headers
app.use(cors()); // CORS
app.use(express.json()); // Parse JSON bodies
app.use(express.urlencoded({ extended: true })); // Parse URL-encoded bodies

// Add request ID middleware
app.use((req, res, next) => {
  req.headers['x-request-id'] = req.headers['x-request-id'] || `req-${Date.now()}-${Math.random().toString(36).substring(2, 15)}`;
  next();
});

// Add basic logging middleware
app.use((req, res, next) => {
  logger.info(`${req.method} ${req.url}`, {
    method: req.method,
    url: req.url,
    ip: req.ip,
    requestId: req.headers['x-request-id']
  });
  next();
});

// Health check endpoint
app.get('/health', (req, res) => {
  res.status(200).json({ status: 'ok', version: '1.0.0' });
});

// API routes
app.use('/v1/auth', authRoutes);
app.use('/v1/models', modelRoutes);
app.use('/v1/deployments', deploymentRoutes);

// Not found handler
app.use(notFoundHandler);

// Error handler
app.use(errorHandler);

// Start the server
const PORT = process.env.PORT || 3000;
app.listen(PORT, async () => {
  logger.info(`Server running on port ${PORT}`);
  
  // Test database connection
  const dbConnected = await testConnection();
  if (dbConnected) {
    logger.info('Database connection successful');
  } else {
    logger.error('Database connection failed');
  }
});

// Handle unhandled promise rejections
process.on('unhandledRejection', (reason, promise) => {
  logger.error('Unhandled Rejection at:', promise, 'reason:', reason);
  // For debugging in development, you might want to crash:
  // process.exit(1);
});

// Handle uncaught exceptions
process.on('uncaughtException', (error) => {
  logger.error('Uncaught Exception:', error);
  // For debugging in development, you might want to crash:
  // process.exit(1);
});

module.exports = app; // Export for testing