const { validationResult, checkSchema } = require('express-validator');
const { ValidationError } = require('./errorHandler');
const logger = require('../utils/logger');

// Schemas for request validation
const schemas = {
  // Schema for GET /models query parameters
  listModels: {
    provider: {
      in: ['query'],
      optional: true,
      isString: {
        errorMessage: 'Provider must be a string'
      }
    },
    is_active: {
      in: ['query'],
      optional: true,
      isBoolean: {
        errorMessage: 'is_active must be a boolean'
      },
      toBoolean: true
    },
    limit: {
      in: ['query'],
      optional: true,
      isInt: {
        options: { min: 1, max: 100 },
        errorMessage: 'Limit must be an integer between 1 and 100'
      },
      toInt: true
    },
    offset: {
      in: ['query'],
      optional: true,
      isInt: {
        options: { min: 0 },
        errorMessage: 'Offset must be a non-negative integer'
      },
      toInt: true
    }
  },
  
  // Schema for POST /deployments request body
  createDeployment: {
    name: {
      in: ['body'],
      isString: {
        errorMessage: 'Name must be a string'
      },
      isLength: {
        options: { min: 1, max: 100 },
        errorMessage: 'Name must be between 1 and 100 characters'
      },
      matches: {
        options: /^[a-zA-Z0-9-]+$/,
        errorMessage: 'Name must contain only alphanumeric characters and hyphens'
      }
    },
    model_id: {
      in: ['body'],
      isInt: {
        errorMessage: 'model_id must be an integer'
      },
      toInt: true
    },
    model_version_id: {
      in: ['body'],
      isInt: {
        errorMessage: 'model_version_id must be an integer'
      },
      toInt: true
    },
    environment: {
      in: ['body'],
      isString: {
        errorMessage: 'Environment must be a string'
      },
      isIn: {
        options: [['development', 'staging', 'production']],
        errorMessage: 'Environment must be one of: development, staging, production'
      }
    },
    scaling: {
      in: ['body'],
      isObject: {
        errorMessage: 'Scaling must be an object'
      },
      custom: {
        options: (value) => {
          if (!value.min_replicas || !value.max_replicas || !value.target_cpu_utilization) {
            throw new Error('Scaling must include min_replicas, max_replicas, and target_cpu_utilization');
          }
          if (typeof value.min_replicas !== 'number' || value.min_replicas < 1) {
            throw new Error('min_replicas must be a positive integer');
          }
          if (typeof value.max_replicas !== 'number' || value.max_replicas < value.min_replicas) {
            throw new Error('max_replicas must be a number greater than or equal to min_replicas');
          }
          if (typeof value.target_cpu_utilization !== 'number' || value.target_cpu_utilization < 1 || value.target_cpu_utilization > 100) {
            throw new Error('target_cpu_utilization must be a number between 1 and 100');
          }
          return true;
        }
      }
    },
    config: {
      in: ['body'],
      isObject: {
        errorMessage: 'Config must be an object'
      },
      custom: {
        options: (value) => {
          if (!value.timeout_seconds || !value.max_tokens || value.temperature === undefined) {
            throw new Error('Config must include timeout_seconds, max_tokens, and temperature');
          }
          if (typeof value.timeout_seconds !== 'number' || value.timeout_seconds < 1) {
            throw new Error('timeout_seconds must be a positive integer');
          }
          if (typeof value.max_tokens !== 'number' || value.max_tokens < 1) {
            throw new Error('max_tokens must be a positive integer');
          }
          if (typeof value.temperature !== 'number' || value.temperature < 0 || value.temperature > 1) {
            throw new Error('temperature must be a number between 0 and 1');
          }
          return true;
        }
      }
    }
  }
};

// Middleware to validate request and handle validation errors
function validate(schema) {
  return [
    checkSchema(schemas[schema]),
    (req, res, next) => {
      const errors = validationResult(req);
      
      if (!errors.isEmpty()) {
        const errorDetails = errors.array().map(error => ({
          field: error.path,
          message: error.msg
        }));
        
        logger.warn('Validation error', { errors: errorDetails, path: req.path });
        
        throw new ValidationError('Invalid request data', errorDetails);
      }
      
      next();
    }
  ];
}

module.exports = {
  validate
};