const { pool } = require('../config/database');
const logger = require('../utils/logger');
const { NotFoundError, ValidationError } = require('../middleware/errorHandler');
const modelService = require('./modelService');

/**
 * Service for Deployment-related database operations
 * 
 * TODO: Implementation tasks:
 * - [ ] Implement deployment status updates
 * - [ ] Add deployment scaling functionality
 * - [ ] Create deployment health monitoring
 * - [ ] Implement deployment logs retrieval
 * - [ ] Add validation for deployment configurations
 * - [ ] Create deployment metrics collection
 * - [ ] Run coverage report to verify 95%+ coverage
 */
class DeploymentService {
  /**
   * Create a new deployment
   * @param {Object} deploymentData - Deployment data
   * @param {string} deploymentData.name - Deployment name
   * @param {number} deploymentData.model_id - Model ID
   * @param {number} deploymentData.model_version_id - Model version ID
   * @param {string} deploymentData.environment - Deployment environment
   * @param {Object} deploymentData.scaling - Scaling configuration
   * @param {Object} deploymentData.config - Model configuration
   * @param {string} createdBy - User who created the deployment
   * @returns {Promise<Object>} Created deployment
   */
  async createDeployment(deploymentData, createdBy) {
    const { 
      name, 
      model_id, 
      model_version_id, 
      environment,
      scaling,
      config 
    } = deploymentData;
    
    // Generate endpoint URL based on name
    const endpoint = `https://api.bluefly.io/v1/inference/deployments/${name}`;
    
    const connection = await pool.getConnection();
    
    try {
      // Start transaction
      await connection.beginTransaction();
      
      // Verify model exists
      await modelService.getModelById(model_id);
      
      // Verify model version exists
      const [modelVersions] = await connection.query(
        'SELECT * FROM model_versions WHERE id = ? AND model_id = ?',
        [model_version_id, model_id]
      );
      
      if (modelVersions.length === 0) {
        throw new NotFoundError(`Model version with ID ${model_version_id} not found for model ${model_id}`);
      }
      
      // Check if deployment name already exists
      const [existingDeployments] = await connection.query(
        'SELECT id FROM deployments WHERE name = ?',
        [name]
      );
      
      if (existingDeployments.length > 0) {
        throw new ValidationError('Deployment name must be unique', [
          { field: 'name', message: 'Deployment name already exists' }
        ]);
      }
      
      // Insert deployment
      const [result] = await connection.query(
        `INSERT INTO deployments 
         (name, model_id, model_version_id, status, environment, endpoint, scaling, config, created_by)
         VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)`,
        [
          name,
          model_id,
          model_version_id,
          'provisioning',
          environment,
          endpoint,
          JSON.stringify(scaling),
          JSON.stringify(config),
          createdBy
        ]
      );
      
      // Get the created deployment
      const [deployments] = await connection.query(
        'SELECT * FROM deployments WHERE id = ?',
        [result.insertId]
      );
      
      // Commit transaction
      await connection.commit();
      
      const deployment = deployments[0];
      
      // Process the deployment to ensure JSON fields are parsed
      return {
        ...deployment,
        scaling: typeof deployment.scaling === 'string' 
          ? JSON.parse(deployment.scaling) 
          : deployment.scaling,
        config: typeof deployment.config === 'string' 
          ? JSON.parse(deployment.config) 
          : deployment.config,
        created_at: deployment.created_at.toISOString(),
        updated_at: deployment.updated_at ? deployment.updated_at.toISOString() : null
      };
    } catch (error) {
      // Rollback transaction on error
      await connection.rollback();
      logger.error('Error creating deployment:', error);
      throw error;
    } finally {
      // Release connection
      connection.release();
    }
  }
  
  /**
   * Get a deployment by ID
   * @param {number} id - Deployment ID
   * @returns {Promise<Object>} Deployment details
   * @throws {NotFoundError} If deployment not found
   */
  async getDeploymentById(id) {
    try {
      const [deployments] = await pool.query(
        'SELECT * FROM deployments WHERE id = ?',
        [id]
      );
      
      if (deployments.length === 0) {
        throw new NotFoundError(`Deployment with ID ${id} not found`);
      }
      
      const deployment = deployments[0];
      
      // Process the deployment to ensure JSON fields are parsed
      return {
        ...deployment,
        scaling: typeof deployment.scaling === 'string' 
          ? JSON.parse(deployment.scaling) 
          : deployment.scaling,
        config: typeof deployment.config === 'string' 
          ? JSON.parse(deployment.config) 
          : deployment.config,
        created_at: deployment.created_at.toISOString(),
        updated_at: deployment.updated_at ? deployment.updated_at.toISOString() : null
      };
    } catch (error) {
      logger.error(`Error getting deployment by ID ${id}:`, error);
      throw error;
    }
  }
  
  /**
   * Get deployments with optional filtering
   * @param {Object} filters - Filtering options
   * @param {string} [filters.status] - Filter by status
   * @param {string} [filters.environment] - Filter by environment
   * @param {number} [filters.model_id] - Filter by model ID
   * @param {number} [filters.limit=20] - Limit results
   * @param {number} [filters.offset=0] - Offset for pagination
   * @returns {Promise<Object>} Deployments list with pagination info
   */
  async getDeployments(filters = {}) {
    const { 
      status, 
      environment, 
      model_id, 
      limit = 20, 
      offset = 0 
    } = filters;
    
    try {
      // Build WHERE clause for filters
      const whereConditions = [];
      const params = [];
      
      if (status) {
        whereConditions.push('status = ?');
        params.push(status);
      }
      
      if (environment) {
        whereConditions.push('environment = ?');
        params.push(environment);
      }
      
      if (model_id) {
        whereConditions.push('model_id = ?');
        params.push(model_id);
      }
      
      const whereClause = whereConditions.length > 0 
        ? `WHERE ${whereConditions.join(' AND ')}` 
        : '';
      
      // Get total count for pagination
      const [countResult] = await pool.query(
        `SELECT COUNT(*) as total FROM deployments ${whereClause}`,
        params
      );
      const total = countResult[0].total;
      
      // Get deployments with pagination
      const limitedParams = [...params, limit, offset];
      const [deployments] = await pool.query(
        `SELECT * FROM deployments 
         ${whereClause} 
         ORDER BY created_at DESC 
         LIMIT ? OFFSET ?`,
        limitedParams
      );
      
      // Process the deployments to ensure JSON fields are parsed
      const processedDeployments = deployments.map(deployment => ({
        ...deployment,
        scaling: typeof deployment.scaling === 'string' 
          ? JSON.parse(deployment.scaling) 
          : deployment.scaling,
        config: typeof deployment.config === 'string' 
          ? JSON.parse(deployment.config) 
          : deployment.config,
        created_at: deployment.created_at.toISOString(),
        updated_at: deployment.updated_at ? deployment.updated_at.toISOString() : null
      }));
      
      return {
        deployments: processedDeployments,
        pagination: {
          total,
          limit,
          offset
        }
      };
    } catch (error) {
      logger.error('Error getting deployments:', error);
      throw error;
    }
  }
}

module.exports = new DeploymentService();