#!/usr/bin/env node

/**
 * Simple Documentation Aggregation Script
 * 
 * This script aggregates documentation from all real LLM Platform projects
 * based on the actual project locations and git remotes.
 */

const fs = require('fs');
const path = require('path');
const { execSync } = require('child_process');

// Real production projects based on git remotes found
const PROJECTS = {
  // NPM Packages with GitLab remotes
  'apple-fm': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/apple-fm',
    remote: 'https://gitlab.bluefly.io/llm/apple-fm.git',
    type: 'npm'
  },
  'bfrfp': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/bfrfp',
    remote: 'https://gitlab.bluefly.io/llm/bfrfp.git',
    type: 'npm'
  },
  'llm-gateway': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/llm-gateway',
    remote: 'https://gitlab.bluefly.io/llm/llm-gateway.git',
    type: 'npm'
  },
  'llm-gateway-sdk': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/llm-gateway-sdk',
    remote: 'https://gitlab.bluefly.io/llm/llm-gateway-sdk.git',
    type: 'npm'
  },
  'llm-mcp': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/llm-mcp',
    remote: 'https://gitlab.bluefly.io/llm/llm-mcp.git',
    type: 'npm'
  },
  'llm-ui': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/llm-ui',
    remote: 'https://gitlab.bluefly.io/llm/llm-ui.git',
    type: 'npm'
  },
  'llmcli': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/llmcli',
    remote: 'https://gitlab.bluefly.io/llm/llmcli.git',
    type: 'npm'
  },
  'tddai': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/tddai',
    remote: 'https://gitlab.bluefly.io/llm/tddai.git',
    type: 'npm'
  },
  'secure-project': {
    path: '/Users/flux423/Sites/LLM/_CommonNPM/secure-project',
    remote: 'https://gitlab.bluefly.io/llm/secure-project.git',
    type: 'npm'
  },
  
  // Models
  'tddai-model': {
    path: '/Users/flux423/Sites/LLM/tddai-model',
    remote: 'https://gitlab.bluefly.io/llm/models/tddai-model.git',
    type: 'model'
  },
  
  // CI/CD Components
  'bfcicomponents': {
    path: '/Users/flux423/Sites/LLM/bfcicomponents',
    remote: 'https://gitlab.bluefly.io/llm/bfcicomponents',
    type: 'cicd'
  }
};

const OUTPUT_DIR = path.join(__dirname, '..', 'docs');

/**
 * Copy and sanitize documentation files recursively
 * @param {string} source - Source directory path
 * @param {string} destination - Destination directory path
 */
function copyAndSanitizeDocs(source, destination) {
  // Create destination directory if it doesn't exist
  if (!fs.existsSync(destination)) {
    fs.mkdirSync(destination, { recursive: true });
  }
  
  const items = fs.readdirSync(source);
  
  for (const item of items) {
    const sourcePath = path.join(source, item);
    const destPath = path.join(destination, item);
    const stat = fs.statSync(sourcePath);
    
    if (stat.isDirectory()) {
      // Recursively copy subdirectories
      copyAndSanitizeDocs(sourcePath, destPath);
    } else if (stat.isFile()) {
      // Check if it's a markdown file
      if (item.endsWith('.md') || item.endsWith('.mdx')) {
        try {
          let content = fs.readFileSync(sourcePath, 'utf8');
          // Sanitize the content
          content = sanitizeMarkdownContent(content);
          fs.writeFileSync(destPath, content, 'utf8');
        } catch (error) {
          console.log(`    ⚠️  Error sanitizing ${item}: ${error.message}`);
          // If sanitization fails, copy the file as-is
          fs.copyFileSync(sourcePath, destPath);
        }
      } else {
        // For non-markdown files, just copy them
        fs.copyFileSync(sourcePath, destPath);
      }
    }
  }
}

/**
 * Count files recursively in a directory
 * @param {string} dirPath - Directory path to count files in
 * @returns {number} - Total number of files
 */
function countFilesRecursively(dirPath) {
  let count = 0;
  const items = fs.readdirSync(dirPath);
  
  for (const item of items) {
    const fullPath = path.join(dirPath, item);
    const stat = fs.statSync(fullPath);
    
    if (stat.isDirectory()) {
      count += countFilesRecursively(fullPath);
    } else if (stat.isFile()) {
      count++;
    }
  }
  
  return count;
}

/**
 * Sanitize markdown content by replacing template variables with placeholders
 * @param {string} content - The markdown content to sanitize
 * @returns {string} - The sanitized content
 */
function sanitizeMarkdownContent(content) {
  // Map of common template variables to their placeholder values
  const templateReplacements = {
    // Company and project info
    'COMPANY_NAME': 'Your Company',
    'company_name': 'Your Company',
    'COMPANY_URL': 'https://company.com',
    'COMPANY_ABBREV': 'CO',
    'PROJECT_NAME': 'Project Name',
    'project_name': 'Project Name',
    
    // Versioning
    'version_number': '1.0.0',
    'VERSION': '1.0.0',
    'version': '1.0.0',
    
    // Registry and repository
    'CI_REGISTRY': 'registry.example.com',
    'CI_REGISTRY_IMAGE': 'registry.example.com/project',
    'registry': 'registry.example.com',
    'repository': 'your-repo',
    'REPOSITORY': 'your-repo',
    
    // Documentation
    'TITLE': 'Documentation',
    'title': 'Documentation',
    'DESCRIPTION': 'Project Description',
    'description': 'Project Description',
    
    // Entity and ID placeholders
    'entity_type': 'entity',
    'entity': 'entity',
    'id': 'item',
    'ID': 'item',
    'uuid': 'uuid-placeholder',
    'UUID': 'uuid-placeholder',
    
    // RFP specific
    'rfp_title': 'RFP Title',
    'rfp_number': 'RFP-001',
    'relevant_experience': 'Experience',
    'problem_understanding': 'Understanding',
    'solution_approach': 'Approach',
    'timeline': 'Timeline',
    'budget': 'Budget',
    
    // API and endpoints
    'API_URL': 'https://api.example.com',
    'api_url': 'https://api.example.com',
    'API_KEY': 'your-api-key',
    'api_key': 'your-api-key',
    'endpoint': '/api/endpoint',
    'ENDPOINT': '/api/endpoint',
    
    // Environment variables
    'ENV': 'production',
    'environment': 'production',
    'NODE_ENV': 'production',
    
    // Dates and timestamps
    'date': new Date().toISOString().split('T')[0],
    'DATE': new Date().toISOString().split('T')[0],
    'timestamp': new Date().toISOString(),
    'TIMESTAMP': new Date().toISOString(),
    'year': new Date().getFullYear().toString(),
    'YEAR': new Date().getFullYear().toString(),
    
    // User and auth
    'username': 'user',
    'USERNAME': 'user',
    'email': 'user@example.com',
    'EMAIL': 'user@example.com',
    'token': 'auth-token',
    'TOKEN': 'auth-token',
    
    // Generic placeholders
    'name': 'Name',
    'NAME': 'Name',
    'value': 'Value',
    'VALUE': 'Value',
    'key': 'Key',
    'KEY': 'Key',
    'type': 'Type',
    'TYPE': 'Type',
    'status': 'Status',
    'STATUS': 'Status',
    'message': 'Message',
    'MESSAGE': 'Message',
    'path': '/path/to/resource',
    'PATH': '/path/to/resource',
    'url': 'https://example.com',
    'URL': 'https://example.com',
    'host': 'example.com',
    'HOST': 'example.com',
    'port': '3000',
    'PORT': '3000',
    
    // Template placeholders
    'TOPIC': 'Topic',
    'OVERVIEW_PLACEHOLDER': 'Overview content goes here',
    'bundle': 'bundle',
    'BUNDLE': 'bundle',
    
    // Additional common variables
    'service': 'service-name',
    'SERVICE': 'service-name',
    'component': 'component-name',
    'COMPONENT': 'component-name',
    'module': 'module-name',
    'MODULE': 'module-name',
    'package': 'package-name',
    'PACKAGE': 'package-name',
    'config': 'configuration',
    'CONFIG': 'configuration',
    'option': 'option-value',
    'OPTION': 'option-value',
    'parameter': 'parameter-value',
    'PARAMETER': 'parameter-value'
  };
  
  // Replace ${variable} pattern
  content = content.replace(/\$\{([^}]+)\}/g, (match, variable) => {
    const trimmedVar = variable.trim();
    if (templateReplacements[trimmedVar]) {
      return templateReplacements[trimmedVar];
    }
    // If not found in our map, return a generic placeholder
    return `[${trimmedVar}]`;
  });
  
  // Replace [{variable}] pattern (with square brackets and curly braces)
  content = content.replace(/\[\{([^}]+)\}\]/g, (match, variable) => {
    const trimmedVar = variable.trim();
    if (templateReplacements[trimmedVar]) {
      return templateReplacements[trimmedVar];
    }
    // If not found in our map, return a generic placeholder
    return `[${trimmedVar}]`;
  });
  
  // Replace [{variable]} pattern (with mismatched brackets - common typo)
  content = content.replace(/\[{([^\]]+)\]}/g, (match, variable) => {
    const trimmedVar = variable.trim();
    if (templateReplacements[trimmedVar]) {
      return templateReplacements[trimmedVar];
    }
    // If not found in our map, return a generic placeholder
    return `[${trimmedVar}]`;
  });
  
  // Replace {{variable}} pattern (double curly braces)
  content = content.replace(/\{\{([^}]+)\}\}/g, (match, variable) => {
    const trimmedVar = variable.trim();
    if (templateReplacements[trimmedVar]) {
      return templateReplacements[trimmedVar];
    }
    // If not found in our map, return a generic placeholder
    return `[${trimmedVar}]`;
  });
  
  // Replace {variable} pattern (without $ and not inside square brackets)
  content = content.replace(/(?<!\$)(?<!\[)\{([^}]+)\}(?!\])/g, (match, variable) => {
    // Skip if this looks like code (contains common code patterns)
    if (variable.includes('=>') || variable.includes('function') || 
        variable.includes('return') || variable.includes(';') ||
        variable.includes(',') || variable.includes(':')) {
      return match;
    }
    
    const trimmedVar = variable.trim();
    if (templateReplacements[trimmedVar]) {
      return templateReplacements[trimmedVar];
    }
    // If not found in our map, return a generic placeholder
    return `[${trimmedVar}]`;
  });
  
  return content;
}

/**
 * Aggregate documentation from a single project
 */
function aggregateProject(projectName, projectConfig) {
  console.log(`📚 Aggregating ${projectName}...`);
  
  const { path: projectPath, type } = projectConfig;
  
  if (!fs.existsSync(projectPath)) {
    console.log(`  ⚠️  Project path not found: ${projectPath}`);
    return;
  }
  
  // Determine output directory based on type
  let outputPath;
  if (type === 'cicd') {
    outputPath = path.join(OUTPUT_DIR, 'cicd', projectName);
  } else if (type === 'model') {
    outputPath = path.join(OUTPUT_DIR, 'models', projectName);
  } else {
    outputPath = path.join(OUTPUT_DIR, 'packages', projectName);
  }
  
  // Create output directory
  if (!fs.existsSync(outputPath)) {
    fs.mkdirSync(outputPath, { recursive: true });
  }
  
  // Copy docs folder if it exists
  const docsPath = path.join(projectPath, 'docs');
  if (fs.existsSync(docsPath)) {
    try {
      // Remove existing docs
      if (fs.existsSync(outputPath)) {
        execSync(`rm -rf "${outputPath}"/*`);
      }
      
      // Copy and sanitize markdown files
      copyAndSanitizeDocs(docsPath, outputPath);
      
      const fileCount = countFilesRecursively(docsPath);
      console.log(`  ✅ Copied and sanitized docs folder (${fileCount} files)`);
    } catch (error) {
      console.log(`  ⚠️  Error copying docs: ${error.message}`);
    }
  }
  
  // Copy README.md as index.md if no index exists
  const readmePath = path.join(projectPath, 'README.md');
  const indexPath = path.join(outputPath, 'index.md');
  
  if (fs.existsSync(readmePath) && !fs.existsSync(indexPath)) {
    try {
      let readmeContent = fs.readFileSync(readmePath, 'utf8');
      
      // Sanitize the content to replace template variables
      readmeContent = sanitizeMarkdownContent(readmeContent);
      
      // Add Docusaurus frontmatter
      const frontmatter = `---
title: ${projectName}
description: "${projectConfig.remote ? `GitLab: ${projectConfig.remote.replace('.git', '')}` : 'Documentation for ' + projectName}"
sidebar_position: 1
---

`;
      
      fs.writeFileSync(indexPath, frontmatter + readmeContent);
      console.log(`  ✅ Created index.md from README.md`);
    } catch (error) {
      console.log(`  ⚠️  Error creating index: ${error.message}`);
    }
  }
  
  // Add package info for NPM packages
  if (type === 'npm') {
    const packageJsonPath = path.join(projectPath, 'package.json');
    if (fs.existsSync(packageJsonPath)) {
      try {
        const packageJson = JSON.parse(fs.readFileSync(packageJsonPath, 'utf8'));
        const packageInfoPath = path.join(outputPath, 'package-info.md');
        
        const dependencies = Object.keys(packageJson.dependencies || {});
        const devDependencies = Object.keys(packageJson.devDependencies || {});
        
        let packageInfo = `---
title: Package Information
description: NPM package details for ${projectName}
sidebar_position: 99
---

# Package Information

## Basic Info
- **Name**: \`${packageJson.name || projectName}\`
- **Version**: \`${packageJson.version || '0.0.0'}\`
- **Description**: ${packageJson.description || 'No description'}
- **License**: ${packageJson.license || 'Unknown'}
- **Repository**: [GitLab](${projectConfig.remote.replace('.git', '')})

## Available Scripts
${Object.entries(packageJson.scripts || {}).map(([name, script]) => 
  `- **\`npm run ${name}\`**: ${script}`
).join('\n')}

## Dependencies (${dependencies.length})
${dependencies.length > 0 ? dependencies.map(dep => `- ${dep}`).join('\n') : 'No dependencies'}

## Dev Dependencies (${devDependencies.length})
${devDependencies.length > 0 ? devDependencies.slice(0, 10).map(dep => `- ${dep}`).join('\n') : 'No dev dependencies'}
${devDependencies.length > 10 ? `\n... and ${devDependencies.length - 10} more` : ''}
`;
        
        // Sanitize the package info content
        packageInfo = sanitizeMarkdownContent(packageInfo);
        
        fs.writeFileSync(packageInfoPath, packageInfo);
        console.log(`  ✅ Generated package info`);
      } catch (error) {
        console.log(`  ⚠️  Error generating package info: ${error.message}`);
      }
    }
  }
}

/**
 * Update the main index.md with all aggregated projects
 */
function updateMainIndex() {
  console.log('📝 Updating main index...');
  
  const npmProjects = Object.entries(PROJECTS).filter(([, config]) => config.type === 'npm');
  const modelProjects = Object.entries(PROJECTS).filter(([, config]) => config.type === 'model');
  const cicdProjects = Object.entries(PROJECTS).filter(([, config]) => config.type === 'cicd');
  
  const indexContent = `---
title: LLM Platform Documentation Hub
description: Comprehensive documentation aggregated from all LLM Platform projects
---

# LLM Platform Documentation Hub

Welcome to the comprehensive documentation for the LLM Platform ecosystem. This documentation is automatically aggregated from all platform components.

## 📦 NPM Packages (${npmProjects.length})

${npmProjects.map(([name, config]) => {
  // Determine status based on docs folder existence and maturity
  const docsPath = path.join(config.path, 'docs');
  const hasExtensiveDocs = fs.existsSync(docsPath) && fs.readdirSync(docsPath).length > 5;
  
  let status = '🔄 In Development';
  if (['apple-fm', 'bfrfp', 'tddai'].includes(name)) {
    status = '✅ Production Ready';
  } else if (['llm-gateway-sdk', 'llmcli'].includes(name) || hasExtensiveDocs) {
    status = '🚀 Feature Complete';
  }
  
  return `- **[${name}](/packages/${name}/)** - ${status}
  - Repository: [GitLab](${config.remote.replace('.git', '')})`;
}).join('\n')}

## 🤖 AI Models (${modelProjects.length})

${modelProjects.map(([name, config]) => 
  `- **[${name}](/models/${name}/)** - Production AI model with MLflow integration
  - Repository: [GitLab](${config.remote.replace('.git', '')})`
).join('\n')}

## 🔧 CI/CD Components (${cicdProjects.length})

${cicdProjects.map(([name, config]) => 
  `- **[${name}](/cicd/${name}/)** - GitLab CI/CD component catalog
  - Repository: [GitLab](${config.remote.replace('.git', '')})`
).join('\n')}

## 🚀 Quick Start

\`\`\`bash
# Install LLM Platform CLI
npm install -g @llm-platform/cli

# Install complete platform with all components
llmcli llm platform:install --with-vector-db --with-ai-services

# Initialize new project
llmcli llm init my-project --template advanced
\`\`\`

## 🏗️ Platform Architecture

The LLM Platform follows a modular architecture with independent NPM packages:

\`\`\`
┌─────────────────────────────────────────────────────────┐
│                   CLI Interface                          │
│                   llmcli                                │
└─────────────────┬───────────────────────────────────────┘
                  │
┌─────────────────▼───────────────────────────────────────┐
│                API Gateway                               │
│              llm-gateway                                │
└─────────────────┬───────────────────────────────────────┘
                  │
┌─────────────────▼───────────────────────────────────────┐
│                Service Layer                             │
│  apple-fm │ bfrfp │ llm-mcp │ tddai │ llm-ui          │
└─────────────────┬───────────────────────────────────────┘
                  │
┌─────────────────▼───────────────────────────────────────┐
│              CI/CD Pipeline                              │
│              bfcicomponents                             │
└─────────────────────────────────────────────────────────┘
\`\`\`

## 📚 Documentation Organization

This documentation hub automatically aggregates content from:

- **Individual project \`/docs\` folders** - Detailed technical documentation
- **README.md files** - Project overviews and quick start guides  
- **Package.json metadata** - Dependency and script information
- **GitLab repository links** - Source code and issue tracking

## 🔗 Project Repositories

All projects are hosted on GitLab at: **https://gitlab.bluefly.io/llm/**

${Object.entries(PROJECTS).map(([name, config]) => 
  `- [${name}](${config.remote.replace('.git', '')})`
).join('\n')}

---

**Last Updated**: ${new Date().toISOString().split('T')[0]} | **Projects**: ${Object.keys(PROJECTS).length} | **Status**: Active Development

*This documentation is automatically synchronized with all LLM Platform projects.*
`;
  
  fs.writeFileSync(path.join(OUTPUT_DIR, 'index.md'), indexContent);
  console.log('  ✅ Updated main index');
}

/**
 * Update footer links in Docusaurus config
 */
function updateFooterLinks() {
  console.log('🔗 Skipping footer links update (config is manually maintained)');
  console.log('  ✅ Footer links left unchanged');
}

/**
 * Main aggregation process
 */
function main() {
  console.log('🚀 Starting LLM Platform Documentation Aggregation\n');
  
  // Clean up old aggregated content
  const aggregatedDirs = ['packages', 'models', 'cicd'];
  aggregatedDirs.forEach(dir => {
    const dirPath = path.join(OUTPUT_DIR, dir);
    if (fs.existsSync(dirPath)) {
      execSync(`rm -rf "${dirPath}"`);
      console.log(`🧹 Cleaned old ${dir} documentation`);
    }
  });
  
  // Aggregate each project
  for (const [projectName, projectConfig] of Object.entries(PROJECTS)) {
    aggregateProject(projectName, projectConfig);
  }
  
  // Update main documentation
  updateMainIndex();
  updateFooterLinks();
  
  console.log(`\n✅ Documentation aggregation complete!`);
  console.log(`📊 Aggregated ${Object.keys(PROJECTS).length} projects`);
  console.log(`📖 View documentation: npm run start`);
}

if (require.main === module) {
  main();
}

module.exports = { PROJECTS, aggregateProject, updateMainIndex };