#!/usr/bin/env node

/**
 * Federated Documentation Aggregator
 * 
 * This script aggregates documentation from all ecosystem projects
 * into the bfdocs hub while keeping projects independent.
 * 
 * Features:
 * - Reference-based aggregation (no copying)
 * - GitLab Pages compatible output
 * - Search index generation
 * - Architectural documentation focus
 */

const fs = require('fs').promises;
const path = require('path');
const yaml = require('yaml');
const EnhancedCollectors = require('./enhanced-collectors.js');

class DocumentationAggregator {
  constructor(configPath = './docs-config.json') {
    this.configPath = configPath;
    this.config = null;
    this.projects = {};
    this.enhancedCollectors = new EnhancedCollectors();
    this.aggregatedIndex = {
      projects: {},
      categories: {},
      searchIndex: [],
      ecosystem: {
        totalProjects: 0,
        totalFiles: 0,
        totalEndpoints: 0,
        technologies: new Set(),
        lastUpdated: new Date().toISOString()
      },
      lastUpdated: new Date().toISOString()
    };
  }

  async loadConfig() {
    try {
      const configContent = await fs.readFile(this.configPath, 'utf-8');
      this.config = JSON.parse(configContent);
      console.log('✅ Loaded configuration for', Object.keys(this.config.projects).length, 'projects');
    } catch (error) {
      console.error('❌ Failed to load config:', error.message);
      process.exit(1);
    }
  }

  async scanProjectDocs(projectKey, projectConfig) {
    const projectPath = path.resolve(projectConfig.path);
    const docsPath = path.join(projectPath, projectConfig.docsPath);
    
    console.log(`📂 Scanning ${projectKey} docs at ${docsPath}`);
    
    try {
      await fs.access(docsPath);
    } catch (error) {
      console.log(`⚠️  No docs directory found for ${projectKey}, creating...`);
      await fs.mkdir(docsPath, { recursive: true });
      await this.createProjectDocTemplate(projectKey, projectConfig, docsPath);
    }

    const projectDocs = {
      ...projectConfig,
      docsPath: docsPath,
      files: [],
      architecture: projectConfig.architecture || {},
      lastScanned: new Date().toISOString()
    };

    try {
      const files = await this.scanDirectory(docsPath);
      projectDocs.files = files;
      
      // Extract architectural information
      projectDocs.architecture = await this.extractArchitecturalInfo(projectPath, projectConfig);
      
      // Use enhanced collectors for comprehensive data collection
      const enhancedData = await this.enhancedCollectors.collectProjectData(projectPath, projectConfig);
      projectDocs.enhancedData = enhancedData;
      
      // Update ecosystem statistics
      this.aggregatedIndex.ecosystem.totalFiles += files.length;
      if (enhancedData.endpoints) {
        this.aggregatedIndex.ecosystem.totalEndpoints += enhancedData.endpoints.length;
      }
      if (enhancedData.technologies) {
        enhancedData.technologies.forEach(tech => this.aggregatedIndex.ecosystem.technologies.add(tech));
      }
      
      console.log(`✅ Found ${files.length} documentation files for ${projectKey}`);
      console.log(`📊 Enhanced data: ${enhancedData.endpoints?.length || 0} endpoints, ${enhancedData.technologies?.length || 0} technologies`);
    } catch (error) {
      console.log(`⚠️  Error scanning ${projectKey}:`, error.message);
      projectDocs.files = [];
    }

    return projectDocs;
  }

  async scanDirectory(dirPath, relativePath = '') {
    const files = [];
    
    try {
      const entries = await fs.readdir(dirPath, { withFileTypes: true });
      
      for (const entry of entries) {
        const fullPath = path.join(dirPath, entry.name);
        const relPath = path.join(relativePath, entry.name);
        
        if (entry.isDirectory() && !entry.name.startsWith('.')) {
          const subFiles = await this.scanDirectory(fullPath, relPath);
          files.push(...subFiles);
        } else if (entry.isFile() && this.isDocumentationFile(entry.name)) {
          const fileInfo = await this.analyzeDocFile(fullPath, relPath);
          files.push(fileInfo);
        }
      }
    } catch (error) {
      console.error(`Error scanning directory ${dirPath}:`, error.message);
    }
    
    return files;
  }

  isDocumentationFile(filename) {
    const docExtensions = ['.md', '.rst', '.txt'];
    const isDocFile = docExtensions.some(ext => filename.toLowerCase().endsWith(ext));
    const isNotExcluded = !filename.startsWith('.') && !filename.includes('node_modules');
    return isDocFile && isNotExcluded;
  }

  async analyzeDocFile(fullPath, relativePath) {
    try {
      const content = await fs.readFile(fullPath, 'utf-8');
      const stats = await fs.stat(fullPath);
      
      // Extract title and description
      const title = this.extractTitle(content, relativePath);
      const description = this.extractDescription(content);
      const tags = this.extractTags(content, relativePath);
      
      return {
        path: relativePath,
        fullPath: fullPath,
        title: title,
        description: description,
        tags: tags,
        size: stats.size,
        modified: stats.mtime.toISOString(),
        type: this.categorizeDoc(relativePath, content)
      };
    } catch (error) {
      return {
        path: relativePath,
        fullPath: fullPath,
        title: path.basename(relativePath, path.extname(relativePath)),
        description: 'Documentation file',
        tags: [],
        size: 0,
        modified: new Date().toISOString(),
        type: 'general',
        error: error.message
      };
    }
  }

  extractTitle(content, relativePath) {
    // Try to extract from frontmatter first
    const frontmatterMatch = content.match(/^---\\s*\\n([\\s\\S]*?)\\n---/);
    if (frontmatterMatch) {
      try {
        const frontmatter = yaml.parse(frontmatterMatch[1]);
        if (frontmatter.title) return frontmatter.title;
      } catch (e) {
        // Continue to next method
      }
    }
    
    // Try to extract from first # heading
    const headingMatch = content.match(/^#\\s+(.+)$/m);
    if (headingMatch) {
      return headingMatch[1].trim();
    }
    
    // Fall back to filename
    return path.basename(relativePath, path.extname(relativePath))
      .split(/[-_]/)
      .map(word => word.charAt(0).toUpperCase() + word.slice(1))
      .join(' ');
  }

  extractDescription(content) {
    // Remove frontmatter
    const withoutFrontmatter = content.replace(/^---\\s*\\n[\\s\\S]*?\\n---\\s*\\n/, '');
    
    // Find first paragraph that's not a heading
    const lines = withoutFrontmatter.split('\\n');
    for (const line of lines) {
      const trimmed = line.trim();
      if (trimmed && !trimmed.startsWith('#') && !trimmed.startsWith('```')) {
        return trimmed.substring(0, 200) + (trimmed.length > 200 ? '...' : '');
      }
    }
    
    return 'Documentation file';
  }

  extractTags(content, relativePath) {
    const tags = new Set();
    
    // Extract from path
    const pathParts = relativePath.split('/');
    pathParts.forEach(part => {
      if (part !== 'docs' && !part.endsWith('.md')) {
        tags.add(part.toLowerCase());
      }
    });
    
    // Extract from content patterns
    if (content.includes('architecture') || content.includes('Architecture')) tags.add('architecture');
    if (content.includes('API') || content.includes('api')) tags.add('api');
    if (content.includes('setup') || content.includes('Setup')) tags.add('setup');
    if (content.includes('deployment') || content.includes('Deployment')) tags.add('deployment');
    if (content.includes('integration') || content.includes('Integration')) tags.add('integration');
    
    return Array.from(tags);
  }

  categorizeDoc(relativePath, content) {
    const path = relativePath.toLowerCase();
    
    if (path.includes('architecture') || content.includes('Architecture')) return 'architecture';
    if (path.includes('api') || content.includes('OpenAPI')) return 'api';
    if (path.includes('setup') || path.includes('install')) return 'setup';
    if (path.includes('deployment') || path.includes('deploy')) return 'deployment';
    if (path.includes('integration')) return 'integration';
    if (path.includes('example')) return 'examples';
    if (path.includes('troubleshoot')) return 'troubleshooting';
    if (path.includes('readme')) return 'overview';
    
    return 'general';
  }

  async extractArchitecturalInfo(projectPath, projectConfig) {
    const architecture = { ...projectConfig.architecture };
    
    try {
      // Try to extract from package.json
      const packageJsonPath = path.join(projectPath, 'package.json');
      try {
        const packageContent = await fs.readFile(packageJsonPath, 'utf-8');
        const packageJson = JSON.parse(packageContent);
        
        architecture.version = packageJson.version;
        architecture.dependencies = Object.keys(packageJson.dependencies || {});
        architecture.scripts = Object.keys(packageJson.scripts || {});
      } catch (e) {
        // Not a Node.js project or no package.json
      }
      
      // Try to extract from Dockerfile
      const dockerfilePath = path.join(projectPath, 'Dockerfile');
      try {
        const dockerContent = await fs.readFile(dockerfilePath, 'utf-8');
        architecture.containerized = true;
        architecture.baseImage = this.extractDockerBaseImage(dockerContent);
      } catch (e) {
        architecture.containerized = false;
      }
      
      // Check for specific technology indicators
      architecture.technologies = await this.detectTechnologies(projectPath);
      
    } catch (error) {
      console.warn(`Could not extract architectural info for ${projectConfig.name}:`, error.message);
    }
    
    return architecture;
  }

  extractDockerBaseImage(dockerContent) {
    const fromMatch = dockerContent.match(/^FROM\\s+([^\\s]+)/m);
    return fromMatch ? fromMatch[1] : 'unknown';
  }

  async detectTechnologies(projectPath) {
    const technologies = new Set();
    
    const indicators = [
      { file: 'package.json', tech: 'Node.js' },
      { file: 'composer.json', tech: 'PHP' },
      { file: 'requirements.txt', tech: 'Python' },
      { file: 'Cargo.toml', tech: 'Rust' },
      { file: 'go.mod', tech: 'Go' },
      { file: 'pom.xml', tech: 'Java' },
      { file: 'tsconfig.json', tech: 'TypeScript' },
      { file: 'jest.config.js', tech: 'Jest' },
      { file: 'vite.config.ts', tech: 'Vite' },
      { file: 'rollup.config.js', tech: 'Rollup' },
      { file: 'k8s', tech: 'Kubernetes' },
      { file: 'helm', tech: 'Helm' },
      { file: '.gitlab-ci.yml', tech: 'GitLab CI' }
    ];
    
    for (const indicator of indicators) {
      try {
        const indicatorPath = path.join(projectPath, indicator.file);
        await fs.access(indicatorPath);
        technologies.add(indicator.tech);
      } catch (e) {
        // File doesn't exist
      }
    }
    
    return Array.from(technologies);
  }

  async createProjectDocTemplate(projectKey, projectConfig, docsPath) {
    const templates = {
      'index.md': this.generateProjectIndexTemplate(projectKey, projectConfig),
      'architecture.md': this.generateArchitectureTemplate(projectKey, projectConfig),
      'api.md': this.generateApiTemplate(projectKey, projectConfig),
      'setup.md': this.generateSetupTemplate(projectKey, projectConfig)
    };
    
    for (const [filename, content] of Object.entries(templates)) {
      const filePath = path.join(docsPath, filename);
      try {
        await fs.access(filePath);
        console.log(`📄 ${filename} already exists for ${projectKey}`);
      } catch (e) {
        await fs.writeFile(filePath, content);
        console.log(`📄 Created ${filename} for ${projectKey}`);
      }
    }
  }

  generateProjectIndexTemplate(projectKey, projectConfig) {
    return `---
title: ${projectConfig.name}
description: ${projectConfig.description}
category: ${projectConfig.category}
repository: ${projectConfig.repository}
lastUpdated: ${new Date().toISOString()}
---

# ${projectConfig.name}

${projectConfig.description}

## Overview

**Category**: ${projectConfig.category}  
**Repository**: [${projectConfig.repository}](${projectConfig.repository})  
**Language**: ${projectConfig.architecture?.language || 'Not specified'}

## Quick Start

\`\`\`bash
# Clone the repository
git clone ${projectConfig.repository}

# Navigate to project
cd ${projectKey}

# Follow setup instructions in setup.md
\`\`\`

## Documentation Structure

- [Architecture](./architecture.md) - System design and technical architecture
- [API Documentation](./api.md) - API endpoints and integration guides  
- [Setup Guide](./setup.md) - Installation and configuration
- [Examples](./examples/) - Usage examples and tutorials

## Related Projects

This project is part of the **Bluefly LLM Platform** ecosystem. See the [ecosystem documentation hub](https://gitlab.bluefly.io/ecosystem/bfdocs) for complete platform documentation.

## Support

- **Issues**: [${projectConfig.repository}/issues](${projectConfig.repository}/issues)
- **Documentation Hub**: [bfdocs](https://gitlab.bluefly.io/ecosystem/bfdocs)
- **Platform Repository**: [LLM Platform](https://gitlab.bluefly.io/ecosystem)
`;
  }

  generateArchitectureTemplate(projectKey, projectConfig) {
    return `---
title: ${projectConfig.name} - Architecture
description: Technical architecture and system design for ${projectConfig.name}
category: Architecture
---

# ${projectConfig.name} - Architecture

## System Overview

${projectConfig.description}

## Architecture Diagram

\`\`\`mermaid
graph TB
    subgraph "${projectConfig.name}"
        A[Input Layer] --> B[Processing Layer]
        B --> C[Output Layer]
    end
    
    ${projectConfig.architecture?.dependencies?.map(dep => 
      `D[${dep}] --> A`
    ).join('\\n    ') || ''}
\`\`\`

## Technical Stack

${projectConfig.architecture ? `
- **Language**: ${projectConfig.architecture.language || 'Not specified'}
- **Port**: ${projectConfig.architecture.port || 'N/A'}
- **Type**: ${projectConfig.architecture.type || 'Not specified'}
${projectConfig.architecture.database ? `- **Database**: ${projectConfig.architecture.database}` : ''}
${projectConfig.architecture.frameworks ? `- **Frameworks**: ${projectConfig.architecture.frameworks.join(', ')}` : ''}
` : 'Architecture details to be documented.'}

## Dependencies

${projectConfig.architecture?.dependencies ? `
This service depends on:
${projectConfig.architecture.dependencies.map(dep => `- **${dep}**`).join('\\n')}
` : 'No external dependencies documented.'}

## Deployment Architecture

### Container Strategy
${projectConfig.architecture?.containerized ? `
- **Containerized**: Yes
- **Base Image**: ${projectConfig.architecture.baseImage || 'Not specified'}
` : '- **Containerized**: No'}

### Scaling Considerations

- **Horizontal Scaling**: [Document scaling approach]
- **Resource Requirements**: [Document resource needs]
- **Performance Characteristics**: [Document performance profile]

## Security Architecture

- **Authentication**: [Document auth approach]
- **Authorization**: [Document authz approach]  
- **Data Security**: [Document data security measures]

## Integration Points

### Internal Services
${projectConfig.architecture?.dependencies ? 
  projectConfig.architecture.dependencies.map(dep => `- **${dep}**: [Document integration]`).join('\\n') :
  '- No internal service dependencies documented'
}

### External Services
- [Document external integrations]

## Monitoring & Observability

- **Metrics**: [Document metrics strategy]
- **Logging**: [Document logging approach]
- **Tracing**: [Document tracing implementation]

## Performance Characteristics

- **Throughput**: [Document expected throughput]
- **Latency**: [Document latency requirements]
- **Resource Usage**: [Document resource consumption]

---

*This architecture documentation is part of the [Bluefly LLM Platform](https://gitlab.bluefly.io/ecosystem/bfdocs) ecosystem.*
`;
  }

  generateApiTemplate(projectKey, projectConfig) {
    return `---
title: ${projectConfig.name} - API Documentation
description: API reference and integration guide for ${projectConfig.name}
category: API
---

# ${projectConfig.name} - API Documentation

## Overview

${projectConfig.description}

${projectConfig.architecture?.port ? `
**Base URL**: \`http://localhost:${projectConfig.architecture.port}\`
` : ''}

## Authentication

[Document authentication approach]

## Endpoints

### Health Check

\`\`\`http
GET /health
\`\`\`

**Response:**
\`\`\`json
{
  "status": "healthy",
  "timestamp": "2025-05-28T00:00:00Z",
  "version": "1.0.0"
}
\`\`\`

### [Add main endpoints here]

## SDK Usage

${projectConfig.category === 'Client Libraries' ? `
### Installation

\`\`\`bash
npm install ${projectKey}
\`\`\`

### Basic Usage

\`\`\`typescript
import { ${projectKey.charAt(0).toUpperCase() + projectKey.slice(1)}Client } from '${projectKey}';

const client = new ${projectKey.charAt(0).toUpperCase() + projectKey.slice(1)}Client({
  baseUrl: 'http://localhost:${projectConfig.architecture?.port || '3000'}',
  apiKey: 'your-api-key'
});

// Example usage
const result = await client.someMethod();
\`\`\`
` : `
For SDK integration, use the [\`llm-gateway-sdk\`](../llm-gateway-sdk) package:

\`\`\`typescript
import { ApiClient } from '@bluefly/api-client';

const client = new ApiClient({
  baseUrl: 'http://localhost:${projectConfig.architecture?.port || '3000'}'
});
\`\`\`
`}

## Error Handling

[Document error response format and common errors]

## Rate Limiting

[Document rate limiting policies]

## Examples

### [Example 1]

\`\`\`bash
curl -X GET http://localhost:${projectConfig.architecture?.port || '3000'}/endpoint \\
  -H "Authorization: Bearer your-token"
\`\`\`

## OpenAPI Specification

${projectConfig.repository ? `
The complete OpenAPI specification is available at:
- [OpenAPI YAML](${projectConfig.repository}/blob/main/openapi.yaml)
- [Interactive Documentation](${projectConfig.repository}/blob/main/public/api-docs.html)
` : 'OpenAPI specification location to be documented.'}

---

*This API documentation is part of the [Bluefly LLM Platform](https://gitlab.bluefly.io/ecosystem/bfdocs) ecosystem.*
`;
  }

  generateSetupTemplate(projectKey, projectConfig) {
    return `---
title: ${projectConfig.name} - Setup Guide
description: Installation and configuration guide for ${projectConfig.name}
category: Setup
---

# ${projectConfig.name} - Setup Guide

## Prerequisites

${projectConfig.architecture?.language === 'TypeScript/Node.js' ? `
- Node.js 18+ 
- npm or yarn package manager
` : ''}
${projectConfig.architecture?.language === 'PHP' ? `
- PHP 8.1+
- Composer
` : ''}
${projectConfig.architecture?.language === 'Python' ? `
- Python 3.9+
- pip package manager
` : ''}
- Git
- Docker (optional)

## Installation

### Local Development

\`\`\`bash
# Clone the repository
git clone ${projectConfig.repository || 'https://gitlab.bluefly.io/ecosystem/' + projectKey}
cd ${projectKey}

${projectConfig.architecture?.language === 'TypeScript/Node.js' ? `
# Install dependencies
npm install

# Copy environment template
cp .env.example .env

# Configure environment variables
nano .env

# Start development server
npm run dev
` : ''}
${projectConfig.architecture?.language === 'PHP' ? `
# Install dependencies
composer install

# Copy environment template
cp .env.example .env

# Configure environment variables
nano .env

# Start development server
php -S localhost:8000
` : ''}
${projectConfig.architecture?.language === 'Python' ? `
# Create virtual environment
python -m venv venv
source venv/bin/activate

# Install dependencies
pip install -r requirements.txt

# Copy environment template
cp .env.example .env

# Configure environment variables
nano .env

# Start development server
python app.py
` : ''}
\`\`\`

### Docker Development

${projectConfig.architecture?.containerized ? `
\`\`\`bash
# Build and run with Docker
docker build -t ${projectKey} .
docker run -p ${projectConfig.architecture?.port || '3000'}:${projectConfig.architecture?.port || '3000'} ${projectKey}

# Or use docker-compose
docker-compose up -d
\`\`\`
` : 'Docker configuration not available yet.'}

## Configuration

### Environment Variables

\`\`\`bash
# Core configuration
NODE_ENV=development
PORT=${projectConfig.architecture?.port || '3000'}
${projectConfig.architecture?.database ? `
# Database configuration
DATABASE_URL=postgresql://user:password@localhost:5432/${projectKey}
` : ''}
# Add project-specific variables here
\`\`\`

### Dependencies

${projectConfig.architecture?.dependencies ? `
This service requires the following services to be running:
${projectConfig.architecture.dependencies.map(dep => `
- **${dep}**: [Link to ${dep} setup documentation](../${dep}/setup.md)
`).join('')}

Start dependencies first before running this service.
` : 'No external service dependencies.'}

## Verification

### Health Check

\`\`\`bash
# Verify service is running
curl http://localhost:${projectConfig.architecture?.port || '3000'}/health
\`\`\`

Expected response:
\`\`\`json
{
  "status": "healthy",
  "timestamp": "2025-05-28T00:00:00Z"
}
\`\`\`

### Running Tests

\`\`\`bash
${projectConfig.architecture?.language === 'TypeScript/Node.js' ? `
# Run all tests
npm test

# Run tests with coverage
npm run test:coverage

# Run specific test suite
npm test -- --testPathPattern=integration
` : ''}
${projectConfig.architecture?.language === 'PHP' ? `
# Run PHPUnit tests
composer test

# Run specific test suite
vendor/bin/phpunit tests/Integration
` : ''}
${projectConfig.architecture?.language === 'Python' ? `
# Run pytest
pytest

# Run with coverage
pytest --cov=src
` : ''}
\`\`\`

## Troubleshooting

### Common Issues

1. **Port already in use**
   \`\`\`bash
   # Change port in .env file
   PORT=${(parseInt(projectConfig.architecture?.port || '3000') + 1) || '3001'}
   \`\`\`

2. **Dependencies not found**
   \`\`\`bash
   # Ensure all dependencies are running
   docker-compose ps
   \`\`\`

3. **Environment variables**
   \`\`\`bash
   # Verify environment is loaded
   printenv | grep ${projectKey.toUpperCase()}
   \`\`\`

### Logs

\`\`\`bash
# View application logs
${projectConfig.architecture?.containerized ? `docker logs ${projectKey}` : 'tail -f logs/app.log'}

# View detailed logs
${projectConfig.architecture?.language === 'TypeScript/Node.js' ? 'DEBUG=* npm start' : 'tail -f logs/debug.log'}
\`\`\`

## Next Steps

- Review [Architecture Documentation](./architecture.md)
- Check [API Documentation](./api.md)  
- Browse [Examples](./examples/)
- Visit [Platform Documentation Hub](https://gitlab.bluefly.io/ecosystem/bfdocs)

---

*This setup guide is part of the [Bluefly LLM Platform](https://gitlab.bluefly.io/ecosystem/bfdocs) ecosystem.*
`;
  }

  async generateAggregatedOutput() {
    const outputDir = path.resolve('./public');
    await fs.mkdir(outputDir, { recursive: true });
    
    // Generate main index
    const indexContent = this.generateMainIndex();
    await fs.writeFile(path.join(outputDir, 'index.html'), indexContent);
    
    // Generate project pages
    for (const [projectKey, projectData] of Object.entries(this.projects)) {
      const projectPageContent = this.generateProjectPage(projectKey, projectData);
      await fs.writeFile(path.join(outputDir, `${projectKey}.html`), projectPageContent);
    }
    
    // Generate search index
    await fs.writeFile(
      path.join(outputDir, 'search-index.json'), 
      JSON.stringify(this.aggregatedIndex, null, 2)
    );
    
    // Generate GitLab Pages index
    await fs.writeFile(
      path.join(outputDir, 'projects.json'),
      JSON.stringify(this.projects, null, 2)
    );
    
    console.log(`📚 Generated aggregated documentation in ${outputDir}`);
  }

  generateMainIndex() {
    const projectsByCategory = {};
    
    Object.entries(this.projects).forEach(([key, project]) => {
      const category = project.category || 'General';
      if (!projectsByCategory[category]) {
        projectsByCategory[category] = [];
      }
      projectsByCategory[category].push({ key, ...project });
    });
    
    return `<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Bluefly LLM Platform - Documentation Hub</title>
    <style>
        body { font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif; margin: 0; padding: 20px; background: #f5f7fa; }
        .container { max-width: 1200px; margin: 0 auto; }
        .header { text-align: center; margin-bottom: 40px; }
        .category { margin-bottom: 30px; }
        .category h2 { color: #2d3748; border-bottom: 2px solid #4299e1; padding-bottom: 10px; }
        .projects { display: grid; grid-template-columns: repeat(auto-fit, minmax(350px, 1fr)); gap: 20px; }
        .project-card { background: white; border-radius: 8px; padding: 20px; box-shadow: 0 2px 10px rgba(0,0,0,0.1); }
        .project-title { color: #2b6cb0; font-size: 1.2em; margin-bottom: 8px; }
        .project-description { color: #4a5568; margin-bottom: 15px; line-height: 1.5; }
        .project-meta { font-size: 0.9em; color: #718096; }
        .tech-tags { display: flex; flex-wrap: wrap; gap: 5px; margin-top: 10px; }
        .tech-tag { background: #edf2f7; color: #4a5568; padding: 2px 8px; border-radius: 4px; font-size: 0.8em; }
        .links { margin-top: 15px; }
        .links a { color: #4299e1; text-decoration: none; margin-right: 15px; }
        .links a:hover { text-decoration: underline; }
    </style>
</head>
<body>
    <div class="container">
        <div class="header">
            <h1>🚀 Bluefly LLM Platform</h1>
            <p>Federated Documentation Hub - Production-Ready Drupal AI Integration Platform</p>
            <div style="background: #edf2f7; padding: 15px; border-radius: 6px; margin: 15px 0;">
                <strong>📊 Ecosystem Overview:</strong> 
                ${this.aggregatedIndex.ecosystem.totalProjects} projects • 
                ${this.aggregatedIndex.ecosystem.totalFiles} documentation files • 
                ${this.aggregatedIndex.ecosystem.totalEndpoints} API endpoints • 
                ${this.aggregatedIndex.ecosystem.technologies.length} technologies
            </div>
            <p><em>Last updated: ${this.aggregatedIndex.lastUpdated}</em></p>
        </div>
        
        ${Object.entries(projectsByCategory).map(([category, projects]) => `
        <div class="category">
            <h2>${category}</h2>
            <div class="projects">
                ${projects.map(project => `
                <div class="project-card">
                    <div class="project-title">${project.name}</div>
                    <div class="project-description">${project.description}</div>
                    <div class="project-meta">
                        ${project.architecture?.language ? `<strong>Language:</strong> ${project.architecture.language}<br>` : ''}
                        ${project.architecture?.port ? `<strong>Port:</strong> ${project.architecture.port}<br>` : ''}
                        <strong>Files:</strong> ${project.files?.length || 0} documentation files<br>
                        ${project.enhancedData?.endpoints ? `<strong>Endpoints:</strong> ${project.enhancedData.endpoints.length}<br>` : ''}
                        ${project.enhancedData?.components ? `<strong>Components:</strong> ${project.enhancedData.components.length}<br>` : ''}
                        ${project.enhancedData?.modules ? `<strong>Modules:</strong> ${project.enhancedData.modules.length}<br>` : ''}
                    </div>
                    ${project.architecture?.technologies ? `
                    <div class="tech-tags">
                        ${project.architecture.technologies.map(tech => `<span class="tech-tag">${tech}</span>`).join('')}
                    </div>
                    ` : ''}
                    <div class="links">
                        <a href="${project.key}.html">📖 View Docs</a>
                        <a href="${project.repository}" target="_blank">🔗 Repository</a>
                        ${project.docsPath ? `<a href="file://${project.docsPath}" target="_blank">📁 Local Docs</a>` : ''}
                    </div>
                </div>
                `).join('')}
            </div>
        </div>
        `).join('')}
        
        <div style="text-align: center; margin-top: 40px; padding-top: 20px; border-top: 1px solid #e2e8f0; color: #718096;">
            <p>🔗 <strong>Ecosystem Links:</strong> 
            <a href="https://gitlab.bluefly.io/ecosystem" target="_blank">GitLab Organization</a> | 
            <a href="./search-index.json" target="_blank">Search Index</a> |
            <a href="./projects.json" target="_blank">Projects API</a>
            </p>
        </div>
    </div>
</body>
</html>`;
  }

  generateProjectPage(projectKey, projectData) {
    return `<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>${projectData.name} - Documentation</title>
    <style>
        body { font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif; margin: 0; padding: 20px; background: #f5f7fa; }
        .container { max-width: 1000px; margin: 0 auto; }
        .header { background: white; padding: 30px; border-radius: 8px; margin-bottom: 20px; box-shadow: 0 2px 10px rgba(0,0,0,0.1); }
        .content { background: white; padding: 30px; border-radius: 8px; box-shadow: 0 2px 10px rgba(0,0,0,0.1); }
        .docs-list { margin-top: 20px; }
        .doc-item { border: 1px solid #e2e8f0; border-radius: 6px; padding: 15px; margin-bottom: 10px; }
        .doc-title { font-weight: bold; color: #2b6cb0; }
        .doc-path { font-family: monospace; font-size: 0.9em; color: #718096; }
        .doc-description { margin-top: 5px; color: #4a5568; }
        .back-link { color: #4299e1; text-decoration: none; }
        .back-link:hover { text-decoration: underline; }
    </style>
</head>
<body>
    <div class="container">
        <div class="header">
            <div><a href="index.html" class="back-link">← Back to Hub</a></div>
            <h1>${projectData.name}</h1>
            <p>${projectData.description}</p>
            <p><strong>Category:</strong> ${projectData.category} | 
               <strong>Repository:</strong> <a href="${projectData.repository}" target="_blank">${projectData.repository}</a></p>
        </div>
        
        <div class="content">
            <h2>📋 Documentation Files (${projectData.files?.length || 0})</h2>
            <div class="docs-list">
                ${(projectData.files || []).map(file => `
                <div class="doc-item">
                    <div class="doc-title">${file.title}</div>
                    <div class="doc-path">${file.path}</div>
                    <div class="doc-description">${file.description}</div>
                    <div style="margin-top: 8px; font-size: 0.8em; color: #718096;">
                        Type: ${file.type} | Modified: ${new Date(file.modified).toLocaleDateString()}
                        ${file.tags?.length ? ` | Tags: ${file.tags.join(', ')}` : ''}
                    </div>
                </div>
                `).join('')}
            </div>
            
            ${projectData.architecture ? `
            <h2>🏗️ Architecture</h2>
            <ul>
                ${projectData.architecture.language ? `<li><strong>Language:</strong> ${projectData.architecture.language}</li>` : ''}
                ${projectData.architecture.port ? `<li><strong>Port:</strong> ${projectData.architecture.port}</li>` : ''}
                ${projectData.architecture.type ? `<li><strong>Type:</strong> ${projectData.architecture.type}</li>` : ''}
                ${projectData.architecture.containerized ? `<li><strong>Containerized:</strong> Yes (${projectData.architecture.baseImage})</li>` : ''}
                ${projectData.architecture.technologies?.length ? `<li><strong>Technologies:</strong> ${projectData.architecture.technologies.join(', ')}</li>` : ''}
            </ul>
            ` : ''}
        </div>
    </div>
</body>
</html>`;
  }

  async run() {
    console.log('🚀 Starting Documentation Aggregation...');
    
    await this.loadConfig();
    
    // Scan all projects
    for (const [projectKey, projectConfig] of Object.entries(this.config.projects)) {
      try {
        this.projects[projectKey] = await this.scanProjectDocs(projectKey, projectConfig);
        
        // Add to search index
        this.aggregatedIndex.projects[projectKey] = {
          name: projectConfig.name,
          category: projectConfig.category,
          fileCount: this.projects[projectKey].files.length,
          enhancedData: this.projects[projectKey].enhancedData || {}
        };
        
        // Add category
        const category = projectConfig.category || 'General';
        if (!this.aggregatedIndex.categories[category]) {
          this.aggregatedIndex.categories[category] = [];
        }
        this.aggregatedIndex.categories[category].push(projectKey);
        
      } catch (error) {
        console.error(`❌ Failed to process ${projectKey}:`, error.message);
      }
    }
    
    // Finalize ecosystem statistics
    this.aggregatedIndex.ecosystem.totalProjects = Object.keys(this.projects).length;
    this.aggregatedIndex.ecosystem.technologies = Array.from(this.aggregatedIndex.ecosystem.technologies);
    this.aggregatedIndex.lastUpdated = new Date().toISOString();
    
    // Generate aggregated output
    await this.generateAggregatedOutput();
    
    console.log('✅ Documentation aggregation complete!');
    console.log(`📊 Processed ${this.aggregatedIndex.ecosystem.totalProjects} projects`);
    console.log(`📁 Total files: ${this.aggregatedIndex.ecosystem.totalFiles}`);
    console.log(`🔗 Total endpoints: ${this.aggregatedIndex.ecosystem.totalEndpoints}`);
    console.log(`🛠️  Technologies: ${this.aggregatedIndex.ecosystem.technologies.length}`);
    console.log(`📄 Generated output in ./public/`);
  }
}

// Run if called directly
if (require.main === module) {
  const aggregator = new DocumentationAggregator();
  aggregator.run().catch(console.error);
}

module.exports = DocumentationAggregator;