/**
 * Aggregate OpenAPI specifications from all LLM Platform projects
 * Generates unified API documentation structure
 */

import fs from 'fs-extra';
import path from 'path';
import { fileURLToPath } from 'url';
import { glob } from 'glob';
import YAML from 'yaml';

const __filename = fileURLToPath(import.meta.url);
const __dirname = path.dirname(__filename);

const PROJECT_ROOT = path.resolve(__dirname, '../..');
const OUTPUT_DIR = path.resolve(__dirname, '../docs/api');

// LLM Platform projects with OpenAPI specs
const PROJECTS = [
  {
    name: 'llm-gateway',
    path: 'llm-gateway',
    title: 'LLM Gateway API',
    description: 'Core gateway API with OpenAI v1 compatibility',
    version: '0.1.8'
  },
  {
    name: 'llm-gateway-sdk',
    path: 'llm-gateway-sdk', 
    title: 'LLM Gateway SDK',
    description: 'TypeScript SDK for LLM Gateway with OpenAI compatibility',
    version: '0.1.6'
  },
  {
    name: 'apple-fm',
    path: 'apple-fm',
    title: 'Apple Foundation Models API',
    description: 'Apple Intelligence integration with privacy-first processing',
    version: '0.1.9'
  },
  {
    name: 'llm-mcp',
    path: 'llm-mcp',
    title: 'Model Context Protocol Server API',
    description: 'MCP server for LLM Platform ecosystem integration',
    version: '0.1.6'
  },
  {
    name: 'llmcli',
    path: 'llmcli',
    title: 'LLMCLI Platform Management API',
    description: 'Platform orchestration and service management API',
    version: '0.1.10'
  },
  {
    name: 'tddai',
    path: 'tddai',
    title: 'TDDAI Framework API',
    description: 'AI-enhanced Test-Driven Development framework API',
    version: '0.1.9'
  },
  {
    name: 'bfrfp',
    path: 'bfrfp',
    title: 'BlueFly RFP Generation API',
    description: 'AI-powered RFP automation platform API',
    version: '0.1.7'
  }
];

async function aggregateOpenAPISpecs() {
  console.log('🔄 Aggregating OpenAPI specifications...');
  
  // Ensure output directory exists
  await fs.ensureDir(OUTPUT_DIR);
  
  const specs = [];
  const unified = {
    openapi: '3.1.0',
    info: {
      title: 'LLM Platform Unified API',
      description: 'Comprehensive API documentation for the entire LLM Platform ecosystem',
      version: '1.0.0',
      contact: {
        name: 'LLM Platform Team',
        email: 'team@llm-platform.dev',
        url: 'https://llm-platform.dev'
      },
      license: {
        name: 'MIT',
        url: 'https://opensource.org/licenses/MIT'
      }
    },
    servers: [
      {
        url: 'https://api.llm-platform.dev',
        description: 'Production LLM Platform API'
      },
      {
        url: 'https://staging-api.llm-platform.dev', 
        description: 'Staging environment'
      },
      {
        url: 'http://localhost:3000',
        description: 'Local development server'
      }
    ],
    paths: {},
    components: {
      schemas: {},
      securitySchemes: {
        BearerAuth: {
          type: 'http',
          scheme: 'bearer',
          bearerFormat: 'JWT'
        },
        ApiKeyAuth: {
          type: 'apiKey',
          in: 'header',
          name: 'X-API-Key'
        }
      }
    },
    tags: []
  };

  // Process each project
  for (const project of PROJECTS) {
    const specPath = path.join(PROJECT_ROOT, project.path, 'openapi.yaml');
    
    console.log(`🔍 Checking: ${specPath}`);
    
    try {
      if (await fs.pathExists(specPath)) {
        console.log(`📖 Processing ${project.name}...`);
        
        const specContent = await fs.readFile(specPath, 'utf8');
        const spec = YAML.parse(specContent);
        
        // Copy individual spec
        await fs.writeFile(
          path.join(OUTPUT_DIR, `${project.name}.yaml`),
          specContent
        );
        
        // Add to unified spec
        if (spec.paths) {
          Object.assign(unified.paths, spec.paths);
        }
        
        if (spec.components?.schemas) {
          Object.assign(unified.components.schemas, spec.components.schemas);
        }
        
        if (spec.tags) {
          unified.tags.push(...spec.tags);
        }
        
        specs.push({
          ...project,
          spec,
          hasSpec: true
        });
        
        console.log(`✅ Processed ${project.name}`);
      } else {
        console.log(`⚠️  No OpenAPI spec found for ${project.name}`);
        specs.push({
          ...project,
          hasSpec: false
        });
      }
    } catch (error) {
      console.error(`❌ Error processing ${project.name}:`, error.message);
      specs.push({
        ...project,
        hasSpec: false,
        error: error.message
      });
    }
  }
  
  // Write unified specification
  await fs.writeFile(
    path.join(OUTPUT_DIR, 'unified.yaml'),
    YAML.stringify(unified, { indent: 2 })
  );
  
  // Generate API index
  const apiIndex = {
    title: 'LLM Platform API Documentation',
    description: 'Complete API reference for all LLM Platform services',
    lastUpdated: new Date().toISOString(),
    specs: specs.map(s => ({
      name: s.name,
      title: s.title,
      description: s.description,
      version: s.version,
      hasSpec: s.hasSpec,
      specFile: s.hasSpec ? `${s.name}.yaml` : null,
      error: s.error || null
    }))
  };
  
  await fs.writeFile(
    path.join(OUTPUT_DIR, 'index.json'),
    JSON.stringify(apiIndex, null, 2)
  );
  
  console.log(`📋 Generated API documentation index with ${specs.filter(s => s.hasSpec).length}/${specs.length} specs`);
  console.log(`📁 Output directory: ${OUTPUT_DIR}`);
  
  return apiIndex;
}

if (import.meta.url === `file://${process.argv[1]}`) {
  aggregateOpenAPISpecs()
    .then(() => {
      console.log('✅ OpenAPI aggregation completed successfully');
      process.exit(0);
    })
    .catch((error) => {
      console.error('❌ OpenAPI aggregation failed:', error);
      process.exit(1);
    });
}

export default aggregateOpenAPISpecs;