#!/usr/bin/env node
/**
 * TDD Compliance Report Generator
 * 
 * This script analyzes the codebase and test coverage results to generate
 * a comprehensive TDD compliance report.
 *
 * TODO: Implementation tasks:
 * - [ ] Enhance TDD compliance metrics
 * - [ ] Add visualization of test coverage over time
 * - [ ] Implement automated test quality assessment
 * - [ ] Create detailed recommendations for improving tests
 * - [ ] Add support for multi-language projects
 * - [ ] Generate compliance badges for README files
 * - [ ] Integrate with GitLab reporting API
 */

const fs = require('fs');
const path = require('path');
const { execSync } = require('child_process');

// Configuration
const SOURCE_DIRS = ['src', 'scripts'];
const TEST_DIRS = ['__tests__', 'test', 'tests'];
const COVERAGE_DIR = 'coverage';
const OUTPUT_DIR = 'public';
const OUTPUT_FILE = path.join(OUTPUT_DIR, 'tdd-compliance-report.html');

// Ensure output directory exists
if (!fs.existsSync(OUTPUT_DIR)) {
  fs.mkdirSync(OUTPUT_DIR, { recursive: true });
}

// Get all JS/TS source files
function getSourceFiles() {
  const files = [];
  
  for (const dir of SOURCE_DIRS) {
    if (!fs.existsSync(dir)) continue;
    
    const cmd = `find ${dir} -type f -name "*.js" -o -name "*.ts" -o -name "*.jsx" -o -name "*.tsx" | grep -v "node_modules" | grep -v ".test."`;
    try {
      const output = execSync(cmd).toString();
      files.push(...output.split('\n').filter(Boolean));
    } catch (error) {
      console.error(`Error finding source files in ${dir}:`, error.message);
    }
  }
  
  return files;
}

// Get all test files
function getTestFiles() {
  const files = [];
  
  // Look in specific test directories
  for (const dir of TEST_DIRS) {
    if (!fs.existsSync(dir)) continue;
    
    const cmd = `find ${dir} -type f -name "*.test.js" -o -name "*.test.ts" -o -name "*.test.jsx" -o -name "*.test.tsx" | grep -v "node_modules"`;
    try {
      const output = execSync(cmd).toString();
      files.push(...output.split('\n').filter(Boolean));
    } catch (error) {
      console.error(`Error finding test files in ${dir}:`, error.message);
    }
  }
  
  // Also look for test files alongside source files
  for (const dir of SOURCE_DIRS) {
    if (!fs.existsSync(dir)) continue;
    
    const cmd = `find ${dir} -type f -name "*.test.js" -o -name "*.test.ts" -o -name "*.test.jsx" -o -name "*.test.tsx" | grep -v "node_modules"`;
    try {
      const output = execSync(cmd).toString();
      files.push(...output.split('\n').filter(Boolean));
    } catch (error) {
      console.error(`Error finding test files in ${dir}:`, error.message);
    }
  }
  
  return files;
}

// Parse coverage data
function getCoverageData() {
  const coveragePath = path.join(COVERAGE_DIR, 'coverage-final.json');
  
  if (!fs.existsSync(coveragePath)) {
    console.warn(`Warning: Coverage data not found at ${coveragePath}`);
    return null;
  }
  
  try {
    const coverageData = JSON.parse(fs.readFileSync(coveragePath, 'utf8'));
    return coverageData;
  } catch (error) {
    console.error('Error parsing coverage data:', error.message);
    return null;
  }
}

// Calculate file-level coverage
function calculateFileCoverage(coverageData) {
  if (!coverageData) return {};
  
  const fileCoverage = {};
  
  for (const filePath in coverageData) {
    const data = coverageData[filePath];
    const statements = data.statementMap ? Object.keys(data.statementMap).length : 0;
    const coveredStatements = data.s ? Object.values(data.s).filter(v => v > 0).length : 0;
    const branches = data.branchMap ? Object.keys(data.branchMap).length * 2 : 0; // Each branch has 2 paths
    const coveredBranches = data.b ? Object.values(data.b).flat().filter(v => v > 0).length : 0;
    const functions = data.fnMap ? Object.keys(data.fnMap).length : 0;
    const coveredFunctions = data.f ? Object.values(data.f).filter(v => v > 0).length : 0;
    
    fileCoverage[filePath] = {
      statements: statements > 0 ? (coveredStatements / statements) * 100 : 100,
      branches: branches > 0 ? (coveredBranches / branches) * 100 : 100,
      functions: functions > 0 ? (coveredFunctions / functions) * 100 : 100,
      lines: 0, // Will calculate from lcov if available
      overall: statements > 0 ? 
        ((coveredStatements + coveredBranches + coveredFunctions) / 
        (statements + branches + functions)) * 100 : 100
    };
  }
  
  // Try to get line coverage from lcov.info if available
  const lcovPath = path.join(COVERAGE_DIR, 'lcov.info');
  if (fs.existsSync(lcovPath)) {
    const lcovContent = fs.readFileSync(lcovPath, 'utf8');
    const fileBlocks = lcovContent.split('end_of_record');
    
    for (const block of fileBlocks) {
      const sourceFileMatch = block.match(/SF:(.+)/);
      if (!sourceFileMatch) continue;
      
      const sourceFile = sourceFileMatch[1];
      const linesTotal = block.match(/LF:(\d+)/);
      const linesCovered = block.match(/LH:(\d+)/);
      
      if (linesTotal && linesCovered && fileCoverage[sourceFile]) {
        fileCoverage[sourceFile].lines = 
          (parseInt(linesCovered[1], 10) / parseInt(linesTotal[1], 10)) * 100;
      }
    }
  }
  
  return fileCoverage;
}

// Match source files to test files
function matchSourceToTestFiles(sourceFiles, testFiles) {
  const matches = {};
  
  for (const sourceFile of sourceFiles) {
    const basename = path.basename(sourceFile).replace(/\.(js|ts|jsx|tsx)$/, '');
    const matchingTests = testFiles.filter(testFile => {
      const testBasename = path.basename(testFile).replace(/\.test\.(js|ts|jsx|tsx)$/, '');
      return testBasename === basename;
    });
    
    matches[sourceFile] = matchingTests;
  }
  
  return matches;
}

// Get git history for first commit dates
function getFileGitHistory(files) {
  const history = {};
  
  for (const file of files) {
    try {
      // Get first commit hash
      const firstCommitHash = execSync(`git log --follow --format="%H" --reverse "${file}" | head -1`, { encoding: 'utf8' }).trim();
      
      if (firstCommitHash) {
        // Get commit date
        const commitDate = execSync(`git show -s --format="%at" "${firstCommitHash}"`, { encoding: 'utf8' }).trim();
        history[file] = {
          firstCommitHash,
          firstCommitDate: parseInt(commitDate, 10) * 1000, // Convert to milliseconds
          firstCommitFormatted: execSync(`git show -s --format="%ci" "${firstCommitHash}"`, { encoding: 'utf8' }).trim()
        };
      }
    } catch (error) {
      console.warn(`Warning: Could not get git history for ${file}:`, error.message);
    }
  }
  
  return history;
}

// Check test-first compliance
function checkTestFirstCompliance(matches, sourceHistory, testHistory) {
  const compliance = {};
  
  for (const [sourceFile, testFiles] of Object.entries(matches)) {
    if (testFiles.length === 0) {
      compliance[sourceFile] = {
        hasTests: false,
        testFirstCompliant: false,
        message: 'No tests found'
      };
      continue;
    }
    
    // Source file history
    const sourceData = sourceHistory[sourceFile];
    if (!sourceData) {
      compliance[sourceFile] = {
        hasTests: true,
        testFirstCompliant: null,
        message: 'Source file history not available'
      };
      continue;
    }
    
    // Find the earliest test file
    let earliestTestDate = Infinity;
    let earliestTestFile = null;
    
    for (const testFile of testFiles) {
      const testData = testHistory[testFile];
      if (testData && testData.firstCommitDate < earliestTestDate) {
        earliestTestDate = testData.firstCommitDate;
        earliestTestFile = testFile;
      }
    }
    
    if (!earliestTestFile) {
      compliance[sourceFile] = {
        hasTests: true,
        testFirstCompliant: null,
        message: 'Test file history not available'
      };
      continue;
    }
    
    // Compare dates
    const testData = testHistory[earliestTestFile];
    const isTestFirst = testData.firstCommitDate <= sourceData.firstCommitDate;
    
    compliance[sourceFile] = {
      hasTests: true,
      testFirstCompliant: isTestFirst,
      testFile: earliestTestFile,
      sourceFirstCommit: sourceData.firstCommitFormatted,
      testFirstCommit: testData.firstCommitFormatted,
      message: isTestFirst ? 
        'Test was written before implementation' : 
        'Implementation was written before test'
    };
  }
  
  return compliance;
}

// Generate the HTML report
function generateHtmlReport(sourceFiles, testFiles, matches, coverage, compliance) {
  const sourceCount = sourceFiles.length;
  const testCount = testFiles.length;
  const testedFileCount = Object.values(matches).filter(tests => tests.length > 0).length;
  const testCoveragePercent = sourceCount > 0 ? (testedFileCount / sourceCount) * 100 : 0;
  
  // Calculate test-first compliance stats
  const compliantCount = Object.values(compliance).filter(c => c.testFirstCompliant === true).length;
  const nonCompliantCount = Object.values(compliance).filter(c => c.testFirstCompliant === false).length;
  const unknownCount = Object.values(compliance).filter(c => c.testFirstCompliant === null).length;
  const tddCompliancePercent = compliantCount + nonCompliantCount > 0 ? 
    (compliantCount / (compliantCount + nonCompliantCount)) * 100 : 0;
  
  // Calculate overall coverage if available
  let overallCoverage = null;
  if (coverage) {
    let statements = 0;
    let branches = 0;
    let functions = 0;
    let lines = 0;
    let count = 0;
    
    for (const data of Object.values(coverage)) {
      statements += data.statements;
      branches += data.branches;
      functions += data.functions;
      lines += data.lines;
      count++;
    }
    
    if (count > 0) {
      overallCoverage = {
        statements: statements / count,
        branches: branches / count,
        functions: functions / count,
        lines: lines / count,
        overall: (statements + branches + functions + lines) / (count * 4)
      };
    }
  }
  
  // Create HTML report
  const html = `
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>TDD Compliance Report</title>
  <style>
    body {
      font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif;
      line-height: 1.5;
      color: #333;
      max-width: 1200px;
      margin: 0 auto;
      padding: 1rem;
    }
    
    h1, h2, h3 {
      color: #0366d6;
    }
    
    .summary {
      display: grid;
      grid-template-columns: repeat(auto-fit, minmax(200px, 1fr));
      gap: 1rem;
      margin-bottom: 2rem;
    }
    
    .metric-card {
      background-color: #f6f8fa;
      border-radius: 6px;
      padding: 1rem;
      box-shadow: 0 1px 3px rgba(0,0,0,0.12);
    }
    
    .metric-value {
      font-size: 2rem;
      font-weight: bold;
      margin: 0.5rem 0;
    }
    
    .status-good {
      color: #28a745;
    }
    
    .status-warning {
      color: #f9c513;
    }
    
    .status-bad {
      color: #d73a49;
    }
    
    table {
      width: 100%;
      border-collapse: collapse;
      margin-bottom: 2rem;
      font-size: 0.9rem;
    }
    
    th, td {
      padding: 0.5rem;
      text-align: left;
      border-bottom: 1px solid #e1e4e8;
    }
    
    th {
      background-color: #f6f8fa;
      font-weight: 600;
    }
    
    tr:hover {
      background-color: #f6f8fa;
    }
    
    .bar-container {
      width: 100%;
      background-color: #e1e4e8;
      border-radius: 3px;
      height: 8px;
    }
    
    .bar {
      height: 100%;
      border-radius: 3px;
    }
    
    .compliance-ok {
      background-color: #28a745;
    }
    
    .compliance-warning {
      background-color: #f9c513;
    }
    
    .compliance-fail {
      background-color: #d73a49;
    }
    
    .color-legend {
      display: flex;
      gap: 1rem;
      margin-bottom: 1rem;
    }
    
    .color-box {
      display: inline-block;
      width: 16px;
      height: 16px;
      border-radius: 3px;
      margin-right: 0.25rem;
      vertical-align: middle;
    }
  </style>
</head>
<body>
  <h1>TDD Compliance Report</h1>
  <p>Generated on ${new Date().toLocaleString()}</p>
  
  <div class="summary">
    <div class="metric-card">
      <h3>Test Coverage</h3>
      <div class="metric-value ${testCoveragePercent >= 95 ? 'status-good' : testCoveragePercent >= 80 ? 'status-warning' : 'status-bad'}">
        ${testCoveragePercent.toFixed(1)}%
      </div>
      <p>${testedFileCount} of ${sourceCount} files have tests</p>
    </div>
    
    <div class="metric-card">
      <h3>TDD Compliance</h3>
      <div class="metric-value ${tddCompliancePercent >= 95 ? 'status-good' : tddCompliancePercent >= 80 ? 'status-warning' : 'status-bad'}">
        ${tddCompliancePercent.toFixed(1)}%
      </div>
      <p>${compliantCount} files follow test-first approach</p>
    </div>
    
    ${overallCoverage ? `
    <div class="metric-card">
      <h3>Code Coverage</h3>
      <div class="metric-value ${overallCoverage.overall >= 95 ? 'status-good' : overallCoverage.overall >= 80 ? 'status-warning' : 'status-bad'}">
        ${overallCoverage.overall.toFixed(1)}%
      </div>
      <p>Overall code coverage from tests</p>
    </div>
    ` : ''}
    
    <div class="metric-card">
      <h3>Test Count</h3>
      <div class="metric-value">
        ${testCount}
      </div>
      <p>Total test files in codebase</p>
    </div>
  </div>
  
  ${overallCoverage ? `
  <h2>Code Coverage Breakdown</h2>
  <table>
    <tr>
      <th>Category</th>
      <th>Coverage</th>
      <th>Status</th>
    </tr>
    <tr>
      <td>Statements</td>
      <td>${overallCoverage.statements.toFixed(1)}%</td>
      <td>
        <div class="bar-container">
          <div class="bar ${overallCoverage.statements >= 95 ? 'compliance-ok' : overallCoverage.statements >= 80 ? 'compliance-warning' : 'compliance-fail'}" 
               style="width: ${Math.min(100, overallCoverage.statements)}%"></div>
        </div>
      </td>
    </tr>
    <tr>
      <td>Branches</td>
      <td>${overallCoverage.branches.toFixed(1)}%</td>
      <td>
        <div class="bar-container">
          <div class="bar ${overallCoverage.branches >= 95 ? 'compliance-ok' : overallCoverage.branches >= 80 ? 'compliance-warning' : 'compliance-fail'}" 
               style="width: ${Math.min(100, overallCoverage.branches)}%"></div>
        </div>
      </td>
    </tr>
    <tr>
      <td>Functions</td>
      <td>${overallCoverage.functions.toFixed(1)}%</td>
      <td>
        <div class="bar-container">
          <div class="bar ${overallCoverage.functions >= 95 ? 'compliance-ok' : overallCoverage.functions >= 80 ? 'compliance-warning' : 'compliance-fail'}" 
               style="width: ${Math.min(100, overallCoverage.functions)}%"></div>
        </div>
      </td>
    </tr>
    <tr>
      <td>Lines</td>
      <td>${overallCoverage.lines.toFixed(1)}%</td>
      <td>
        <div class="bar-container">
          <div class="bar ${overallCoverage.lines >= 95 ? 'compliance-ok' : overallCoverage.lines >= 80 ? 'compliance-warning' : 'compliance-fail'}" 
               style="width: ${Math.min(100, overallCoverage.lines)}%"></div>
        </div>
      </td>
    </tr>
  </table>
  ` : ''}
  
  <h2>TDD Compliance Details</h2>
  <div class="color-legend">
    <div><span class="color-box compliance-ok"></span> Test-first compliant</div>
    <div><span class="color-box compliance-warning"></span> Has tests but not test-first</div>
    <div><span class="color-box compliance-fail"></span> Missing tests</div>
  </div>
  
  <table>
    <tr>
      <th>Source File</th>
      <th>Tests</th>
      <th>TDD Compliance</th>
      <th>Test Written</th>
      <th>Implementation Written</th>
      ${coverage ? '<th>Coverage</th>' : ''}
    </tr>
    ${sourceFiles.map(file => {
      const fileMatches = matches[file] || [];
      const fileCompliance = compliance[file] || { hasTests: false, testFirstCompliant: false, message: 'Unknown' };
      const fileCoverage = coverage && coverage[file];
      
      return `
      <tr>
        <td>${file}</td>
        <td>${fileMatches.length > 0 ? fileMatches.join('<br>') : 'None'}</td>
        <td>
          <div class="bar-container">
            <div class="bar ${fileCompliance.testFirstCompliant ? 'compliance-ok' : fileCompliance.hasTests ? 'compliance-warning' : 'compliance-fail'}" 
                 style="width: ${fileCompliance.testFirstCompliant ? 100 : fileCompliance.hasTests ? 50 : 0}%"></div>
          </div>
          ${fileCompliance.message}
        </td>
        <td>${fileCompliance.testFirstCompliant !== null ? (fileCompliance.testFirstCommit || 'Unknown') : 'N/A'}</td>
        <td>${fileCompliance.sourceFirstCommit || 'Unknown'}</td>
        ${fileCoverage ? `
        <td>
          <div class="bar-container">
            <div class="bar ${fileCoverage.overall >= 95 ? 'compliance-ok' : fileCoverage.overall >= 80 ? 'compliance-warning' : 'compliance-fail'}" 
                 style="width: ${Math.min(100, fileCoverage.overall)}%"></div>
          </div>
          ${fileCoverage.overall.toFixed(1)}%
        </td>
        ` : ''}
      </tr>
      `;
    }).join('')}
  </table>
  
  <h2>Improvement Recommendations</h2>
  <ul>
    ${sourceFiles.filter(file => !matches[file] || matches[file].length === 0).length > 0 ? `
    <li>
      <strong>Add tests for these files:</strong>
      <ul>
        ${sourceFiles.filter(file => !matches[file] || matches[file].length === 0).map(file => `
        <li>${file}</li>
        `).join('')}
      </ul>
    </li>
    ` : ''}
    
    ${Object.entries(compliance)
      .filter(([_, data]) => data.hasTests && data.testFirstCompliant === false)
      .length > 0 ? `
    <li>
      <strong>Review implementation-first files:</strong>
      <ul>
        ${Object.entries(compliance)
          .filter(([_, data]) => data.hasTests && data.testFirstCompliant === false)
          .map(([file, data]) => `
        <li>${file} - Test was written ${new Date(new Date(data.testFirstCommit) - new Date(data.sourceFirstCommit)).getUTCDate() - 1} days after implementation</li>
        `).join('')}
      </ul>
    </li>
    ` : ''}
    
    ${coverage ? Object.entries(coverage)
      .filter(([_, data]) => data.overall < 95)
      .length > 0 ? `
    <li>
      <strong>Improve coverage for these files:</strong>
      <ul>
        ${Object.entries(coverage)
          .filter(([_, data]) => data.overall < 95)
          .sort((a, b) => a[1].overall - b[1].overall)
          .map(([file, data]) => `
        <li>${file} - Current: ${data.overall.toFixed(1)}%, Target: 95%</li>
        `).join('')}
      </ul>
    </li>
    ` : '' : ''}
  </ul>
  
  <p>Learn more about TDD compliance in the <a href="../TDD-ENFORCEMENT-SUMMARY.md">TDD Enforcement Summary</a> and <a href="../TDD-TASK-TRACKING.md">TDD Task Tracking</a> documents.</p>
</body>
</html>
  `;
  
  fs.writeFileSync(OUTPUT_FILE, html);
  console.log(`TDD compliance report generated at ${OUTPUT_FILE}`);
}

// Main execution
(async function main() {
  console.log('Generating TDD compliance report...');
  
  // Step 1: Get all source and test files
  const sourceFiles = getSourceFiles();
  const testFiles = getTestFiles();
  console.log(`Found ${sourceFiles.length} source files and ${testFiles.length} test files`);
  
  // Step 2: Match source files to test files
  const matches = matchSourceToTestFiles(sourceFiles, testFiles);
  
  // Step 3: Get coverage data if available
  const coverageData = getCoverageData();
  const fileCoverage = coverageData ? calculateFileCoverage(coverageData) : null;
  
  // Step 4: Get git history for source and test files
  const sourceHistory = getFileGitHistory(sourceFiles);
  const testHistory = getFileGitHistory(testFiles);
  
  // Step 5: Check test-first compliance
  const compliance = checkTestFirstCompliance(matches, sourceHistory, testHistory);
  
  // Step 6: Generate HTML report
  generateHtmlReport(sourceFiles, testFiles, matches, fileCoverage, compliance);
})();