#!/bin/bash
# Test-First Validation Script
# Ensures tests were written before implementation files by comparing git history
#
# TODO: Implementation tasks:
# - [ ] Add pre-commit hooks to validate tests exist for new code
# - [ ] Enhance test-first validation with accurate timestamp checking
# - [ ] Implement bypass approval workflow for legacy code
# - [ ] Create dashboard for TDD compliance metrics
# - [ ] Add reporting to CI/CD pipeline
# - [ ] Support configuration for custom test directories
# - [ ] Implement documentation of test requirements

set -e

BLUE='\033[0;34m'
GREEN='\033[0;32m'
RED='\033[0;31m'
YELLOW='\033[1;33m'
NC='\033[0m' # No Color

echo -e "${BLUE}======================================================${NC}"
echo -e "${BLUE}           TEST-FIRST VALIDATION CHECK                ${NC}"
echo -e "${BLUE}======================================================${NC}"
echo ""

# Function to get the first commit hash for a file
get_first_commit() {
  git log --follow --format="%H" --reverse "$1" | head -1
}

# Function to get the commit date for a hash
get_commit_date() {
  git show -s --format="%at" "$1"
}

# Get all modified files in the current branch/PR
if [ -n "$CI_MERGE_REQUEST_TARGET_BRANCH_NAME" ]; then
  # Running in a merge request
  TARGET_BRANCH="$CI_MERGE_REQUEST_TARGET_BRANCH_NAME"
else
  # Running on a branch
  TARGET_BRANCH="origin/main"
fi

echo -e "${YELLOW}Comparing against:${NC} $TARGET_BRANCH"
echo ""

# Get all modified JS/TS files that are not test files
IMPLEMENTATION_FILES=$(git diff --name-only "$TARGET_BRANCH" | grep -E '\.jsx?$|\.tsx?$' | grep -v '\.test\.' || true)

if [ -z "$IMPLEMENTATION_FILES" ]; then
  echo -e "${GREEN}No implementation files were modified. Test-first check passed!${NC}"
  exit 0
fi

FAILED=0
PASSED=0
SKIPPED=0
TOTAL=0

for IMPL_FILE in $IMPLEMENTATION_FILES; do
  TOTAL=$((TOTAL + 1))
  
  # Determine the expected test file path
  FILE_DIR=$(dirname "$IMPL_FILE")
  FILE_BASE=$(basename "$IMPL_FILE" | sed -E 's/\.(js|ts|jsx|tsx)$//')
  
  # Check for test files in various common test locations
  TEST_FILE=""
  
  # Test files in __tests__ directory (Jest standard)
  POSSIBLE_TEST_PATHS=(
    "$FILE_DIR/__tests__/${FILE_BASE}.test.js"
    "$FILE_DIR/__tests__/${FILE_BASE}.test.ts"
    "__tests__/$FILE_DIR/${FILE_BASE}.test.js"
    "__tests__/$FILE_DIR/${FILE_BASE}.test.ts"
    "__tests__/${FILE_BASE}.test.js"
    "__tests__/${FILE_BASE}.test.ts"
    "test/$FILE_DIR/${FILE_BASE}.test.js"
    "test/$FILE_DIR/${FILE_BASE}.test.ts"
    "tests/$FILE_DIR/${FILE_BASE}.test.js"
    "tests/$FILE_DIR/${FILE_BASE}.test.ts"
    "$FILE_DIR/${FILE_BASE}.test.js"
    "$FILE_DIR/${FILE_BASE}.test.ts"
  )
  
  for PATH in "${POSSIBLE_TEST_PATHS[@]}"; do
    if [ -f "$PATH" ]; then
      TEST_FILE="$PATH"
      break
    fi
  done
  
  if [ -z "$TEST_FILE" ]; then
    echo -e "${RED}✗ No test found for:${NC} $IMPL_FILE"
    FAILED=$((FAILED + 1))
    continue
  fi
  
  # Get first commit for both files
  IMPL_FIRST_COMMIT=$(get_first_commit "$IMPL_FILE")
  TEST_FIRST_COMMIT=$(get_first_commit "$TEST_FILE")
  
  # If we can't determine history, skip
  if [ -z "$IMPL_FIRST_COMMIT" ] || [ -z "$TEST_FIRST_COMMIT" ]; then
    echo -e "${YELLOW}⚠ Skipping file with incomplete history:${NC} $IMPL_FILE"
    SKIPPED=$((SKIPPED + 1))
    continue
  fi
  
  # Get commit dates
  IMPL_COMMIT_DATE=$(get_commit_date "$IMPL_FIRST_COMMIT")
  TEST_COMMIT_DATE=$(get_commit_date "$TEST_FIRST_COMMIT")
  
  # Compare dates
  if [ "$TEST_COMMIT_DATE" -le "$IMPL_COMMIT_DATE" ]; then
    echo -e "${GREEN}✓ Test was written before implementation:${NC} $TEST_FILE"
    PASSED=$((PASSED + 1))
  else
    echo -e "${RED}✗ Test was written AFTER implementation:${NC} $IMPL_FILE"
    echo -e "  ${YELLOW}Implementation first commit:${NC} $(git show -s --format="%h %ci" "$IMPL_FIRST_COMMIT")"
    echo -e "  ${YELLOW}Test first commit:${NC} $(git show -s --format="%h %ci" "$TEST_FIRST_COMMIT")"
    FAILED=$((FAILED + 1))
  fi
done

echo ""
echo -e "${BLUE}======================================================${NC}"
echo -e "${BLUE}              TEST-FIRST VALIDATION RESULTS           ${NC}"
echo -e "${BLUE}======================================================${NC}"
echo -e "Total files checked: ${TOTAL}"
echo -e "Passed: ${GREEN}${PASSED}${NC}"
echo -e "Failed: ${RED}${FAILED}${NC}"
echo -e "Skipped (incomplete history): ${YELLOW}${SKIPPED}${NC}"
echo ""

if [ $FAILED -gt 0 ]; then
  echo -e "${RED}TEST-FIRST VALIDATION FAILED!${NC}"
  echo -e "${RED}There are implementation files that were written before their tests.${NC}"
  echo -e "${YELLOW}Remember: In TDD, tests must be written before implementation.${NC}"
  
  # Allow override with environment variable if needed
  if [ "$ENFORCE_TEST_FIRST" = "false" ]; then
    echo -e "${YELLOW}Warning: Validation failed but ENFORCE_TEST_FIRST=false, allowing pipeline to continue${NC}"
    exit 0
  else
    exit 1
  fi
else
  echo -e "${GREEN}TEST-FIRST VALIDATION PASSED!${NC}"
  echo -e "${GREEN}All implementation files have tests that were written first.${NC}"
  exit 0
fi