#!/usr/bin/env node

/**
 * Documentation Consolidation Script
 * 
 * This script consolidates scattered documentation from across the LLM platform
 * into a properly organized structure within the llm-docs project.
 */

const fs = require('fs').promises;
const path = require('path');

const LLM_ROOT = '/Users/flux423/Sites/LLM';
const DOCS_ROOT = '/Users/flux423/Sites/LLM/_CommonNPM/llm-docs';
const DOCS_DIR = path.join(DOCS_ROOT, 'docs');

// Source projects and their documentation
const PROJECTS = {
  packages: [
    'apple-fm',
    'bfrfp', 
    'llm-gateway',
    'llm-gateway-sdk',
    'llm-mcp',
    'llm-ui',
    'llmcli',
    'secure-project',
    'tddai'
  ],
  drupal: {
    basePath: path.join(LLM_ROOT, 'DEMO/web/modules/custom'),
    modules: [
      'ai_provider_apple',
      'ai_provider_langchain', 
      'ai_agent_orchestra',
      'alternative_services',
      'api_normalization',
      'gov_compliance',
      'llm',
      'mcp_client_extras',
      'recipe_onboarding'
    ]
  },
  cicd: {
    basePath: path.join(LLM_ROOT, 'bfcicomponents'),
    project: 'bfcicomponents'
  },
  models: {
    basePath: path.join(LLM_ROOT, 'tddai-model'),
    project: 'tddai-model'
  }
};

class DocumentationConsolidator {
  constructor() {
    this.consolidatedFiles = [];
    this.brokenLinks = [];
    this.duplicateFiles = [];
  }

  async run() {
    console.log('🚀 Starting documentation consolidation...');
    
    try {
      // Step 1: Clean up __DELETE_LATER and preserve valuable content
      await this.processDeleteLaterDirectory();
      
      // Step 2: Consolidate NPM package documentation
      await this.consolidatePackageDocumentation();
      
      // Step 3: Consolidate Drupal module documentation
      await this.consolidateDrupalDocumentation();
      
      // Step 4: Consolidate CI/CD documentation  
      await this.consolidateCICDDocumentation();
      
      // Step 5: Fix navigation and links
      await this.fixNavigationAndLinks();
      
      // Step 6: Generate consolidated index pages
      await this.generateIndexPages();
      
      // Step 7: Generate final report
      await this.generateConsolidationReport();
      
      console.log('✅ Documentation consolidation complete!');
      
    } catch (error) {
      console.error('❌ Consolidation failed:', error);
      process.exit(1);
    }
  }

  async processDeleteLaterDirectory() {
    console.log('📂 Processing __DELETE_LATER directory...');
    
    const deleteLaterPath = path.join(DOCS_ROOT, '__DELETE_LATER');
    
    try {
      const items = await fs.readdir(deleteLaterPath);
      
      for (const item of items) {
        const itemPath = path.join(deleteLaterPath, item);
        const stats = await fs.stat(itemPath);
        
        if (stats.isDirectory()) {
          // Check if directory contains valuable documentation
          const hasValueableContent = await this.hasValueableContent(itemPath);
          
          if (hasValueableContent) {
            await this.preserveValueableContent(itemPath, item);
          }
        } else if (item.endsWith('.md')) {
          // Check individual markdown files
          const content = await fs.readFile(itemPath, 'utf8');
          if (this.isValueableContent(content)) {
            await this.preserveFile(itemPath, item);
          }
        }
      }
      
    } catch (error) {
      console.warn('⚠️  Could not process __DELETE_LATER:', error.message);
    }
  }

  async hasValueableContent(dirPath) {
    try {
      const files = await this.getMarkdownFiles(dirPath);
      
      for (const file of files) {
        const content = await fs.readFile(file, 'utf8');
        if (this.isValueableContent(content)) {
          return true;
        }
      }
      
      return false;
    } catch (error) {
      return false;
    }
  }

  isValueableContent(content) {
    // Check for valuable content indicators
    const valuablePatterns = [
      /api\s+(documentation|reference|guide)/i,
      /installation\s+guide/i,
      /configuration\s+guide/i,
      /troubleshooting/i,
      /architecture\s+(diagram|overview)/i,
      /migration\s+guide/i,
      /security\s+(guide|best\s+practices)/i,
      /deployment\s+(guide|instructions)/i
    ];
    
    // Exclude patterns for non-valuable content
    const excludePatterns = [
      /^\s*#\s*(TODO|FIXME)/m,
      /placeholder\s+content/i,
      /coming\s+soon/i,
      /not\s+implemented/i
    ];
    
    // Check for valuable patterns
    const hasValuableContent = valuablePatterns.some(pattern => pattern.test(content));
    
    // Check for exclusion patterns
    const hasExcludedContent = excludePatterns.some(pattern => pattern.test(content));
    
    // Content is valuable if it has valuable patterns and is substantial
    return hasValuableContent && !hasExcludedContent && content.length > 500;
  }

  async preserveValueableContent(sourcePath, itemName) {
    console.log(`  📄 Preserving valuable content from: ${itemName}`);
    
    // Create archive directory for preserved content
    const archivePath = path.join(DOCS_DIR, 'archive', itemName);
    await this.ensureDirectory(archivePath);
    
    // Copy valuable files
    await this.copyDirectory(sourcePath, archivePath);
    
    this.consolidatedFiles.push({
      source: sourcePath,
      destination: archivePath,
      type: 'archived_valuable_content'
    });
  }

  async consolidatePackageDocumentation() {
    console.log('📦 Consolidating NPM package documentation...');
    
    for (const packageName of PROJECTS.packages) {
      const packagePath = path.join(LLM_ROOT, '_CommonNPM', packageName);
      const packageDocsPath = path.join(DOCS_DIR, 'packages', packageName);
      
      await this.consolidateProjectDocs(packagePath, packageDocsPath, packageName);
    }
  }

  async consolidateDrupalDocumentation() {
    console.log('🌐 Consolidating Drupal module documentation...');
    
    for (const moduleName of PROJECTS.drupal.modules) {
      const modulePath = path.join(PROJECTS.drupal.basePath, moduleName);
      const moduleDocsPath = path.join(DOCS_DIR, 'drupal', 'modules', moduleName);
      
      await this.consolidateProjectDocs(modulePath, moduleDocsPath, moduleName);
    }
  }

  async consolidateCICDDocumentation() {
    console.log('🔧 Consolidating CI/CD documentation...');
    
    const cicdPath = PROJECTS.cicd.basePath;
    const cicdDocsPath = path.join(DOCS_DIR, 'cicd', 'bfcicomponents');
    
    await this.consolidateProjectDocs(cicdPath, cicdDocsPath, 'bfcicomponents');
  }

  async consolidateProjectDocs(sourcePath, destPath, projectName) {
    try {
      // Check if source exists
      await fs.access(sourcePath);
      
      console.log(`  📁 Processing ${projectName}...`);
      
      // Ensure destination directory exists
      await this.ensureDirectory(destPath);
      
      // Copy main documentation files
      await this.copyMainDocFiles(sourcePath, destPath);
      
      // Copy docs directory if it exists
      const docsPath = path.join(sourcePath, 'docs');
      try {
        await fs.access(docsPath);
        await this.copyDirectory(docsPath, destPath);
      } catch (error) {
        // No docs directory, that's okay
      }
      
      // Create package info file
      await this.createPackageInfo(sourcePath, destPath, projectName);
      
      this.consolidatedFiles.push({
        source: sourcePath,
        destination: destPath,
        type: 'project_consolidation',
        project: projectName
      });
      
    } catch (error) {
      console.warn(`  ⚠️  Could not process ${projectName}:`, error.message);
    }
  }

  async copyMainDocFiles(sourcePath, destPath) {
    const mainFiles = ['README.md', 'CLAUDE.md', 'ROADMAP.md', 'CHANGELOG.md', 'CONTRIBUTING.md'];
    
    for (const fileName of mainFiles) {
      const sourceFil = path.join(sourcePath, fileName);
      const destFile = path.join(destPath, fileName);
      
      try {
        await fs.access(sourceFil);
        await fs.copyFile(sourceFil, destFile);
      } catch (error) {
        // File doesn't exist, skip
      }
    }
  }

  async createPackageInfo(sourcePath, destPath, projectName) {
    try {
      // Read package.json for metadata
      const packageJsonPath = path.join(sourcePath, 'package.json');
      let packageInfo = {};
      
      try {
        const packageJson = await fs.readFile(packageJsonPath, 'utf8');
        packageInfo = JSON.parse(packageJson);
      } catch (error) {
        // No package.json, that's okay for Drupal modules
      }
      
      const packageInfoContent = `---
title: ${projectName}
description: ${packageInfo.description || `Documentation for ${projectName}`}
version: ${packageInfo.version || 'N/A'}
---

# ${projectName}

${packageInfo.description || `Documentation for the ${projectName} project.`}

## Quick Links

- [README](./README.md)
${packageInfo.version ? `- **Version**: ${packageInfo.version}` : ''}
${packageInfo.repository ? `- **Repository**: ${packageInfo.repository.url || packageInfo.repository}` : ''}
${packageInfo.homepage ? `- **Homepage**: ${packageInfo.homepage}` : ''}

## Dependencies

${packageInfo.dependencies ? `### Production Dependencies
${Object.keys(packageInfo.dependencies).map(dep => `- ${dep}: ${packageInfo.dependencies[dep]}`).join('\n')}` : ''}

${packageInfo.devDependencies ? `### Development Dependencies  
${Object.keys(packageInfo.devDependencies).slice(0, 5).map(dep => `- ${dep}: ${packageInfo.devDependencies[dep]}`).join('\n')}
${Object.keys(packageInfo.devDependencies).length > 5 ? '- ...' : ''}` : ''}

## Scripts

${packageInfo.scripts ? Object.keys(packageInfo.scripts).map(script => `- \`npm run ${script}\`: ${packageInfo.scripts[script]}`).join('\n') : 'No scripts defined'}

---

*Last updated: ${new Date().toISOString().split('T')[0]}*
`;

      await fs.writeFile(path.join(destPath, 'package-info.md'), packageInfoContent);
      
    } catch (error) {
      console.warn(`  ⚠️  Could not create package info for ${projectName}:`, error.message);
    }
  }

  async fixNavigationAndLinks() {
    console.log('🔗 Fixing navigation and links...');
    
    // Fix sidebars.js to use proper doc references
    await this.fixSidebarsConfig();
    
    // Fix main index page links
    await this.fixIndexPageLinks();
    
    // Scan for broken internal links
    await this.scanForBrokenLinks();
  }

  async fixSidebarsConfig() {
    const sidebarPath = path.join(DOCS_ROOT, 'sidebars.js');
    
    const newSidebarContent = `// @ts-check

/** @type {import('@docusaurus/plugin-content-docs').SidebarsConfig} */
const sidebars = {
  mainSidebar: [
    // Documentation Hub
    'index',

    // NPM Packages
    {
      type: 'category',
      label: '📦 NPM Packages',
      collapsed: false,
      items: [
        'packages/llm-gateway/index',
        'packages/llm-gateway-sdk/index', 
        'packages/llm-mcp/index',
        'packages/apple-fm/index',
        'packages/bfrfp/index',
        'packages/tddai/index',
        'packages/llmcli/index',
        'packages/llm-ui/index',
        'packages/secure-project/index',
      ],
    },

    // Drupal Integration
    {
      type: 'category',
      label: '🌐 Drupal Integration',
      collapsed: true,
      items: [
        'drupal/modules/llm/index',
        'drupal/modules/ai_provider_apple/index',
        'drupal/modules/mcp_client_extras/index',
        'drupal/modules/recipe_onboarding/index',
      ],
    },

    // CI/CD Components
    {
      type: 'category',
      label: '🔧 CI/CD Components', 
      collapsed: true,
      items: [
        'cicd/bfcicomponents/index',
      ],
    },

    // Architecture & Guides
    {
      type: 'category',
      label: '📖 Guides & Architecture',
      collapsed: true,
      items: [
        'architecture/overview',
        'guides/installation',
        'guides/quickstart',
        'api/reference',
      ],
    },
  ],
};

module.exports = sidebars;
`;

    await fs.writeFile(sidebarPath, newSidebarContent);
    console.log('  ✅ Fixed sidebars.js configuration');
  }

  async fixIndexPageLinks() {
    const indexPath = path.join(DOCS_DIR, 'index.md');
    
    const newIndexContent = `---
title: LLM Platform Documentation Hub
description: Comprehensive documentation for the LLM Platform ecosystem
---

# LLM Platform Documentation Hub

Welcome to the comprehensive documentation for the LLM Platform ecosystem. This documentation is automatically aggregated from all platform components.

## 📦 NPM Packages (9)

- **[llm-gateway](packages/llm-gateway/)** - 🚀 Feature Complete API Gateway
- **[llm-gateway-sdk](packages/llm-gateway-sdk/)** - 🚀 Feature Complete TypeScript SDK  
- **[llm-mcp](packages/llm-mcp/)** - 🚀 Feature Complete MCP Server
- **[apple-fm](packages/apple-fm/)** - ✅ Production Ready Apple Foundation Models
- **[bfrfp](packages/bfrfp/)** - ✅ Production Ready RFP Generator
- **[tddai](packages/tddai/)** - ✅ Production Ready TDD AI Framework
- **[llmcli](packages/llmcli/)** - 🚀 Feature Complete CLI Tools
- **[llm-ui](packages/llm-ui/)** - 🚀 Feature Complete UI Components
- **[secure-project](packages/secure-project/)** - 🚀 Feature Complete Security Framework

## 🌐 Drupal Integration (4)

- **[LLM Core Module](drupal/modules/llm/)** - Core LLM integration for Drupal
- **[Apple AI Provider](drupal/modules/ai_provider_apple/)** - Apple Intelligence integration
- **[MCP Client Extras](drupal/modules/mcp_client_extras/)** - Enhanced MCP client features
- **[Recipe Onboarding](drupal/modules/recipe_onboarding/)** - Guided setup workflows

## 🔧 CI/CD Components (1)

- **[BF CI Components](cicd/bfcicomponents/)** - GitLab CI/CD component catalog

## 🚀 Quick Start

\`\`\`bash
# Install LLM Platform CLI
npm install -g @bluefly/llmcli

# Install complete platform
llmcli llm platform:install --template full

# Initialize new project  
llmcli project init my-project --template typescript
\`\`\`

## 🏗️ Platform Architecture

The LLM Platform follows a modular microservices architecture:

\`\`\`
┌─────────────────────────────────────────────────────────┐
│                   CLI Interface                          │
│                   llmcli                                │
└─────────────────┬───────────────────────────────────────┘
                  │
┌─────────────────▼───────────────────────────────────────┐
│                API Gateway                               │
│              llm-gateway                                │
└─────────────────┬───────────────────────────────────────┘
                  │
┌─────────────────▼───────────────────────────────────────┐
│                Service Layer                             │
│  apple-fm │ bfrfp │ llm-mcp │ tddai │ llm-ui          │
└─────────────────┬───────────────────────────────────────┘
                  │
┌─────────────────▼───────────────────────────────────────┐
│              CI/CD Pipeline                              │
│              bfcicomponents                             │
└─────────────────────────────────────────────────────────┘
\`\`\`

## 📚 Documentation Organization

This documentation hub automatically aggregates content from:

- **Individual project /docs folders** - Detailed technical documentation
- **README.md files** - Project overviews and quick start guides  
- **Package.json metadata** - Dependency and script information
- **GitLab repository links** - Source code and issue tracking

---

**Last Updated**: ${new Date().toISOString().split('T')[0]} | **Projects**: 14 | **Status**: Active Development

*This documentation is automatically synchronized with all LLM Platform projects.*
`;

    await fs.writeFile(indexPath, newIndexContent);
    console.log('  ✅ Fixed main index page links');
  }

  async generateIndexPages() {
    console.log('📄 Generating index pages for sections...');
    
    // Generate index pages for each package that doesn't have one
    for (const packageName of PROJECTS.packages) {
      const packageIndexPath = path.join(DOCS_DIR, 'packages', packageName, 'index.md');
      
      try {
        await fs.access(packageIndexPath);
        // Index already exists
      } catch (error) {
        // Create index page
        await this.createPackageIndexPage(packageName);
      }
    }
  }

  async createPackageIndexPage(packageName) {
    const packagePath = path.join(DOCS_DIR, 'packages', packageName);
    const indexPath = path.join(packagePath, 'index.md');
    
    try {
      await this.ensureDirectory(packagePath);
      
      const indexContent = `---
title: ${packageName}
description: Documentation for ${packageName}
---

# ${packageName}

Documentation for the ${packageName} package.

## Quick Links

- [README](./README.md) - Project overview and setup
- [Package Information](./package-info.md) - Dependencies and scripts

## Available Documentation

This section contains all documentation for the ${packageName} package. Use the sidebar navigation to explore specific topics.

---

*For the latest updates, check the [project repository](https://gitlab.bluefly.io/llm/${packageName}).*
`;

      await fs.writeFile(indexPath, indexContent);
      console.log(`  ✅ Created index page for ${packageName}`);
      
    } catch (error) {
      console.warn(`  ⚠️  Could not create index for ${packageName}:`, error.message);
    }
  }

  async generateConsolidationReport() {
    const reportPath = path.join(DOCS_ROOT, 'consolidation-report.md');
    
    const reportContent = `# Documentation Consolidation Report

Generated: ${new Date().toISOString()}

## Summary

- **Total files consolidated**: ${this.consolidatedFiles.length}
- **Broken links found**: ${this.brokenLinks.length}
- **Duplicate files detected**: ${this.duplicateFiles.length}

## Consolidated Files

${this.consolidatedFiles.map(file => `- ${file.type}: ${file.source} → ${file.destination}`).join('\n')}

## Broken Links

${this.brokenLinks.length > 0 ? this.brokenLinks.map(link => `- ${link.file}: ${link.link}`).join('\n') : 'No broken links detected'}

## Next Steps

1. Review consolidated documentation for accuracy
2. Test navigation and links in documentation site
3. Update any remaining broken links
4. Clean up source directories after verification

---

*This report was automatically generated by the documentation consolidation script.*
`;

    await fs.writeFile(reportPath, reportContent);
    console.log('📋 Generated consolidation report');
  }

  // Utility methods
  async ensureDirectory(dirPath) {
    try {
      await fs.mkdir(dirPath, { recursive: true });
    } catch (error) {
      // Directory might already exist
    }
  }

  async copyDirectory(source, destination) {
    await this.ensureDirectory(destination);
    
    const items = await fs.readdir(source);
    
    for (const item of items) {
      const sourcePath = path.join(source, item);
      const destPath = path.join(destination, item);
      
      const stats = await fs.stat(sourcePath);
      
      if (stats.isDirectory()) {
        await this.copyDirectory(sourcePath, destPath);
      } else {
        await fs.copyFile(sourcePath, destPath);
      }
    }
  }

  async getMarkdownFiles(dirPath) {
    const files = [];
    
    async function scan(currentPath) {
      const items = await fs.readdir(currentPath);
      
      for (const item of items) {
        const itemPath = path.join(currentPath, item);
        const stats = await fs.stat(itemPath);
        
        if (stats.isDirectory()) {
          await scan(itemPath);
        } else if (item.endsWith('.md')) {
          files.push(itemPath);
        }
      }
    }
    
    await scan(dirPath);
    return files;
  }

  async scanForBrokenLinks() {
    // This would scan for broken internal links
    // Implementation would check relative links within docs
    console.log('  🔍 Scanning for broken links...');
    // TODO: Implement link scanning
  }
}

// Run the consolidator
if (require.main === module) {
  const consolidator = new DocumentationConsolidator();
  consolidator.run().catch(console.error);
}

module.exports = DocumentationConsolidator;