#!/usr/bin/env node

/**
 * VuePress Home Page to Docusaurus React Component Converter
 * Converts VuePress home page with features to a Docusaurus landing page
 */

const fs = require('fs-extra');
const path = require('path');
const matter = require('gray-matter');

async function convertHomePageToReact(frontmatter, targetDir) {
  const fs = require('fs-extra');
  const componentContent = `import React from 'react';
import clsx from 'clsx';
import Link from '@docusaurus/Link';
import useDocusaurusContext from '@docusaurus/useDocusaurusContext';
import Layout from '@theme/Layout';
import HomepageFeatures from '@site/src/components/HomepageFeatures';

import styles from './index.module.css';

function HomepageHeader() {
  const {siteConfig} = useDocusaurusContext();
  return (
    <header className={clsx('hero hero--primary', styles.heroBanner)}>
      <div className="container">
        <h1 className="hero__title">${frontmatter.heroText || '{siteConfig.title}'}</h1>
        <p className="hero__subtitle">${frontmatter.tagline || '{siteConfig.tagline}'}</p>
        <div className={styles.buttons}>
          ${generateActionButtons(frontmatter.actions)}
        </div>
      </div>
    </header>
  );
}

export default function Home() {
  const {siteConfig} = useDocusaurusContext();
  return (
    <Layout
      title={\`\${siteConfig.title}\`}
      description="${frontmatter.tagline || 'Description will go into a meta tag in <head />'}">
      <HomepageHeader />
      <main>
        <HomepageFeatures />
      </main>
    </Layout>
  );
}`;

  // Create the home page component
  const homePagePath = path.join(targetDir, 'src/pages/index.js');
  await fs.ensureDir(path.dirname(homePagePath));
  await fs.writeFile(homePagePath, componentContent);

  // Create the CSS module
  const cssContent = `/* stylelint-disable docusaurus/copyright-header */

.heroBanner {
  padding: 4rem 0;
  text-align: center;
  position: relative;
  overflow: hidden;
}

@media screen and (max-width: 996px) {
  .heroBanner {
    padding: 2rem;
  }
}

.buttons {
  display: flex;
  align-items: center;
  justify-content: center;
  gap: 1rem;
}`;

  const cssPath = path.join(targetDir, 'src/pages/index.module.css');
  await fs.writeFile(cssPath, cssContent);

  // Create HomepageFeatures component if features exist
  if (frontmatter.features && frontmatter.features.length > 0) {
    createFeaturesComponent(frontmatter.features, targetDir);
  }
}

function generateActionButtons(actions) {
  if (!actions || actions.length === 0) return '';
  
  return actions.map(action => {
    const className = action.type === 'primary' 
      ? 'button button--primary button--lg' 
      : 'button button--secondary button--lg';
    return `<Link
            className="${className}"
            to="${action.link}">
            ${action.text}
          </Link>`;
  }).join('\n          ');
}

async function createFeaturesComponent(features, targetDir) {
  const fs = require('fs-extra');
  const featureList = features.map((feature, idx) => ({
    title: feature.title,
    description: feature.details,
  }));

  const componentContent = `import React from 'react';
import clsx from 'clsx';
import styles from './styles.module.css';

const FeatureList = ${JSON.stringify(featureList, null, 2)};

function Feature({title, description}) {
  return (
    <div className={clsx('col col--4')}>
      <div className="text--center padding-horiz--md">
        <h3>{title}</h3>
        <p>{description}</p>
      </div>
    </div>
  );
}

export default function HomepageFeatures() {
  return (
    <section className={styles.features}>
      <div className="container">
        <div className="row">
          {FeatureList.map((props, idx) => (
            <Feature key={idx} {...props} />
          ))}
        </div>
      </div>
    </section>
  );
}`;

  const featuresDir = path.join(targetDir, 'src/components/HomepageFeatures');
  await fs.ensureDir(featuresDir);
  
  await fs.writeFile(
    path.join(featuresDir, 'index.js'),
    componentContent
  );

  // Create CSS for features
  const featuresCss = `.features {
  display: flex;
  align-items: center;
  padding: 2rem 0;
  width: 100%;
}`;

  await fs.writeFile(
    path.join(featuresDir, 'styles.module.css'),
    featuresCss
  );
}

module.exports = { convertHomePageToReact };

// If run directly
if (require.main === module) {
  const [sourceFile, targetDir] = process.argv.slice(2);
  
  if (!sourceFile || !targetDir) {
    console.error('Usage: node convert-home-page.js <source-file> <target-dir>');
    process.exit(1);
  }
  
  const content = fs.readFileSync(sourceFile, 'utf-8');
  const { data: frontmatter } = matter(content);
  
  convertHomePageToReact(frontmatter, targetDir);
  console.log('✅ Home page converted successfully!');
}