#!/usr/bin/env node

/**
 * Create project reference pages that link to actual project documentation
 * without copying files
 */

const fs = require('fs').promises;
const path = require('path');

class ProjectReferenceGenerator {
  constructor(configPath = './docs-config.json') {
    this.configPath = configPath;
    this.config = null;
  }

  async loadConfig() {
    const configContent = await fs.readFile(this.configPath, 'utf-8');
    this.config = JSON.parse(configContent);
  }

  async generateProjectReferences() {
    const projectsDir = path.resolve('./docs/projects');
    await fs.mkdir(projectsDir, { recursive: true });

    // Create index page
    await this.createProjectIndex(projectsDir);

    // Create individual project reference pages
    for (const [projectKey, projectConfig] of Object.entries(this.config.projects)) {
      await this.createProjectReference(projectKey, projectConfig, projectsDir);
    }

    console.log('✅ Created project reference pages');
  }

  async createProjectIndex(projectsDir) {
    const indexContent = `# Project Documentation

This section provides links to documentation for all projects in the Bluefly LLM Platform ecosystem.

## Architecture Overview

The platform consists of ${Object.keys(this.config.projects).length} integrated projects:

\`\`\`mermaid
graph TB
    subgraph "Core Services"
        BFAPI[BFAPI - API Gateway]
        BFLLM[BFLLM - LLM Services]
        llm-mcp[llm-mcp - Model Control]
    end
    
    subgraph "Developer Tools"
        llmcli[llmcli - CLI Tool]
        BFCI[BFCI Components]
    end
    
    subgraph "Frontend & SDKs"
        llm-ui[llm-ui - React Components]
        llmcliENT[BFAPI Client SDK]
        BFRFP[BFRFP - Content Gen]
    end
    
    subgraph "Integration Layer"
        DRUPAL[Drupal Modules]
        UNIFIED[Unified Compliance]
    end
    
    llmcli --> BFAPI
    BFAPI --> BFLLM
    BFAPI --> llm-mcp
    llm-ui --> llmcliENT
    llmcliENT --> BFAPI
    DRUPAL --> BFAPI
    BFRFP --> BFLLM
\`\`\`

## Projects by Category

${this.generateCategoryIndex()}

## Quick Links

- [Platform Architecture](/architecture/)
- [Getting Started Guide](/getting-started/)
- [API Documentation](/api/)
- [Development Standards](/development/)
- [Deployment Guide](/deployment/)
`;

    await fs.writeFile(path.join(projectsDir, 'index.md'), indexContent);
  }

  generateCategoryIndex() {
    const categories = {};
    
    // Group projects by category
    Object.entries(this.config.projects).forEach(([key, project]) => {
      const category = project.category || 'Other';
      if (!categories[category]) {
        categories[category] = [];
      }
      categories[category].push({ key, ...project });
    });

    // Generate markdown for each category
    return Object.entries(categories).map(([category, projects]) => `
### ${category}

${projects.map(project => `- [${project.name}](./${project.key}.md) - ${project.description}`).join('\\n')}
`).join('\\n');
  }

  async createProjectReference(projectKey, projectConfig, projectsDir) {
    const projectPath = path.resolve(projectConfig.path);
    const docsPath = path.join(projectPath, projectConfig.docsPath);
    
    // Check if project docs exist
    let hasLocalDocs = false;
    try {
      await fs.access(docsPath);
      hasLocalDocs = true;
    } catch (e) {
      // Docs directory doesn't exist
    }

    const content = `# ${projectConfig.name}

${projectConfig.description}

## Project Information

- **Category**: ${projectConfig.category}
- **Repository**: [${projectConfig.repository}](${projectConfig.repository})
- **Local Path**: \`${projectConfig.path}\`
${projectConfig.architecture ? this.generateArchitectureInfo(projectConfig.architecture) : ''}

## Documentation

${hasLocalDocs ? `
### Local Documentation
The project documentation is available in the project directory:
- [View Local Docs](file://${docsPath})
- Path: \`${docsPath}\`

### Available Documentation
${projectConfig.docs ? this.generateDocsLinks(projectConfig.docs, projectKey) : 'Documentation structure not defined in config.'}
` : `
### Documentation Not Found
No documentation directory found at \`${docsPath}\`.

To create documentation for this project:
1. Navigate to \`${projectConfig.path}\`
2. Create a \`${projectConfig.docsPath}\` directory
3. Add documentation files
4. Run \`npm run docs:sync\` to update references
`}

## Integration with Platform

${this.generateIntegrationInfo(projectKey, projectConfig)}

## Quick Commands

\`\`\`bash
# Navigate to project
cd ${projectConfig.path}

# View documentation
open ${docsPath}

# Start development
${projectConfig.architecture?.language === 'TypeScript/Node.js' ? 'npm run dev' : '# See project README for start command'}
\`\`\`

## Related Documentation

- [Platform Overview](/architecture/)
- [Integration Patterns](/integration/)
- [Development Guide](/development/)
${projectConfig.architecture?.dependencies ? `
### Dependencies
${projectConfig.architecture.dependencies.map(dep => `- [${dep}](./${dep.toLowerCase()}.md)`).join('\\n')}
` : ''}

---

*This is a reference page. The actual documentation is maintained in the [project repository](${projectConfig.repository}).*
`;

    await fs.writeFile(path.join(projectsDir, `${projectKey}.md`), content);
    console.log(`📄 Created reference for ${projectKey}`);
  }

  generateArchitectureInfo(architecture) {
    const info = [];
    
    if (architecture.language) info.push(`- **Language**: ${architecture.language}`);
    if (architecture.type) info.push(`- **Type**: ${architecture.type}`);
    if (architecture.port) info.push(`- **Port**: ${architecture.port}`);
    if (architecture.database) info.push(`- **Database**: ${architecture.database}`);
    if (architecture.frameworks) info.push(`- **Frameworks**: ${architecture.frameworks.join(', ')}`);
    if (architecture.technologies) info.push(`- **Technologies**: ${architecture.technologies.join(', ')}`);
    
    return info.length > 0 ? '\\n' + info.join('\\n') : '';
  }

  generateDocsLinks(docs, projectKey) {
    const sections = [];
    
    Object.entries(docs).forEach(([category, files]) => {
      const links = files.map(file => {
        const name = file.replace('.md', '').split('-').map(w => 
          w.charAt(0).toUpperCase() + w.slice(1)
        ).join(' ');
        return `  - ${name}`;
      }).join('\\n');
      
      sections.push(`**${category.charAt(0).toUpperCase() + category.slice(1)}**:\\n${links}`);
    });
    
    return sections.join('\\n\\n');
  }

  generateIntegrationInfo(projectKey, projectConfig) {
    const integrations = [];
    
    if (projectConfig.architecture?.dependencies) {
      integrations.push(`### Service Dependencies
This service integrates with:
${projectConfig.architecture.dependencies.map(dep => `- **${dep}**: Direct service integration`).join('\\n')}`);
    }
    
    if (projectConfig.architecture?.port) {
      integrations.push(`### API Access
- **Default Port**: ${projectConfig.architecture.port}
- **Health Check**: \`http://localhost:${projectConfig.architecture.port}/health\`
- **API Docs**: \`http://localhost:${projectConfig.architecture.port}/api-docs\``);
    }
    
    if (projectConfig.category === 'Drupal Integration') {
      integrations.push(`### Drupal Integration
This module integrates with Drupal 11 and can be installed via:
\`\`\`bash
drush en ${projectKey}
\`\`\``);
    }
    
    return integrations.join('\\n\\n') || 'No specific integration details available.';
  }
}

// Run if called directly
if (require.main === module) {
  const generator = new ProjectReferenceGenerator();
  generator.loadConfig()
    .then(() => generator.generateProjectReferences())
    .catch(console.error);
}

module.exports = ProjectReferenceGenerator;