#!/usr/bin/env node
/**
 * Automated API Documentation Generator
 */

const fs = require('fs');
const path = require('path');
const { execSync } = require('child_process');
const yaml = require('js-yaml');

class DocumentationGenerator {
  constructor() {
    this.outputDir = path.join(__dirname, '../build/api-docs');
    this.specsDir = '/Users/flux423/Sites/LLM/_CommonNPM';
  }

  async generateAll() {
    console.log('📚 Generating comprehensive API documentation...');
    
    await this.ensureOutputDir();
    await this.generateReDocDocs();
    await this.generateSwaggerUIDocs();
    await this.generateMarkdownDocs();
    await this.generatePDFDocs();
    await this.generateInteractiveDocs();
    
    console.log('✅ Documentation generation complete!');
  }

  async ensureOutputDir() {
    if (fs.existsSync(this.outputDir)) {
      execSync(`rm -rf ${this.outputDir}`);
    }
    fs.mkdirSync(this.outputDir, { recursive: true });
  }

  async generateReDocDocs() {
    console.log('📖 Generating ReDoc documentation...');
    
    const specs = this.findOpenAPISpecs();
    const redocDir = path.join(this.outputDir, 'redoc');
    fs.mkdirSync(redocDir, { recursive: true });

    for (const spec of specs) {
      const specName = path.basename(spec.path, path.extname(spec.path));
      const outputFile = path.join(redocDir, `${specName}.html`);
      
      try {
        execSync(`npx @redocly/cli build-docs ${spec.path} --output ${outputFile}`, {
          stdio: 'inherit'
        });
        console.log(`   ✅ ${spec.name}: ${outputFile}`);
      } catch (error) {
        console.log(`   ❌ ${spec.name}: Failed to generate ReDoc`);
      }
    }
  }

  async generateSwaggerUIDocs() {
    console.log('🔧 Generating Swagger UI documentation...');
    
    const specs = this.findOpenAPISpecs();
    const swaggerDir = path.join(this.outputDir, 'swagger-ui');
    fs.mkdirSync(swaggerDir, { recursive: true });

    // Create index HTML that loads all specs
    const indexHtml = `<!DOCTYPE html>
<html>
<head>
  <title>LLM Platform API Documentation</title>
  <link rel="stylesheet" type="text/css" href="https://unpkg.com/swagger-ui-dist@5.0.0/swagger-ui.css" />
  <style>
    body { margin: 0; padding: 20px; font-family: sans-serif; }
    .spec-selector { margin-bottom: 20px; }
    select { padding: 10px; font-size: 16px; }
  </style>
</head>
<body>
  <div class="spec-selector">
    <label for="spec-select">Select API Specification:</label>
    <select id="spec-select">
      ${specs.map(spec => `<option value="${spec.path}">${spec.name}</option>`).join('')}
    </select>
  </div>
  <div id="swagger-ui"></div>
  
  <script src="https://unpkg.com/swagger-ui-dist@5.0.0/swagger-ui-bundle.js"></script>
  <script>
    const specs = ${JSON.stringify(specs)};
    let ui;
    
    function loadSpec(specPath) {
      ui = SwaggerUIBundle({
        url: specPath,
        dom_id: '#swagger-ui',
        presets: [
          SwaggerUIBundle.presets.apis,
          SwaggerUIBundle.presets.standalone
        ]
      });
    }
    
    document.getElementById('spec-select').addEventListener('change', (e) => {
      loadSpec(e.target.value);
    });
    
    // Load first spec by default
    if (specs.length > 0) {
      loadSpec(specs[0].path);
    }
  </script>
</body>
</html>`;

    fs.writeFileSync(path.join(swaggerDir, 'index.html'), indexHtml);
    console.log(`   ✅ Swagger UI: ${path.join(swaggerDir, 'index.html')}`);
  }

  async generateMarkdownDocs() {
    console.log('📝 Generating Markdown documentation...');
    
    const specs = this.findOpenAPISpecs();
    const markdownDir = path.join(this.outputDir, 'markdown');
    fs.mkdirSync(markdownDir, { recursive: true });

    for (const spec of specs) {
      try {
        const specContent = yaml.load(fs.readFileSync(spec.path, 'utf8'));
        const markdown = this.generateMarkdownFromSpec(specContent, spec.name);
        
        const outputFile = path.join(markdownDir, `${path.basename(spec.path, path.extname(spec.path))}.md`);
        fs.writeFileSync(outputFile, markdown);
        console.log(`   ✅ ${spec.name}: ${outputFile}`);
      } catch (error) {
        console.log(`   ❌ ${spec.name}: Failed to generate Markdown`);
      }
    }
  }

  generateMarkdownFromSpec(spec, name) {
    let markdown = `# ${spec.info?.title || name}\n\n`;
    
    if (spec.info?.description) {
      markdown += `${spec.info.description}\n\n`;
    }

    markdown += `**Version:** ${spec.info?.version || 'Unknown'}\n\n`;

    if (spec.servers && spec.servers.length > 0) {
      markdown += `## Servers\n\n`;
      spec.servers.forEach(server => {
        markdown += `- **${server.url}**: ${server.description || 'Server'}\n`;
      });
      markdown += '\n';
    }

    if (spec.paths) {
      markdown += `## Endpoints\n\n`;
      
      for (const [path, pathItem] of Object.entries(spec.paths)) {
        for (const [method, operation] of Object.entries(pathItem)) {
          if (['get', 'post', 'put', 'delete', 'patch'].includes(method)) {
            markdown += `### ${method.toUpperCase()} ${path}\n\n`;
            
            if (operation.summary) {
              markdown += `**Summary:** ${operation.summary}\n\n`;
            }
            
            if (operation.description) {
              markdown += `${operation.description}\n\n`;
            }

            if (operation.parameters) {
              markdown += `**Parameters:**\n\n`;
              operation.parameters.forEach(param => {
                const required = param.required ? ' (required)' : ' (optional)';
                markdown += `- **${param.name}**${required}: ${param.description || 'No description'}\n`;
              });
              markdown += '\n';
            }

            if (operation.responses) {
              markdown += `**Responses:**\n\n`;
              for (const [statusCode, response] of Object.entries(operation.responses)) {
                markdown += `- **${statusCode}**: ${response.description || 'No description'}\n`;
              }
              markdown += '\n';
            }
            
            markdown += '---\n\n';
          }
        }
      }
    }

    return markdown;
  }

  async generatePDFDocs() {
    console.log('📄 Generating PDF documentation...');
    
    // Note: In a real implementation, you would use a tool like puppeteer
    // to convert HTML/Markdown to PDF. For this example, we'll create a placeholder
    const pdfDir = path.join(this.outputDir, 'pdf');
    fs.mkdirSync(pdfDir, { recursive: true });
    
    const placeholder = 'PDF generation requires additional setup with puppeteer or similar tools.\nSee the generated HTML and Markdown documentation.';
    fs.writeFileSync(path.join(pdfDir, 'README.txt'), placeholder);
    
    console.log(`   ℹ️  PDF generation placeholder created`);
  }

  async generateInteractiveDocs() {
    console.log('🎮 Generating interactive documentation...');
    
    const interactiveDir = path.join(this.outputDir, 'interactive');
    fs.mkdirSync(interactiveDir, { recursive: true });

    const specs = this.findOpenAPISpecs();
    
    // Create an interactive documentation page with try-it features
    const interactiveHtml = `<!DOCTYPE html>
<html>
<head>
  <title>LLM Platform Interactive API Documentation</title>
  <style>
    body { 
      font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif;
      margin: 0; 
      padding: 20px; 
      background: #f5f5f5;
    }
    .container { max-width: 1200px; margin: 0 auto; }
    .header { text-align: center; margin-bottom: 40px; }
    .api-grid { display: grid; grid-template-columns: repeat(auto-fit, minmax(300px, 1fr)); gap: 20px; }
    .api-card { 
      background: white; 
      border-radius: 8px; 
      padding: 20px; 
      box-shadow: 0 2px 4px rgba(0,0,0,0.1);
      transition: transform 0.2s;
    }
    .api-card:hover { transform: translateY(-2px); }
    .api-title { font-size: 1.5em; font-weight: bold; margin-bottom: 10px; color: #333; }
    .api-description { color: #666; margin-bottom: 15px; }
    .try-button { 
      background: #007cba; 
      color: white; 
      border: none; 
      padding: 10px 20px; 
      border-radius: 4px; 
      cursor: pointer;
      text-decoration: none;
      display: inline-block;
    }
    .try-button:hover { background: #005a87; }
  </style>
</head>
<body>
  <div class="container">
    <div class="header">
      <h1>🚀 LLM Platform API Documentation</h1>
      <p>Interactive documentation with live examples and try-it features</p>
    </div>
    
    <div class="api-grid">
      ${specs.map(spec => {
        const specContent = this.loadSpecContent(spec.path);
        return `
          <div class="api-card">
            <div class="api-title">${specContent?.info?.title || spec.name}</div>
            <div class="api-description">${specContent?.info?.description || 'API Documentation'}</div>
            <a href="../swagger-ui/index.html?url=${encodeURIComponent(spec.path)}" class="try-button">
              Try API →
            </a>
          </div>
        `;
      }).join('')}
    </div>
  </div>
</body>
</html>`;

    fs.writeFileSync(path.join(interactiveDir, 'index.html'), interactiveHtml);
    console.log(`   ✅ Interactive docs: ${path.join(interactiveDir, 'index.html')}`);
  }

  loadSpecContent(specPath) {
    try {
      const content = fs.readFileSync(specPath, 'utf8');
      if (specPath.endsWith('.yml') || specPath.endsWith('.yaml')) {
        return yaml.load(content);
      }
      return JSON.parse(content);
    } catch (error) {
      return null;
    }
  }

  findOpenAPISpecs() {
    const specs = [];
    const searchPaths = [
      '/Users/flux423/Sites/LLM/_CommonNPM/llm-gateway/openapi',
      '/Users/flux423/Sites/LLM/_CommonNPM/llm-mcp/openapi',
      '/Users/flux423/Sites/LLM/_CommonNPM/apple-fm/openapi'
    ];

    for (const searchPath of searchPaths) {
      if (fs.existsSync(searchPath)) {
        const files = fs.readdirSync(searchPath);
        for (const file of files) {
          if (file.endsWith('.yml') || file.endsWith('.yaml') || file.endsWith('.json')) {
            const fullPath = path.join(searchPath, file);
            const name = path.basename(file, path.extname(file));
            specs.push({
              name: name.replace(/-/g, ' ').replace(/\b\w/g, l => l.toUpperCase()),
              path: fullPath
            });
          }
        }
      }
    }

    return specs;
  }
}

// Execute documentation generation
async function main() {
  try {
    const generator = new DocumentationGenerator();
    await generator.generateAll();
  } catch (error) {
    console.error('❌ Documentation generation failed:', error);
    process.exit(1);
  }
}

if (require.main === module) {
  main();
}

module.exports = { DocumentationGenerator };
