#!/usr/bin/env node

const fs = require('fs');
const path = require('path');
const { execSync } = require('child_process');

// Configuration
const config = {
  sourceDirs: [
    '../llmcli/docs',
    '../bfapi/docs',
    '../bfllm/docs',
    '../llm-mcp/docs',
    '../bfrfp/docs',
    '../llm-ui/docs',
    '../_DrupalModules/api_normalization/docs',
    '../_DrupalModules/secure_drupal/docs',
    '../_DrupalModules/drupal_migration_plus/docs',
    '../_DrupalModules/blockchain_manager/docs'
  ],
  targetDir: './docs',
  componentMap: {
    'llmcli': 'components/llmcli',
    'bfapi': 'components/bfapi',
    'bfllm': 'components/bfllm',
    'llm-mcp': 'components/llm-mcp',
    'bfrfp': 'components/bfrfp',
    'llm-ui': 'components/llm-ui',
    'api_normalization': 'drupal/modules/api_normalization',
    'secure_drupal': 'drupal/modules/secure_drupal',
    'drupal_migration_plus': 'drupal/modules/migration_plus',
    'blockchain_manager': 'drupal/modules/blockchain_manager'
  }
};

// Utility functions
function ensureDir(dir) {
  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir, { recursive: true });
  }
}

function cleanContent(content) {
  // Remove emojis
  content = content.replace(/[\u{1F300}-\u{1F9FF}]/gu, '');
  
  // Remove marketing language
  const marketingTerms = [
    'revolutionary',
    'ultimate',
    'innovative',
    'comprehensive',
    'powerful',
    'advanced',
    'cutting-edge',
    'state-of-the-art',
    'next-generation'
  ];
  
  marketingTerms.forEach(term => {
    const regex = new RegExp(`\\b${term}\\b`, 'gi');
    content = content.replace(regex, '');
  });
  
  // Clean up extra whitespace
  content = content.replace(/\n{3,}/g, '\n\n');
  
  return content;
}

function processFile(sourcePath, targetPath) {
  console.log(`Processing ${sourcePath} -> ${targetPath}`);
  
  // Read source file
  const content = fs.readFileSync(sourcePath, 'utf8');
  
  // Clean content
  const cleanedContent = cleanContent(content);
  
  // Ensure target directory exists
  ensureDir(path.dirname(targetPath));
  
  // Write cleaned content
  fs.writeFileSync(targetPath, cleanedContent);
}

function migrateDocs() {
  // Create target directory structure
  Object.values(config.componentMap).forEach(dir => {
    ensureDir(path.join(config.targetDir, dir));
  });
  
  // Process each source directory
  config.sourceDirs.forEach(sourceDir => {
    const componentName = path.basename(sourceDir).replace('-docs', '');
    const targetBase = config.componentMap[componentName];
    
    if (!targetBase) {
      console.warn(`No mapping found for ${componentName}`);
      return;
    }
    
    // Find all markdown files
    const findCmd = `find ${sourceDir} -name "*.md"`;
    const files = execSync(findCmd).toString().trim().split('\n');
    
    files.forEach(file => {
      if (!file) return;
      
      const relativePath = path.relative(sourceDir, file);
      const targetPath = path.join(config.targetDir, targetBase, relativePath);
      
      processFile(file, targetPath);
    });
  });
  
  console.log('Documentation migration complete!');
}

// Run migration
migrateDocs(); 