#!/usr/bin/env node

/**
 * VuePress to Docusaurus Migration Tool
 * Automatically migrates VuePress documentation to Docusaurus format
 */

const fs = require('fs-extra');
const path = require('path');
const glob = require('glob');
const matter = require('gray-matter');
const chalk = require('chalk');
const { convertHomePageToReact } = require('./convert-home-page');
const { pluginMappings, generatePluginMigrationGuide, generateThemeMigrationGuide } = require('./plugin-theme-migrator');

class VuePressToDocusaurusMigrator {
  constructor(sourceDir, targetDir) {
    this.sourceDir = sourceDir;
    this.targetDir = targetDir;
    this.report = {
      totalFiles: 0,
      convertedFiles: 0,
      copiedAssets: 0,
      errors: [],
      warnings: [],
      configMigration: {},
      navigationStructure: {},
      startTime: new Date()
    };
  }

  async migrate() {
    console.log(chalk.blue('🚀 Starting VuePress to Docusaurus migration...'));
    
    try {
      // 1. Create Docusaurus project structure
      await this.createDocusaurusStructure();
      
      // 2. Migrate configuration
      await this.migrateConfig();
      
      // 3. Migrate markdown files
      await this.migrateMarkdownFiles();
      
      // 4. Migrate static assets
      await this.migrateStaticAssets();
      
      // 5. Create Docusaurus configuration files
      await this.createDocusaurusConfig();
      
      // 6. Analyze and document plugin migrations
      await this.analyzePluginMigration();
      
      // 7. Generate migration report
      await this.generateReport();
      
      console.log(chalk.green('✅ Migration completed successfully!'));
    } catch (error) {
      console.error(chalk.red('❌ Migration failed:'), error);
      this.report.errors.push(error.message);
      await this.generateReport();
      process.exit(1);
    }
  }

  async createDocusaurusStructure() {
    console.log(chalk.yellow('📁 Creating Docusaurus project structure...'));
    
    const dirs = [
      'docs',
      'blog',
      'src/components',
      'src/pages',
      'src/css',
      'static/img',
      'i18n'
    ];
    
    for (const dir of dirs) {
      await fs.ensureDir(path.join(this.targetDir, dir));
    }
  }

  async migrateConfig() {
    console.log(chalk.yellow('⚙️  Migrating VuePress configuration...'));
    
    const vuepressConfigPath = path.join(this.sourceDir, 'docs/.vuepress/config.mjs');
    
    if (await fs.pathExists(vuepressConfigPath)) {
      const configContent = await fs.readFile(vuepressConfigPath, 'utf-8');
      
      // Parse VuePress config (simplified - in production, use proper JS parser)
      const config = await this.parseVuePressConfig(configContent);
      
      this.report.configMigration = {
        title: config.title,
        description: config.description,
        base: config.base,
        themeConfig: config.themeConfig
      };
    }
  }

  async parseVuePressConfig(configContent) {
    // Extract configuration using regex (simplified approach)
    const config = {};
    
    // Extract title
    const titleMatch = configContent.match(/title:\s*['"`](.+?)['"`]/);
    if (titleMatch) config.title = titleMatch[1];
    
    // Extract description
    const descMatch = configContent.match(/description:\s*['"`](.+?)['"`]/);
    if (descMatch) config.description = descMatch[1];
    
    // Extract base
    const baseMatch = configContent.match(/base:\s*['"`](.+?)['"`]/);
    if (baseMatch) config.base = baseMatch[1];
    
    // Extract nav items
    const navMatch = configContent.match(/nav:\s*\[([\s\S]*?)\]/);
    if (navMatch) {
      config.nav = this.parseNavItems(navMatch[1]);
    }
    
    // Extract sidebar
    const sidebarMatch = configContent.match(/sidebar:\s*{([\s\S]*?)}\s*}/);
    if (sidebarMatch) {
      config.sidebar = this.parseSidebar(sidebarMatch[1]);
    }
    
    return { ...config, themeConfig: { nav: config.nav, sidebar: config.sidebar } };
  }

  parseNavItems(navContent) {
    const items = [];
    const itemRegex = /{\s*text:\s*['"`](.+?)['"`],\s*link:\s*['"`](.+?)['"`]\s*}/g;
    let match;
    
    while ((match = itemRegex.exec(navContent)) !== null) {
      items.push({
        label: match[1],
        to: match[2].startsWith('http') ? match[2] : match[2].replace(/\/$/, '')
      });
    }
    
    return items;
  }

  parseSidebar(sidebarContent) {
    // Simplified sidebar parsing - in production, use proper AST parser
    const sidebar = {};
    
    // Extract route patterns
    const routeRegex = /'([^']+)':\s*\[([\s\S]*?)\]/g;
    let match;
    
    while ((match = routeRegex.exec(sidebarContent)) !== null) {
      const route = match[1];
      const content = match[2];
      sidebar[route] = this.parseSidebarItems(content);
    }
    
    return sidebar;
  }

  async analyzePluginMigration() {
    console.log(chalk.yellow('🔌 Analyzing plugin migration...'));
    
    const vuepressConfigPath = path.join(this.sourceDir, 'docs/.vuepress/config.mjs');
    const packageJsonPath = path.join(this.sourceDir, 'package.json');
    
    const detectedPlugins = [];
    
    // Check config for plugins
    if (await fs.pathExists(vuepressConfigPath)) {
      const configContent = await fs.readFile(vuepressConfigPath, 'utf-8');
      const pluginRegex = /@vuepress\/plugin-[\w-]+/g;
      let match;
      while ((match = pluginRegex.exec(configContent)) !== null) {
        if (!detectedPlugins.includes(match[0])) {
          detectedPlugins.push(match[0]);
        }
      }
    }
    
    // Check package.json for plugins
    if (await fs.pathExists(packageJsonPath)) {
      const pkg = await fs.readJson(packageJsonPath);
      const allDeps = {...(pkg.dependencies || {}), ...(pkg.devDependencies || {})};
      
      for (const dep of Object.keys(allDeps)) {
        if (dep.startsWith('@vuepress/plugin-') && !detectedPlugins.includes(dep)) {
          detectedPlugins.push(dep);
        }
      }
    }
    
    if (detectedPlugins.length > 0) {
      this.report.detectedPlugins = detectedPlugins;
      const pluginGuide = generatePluginMigrationGuide(detectedPlugins);
      await fs.writeFile(
        path.join(this.targetDir, 'PLUGIN_MIGRATION_GUIDE.md'),
        pluginGuide
      );
    }
  }

  parseSidebarItems(content) {
    const items = [];
    
    // Parse sidebar groups
    const groupRegex = /{\s*title:\s*['"`](.+?)['"`],[\s\S]*?children:\s*\[([\s\S]*?)\]\s*}/g;
    let match;
    
    while ((match = groupRegex.exec(content)) !== null) {
      const group = {
        type: 'category',
        label: match[1],
        items: []
      };
      
      // Parse children
      const childrenContent = match[2];
      const childRegex = /['"`]([^'"`]+)['"`]/g;
      let childMatch;
      
      while ((childMatch = childRegex.exec(childrenContent)) !== null) {
        const childPath = childMatch[1];
        if (childPath) {
          group.items.push({
            type: 'doc',
            id: childPath === '' ? 'index' : childPath
          });
        }
      }
      
      items.push(group);
    }
    
    return items;
  }

  async migrateMarkdownFiles() {
    console.log(chalk.yellow('📝 Migrating markdown files...'));
    
    const mdFiles = glob.sync('**/*.md', {
      cwd: path.join(this.sourceDir, 'docs'),
      ignore: ['node_modules/**', '.vuepress/**']
    });
    
    this.report.totalFiles = mdFiles.length;
    
    for (const file of mdFiles) {
      try {
        await this.convertMarkdownFile(file);
        this.report.convertedFiles++;
      } catch (error) {
        this.report.errors.push(`Failed to convert ${file}: ${error.message}`);
      }
    }
  }

  async convertMarkdownFile(filePath) {
    const sourcePath = path.join(this.sourceDir, 'docs', filePath);
    const content = await fs.readFile(sourcePath, 'utf-8');
    
    // Parse frontmatter and content
    const { data: frontmatter, content: markdown } = matter(content);
    
    // Handle home page specially
    if (frontmatter.home) {
      await convertHomePageToReact(frontmatter, this.targetDir);
      this.report.warnings.push(`${filePath}: Converted to React component at src/pages/index.js`);
      return;
    }
    
    // Convert frontmatter
    const newFrontmatter = this.convertFrontmatter(frontmatter, filePath);
    
    // Convert markdown content
    const newContent = this.convertMarkdownContent(markdown, filePath);
    
    // Determine target path
    const targetPath = this.getTargetPath(filePath);
    
    // Write converted file
    const finalContent = matter.stringify(newContent, newFrontmatter);
    await fs.ensureDir(path.dirname(targetPath));
    await fs.writeFile(targetPath, finalContent);
    
    console.log(chalk.gray(`  ✓ Converted ${filePath}`));
  }

  convertFrontmatter(frontmatter, filePath) {
    const converted = {};
    
    // Skip home page frontmatter conversion (handled separately)
    if (frontmatter.home) {
      return {};
    }
    
    // Convert common properties
    if (frontmatter.title) converted.title = frontmatter.title;
    if (frontmatter.description) converted.description = frontmatter.description;
    
    // Generate ID from filepath
    converted.id = this.generateDocId(filePath);
    
    // Convert sidebar position
    if (frontmatter.order !== undefined) {
      converted.sidebar_position = frontmatter.order;
    }
    
    // Handle tags
    if (frontmatter.tags) {
      converted.tags = Array.isArray(frontmatter.tags) ? frontmatter.tags : [frontmatter.tags];
    }
    
    // Handle slug
    if (frontmatter.permalink) {
      converted.slug = frontmatter.permalink;
    }
    
    return converted;
  }

  generateDocId(filePath) {
    // Remove .md extension and convert path to ID
    const id = filePath
      .replace(/\.md$/, '')
      .replace(/\//g, '-')
      .replace(/index$/, '')
      .replace(/-$/, '');
    
    return id || 'index';
  }

  convertMarkdownContent(content, filePath) {
    let converted = content;
    
    // Convert VuePress specific syntax to Docusaurus
    
    // 1. Convert ::: tip/warning/danger blocks to admonitions
    converted = converted.replace(
      /:::\s*(tip|warning|danger|info|note)\s*(.*?)\n([\s\S]*?):::/g,
      (match, type, title, content) => {
        const docusaurusType = type === 'danger' ? 'danger' : type;
        const titleLine = title ? `[${title}]` : '';
        return `:::${docusaurusType}${titleLine}\n${content}:::`;
      }
    );
    
    // 2. Convert component imports (VuePress components to MDX)
    if (converted.includes('<') && converted.includes('>')) {
      this.report.warnings.push(`${filePath}: Contains HTML/Components - may need MDX conversion`);
    }
    
    // 3. Convert internal links
    converted = converted.replace(
      /\[([^\]]+)\]\(([^)]+)\.md\)/g,
      '[$1]($2)'
    );
    
    // 4. Convert image paths
    converted = converted.replace(
      /!\[([^\]]*)\]\(\/(.+?)\)/g,
      '![$1](/img/$2)'
    );
    
    // 5. Handle VuePress variables
    converted = converted.replace(/\$site\.title/g, 'siteConfig.title');
    converted = converted.replace(/\$page\./g, '');
    
    return converted;
  }

  getTargetPath(filePath) {
    // Special handling for different file locations
    if (filePath === 'index.md' || filePath === '_index.md') {
      // Convert to intro.md for Docusaurus
      return path.join(this.targetDir, 'docs/intro.md');
    }
    
    // Regular docs go to docs directory
    return path.join(this.targetDir, 'docs', filePath);
  }

  async migrateStaticAssets() {
    console.log(chalk.yellow('🖼️  Migrating static assets...'));
    
    // Copy images
    const publicDir = path.join(this.sourceDir, 'docs/.vuepress/public');
    if (await fs.pathExists(publicDir)) {
      await fs.copy(publicDir, path.join(this.targetDir, 'static/img'));
      this.report.copiedAssets++;
    }
    
    // Copy other assets
    const assetsDir = path.join(this.sourceDir, 'docs/assets');
    if (await fs.pathExists(assetsDir)) {
      await fs.copy(assetsDir, path.join(this.targetDir, 'static/assets'));
      this.report.copiedAssets++;
    }
  }

  async createDocusaurusConfig() {
    console.log(chalk.yellow('🔧 Creating Docusaurus configuration...'));
    
    // Create docusaurus.config.js
    const docusaurusConfig = this.generateDocusaurusConfig();
    await fs.writeFile(
      path.join(this.targetDir, 'docusaurus.config.js'),
      docusaurusConfig
    );
    
    // Create sidebars.js
    const sidebarsConfig = this.generateSidebarsConfig();
    await fs.writeFile(
      path.join(this.targetDir, 'sidebars.js'),
      sidebarsConfig
    );
    
    // Create package.json
    const packageJson = this.generatePackageJson();
    await fs.writeFile(
      path.join(this.targetDir, 'package.json'),
      JSON.stringify(packageJson, null, 2)
    );
    
    // Create custom CSS
    const customCss = this.generateCustomCss();
    await fs.writeFile(
      path.join(this.targetDir, 'src/css/custom.css'),
      customCss
    );
  }

  generateDocusaurusConfig() {
    const config = this.report.configMigration;
    
    return `// @ts-check
// Note: type annotations allow type checking and IDEs autocompletion

const lightCodeTheme = require('prism-react-renderer/themes/github');
const darkCodeTheme = require('prism-react-renderer/themes/dracula');

/** @type {import('@docusaurus/types').Config} */
const config = {
  title: '${config.title || 'Documentation'}',
  tagline: '${config.description || 'Documentation site'}',
  favicon: 'img/favicon.ico',

  // Set the production url of your site here
  url: 'https://your-docusaurus-site.com',
  // Set the /<baseUrl>/ pathname under which your site is served
  baseUrl: '${config.base || '/'}',

  // GitHub pages deployment config.
  organizationName: 'your-org',
  projectName: 'your-project',

  onBrokenLinks: 'throw',
  onBrokenMarkdownLinks: 'warn',

  i18n: {
    defaultLocale: 'en',
    locales: ['en'],
  },

  presets: [
    [
      'classic',
      /** @type {import('@docusaurus/preset-classic').Options} */
      ({
        docs: {
          sidebarPath: require.resolve('./sidebars.js'),
          editUrl: '${config.themeConfig?.repo || 'https://github.com/your-org/your-project'}/tree/main/',
        },
        blog: {
          showReadingTime: true,
          editUrl: '${config.themeConfig?.repo || 'https://github.com/your-org/your-project'}/tree/main/',
        },
        theme: {
          customCss: require.resolve('./src/css/custom.css'),
        },
      }),
    ],
  ],

  themeConfig:
    /** @type {import('@docusaurus/preset-classic').ThemeConfig} */
    ({
      image: 'img/docusaurus-social-card.jpg',
      navbar: {
        title: '${config.title || 'Documentation'}',
        logo: {
          alt: 'Logo',
          src: 'img/logo.png',
        },
        items: [
          {
            type: 'docSidebar',
            sidebarId: 'tutorialSidebar',
            position: 'left',
            label: 'Docs',
          },
          ${this.generateNavItems(config.themeConfig?.nav || [])}
          {
            href: '${config.themeConfig?.repo || 'https://github.com/your-org/your-project'}',
            label: 'GitHub',
            position: 'right',
          },
        ],
      },
      footer: {
        style: 'dark',
        copyright: \`Copyright © \${new Date().getFullYear()} Built with Docusaurus.\`,
      },
      prism: {
        theme: lightCodeTheme,
        darkTheme: darkCodeTheme,
      },
    }),
};

module.exports = config;
`;
  }

  generateNavItems(navItems) {
    if (!navItems || navItems.length === 0) return '';
    
    return navItems
      .filter(item => item.link && !item.link.startsWith('http'))
      .map(item => `{
            to: '${item.to || item.link}',
            label: '${item.label || item.text}',
            position: 'left',
          }`)
      .join(',\n          ');
  }

  generateSidebarsConfig() {
    const sidebar = this.report.configMigration.themeConfig?.sidebar || {};
    
    // Convert VuePress sidebar to Docusaurus format
    const docusaurusSidebar = {};
    
    // Create main tutorial sidebar
    docusaurusSidebar.tutorialSidebar = [
      {
        type: 'doc',
        id: 'intro',
      },
    ];
    
    // Add categories from VuePress sidebar
    for (const [route, items] of Object.entries(sidebar)) {
      if (Array.isArray(items) && items.length > 0) {
        docusaurusSidebar.tutorialSidebar.push(...items);
      }
    }
    
    return `// @ts-check

/** @type {import('@docusaurus/plugin-content-docs').SidebarsConfig} */
const sidebars = {
  tutorialSidebar: ${JSON.stringify(docusaurusSidebar.tutorialSidebar, null, 2)},
};

module.exports = sidebars;
`;
  }

  generatePackageJson() {
    return {
      name: 'docusaurus-site',
      version: '0.0.0',
      private: true,
      scripts: {
        docusaurus: 'docusaurus',
        start: 'docusaurus start',
        build: 'docusaurus build',
        swizzle: 'docusaurus swizzle',
        deploy: 'docusaurus deploy',
        clear: 'docusaurus clear',
        serve: 'docusaurus serve',
        'write-translations': 'docusaurus write-translations',
        'write-heading-ids': 'docusaurus write-heading-ids'
      },
      dependencies: {
        '@docusaurus/core': '3.0.0',
        '@docusaurus/preset-classic': '3.0.0',
        '@mdx-js/react': '^3.0.0',
        'clsx': '^2.0.0',
        'prism-react-renderer': '^2.1.0',
        'react': '^18.0.0',
        'react-dom': '^18.0.0'
      },
      devDependencies: {
        '@docusaurus/module-type-aliases': '3.0.0'
      },
      browserslist: {
        production: ['>0.5%', 'not dead', 'not op_mini all'],
        development: ['last 1 chrome version', 'last 1 firefox version', 'last 1 safari version']
      },
      engines: {
        node: '>=18.0'
      }
    };
  }

  generateCustomCss() {
    return `/**
 * Any CSS included here will be global. The classic template
 * bundles Infima by default. Infima is a CSS framework designed to
 * work well for content-centric websites.
 */

/* You can override the default Infima variables here. */
:root {
  --ifm-color-primary: #2e8555;
  --ifm-color-primary-dark: #29784c;
  --ifm-color-primary-darker: #277148;
  --ifm-color-primary-darkest: #205d3b;
  --ifm-color-primary-light: #33925d;
  --ifm-color-primary-lighter: #359962;
  --ifm-color-primary-lightest: #3cad6e;
  --ifm-code-font-size: 95%;
  --docusaurus-highlighted-code-line-bg: rgba(0, 0, 0, 0.1);
}

/* For readability concerns, you should choose a lighter palette in dark mode. */
[data-theme='dark'] {
  --ifm-color-primary: #25c2a0;
  --ifm-color-primary-dark: #21af90;
  --ifm-color-primary-darker: #1fa588;
  --ifm-color-primary-darkest: #1a8870;
  --ifm-color-primary-light: #29d5b0;
  --ifm-color-primary-lighter: #32d8b4;
  --ifm-color-primary-lightest: #4fddbf;
  --docusaurus-highlighted-code-line-bg: rgba(0, 0, 0, 0.3);
}
`;
  }

  async generateReport() {
    console.log(chalk.yellow('📊 Generating migration report...'));
    
    this.report.endTime = new Date();
    this.report.duration = (this.report.endTime - this.report.startTime) / 1000;
    
    const reportPath = path.join(this.targetDir, 'MIGRATION_REPORT.md');
    const reportContent = this.formatReport();
    
    await fs.writeFile(reportPath, reportContent);
    console.log(chalk.green(`\n📄 Migration report saved to: ${reportPath}`));
  }

  formatReport() {
    const { report } = this;
    
    return `# VuePress to Docusaurus Migration Report

Generated: ${report.endTime.toISOString()}
Duration: ${report.duration.toFixed(2)} seconds

## Summary

- **Total Files Processed**: ${report.totalFiles}
- **Successfully Converted**: ${report.convertedFiles}
- **Assets Copied**: ${report.copiedAssets}
- **Errors**: ${report.errors.length}
- **Warnings**: ${report.warnings.length}

## Configuration Migration

\`\`\`json
${JSON.stringify(report.configMigration, null, 2)}
\`\`\`

## Conversion Details

### Successfully Converted Files
- ${report.convertedFiles} markdown files converted
- Frontmatter updated to Docusaurus format
- Internal links adjusted
- VuePress components flagged for manual conversion

### Errors
${report.errors.length === 0 ? 'No errors encountered.' : report.errors.map(e => `- ${e}`).join('\n')}

### Warnings
${report.warnings.length === 0 ? 'No warnings.' : report.warnings.map(w => `- ${w}`).join('\n')}

## Next Steps

1. **Install Dependencies**
   \`\`\`bash
   cd ${path.basename(this.targetDir)}
   npm install
   \`\`\`

2. **Review and Update Configuration**
   - Update \`docusaurus.config.js\` with your site details
   - Review \`sidebars.js\` for correct navigation structure
   - Update theme colors in \`src/css/custom.css\`

3. **Handle Special Content**
   - Convert VuePress components to React/MDX components
   - Review pages with HTML content for MDX conversion
   - Update any custom theme components

4. **Test the Site**
   \`\`\`bash
   npm start
   \`\`\`

5. **Build for Production**
   \`\`\`bash
   npm run build
   \`\`\`

## Manual Tasks Required

1. **Home Page**: ${report.warnings.some(w => w.includes('React component')) ? '✅ Automatically converted to React component' : 'The VuePress home page layout needs to be recreated as a custom React component'}
2. **Custom Components**: Any Vue components need to be converted to React
3. **Search**: Configure Algolia DocSearch or use local search plugin
4. **Deployment**: Update deployment configuration for your hosting platform
5. **Plugins**: ${report.detectedPlugins ? `Review PLUGIN_MIGRATION_GUIDE.md for ${report.detectedPlugins.length} detected plugins` : 'No VuePress plugins detected'}

## File Structure

The migrated documentation follows Docusaurus conventions:
- \`/docs\` - Documentation pages
- \`/blog\` - Blog posts (if applicable)
- \`/src\` - Custom components and pages
- \`/static\` - Static assets
- \`docusaurus.config.js\` - Main configuration
- \`sidebars.js\` - Sidebar configuration
`;
  }
}

// CLI Interface
async function main() {
  const args = process.argv.slice(2);
  
  if (args.length < 2) {
    console.error(chalk.red('Usage: node migrate-to-docusaurus.js <source-dir> <target-dir>'));
    console.error(chalk.yellow('Example: node migrate-to-docusaurus.js . ../bfdocs-docusaurus'));
    process.exit(1);
  }
  
  const sourceDir = path.resolve(args[0]);
  const targetDir = path.resolve(args[1]);
  
  // Validate source directory
  if (!await fs.pathExists(sourceDir)) {
    console.error(chalk.red(`Source directory does not exist: ${sourceDir}`));
    process.exit(1);
  }
  
  // Create target directory
  await fs.ensureDir(targetDir);
  
  // Run migration
  const migrator = new VuePressToDocusaurusMigrator(sourceDir, targetDir);
  await migrator.migrate();
}

// Run if called directly
if (require.main === module) {
  main().catch(error => {
    console.error(chalk.red('Fatal error:'), error);
    process.exit(1);
  });
}

module.exports = VuePressToDocusaurusMigrator;