#!/usr/bin/env node

/**
 * VuePress Plugin and Theme Migration Helper
 * Maps VuePress plugins to Docusaurus equivalents
 */

const pluginMappings = {
  // Search plugins
  '@vuepress/plugin-search': {
    docusaurus: '@docusaurus/plugin-content-docs',
    config: {
      note: 'Use Algolia DocSearch or @easyops-cn/docusaurus-search-local',
      setup: `
// For Algolia DocSearch:
themeConfig: {
  algolia: {
    appId: 'YOUR_APP_ID',
    apiKey: 'YOUR_API_KEY',
    indexName: 'YOUR_INDEX_NAME',
  },
}

// For local search:
// npm install @easyops-cn/docusaurus-search-local
plugins: [
  [
    require.resolve("@easyops-cn/docusaurus-search-local"),
    {
      hashed: true,
      language: ["en"],
    },
  ],
]`
    }
  },
  
  // PWA plugins
  '@vuepress/plugin-pwa': {
    docusaurus: '@docusaurus/plugin-pwa',
    config: {
      note: 'Install @docusaurus/plugin-pwa separately',
      setup: `
plugins: [
  [
    '@docusaurus/plugin-pwa',
    {
      debug: true,
      offlineModeActivationStrategies: [
        'appInstalled',
        'standalone',
        'queryString',
      ],
      pwaHead: [
        {
          tagName: 'link',
          rel: 'icon',
          href: '/img/logo.png',
        },
        {
          tagName: 'link',
          rel: 'manifest',
          href: '/manifest.json',
        },
        {
          tagName: 'meta',
          name: 'theme-color',
          content: 'rgb(37, 194, 160)',
        },
      ],
    },
  ],
]`
    }
  },
  
  // Google Analytics
  '@vuepress/plugin-google-analytics': {
    docusaurus: '@docusaurus/plugin-google-gtag',
    config: {
      note: 'Use @docusaurus/plugin-google-gtag for GA4',
      setup: `
plugins: [
  [
    '@docusaurus/plugin-google-gtag',
    {
      trackingID: 'G-XXXXXXXXXX',
      anonymizeIP: true,
    },
  ],
]`
    }
  },
  
  // Sitemap
  '@vuepress/plugin-sitemap': {
    docusaurus: '@docusaurus/plugin-sitemap',
    config: {
      note: 'Sitemap is included by default in preset-classic',
      setup: `
// Already included in @docusaurus/preset-classic
// Configure in docusaurus.config.js:
presets: [
  [
    'classic',
    {
      sitemap: {
        changefreq: 'weekly',
        priority: 0.5,
        ignorePatterns: ['/tags/**'],
        filename: 'sitemap.xml',
      },
    },
  ],
]`
    }
  },
  
  // Blog
  '@vuepress/plugin-blog': {
    docusaurus: '@docusaurus/plugin-content-blog',
    config: {
      note: 'Blog is included in preset-classic',
      setup: `
// Already included in @docusaurus/preset-classic
// Configure in docusaurus.config.js:
presets: [
  [
    'classic',
    {
      blog: {
        path: 'blog',
        routeBasePath: 'blog',
        include: ['*.md', '*.mdx'],
        blogSidebarCount: 10,
        blogSidebarTitle: 'Recent posts',
        showReadingTime: true,
        editUrl: 'https://github.com/your-org/your-repo/edit/main/',
      },
    },
  ],
]`
    }
  },
  
  // Container (custom blocks)
  '@vuepress/plugin-container': {
    docusaurus: 'Native Admonitions',
    config: {
      note: 'Use native Docusaurus admonitions',
      setup: `
// VuePress: ::: tip
// Docusaurus: :::tip

// VuePress: ::: warning Title
// Docusaurus: :::warning[Title]

// Available types: note, tip, info, caution, danger`
    }
  },
  
  // Code copy
  '@vuepress/plugin-copy-code': {
    docusaurus: 'Built-in',
    config: {
      note: 'Code copy button is built into Docusaurus',
      setup: '// Enabled by default in code blocks'
    }
  },
  
  // Back to top
  '@vuepress/plugin-back-to-top': {
    docusaurus: 'Theme Feature',
    config: {
      note: 'Built into theme',
      setup: `
// Enable in theme config:
themeConfig: {
  scrollToTop: true,
}`
    }
  },
  
  // NProgress
  '@vuepress/plugin-nprogress': {
    docusaurus: 'Built-in',
    config: {
      note: 'Progress bar is built into Docusaurus',
      setup: '// Enabled by default'
    }
  },
  
  // Medium zoom
  '@vuepress/plugin-medium-zoom': {
    docusaurus: 'plugin-ideal-image',
    config: {
      note: 'Use @docusaurus/plugin-ideal-image',
      setup: `
plugins: [
  [
    '@docusaurus/plugin-ideal-image',
    {
      quality: 70,
      max: 1030,
      min: 640,
      steps: 2,
      disableInDev: false,
    },
  ],
]`
    }
  }
};

const themeMappings = {
  '@vuepress/theme-default': {
    docusaurus: '@docusaurus/preset-classic',
    note: 'Default theme is included in preset-classic'
  },
  '@vuepress/theme-blog': {
    docusaurus: '@docusaurus/preset-classic',
    note: 'Blog functionality is included in preset-classic'
  }
};

function generatePluginMigrationGuide(vuepressPlugins) {
  const guide = [];
  
  guide.push('# VuePress to Docusaurus Plugin Migration Guide\n');
  guide.push('## Plugin Mappings\n');
  
  for (const plugin of vuepressPlugins) {
    const mapping = pluginMappings[plugin] || null;
    
    if (mapping) {
      guide.push(`### ${plugin}`);
      guide.push(`**Docusaurus Equivalent**: ${mapping.docusaurus}\n`);
      guide.push(`**Note**: ${mapping.config.note}\n`);
      guide.push('**Setup**:');
      guide.push('```javascript');
      guide.push(mapping.config.setup.trim());
      guide.push('```\n');
    } else {
      guide.push(`### ${plugin}`);
      guide.push('**Status**: No direct equivalent - requires custom implementation\n');
    }
  }
  
  return guide.join('\n');
}

function generateThemeMigrationGuide(vuepressTheme) {
  const guide = [];
  
  guide.push('## Theme Migration\n');
  
  const mapping = themeMappings[vuepressTheme] || null;
  
  if (mapping) {
    guide.push(`### ${vuepressTheme}`);
    guide.push(`**Docusaurus Equivalent**: ${mapping.docusaurus}`);
    guide.push(`**Note**: ${mapping.note}\n`);
  } else {
    guide.push(`### ${vuepressTheme}`);
    guide.push('**Status**: Custom theme - will need to be recreated as Docusaurus theme\n');
  }
  
  guide.push('## Theme Configuration Migration\n');
  guide.push('### Navigation');
  guide.push('```javascript');
  guide.push(`// VuePress
themeConfig: {
  nav: [
    { text: 'Home', link: '/' },
    { text: 'Guide', link: '/guide/' }
  ]
}

// Docusaurus
themeConfig: {
  navbar: {
    items: [
      { to: '/', label: 'Home', position: 'left' },
      { to: '/docs/intro', label: 'Guide', position: 'left' }
    ]
  }
}`);
  guide.push('```\n');
  
  guide.push('### Sidebar');
  guide.push('```javascript');
  guide.push(`// VuePress (in config)
themeConfig: {
  sidebar: {
    '/guide/': [
      '',
      'getting-started',
      'configuration'
    ]
  }
}

// Docusaurus (in sidebars.js)
module.exports = {
  guideSidebar: [
    'intro',
    'getting-started',
    'configuration'
  ]
}`);
  guide.push('```\n');
  
  return guide.join('\n');
}

module.exports = {
  pluginMappings,
  themeMappings,
  generatePluginMigrationGuide,
  generateThemeMigrationGuide
};

// If run directly
if (require.main === module) {
  const samplePlugins = [
    '@vuepress/plugin-search',
    '@vuepress/plugin-google-analytics',
    '@vuepress/plugin-pwa',
    '@vuepress/plugin-container'
  ];
  
  const guide = generatePluginMigrationGuide(samplePlugins);
  const themeGuide = generateThemeMigrationGuide('@vuepress/theme-default');
  
  console.log(guide);
  console.log(themeGuide);
}