#!/usr/bin/env node

/**
 * Reorganize documentation structure
 * - Move duplicated project docs to reference-copies
 * - Move historical docs to archive
 * - Keep only bfdocs-specific content in main structure
 */

const fs = require('fs').promises;
const path = require('path');

class DocsReorganizer {
  constructor() {
    this.docsDir = path.resolve('./docs');
    this.moves = [];
  }

  async reorganize() {
    console.log('📂 Starting documentation reorganization...');
    
    // Create target directories
    await this.createDirectories();
    
    // Move duplicated project documentation
    await this.moveProjectDuplicates();
    
    // Move historical/consolidated docs
    await this.moveHistoricalDocs();
    
    // Generate summary
    await this.generateSummary();
    
    console.log('✅ Reorganization complete!');
    console.log(`📊 Moved ${this.moves.length} directories`);
  }

  async createDirectories() {
    const dirs = [
      'docs/archive',
      'docs/archive/historical',
      'docs/archive/project-snapshots',
      'docs/reference-copies'
    ];
    
    for (const dir of dirs) {
      await fs.mkdir(dir, { recursive: true });
      console.log(`📁 Created ${dir}`);
    }
  }

  async moveProjectDuplicates() {
    // These are copies from other projects
    const projectDuplicates = [
      // Top-level project copies
      { from: 'BFAPI', to: 'reference-copies/BFAPI' },
      { from: 'llmcli', to: 'reference-copies/llmcli' },
      { from: 'BFLLM', to: 'reference-copies/BFLLM' },
      { from: 'llm-mcp', to: 'reference-copies/llm-mcp' },
      { from: 'BFRFP', to: 'reference-copies/BFRFP' },
      { from: 'llm-ui', to: 'reference-copies/llm-ui' },
      { from: 'BFCIComponents', to: 'reference-copies/BFCIComponents' },
      
      // Component subdirectory copies
      { from: 'components/bfapi', to: 'reference-copies/components/bfapi' },
      { from: 'components/llmcli', to: 'reference-copies/components/llmcli' },
      { from: 'components/bfllm', to: 'reference-copies/components/bfllm' },
      { from: 'components/llm-mcp', to: 'reference-copies/components/llm-mcp' },
      { from: 'components/bfrfp', to: 'reference-copies/components/bfrfp' },
      { from: 'components/llm-ui', to: 'reference-copies/components/llm-ui' },
      { from: 'components/bfcicomponents', to: 'reference-copies/components/bfcicomponents' },
      { from: 'components/bluefly_insights', to: 'reference-copies/components/bluefly_insights' },
      
      // Module-specific copies
      { from: 'drupal_native_ai_nativeaijs', to: 'reference-copies/drupal_modules/drupal_native_ai_nativeaijs' },
      { from: 'field_eca_bridge', to: 'reference-copies/drupal_modules/field_eca_bridge' },
      { from: 'langchain', to: 'reference-copies/drupal_modules/langchain' },
      { from: 'llm', to: 'reference-copies/drupal_modules/llm' },
      { from: 'secure_drupal', to: 'reference-copies/drupal_modules/secure_drupal' },
      { from: 'ai_coding_assistant', to: 'reference-copies/drupal_modules/ai_coding_assistant' },
      { from: 'ai_swarm_intelligence', to: 'reference-copies/drupal_modules/ai_swarm_intelligence' },
      { from: 'api_documentation', to: 'reference-copies/drupal_modules/api_documentation' },
      { from: 'model-registry-api', to: 'reference-copies/services/model-registry-api' }
    ];
    
    for (const move of projectDuplicates) {
      await this.moveDirectory(move.from, move.to);
    }
  }

  async moveHistoricalDocs() {
    // These are historical/consolidated documentation
    const historicalDocs = [
      { from: 'move_consolidated', to: 'archive/historical/move_consolidated' },
      { from: 'demo_consolidated', to: 'archive/historical/demo_consolidated' },
      { from: 'platform/archived', to: 'archive/historical/platform_archived' },
      { from: 'modules/archived', to: 'archive/historical/modules_archived' }
    ];
    
    for (const move of historicalDocs) {
      await this.moveDirectory(move.from, move.to);
    }
  }

  async moveDirectory(from, to) {
    const fromPath = path.join(this.docsDir, from);
    const toPath = path.join(this.docsDir, to);
    
    try {
      // Check if source exists
      await fs.access(fromPath);
      
      // Create parent directory for destination
      await fs.mkdir(path.dirname(toPath), { recursive: true });
      
      // Move the directory
      await fs.rename(fromPath, toPath);
      
      this.moves.push({ from, to });
      console.log(`✅ Moved ${from} → ${to}`);
      
    } catch (error) {
      if (error.code !== 'ENOENT') {
        console.log(`⚠️  Could not move ${from}: ${error.message}`);
      }
    }
  }

  async generateSummary() {
    const summary = `# Documentation Reorganization Summary

Generated: ${new Date().toISOString()}

## Moved Directories

${this.moves.map(m => `- \`${m.from}\` → \`${m.to}\``).join('\\n')}

## New Structure

### Core BFDOCS Documentation (Preserved)
- \`/architecture/\` - Platform architecture documentation
- \`/guides/\` - Implementation and integration guides  
- \`/development/\` - Development standards and practices
- \`/operations/\` - Operations and deployment docs
- \`/security/\` - Security documentation
- \`/testing/\` - Testing strategies
- \`/integration/\` - Integration patterns
- \`/migration/\` - Migration guides
- \`/planning/\` - Roadmaps and planning docs
- \`/projects/\` - Project reference pages (links to actual projects)

### Reference Copies (Moved)
- \`/reference-copies/\` - Snapshots of project documentation
  - For reference only - actual docs maintained in project repos
  - Will be replaced by dynamic references in future

### Archive (Moved)
- \`/archive/historical/\` - Historical documentation
  - Consolidated docs from previous reorganizations
  - Kept for reference and context

## Next Steps

1. Update VuePress configuration to reflect new structure
2. Set up automated sync from project repositories
3. Configure GitLab Pages deployment
4. Remove reference copies once dynamic system is in place
`;

    await fs.writeFile(
      path.join(this.docsDir, 'REORGANIZATION_SUMMARY.md'),
      summary
    );
    
    console.log('📄 Generated reorganization summary');
  }
}

// Run if called directly
if (require.main === module) {
  const reorganizer = new DocsReorganizer();
  reorganizer.reorganize().catch(console.error);
}

module.exports = DocsReorganizer;