#!/bin/bash

# AI Documentation Setup Script
# Sets up AI-powered documentation generation for your project

echo "🤖 AI Documentation Generator Setup"
echo "=================================="

# Check if running in bfdocs directory
if [ ! -f "package.json" ]; then
  echo "❌ Error: Run this script from the bfdocs directory"
  exit 1
fi

# Install dependencies
echo "📦 Installing dependencies..."
npm install --save-dev \
  @langchain/core \
  @langchain/community \
  @langchain/openai \
  @langchain/anthropic \
  langchain \
  @qdrant/js-client-rest \
  @tensorflow-models/universal-sentence-encoder

# Check for Ollama
echo ""
echo "🔍 Checking for Ollama..."
if command -v ollama &> /dev/null; then
  echo "✅ Ollama is installed"
  echo "📋 Available models:"
  ollama list
else
  echo "⚠️  Ollama not found. Would you like to install it? (y/n)"
  read -r response
  if [[ "$response" == "y" ]]; then
    echo "📥 Installing Ollama..."
    curl -fsSL https://ollama.com/install.sh | sh
    
    echo "📥 Pulling recommended models..."
    ollama pull mixtral:latest
    ollama pull nomic-embed-text
    echo "✅ Ollama installed with mixtral and embedding models"
  fi
fi

# Create AI config
echo ""
echo "🔧 Creating AI configuration..."
cat > ~/.ai-doc-config.json << 'EOF'
{
  "providers": {
    "ollama": {
      "enabled": true,
      "baseUrl": "http://localhost:11434",
      "models": {
        "chat": "mixtral:latest",
        "embedding": "nomic-embed-text"
      }
    },
    "openai": {
      "enabled": false,
      "apiKey": "${OPENAI_API_KEY}",
      "models": {
        "chat": "gpt-4-turbo-preview",
        "embedding": "text-embedding-3-small"
      }
    },
    "anthropic": {
      "enabled": false,
      "apiKey": "${ANTHROPIC_API_KEY}",
      "models": {
        "chat": "claude-3-haiku-20240307"
      }
    }
  },
  "documentation": {
    "outputDir": "docs/ai-generated",
    "filePatterns": [
      "src/**/*.{js,ts,jsx,tsx}",
      "lib/**/*.{js,ts}",
      "packages/*/src/**/*.{js,ts}"
    ],
    "excludePatterns": [
      "**/node_modules/**",
      "**/dist/**",
      "**/*.test.*",
      "**/*.spec.*"
    ]
  },
  "prompts": {
    "enhanceContext": "You are an expert technical writer. Enhance documentation to be clear, comprehensive, and helpful.",
    "codeContext": "Focus on explaining the why, not just the what. Include architecture decisions and best practices."
  }
}
EOF

echo "✅ Created ~/.ai-doc-config.json"

# Add npm scripts
echo ""
echo "📝 Adding npm scripts..."
npm pkg set scripts.docs:ai="node scripts/ai-doc-generator.js"
npm pkg set scripts.docs:ai:create="AI_MODE=create npm run docs:ai"
npm pkg set scripts.docs:ai:enhance="AI_MODE=enhance npm run docs:ai"
npm pkg set scripts.docs:ai:review="AI_MODE=review npm run docs:ai"
npm pkg set scripts.docs:ai:ollama="AI_PROVIDER=ollama npm run docs:ai"

# Create example GitLab CI
echo ""
echo "📄 Creating example GitLab CI configuration..."
if [ ! -f ".gitlab-ci.yml" ]; then
  cp .gitlab-ci.example-ai.yml .gitlab-ci.yml
  echo "✅ Created .gitlab-ci.yml with AI documentation jobs"
else
  echo "ℹ️  .gitlab-ci.yml already exists. Example saved as .gitlab-ci.example-ai.yml"
fi

# Test the setup
echo ""
echo "🧪 Testing AI documentation generator..."
echo "Running a quick test on a sample file..."

# Create a test file
mkdir -p test-docs
cat > test-docs/sample.js << 'EOF'
/**
 * Example user service for testing AI documentation
 */
class UserService {
  constructor(database) {
    this.db = database;
    this.cache = new Map();
  }

  async createUser(userData) {
    const user = await this.db.users.create(userData);
    this.cache.set(user.id, user);
    return user;
  }

  async getUser(id) {
    if (this.cache.has(id)) {
      return this.cache.get(id);
    }
    const user = await this.db.users.findById(id);
    if (user) {
      this.cache.set(id, user);
    }
    return user;
  }
}

module.exports = UserService;
EOF

# Run the generator
AI_MODE=create FILE_PATTERNS="test-docs/*.js" DOC_OUTPUT_DIR="test-docs/output" node scripts/ai-doc-generator.js

if [ -f "test-docs/output/sample.md" ]; then
  echo ""
  echo "✅ AI Documentation Generator is working!"
  echo "📄 Sample documentation created at: test-docs/output/sample.md"
  echo ""
  echo "Preview:"
  echo "--------"
  head -20 test-docs/output/sample.md
else
  echo "⚠️  Test failed. Please check the logs above."
fi

# Cleanup test files
rm -rf test-docs

echo ""
echo "🎉 Setup Complete!"
echo ""
echo "📚 Quick Start Commands:"
echo "  npm run docs:ai           # Run with default settings"
echo "  npm run docs:ai:create    # Create new documentation"
echo "  npm run docs:ai:enhance   # Enhance existing docs"
echo "  npm run docs:ai:review    # Review documentation quality"
echo ""
echo "🔧 Configuration:"
echo "  ~/.ai-doc-config.json     # Edit AI settings"
echo "  .gitlab-ci.yml            # GitLab CI integration"
echo ""
echo "📖 Documentation:"
echo "  docs/guides/AI_DOCUMENTATION_GUIDE.md"
echo ""
echo "🚀 Next steps:"
echo "  1. Run 'npm run docs:ai:create' to generate initial docs"
echo "  2. Push to GitLab to trigger automated documentation"
echo "  3. Customize prompts in scripts/ai-doc-generator.js"
echo ""

# Make the script executable
chmod +x scripts/ai-doc-generator.js