#!/usr/bin/env node

/**
 * Documentation Sync Script
 * 
 * This script can be run by individual projects to sync their documentation
 * to the bfdocs aggregation hub. It creates a standardized payload and
 * triggers the bfdocs rebuild process.
 * 
 * Usage:
 *   npm run docs:sync
 *   node scripts/sync-to-bfdocs.js
 *   node scripts/sync-to-bfdocs.js --project-key=bfapi --docs-path=docs
 */

const fs = require('fs').promises;
const path = require('path');
const https = require('https');
const { URL } = require('url');

class DocsSyncClient {
  constructor(options = {}) {
    this.options = {
      bfdocsUrl: options.bfdocsUrl || process.env.BFDOCS_URL || 'https://gitlab.bluefly.io/ecosystem/bfdocs',
      apiToken: options.apiToken || process.env.BFDOCS_API_TOKEN,
      webhookUrl: options.webhookUrl || process.env.BFDOCS_WEBHOOK_URL,
      projectKey: options.projectKey || this.inferProjectKey(),
      docsPath: options.docsPath || 'docs',
      includeOpenAPI: options.includeOpenAPI !== false,
      openAPIPath: options.openAPIPath || 'openapi.yaml',
      includePackageInfo: options.includePackageInfo !== false,
      dryRun: options.dryRun || false
    };
    
    this.payload = {
      project_key: this.options.projectKey,
      sync_timestamp: new Date().toISOString(),
      sync_source: 'npm_script',
      repository: this.inferRepositoryUrl(),
      branch: this.inferBranch(),
      commit: this.inferCommit()
    };
  }

  inferProjectKey() {
    try {
      const packageJsonPath = path.join(process.cwd(), 'package.json');
      if (require('fs').existsSync(packageJsonPath)) {
        const packageJson = require(packageJsonPath);
        return packageJson.name?.replace('@bluefly/', '') || path.basename(process.cwd());
      }
    } catch (e) {
      // Fall back to directory name
    }
    
    return path.basename(process.cwd());
  }

  inferRepositoryUrl() {
    try {
      const packageJsonPath = path.join(process.cwd(), 'package.json');
      if (require('fs').existsSync(packageJsonPath)) {
        const packageJson = require(packageJsonPath);
        if (packageJson.repository) {
          if (typeof packageJson.repository === 'string') {
            return packageJson.repository;
          }
          return packageJson.repository.url;
        }
      }
    } catch (e) {
      // Continue with git inference
    }
    
    // Try to infer from git remote
    try {
      const { execSync } = require('child_process');
      const remoteUrl = execSync('git remote get-url origin', { encoding: 'utf8' }).trim();
      return remoteUrl;
    } catch (e) {
      return 'unknown';
    }
  }

  inferBranch() {
    try {
      const { execSync } = require('child_process');
      return execSync('git branch --show-current', { encoding: 'utf8' }).trim();
    } catch (e) {
      return 'unknown';
    }
  }

  inferCommit() {
    try {
      const { execSync } = require('child_process');
      return execSync('git rev-parse HEAD', { encoding: 'utf8' }).trim();
    } catch (e) {
      return 'unknown';
    }
  }

  async gatherProjectData() {
    console.log(`📊 Gathering project data for ${this.options.projectKey}...`);
    
    // Add documentation files
    await this.addDocumentationFiles();
    
    // Add OpenAPI spec if enabled and available
    if (this.options.includeOpenAPI) {
      await this.addOpenAPISpec();
    }
    
    // Add package metadata if enabled
    if (this.options.includePackageInfo) {
      await this.addPackageMetadata();
    }
    
    // Add project structure information
    await this.addProjectStructure();
    
    console.log(`✅ Project data gathered successfully`);
  }

  async addDocumentationFiles() {
    const docsPath = path.join(process.cwd(), this.options.docsPath);
    
    try {
      await fs.access(docsPath);
      
      const files = [];
      await this.scanDocumentationDirectory(docsPath, '', files);
      
      this.payload.documentation = {
        path: this.options.docsPath,
        files: files,
        total_files: files.length
      };
      
      console.log(`📄 Found ${files.length} documentation files`);
    } catch (error) {
      console.log(`⚠️  No documentation directory found at ${this.options.docsPath}`);
      this.payload.documentation = {
        path: this.options.docsPath,
        files: [],
        total_files: 0,
        error: 'Directory not found'
      };
    }
  }

  async scanDocumentationDirectory(dirPath, relativePath, files) {
    try {
      const entries = await fs.readdir(dirPath, { withFileTypes: true });
      
      for (const entry of entries) {
        const fullPath = path.join(dirPath, entry.name);
        const relPath = path.join(relativePath, entry.name);
        
        if (entry.isDirectory() && !entry.name.startsWith('.')) {
          await this.scanDocumentationDirectory(fullPath, relPath, files);
        } else if (entry.isFile() && this.isDocumentationFile(entry.name)) {
          const fileInfo = await this.analyzeDocumentationFile(fullPath, relPath);
          files.push(fileInfo);
        }
      }
    } catch (error) {
      console.warn(`Warning: Could not scan directory ${dirPath}:`, error.message);
    }
  }

  isDocumentationFile(filename) {
    const docExtensions = ['.md', '.rst', '.txt', '.adoc'];
    return docExtensions.some(ext => filename.toLowerCase().endsWith(ext)) &&
           !filename.startsWith('.') &&
           !filename.includes('node_modules');
  }

  async analyzeDocumentationFile(fullPath, relativePath) {
    try {
      const content = await fs.readFile(fullPath, 'utf-8');
      const stats = await fs.stat(fullPath);
      
      return {
        path: relativePath,
        size: stats.size,
        modified: stats.mtime.toISOString(),
        title: this.extractTitle(content, relativePath),
        description: this.extractDescription(content),
        type: this.categorizeDoc(relativePath, content)
      };
    } catch (error) {
      return {
        path: relativePath,
        error: error.message,
        title: path.basename(relativePath, path.extname(relativePath)),
        type: 'unknown'
      };
    }
  }

  extractTitle(content, relativePath) {
    // Try to extract from first # heading
    const headingMatch = content.match(/^#\s+(.+)$/m);
    if (headingMatch) {
      return headingMatch[1].trim();
    }
    
    // Fall back to filename
    return path.basename(relativePath, path.extname(relativePath))
      .split(/[-_]/)
      .map(word => word.charAt(0).toUpperCase() + word.slice(1))
      .join(' ');
  }

  extractDescription(content) {
    const lines = content.split('\n');
    for (const line of lines) {
      const trimmed = line.trim();
      if (trimmed && !trimmed.startsWith('#') && !trimmed.startsWith('```')) {
        return trimmed.substring(0, 200) + (trimmed.length > 200 ? '...' : '');
      }
    }
    return 'Documentation file';
  }

  categorizeDoc(relativePath, content) {
    const path = relativePath.toLowerCase();
    
    if (path.includes('readme')) return 'overview';
    if (path.includes('api') || content.includes('OpenAPI')) return 'api';
    if (path.includes('setup') || path.includes('install')) return 'setup';
    if (path.includes('architecture')) return 'architecture';
    if (path.includes('deployment')) return 'deployment';
    if (path.includes('example')) return 'examples';
    if (path.includes('troubleshoot')) return 'troubleshooting';
    
    return 'general';
  }

  async addOpenAPISpec() {
    const openAPIPath = path.join(process.cwd(), this.options.openAPIPath);
    
    try {
      await fs.access(openAPIPath);
      const spec = await fs.readFile(openAPIPath, 'utf-8');
      
      // Try to parse as JSON or YAML
      let parsedSpec;
      try {
        parsedSpec = JSON.parse(spec);
      } catch (e) {
        // Assume YAML, store as string for now
        parsedSpec = { raw: spec, format: 'yaml' };
      }
      
      this.payload.openapi = {
        path: this.options.openAPIPath,
        version: parsedSpec.openapi || parsedSpec.swagger || 'unknown',
        title: parsedSpec.info?.title || 'API Documentation',
        description: parsedSpec.info?.description || '',
        endpoints: this.countEndpoints(parsedSpec),
        spec: spec
      };
      
      console.log(`📖 OpenAPI spec included: ${this.payload.openapi.title}`);
      console.log(`🔗 Endpoints found: ${this.payload.openapi.endpoints}`);
    } catch (error) {
      console.log(`⚠️  OpenAPI spec not found at ${this.options.openAPIPath}`);
    }
  }

  countEndpoints(spec) {
    if (!spec.paths) return 0;
    
    let count = 0;
    for (const path in spec.paths) {
      for (const method in spec.paths[path]) {
        if (['get', 'post', 'put', 'delete', 'patch', 'head', 'options'].includes(method.toLowerCase())) {
          count++;
        }
      }
    }
    return count;
  }

  async addPackageMetadata() {
    // Add package.json if it exists
    const packageJsonPath = path.join(process.cwd(), 'package.json');
    try {
      await fs.access(packageJsonPath);
      const packageJson = JSON.parse(await fs.readFile(packageJsonPath, 'utf-8'));
      
      this.payload.package_json = {
        name: packageJson.name,
        version: packageJson.version,
        description: packageJson.description,
        dependencies: Object.keys(packageJson.dependencies || {}),
        devDependencies: Object.keys(packageJson.devDependencies || {}),
        scripts: Object.keys(packageJson.scripts || {}),
        engines: packageJson.engines
      };
      
      console.log(`📦 Package.json included: ${packageJson.name}@${packageJson.version}`);
    } catch (error) {
      // Not a Node.js project
    }
    
    // Add composer.json if it exists
    const composerJsonPath = path.join(process.cwd(), 'composer.json');
    try {
      await fs.access(composerJsonPath);
      const composerJson = JSON.parse(await fs.readFile(composerJsonPath, 'utf-8'));
      
      this.payload.composer_json = {
        name: composerJson.name,
        description: composerJson.description,
        version: composerJson.version,
        type: composerJson.type,
        require: Object.keys(composerJson.require || {}),
        require_dev: Object.keys(composerJson['require-dev'] || {}),
        autoload: composerJson.autoload
      };
      
      console.log(`📦 Composer.json included: ${composerJson.name}`);
    } catch (error) {
      // Not a PHP project
    }
  }

  async addProjectStructure() {
    const technologies = new Set();
    const projectFiles = [];
    
    // Detect technologies
    const indicators = [
      { file: 'package.json', tech: 'Node.js' },
      { file: 'composer.json', tech: 'PHP' },
      { file: 'requirements.txt', tech: 'Python' },
      { file: 'Cargo.toml', tech: 'Rust' },
      { file: 'go.mod', tech: 'Go' },
      { file: 'tsconfig.json', tech: 'TypeScript' },
      { file: 'Dockerfile', tech: 'Docker' },
      { file: '.gitlab-ci.yml', tech: 'GitLab CI' },
      { file: 'jest.config.js', tech: 'Jest' },
      { file: 'vite.config.ts', tech: 'Vite' }
    ];
    
    for (const indicator of indicators) {
      try {
        await fs.access(path.join(process.cwd(), indicator.file));
        technologies.add(indicator.tech);
        projectFiles.push(indicator.file);
      } catch (e) {
        // File doesn't exist
      }
    }
    
    this.payload.project_structure = {
      technologies: Array.from(technologies),
      config_files: projectFiles,
      root_files: await this.getRootFiles()
    };
    
    console.log(`🛠️  Technologies detected: ${Array.from(technologies).join(', ')}`);
  }

  async getRootFiles() {
    try {
      const entries = await fs.readdir(process.cwd(), { withFileTypes: true });
      return entries
        .filter(entry => entry.isFile())
        .map(entry => entry.name)
        .filter(name => !name.startsWith('.') && !name.includes('node_modules'));
    } catch (error) {
      return [];
    }
  }

  async sendSync() {
    if (this.options.dryRun) {
      console.log('🏃‍♂️ Dry run mode - payload would be:');
      console.log(JSON.stringify(this.payload, null, 2));
      return;
    }
    
    console.log('🚀 Sending documentation sync...');
    
    // Try API endpoint first
    if (this.options.apiToken) {
      const success = await this.sendToAPI();
      if (success) {
        console.log('✅ Documentation synced via API');
        return;
      }
    }
    
    // Fallback to webhook
    if (this.options.webhookUrl) {
      const success = await this.sendToWebhook();
      if (success) {
        console.log('✅ Documentation synced via webhook');
        return;
      }
    }
    
    console.log('⚠️  No sync method succeeded. Check configuration.');
    console.log('💡 Payload saved locally for debugging:');
    await fs.writeFile('docs-sync-payload.json', JSON.stringify(this.payload, null, 2));
    console.log('📄 Payload written to docs-sync-payload.json');
  }

  async sendToAPI() {
    const apiUrl = `${this.options.bfdocsUrl}/api/v1/projects/${this.options.projectKey}/sync`;
    
    try {
      const response = await this.makeHttpRequest(apiUrl, {
        method: 'POST',
        headers: {
          'Authorization': `Bearer ${this.options.apiToken}`,
          'Content-Type': 'application/json'
        },
        body: JSON.stringify(this.payload)
      });
      
      if (response.statusCode >= 200 && response.statusCode < 300) {
        console.log('📡 API sync successful');
        return true;
      } else {
        console.log(`⚠️  API sync failed with status ${response.statusCode}`);
        return false;
      }
    } catch (error) {
      console.log(`⚠️  API sync error: ${error.message}`);
      return false;
    }
  }

  async sendToWebhook() {
    try {
      const response = await this.makeHttpRequest(this.options.webhookUrl, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: JSON.stringify(this.payload)
      });
      
      if (response.statusCode >= 200 && response.statusCode < 300) {
        console.log('🔔 Webhook triggered successfully');
        return true;
      } else {
        console.log(`⚠️  Webhook failed with status ${response.statusCode}`);
        return false;
      }
    } catch (error) {
      console.log(`⚠️  Webhook error: ${error.message}`);
      return false;
    }
  }

  makeHttpRequest(url, options) {
    return new Promise((resolve, reject) => {
      const urlObj = new URL(url);
      const requestOptions = {
        hostname: urlObj.hostname,
        port: urlObj.port,
        path: urlObj.pathname + urlObj.search,
        method: options.method,
        headers: options.headers
      };
      
      const req = https.request(requestOptions, (res) => {
        let data = '';
        res.on('data', chunk => data += chunk);
        res.on('end', () => {
          resolve({
            statusCode: res.statusCode,
            body: data
          });
        });
      });
      
      req.on('error', reject);
      
      if (options.body) {
        req.write(options.body);
      }
      
      req.end();
    });
  }

  async run() {
    console.log('📡 Starting documentation sync...');
    console.log(`Project: ${this.options.projectKey}`);
    console.log(`Docs path: ${this.options.docsPath}`);
    console.log(`Dry run: ${this.options.dryRun}`);
    
    await this.gatherProjectData();
    await this.sendSync();
    
    console.log('✅ Documentation sync complete!');
  }
}

// CLI support
if (require.main === module) {
  const args = process.argv.slice(2);
  const options = {};
  
  args.forEach(arg => {
    if (arg.startsWith('--')) {
      const [key, value] = arg.substring(2).split('=');
      const camelKey = key.replace(/-([a-z])/g, (_, letter) => letter.toUpperCase());
      options[camelKey] = value === 'true' ? true : value === 'false' ? false : value;
    }
  });
  
  if (args.includes('--help') || args.includes('-h')) {
    console.log(`
Documentation Sync Script

Usage:
  node sync-to-bfdocs.js [options]

Options:
  --project-key=KEY        Project identifier (default: inferred from package.json)
  --docs-path=PATH         Documentation directory path (default: docs)
  --bfdocs-url=URL         bfdocs repository URL
  --api-token=TOKEN        API token for bfdocs
  --webhook-url=URL        Webhook URL for triggering sync
  --include-openapi=BOOL   Include OpenAPI specification (default: true)
  --openapi-path=PATH      Path to OpenAPI spec (default: openapi.yaml)
  --dry-run=BOOL           Show payload without sending (default: false)
  --help, -h               Show this help

Environment Variables:
  BFDOCS_URL              bfdocs repository URL
  BFDOCS_API_TOKEN        API token for bfdocs
  BFDOCS_WEBHOOK_URL      Webhook URL for triggering sync

Examples:
  node sync-to-bfdocs.js
  node sync-to-bfdocs.js --dry-run=true
  node sync-to-bfdocs.js --project-key=bfapi --docs-path=documentation
    `);
    process.exit(0);
  }
  
  const client = new DocsSyncClient(options);
  client.run().catch(error => {
    console.error('❌ Sync failed:', error.message);
    process.exit(1);
  });
}

module.exports = DocsSyncClient;