/**
 * @module bfdocs
 * @description Documentation system for the Bluefly AI Platform
 * @version 1.0.0
 * @author Bluefly Engineering <engineering@bluefly.ai>
 * @license MIT
 *
 * TODO: Implementation tasks:
 * - [ ] Create comprehensive tests for exported API functions
 * - [ ] Complete implementation of validation API
 * - [ ] Enhance sync functionality with error handling
 * - [ ] Implement documentation analytics
 * - [ ] Add support for validation reporting and visualization
 * - [ ] Run coverage report to verify 95%+ coverage
 */

const DocumentationValidator = require('../scripts/validate-docs');
const ProjectSyncService = require('./services/project-sync');
const DocumentationAggregator = require('./services/documentation-aggregator');

/**
 * BFDocs API - Provides access to the documentation system functionality
 * @public
 */
module.exports = {
  /**
   * Validates documentation structure and content
   * @param {Object} options - Validation options
   * @param {string} [options.configPath='./docs-config.json'] - Path to configuration file
   * @param {boolean} [options.verbose=false] - Enable verbose output
   * @returns {Promise<Object>} Validation report
   * @example
   * const bfdocs = require('@bluefly/docs');
   * const report = await bfdocs.validateDocs();
   * console.log(`Validation result: ${report.summary.status}`);
   */
  validateDocs: async (options = {}) => {
    const validator = new DocumentationValidator(options.configPath);
    await validator.loadConfig();
    await validator.validateAllProjects();
    return validator.generateValidationReport();
  },
  
  /**
   * Synchronizes documentation structure across projects
   * @param {Object} options - Sync options
   * @param {string} [options.configPath='./docs-config.json'] - Path to configuration file
   * @param {boolean} [options.force=false] - Force overwrite existing files
   * @returns {Promise<Object>} Sync report
   * @example
   * const bfdocs = require('@bluefly/docs');
   * const report = await bfdocs.syncProjects();
   * console.log(`Synced ${report.projectsUpdated} projects`);
   */
  syncProjects: async (options = {}) => {
    const syncService = new ProjectSyncService(options.configPath);
    return syncService.syncAllProjects(options.force);
  },
  
  /**
   * Aggregates documentation from all projects
   * @param {Object} options - Aggregation options
   * @param {string} [options.configPath='./docs-config.json'] - Path to configuration file
   * @param {string} [options.outputDir='./public'] - Output directory
   * @returns {Promise<Object>} Aggregation report
   * @example
   * const bfdocs = require('@bluefly/docs');
   * const report = await bfdocs.aggregateDocs();
   * console.log(`Aggregated ${report.filesGenerated} files`);
   */
  aggregateDocs: async (options = {}) => {
    const aggregator = new DocumentationAggregator(options.configPath);
    return aggregator.aggregateAllProjects(options.outputDir || './public');
  },
  
  // Export classes for advanced usage
  DocumentationValidator,
  ProjectSyncService,
  DocumentationAggregator
};