/**
 * @module ProjectSyncService
 * @description Service to synchronize documentation structure across projects
 *
 * TODO: Implementation tasks:
 * - [ ] Review tests for sync functionality 
 * - [ ] Implement two-way sync between projects and hub
 * - [ ] Add support for syncing custom templates
 * - [ ] Create conflict resolution mechanisms
 * - [ ] Implement automated documentation quality assessment
 * - [ ] Add better error handling and reporting
 * - [ ] Run coverage report to verify 95%+ coverage
 */

const fs = require('fs').promises;
const path = require('path');

/**
 * Service for synchronizing documentation structure across projects
 * @class
 */
class ProjectSyncService {
  /**
   * Create a ProjectSyncService
   * @param {string} configPath - Path to configuration file
   */
  constructor(configPath = './docs-config.json') {
    this.configPath = configPath;
    this.config = null;
    this.stats = {
      projectsUpdated: 0,
      filesCreated: 0,
      filesUpdated: 0,
      errors: []
    };
  }

  /**
   * Load configuration file
   * @returns {Promise<Object>} Configuration object
   * @throws {Error} If configuration cannot be loaded
   */
  async loadConfig() {
    try {
      const configContent = await fs.readFile(this.configPath, 'utf-8');
      this.config = JSON.parse(configContent);
      return this.config;
    } catch (error) {
      throw new Error(`Failed to load configuration: ${error.message}`);
    }
  }

  /**
   * Synchronize all projects
   * @param {boolean} force - Force overwrite existing files
   * @returns {Promise<Object>} Sync report
   */
  async syncAllProjects(force = false) {
    if (!this.config) {
      await this.loadConfig();
    }

    for (const [projectKey, projectConfig] of Object.entries(this.config.projects)) {
      try {
        await this.syncProject(projectKey, projectConfig, force);
        this.stats.projectsUpdated++;
      } catch (error) {
        this.stats.errors.push({
          project: projectKey,
          error: error.message
        });
      }
    }

    return this.stats;
  }

  /**
   * Synchronize a single project
   * @param {string} projectKey - Project key
   * @param {Object} projectConfig - Project configuration
   * @param {boolean} force - Force overwrite existing files
   * @returns {Promise<void>}
   */
  async syncProject(projectKey, projectConfig, force) {
    const projectPath = path.resolve(projectConfig.path);
    const docsPath = path.join(projectPath, projectConfig.docsPath || 'docs');
    
    // Ensure docs directory exists
    try {
      await fs.mkdir(docsPath, { recursive: true });
    } catch (error) {
      // Ignore if directory already exists
    }
    
    // Sync required files
    await this.syncRequiredFiles(projectKey, projectConfig, docsPath, force);
  }

  /**
   * Synchronize required documentation files
   * @param {string} projectKey - Project key
   * @param {Object} projectConfig - Project configuration
   * @param {string} docsPath - Documentation directory path
   * @param {boolean} force - Force overwrite existing files
   * @returns {Promise<void>}
   */
  async syncRequiredFiles(projectKey, projectConfig, docsPath, force) {
    const requiredFiles = ['index.md', 'architecture.md', 'api.md', 'setup.md'];
    
    for (const file of requiredFiles) {
      const filePath = path.join(docsPath, file);
      
      // Check if file exists
      let fileExists = false;
      try {
        await fs.access(filePath);
        fileExists = true;
      } catch (error) {
        // File doesn't exist
      }
      
      if (!fileExists || force) {
        // Create or update file
        await this.createDocumentationFile(projectKey, projectConfig, filePath, file);
        
        if (fileExists) {
          this.stats.filesUpdated++;
        } else {
          this.stats.filesCreated++;
        }
      }
    }
  }

  /**
   * Create a documentation file
   * @param {string} projectKey - Project key
   * @param {Object} projectConfig - Project configuration
   * @param {string} filePath - File path
   * @param {string} fileName - File name
   * @returns {Promise<void>}
   */
  async createDocumentationFile(projectKey, projectConfig, filePath, fileName) {
    let content = '';
    
    // Generate frontmatter
    content += '---\n';
    content += `title: ${this.getFileTitle(fileName, projectConfig.name)}\n`;
    content += `description: ${this.getFileDescription(fileName, projectConfig.name)}\n`;
    content += `project: ${projectKey}\n`;
    content += `category: ${projectConfig.category || 'Uncategorized'}\n`;
    content += `version: ${projectConfig.version || '1.0.0'}\n`;
    content += '---\n\n';
    
    // Generate content based on file type
    switch (fileName) {
      case 'index.md':
        content += `# ${projectConfig.name}\n\n`;
        content += `${projectConfig.description || 'Project description'}\n\n`;
        content += '## Overview\n\n';
        content += '[Add project overview here]\n\n';
        content += '## Quick Start\n\n';
        content += '```bash\n# Installation\n[Add installation commands here]\n```\n\n';
        content += '## Features\n\n';
        content += '- [Feature 1]\n- [Feature 2]\n- [Feature 3]\n\n';
        break;
        
      case 'architecture.md':
        content += `# ${projectConfig.name} Architecture\n\n`;
        content += '## System Design\n\n';
        content += '[Add architecture overview here]\n\n';
        content += '## Components\n\n';
        content += '- [Component 1]\n- [Component 2]\n- [Component 3]\n\n';
        content += '## Integration Points\n\n';
        content += '[Add integration points here]\n\n';
        break;
        
      case 'api.md':
        content += `# ${projectConfig.name} API\n\n`;
        content += '## API Reference\n\n';
        content += '[Add API documentation here]\n\n';
        content += '## Examples\n\n';
        content += '```javascript\n// Example API usage\n[Add code example here]\n```\n\n';
        break;
        
      case 'setup.md':
        content += `# ${projectConfig.name} Setup\n\n`;
        content += '## Prerequisites\n\n';
        content += '- [Prerequisite 1]\n- [Prerequisite 2]\n\n';
        content += '## Installation\n\n';
        content += '```bash\n# Installation steps\n[Add installation steps here]\n```\n\n';
        content += '## Configuration\n\n';
        content += '[Add configuration instructions here]\n\n';
        break;
    }
    
    // Write file
    await fs.writeFile(filePath, content, 'utf-8');
  }

  /**
   * Get file title based on file name
   * @param {string} fileName - File name
   * @param {string} projectName - Project name
   * @returns {string} File title
   */
  getFileTitle(fileName, projectName) {
    switch (fileName) {
      case 'index.md':
        return `${projectName} Overview`;
      case 'architecture.md':
        return `${projectName} Architecture`;
      case 'api.md':
        return `${projectName} API Reference`;
      case 'setup.md':
        return `${projectName} Setup Guide`;
      default:
        return fileName.replace('.md', '').replace(/^[a-z]/, c => c.toUpperCase());
    }
  }

  /**
   * Get file description based on file name
   * @param {string} fileName - File name
   * @param {string} projectName - Project name
   * @returns {string} File description
   */
  getFileDescription(fileName, projectName) {
    switch (fileName) {
      case 'index.md':
        return `Overview and quick start guide for ${projectName}`;
      case 'architecture.md':
        return `Technical architecture and design for ${projectName}`;
      case 'api.md':
        return `API documentation and examples for ${projectName}`;
      case 'setup.md':
        return `Installation and configuration guide for ${projectName}`;
      default:
        return `Documentation for ${projectName}`;
    }
  }
}

module.exports = ProjectSyncService;